C
C     SCALA
C     Copyright (C) 1998-2008 Phil Evans
C
C     This code is distributed under the terms and conditions of the
C     CCP4 licence agreement as `Part 2' (Annex 2) software.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
C
C
C
C
C ---------------------------------------------------------
C
      program scala
c     =============
c
c  This program scales together multiple observations of reflections,
c  expressing the scale factor as an interpolated function of the
c  3-dimensional coordinate of the observation, in a similar way to the
c  method of Kabsch (W. Kabsch, J.Appl.Cryst. 21, 916-924 (1988))
c
c  The program was originally designed to determine adjustments to the input
c  standard deviations to match the observed scatter, but this DID NOT
c  WORK in its original implementation, so is inactivated, though some
c  vestigial code remains.
c  From version 3.3.0, an alternative refinement of SD correction parameters
c  was implemented which does work.
c
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'flow.fh'
      include 'version.fh'
      include 'chtml.fh'
c
c Global arrays
c NB the maximum number supported by ccpalc is 12!
      integer maxgar
      parameter (maxgar = 8)
      integer nrfgar, lengths(maxgar), narrays, maxmtx,
     $   ningar, nregar
      character types(maxgar)*1
c
      integer ldum, ifail, l
      character line*80, linest*80, dna_filename*80
c
      integer lenstr
      external lenstr, scala_main
c
c
      lunout = 6
      lundmp = 1
      lunrst = 2
c
      ldum = 0
      ifail = 0
c  Write html'ery
      lchtml = .true.
c          ********************************************
      call htm_start()
      call ccpfyp
      call mtzini
      call ccpdpn(5,'DATA','READONLY','F',ldum,ifail)
      call ccpdpn(lunout,'PRINTER','PRINTER','F',ldum,ifail)
      call ccp4_prog_version(versn(1:lenstr(versn)),0)
      call ccprcs(lunout,'Scala','$Date: '//vrdate//'$')
      call htm_index()
c          ********************************************
c     switch the DNA things on - based on the DNAOUT command line
c     assignment (the use of 'DNA' is to be phased out)
      call ugtenv('DNAOUT', dna_filename)
      if (dna_filename .eq. ' ') call ugtenv('DNA', dna_filename)
      if(dna_filename .ne. ' ') then
         call dna_start(dna_filename, 'scala')
      else
         call dna_set_no_output
      end if

      call dna_table_start('header_info')
      call dna_list_start('version_information')
      call dna_character_item('version', versn(1:lenstr(versn)))
      call dna_character_item('author', 'Phil Evans')
      call dna_character_item('date', vrdate(1:lenstr(vrdate)))
      call dna_list_end
      call dna_table_end



c
c     end-dna
c

      linest = ' *******************************************'
      l = lenstr(linest)
      write (lunout,'(//2(15x,a/),15x,a)')
     $     linest(1:l),
     $     ' *    SCALA - continuous scaling program   *',
     $     ' *                                         *'
      write (line, '(a,a)')
     $     ' *    Version: ',versn(1:lenstr(versn))
      line(l:l) = '*'
      write (lunout, '(15x,a)') line(1:l)
      write (line, '(a,a)')
     $     ' *    Date   : ',vrdate(1:lenstr(vrdate))
      line(l:l) = '*'
      write (lunout, '(15x,a)') line(1:l)
      write (lunout, '(3(15x,a/),15x,a)')
     $     ' *                                         *',
     $     ' *   Phil Evans, MRC LMB, Cambridge, UK    *',
     $     ' *   pre@mrc-lmb.cam.ac.uk                 *',
     $     ' *                                         *'
      write (lunout, '(15x,a/)') linest(1:l)
c
      write (lunout, '(/a/a,a/a,a)')
     $   '$TEXT:Reference: $$ Please reference $$',
     $   'P.R.Evans, "Scaling and assessment  of data quality",',
     $   ' Acta Cryst. D62, 72-82  (2006).',
     $   'Note that definitions of Rmeas and Rpim in this paper',
     $   ' are missing a square-root on the (n/n-1) factor'
      write (lunout, '(a/)') '$$'

c     $   '"Scala" P.R.Evans (1997), Joint CCP4 and ESF-EACBM ',
c     $   'Newsletter 33, 22-24'
c
c
c Open input reflection file, get number of reflections
      call setupf(nrfgar)
      if (nrfgar .le. 0) then
         call dna_error('no reflections in HKLIN')
         call ccperr(1,'*** No reflections in HKLIN file ***')
      endif
c Read commands
      call keyin
c Get array size
      call mtxsiz(maxmtx)
c---- Set up global arrays
      narrays = maxgar
c  Reflection arrays: all lengths of basic arrays are the same
c   Array 1 is an a 1D array of integers (nrfgar)
c            This serves to pass the number of reflections
c   Array 2 is an a 2D array of integers (nrfgar,ningar)
c   Array 3 is an a 2D array of reals (nrfgar,nregar)
c *** NB ***  If you change ningar & nregar here you must also
c             change s/r scala_main to match
      ningar = 5
      nregar = 6
      types(1) = 'I'
      lengths(1) = nrfgar
      types(2) = 'I'
      lengths(2) = nrfgar * ningar
      types(3) = 'R'
      lengths(3) = nrfgar * nregar
      l = 3
c  amatrx
      l = l+1
      types(l)   = 'D'
      lengths(l) = maxmtx*maxmtx
c  eigval
      l = l+1
      types(l)   = 'D'
      lengths(l) = maxmtx
c  eigvec
      l = l+1
      types(l)   = 'D'
      lengths(l) = maxmtx*maxmtx
c  vecinv
      l = l+1
      types(l)   = 'D'
      lengths(l) = maxmtx
c  work
      l = l+1
      types(l)   = 'D'
      lengths(l) = maxmtx*5
c
      call ccpalc(scala_main, narrays, types, lengths)
      end
      subroutine ad1sts(nkept, DelI, aihl, sdihl,
     $    irun, mbn_b, mbn_t, mbn_s, mbn_i)
c     ============================================
c
c  Add some things into sums for statistics
c    deviations relative to overall mean etc
c
c On entry:
c  nkept        number of observations
c  DelI         deviation, Ihl - <I>
c  aihl         scaled intensity Ihl
c  sdihl        sd(Ihl)
c  irun         run number
c  mbn_b        bin on rotation within run
c  mbn_t        bin on rotation overall
c  mbn_s        bin on resolution
c  mbn_i        bin on intensity

      real    DelI, aihl, sdihl
      integer  nkept, irun, mbn_b, mbn_t, mbn_s, mbn_i
c
      include 'parameter.fh'
      include 'stats.fh'
c
      real an, wt_mul, wt_pim
c
      an = float(nkept)
      wt_mul = sqrt(an / (an - 1.0))
      wt_pim = sqrt(1. / (an - 1.0))

c
c  Abs(DelI)
      sum_o_del_t(mbn_t) = sum_o_del_t(mbn_t) + abs(DelI)
      sum_o_del_s(mbn_s) = sum_o_del_s(mbn_s) + abs(DelI)
      sum_o_wdl_s(mbn_s) = sum_o_wdl_s(mbn_s) + wt_mul * abs(DelI)
      sum_o_ndl_s(mbn_s) = sum_o_ndl_s(mbn_s) + wt_pim * abs(DelI)
      sum_o_del_i(mbn_i) = sum_o_del_i(mbn_i) + abs(DelI)
      n_o_del_t(mbn_t) = n_o_del_t(mbn_t) + 1
      n_o_del_s(mbn_s) = n_o_del_s(mbn_s) + 1
      n_o_del_i(mbn_i) = n_o_del_i(mbn_i) + 1
c  Sigma, for RMS scatter
      sum_o_sig_t(mbn_t) = sum_o_sig_t(mbn_t) + DelI**2
      sum_o_sig_s(mbn_s) = sum_o_sig_s(mbn_s) + DelI**2
      sum_o_sig_i(mbn_i) = sum_o_sig_i(mbn_i) + DelI**2
      n_o_sig_t(mbn_t) = n_o_sig_t(mbn_t) + 1
      n_o_sig_s(mbn_s) = n_o_sig_s(mbn_s) + 1
      n_o_sig_i(mbn_i) = n_o_sig_i(mbn_i) + 1
c  Imean
      sum_o_i_t(mbn_t) = sum_o_i_t(mbn_t) + aihl
      sum_o_i_s(mbn_s) = sum_o_i_s(mbn_s) + aihl
      sum_o_i_i(mbn_i) = sum_o_i_i(mbn_i) + aihl
      n_o_i_t(mbn_t) = n_o_i_t(mbn_t) + 1
      n_o_i_s(mbn_s) = n_o_i_s(mbn_s) + 1
      n_o_i_i(mbn_i) = n_o_i_i(mbn_i) + 1
c  sd, for mean
      sum_o_sd_t(mbn_t) = sum_o_sd_t(mbn_t) + sdihl
      sum_o_sd_s(mbn_s) = sum_o_sd_s(mbn_s) + sdihl
      sum_o_sd_i(mbn_i) = sum_o_sd_i(mbn_i) + sdihl
      n_o_sd_t(mbn_t) = n_o_sd_t(mbn_t) + 1
      n_o_sd_s(mbn_s) = n_o_sd_s(mbn_s) + 1
      n_o_sd_i(mbn_i) = n_o_sd_i(mbn_i) + 1
c
      return
      end
      subroutine ad2sts(nkept, DelI, vardli, FrcDlr, aihl, sdihl, DelIf,
     $   lfull, irun, m, mbn_b, mbn_t, mbn_s, mbn_i, mbn_x, mbn_y)
c     =====================================================
c
c  Add some things into sums for statistics
c    deviations relative to I+ or I- mean etc
c
c On entry:
c  nkept        number of observations in this class (I+, I- or all I)
c  DelI         deviation, Ihl - <I>
c  vardli       var(<I>)
c  FrcDlr       (Ihl - <I>others)/sqrt[sd(Ihl)**2 + sd(<I>others)**2]
c  aihl         scaled intensity Ihl
c  sdihl        sd(Ihl)
c  DelIf        deviation, Ihl - <I>, fulls only (only if lfull .true.)
c  lfull        .true. if DelIf valid
c  irun         run number
c  m            = 0 full, = +1 partial
c  mbn_b        bin on rotation within run
c  mbn_t        bin on rotation overall
c  mbn_s        bin on resolution
c  mbn_i        bin on intensity
c  mbn_x, mbn_y bins on Xdet, Ydet

      real    DelI, aihl, sdihl, vardli, FrcDlr, DelIf
      integer  nkept, irun, m, mbn_b, mbn_t, mbn_s, mbn_i, mbn_x, mbn_y
      logical lfull
c
      include 'parameter.fh'
      include 'stats.fh'
c
      integer j, k
      real FrcDel, an, wt_mul, wt_pim
c
      an = float(nkept)
      wt_mul = sqrt(an / (an - 1.0))
      wt_pim = sqrt(1. / (an - 1.0))

c  Abs(DelI)
      sum_m_del_t(mbn_t) = sum_m_del_t(mbn_t) + abs(DelI)
      sum_m_del_s(mbn_s) = sum_m_del_s(mbn_s) + abs(DelI)
      sum_m_wdl_s(mbn_s) = sum_m_wdl_s(mbn_s) + wt_mul * abs(DelI)
      sum_m_ndl_s(mbn_s) = sum_m_ndl_s(mbn_s) + wt_pim * abs(DelI)
      sum_m_del_i(mbn_i) = sum_m_del_i(mbn_i) + abs(DelI)
      if (lfull) then
         sum_f_del_s(mbn_s) = sum_f_del_s(mbn_s) + abs(DelIf)
         sum_f_del_i(mbn_i) = sum_f_del_i(mbn_i) + abs(DelIf)
         sum_f_i_s(mbn_s) = sum_f_i_s(mbn_s) + aihl
         sum_f_i_i(mbn_i) = sum_f_i_i(mbn_i) + aihl
      endif
      sum_m_del_rs(irun,mbn_s) = sum_m_del_rs(irun,mbn_s) + abs(DelI)
      n_m_del_t(mbn_t) = n_m_del_t(mbn_t) + 1
      n_m_del_s(mbn_s) = n_m_del_s(mbn_s) + 1
      n_m_del_i(mbn_i) = n_m_del_i(mbn_i) + 1
      n_m_del_rs(irun,mbn_s) = n_m_del_rs(irun,mbn_s) + 1
      if (m .eq. 0) then
         n_m_full_t(mbn_t) = n_m_full_t(mbn_t) + 1
         j = 1
      else
         n_m_part_t(mbn_t) = n_m_part_t(mbn_t) + 1
         j = 2
      endif
c  Sigma, for RMS scatter
      sum_m_sig_t(mbn_t) = sum_m_sig_t(mbn_t) + DelI**2
      sum_m_sig_s(mbn_s) = sum_m_sig_s(mbn_s) + DelI**2
      sum_m_sig_i(mbn_i) = sum_m_sig_i(mbn_i) + DelI**2
      n_m_sig_t(mbn_t) = n_m_sig_t(mbn_t) + 1
      n_m_sig_s(mbn_s) = n_m_sig_s(mbn_s) + 1
      n_m_sig_i(mbn_i) = n_m_sig_i(mbn_i) + 1
c  Imean
      sum_m_i_t(mbn_t) = sum_m_i_t(mbn_t) + aihl
      sum_m_i_s(mbn_s) = sum_m_i_s(mbn_s) + aihl
      sum_m_i_i(mbn_i) = sum_m_i_i(mbn_i) + aihl
      sum_m_i_rs(irun,mbn_s) = sum_m_i_rs(irun,mbn_s) + aihl
      n_m_i_t(mbn_t) = n_m_i_t(mbn_t) + 1
      n_m_i_s(mbn_s) = n_m_i_s(mbn_s) + 1
      n_m_i_i(mbn_i) = n_m_i_i(mbn_i) + 1
c  sd, for mean
      sum_m_sd_t(mbn_t) = sum_m_sd_t(mbn_t) + sdihl
      sum_m_sd_s(mbn_s) = sum_m_sd_s(mbn_s) + sdihl
      sum_m_sd_i(mbn_i) = sum_m_sd_i(mbn_i) + sdihl
      n_m_sd_t(mbn_t) = n_m_sd_t(mbn_t) + 1
      n_m_sd_s(mbn_s) = n_m_sd_s(mbn_s) + 1
      n_m_sd_i(mbn_i) = n_m_sd_i(mbn_i) + 1
c  sd analysis
c      FrcDel = DelI/sqrt(sdihl**2 + vardli)
c  j = 1 full, j = 2 partial
      FrcDel=FrcDlr
      if (FrcDel .ne. 0.0) then
         sum_m_isq_irf(mbn_i,irun,j)   =
     $      sum_m_isq_irf(mbn_i,irun,j) + sign(aihl**2, aihl)
         sum_m_frd_irf(mbn_i,irun,j)   =
     $      sum_m_frd_irf(mbn_i,irun,j) + FrcDel
         sum_m_frdsq_irf(mbn_i,irun,j) =
     $      sum_m_frdsq_irf(mbn_i,irun,j) + (FrcDel)**2
         n_m_sd_irf(mbn_i,irun,j)      =
     $      n_m_sd_irf(mbn_i,irun,j)    + 1
         k = min(max(int((Frcdel+5.0)*2.)+1, 1), 21) 
         n_m_sdhist(k,j) = n_m_sdhist(k,j) + 1
c
         sum_m_frd_xy(mbn_x,mbn_y)   =
     $      sum_m_frd_xy(mbn_x,mbn_y) + FrcDel
         sum_m_frdsq_xy(mbn_x,mbn_y) =
     $      sum_m_frdsq_xy(mbn_x,mbn_y) + FrcDel**2
         n_m_frd_xy(mbn_x,mbn_y) =
     $      n_m_frd_xy(mbn_x,mbn_y) + 1
      endif
c
      return
      end
      subroutine ad3sts(DelI, aih,
     $    irun, mbn_b, mbn_t, mbn_s, mbn_i)
c     ======================================
c
c  Add some things into sums for statistics
c    Full/partial Bias stuff
c
c On entry:
c  DelI         deviation, <Ifull> - Ihl(partial),
c                where <Ifull> is scale-weighted mean of fulls
c                If there are no fulls, this replaced by the summed partials
c                with the smallest number of slots
c  aih          mean intensity Imean
c  irun         run number
c  mbn_b        bin on rotation within run
c  mbn_t        bin on rotation overall
c  mbn_s        bin on resolution
c  mbn_i        bin on intensity

      real    DelI, aih
      integer  irun, mbn_b, mbn_t, mbn_s, mbn_i
c
      include 'parameter.fh'
      include 'stats.fh'
c
c  DelI = bias
      sum_b_bias_rs(irun,mbn_s) = sum_b_bias_rs(irun,mbn_s) + DelI
      sum_b_bias_ri(irun,mbn_i) = sum_b_bias_ri(irun,mbn_i) + DelI
      n_b_bias_rs(irun,mbn_s) = n_b_bias_rs(irun,mbn_s) +  1
      n_b_bias_ri(irun,mbn_i) = n_b_bias_ri(irun,mbn_i) +  1
c  <Imean>
      sum_b_i_rs(irun,mbn_s) = sum_b_i_rs(irun,mbn_s) + aih
      sum_b_i_ri(irun,mbn_i) = sum_b_i_ri(irun,mbn_i) + aih
      return
      end
      subroutine ad4sts(aihpls, aihmns, mbn_s, mbn_i)
c     ===============================================
c
c  Add some things into sums for statistics
c    Anomalous deviations
c
c On entry:
c  aihpls       scaled intensity Ihl, I+
c  aihmns       scaled intensity Ihl, I+
c  mbn_s        bin on resolution
c  mbn_i        bin on intensity

      real    aihpls, aihmns
      integer  mbn_s, mbn_i
c
      include 'parameter.fh'
      include 'stats.fh'
c
      sum_anom_dif_s(mbn_s) = sum_anom_dif_s(mbn_s) + abs(aihpls-aihmns)
      sum_anom_dif_i(mbn_i) = sum_anom_dif_i(mbn_i) + abs(aihpls-aihmns)
      sum_anom_sum_s(mbn_s) = sum_anom_sum_s(mbn_s) + aihpls+aihmns
      sum_anom_sum_i(mbn_i) = sum_anom_sum_i(mbn_i) + aihpls+aihmns
      n_anom_s(mbn_s) = n_anom_s(mbn_s) + 1
      n_anom_i(mbn_i) = n_anom_i(mbn_i) + 1
c
      return
      end
      subroutine ad5sts(aih, sdih, mbn_s)
c     =============================
c
c  Add some things into sums for statistics
c     Mean <I>/sd(<I>)
c
c On entry:
c  aih         scaled intensity <I>
c  sdih        sd(<I>)
c  mbn_s       bin on resolution

      real    aih, sdih
      integer  mbn_s
c
      include 'parameter.fh'
      include 'stats.fh'
c
      if (sdih .gt. 0.0) then
         sum_iovrsd_s(mbn_s) = sum_iovrsd_s(mbn_s) + aih/sdih
         n_iovrsd_s(mbn_s) = n_iovrsd_s(mbn_s) + 1
      endif
c
      return
      end
      subroutine ad6sts(sumDI2o, nuall, aih, janom, mbn_s,
     $   sumDI2b, sumDI2p, nupls, aihpls, sumDI2m, numns, aihmns,
     $   scatob, scatmn)
c     =============================
c
c  Add some things into sums for statistics
c For PCV
c
c On entry:
c  sumDI2o     sum (DelI**2) for all observations
c  nuall       number of observations
c  aih         scaled intensity <I>
c  janom        = 0 if this reflection has no anomalous
c                    (anomalous off or centric)
c               else = 1
c  mbn_s       bin on resolution
c  sumDI2b     sum (DelI**2) for I+&I- observations (janom = 0)
c  sumDI2p     sum (DelI**2) for I+ observations
c  nupls       number of I+ observations
c  aihpls      scaled intensity <I+>
c  sumDI2m     sum (DelI**2) for I- observations
c  numns       number of I- observations
c  aihmns      scaled intensity <I->
c
c On exit:
c  scatob      population scatter = sqrt(Sum(DelI**2)/(n-1))
c               appropriately selected on I+ & I-
c  scatmn      sd of mean estimated from scatter
c               = population scatter / sqrt(n)
c
      real sumDI2o, aih, sumDI2b, sumDI2p, aihpls, sumDI2m, aihmns,
     $   scatob, scatmn
      integer nuall, janom, mbn_s, nupls, numns
c
      include 'parameter.fh'
      include 'stats.fh'
c
      real an, anpop
c
c  relative to overall mean
      an = nuall
      sum_o_wsg_s(mbn_s) =
     $   sum_o_wsg_s(mbn_s) + sqrt((1./(an-1.))*sumDI2o)
      sum_o_im_s(mbn_s) =
     $   sum_o_im_s(mbn_s) + aih
      scatob = 0.0
      scatmn = 0.0
      anpop  = 0.0
c  relative to I+ or I-
      if (janom .eq. 0) then
         an = nuall
         sum_m_wsg_s(mbn_s) =
     $      sum_m_wsg_s(mbn_s) + sqrt((1./(an-1.))*sumDI2b)
         sum_m_im_s(mbn_s) =
     $      sum_m_im_s(mbn_s) + aih
         scatob = sumDI2b
         anpop = nuall
      else
c  I+
         if (nupls .gt. 1) then
            an = nupls
            sum_m_wsg_s(mbn_s) =
     $         sum_m_wsg_s(mbn_s) + sqrt((1./(an-1.))*sumDI2p)
            sum_m_im_s(mbn_s) =
     $         sum_m_im_s(mbn_s) + aihpls
            scatob = scatob + sumDI2p
            anpop  = anpop + an
         endif
c  I-
         if (numns .gt. 1) then
            an = numns
            sum_m_wsg_s(mbn_s) =
     $         sum_m_wsg_s(mbn_s) + sqrt((1./(an-1.))*sumDI2m)
            sum_m_im_s(mbn_s) =
     $         sum_m_im_s(mbn_s) + aihmns
            scatob = scatob + sumDI2m
            anpop  = anpop + an
         endif
      endif
c
      if (anpop .gt. 1.1) then
         scatob = sqrt(scatob/(anpop-1.0))
         scatmn = scatob/sqrt(anpop)
      endif
c
c
      return
      end
c
c
c
      subroutine adcllb
     $   (icol, jcol, ncolou, labels, labelu, ctypes, ctypeu, nconew)
c     =================
c
c Add next optional column label & type for assignment
c
      integer icol, jcol, ncolou,  nconew
      character*(*) labels(*), labelu(*), ctypes(*), ctypeu(*)
c
      icol = icol+1
      ncolou = ncolou+1
      labels(icol) = labelu(jcol)
      ctypes(icol) = ctypeu(jcol)
      nconew = ncolou
      return
      end
      subroutine adfsts(DelI, aih, mbn_f, mbn_i, maxmbi)
c     ==========================================
c
c  Add some things into sums for statistics
c    Full/partial Bias v. fractioncalc, for true fulls and 2-part partials only
c
c On entry:
c  DelI         deviation, <Ifull> - Ihl(partial),
c                where <Ifull> is scale-weighted mean of fulls
c                 for true fulls and 2-part partials only
c  aih          mean intensity Imean
c  mbn_f        bin on fractioncalc
c  mbn_i        bin on intensity
c  maxmbi       maximum intensity bin number: accumulate totals
c                for top intensity bin

      real    DelI, aih
      integer  mbn_f, mbn_i, maxmbi
c
      include 'parameter.fh'
      include 'stats.fh'
c
c  DelI = bias
      sum_b_bias_c(mbn_f, 1) = sum_b_bias_c(mbn_f, 1) + DelI
      n_b_bias_c(mbn_f, 1) = n_b_bias_c(mbn_f, 1) +  1
c  <Imean>
      sum_b_i_c(mbn_f, 1) = sum_b_i_c(mbn_f, 1) + aih
c Sums for top intensity bin
      if (mbn_i .eq. maxmbi) then
c  DelI = bias
         sum_b_bias_c(mbn_f, 2) = sum_b_bias_c(mbn_f, 2) + DelI
         n_b_bias_c(mbn_f, 2) = n_b_bias_c(mbn_f, 2) +  1
c  <Imean>
         sum_b_i_c(mbn_f, 2) = sum_b_i_c(mbn_f, 2) + aih
      endif 
      return
      end
      subroutine adsdst(sdffrf0, sdffrf, mbn_i, irun, m, swihc)
c    ===============
c
c    Statistics of SD correction (including singletons)
c
c On entry:
c  sdffrf0   original uncorrected SD
c  sdffrf    corrected SD
c  mbn_i     intensity bin number
c  irun      run
c  m         = 0 full, = +1 partial
c  swihc     mean intensity used for correction
c
      real sdffrf0, sdffrf
      integer  mbn_i, irun, mrf
c
      include 'parameter.fh'
      include 'stats.fh'
c
      integer j
c
      if (sdffrf0 .gt. 0.0) then
         corratio = sdffrf/sdffrf0
c
         j = m+1
         sum_a_i_rf(mbn_i, irun, j) = sum_a_i_rf(mbn_i, irun, j) +
     $      corratio
         n_a_i_rf(mbn_i, irun, j) = n_a_i_rf(mbn_i, irun, j) + 1
         smax_a_i_rf(mbn_i, irun, j) =
     $      max(smax_a_i_rf(mbn_i, irun, j), corratio)
         smin_a_i_rf(mbn_i, irun, j) =
     $      min(smin_a_i_rf(mbn_i, irun, j), corratio)
      endif
      return
      end
      subroutine adssts(scalfc, Dfhl, nprtds, irun, mbn_t, mbn_s)
c     ========================================================
c
c  Add some things into sums for statistics
c    Mean applied scales
c
c On entry:
c  scalfc       scale factor (= (input scale) / ghl)
c  Dfhl         tails correction, if set
c  nprtds       .gt. 0 if Dfhl valid
c  irun         run number
c  mbn_t        bin on rotation overall
c  mbn_s        bin on resolution

      real     scalfc, Dfhl
      integer  nprtds, irun, mbn_t, mbn_s
c
      include 'parameter.fh'
      include 'stats.fh'
c
c  Mean scale factor (inverted already)
      sum_a_k_ts(mbn_t,mbn_s) = sum_a_k_ts(mbn_t,mbn_s) + scalfc
      n_a_k_ts(mbn_t,mbn_s) = n_a_k_ts(mbn_t,mbn_s) + 1
      n_o_t(mbn_t) = n_o_t(mbn_t) + 1
c  Mean Dfhl
      sum_a_t_rs(irun,mbn_s) = sum_a_t_rs(irun,mbn_s) + 1./Dfhl
      n_a_t_rs(irun,mbn_s) = n_a_t_rs(irun,mbn_s) + 1
c  Min & max Dfhl, for each run
      exm_t_r(1,irun) = min(exm_t_r(1,irun), 1./Dfhl)
      exm_t_r(2,irun) = max(exm_t_r(2,irun), 1./Dfhl)
c
      return
      end
      subroutine anlini(maxmtx, amatrx, eigval, eigvec, vecinv, work)
c     ==============================================================
c
c  Initial analysis: tabulate overlaps between rotation ranges,
c     and get estimate of initial scale factors
c
      include 'bigarg.fh'
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'rfile.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'refcon.fh'
      include 'flags.fh'
      include 'dump.fh'
      include 'column.fh'
      include 'glocon.fh'
      include 'errors.fh'
      include 'sclinimeans.fh'
c
      logical hkleq
      external hkleq
c
      real  s, ff, sdff, ffpr,sdffpr, xd, yd, zd, time,
     $     sclinp, sdscli, errlim, philm1, philm2, frccal, rotrng,
     $     clp, sumins, phierrmax, ffi, temp
      integer jhkl(3), ihkl(3), nrs, m, mpart, isym, ibatch, istat,
     $     isplit, ireso, i1, i2, jserbt, kbatsc, nimgm1, npherr,
     $     nsumin, imidan, ninrfr, nfull, jser, kpflag, nxyerr,
     $     maxnpherr, iparnorm
      logical eof, lpart, first, lmxref, lmpt10(maxrun), lsanom,
     $   luseit
c
      real ss, stol, ffrf(maxrfl), sdffrf(maxrfl),
     $   ffprrf(maxrfl), sdfprf(maxrfl), frclrf(maxrfl),
     $   xdrf(maxrfl), ydrf(maxrfl), zdrf(maxrfl), sclini(10),
     $   ghl(maxrfl), phirf(maxrfl), xdetrf(maxrfl), ydetrf(maxrfl),
     $   dghldc(maxpmr,maxrfl),
     $   sclirf(maxrfl), sdscrf(maxrfl), clprf(maxrfl),
     $   timerf(maxrfl), xsirf(maxrfl), Dfhl(maxrfl),
     $   datarf(maxcol,maxrfl)
      integer irunrf(maxrfl), isttrf(maxrfl), mrf(maxrfl),
     $   isymrf(maxrfl), ibatrf(maxrfl), mprf(maxrfl), kmptrf(maxrfl),
     $   nmprrf(maxrfl), jmprrf(maxrfl), lflgrf(maxrfl), iserrf(maxrfl)
c
      integer irun, i, j, k, l, nmtrng, nuse, nsplit, lrflag, novtot
      integer nsrmax, NsumI(maxrot, maxres, maxrun), indx(maxrfl),
     $     i1ovlp(maxrun), nobspar(maxbat),
     $     nrrang(maxrun), icnres(maxres), nmeanI, nrejpt(6)
      real sumI(maxrot, maxres, maxrun), scnorm, ameanI,
     $    sclrun(maxrun), sclbtc(maxbat), sclfac
c
      character label(maxrot*maxrun)*6, lab*6, dna_label*80
c
      external get_anl_imean
      integer ccp4_hash_lookup
      external ccp4_hash_lookup

c
c-save-      save /scales/
c
      maxnpherr = 30
      phierrmax = 0.0
      nerpdc = 0
      errlim = 1.0e-6
      nsrmax = 0
      npherr = 0
      nxyerr = 0
      ninrfr = 0
      novtot = 0
      nrejpt(1) = 0
      nrejpt(2) = 0
      nrejpt(3) = 0
      nrejpt(4) = 0
      nrejpt(5) = 0
      nrejpt(6) = 0
c Keep partials?
      if (kprsum .eq. -1) then
c  fulls only 
         kpflag = -1
      elseif (kprsum .eq. 0) then
c  fulls + previously scaled partials
         kpflag =  0
      else
c  everything
         kpflag = +1
      endif
c Separate I+ & I- in scaling if kanoms = +1
      if (kanoms .ge. +1) then
         lsanom = .true.
      else
         lsanom = .false.
      endif
      sumins = 0.0
      nsumin = 0
      do i = 1,maxpmr
         do j = 1, maxrfl
            dghldc(i,j) = 0.0
         enddo
      enddo
c
c     i1ovlp is a pointer to the 1st entry for
c     each run. nmtrng is the number of rotation ranges in all runs
c--- This piece of code is partly copied in s/r mtxsiz
      nmtrng = 1
      kbatsc = 0
      do 1, irun = 1, numrun
         i1ovlp(irun) = nmtrng
         if (mbatsc(irun) .ne. 0) then
            if (lbtslp(irun)) then
c     Slope
               nrrang(irun) = max(1, ncscls(1,irun)/2)
            else
c     Plain batch
               nrrang(irun) = max(1, ncscls(1,irun))
            endif
            if (kbatsc .eq. 0) then
c      no runs so far "batch" mode
               kbatsc = +1
            elseif (kbatsc .lt. 0) then
               kbatsc = +2
            endif
         else
            nrrang(irun) = max(1, ncscls(1,irun) - 1)
            if (kbatsc .eq. 0) then
               kbatsc = -1
            elseif (kbatsc .gt. 0) then
               kbatsc = -2
            endif
         endif
         nmtrng = nmtrng + nrrang(irun)
c
         lmpt10(irun) = .false.
 1    continue
      nmtrng = nmtrng-1
c
c kbatsc = +1  all runs are batch mode
c        = -1  no runs are batch mode
c        = +/-2 mixed
c
      if (nmtrng .gt. maxmtx) then
         write (lunout, '(//a,i8,a,i8,a//)')
     $        ' **** Too many rotation ranges for analysis:- ',nmtrng,
     $        ', maximum is ',maxmtx,' ****'
         call dna_error('too many rotation ranges')
         call ccperr(1,'** Too many rotation ranges **')
      endif
c
      call clrsts
      call ccpzr(sumI,maxres*maxrot*maxrun)
      call ccpzi(NsumI,maxres*maxrot*maxrun)
c  Clear amatrx
      do 10 j = 1,nmtrng
         do 11, i = 1, nmtrng
            amatrx(i,j) = 0.0
 11      continue 
 10   continue 
c
      call lrrewd(1)
      first = .true.
      lmxref = .false.
c
c  Tell sgetref that we're starting to read the file
      eof = .true.
c     
cooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c     Loop reflections
c   Get all observations of this reflection
c     
 20   call sgetref(lsanom, maxrfl, ninrfr, .true., kpflag, .true.,
     $   eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $   ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $   isymrf, sclirf, sdscrf, lflgrf, frclrf, clprf, isttrf,
     $   timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $   nmprrf, datarf, istat)
      if (istat .eq. +1) then
c  Beyond resolution limits
         go to 90
      endif
      if (nrs .gt. 0) then
         do 25, l = 1,nrs
            ghl(l) = 1.0
            irun  = irunrf(l)
            m     = mrf(l)
            mpart = mprf(l)
            zd    = phirf(l)
            if (lbtsmooth(irun)) then
               zd = ibatrf(l)
            endif
c  Internal batch number within run, jserbt
            jser = ccp4_hash_lookup(ibatrf(l))
            jserbt = isrrbt(jser)
c
            ibatch = ibatrf(l)
            isym = isymrf(l)
c  keep reflections rejected only because too weak, reject too strong
c     for mean intensity analysis
c     (these have istat = lqweak or lqstrn)
            if (isttrf(l) .eq. lqstrn) then
c   Change flag for "Strong" observations to reject
               isttrf(l) = lqstrj
            endif
c Store flags for mpart = 10 found for any run
            if (mpart .eq. 10) then
               lmpt10(irun) = .true.
            endif
c     
            if (irun .ne. irefrn) then
c Check for valid rotation value
c     Check for reflection with "rotation" outside allowed range. 
c     This shouldn't happen, but occasionally has.
c Reflections flagged with M = 10 are summed partials and may occur in the
c     next batch (since they are always allocated to the 1st image)
c Same applies to any partials
               if (m .ne. 0 .or. mpart .eq. 10) then
c Get image rotation range
c   errlim = small number
                  rotrng = philim(2,jserbt,irun) - philim(1,jserbt,irun)
c how many images is this partial split into? nimgm1 is number - 1
c  packing factor may be either 10 or 100
                  if (mpart .gt. 200) then
                     nimgm1 = mpart/100 - 1
                  elseif (mpart .gt. 20) then
                     nimgm1 = mpart/10 - 1
                  else
                     nimgm1 = 1
                  endif
                  philm1 = philim(1,jserbt,irun) -
     $               rotrng * nimgm1
                  philm2 = philim(2,jserbt,irun) +
     $               rotrng * nimgm1
               else
                  philm1 = philim(1,jserbt,irun)
                  philm2 = philim(2,jserbt,irun)
               endif
               if (philm1 .gt. philm2) then
c  force philm1 < philm2 for test
                  temp = philm1
                  philm1 = philm2
                  philm2 = temp
               endif
               philm1 = philm1-errlim
               philm2 = philm2+errlim
c Check rotation range?
               if (ncz .ne. 0 .and. abs(philm2-philm1) .gt. 0.01) then
                  if ((philm1-zd) .gt. 0.01 .or.
     $                (zd-philm2) .gt. 0.01) then
                     phierrmax = max(phierrmax,
     $                  min(abs(zd-philm1), abs(zd-philm2)))
                     if (npherr .le. maxnpherr) then
                        write (lunout,
     $                     '(/a,a/a,3i4,a,i4,a,i8/a,f12.4,a,2f12.4/)')
     $                ' **** Reflection rotation value outside range ',
     $                     'in Batch header ****',
     $         ' Reflection ',jhkl, ' Isym = ', isym, ' Batch', ibatch,
     $                  ' Rotation', zd,
     $                  ' should be in range ',
     $                     philm1, philm2
                        npherr = npherr + 1
                        if (npherr .gt. maxnpherr) then
                           write (lunout,'(/a/a,i4,a/)')
     $                        ' WARNING  WARNING  WARNING  WARNING ',
     $                        ' **** More than ', maxnpherr,
     $                        ' Phi range errors ****'
                        endif
                     endif
                  endif
               endif
c Check for valid detector coordinates, but only if they are going to be used
               if (ltsdet .and. ncscls(2,irun) .gt. 1) then
                  xd = xdetrf(l) 
                  yd = ydetrf(l)
                  if (
     $               xd .lt. dlimrn(1,1,irun) .or.
     $               xd .gt. dlimrn(2,1,irun) .or.
     $               yd .lt. dlimrn(1,2,irun) .or.
     $               yd .gt. dlimrn(2,2,irun)) then
                     write (lunout, 
     $                  '(//a,a/a,3i4,a,i4,a,i8/a,2f9.3,a,4f9.3/)')
     $           ' **** Reflection detector cordinates outside range ',
     $                  'in Batch header ****',
     $         ' Reflection ',jhkl, ' Isym = ', isym, ' Batch', ibatch,
     $                  ' Xdet, Ydet: ', xd, yd,
     $                  ' should be in range ',
     $                  ((dlimrn(i,j,irun),i=1,2),j=1,2)
                     nxyerr = nxyerr + 1
                     if (nxyerr .gt. 50) then
                        write (lunout,'(/a/)')
     $                     ' **** Too many XY range errors ****'
                        call dna_error('reflection coordinates '//
     +                       'out of range')                        
                        call ccperr(1,
     $         ' *** Reflection detector coordinates out of range ***')
                     endif
                  endif
               endif
            endif
 25      continue
      endif
c
      if (nrs .gt. 0) then
c Process partials if required
         nuse = nrs
         call partls(.true.,
     $      jhkl, kprsum, lpart, stol, nrs, nprtds, ltdsrn,
     $      ffrf, sdffrf, ffprrf, sdfprf,
     $      ghl, Dfhl, xdrf, ydrf, zdrf, phirf, xsirf,
     $      frclrf, kmptrf, isttrf, mrf, mprf, isymrf, ibatrf, irunrf, 
     $      nmprrf, jmprrf, kprflg, kprmis, mxprwd, imid,
     $      nrejpt, nuse, prthld, prlims, prmaxs, dghldc, .true.,
     $      lavpsc)
c&&         if (nuse .le. 1) go to 90
c Check summed or scaled partials against intensity limits
         call  chkilm(
     $      nrs, ffrf, sdffrf, isttrf, irunrf, nmprrf, sclirf, istat)
c     Reject all observations of this reflection if any is too strong
         if (istat .eq.lqstrn) then
            go to 90
         endif
c...................................................................
c sums for previous reflection even if only one measurement
         ireso = min(int(ss/srange) + 1, maxres)
         do 30, l = 1,nrs
            irun = irunrf(l)
c  Count 2nd & subsequent parts of partials if it will be used in scaling
c   ie  (a) batch scales for this run
c       (b) partials included in refinement (kprsum .gt. 0)
            if (isttrf(l) .ge. lqOK .or.
     $         (kprsum .gt. 0 .and. jmprrf(l) .gt. 0 .and.
     $         mbatsc(irun) .ne. 0)) then
               if (mbatsc(irun) .ne. 0) then
                  if (lbtslp(irun)) then
                     isplit =
     $                    min(nint(zdrf(l)/btslpf), nrrang(irun))
                  else
                     isplit = min(nint(zdrf(l)), nrrang(irun))
                  endif
               else
                  isplit = max(1, min(int(zdrf(l)) + 1, nrrang(irun)))
               endif
               nsrmax = max(nsrmax, ireso)
c     Sums for mean intensity in rotation ranges & resolution
c       use (a) fulls 
c           (b) partials if used in scaling
c               if batch mode, use every part of partial but use the summed 
c               intensity
               luseit = .false.
               if (nmprrf(l) .eq. 1) then
c  Full
                  ffi = ffrf(l)
                  luseit = .true.
               elseif (kprsum .gt. 0) then
c  Partials being used
                  if (jmprrf(l) .eq. 0) then
c     1st slot
                     ffi = ffrf(l)
                     luseit = .true.
                  elseif (jmprrf(l) .gt. 0) then
c     subsequent slots, pick up I from 1st slot
                     ffi = ffrf(jmprrf(l))
                     luseit = .true.
                  endif
               endif 
               if (luseit) then
                  sumI(isplit,ireso,irun) =
     $               sumI(isplit,ireso,irun) + ffi
                  NsumI(isplit,ireso,irun) =
     $               NsumI(isplit,ireso,irun) + 1
                  sumins = sumins + ffi
                  nsumin = nsumin + 1
               endif
               if (nuse .gt. 1) then
                  indx(l) = i1ovlp(irun) + isplit - 1
c     only include reflections which will be used for scaling
                  if (l .gt. 1) then
                     do 40, m = 1,l-1
                        if (isttrf(m) .eq. lqOK .or.
     $                     (kprsum .gt. 0 .and. jmprrf(m) .gt. 0 .and.
     $                     mbatsc(irunrf(m)) .ne. 0)) then
                           novtot = novtot +1
                           amatrx(indx(l), indx(m)) =
     $                        amatrx(indx(l), indx(m)) + 1
                           if (indx(l) .ne. indx(m))
     $                        amatrx(indx(m), indx(l)) =
     $                        amatrx(indx(m), indx(l)) + 1
                        endif
 40                  continue
                  endif
               endif
            endif
 30      continue
      endif
c     
c     
 90   if (.not.eof) go to 20
c     
c--------------------------------------------------------------------
c  end of reflections
 100  continue
c Phi range errors      
      if (npherr .gt. maxnpherr) then
         write (lunout, '(/a,f10.3,a/)')
     $      ' *** Maximum phi range error = ', phierrmax, ' degrees ***'
      endif
c
      if (lovmap) write (lunout, 6100)
 6100 format(//
     $        ' Matrix of number of overlaps between rotation',
     $        ' ranges for each run,'/'  for reflection observations',
     $        ' which will be used for scaling',/,
     $        ' Each group is labelled "run_number.range"'/)
      if (kbatsc .eq. +1) then
c     all batch mode
         write (lunout, '(a//)')
     $        ' Each rotation range is one batch'
      elseif (kbatsc .eq. -1) then
c     none batch mode
         write (lunout, '(a,a//)')
     $        ' Note that the number of ranges is one less than the',
     $        ' number of scale factors'
      else
         write (lunout, '(a,a//)')
     $        ' Note that the number of ranges is one less than the',
     $        ' number of scale factors except for runs in "Batch" mode'
      endif
c
      k = 0
      do 110, irun = 1, numrun
         do 115, i = 1, nrrang(irun)
            k=k+1
            write (lab, '(i2,a1,i3)') irunum(irun), '.', i
            call squeez(lab, label(k))
 115     continue
 110  continue
c
      call provlm(amatrx, maxmtx, nmtrng, label, kprint(3), istat)
      if (novtot .le. 0) then
         write (lunout, '(/a/)')
     $      ' **** FATAL ERROR: no overlaps anywhere ****'
         istat = -1
      endif
      if (istat .ne. 0) then
         if (kprsum .eq. 0) then 
            write (lunout, '(/a/a,a,a/a/)')
     $         ' **** You have selected INTENSITIES FULL [by default]',
     $         ' **** If the dataset contains partially recorded,',
     $         ' reflections, you can include them in scaling',
     $         ' with the command INTENSITIES PARTIALS',
     $         ' **** This may increase the number of overlaps ****'
         endif
         write (lunout, '(/a/a/)')
     $      ' ====| WARNING WARNING WARNING WARNING WARNING  |====',
     $      '       At least one part of data has no overlaps'
cc         call ccperr(1,'**** No overlaps !!! ****')
      endif
c
      write (lunout, '(///)')
c     
      if (lovmap) then
         call wrovlm(amatrx, eigvec, nmtrng, maxmtx)
      endif
c     
c     Get eigenvalues & eigenvectors of overlap matrix (double precision
c     now)
c      call ea06cd(
c     $     amatrx, eigval, eigvec, nmtrng, maxmtx, maxmtx, work)
c     
c      write (lunout, '(a)') ' Eigenvalues of overlap matrix:'
c      write(lunout,'(100(1X,10G12.3/))') (eigval(i),i=1,nmtrng)
c     
c --- Determine initial scale factors
c

      if (lrstr) then
c  -- Restored from file, so don't do anything
         continue
c
      elseif (iniscl .eq. 0) then
c  -- Set from mean I
c  Use these to get mean intensity, hence initial scale factors
c First bin means over all batches in each run (ie get run scales)
c if more than one run
         if (numrun .eq. 1) then
            sclrun(1) = 1.0
         else
            do 300, irun = 1, numrun
               do 320, ireso = 1, nsrmax
                  aimean(ireso, irun) = 0.0
                  nimean(ireso, irun) = 0
                  do 310, isplit = 1, nrrang(irun)
                     aimean(ireso, irun) = aimean(ireso, irun)
     $                  + sumI(isplit,ireso,irun)
                     nimean(ireso, irun) = nimean(ireso, irun)
     $                  + NsumI(isplit,ireso,irun)
 310              continue
                  if (nimean(ireso, irun) .gt. 0) then
                     aimean(ireso, irun) =
     $                  aimean(ireso, irun)/float(nimean(ireso, irun))
                  endif
 320           continue
 300        continue
c
            if (knormk .gt. 0) then
               iparnorm = knormk
            else
               iparnorm = 1
            endif
            nmnresmax = nsrmax
            nmngroup = numrun
            kmncount = 0
            call FoxHolmes(
     $         get_anl_imean, numrun, iparnorm, sclrun, nobspar,
     $         maxmtx, amatrx, eigval, eigvec, vecinv,  work)
c            call sclbat(aimean, nimean, maxres, nsrmax, numrun, sclrun,
c     $         maxmtx, amatrx, eigval, eigvec, vecinv,  work)
         endif
c
c Now get scales within each run:
c note that cscals array contains dividing scales
         do 400, irun = 1, numrun
            if (nrrang(irun) .eq. 1) then
               do i = 1,ncscls(2,irun)
                  cscals(i, 1, irun) = sclrun(irun)
               enddo
            else
               if (mbatsc(irun) .ne. 0) then
                  nsplit = nrrang(irun)
               else
                  nsplit = nrrang(irun) + 1
               endif
               do isplit = 1, nsplit
                  aimean(ireso, isplit) = 0.0
                  nimean(ireso, isplit) = 0
               enddo
               do 410, isplit = 1, nsplit
                  do 420, ireso = 1, nsrmax
                     if (mbatsc(irun) .ne. 0) then
c Batch mode, sums apply directly to range
                        if (NsumI(isplit,ireso,irun) .gt. 0)
     $                     aimean(ireso, isplit) =
     $                     sumI(isplit,ireso,irun)/
     $                     float(NsumI(isplit,ireso,irun))
                        nimean(ireso, isplit) =
     $                     NsumI(isplit,ireso,irun)
                     else
c Rotation mode, scales are before & after each range, on the boundaries, 
C so average the middle ones & set the end ones 
                        if (isplit .eq. 1) then
                           if (NsumI(isplit,ireso,irun) .gt. 0)
     $                        aimean(ireso, isplit) =
     $                        sumI(isplit,ireso,irun)/
     $                        float(NsumI(isplit,ireso,irun))
                           nimean(ireso, isplit) =
     $                        NsumI(isplit,ireso,irun)
                        elseif (isplit .eq. nrrang(irun)+1) then
                           if (NsumI(isplit-1,ireso,irun) .gt. 0)
     $                        aimean(ireso, isplit) =
     $                        sumI(isplit-1,ireso,irun)/
     $                        float(NsumI(isplit-1,ireso,irun))
                           nimean(ireso, isplit) =
     $                        NsumI(isplit-1,ireso,irun)
                        else
                           nimean(ireso, isplit) =
     $                        (NsumI(isplit-1,ireso,irun) +
     $                        NsumI(isplit,ireso,irun))
                           if (nimean(ireso, isplit) .gt. 0)
     $                        aimean(ireso, isplit) =
     $                        (sumI(isplit-1,ireso,irun) +
     $                        sumI(isplit,ireso,irun))/
     $                        float(nimean(ireso, isplit))
                        endif
                     endif
 420              continue
 410           continue
c   get scales within run into sclbtc
               iparnorm = 1
               nmnresmax = nsrmax
               nmngroup = nsplit
               kmncount = 0
               call FoxHolmes(
     $            get_anl_imean, nsplit, iparnorm, sclbtc, nobspar,
     $            maxmtx, amatrx, eigval, eigvec, vecinv,  work)
c               call sclbat(
c     $            aimean, nimean, maxres, nsrmax, nsplit, sclbtc,
c     $            maxmtx, amatrx, eigval, eigvec, vecinv,  work)
c     
               do 460, isplit = 1, nsplit
                  do 450, i = 1,ncscls(2,irun)
                     sclfac = sclrun(irun) * sclbtc(isplit)
                     if (lbtslp(irun)) then
                        cscals(i, 2*isplit-1, irun) = sclfac
                        cscals(i, 2*isplit, irun) = sclfac
                     else
                        cscals(i, isplit, irun) = sclfac
                     endif
 450              continue
 460           continue
            endif
 400     continue
c------
      elseif (iniscl .eq. 2) then
         do 470, irun = 1, numrun
            do 480, j = 1,ncscls(1,irun)
               do 481, i = 1,ncscls(2,irun)
                  if (sclset(irun) .gt. 0.0) then
                     cscals(i,j,irun) = 1.0 / sclset(irun)
                  else
                     cscals(i,j,irun) = 1.0
                  endif
 481           continue
 480        continue
 470     continue
      endif
      if (iniscl .ne. 1 .and. .not.lrstr) then
c     
c Print out scales
         call dna_table_start('initial_scales')
         do 500, irun = 1, numrun
            write (lunout, '(//a,i5)')
     $           ' Initial scales for run ', irunum(irun)
            i = 0

            call dna_list_start('run')
            call dna_integer_item('run_number', irun)
            call dna_integer_item('number_of_scales',
     +           irunum(irun))

 1001       format('scale_', i1)
 1002       format('scale_', i2)
 1003       format('scale_', i3)
            
            do 510, isplit = 1, ncscls(1,irun)
               dna_label = ' '
               if (isplit .lt. 10) then 
                  write(dna_label, 1001) isplit
               else if (isplit .lt. 100) then 
                  write(dna_label, 1002) isplit
               else 
                  write(dna_label, 1003) isplit
               end if

               i = i+1
               if (cscals(1,isplit,irun) .ne. 0.0) then
                  sclini(i) = 1.0 / cscals(1,isplit,irun)
               else
                  sclini(i) = 0.0
               endif

               call dna_real_item(dna_label,
     +              sclini(i))


               if (i .eq. 10 .or. isplit .eq. ncscls(1,irun)) then
                  write (lunout, 6500)
     $                 (sclini(j), j = 1, i)
 6500             format(1x,10f10.4)
                  i = 0
               endif
 510        continue

            call dna_list_end

 500     continue

         call dna_table_end

         write (lunout, '(//)')
      endif
c     
c Check & warn if MPART=10 reflections (previously summed partials)
c are present for run with BATCH scaling
      do 600, irun = 1,numrun
         if (lmpt10(irun) .and. (mbatsc(irun) .ne. 0)) then
            write (lunout, 6601) ' || WARNING || for run ',
     $         irun,': BATCH scaling is not recommended for data ',
     $         ' containing previously summed partials'
 6601       format (/a,i4,a,a,/)
         endif
 600  continue 
c
c Average intensity
      if (nsumin .gt. 0) avgins = sumins/float(nsumin)
c Clear error print flag
      if (nerpdc .gt. 0) lerpdc = .false. 
c
      return
      end
      subroutine anomcn(icntrc, nupls, numns, multcy, mbn_s)
c    ===============================================
c
c Count anomalous data for completeness calculation
c
c On entry:
c  icntrc      centric flag, = 1 centric else   = 0
c  nupls       number of I+ observations
c  numns       number of I- observations
c  multcy      number of times this reflection will occur in a
c               complete sphere of data
c  mbn_s       bin on resolution
c
      integer icntrc, nupls, numns, multcy, mbn_s
c
      include 'parameter.fh'
      include 'stats.fh'
c
      if (icntrc .eq. 0) then
c  Acentric
         n_a_acen_s(mbn_s) =n_a_acen_s(mbn_s) + multcy
         if (nupls .gt. 0 .and. numns .gt. 0) then
c  count acentrics with anomalous difference (ie both I+ and I-)
            n_a_anom_s(mbn_s) =n_a_anom_s(mbn_s) + multcy
         endif
      else
c  Centric
         n_a_cent_s(mbn_s) =n_a_cent_s(mbn_s) + multcy
      endif
      return
      end
      subroutine bangle(rotmtx, rotang)
c     =================================
c
c  Get rotation angle of principle axes from eigenvector matrix
c
c On entry:
c  rotmtx(2,2)   eigenvector matrix (ration) NB double precision
c                This matrix is not checked in any way
c
c On exit:
c  rotang        angle of rotation in degrees
c
      double precision rotmtx(2,2)
      real rotang
c
      real rtod
c
      rtod = 45./atan(1.)
      rotang = rtod * atan2(rotmtx(2,1), rotmtx(1,1))
      return
      end
c
c
c
      subroutine batclw(ibatch, bcell, lbcell, wavel, amosaic, istat)
c     ===================================================
c
c  Return batch cell & wavelength information from orientation block
c
c  On entry:
c     ibatch     batch number
c
c  On exit
c     bcell(6)   cell
c     lbcell(6)   cell flags: = -1 variable, = 0 fixed,
c                            > 0 constrained to other (eg b=a)
c     wavel      wavelength
c     amosaic    mosaicity
c     istat           status: =  0 valid orientation block
c                             = +1 valid cell, no wavelength
c                             .lt. 0, invalid data
c                             = -1 no valid cell or wavelength
c
c
      implicit none
c
      integer ibatch, lbcell(6), istat
      real bcell(6), wavel, amosaic
c
      include 'orient.fh'
c
c Locals
      integer i, j, iprint
c
c clear return values
      call ccpzr(bcell, 6)
      call ccpzi(lbcell, 6)
      wavel = 0.0
c Read orientation data
      call lrbres(1, ibatch)
      iprint = 0
C     Read orientation block
      call lrbat(1,j,nwords,btitle,iprint)
c
c Is this a valid orientation block?
      if (nwords. le. 0) then
c No, return status flag: this may be acceptable for reference data, or
c     for batch mode
         istat = -1
         return
c        ======
      endif
c
c Check for null cell
      istat = +1
      do  i = 1,6
         if (cell(i) .lt. 0.001) then
            istat = -1
         else 
            bcell(i) = cell(i)
            lbcell(i) = lcell(i)
         endif
      enddo
      if (alambd .gt. 0.001) then
         wavel = alambd
         istat = 0
      endif
      amosaic = etad
c
      return
      end
      subroutine batinf(ibatch, ttime1, ttime2, pphi1, pphi2, delphi,
     $   xydlim, dtmmat, e1e2mat, e1e2inv, phi3scan, ubmat, umat, bmat,
     $    wvlnth, ss0, lphi, ldet, lmat, istat)
c     =======================================================
c
c  Return batch information from orientation block
c
c  On entry:
c     ibatch     batch number
c
c  On exit
c     ttime1,ttime2   start, stop time
c     pphi1, pphi2    start, stop rotation (if lphi .true.)
c     delphi          phi range, if present in file, else = 0
c     xydlim(2,2)     min/max, x/y detector limits (if ldet .true.)
c     dtmmat(3,3)     goniostat matrix at datum setting (typically Identity)
c          Cases: single axis (phi)  [D] = [Phi0]
c                 3-axis:
c                   omega scan   [D] = [E1-0][E2][E3] =
c                        [Omega0][Kappa][Phi] or [Omega0][Chi][Phi]
c                   phi scan     [D] = [Phi0]
c     e1e2mat(3,3)    for 3-axis phi scans [E1][E2], else [I]
c     e1e2inv(3,3)      [E1E2]^-1
c     phi3scan        != 0 if 3-axis phi scan, +1 for positive, -1 negative
c                       (ie sign of e3(3))
c     ubmat(3,3)      setting matrix UB  (if lmat .true.)
c     umat(3,3)       orientation matrix U
c     bmat(3,3)       orthogonalisation matrix B
c     wvlnth          wavelength
c     ss0(3)          source vector
c     lphi            .true. if valid phi limits are returned
c     ldet            .true. if valid detector limits are returned
c     lmat            .true. if valid orientation data are returned
c     istat           status: =  0 valid orientation block
c                             = +1 valid orientation data but
c                                   Umat has negative determinant
c                                   (ie inverts hand)
c                             = -1 no valid orientation data
c
c
      implicit none
c
      integer ibatch, istat, phi3scan
      real ttime1, ttime2, pphi1, pphi2, delphi,
     $   xydlim(2,2),
     $   dtmmat(3,3), e1e2mat(3,3), e1e2inv(3,3), ubmat(3,3), umat(3,3),
     $   bmat(3,3), ss0(3),
     $   wvlnth
      logical lphi, ldet, lmat
c
      include 'orient.fh'
c
c Locals
      real pxyzmt(3,3), amat(3,3), 
     $     axisx(3), axisy(3), axisz(3), dtor, tmat(3,3), detrmn
      real e0(3), Q(3,3)
      integer i, j, iprint, iaxes(3)
      external matmul
c
      data axisx/1.0,0.0,0.0/
      data axisy/0.0,1.0,0.0/
      data axisz/0.0,0.0,1.0/
      data iaxes/1,2,3/
c
      dtor = atan(1.0) / 45.0
c
      lphi = .false.
      ldet = .false.
      lmat = .false.
c Read orientation data
      call lrbres(1, ibatch)
      iprint = 0
      istat = 0
C     Read orientation block
      call lrbat(1,j,nwords,btitle,iprint)
c
c Is this a valid orientation block?
      if (nwords. le. 0 .or. ldtype .le. 0) then
c No, return status flag: this may be acceptable for reference data, or
c     for batch mode
         istat = -1
c clear return values
         ttime1 = 0.0
         ttime2 = 0.0
         pphi1  = 0.0
         pphi2  = 0.0
         delphi = 0.0
         xydlim(1,1) = 0.0
         xydlim(2,1) = 0.0
         xydlim(1,2) = 0.0
         xydlim(2,2) = 0.0
         return
c       ======
      endif
c
c Extract required information
      ttime1 = time1
      ttime2 = time2
      pphi1 = phistt
      pphi2 = phiend
c
      lphi = .false.
      ldet = .true. 
      lmat = .true.
c
      if (phirange .ne. 0.0) then
         delphi = phirange
      else
         delphi = 0.0
      endif
c Check for valid limits (ie phi & time limits not all zero)
      if (abs(pphi2-pphi1) .gt. 0.0001 .or.
     $   abs(ttime2-ttime1) .gt. 0.0001) then
c   Phi OK
         lphi =.true.
      endif
      do 10, j=1,2
         do 11, i=1,2
            xydlim(i,j) = detlm1(i,j)
 11      continue
c Check for valid limits
         if (abs(xydlim(2,j) - xydlim(1,j)) .lt. 0.001) then
            ldet = .false.
         endif
 10   continue
c
c  Single axis system: e1 = Phi
c  Three axis system:  e1 = Omega; e2 = kappa||chi; e3 = Phi
c   Single axis, rotation about e1

c Principle axis is always e1
      call vset(e0, e1)
c Get transformation matrix into Cambridge frame
c  e0 is principle rotation axis
      call mkqmat(source, e0, Q)
c      write (6,6011) ((Q(i,j),j=1,3),i=1,3)
c 6011 format(/'Qmat:',3(/3f10.4)//)
c
c Transform all relevent vectors
      call trnsfmvec(e1, Q)
      call trnsfmvec(e2, Q)
      call trnsfmvec(e3, Q)
      call trnsfmvec(s0, Q)
      call trnsfmvec(source, Q)
c
c  Goniostat Datum matrix
c  Cope with cases of 1, 2 or 3 axes, treat all as 3-axis cases,
c dummy 2nd or 3rd axes as required
      call unitmt(e1e2mat)
      call unitmt(e1e2inv)
      phi3scan = 0
      if (ngonax .eq. 1) then
c One axis, dummy 2nd axis
         datum(2) = 0.0
         call vset(e2, axisy)
      endif
      if (ngonax .lt. 3) then
c One or two axes, dummy 3rd axis
         datum(3) = 0.0
         call vset(e3, axisx)
      else if (jscaxs .eq. 3) then
c  3 axis, Phiscan
c     [E1][E2] = [Omega][Chi/Kappa]
         call gn3cmp(dtor*datum(1), dtor*datum(2), 0.0,
     $      E1, E2, E3, 0, 0, e1e2mat)
         call minv3(e1e2inv, e1e2mat, detrmn)
         if (abs(detrmn - 1.0) .gt. 0.001) then
            lmat = .false.
         endif
c   Phi axis e3 should be 0,0,+/-1
         call unit(e3)
         if (abs(e3(1)) .gt. 0.0001 .or. abs(e3(2)) .gt. 0.0001
     $      .or. abs(abs(e3(3))-1.0) .gt. 0.0001) then
            lmat = .false.
         else 
            phi3scan = +1
            if (e3(3) .lt. 0.0) phi3scan = -1
            datum(1) = 0.0
            datum(2) = 0.0
         endif
      endif
      call gn3cmp(dtor*datum(1), dtor*datum(2), dtor*datum(3),
     $   E1, E2, E3, 0, 0, dtmmat)
c Check determinant
      call minv3(tmat, dtmmat, detrmn)
c
      if (misflg .gt. 0) then
c  Get matrix pxyzmt corresponding to Phxyz
         call gn3cmp(phxyz(1,1)*dtor, phxyz(2,1)*dtor,phxyz(3,1)*dtor,
     $      axisx, axisy, axisz,0, 0, pxyzmt)
c  Amat = Phixyz * U
         call matmul(amat, pxyzmt, umatr)
      else
         call moveit(umatr, amat, 9)
      endif
c  Transform matrix
      call trnsfmmat(amat, Q)
c Check for valid U matrix (amat)
      call minv3(tmat, amat, detrmn)
      if (abs(detrmn - 1.0) .gt. 0.001) then
c  Check for determinant = -1, inversion (very dangerous)
         if (abs(detrmn + 1.0) .gt. 0.001) then
c   not even -1, fail
            lmat = .false.
         else
c    = -1, pass for now, but flag
            istat = +1
         endif
      endif
c
c Check for null cell
      do 20, i = 1,6
         if (cell(i) .lt. 0.001) lmat = .false.
 20   continue
      if (lmat) then
c  Cell matrix
         call mkbmat(cell,bmat,alambd,iaxes)
         wvlnth = alambd
c Check for valid cell
         call minv3(tmat, bmat, detrmn)
         if (detrmn .le. 0.0) then
            lmat = .false.
         endif
c
c  UBmat = Phixyz * U * B
         call matmul(ubmat, amat, bmat)
c  export Umat
         call moveit(amat, umat, 9)
c
         call moveit(s0, ss0, 3)
      endif
c
      if (istat .eq. 0 .and. .not.lmat) istat = -1
c
      return
      end
c
c
c
      subroutine mkqmat(s0, e0, Q)
c    ========================
c
c Make transformation matrix [Q] to convert from orientation block
c frame to standard "Cambridge" frame
c
c source vector s0 (anti-parallel to beam)
c        approximately along -x in Cambridge frame
c principal rotation axis e0
c        along z in Cambridge frame
c
c Returns [Q]
c   x(Cambridge) = [Q] x(input)
c
      real s0(3), e0(3), Q(3,3)
c
      real xq(3), yq(3), zq(3)
      integer i
c
c  xq' = ||-s0|| approximate xq
      do i=1,3
         xq(i) = -s0(i)
      enddo
      call unit(xq)
c  zq = ||e0||
      do i=1,3
         zq(i) = e0(i)
      enddo
      call unit(zq)
c  yq = zq x xq
      call cross(yq, zq, xq)
      call unit (yq)
c  xq = yq x zq
      call cross(xq, yq, zq)
      call unit(xq)
c
      do i=1,3
         Q(1,i) = xq(i)
         Q(2,i) = yq(i)
         Q(3,i) = zq(i)
      enddo
      return
      end
c
c
c
      subroutine trnsfmvec(v, Q)
c    =======================
c transform 3-vector in place
c
c  v = Q v
c
      real v(3), Q(3,3)
c
      real t(3)
c
      call matvec(t, Q, v)
      call vset(v, t)
      return 
      end
c
c
c
      subroutine trnsfmmat(R, Q)
c    =======================
c
c Transform matrix R
c
c  [R]' = [Q][R] [Q]^-1
c
      real R(3,3), Q(3,3)
      real Qinv(3,3), QR(3,3), det
      external matmul

      call minv3(Qinv, Q, det)
      if (det .eq. 0.0) call ccperr(1,
     $   '**** TRNSFMMAT zero determinant ****')

      call matmul(QR, Q, R)
      call matmul(R, QR, Qinv)

      return
      end

      subroutine bemvec(mbeams, ihkl, isym, irun, ibatch, phi,
     $   s0c, s2c)
c     ===============================================
c
c  Determine primary & secondary beam directions in
c unpermuted crystal frame
c
c On entry:
c  mbeams         +1 crystal frame, -1 diffractometer frame
c  ihkl(3)        h, k, l, original indices
c  isym           symmetry number (to get original indices)
c  irun           run number
c  ibatch         batch number
c  phi            rotation angle
c  jserbt         batch number within run
c
c On exit:
c  s0c(3)         direction cosines of primary beam in given frame
c  s2c(3)         direction cosines of diffracted beam in given frame
c
      implicit none
      integer mbeams, ihkl(3), isym, irun, ibatch, jserbt
      real  phi, s0c(3), s2c(3), R(3,3), T(3,3), Rinv(3,3)
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
c
       real    P(3,3), PDU(3,3), dtor, hkl(3), s(3),
     $   PDUinv(3,3), det, phi0, omega0, phi2, omega2
      integer jser, kmat, i
c
      integer ccp4_hash_lookup
       external ccp4_hash_lookup, matmul
c
      dtor = atan(1.)/45.
c
      jser = ccp4_hash_lookup(ibatch)
      jserbt = isrrbt(jser)
c Pointer to matrices
      kmat = jmtxbt(jserbt,irun)
      if (kmat .gt. 0) then
c
c Phi matrix [P]
         call rtzmat(phi*dtor, P)
         if (mbeams .gt. 0) then
c  Crystal frame, [R] = [PDU] = [P] [DU] for primary beam
c                 [T] = [B] for secondary beam
            call matmul(R, P, dumatxs(1,1,kmat))
            call moveit(bmatxs(1,1,kmat), T, 9)
         else
c  Diffractometer frame, [R] = [P]
c                        [T] = [D][U][B] for secondary beam
            call moveit(P, R, 9)
            call matmul(T, dumatxs(1,1,kmat), bmatxs(1,1,kmat))
         endif
c Original indices ihkl
         hkl(1) = ihkl(1)
         hkl(2) = ihkl(2)
         hkl(3) = ihkl(3)
c s(frame) = [T] h
         call matvec(s, T, hkl)
c s0(frame) = [R]**-1  s0
         call minv3(Rinv, R, det)
         if (abs(det - 1.0) .gt. 0.001) then
            call dna_error('internal error')
            call ccperr(1,'*** Determinant error in s/r bemvec ***')
         endif
         call matvec(s0c, Rinv, s0xs(1,kmat))
         call unit(s0c)
c s2(frame) = S - s0
         call vdif(s2c, s, s0c)
         call unit(s2c)
c
c*c*c
cc Polar angles for s0c & s2c
c      call polar(s0c, phi0, omega0, dtor)
c      call polar(s2c, phi2, omega2, dtor)
cc
c      write (13, 6001) ibatch, phi, phi0, omega0, phi2, omega2
c 6001 format (i6,5f10.1)
c*c*c
      else
         call ccpzr(s0c, 3)
         call ccpzr(s2c, 3)
      endif
c
      return
      end
c
c
c
      subroutine polar(s, phi, omega, dtor)
c    ================================
c
c  Get polar angles from direction cosines
c
c On entry:
c  s(3)    vector (unit)
c  dtor    degrees to radians
c
c On exit:
c  phi     logitude (degrees)
c  omega   "latitude" (from pole along z) (degrees)
c
      real s(3), dtor, phi, omega
      real sinomega
c
      omega = acos(sign(min(abs(s(3)),1.0), s(3)))
c
      sinomega = sin(omega)
      if (abs(sinomega) .gt. 0.00000001) then
         phi = atan2(s(1), s(2))
      else 
         phi = 0.0
      endif
c
      phi = phi/dtor
      omega = omega/dtor
c
      return 
      end

      subroutine bfcitp(irun, time, bfc)
c     =================================
c
c  Interpolate anisotropic B-factors on rotation (for printing)
c  This duplicates a small piece of code from s/r sclder
c
c On entry:
c  irun      run number
c  time      time (= rotation usually)
c
c On exit:
c  bfc(3)    interpolated B-factors (yy,yz,zz)
c
      integer irun
      real    time, bfc(3)
c
      include 'parameter.fh'
c     
c     Commons
      include 'scales.fh'
      include 'params.fh'
      include 'refcon.fh'
c
      integer i, j
      real del, p(maxrot), sumqbf(maxbfc), sumq
c
      sumq   = 0.0
c
      do 2, j = 1,nbfpos(irun)
         sumqbf(j) = 0.0
 2    continue

      if (nbfact(irun) .eq. 1) then
         do 5, j = 1,nbfpos(irun)
            bfc(j) = bfactr(j,1,irun)
 5       continue
      else
c
         do 10, i = 1, nbfact(irun)
            del = (posbfc(i,irun) - time)**2/Vwt
            if (del .lt. prblt) then
               p(i) = exp(-del)
               do 11, j = 1,nbfpos(irun)
                  sumqbf(j) = sumqbf(j) + p(i)*bfactr(j,i,irun)
 11            continue
               sumq   = sumq + p(i)
            else
               p(i) = 0.0
            endif
 10      continue
c
         do 20, j = 1,nbfpos(irun)
            bfc(j) = sumqbf(j)/sumq
 20      continue
      endif
      return
      end
      subroutine bfnorm(bf, bnorm)
c     ===========================
c
c  Normalize 2x2 Bfactor matrix to make sum of eigenvalues equal
c
c  [Bf]  = ( B11  B12 )
c          ( B12  B22 )
c  If eigenvalues of [Bf] are v1, v2,
c then bnorm = (v1 + v2)/2
c
c On entry:
c  bf(3)        B11, B12, B22
c
c On exit:
c  bnorm        normalization shift, number to subtract from diagonal
c               Bfactor (B11, B22)
c
      real bf(3), bnorm
c
      integer maxbmt
      parameter (maxbmt = 2)
      double precision B(maxbmt,maxbmt), eigval(maxbmt),
     $   eigvec(maxbmt,maxbmt), w(5*maxbmt)
      integer m, ia, ix, istat
c
      B(1,1) = bf(1)
      B(1,2) = bf(2)
      B(2,1) = bf(2)
      B(2,2) = bf(3)
c
      m = maxbmt
      ia = maxbmt
      ix = maxbmt
c      call ea06cd(B, eigval, eigvec, m, ia, ix, w)
      call dsyev('N', 'L', m, B, ia, eigval, w, 5*maxbmt, istat)
      if (istat .ne. 0) then
         call dna_error('internal error')
         call ccperr(1,'*** Error in bfnorm/dsyev ***')
      endif
c
c
      bnorm = eigval(1) + eigval(2)
      bnorm = bnorm*0.5
      return
      end
      subroutine bfrots(bf, bfcprn, rotang)
c     ====================================
c
c  Analyse Bfactor matrix into principle components
c
c  [Bf]  = ( B11  B12 )
c          ( B12  B22 )
c  If eigenvalues of [Bf] are v1, v2
c
c On entry:
c  bf(3)        B11, B12, B22
c
c On exit:
c  bfcprn(2)    principle components (smaller first)
c  rotang       angle of ellipse to yd (degrees)
c
      real bf(3), bfcprn(2), rotang
c
      integer maxbmt
      parameter (maxbmt = 2)
      double precision B(maxbmt,maxbmt), eigval(maxbmt),
     $   eigvec(maxbmt,maxbmt), w(5*maxbmt), R(2,2)
      integer m, ia, ix, i, j, istat


      B(1,1) = bf(1)
      B(1,2) = bf(2)
      B(2,1) = bf(2)
      B(2,2) = bf(3)
c
      m = maxbmt
      ia = maxbmt
      ix = maxbmt
c      call ea06cd(B, eigval, eigvec, m, ia, ix, w)
      call dsyev('V', 'L', m, B, ia, eigval, w, 5*maxbmt, istat)
      if (istat .ne. 0) then
         call dna_error('internal error')
         call ccperr(1,'*** Error in bfrots/dsyev ***')
      endif
      do j = 1, m
         do i = 1, m
            eigvec(i,j) = B(i,j)
         enddo
      enddo
c  Rows of eigvec are eigenvectors
c  eigvec is transpose of rotation matrix, so transpose it into [R]
c  [RT] = [R]T
      do 10, i = 1,2
         do 11, j = 1,2
            R(i,j) = eigvec(j,i)
 11      continue
 10   continue
c
      call bangle(R, rotang)
c
c Eigenvalues are principle components
      if (eigval(1) .le. eigval(2)) then
         bfcprn(1) = eigval(1)
         bfcprn(2) = eigval(2)
      else
c   smaller first
         bfcprn(1) = eigval(2)
         bfcprn(2) = eigval(1)
         rotang = rotang + 90.0
      endif
c
c Put rotation angle in range -90 -> +90
      if (rotang .gt. 90.0) then
         rotang = rotang - 180.0
      elseif (rotang .lt. -90.0) then
         rotang = rotang + 180.0
      endif
c
      return
c
      end
      subroutine chemax(nrs, lprint, 
     $   icntrc, epsiln,
     $   jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $   isymrf, ibatrf, mbn_s,
     $   eorf, nrej, istat)
c
c  Check observations for values that are too large
c  For each valid observation, calculate normalized structure factor E
c    E  =  Sqrt(I / Eps <I>)
c            where <I> is mean intensity/epsilon for that
c            resolution shell and run
c
c  Then p(E) = {  exp(-E**2)             acentric
c              {  erfc(E/sqrt(2))         centric
c
c  Flag as rejected any observation with E > Emax (if set by user)
c  Maximum values are in eacmax, ecnmax
c
c  Idea taken from Randy Read (CCP4 Sheffield January1999)
c
c On entry:
c  nrs           number of observations
c  icntrc        centric reflection if icntrc = 1, else = 0
c  epsiln        Epsilon value
c  jhkl(3)       reflection indices
c  ffrf(nrs)     intensity 
c  sdffrf(nrs)   sd(intensity)
c  ghl(nrs)      inverse scale
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isttrf(nrs)   status flag
c  isymrf(nrs)   symmetry number
c  ibatrf(nrs)  batch numbers
c  mbn_s         resolution shell number
c
c In common /means/
c  eacmax        maximum allowed E for acentric reflection (.lt. for no test)
c  ecnmax        maximum allowed E for centric reflection
 
c
c On exit
c  isttrf(nrs)   updated status flag
c  eorf(nrs)     normalized amplitudes E
c  nrej          number rejected
c  istat         =  0  OK
c                = -1  no mean intensity for this resolution range
c  
      integer nrs, icntrc, nrej, iselds, mbn_s, istat
      integer jhkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs), isymrf(nrs),
     $   ibatrf(nrs)
      real epsiln, eorf(nrs)
      real  ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs)
      logical lprint
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'flags.fh'
      include 'means.fh'
      include 'inout.fh'
c Locals
      real emax, aihl, e, aimean
      integer janom, l
      logical useflg(maxrfl), lkfull
      character labcen(2)*8
      data labcen /'acentric','centric'/
c
      istat = 0
      nrej = 0
c No test if eacmax .lt. 0
      if (eacmax .le. 0.0) return
c
c  Set flag useflg for whether to use observation:
c   accept (.true.) if all of
c   (a) isstrf .ge. lqOK
c   (b) irunrf .ne. irefrn (if irefrn .ne. 0)
c   (c) isymrf appropriate for janom flag
c   (d) irunrf belongs to dataset iselds if iselds .gt. 0
c   (e) mrf .eq. 0  if lufull .true. (fulls only)
      janom = 0
      iselds = 0
      lkfull = .false.
      call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf, 
     $   irefrn, iselds, lkfull, useflg)
c
      if (icntrc .eq. 0) then
         emax = eacmax
      else
         emax = ecnmax
      endif
      do 10, l = 1,nrs
         if (useflg(l)) then
            aihl = ffrf(l) / ghl(l)
            if (aihl .gt. 0.0) then
               aimean = qimnsr(mbn_s, kdtsrn(irunrf(l)))
               if (aimean .le. 0.0) then
c  If mean for this run is missing, try mean over all datasets
                  aimean = qimnsm(mbn_s)
                  if (aimean .le. 0.0) then
                     istat = -1
                     return
c                    ======
                  endif
               endif
               e = sqrt(aihl/(epsiln*aimean))
c  & test against limit set by user (test is on E**2)
               if (e .gt. emax) then
                  isttrf(l) = lqtoobig
                  nrej = nrej+1
                  if (lprint) then
                     write (lunout, '(a,3i4,i7,f8.3,3x,a)')
     $    ' Observation rejected by Emax test: h,k,l, Batch, E: ',
     $                  jhkl, ibatrf(l), e, labcen(icntrc+1)
                  endif
               endif
               eorf(l) = e
            endif
         endif
 10   continue
c
      return
      end
c
c
c
      subroutine chkarc(xd, yd, circen, cirrad, philim, lioarc, iflag)
c     ==============================================================
c
c Check if observation falls into rejected arc area
c
c On entry: 
c
c  xd, yd      detector coordinates
c  circen(2)   centre of arc
c  cirrad      radius of arc
c  philim(2)   limits of arc (radians)
c              philim(1) = philim(2) means exclude complete circle
c  lioarc      .true. exclude inside arc, .false. outside
c
c On exit:
c  iflag       = 0 OK, = +1 rejected
c
c
      integer iflag
      real xd, yd, circen(2), cirrad, philim(2)
      logical lioarc
c
c
      integer k
      real dx, dy, twopi, radius2, phi
      logical lrejct, lcircle
c
      iflag = 0
      twopi = 8.*atan(1.)
c
      lcircle = (abs(philim(1)-philim(2)) .lt. 0.001)
c
      dx = xd - circen(1)
      dy = yd - circen(2)
c Test radius**2 first
      radius2 = dx**2 + dy**2
      lrejct = .false.
      if (lioarc .eqv.
     $   ((cirrad-radius2) .gt. 0.0)) then
         if (lcircle) then
            lrejct = .true.
         else
c   possible exclusion based on radius, test phi
            phi = atan2(dy, dx)
            if (phi .ge. philim(1)) then
c   phi > phi1
               if (phi .le. philim(2)) then
c  reject
                  lrejct = .true.
               else
c   phi > phi2, reject if phi - 2pi >= phi1
                  if (phi - twopi .ge. philim(1)) then
c  reject
                     lrejct = .true.
                  endif
               endif
            else
c   phi < phi1, OK if phi+2pi <= phi2
               if (phi + twopi .le. philim(2)) then
c  reject
                  lrejct = .true.
               endif
            endif
         endif
      endif
c
      if (lrejct) then
         iflag = +1
         return
      endif
c
      return
      end


               
c
c
c
      subroutine chkclw(idts)
c     ======================
c
c Check if dataset has a valid cell & wavelength
c If not, try to extract it from batch header information
c
c On entry:
c  idts     dataset number (output) to check
c
      integer idts
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'nbtchc.fh'
      include 'datasets.fh'
c
      integer i, n, nw, ib, istat, lbcell(6), ierr
      real bcell(6), bwavel, cell(6), sdcell(6), wavel, sdwavel,
     $   dmosaic
      logical lcell, lwavel, lerrcell, lerrwavel, lbstat, lbtcell
      double precision sum_cell(6), sum_cell2(6), sn, sum_wvl, sum_wvl2 
      double precision dc, dcell(6), dsdcell(6)
      real errlen, errang
c
c  Tolerance for wavelength, cell length, cell angle
c before warning is printed
      real tolwvl, tolcll, tolcng
      data tolwvl, tolcll, tolcng /0.0001, 0.05, 0.05/
c
      lbstat = .true.
      ierr = 0
c
      lcell = .true. 
      do i = 1,6
         if (dcell_out(i,idts) .lt. 0.01) then
            lcell = .false.
         endif
      enddo
      lwavel = .true. 
      if (dwavel_out(idts) .lt. 0.01) lwavel = .false.
      dmosaic_out(idts) = 0.0
c
cc Go through all batches for this dataset, extraction cell &
c wavelength, and accumulating sums for averaging
      do i = 1,6
         sum_cell(i)  = 0.0
         sum_cell2(i) = 0.0
      enddo
      sum_wvl  = 0.0
      sum_wvl2 = 0.0
      sum_mosaic = 0.0
      n = 0
      nc = 0
      nw = 0
      lbtcell = .false.
c
      do ib = 1, nbtall
         if (kdtsbt(ib) .eq. idts) then
            call batclw(ibtall(ib), bcell, lbcell, bwavel, dmosaic,
     $         istat)

            n = n+1
            if (istat .ge. 0) then
               do i = 1,6
                  dc = bcell(i)
                  sum_cell(i)  = sum_cell(i)  + dc
                  sum_cell2(i) = sum_cell2(i) + dc**2
                  lbtcell = .true.
               enddo
               nc = nc+1
               if (istat .eq. 0) then
                  dc = bwavel
                  sum_wvl  = sum_wvl  + dc
                  sum_wvl2 = sum_wvl2 + dc**2
                  sum_mosaic = sum_mosaic + dmosaic
                  nw = nw+1
               endif
            else 
               lbstat = .false.
            endif
         endif
      enddo
c
      if (n .eq. 0) then
c-- This dataset includes no batches, exit now
         write (lunout, '(/a/)')
     $      ' WARNING: dataset contains no batches'
         return
      endif        
c Check lbcell is valid, at least one element should be non-zero
      if (lcell .or.lbtcell) then
         lbstat = .false.
         do i=1,6
            if (lbcell(i) .ne. 0) lbstat = .true.
         enddo
      endif
c
      lerrcell  = .false.
      lerrwavel = .false.
      call ccpzr(cell, 6)
      call ccpzr(sdcell, 6)
      wavel = 0.0
      sdwavel = 0.0
      if (nc .gt. 0) then
c Get mean & sd for cell & wavelength for this dataset
         sn = nc
         do i = 1,6
            dcell(i) = sum_cell(i)/sn
            dsdcell(i) = sum_cell2(i)/sn - dcell(i)**2
            if (dsdcell(i) .gt. 0.0) then
               sdcell(i) = sqrt(dsdcell(i))
            else 
               sdcell(i) = 0.0
            endif
            cell(i) = dcell(i)
         enddo
c Enforce symmetry constraints on cell
         if (lbstat) call constraincell(
     $      cell, lbcell, errlen, errang, ierr)
c
         if (nw .gt. 0) then
            sn = nw
            dc = sum_wvl/sn
            sdwavel = sum_wvl2/sn - dc**2
            if (sdwavel .gt. 0.0) then
               sdwavel=sqrt(sdwavel)
            else 
               sdwavel = 0.0
            endif
            wavel = dc
         endif
         if (.not. lcell) then
c  No cell previously set, so set it
            do i = 1,6
               dcell_out(i, idts) = cell(i)
            enddo
         else
c  Cell previously set, check for significant difference
            do i = 1,3
               if (abs(dcell_out(i, idts) - cell(i))
     $            .gt. tolcll .or.
     $            sdcell(i) .gt. tolcll) lerrcell = .true.
            enddo
            do i = 4,6
               if (abs(dcell_out(i, idts) - cell(i))
     $            .gt. tolcng .or.
     $            sdcell(i) .gt. tolcng) lerrcell = .true.
            enddo
         endif
c  Same for wavelength
         if (.not. lwavel) then
            dwavel_out(idts) = wavel
         else
            if (abs(dwavel_out(idts) - wavel)
     $         .gt. tolwvl) lerrwavel = .true.
         endif
c Average mosaicity
         dmosaic = sum_mosaic/sn
         dmosaic_out(idts) = dmosaic
c
c Print

         call dna_table_start('batch_information')
         call dna_list_start('unit_cell')
         call dna_real_item('a', cell(1))
         call dna_real_item('b', cell(2))
         call dna_real_item('c', cell(3))
         call dna_real_item('alpha', cell(4))
         call dna_real_item('beta', cell(5))
         call dna_real_item('gamma', cell(6))
         call dna_list_end
         call dna_list_start('unit_cell_errors')
         call dna_real_item('a', sdcell(1))
         call dna_real_item('b', sdcell(2))
         call dna_real_item('c', sdcell(3))
         call dna_real_item('alpha', sdcell(4))
         call dna_real_item('beta', sdcell(5))
         call dna_real_item('gamma', sdcell(6))
         call dna_list_end
         call dna_list_start('experiment_parameters')
         call dna_real_item('wavelength', wavel)
         call dna_real_item('wavelength_error', sdwavel)
         call dna_list_end
         call dna_table_end
         

         write (lunout, '(/a/a,f9.6,a,6f10.3/a,f9.6,a,6f10.3)')
     $      '* Wavelength and cell extracted from Batch headers, '//
     $      'with rms variation:',
     $      '* Wavelength: ', wavel, '  Cell: ', cell,
     $      '*   rms       ',sdwavel,'   rms  ', sdcell
         if (lerrwavel) then
            write (lunout, '(a/a,a)')
     $         ' WARNING: some batches have different wavelengths',
     $         '          Wavelength set = 0.0 to indicate this ',
     $         'for this dataset' 
            dwavel_out(idts) = 0.0
         endif
         if (lerrcell) then
            write (lunout, '(a/a/a,6f8.2)')
     $         ' WARNING: some batches have different cells',
     $         '          Cell reset to average for this dataset',
     $         ' New cell: ', cell
            do i = 1,6
               dcell_out(i, idts) = cell(i)
            enddo
         endif
         if (ierr .gt. 0) then
            write (lunout, 6001) errlen, errang
 6001       format (/
     $       '****ERROR: cell deviates too much from constraint ****',/,
     $         '    Relative length error ',f8.3,'  Angle error ',f8.2/)
            call prtcon(lbcell)
            call ccperr(1,
     $         '*** Error in batch cell dimensions ***')
         endif
      endif
c Enforce symmetry constraints on cell
      if (lbstat) call constraincell(dcell_out(1,idts), lbcell,
     $   errlen, errang, ierr)
c
      end
c
c
c
      subroutine constraincell(cell, lbcell, errlen, errang, ierr)
c    ====================================================
c
c Constrain cell according to flags lbcell
c
      real cell(6), errlen, errang
      integer lbcell(6), ierr
c
      integer i, n
      real sc, tollen, angdif, tolang
      data tollen/0.03/
      data tolang/0.5/
c
      ierr = 0
      errlen = 0.0
      errang = 0.0
c Lengths
      n=0
      sc = 0.0
      do i=1,3
         if (lbcell(i) .gt. 0) then
            sc = sc + cell(i)
            if (n .eq. 0) sc = sc + cell(lbcell(i))
            n = n+1
         endif
      enddo
      if (n .gt. 0) then
         sc = sc/float(n+1)
         do i=1,3
            if (lbcell(i) .gt. 0) then
               if (abs(cell(i)-sc)/sc .gt. tollen) then
                  ierr = ierr+1
                  errlen = abs(cell(i)-sc)/sc

               endif
               cell(i) = sc
               cell(lbcell(i)) = sc
            endif
         enddo
      endif
c
c Angles: assume within 1/2 degree
      do i=4,6
         if (lbcell(i) .eq. 0) then
            angdif = min(abs(cell(i)-90.0),abs(cell(i)-120.))
            if (angdif .gt. tolang) then
               ierr = ierr+1
               errang = angdif
            endif
            cell(i) = float(nint(cell(i)))
         endif 
      enddo
      return
      end
c
c
c
      subroutine prtcon(lbcell)
c    ======================
c
c Pretty print of cell constraints
      include 'inout.fh'

      integer lbcell(6)
      integer i, k
      logical lbstat
      character*1 abc(3)
      character*5 abg(3)
      character*80 line

      integer lenstr
      external lenstr

      data abc/'a','b','c'/
      data abg/'alpha',' beta','gamma'/

      lbstat =.false. 
      do i=1,6
         if (lbcell(i) .ne. 0) lbstat = .true.
      enddo 

      line = ''
      if (lbstat) then
         do i=1,3
            if (lbcell(i) .gt. 0) then
               if (line .eq. '') line = abc(i)
               line = line(1:lenstr(line))//'='//abc(lbcell(i))
            endif
         enddo
         if (line .ne. '') line = line(1:lenstr(line))//'  '
         k = 0
         do i=4,6
            if (lbcell(i) .gt. 0) then
               if (k .eq. 0) line = line(1:lenstr(line))//abg(i)
               line = line(1:lenstr(line))//'='//abg(lbcell(i))
               k = 1
            endif
         enddo
      endif

      write (lunout, '(a,a/)')
     $   '  >>> Cell constraints:  ', line
      return
      end
      subroutine chkfix(iptpar, npar, istat)
c     =====================================
c
c  Check if undetermined parameters are FIXed
c
c On entry:
c  iptpar(npar)   pointers from all global parameters to compressed set,
c                 = 0 for undetermined parameters
c
c On exit:
c  istat          =  0  OK, missing parameters are fixed
c                 = -1  not OK, some missing parameters are not fixed
c
      integer npar, iptpar(npar), istat
c
      include 'parameter.fh'
      include 'params.fh'
      include 'scales.fh'
      include 'refcon.fh'
c
      integer i, j, k, irun
c
      istat = 0
c
      do 10, i = 1, npar
         if (iptpar(i) .eq. 0) then
            istat = -1
c  Pointer to local parameter
            irun = iabs(ipnrun(i))
            j = ipnloc(i)
            if (j .lt. 0) then
               j = iplloc(-j, irun)
            endif
c  Currently, only TAILS parameters may be fixed
            if (nrntds(irun) .gt. 0) then
               k = j - ipntds(irun) + 1
               if (k .gt. 0) then
                  if (lptfix(k)) then
                     istat = 0
                  endif
               endif
            endif
         endif
 10   continue
      return 
      end
c
c
c
      subroutine chkilm(
     $   nrs, ffrf, sdffrf, isttrf, irunrf, nmprrf, sclirf, istat)
c     ===========================================================
c
c  Check fulls &  processed partials against intensity limits, 
c Check against input intensity, ie allowing for scale on input file,
c taken as scale for first part (should be approximately right)
c
c On entry:
c     nrs          number of observations
c     ffrf(nrs)    intensity
c     sdffrf(nrs)  sd(intensity)
c     isttrf(nrs)  status flags
c     irunrf(nrs)  run numbers
c     nmprrf(nrs) number of parts summed into this slot
c              = 1 for full
c              = 0 unmatched partial or 2nd & subsequent parts of partial
c                    (ie those flagged as lquprt)
c              .gt. 1 for summed partial (1st slot)
c              negated for scaled partial, = - number of parts
c     sclirf(nrs)  input scales from file
c
c
c On exit:
c  istat    = 0        all observations pass tests
c           = lqweak   too weak
c           = lqstrn   too strong
c
c
      integer nrs,  isttrf(nrs), irunrf(nrs), nmprrf(nrs), istat
      real    ffrf(nrs), sdffrf(nrs), sclirf(nrs)
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'rfile.fh'
      include 'rundef.fh'
c
      integer l, irun
      real    ff, sdff
c
      istat = 0
      do 10, l = 1,nrs
         if (isttrf(l) .ne. lqeflg) then
            irun = irunrf(l)
            if (irun .gt. 0 .and. irun .ne. irefrn) then
c  Not reference run
               if (nmprrf(l) .ge. 1 .or. nmprrf(l) .lt. 0) then
c    Full, Summed or scaled partial
c  Correct for input scale
                  ff = ffrf(l) / sclirf(l)
                  sdff = sdffrf(l) / sclirf(l)
c     Selection on reflection strength
c     Sd cutoffs
                  if (ff .lt. sdImin(irun)*sdff) then
c     Omit reflection from scaling
                     istat = lqweak
                     isttrf(l) = istat
                  elseif (ff .gt. sdImax(irun)*sdff .or.
     $                  ff .gt. abImax(irun)) then
                     istat = lqstrn
                     isttrf(l) = istat
                  endif
               endif
            endif
         endif
 10   continue
c
      return
      end
c
c
c
      subroutine chkkpf(kprflg, ncmprt, lunout)
c     =========================================
c
c Check validity of partial check/test flag, set default if needed
c
c kprflg   flag, possibly updated on output
c          entry values to be reset:-
c                 +-1000   unset
c                 +-10    CHECK|NOCHECK set
c          values on exit
c                 .lt. 0 CHECK (not valid if ncmprt = 0)
c                 .gt. 0 NOCHECK
c                 +-1 no test on total fraction
c                 +-2 test total fraction for acceptance or scaling
c             default values are:-
c                 -2 if CHECK on  [if MPART present]
c                 +1 if CHECK off [if MPART absent]
c ncmprt   column number of MPART column, = 0 if absent
c
      integer kprflg, ncmprt, lunout
      integer k, kprdef
c
c  Set appropriate default depending on whether MPART is present
      if (ncmprt .eq. 0) then
c   If no MPART, then  NOCHECK TEST
         kprdef = +2
      else
c   If MPART, then  CHECK TEST
         kprdef = -2
      endif
c
      k = kprflg
      if (iabs(kprflg) .eq. +1000) then
c kprflg was unset, set to default
         k = kprdef
      elseif (iabs(kprflg) .eq. 10) then
c CHECK/NOCHECK was set, but nothing else, so preserve sign
         k = sign(iabs(kprdef), kprflg)
c else kprflg was already set
      endif
      if (ncmprt .eq. 0) then
c CHECK invalid if no MPART column, so reset
         k = iabs(k)
         if (kprflg .lt. 0) then
c  Print warning if CHECK was explicitly set and is invalid
            write (lunout, '(/a,a)')
     $         ' ==== WARNING: PARTIAL CHECK option requires an ',
     $         'MPART column, changed to NOCHECK'
         endif
      endif
      kprflg = k
c
      return
      end

      subroutine chkprt(nrs, l, isymrf, ibatrf, isttrf, mprf,
c     =======================================================
     $   frclrf, irunrf, nfound, numprt, totfrc, istat)
c 
c
c  Count & check for matching partials to this one
c
c On entry:
c  nrs          reflection array length (= last entry to check)
c  l            index into reflection arrays for 1st partial of this set
c  isymrf(nrs)  array of symmetry numbers (same for matching partials)
c  ibatrf(nrs)  batch numbers (contiguous for matching partials)
c  isttrf(nrs)  status flags, = lqprt for partials
c  mprf(nrs)    mpart flags
c  frclrf(nrs)  calculated fraction
c  irunrf(nrs)  run numbers
c
c On exit:
c  nfound       number of parts found
c  numprt       number of parts expected ( = -1 if unknown)
c  totfrc       total fraction
c  istat        status
c               = +1  MPART flags checked & all correct, OK
c               =  0  OK,no error
c               = -1  number of parts found different to that given
c                     by first MPART flag
c               = -2  MPART flags not all consistent with each other
c               = -3  missing slots, ie matching parts on non-sequential
c                     batches (only a gap of one allowed)
c               = -4  some parts flagged as bad (not lqprt, probably lqeflg)
c
      integer nrs, l, nfound, numprt, istat
      integer isymrf(nrs), ibatrf(nrs), isttrf(nrs), mprf(nrs),
     $   irunrf(nrs)
      real    frclrf(nrs), totfrc
c
      include 'glocon.fh'
      include 'inout.fh'
      include 'flags.fh'
c
c Locals
      integer i, k, n, last, mpart, mptpad
      logical lmiss, lbad
      character flags(100)*1
c
c
      istat = 0
      totfrc = frclrf(l)
      if (mprf(l) .gt. 200) then
         mptpad = 100
         numprt = mprf(l)/mptpad
      elseif (mprf(l) .gt. 20) then
         mptpad = 10
         numprt = mprf(l)/mptpad
      else
         numprt = -1
      endif
      lmiss = .false.
c
c   Always search to end of list 
      last = nrs
c check status flags for each part, reject if any are bad (or not lqprt)
      lbad = .false.
      if (isttrf(l) .ne. lqprt) lbad = .true.
c
      k = l+1
c Loop starting at 2nd entry: finish on next non-matching observation or
c on end of list
c  do while (k .le.  last)
 10   if (k .le. last) then
         if ((isymrf(k) .eq. isymrf(l)) .and.
     $      (irunrf(k) .eq. irunrf(l))) then
c     matching symmetry numbers & same run
c     Contiguous batch numbers?
            if (ibatrf(k) .eq. ibatrf(k-1)+2) then
c  Allow for gap of one, but flag
               lmiss = .true.
            elseif (ibatrf(k) .ne. ibatrf(k-1)+1) then
c  not contiguous so reject
               go to 11
            endif               
c      observation k is part of same set as l
            if (isttrf(k) .eq. lqprt) then 
c Total fraction
               totfrc = totfrc + frclrf(k)
            else
c   error flag set, 
               lbad = .true.
            endif
            k = k+1
            go to 10
         endif
      endif
c Set consists of l to k-1
 11   n = k-l
c  k is last one
      k = k-1
      if (n .gt. 1) then
c  We have a set
c  If MPART flags are present, check for validity of flags
         if (numprt .gt. 0) then
            if (lbad) then
c  Not all parts status lqprt (perhaps lqeflg)
               istat = -4
            elseif (n .ne. numprt) then
c  Number of parts found different to expected number
               istat = -1
            else
c    check MPART
               istat = +1
               do 20, i = 1, n
                  if (mprf(i+l-1) .gt. 200) then
                     mptpad = 100
                  elseif (mprf(i+l-1) .gt. 20) then
                     mptpad = 10
                  else
c            can happen with Postref output
                     istat = -2
                  endif
                  mpart = numprt*mptpad + i
                  if (mprf(i+l-1) .ne. mpart) then
                     istat = -2
                  endif
 20            continue
            endif
c*debug
            if (kprint(1) .ge. 4) then
               if (istat .lt. 0) then
                  do i = 1, nrs
                     if (i .lt. l .or. i .gt. k) then
                        flags(i) = ' '
                     else
                        flags(i) = '*'
                     endif
                  enddo
                  write (lunout,'(a,f8.3/a/,20(8(1x,a1,i6,2i4,f8.3/)))')
     $               ' CHKPRT: incomplete reflections, total: ',totfrc,
     $               ' Batch, Isym, Mpart, Frccal:',
     $               (flags(i), ibatrf(i), isymrf(i),
     $               mprf(i), frclrf(i), i=1,nrs)
               endif
            endif
         endif
      endif
c  Missing slot overrides previous flags
      if (lmiss) then
         istat = -3
      endif
      nfound = n
      return
      end
      subroutine chkref(lfflag, kpflag, s, data,
     $     m, mpart, isym, ff, sdff, ffpr, sdffpr, ibatch, irun, jserbt,
     $     phi, xd, yd, zd, time, sclinp, sdscli, frccal, clp,
     $     lrflag, bgpk,
     $     istat)
c     ==================================================================
c     
c     Extract reflection information out of reflection record, and
c     do a few checks
c     
c     On entry:
c     lfflag   = .false., if both integrated & profile-fitted intensities
c                are in the file, return I, SIGI in ff, sdff and IPR, SIGIPR
c                in ffpr, sdffpr
c                If no IPR observation available, return ffpr=ff, sdffpr=sdff
c              = .true., return selected intensity (depending on setting
c                of IMID) in ff, sdff, nothing in ffpr, sdffpr. If no profile
c                fitted intensity for this observation, return integrated I
c     kpflag   = -1   accept fulls only, return m = -1 if rejected
c              =  0   accept fulls & previously summed partials (mpart=10)
c              = +1   accept fulls & partials
c     s        1/d**2 for this reflection
c     data     reflection record
c     
c     On exit:
c     m        partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c              = -1 for ignored partial depending on kpflag 
c              =  1 for previously summed partial (mpart = 10)
c                    Note that these will have istat = lqOK
c     mpart    partial flag: number of images etc
c              = 10 summed partial
c              .gt. 200, number of images * 100 + image number
c              .gt. 20, number of images * 10 + image number
c     isym     symmetry number
c     ff       intensity
c     sdff     sd(I)
c     ffpr     profile-fitted intensity
c     sdffpr   sd(Ipr)
c     ibatch   batch number
c     irun     run number, = -1 if rejected
c     jserbt   batch serial number within run (for SCALE BATCH)
c     phi      rotation angle (corrected by offset)
c     xd, yd   detector coordinates
c     zd       rotation angle (corrected by offset)
c               or for SMOOTH option batch number
c     time     time or rotation angle
c     sclinp   scale factor read from SCALE column in input file, = 1.0 if none
c     sdscli   sd(scale factor) read from SIGSCALE column in input file,
c                = 0.0 if none
c     frccal   calculated fraction (if present in file)
c     clp      LP factor if present in file, else = 0.0
c     lrflag   flag column from file (if present, else = 0)
c              if non-zero, reflection is marked as rejected (lqeflg)
c     bgpk     packed bg/peak ratios (if present, else = 0.0)
c     istat    status flag
c              = 0 conditionally OK
c            .gt.0 unconditionally OK
c            .lt.0 rejected
c      Possible values of istat set here
c       Accepted:- 
c        lqOK (+1) unconditionally OK (including previously summed partials)
c        lqprt    unsummed partial (may be summed in future)
c       Rejected:-
c        lqeflg   non-zero error flag on input
c                  *** NB lqeflg may be reset later, so must be first here, so
c                         that any other reject flags take precedence
c        lqreso   outside resolution limits
c        lqzscl   zero input scale (when input scale column is used)
c                   (this flag will not be set by a missing scale column)
c        lqsd0    zero standard deviation
c        lqxyrj   bad detector area
c     
      implicit none
c     
      include 'parameter.fh'
      logical lfflag
      integer m, isym, ibatch, irun, jser, jserbt, istat, mpart,
     $   lrflag, kpflag, k
      real s, data(maxcol), ff, sdff, ffpr, sdffpr, phi,
     $   xd, yd, zd, time
      real sclinp, sdscli, frccal, clp, bgpk, crnscale
c     
      include 'column.fh'
      include 'rfile.fh'
      include 'flags.fh'
      include 'rundef.fh'
      include 'scales.fh'
c**
      include 'symmty.fh'
c
      logical logmss(maxcol)
c
      integer misym, iflag
      integer ccp4_hash_lookup
      external ccp4_hash_lookup
      real cnr_correction
      external cnr_correction

c
c  Missing data flags from file
      call lrrefm(1, logmss)
cc      write (6,*) (logmss(k), k=1,ncol)
c     
      istat = lqOK
c     
c     3d coordinates
c     Mode = 1 (detector frame): for mode = 2, these will be replaced later
      xd = 0.0
      yd = 0.0
      zd = 0.0
      phi = 0.0
      if (ncx .gt. 0) then
        if (.not.logmss(ncx)) xd = data(ncx)
      endif
      if (ncy .gt. 0) then
        if (.not.logmss(ncy)) yd = data(ncy)
      endif
c     Batch
      if (logmss(ncbatc)) then
        call ccperr(1,' *** ERROR: MNF found in BATCH. ***')
      else
        ibatch = nint(data(ncbatc))
      endif
      jser = ccp4_hash_lookup(ibatch)
      if (ncz .gt. 0) then
        if (.not.logmss(ncz)) then
           phi = data(ncz)
c Apply offset to Phi
           phi = phi + phioffset(jser)
           zd = phi
        endif
      endif
c     
c     Get "run" number, Irun
      call runget(ibatch, zd, irun, jser, jserbt)
c--   irun = -1 rejected, so skip all other checks 
      if (irun .le. 0) return
c
c  SMOOTH mode, use batch number of Zd coordinate
      if (lbtsmooth(irun)) then
         zd = ibatch
      endif
c     "Time"
      time = zd
c   if Phi decreasing, time = -phi
      if (negphi(irun)) time = -zd
      if (nct .gt. 0) then
         if (.not. logmss(nct)) then
            time = data(nct)
         endif
      endif
c
c     Get partial flag & symmetry
      if (logmss(ncmsym)) then
        call ccperr(1,' *** ERROR: MNF found in M/ISYM. ***')
      else
        misym = nint(data(ncmsym))
      endif
      m = misym/256
      isym = misym - m*256
c**
      if (isym .gt. nsym*2) then
         write (6,*) 'Symmetry error', isym
      endif 
c**
      if (ncmprt .gt. 0) then
         if (.not. logmss(ncmprt)) then
            mpart = nint(data(ncmprt))
            if (mpart .eq. 10) then
c . . . classify previously summed partials as partial
c        m = 1, but mpart = 10
               istat = lqOK
               m = 1
            endif
         endif
      else
         mpart = m
      endif
c
c Keep partials?
      if (mpart .eq. 10) then
         if (kpflag .lt. 0) then
            m = -1
            return
c           ======
         endif
      elseif (m .eq. 1) then
         if (kpflag .le. 0) then
            m = -1
            return
c           ======
         endif
      endif
c     
c     
c     Get I & sd(I)
      if (logmss(nci)) then
         ff = 0.0
         sdff = 0.0
      else
         ff = data(nci)
         sdff = data(ncsdi)
      endif
c     
      if (ncip .gt. 0) then
         if (logmss(ncip)) then
            ffpr = 0.0
            sdffpr = 0.0
         else
            ffpr = data(ncip)
            sdffpr = data(ncsdip)
         endif
      else
c  if no IPR, copy I into ffpr
         ffpr = data(nci)
         sdffpr = data(ncsdi)
      endif
c
      if (lfflag) then
c  Select either summed intensity or profile-fitted intensity as
c defined by imid flag
         call selint(ff, sdff, ffpr, sdffpr, m, imid, ipower)
      endif
c
c     
c From here on set istat flags: it is important that this is
c done in the right order!
c  Full/partial/reject flag: partial
      if (m .eq. 1 .and. mpart .ne. 10) then
         istat = lqprt
      endif
c Next, Mosflm FLAG, if present - this can be overridden with harder
c reject conditions
      lrflag = 0
      if (ncflag .gt. 0) then
         if (.not. logmss(ncflag)) then
            lrflag = nint(data(ncflag))
            if (lrflag .ne. 0) then
               istat = lqeflg
            endif
         endif
      endif
c
c  Zero standard deviation
      if (sdff .lt. 0.000001) then
         istat = lqsd0
      endif
c  Full/partial/reject flag: reject
      if (m .ge. 2) then
         istat = lqrejm
      endif
c
      frccal = 0.0
      if (ncfrcl .gt. 0) then
         if (.not.logmss(ncfrcl)) then
            frccal = data(ncfrcl)
         endif
      endif
c LP factor
      clp = 1.0
      if (nclp .gt. 0) then
         if (.not.logmss(nclp)) then
            clp = data(nclp)
         endif
      endif
c
c     
c BG/PK ratio
      bgpk = 0.0
      if (ncbgpk .gt. 0) then
         if (.not.logmss(ncbgpk)) then
            bgpk = data(ncbgpk)
         endif
      endif
c
c
c Check detector coordinates if required
      if (mrjrct .ne. 0) then
         if (irun .gt. 0 .and. nrjrct(irun) .gt. 0) then
            do 100, k = 1, nrjrct(irun)
               if (xd .ge. rejrct(1,1,k,irun) .and.
     $             xd .le. rejrct(2,1,k,irun)) then
                  if (yd .ge. rejrct(1,2,k,irun) .and.
     $                yd .le. rejrct(2,2,k,irun)) then
                     istat = lqxyrj
                     go to 110
                  endif
               endif
 100        continue
 110        continue
         endif
      endif
c Arcs
      if (mrjarc .ne. 0) then
         if (irun .gt. 0 .and. nrjarc(irun) .gt. 0) then
            do k = 1, nrjarc(irun)
               call chkarc(
     $            xd, yd, rcrarc(1,k,irun), rcrarc(3,k,irun),
     $            phiarc(1,k,irun), linarc(k,irun), iflag)
               if (iflag .ne. 0) then
                  istat = lqxyrj
                  go to 120
               endif
            enddo
 120        continue
         endif
      endif
c     
c     Apply existing scalefactor column if present
      sclinp  = 1.0
      sdscli = 0.0
      if (ncsc .gt. 0 .and. lappsc) then
         if (.not.logmss(ncsc)) then
            sclinp = data(ncsc)
            if (ncsdsc .gt. 0) then
               if (.not.logmss(ncsdsc)) then
                  sdscli = data(ncsdsc)
               endif
            endif
         endif
         ff = sclinp*ff
         sdff = sqrt((sclinp * sdff)**2 + (sdscli*ff)**2)
         if (ncip .gt. 0) then
            ffpr = sclinp*ffpr
            sdffpr = sqrt((sclinp * sdffpr)**2 + (sdscli*ff)**2)
         endif
         if (sclinp .eq. 0.0) istat = lqzscl
      endif
c Appy corner correction, if wanted
      if (lcorner) then
         crnscale = cnr_correction(xd, yd)
         ff = crnscale*ff
         sdff = crnscale * sdff
         if (ncip .gt. 0) then
            ffpr = crnscale*ffpr
            sdffpr = crnscale * sdffpr
         endif
      endif
c
      if (irun .gt. 0) then
c     Resolution limits: rejected reflections are ignored
         if (s.lt.smin(irun) .or. s.gt.smax(irun)) then
            istat = lqreso
         endif
      endif
c     
c      
      return
c
      end
      subroutine chkrun(nrun, irun)
c     =============================
c
c  Check if run occurs in list
c
c  On entry:
c     nrun      run number to check
c
c  On exit:
c     irun      internal run number, = -1 if not found
c               defaults = 1 if no runs defined
c
      implicit none
c
      integer nrun, irun
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
c
      if (numrun .gt. 0) then
         do 10, irun = 1, numrun
            if (irunum(irun) .eq. nrun) then
c     OK
               go to 100
            endif
 10      continue
      else
c     No run defined, 1 by default
         if (nrun .eq. 1) then
c     OK
            irun = 1
            go to 100
         endif
      endif
c     This run is not defined
      write (lunout, '(A,I4,A)') ' **** Run number', nrun,
     $     ' has not yet been defined    ****'
      write (lunout, '(A)')
     $   ' **** Definition of run must precede its use ****'
      irun = -1
c
 100  return
      end
      subroutine chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf, 
     $   irefrun, iselds, lufull, useflg)
c     =============================================================
c
c  Set flag useflg for whether to use observation:
c   accept (.true.) if all of
c   (a) isstrf .ge. lqOK
c   (b) irunrf .ne. irefrun (if irefrun .ne. 0)
c   (c) isymrf appropriate for janom flag
c   (d) irunrf belongs to dataset iselds if iselds .gt. 0
c   (e) mrf .eq. 0  if lufull .true. (fulls only)
c
c
c On entry:
c  nrs           number of observations
c  janom         = 0 use all valid observations
c                = 1 use I+ only (isym odd)
c                = 2 use I- only (isym even)
c  isttrf        status flags
c  isymrf        symmetry numbers (odd for I+, even for I-)
c  irunrf        run numbers
c  irefrun       reference run number
c  mrf           full/partial flag (= 0 full, = 1 partial)
c  iselds        dataset number to select ( = 0 to select all datasets)
c  lufull        .true. to use fulls only, .false. partials as well
c
c On exit:
c  useflg(nrs)   .true. for each observation to use
c
      integer nrs, janom, isttrf(nrs), isymrf(nrs), irunrf(nrs),
     $    mrf(nrs), irefrun, iselds
      logical useflg(nrs), lufull
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'rundef.fh'
c
      integer l
c
      do 10, l = 1, nrs
         useflg(l) = .true.
c  Status flag
         if (isttrf(l) .lt. lqOK) then
            useflg(l) = .false.
         endif
         if (irefrun .gt. 0) then
            if (irunrf(l) .eq. irefrun) then
               useflg(l) = .false.
            endif
         endif
         if (janom .eq. 1) then
            if (mod(isymrf(l), 2) .eq. 0) useflg(l) = .false.
         elseif (janom .eq. 2) then
            if (mod(isymrf(l), 2) .eq. 1) useflg(l) = .false.
         endif
         if (iselds .gt. 0) then
            if (kdtsrn(irunrf(l)) .ne. iselds) useflg(l) = .false.
         endif
         if (lufull .and. mrf(l) .ne. 0) then
c  Fulls only
            useflg(l) = .false.
         endif
 10   continue
      return
      end
      subroutine choutl(nrs, kanom, lprint, lrejct, lselds,
     $   icntrc, jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $   isymrf, lanmtc, nuse,
     $   nrej, sdrej, sdrej2, sdreja, sdrej2a, krej2, delrf,
     $   swih, swihpl, swihmn)
c     ==================================================
c
c  Check outliers
c
c On entry:
c  nrs           number of observations
c  kanom         anomalous flag, = 0 no anomalous
c  lprint        .true. to print outliers
c  lrejct        if .true., reject outliers, else just calculate deviations
c  lselds        if .true., test outliers only within datasets
c  icntrc        centric reflection if icntrc = 1, else = 0
c  jhkl(3)       reflection indices
c  ffrf(nrs)     intensity 
c  sdffrf(nrs)   sd(intensity)
c  wtrf          weight for this observation, eg 1/sd(I)**2
c                  (note that this is unscaled, so needs to be
c                   multiplied by ghl**2)
c  ghl(nrs)      inverse scale
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isttrf(nrs)   status flag
c  isymrf(nrs)   symmetry number
c  lanmtc(nrs)   pointer to matched pair (=0 if none)
c  nuse          number of observations used
c  sdrej         rejection limit for reflections measured more than twice
c  sdrej2        rejection limit for reflections measured twice
c  sdreja        rejection limit for reflections measured more than twice,
c                  including all observations (I+ & I-) (.le. 0 for no test)
c                  (not used if kanom = 0)
c  sdrej2a       rejection limit for reflections measured twice,
c                  including all observations (I+ & I-) (.le. 0 for no test)
c                  (not used if kanom = 0)
c  krej2         what to do with 2 observations which disagree
c                = -1   reject both
c                =  0   keep both
c                = +1   reject larger
c                = +2   reject smaller
c
c On exit
c  swih          mean intensity (weighted)
c  swihpl        mean intensity I+ (weighted)
c  swihmn        mean intensity I- (weighted)
c  isttrf(nrs)   updated status flag
c  delrf(nrs)    weighted deviation = deviation/sigma (= 0.0 if none)
c                Note that even if REJECT ALL is selected (sdreja .gt. 0),
c                 this deviation still applies to comparison within the
c                 I+ or I- set (when anomalous is present) except for
c                 observations which are rejected on the I+- comparison
c  nuse          updated number of "useful" ones
c  nrej          number rejected: this may be .gt. 0 even if lrejct .true.
c                if observations are flagged as rejected outiers on entry,
c                in array isttrf
c  
      integer nrs, kanom, icntrc, nuse, nrej, krej2, iselds
      integer jhkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs), isymrf(nrs),
     $   lanmtc(nrs)
      real  ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs),delrf(nrs)
      real  swih, swihpl, swihmn, sdrej, sdrej2, sdreja, sdrej2a
      logical lprint, lrejct, lselds
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'flags.fh'
      include 'datasets.fh'
c Locals
      integer nrejpl, nrejmn, nkept, janom, idts, ndts, nreja, i
      real    sdih, uh, delarf(maxrfl)
      logical useflg(maxrfl), lkfull
c
      call ccpzr(delrf,nrs)
c Fulls & partials in outlier check (ie not fulls only)
      lkfull = .false.
c
      if (lselds) then
         ndts = ndtsetsout
      else
         ndts = 1
      endif
      nuse = 0
      nrej = 0
c
      do 10, idts = 1, ndts
         if (lselds) then
            iselds = idts
         else
            iselds = 0
         endif
c
         if (kanom .eq. 0 .or. icntrc .eq. 1) then
c  No anomalous
            janom = 0
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf, lanmtc,
     $         swih, nreja,  sdrej, sdrej2, krej2, lqoutl, delrf, nkept)
            swihpl = 0.0
            swihmn = 0.0
            nuse = nuse + nkept
            nrej = nrej + nreja
         elseif (kanom .ge. 1) then
c  Anomalous
c   I+
            janom = 1
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf, lanmtc,
     $         swihpl, nrejpl, sdrej, sdrej2, krej2, lqoutl,
     $         delrf, nkept)
c   I-
            janom = 2               
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf, lanmtc,
     $         swihmn, nrejmn,  sdrej, sdrej2, krej2, lqoutl,
     $         delrf, nkept)
            nrej = nrej + nrejpl +nrejmn
c   All together
            if (sdreja .gt. 0.0) then
               janom = 0
               call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $           lkfull, jhkl, ffrf, 
     $           sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf, lanmtc,
     $           swih, nreja,  sdreja, sdrej2a, krej2, lqoutlan,
     $            delarf, nkept)
               if (nreja .gt. 0) then
                  nrej = nrej + nreja
c Transfer deviation for rejected observations to usual array for
c rogue printing
                  do i = 1,nrs
                     if (isttrf(i) .eq. lqoutlan) then
                        delrf(i) = delarf(i)
                     endif
                  enddo
               endif
c   Then recalculate means of I+, I-
               janom = 1
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swihpl, sdih, uh, nuse)
               janom = 2
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swihmn, sdih, uh, nuse)
            else
c  Overall mean after rejections (if no overall test)
               janom = 0
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swih, sdih, uh, nuse)
            endif
         endif
 10   continue
c
      return
      end
      subroutine clrsts
c     =================
c
c  Clear all statistics tables for output pass
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'sphhrm.fh'
      include 'rundef.fh'
c
      integer i, j, k, irun
c
      call ccpzr(sum_a_k_ts,maxrtr*maxres)
      call ccpzr(sum_a_t_rs,maxrun*maxres)
      call ccpzr(sum_o_i_t,maxrtr)
      call ccpzr(sum_o_i_s,maxres)
      call ccpzr(sum_o_i_i,maxinb)
      call ccpzr(sum_o_del_t,maxrtr)
      call ccpzr(sum_o_del_s,maxres)
      call ccpzr(sum_o_del_i,maxinb)
      call ccpzr(sum_o_sig_t,maxrtr)
      call ccpzr(sum_o_sig_s,maxres)
      call ccpzr(sum_o_sig_i,maxinb)
      call ccpzr(sum_o_sd_t,maxrtr)
      call ccpzr(sum_o_sd_s,maxres)
      call ccpzr(sum_o_sd_i,maxinb)
      call ccpzr(sum_m_i_t,maxrtr)
      call ccpzr(sum_m_i_s,maxres)
      call ccpzr(sum_m_i_i,maxinb)
      call ccpzr(sum_m_del_t,maxrtr)
      call ccpzr(sum_m_del_s,maxres)
      call ccpzr(sum_m_del_i,maxinb)
      call ccpzr(sum_m_sig_t,maxrtr)
      call ccpzr(sum_m_sig_s,maxres)
      call ccpzr(sum_m_sig_i,maxinb)
      call ccpzr(sum_m_sd_t,maxrtr)
      call ccpzr(sum_m_sd_s,maxres)
      call ccpzr(sum_m_sd_i,maxinb)
      call ccpzr(sum_m_del_rs,maxrun*maxres)
      call ccpzr(sum_m_i_rs,maxrun*maxres)
      call ccpzr(sum_b_bias_rs,maxrun*maxres)
      call ccpzr(sum_b_i_rs,maxrun*maxres)
      call ccpzr(sum_b_bias_ri,maxrun*maxinb)
      call ccpzr(sum_b_i_ri,maxrun*maxinb)
      call ccpzr(sum_b_bias_c,maxfrc*2)
      call ccpzr(sum_b_i_c,maxfrc*2)
      call ccpzr(sum_f_i_s,maxres)
      call ccpzr(sum_f_i_i,maxinb)
      call ccpzr(sum_f_del_s,maxres)
      call ccpzr(sum_f_del_i,maxinb)
      call ccpzr(sum_anom_dif_s,maxres)
      call ccpzr(sum_anom_sum_s,maxres)
      call ccpzr(sum_anom_dif_i,maxinb)
      call ccpzr(sum_anom_sum_i,maxinb)
      call ccpzr(sum_m_isq_irf,maxinb*maxrun*2)
      call ccpzr(sum_m_frd_irf,maxinb*maxrun*2)
      call ccpzr(sum_m_frdsq_irf,maxinb*maxrun*2)
      call ccpzr(sum_iovrsd_s,maxres)
      call ccpzr(sum_m_frd_xy,maxxyn*maxxyn)
      call ccpzr(sum_m_frdsq_xy,maxxyn*maxxyn)
      call ccpzr(sum_m_wdl_s,maxres)
      call ccpzr(sum_m_wsg_s,maxres)
      call ccpzr(sum_m_im_s,maxres)
      call ccpzr(sum_o_wdl_s,maxres)
      call ccpzr(sum_o_wsg_s,maxres)
      call ccpzr(sum_o_im_s,maxres)
      call ccpzr(sum_m_ndl_s,maxres)
      call ccpzr(sum_o_ndl_s,maxres)
c

c Min & max Dfhl
      do 1, irun = 1, maxrun
         exm_t_r(1,irun) = +1.0e10
         exm_t_r(2,irun) = -1.0e10
 1    continue
c
      call ccpzi(n_a_k_ts, maxrtr*maxres)
      call ccpzi(n_a_t_rs, maxrun*maxres)
      call ccpzi(n_o_i_t, maxrtr)
      call ccpzi(n_o_i_s, maxres)
      call ccpzi(n_o_i_i, maxinb)
      call ccpzi(n_o_del_t, maxrtr)
      call ccpzi(n_o_del_s, maxres)
      call ccpzi(n_o_del_i, maxinb)
      call ccpzi(n_o_sig_t, maxrtr)
      call ccpzi(n_o_sig_s, maxres)
      call ccpzi(n_o_sig_i, maxinb)
      call ccpzi(n_o_sd_t, maxrtr)
      call ccpzi(n_o_sd_s, maxres)
      call ccpzi(n_o_sd_i, maxinb)
      call ccpzi(n_m_full_t, maxrtr)
      call ccpzi(n_m_part_t, maxrtr)
      call ccpzi(n_m_i_t,maxrtr)
      call ccpzi(n_m_i_s,maxres)
      call ccpzi(n_m_i_i,maxinb)
      call ccpzi(n_m_del_t, maxrtr)
      call ccpzi(n_m_del_s, maxres)
      call ccpzi(n_m_del_i, maxinb)
      call ccpzi(n_m_sig_t, maxrtr)
      call ccpzi(n_m_sig_s, maxres)
      call ccpzi(n_m_sig_i, maxinb)
      call ccpzi(n_m_sd_t, maxrtr)
      call ccpzi(n_m_sd_s, maxres)
      call ccpzi(n_m_sd_i, maxinb)
      call ccpzi(n_m_del_rs,maxrun*maxres)
      call ccpzi(n_b_bias_rs, maxrun*maxres)
      call ccpzi(n_b_bias_ri, maxrun*maxinb)
      call ccpzi(n_b_bias_c, maxfrc*2)
      call ccpzi(ntotrjs, maxbat*maxrun)
      call ccpzi(nrejs, maxbat*maxrun)
      call ccpzi(nrejm, maxbat*maxrun)
      call ccpzi(n_anom_s,maxres)
      call ccpzi(n_anom_i,maxinb)
      call ccpzi(n_m_sd_irf,maxinb*maxrun*2)
      call ccpzi(n_m_sdhist,21*2)
      call ccpzi(n_uniq_s,maxres)
      call ccpzi(n_uniq_i,maxinb)
      call ccpzi(n_cent_s,maxres)
      call ccpzi(n_cent_i,maxinb)
      call ccpzi(n_o_t,maxrtr)
      call ccpzi(n_o_meas_s,maxres)
      call ccpzi(n_o_uniq_s,maxres)
      call ccpzi(n_o_cent_s,maxres)
      call ccpzi(n_m_uniq_s,maxres)
      call ccpzi(n_o_totl_s,maxres)
      call ccpzi(n_iovrsd_s,maxres)
      call ccpzi(n_m_frd_xy,maxxyn*maxxyn)
      call ccpzi(n_m_rej_xy,maxxyn*maxxyn)
      call ccpzi(n_a_cent_s,maxres)
      call ccpzi(n_a_acen_s,maxres)
      call ccpzi(n_a_anom_s,maxres)
      call ccpzi(n_a_i_rf, maxinb*maxrun*2)
c
c
      do i=1,maxres
         sn_m_meas_s(i) = 0
      enddo
      call hstset
c
      do irun = 1, numrun
         do i = 1, mxstheta
            do j = 1, mxsphi
               nsphith(i,j,irun) = 0
               avphith(i,j,irun) = 0
            enddo
         enddo 
         sphmnmx(1,irun) =  1.0E+10
         sphmnmx(2,irun) = -1.0E+10
         do i = 1,maxinb
            sum_a_i_rf(i,irun,1) = 0.0
            sum_a_i_rf(i,irun,2) = 0.0
            smax_a_i_rf(i,irun,1) = -1000000.
            smax_a_i_rf(i,irun,2) = -1000000.
            smin_a_i_rf(i,irun,1) = +1000000.
            smin_a_i_rf(i,irun,2) = +1000000.
         enddo
      enddo
c
      return
      end
      subroutine cmpmvc
     $   (maxmtx, amatrx, gradnt, nrfc, npar, nparcm, iptpar)
c     =====================================================
c
c  Compress normal matrix and gradient vector to remove elements
c     corresponding to parameters with no observations
c
c  On entry:
c     maxmtx                   matrix dimension
c     amatrx(maxmtx,maxmtx)    normal matrix for npar parameters
c     gradnt(npar)             gradient vector for npar parameters
c     nrfc(npar)               number of observations for each parameter
c     npar                     number of parameters
c
c  On exit:
c     amatrx(maxmtx,maxmtx)    compressed normal matrix for nparcm parameters
c     gradnt(nparcm)           gradient vector for nparcm parameters
c     nparcm                   number of parameters after compression
c     iptpar(npar)             pointers from full parameter set to
c                              compressed values, = 0 for suppressed parameters
c                              ie if new parameter k == old parameter i
c                              then iptpar(i) = k
c
c
c
      integer maxmtx
cDP      real amatrx(maxmtx,maxmtx)
      double precision amatrx(maxmtx,maxmtx)
      include 'parameter.fh'
c
      integer npar, nrfc(npar), nparcm, iptpar(npar)
      real gradnt(npar)
c
      integer i,j,k,l
c
c
      k = 0
c
      do 10, i = 1, npar
         iptpar(i) = 0
         if (nrfc(i) .gt. 0) then
c  i'th parameter is valid
            k = k+1
            iptpar(i) = k
            if (k .ne. i) then
c    Copy gradient element
               gradnt(k) = gradnt(i)
c    Copy matrix elements
               l = 0
               do 20, j = 1,i
                  if (nrfc(j) .gt. 0) then
                     l = l+1
                     amatrx(k,l) = amatrx(i,j)
                  endif
 20            continue
            endif
         endif
 10   continue
c
      nparcm = k
      return
      end
c
c
c
      subroutine cntsph(rtheta, rphi, irun, Shl)
c     ====================================
c
c Count & average contribution to spherical harmonic coefficients
c
c On entry:
c  rtheta     theta  radians
c  rphi       phi radians
c  irun       run number (pointer to parameter set if runs linked)
c  Shl        calculated spherical harmonic sum
c
      real rtheta, rphi, Shl
      integer irun
c
      include 'parameter.fh'
      include 'sphhrm.fh'
c
c
      real dtor, theta, phi
      integer ntheta, nphi
c
      dtor = atan(1.)/45.
c
      ntheta = nint((rtheta/dtor)/dpltheta) + 1
      phi    = mod((rphi/dtor), 360.)
 10   if (phi .lt. 0.) then
         phi = phi + 360.
         go to 10
      endif
      nphi = nint(phi/dplphi) + 1
c
      nsphith(ntheta, nphi, irun) = nsphith(ntheta, nphi, irun) + 1
      avphith(ntheta, nphi, irun) = avphith(ntheta, nphi, irun) + Shl
      sphmnmx(1, irun) = min(sphmnmx(1, irun), Shl)
      sphmnmx(2, irun) = max(sphmnmx(2, irun), Shl)
c
      end
c  corner.f
c  ========
c
c A group of routines to apply the "corner correction" to eg MTZ
c data from eg Mosflm, using a look-up table provided as an image
c
c All internal data is stored in the common block /corner/,
c included from file corner.fh. This file should not be included
c in any other routine
c 
c Routines - call them in this order:
c
c >>> subroutine cnr_read_table(tabfilename, istat)
c 
c      Read file of correction table, as an ADSC style (SMV)
c      format image, Store everything in common block
c 
c      On entry:
c        tabfilename     filename or logical (TABLE) for table file
c
c      On exit:
c        istat     = 0 OK, = -1 fail
c
c >>>  subroutine cnr_setup(mtzin, xdetmax, ydetmax, istat)
c
c      Set up corner correction mapping, depending on detector size
c      taken eg from MTZ batch header
c
c      On entry:
c        xdetmax, ydetmax    maximum detector pixel coordinates
c        mtzin             mtz stream number for already opened file
c                            = size of detector
c      On exit:
c        istat   = 0 OK
c                > 0 fail, detector size does not match correction table
c                < 0 fail, 
c
c >>>  real function cnr_correction(xdet, ydet)
c
c       Returns correction factor for observation at pixel coordinates
c       XDET, YDET (from MTZ file). Also accumulates histogram of
c       correction factors
c
c >>>  subroutine cnr_print_histo
c
c       Print histogram of aplied correction factors
c
c Other routines:
c
c >>>   subroutine cnr_clear_histo
c
c       setup & clear histograms of corrections applied. This is
c       called automatically from within cnr_setup so doesn't normally
c       need to be called
c
c Internal routines:
c
c ...    subroutine cnr_SMVheader(header, k, key, value)
c
c        read one item from SMV style header block
c
c ...    integer*2 function i2swap(i2)
c
c        swap bytes in a 2-byte integer
c
c ...    integer function intfp (chstr, fp, iv) 
c
c        decode character string to number
c
c Dependencies:
c   CCP4 routines (libccp4f, libccp4c)
c
c      diskio routine (qopen, qmode, qread) to read table file
c      ugtenv to translate "logical" filename
c      ccperr  failure report


      subroutine cnr_read_table(tabfilename, istat)
c    =========================================
c
c  Read file of corner correction table, create table for correction
c
c  File is read as an ADSC image, SMV format
c  Format is SMV.  In this case, the header is 512 bytes.  There is
c  some useful info in the header which could be used in the code
c  below, but for the present it is not.  Particularly, SIZE1 and SIZE2
c  correspond to the NBINSX/Y values set above, and the 4096 value is
c  contained in the header as well (SCALE).

c  The file refers to pixels in ADSC format which is flipped compared
c  to the Mosflm convention (ie interchange x & y)
c
c On entry:
c  tabfilename     filename or logical (TABLE) for table file
c
c On exit:
c  istat     = 0 OK, = -1 fail
c 
c  In common /corner/
c  corfac     table in common /corner/
c  + other things, see corner.fh
c 
      character*(*) tabfilename
      integer istat

      include 'corner.fh'

c Locals      
      integer ltabfil, ifail, k, ival, mode, isize, ixa, iya
      real fval
c  scale factor for values in file: these are scale/corfac
      real scale
c  array size in file, nbinsx is fast index
      integer nbinsx, nbinsy
      integer img1, img2, detector_num, nunity
c line buffer
      integer*2 i2line(MAXBIN)

      integer LENHEADER
      parameter (LENHEADER = 512)
      character*(LENHEADER) header

      character*40 key, value
      character*200 realfilename

      integer intfp, lenstr
      integer*2 i2swap
      external intfp, i2swap, lenstr
      logical litend
      external litend
c
c--  Defaults
      istat = 0
      nbinsx = 384
      nbinsy = 384
      img1 = 3072
      img2 = 3072
      litendfile = .true.
      scale = 4096.
      detector_num = 0
c don't swap bytes
      swap = .false.
c do flip image
      flip = .true.
c--

c Header of 512 bytes to be interpreted as characters
c followed by nbinsx * nbinsy integer*2 numbers
      call ugtenv(tabfilename, realfilename)
      if (realfilename .eq. '') realfilename = tabfilename
      call qopen(ltabfil, tabfilename, 'READONLY')

      call qreadc(ltabfil, header, ifail)

      k = 0
 10   call cnr_SMVheader(header, k, key, value)
      if (k .gt. 0) then
c  Try to decode value
c  ifail  =  0  not number
c         = -1  blank
c         = +1  integer
c         = +2  float
         ifail = intfp(value, fval, ival)

c Sample header block
c   HEADER_BYTES=  512;
c   COMMENT=Written by xds2cor;
c   DIM=2;
c   SIZE1=384;
c   SIZE2=384;
c   DETECTOR_SN=917;
c   DATE=Tue Aug 15 17:02:42 2006
c   TYPE=unsigned_short;
c   BYTE_ORDER=little_endian;
c   SCALE=4096;
c   BINNING=8;
c   IMG_PIXEL_SIZE=0.051294;
c   IMG_SIZE1=6144;
c   IMG_SIZE2=6144;
         if (index(key, 'IMG_SIZE1') .ne. 0) then
            if (ifail .eq. 0) go to 999
            img1 = ival
         else if (index(key, 'IMG_SIZE2') .ne. 0) then
            if (ifail .eq. 0) go to 999
            img2 = ival
         else if (index(key, 'SIZE1') .ne. 0) then
c  NB  this one must come after IMG_SIZEx
            if (ifail .eq. 0) go to 999
            nbinsy = ival
         else if (index(key, 'SIZE2') .ne. 0) then
            if (ifail .eq. 0) go to 999
            nbinsx = ival
         else if (index(key, 'SCALE') .ne. 0) then
            if (ifail .eq. 0) go to 999
            scale = fval
         else if (index(key, 'DETECTOR_SN') .ne. 0) then
            if (ifail .eq. 0) go to 999
            detector_num = ival
         else if (index(key, 'BYTE_') .ne. 0) then
         endif
         go to 10
      endif
      if (nbinsx .gt. MAXBIN .or. nbinsy .gt. MAXBIN) then
         write (6,*) ' **** Array in TABLE file too large: ',
     $      nbinsx, nbinsy, MAXBIN
         istat = -1
         return
      endif

c  Do we need to swap bytes?
c  litend() function in ccplib returns .true. if this machine
c is little-ended
      swap = (litend(1) .neqv. litendfile)
      
c ADSC call their fast index "x" (=ixa) and slow index "y" (=iya)
c Corresponding dimensions of array in file are nbinsx, nbinsy
c  Mosflm convetntion is flipped
      if (flip) then
         ncfcx = nbinsy
         ncfcy = nbinsx
         img_xdet = img2
         img_ydet = img1
      else
         ncfcx = nbinsx
         ncfcy = nbinsy
         img_xdet = img1
         img_ydet = img2
      endif

c Loop to read rows of data from file
c  Set mode = 1 = short integer
      mode = 1
      call qmode(ltabfil, mode, isize)
      cormax = -1.0
      nunity = 0

      do iya=1,nbinsy
         call qread(ltabfil, i2line, nbinsx, ifail)
         if (ifail .ne. 0) then
            call ccperr(1,'**** Crash in reading TABLE ****')
         endif

         do ixa=1,nbinsx
            if (swap) then
               ival = i2swap(i2line(ixa))
            else
               ival = i2line(ixa)
            endif
            fval = scale/float(ival)
            cormax = max(cormax, fval)
c   Count null correction pixels
            if (abs(fval-1.0) .lt. 0.00001) nunity = nunity+1

            if (flip) then
               corfac(iya, ixa) = fval
            else
               corfac(ixa, iya) = fval
            endif
         enddo
      enddo

      write (6, 6001) realfilename(1:lenstr(realfilename)),
     $   nbinsx, nbinsy,
     $   scale, img_xdet, img_ydet, detector_num,
     $   nunity, nbinsx*nbinsy
 6001 format(
     $   /'Corner correction table read from file ',a,//,
     $   '  Array size: ' , 2i5, '  scale = ', f7.0,
     $   /,'  Corresponding to image size', 2i6,
     $   ' or half if binned'/
     $   '  Detector serial number ', i8,/,
     $   '  Number of table values of 1, ie no correction = ', i8,
     $   ' out of ', i8) 
      if (swap)
     $   write (6,*) ' Bytes swapped in reading table file'
      if (flip)
     $   write (6,*)
     $   ' Correction image flipped to match Mosflm convention'
      write (6,6002) cormax
 6002 format('  Maximum correction factor:', f8.4)
      write (6,'(//)')

      kstatus = +1

      return

 999  write (6,*) 'Syntax error', key, value
      end


      subroutine cnr_setup(mtzin, xdetmax, ydetmax, istat)
c    =============================================
c
c  Set up corner correction mapping
c
c On entry:
c  mtzin               mtz stream number for already opened file
c  xdetmax, ydetmax    maximum detector pixel coordinates
c                      = size of detector
c
c On exit:
c  istat   = 0 OK
c          > 0 fail, detector size does not match correction table
c          < 0 fail, correction already applied
c
      integer mtzin, xdetmax, ydetmax, istat

      include 'corner.fh'

c Check if correction has been applied already
      call cnr_check_corrected(mtzin, istat)
      if (istat .ne. 0) then
         write (6,*)
     $      '**** Corner correction has been applied previously ****'
         return
      endif

c  Check for valid size: binned image may be half notional size
      istat = 0
      if (xdetmax .ne. img_xdet .and. xdetmax .ne. img_xdet/2)
     $   istat = +1
      if (ydetmax .ne. img_ydet .and. ydetmax .ne. img_ydet/2)
     $   istat = +2
      if (istat .ne. 0) then
         write (6, 6001) xdetmax, ydetmax, img_xdet, img_ydet,
     $      img_xdet/2, img_ydet/2
 6001    format(//
     $      '**** ERROR: detector size (pixels) from HKLIN file ',
     $      2i6, ' do not match size from correction TABLE ****'/
     $      'Size should be ', 2i6, ' or ', 2i6//)
         return
      endif

      max_xdet = xdetmax
      max_ydet = ydetmax

c  Pixel coordinates XDET, YDET are mapped to correction table
c coordinates by multiplying by xmap, ymap
c      ixtable = min(max(nint(xdet*xmap+0.499),1),ncfcx)
c      iytable = min(max(nint(ydet*ymap+0.499),1),ncfcy)
      xmap = float(ncfcx)/float(max_xdet)
      ymap = float(ncfcy)/float(max_ydet)

      kstatus = +2
      call cnr_clear_histo

      end

      subroutine cnr_clear_histo
c    =======================
c
c  setup & clear histograms of corrections applied
c
      include 'corner.fh'

      integer i

      if (kstatus .lt. +2) then
         call ccperr(1,'**** CNR_CLEAR_HISTO table not set up ****')
      endif 

      nhisto = MAXHISTO
      histo_bin = (cormax-1.0)/float(nhisto)

      do i=1,nhisto
         histo_counts(i) = 0
      enddo

      ncnall = 0
      ncncor = 0

      kstatus = +3
      return
      end

      subroutine cnr_print_histo
c    =======================
c
c Print histogram
c
      include 'corner.fh'

      real bin_limits(MAXHISTO+1)
      integer i

      if (kstatus .lt. +3) then
         call ccperr(1,'**** CNR_PRINT_HISTO histogram not set up ****')
      endif       

      do i=1,nhisto+1
         bin_limits(i) = 1.0 + (i-1) * histo_bin
      enddo

      write (6, 6001) ncncor, ncnall, (bin_limits(i), i=1,nhisto+1)
 6001 format(//'Detector corner correction: ', i8,
     $   ' observations corrected out of total ', i8//
     $   'Histogram of corner correction factors:',
     $   ' number corrected in each bin'//
     $   1x,f5.3,10(' - ',f5.3))
      write (6,6002) (histo_counts(i), i=1,nhisto)
 6002 format(/1x,10(i8))
      write (6,'(/)')
      return
      end

      real function cnr_correction(xdet, ydet)
c    ===================================
c
c Returns correction factor for observation at pixel coordinates
c xdet, ydet
c
      real xdet, ydet
      include 'corner.fh'

      integer ixtable, iytable, ihist
      real correction

      if (kstatus .lt. +3) then
         call ccperr(1,'**** CNR_HISTOGRAM table not set up ****')
      endif

      ixtable = min(max(nint(xdet*xmap+0.499),1),ncfcx)
      iytable = min(max(nint(ydet*ymap+0.499),1),ncfcy)

      correction = corfac(ixtable, iytable)

c  count total and those with non-unit correction
      ncnall = ncnall+1
      if (abs(correction - 1.0) .gt. 0.001) then
         ncncor = ncncor+1

         ihist =
     $      min(max(int((correction-1.0)/histo_bin)+1, 1), nhisto)
         histo_counts(ihist) = histo_counts(ihist) + 1
      endif

      cnr_correction = correction
      return
      end


      subroutine cnr_SMVheader(header, k, key, value)
c    =========================================
c
c  Reads lines of form "key=value;" from header
c
c On entry:
c  header    complete header block
c  k         current position in block
c            if k .le. 0, skip initial "{" first
c
c On exit:
c  key       keyword
c  value     value
c  k         updated position, = -1 for end of block
      character*(*) header, key, value
      integer k

      integer length, i

      length = len(header)

      if (k .le. 0) then
         k = 1
c Skip initial "{\n"
 1       if (k .lt. length) then
            if (header(k:k) .eq. '{') go to 10
            k = k+1
            go to 1
         endif
 10      k = k+1
      endif
      
 11   if (k .lt. length) then
c start new line, look for '='
         key = ''
         i = 0
 20      if (header(k:k) .ne. '=') then
            if (header(k:k) .eq. '}') then
               k = -1
               return
            endif
            if (header(k:k) .ne. '\n') then
               i = i+1
               key(i:i) = header(k:k)
            endif
            k = k+1
            go to 20
         endif
         k = k+1
         i = 0
         value = ''
 30      if (header(k:k) .ne. ';') then
             if (header(k:k) .eq. '\n') then
c  end of line without a ';'
                go to 31
             endif
             i = i+1
             value(i:i) = header(k:k)
             k = k+1
             go to 30
         endif
c  ";" found
         k=k+1
 31      if (header(k:k) .eq. "}") then
            k = -1
         endif 
         return
      endif
      k = -1
      return
      end



      subroutine cnr_check_corrected(mtzin, istat)
c    ======================================
c
c Check if correction has been applied already
c Test all history lines for names of correction programs
c
c  mtzin               mtz stream number for already opened file
c 
c Return istat = -1 if it has
c
      integer mtzin, istat
      integer nhistlines
      parameter (nhistlines=20)
      character*(80) history(nhistlines)
      integer nlines

      nlines = nhistlines
      call lrhist(mtzin, history, nlines)

      istat = 0
      do i=1, nlines
         if (index(history(i), 'correct_detector') .ne. 0) istat = -1
         if (index(history(i), 'fix_mosflm_output') .ne. 0) istat = -1
      enddo

      return
      end


      integer*2 function i2swap(i2)
c      ================================
c
c  Swap bytes in integer*2, returns swapped value
      integer*2 i2

      integer*2 ieight, i255
      parameter (ieight = 8, i255 = 255)

      i2swap = ior(iand(ishft(i2,-ieight),i255), ishft(i2,ieight))
      return
      end


C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C***********************
C**     INTFP         **
C***********************
C
C
       INTEGER FUNCTION INTFP (CHSTR, FP, IV) 
C
C PURPOSE:    Decode a character string into a number (floating and integer)
C
C AUTHOR:     John W. Campbell
C
C RETURN:     = 0  Syntax error in the number
C             = 1  integer  (no decimal point present)
C             = 2  floating point (decimal point present or E format)
C             = -1 blank string
C                                             
C ARGUMENTS:
C
C      CHSTR    (R)     Character string containing the number to be decoded
C                       (Leading spaces ignored; further space terminates
C                        the number string interpreted)
C      FP       (W)     Returns the value as a floating point value (0.0 if
C                       syntax error)
C      IV       (W)     Returns the value as an integer (the nearest integer
C                       if value contained a decimal point) (0 if syntax error)
C
C
C Declarations
C ============
C
C LOCALS:
C
      CHARACTER*(*) CHSTR
      CHARACTER*15 DIG
      LOGICAL END, START
      DOUBLE PRECISION FPVAL, X, S
      real fp
      integer iv, iexp, m1, m2, i, imax, id, ifdec, j, iex
      DATA DIG /'eE0123456789.+-'/
C
C-------------------------------------------------------------------------------
C
C
C====== Initialisations
C
      FPVAL = 0.0
      FP = 0.0
      IV = 0
      END = .FALSE.
      START = .FALSE.
      IEXP = 0
      M1 = 3
      M2 = 15
      I = 0
      IMAX = LEN (CHSTR) 
      IF (CHSTR.EQ.' ') THEN
         INTFP = -1
         RETURN
      ENDIF
C
C====== Interpret Number
C
100   S = 1.0
      ID = 0
      IFDEC = 0
      X = 0.0
120   I = I + 1
      IF (I.GT.IMAX) THEN
         END = .TRUE.
         IF (IEXP.EQ.1) GO TO 600
         GO TO 500
      ENDIF
      IF (CHSTR(I:I).EQ.' ') THEN
         IF (START) I = IMAX
         GO TO 120
      ENDIF
      J = INDEX(DIG(M1:M2),CHSTR(I:I))
      IF (J.EQ.0) GO TO 800
      START = .TRUE.
      J = J + M1 - 1
      M1 = 1 + 2*IEXP
C
C====== Digit
C
      IF (J.GE.3.AND.J.LE.12) THEN
         ID = ID + IFDEC
         X = 10.0 * X + J - 3
         IF (M2.GT.13) M2 = 13
         GO TO 120
C
C====== Decimal point
C
      ELSE IF (J.EQ.13) THEN
         IF (IEXP.EQ.1) GO TO 800
         IFDEC = 1
         M2 = 12
         GO TO 120
C
C====== Sign
C
      ELSE IF (J.GE.14) THEN
         IF (J.EQ.15) S = -1.0
         M2 = 13
         GO TO 120
C
C====== Exponent
C
      ELSE
         IEXP = 1
         M1 = 3
         M2 = 15
      ENDIF
C
C====== Evaluate number
C
500   FPVAL = S * X / (10**ID)
      IF (END) THEN
         FP = FPVAL
         IV = NINT (FP)
         INTFP = 1
         IF (IFDEC.GT.0) INTFP = 2
         RETURN
      ENDIF
      GO TO 100
C
C====== Evaluate and apply exponent
C
600   IEX = NINT (S*X)
      FPVAL = FPVAL * 10.0**IEX
      FP = FPVAL
      IV = NINT (FP)
      INTFP = 2
      RETURN
C
C====== Error Condition
C
800   INTFP = 0
      RETURN
      END

      subroutine corvar(lflag, irun, sdIin, cLP, cIhl, mpt,
     $   sdI, dshldq, dshldg)
c     ================================================================
c
c
c  Calculate corrected Variance(Ihl), & optionally partial derivatives
c
c Uses correction factors sdfact(3,irun) from common sdfacc
c
c Changes:
c  3.3.0 included LP term on SdB
c  3.3.9 removed cLP term
c!old sdI =  sdfact(1,irun) * sqrt[(sdIin**2 +
c!old                               sdfact(2,irun) * cIhl * cLP
c!old                               + (sdfact(3,irun) * cIhl)**2)]
c sdI =  sdfact(1,irun) * sqrt[(sdIin**2 +
c                               sdfact(2,irun) * cIhl
c                               + (sdfact(3,irun) * cIhl)**2)]
c     where cIhl = intensity
c
c On entry:
c     lflag     if .true.  derivatives calculated
c     irun      run number for this observation
c     sdIin     input sd(Ihl) ie raw standard deviation
c     cLP       LP factor   (not used now! from version 3.3.9)
c     cIhl      intensity
c     mpt       = 0 full, = 1 partial
c
c On exit:
c     sdI       "corrected" sd(Ihl)
c
c   if iflag .gt. 0
c     dshldq(6) partial derivatives of sd(Ihl) (NB! NOT of variance)
c                 with respect to variance correction parameters
c                 sdfact(1-maxsdf). 1-3, 1st derivatives;
c                 4-6 diagonal 2nd derivatives
c     dshldg    partial derivative of sd(Ihl) with respect to
c                 scale factor for this reflection,
c                 d sd(Ihl) / d  ghl 
c
      implicit none
c
      logical lflag
      integer irun, mpt
      real varIin, cIhl, dshldq(*), dshldg, sdIin, sdI, cLP
c
      include 'parameter.fh'
      include 'sdfacc.fh'
c
      real ss, sqs, qsqs, sdfmin, sdf3sq, sgnsdf3sq
      integer mp
c
      data sdfmin/0.1/
c
      if (mpt .eq. 0) then
         mp = 1
      else
         mp = 2
      endif
      varIin = sdIin**2
c Allow for negative SDADDD
      sgnsdf3sq = +1.0
      sdf3sq = sdfact(3,irun,mp)**2
      if (sdfact(3,irun,mp) .lt. 0.0) then
         sdf3sq = -sdf3sq
         sgnsdf3sq = -1.0
      endif
c
      ss = (varIin + cIhl*(sdfact(2,irun,mp) +
     $   cIhl * sdf3sq))
      ss = max(ss, VarIin*sdfmin)
      sdI = sqrt(ss) * sdfact(1,irun,mp)

c
      if (lflag) then
c
c FIXME: check sgnsdf3sq in the following derivatives
c Calculate derivatives
c   d sd(Ihl) / d sdfact
         sqs = sqrt(ss)
         dshldq(1) = sqs
         qsqs = 0.5 * sdfact(1,irun,mp) / sqs
         dshldq(2) = qsqs * cIhl
         dshldq(3) = dshldq(2) * cIhl * 2.0 * sdfact(3,irun,mp)
     $      * sgnsdf3sq 
c  diagonal 2nd derivatives
         dshldq(4) = 0.0
         dshldq(5) = -0.5 * dshldq(2) * cIhl / ss
         dshldq(6) = dshldq(5) * cIhl * cIhl * 2.0 *
     $      sdfact(3,irun,mp) * sgnsdf3sq + 2.0 * cIhl * dshldq(2)
c   d sd(Ihl) / d (ghl Ih)
         dshldg = qsqs *
     $        (cIhl * (sdfact(2,irun,mp) +
     $        2.0 * sdfact(3,irun,mp)**2 * cIhl * sgnsdf3sq))
      endif
c
      return
      end
      subroutine cpybth(jdataset, kout)
C    ============================
C
C  Copy batch headers from input to output file, for all batches
C     except those excluded, and reference batch if lrefou is not set
C
c  Write out headers only for jdataset'th dataset, = 0 for all
c
c kout is outpu MTZ stream number
c
      implicit none
C
      integer jdataset, kout
c
      include 'parameter.fh'
      include 'nbtchc.fh'
      include 'orient.fh'
      include 'rundef.fh'
      include 'outcon.fh'
      include 'inout.fh'
      include 'datasets.fh'
C     ..
      integer i, j, ibatch, iprint
C
c-save-      save /nbtchc/
C
      do 10, i  = 1, nbtall
C---- Wanted batches only, based on output datasets
         if (irunbt(i) .gt. 0) then
            if (jdataset .eq. 0 .or. kdtsbt(i) .eq. jdataset) then
               if (irunbt(i) .ne. irefrn .or. lrefou) then
                  ibatch = ibtall(i)
C Position header read to batch IBATCH
                  call lrbres(1, ibatch)
                  iprint = 0
C     Read orientation block
                  call lrbat(1,j,nwords,btitle,iprint)
C Check right one (I'm not sure this can ever be wrong)
                  if (j .ne. ibatch) then
                     write (lunout,'(A,I8,A,I8)')
     $                  ' !!!! Orientation data for batch',
     $                  j,' found when expecting data for batch',ibatch
                     call dna_error('invalid orientation')
                     call ccperr(1,'*** Invalid orientation data ***')
                  endif
c
                  call lwbat(kout, ibatch, nwords, btitle)
c  (re)set dataset
c*4*                  call lwbsetid(kout,ibatch,
c*4*     $               pname_out( kdtsbt(i)),dname_out( kdtsbt(i)))
                  call lwbsetidx(kout,ibatch,
     $               xname_out( kdtsbt(i)),dname_out( kdtsbt(i)))
c*5*
               endif
            endif
         endif
C
 10   continue
C
      end
      subroutine crycdn(ihkl, isym, irun, ibatch, phi,
     $   xsi, xdn, ydn, zdn, jserbt)
c     ===============================================
c
c  Determine crystal frame coordinates of scattering vector
c On entry:
c  ihkl(3)        h, k, l, reduced indices
c  isym           symmetry number (to get original indices)
c  irun           run number
c  ibatch         batch number
c  phi             rotation angle
c  jserbt         batch number within run (for BATCH mode)
c
c On exit:
c  xsi            radius from rotation axis (=sqrt(x**2 + y**2) (if available)
c  xdn, ydn       polar angles in permuted crystal frame
c                       x = theta (radians), y = phi (radians)
c  zdn            normalized rotation angle
c
      implicit none
      integer ihkl(3), isym, irun, ibatch, jserbt
      real  phi, xsi, xdn, ydn, zdn
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
c
      real    hkl(3), xyz(3), rxy
      integer jhkl(3), kmat, i
c
      integer ccp4_hash_lookup
      external ccp4_hash_lookup, matmul
c
c Original indices jhkl
      call asuget(ihkl, jhkl, isym)
      hkl(1) = jhkl(1)
      hkl(2) = jhkl(2)
      hkl(3) = jhkl(3)
c Pointer to UB matrix
      kmat = jmtxbt(jserbt,irun)
c
      if (kmat .gt. 0) then
c x = [UB} h
         call matvec(xyz, dubmtxs(1,1,kmat), hkl)
c Cylindrical radius from rotation axis
         xsi = sqrt(xyz(1)**2 + xyz(2)**2)
      else
         xsi = 0.0
      endif
c
      call matvec(xyz, bmatrn(1,1,irun), hkl)
c  z is polar axis
      rxy = sqrt(xyz(1)**2 + xyz(2)**2)
c   theta
      xdn = atan2(rxy, xyz(3))
c   phi
      if (rxy .ne. 0.0) then
         ydn = atan2(xyz(2), xyz(1))
      else
         ydn = 0.0
      endif
c
      if (mbatsc(irun) .ne. 0) then
         if (lbtslp(irun)) then
c  Sloped batch scale, store relative phi (0-1 within rotation range)
c     packed with batch number within run
            zdn = btslpf*jserbt +
     $           (phi - philim(1,jserbt,irun))/phirng(jserbt,irun)
         else
c  Plain batch scale
            zdn = jserbt
         endif
      else
         zdn = (phi - zlimrn(1,irun)) / spcscl(3,irun)
      endif
c
      return
      end
      SUBROUTINE DCOSFD(ROT,DIRCOS,KAPPA)
C     ===================================
C******************************************************
C       Given a rotation matrix ROTN, expressing a rotation
C       through an angle KAPPA right-handedly about an axis
C       with direction cosines DIRCOS(I),
C       DCOSFD determines DIRCOS() and KAPPA, given
C       that DIRCOS(1) is defined to be positive (otherwise
C       one would have the other end of the rotation axis).
C       KAPPA is returned in degrees
C
C       Expression for rotation matrix is (see J & J, "Mathl.
C               Phys.", p. 122):-
C
C  cw + n(1)n(1)(1-cw)    n(1)n(2)(1-cw)-n(3)sw  n(1)n(3)(1-cw)+n(3)sw
C  n(1)n(2)(1-cw)+n(3)sw  cw + n(2)n(2)(1-cw)    n(2)n(3)(1-cw)-n(1)sw
C  n(3)n(1)(1-cw)-n(2)sw  n(3)n(2)(1-cw)+n(1)sw  cw + n(3)n(3)(1-cw)
C
C       where cw = cos(KAPPA), sw = sin(KAPPA),
C       & n(1), n(2), n(3) are the direction cosines.
C
C*******************************************************}
C
        REAL ROT(3,3),DIRCOS(3),S(3),P(3),D(3)
        REAL KAPPA
      REAL DIFF,PI,TRACE,R2,R3,SINKAP,COSKAP
        INTEGER NS(3),NP(3),NCS(3),ND(3),KNTRL,J,JMX
C
          DIFF=1.0E-04
          PI=4.0*ATAN(1.)
          KNTRL=0
C       Trace = 1 + 2.cos w
          TRACE=ROT(1,1)+ROT(2,2)+ROT(3,3)
C       S(1)=2.n(1).sin w
C       S(2)=2.n(2).sin w
C       S(3)=2.n(3).sin w
          S(1)=ROT(3,2)-ROT(2,3)
          S(2)=ROT(1,3)-ROT(3,1)
          S(3)=ROT(2,1)-ROT(1,2)
          CALL ORDR3(KNTRL,S,NS)
C--Is biggest S() zero (i.e. is sin w = 0)?
          IF (ABS(S(NS(1))).GT.DIFF) THEN
            R2=S(NS(2))/S(NS(1))
            R3=S(NS(3))/S(NS(1))
            DIRCOS(NS(1))=1.0/SQRT(1 + R2**2 + R3**2)
            DIRCOS(NS(2))=R2*DIRCOS(NS(1))
            DIRCOS(NS(3))=R3*DIRCOS(NS(1))
            GO TO 123
          END IF
C--Calculation when sin w = 0 (esp. when w = 180').
C       P(1)=n(2).n(3).{0(W=0)or2(w=180)}
C       P(2)=n(3).n(1).{0(W=0)or2(w=180)}
C       P(3)=n(1).n(2).{0(W=0)or2(w=180)}
 30       P(1)=ROT(3,2)
          P(2)=ROT(1,3)
          P(3)=ROT(2,1)
          CALL ORDR3(KNTRL,P,NP)
C--Is biggest P() zero (i.e. are all off-diag. terms zero)?
          IF (ABS(P(NP(1))).LT.DIFF) THEN
            IF (TRACE.GT.0.0) THEN
C--Matrix is unit matrix.
                KAPPA=0.0
                DIRCOS(1)=0.0
                DIRCOS(2)=0.0
                DIRCOS(3)=1.0
                RETURN
            END IF
C--Trace -ve, so dyad about x,y or z.
            KAPPA=PI
            DO 40 J=1,3
 40           D(J)=ROT(J,J)+1.0
            CALL ORDR3(KNTRL,D,ND)
            DIRCOS(ND(1))=1.0
            DIRCOS(ND(2))=0.0
            DIRCOS(ND(3))=0.0
            RETURN
          END IF
        IF (ABS(P(NP(2))).LT.DIFF) THEN
C--One d.c. is zero.
          DIRCOS(NP(1))=0.0
          DIRCOS(NP(2))=SQRT(0.5*(1.0+ROT(NP(2),NP(2))))
          DIRCOS(NP(3))=SQRT(0.5*(1.0+ROT(NP(3),NP(3))))
          IF (P(NP(1)).LT.0.0) DIRCOS(NP(2))=-DIRCOS(NP(2))
          GO TO 123
        END IF
C--All d.c's are nonzero.
C       R2=n(NP(1))/n(NP(2))
C       R3=n(NP(1))/n(NP(3))
          R2=P(NP(2))/P(NP(1))
          R3=P(NP(3))/P(NP(1))
          DIRCOS(NP(2))=1.0/SQRT(1 + R2**2 + (R2/R3)**2)
          DIRCOS(NP(3))=1.0/SQRT(1 + R3**2 + (R3/R2)**2)
          DIRCOS(NP(1))=SQRT(1.0-DIRCOS(NP(2))**2-DIRCOS(NP(3))**2)
C--Adjust signs of DIRCOS().
          JMX=0
          DO 20 J=1,3
            IF (P(J).GT.0.0) THEN
              IF (JMX.GT.0) JMX=-1
              IF (JMX.EQ.0) JMX=J
            END IF
 20       CONTINUE
          IF (JMX.GT.0) DIRCOS(JMX)=-DIRCOS(JMX)
          IF (JMX.EQ.0) WRITE(*,1358)
 1358     FORMAT(' All P()s. are negative or zero')
C--Given d.c's, calculate KAPPA.
123       KNTRL=1
          CALL ORDR3(KNTRL,DIRCOS,NCS)
C       Find KAPPA.
          SINKAP=ROT(NCS(3),NCS(2))-ROT(NCS(2),NCS(3))
          SINKAP=0.5*SINKAP/DIRCOS(NCS(1))
          COSKAP=0.5*(TRACE-1.0)
          KAPPA=ATAN2(SINKAP,COSKAP)
          KAPPA=KAPPA*180.0/PI
        RETURN
        END
      subroutine detcdn(xd, yd, zd, irun, xdn, ydn, zdn, jserbt)
c     ==========================================================
c
c Determine normalized detector coordinate, using scale layout from
c     common /scales/
c
c On entry:
c     xd, yd   position on detector
c     zd       rotation (phi or batch)
c     irun     run number
c     jserbt   batch number within run (for BATCH mode)
c
c On exit:
c     xdn, ydn  normalized detector coordinates, ie = 0.0 at beginning,
c     zdn       normalized rotation
c              for BATCH mode = batch number within run
c              for SLOPE mode = 100 * (batch number within run) +
c                               relative phi within run
c              else   = 1, 2, 3 etc at positions of scales
c
      implicit none
c
      real xd, yd, zd, xdn, ydn, zdn
      integer irun, jserbt
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
c
      xdn = (xd - dlimrn(1,1,irun)) / spcscl(1,irun)
      ydn = (yd - dlimrn(1,2,irun)) / spcscl(2,irun)
c
      if (mbatsc(irun) .ne. 0) then
         if (lbtslp(irun)) then
c  Sloped batch scale, store relative phi (0-1 within rotation range)
c     packed with batch number within run (OK if phirng < 0)
            zdn = btslpf*jserbt +
     $           (zd - philim(1,jserbt,irun))/phirng(jserbt,irun)
         else
c  Plain batch scale
            zdn = jserbt
         endif
      else
         zdn = (zd - zlimrn(1,irun)) / spcscl(3,irun)
      endif
c
      return
      end
      subroutine dffscn(irun, stovl, phi, xsi, fractn, ibatch, nmpr,
     $   ghl, Dfhl, dghldc, istat)
c     ===========================
c
c  Diffuse scattering (tails) correction
c  Calculate updated scale ghl & partial derivative vector dghldc
c
c On entry:
c   irun     internal run number
c   ghl      current dividing scale (from subroutine sclder)
c   stovl    (sin theta/lambda)**2
c   phi      rotation angle for this reflection
c   xsi      radius from rotation axis
c   fractn   partial fraction, .gt. 1 for full or summed partial
c   ibatch   batch number
c   nmpr     number of images contributing to this reflection
c
c On exit:
c   ghl      updated dividing scale
c   Dfhl     diffuse (tail) contribution, ie ghl' := ghl * Dfhl
c   dghldc(npar)  partial derivatives d(ghl) / d(parameter)
c            derivatives w.r.t diffuse correction parameters are at end
c              (at present, v, alpha0, alpha1)
c            array lptfix (common /refcon/) is consulted for FIX flags:
c            for FIXed parameters, the derivative is set = 0
c            the other derivatives are multiplied by Dfhl
c   istat    status
c            =  0  OK
c            = -1  final batch outside range of run (fatal)
c
c====== Algorithm ========
c
c Some of the ideas are based on the method described by R.H.Blessing,
c Cryst. Rev. 1,3-58, (1987)
c
c We consider the profile on phi of the reflection normalized into the
c reciprocal space coordinate q (in A**-1). A rotation through an angle
c delphi moves the reflection a distance xsi * delphi in q-space, where
c xsi is the radius from the rotation axis. In q-space, the reflection
c width will be much more uniform than in phi-space.
c
c The profile of the reflection is considered in two parts: the Bragg
c reflection profile, about which we make no assumptions, sitting on top
c of a diffuse scattering peak, which we here model as a triangle. This
c triangle is defined by two parameters, the half-width v and the height h.
c                         h is in turn a function of alpha0, defined as 
c           / \           follows.
c          / | \
c         /  |  \        
c        /   |h  \       
c       /    |    \           
c      /_____|_____\   ---> q
c         v     v
c
c       ^      ^
c eg    u1     u2           first and last points used in integration
c
c-- For a full profile (ie sampled at least from -v to +v along q):
c
c apparent (measured) intensity I = J * (1 + alpha) where J is the "true"
c intensity
c
c     I - J = h * v = J * alpha 
c
c or  J = I / (1+alpha)
c
c BUT I don't trust this model well enough to rely on it to do a true
c diffuse scattering correction, & the (1+alpha) correction scales down
c high resolution reflections by a substantial amount relative to low
c reoslution (ie increases the overall B factor a lot), since there is
c more diffuse scattering at high resolution.
c
c The principle desired effect of this correction is to make partials
c agree better with fulls, ie a correction based on the scan width.
c For this reason, the "diffuse scattering" correction is renormalized
c so that a reflection scanned over a width greater than v is not altered
c (scale = 1.0)
c
c     h = J * alpha / v
c
c
c-- If the profile is truncated on q (ie sampled from u1 to u2):
c
c   then less of the diffuse scattering tail will be included in the 
c integrated intensity, by an amount depending on the shape & size of
c the tails
c
c  Writing the first sampled point as u1, the last as u2
c     Ip - Jp  = h * v * (1 - C1 - C2)
c
c    where Jp is the "true" intensity for the part sampled (= J for a
c fully recorded reflection even if the tails have been clipped), Ip
c is the measured intensity (intensity corrected for partiality would
c be I = Ip/f: note that usually f = 1.0 since we are working with
c summed partials or fulls)
c
c    where C1 = {  0                      if u1 > v
c               {  ((v-u1)/v)**2 / 2      if  0 < u1 < v
c               {  1 - ((v+u1)/v)**2 / 2  if u1 < 0
c
c    and similarly for C2(u2)
c         C1 & C2 are related to the areas of the tails clipped off by
c                  A  =  h v C
c
c  Substituting for h = J * alpha / v    [NB J is full intensity here]
c  and J = Jp/f  where f is fraction (in range 0 -> 1)
c
c    Ip - Jp = (Jp / f) * t
c
c where
c  t(u1,u2,s**2,alpha0,alpha1,v) = alpha *  (1 - C1 - C2)
c
c  alpha = alpha0 + alpha1 * (sin theta/lambda)**2  (isotropic assumption)
c     (this simple model is based on a suggestion by Blessing, but
c      adding the zero-angle term)
c     Alternative expressions for alpha could be possible, eg anisotropic
c
c  Then  J = (1/(1+t/f)) I
c
c  This would correct for the diffuse scattering model
c
c- Step 2: Renormalization to remove overall theta dependence
c
c   J = [(1 + alpha/f)/(1 + t/f)] I
c
c   Dfhl = (1 + t/f)/(1 + alpha/f)       dividing scale factor (like ghl)
c
c This = 1.0 if C1 = C2 = 0, ie for a complete profile
c  
c---
c  u1 = xsi * (phi - phi1)
c  u2 = xsi * (phi2 - phi)
c     where
c           phi   rotation angle of reflection
c           phi1  rotation angle at start of first image
c           phi2  rotation angle at end of last image
c
c  This makes Dfhl = g a function of 3 global parameters,
c v, alpha0 & alpha1
c
c  NB but what happens if f is small?? This correction is at present
c not used when f is small, and usually f = 1.0
c
c
c========================================================================
c
c
      include 'parameter.fh'
      include 'params.fh'
      include 'rundef.fh'
      include 'scales.fh'
      include 'refcon.fh'
c
      integer irun, ibatch, nmpr, istat
      real ghl, stovl, xsi, phi, fractn, Dfhl, dghldc(maxpmr)
c
c Locals
      integer i, j, k, jser, jserbt, jserb2, ir
      real phi1, phi2, v, alpha0, c1, c2, dc1dv, dc2dv, dbdv,
     $   u1, u2, vbb, t, dtor, f, alpha, alpha1, dbdal0, dbdal1,
     $   tfp1, afp1, temp
c
      integer ccp4_hash_lookup
      external ccp4_hash_lookup
c
c  Fraction in range 0.01 to 1
      f = min(max(0.01, fractn), 1.0)
c
      dtor = atan(45.)/45.
c
      istat = 0
c
c  Global parameters alpha0 & v
c  Allow for linked parameters
      ir = itdspt(irun)
      v = partds(1,ir)
      alpha0 = partds(2,ir)
      alpha1 = partds(3,ir)
      alpha  = alpha0 + alpha1 * stovl
c 
c  Internal batch number within run, jserbt
      jser = ccp4_hash_lookup(ibatch)
      jserbt = isrrbt(jser)
      jserb2 = jserbt + nmpr - 1
      if (jserb2 .gt. nbatrn(irun)) then
         istat = -1
         return
      endif
c
c  Phi1 = start of first image
c  Phi2 = end of last image
      phi1 = philim(1,jserbt,irun)
      phi2 = philim(2,jserb2,irun)
      if (phi1 .gt. phi2) then
c  force phi1 < phi2 for test
         temp = phi1
         phi1 = phi2
         phi2 = temp
      endif
c
c  u1 = distance in reciprocal space between spot & its position at
c start of rotation = xsi * (phi - phi1)
      u1 = xsi * (phi - phi1) * dtor
c  u2 = distance in reciprocal space between spot & its position at
c end of rotation = xsi * (phi2 - phi)
      u2 = xsi * (phi2 - phi) * dtor
c
c c1 & c2 are the missing areas/(height*v) at start & end
c dc1dv & dc2dv are their derivatives w.r.t. v
      if (abs(u1) .gt. v) then
         c1 = 0.0
         dc1dv = 0.0
      elseif (u1 .ge. 0.0) then
         c1 = 0.5 * ((v - u1)/v)**2
         dc1dv = u1 * (v - u1) / (v * v**2)
      else
c  u1 .lt. 0 
         c1 = 0.5 * ( 2.0 - ((v + u1)/v)**2)
         dc1dv = - u1 * (v + u1) / (v * v**2)
      endif
      if (abs(u2) .gt. v) then
         c2 = 0.0
         dc2dv = 0.0
      elseif (u2 .ge. 0.0) then
         c2 = 0.5 * ((v - u2)/v)**2
         dc2dv = u2 * (v - u2) / (v * v**2)
      else
c  u2 .lt. 0 
         c2 = 0.5 * ( 2.0 - ((v + u2)/v)**2)
         dc2dv = - u2 * (v + u2) / (v * v**2)
      endif
c
      vbb = 1 - c1 - c2
      t = alpha * vbb
c  Scale contribution
      tfp1 = 1.0 + t / f
      afp1 = 1.0 + alpha / f
      Dfhl = tfp1 / afp1
c
c  d/dalpha0
      dbdal0 = vbb - Dfhl
c  d/dalpha1
      dbdal1 = stovl * (vbb - Dfhl)
c  d/dv
      dbdv   = alpha * ( - dc1dv - dc2dv)
c  Put dg/dp in derivative vector, first v then alpha0, alpha1
      k = ipntds(irun)
      dghldc(k)   = dbdv / (f * afp1)
      dghldc(k+1) = dbdal0 / (f * afp1)
      dghldc(k+2) = dbdal1 / (f * afp1)
c Fix if required
      do 10, i = 1, nrntds(irun)
         if (lptfix(i)) then
            dghldc(k+i-1) = 0.0
         endif
 10   continue
c  Update scale
      ghl = ghl * Dfhl
c
c  Update derivatives
      do 20, i = 1, nprrun(irun) - nrntds(irun)
         dghldc(i) = dghldc(i) * Dfhl
 20   continue
      return
      end
      subroutine difcdn(lxyflg, ihkl, isym, irun, ibatch, phi,
     $   xsi, xdn, ydn, zdn, jserbt)
c     ===============================================
c
c  Determine diffracted beam or diffraction vector directions
c
c On entry:
c  lxyflg      = 0 no x,y coordinates calculated
c              = +/- 1 secondary, calculate polar angles of secondary beam
c                +1 diffractometer frame
c                -1 crystal frame
c              = +2 anisotropic B
c                  xy = projection of scattering vector perpendicular to beam,
c                 = y,z components of scattering vector
c              = +3 scattering vector xdn, ydn, zdn in diffractometer frame

c  ihkl(3)        h, k, l, reduced indices
c  isym           symmetry number (to get original indices)
c  irun           run number
c  ibatch         batch number
c  phi            rotation angle
c  jserbt         batch number within run (for BATCH mode)
c
c On exit:
c  xsi            radius from rotation axis (=sqrt(x**2 + y**2)
c  xdn, ydn       (1)  polar angles of secondary beam in diffractometer
c                      frame at zero phi angle (ie crystal-fixed) or
c                      in crystal frame
c                       x = theta (radians), y = phi (radians)
c     OR
c                 (2) y, z components of 1/2 scattering vector for reflection
c                         in diffraction position
c  zdn            normalized rotation angle
c
c     OR
c  xdn,ydn,zdn    (3) scattering vector xdn, ydn, zdn in diffractometer frame
c                     in 1/A units (ie divided by wavelength)
c
      implicit none
      integer lxyflg, ihkl(3), isym, irun, ibatch, jserbt
      real  phi, xsi, xdn, ydn, zdn
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
c
       real    P(3,3), dtor, hkl(3), xyz(3), s2(3), rxy,
     $   Pinv(3,3), s2z(3), UQ(3,3), UQinv(3,3), det,
     $   sphi0(3), s0phi0(3), tmat(3,3), t(3), x(3), tmat2(3,3)
      integer jser, jhkl(3), kmat, i, j
c
      integer ccp4_hash_lookup
      external ccp4_hash_lookup, matmul
c
      dtor = atan(1.)/45.
c
      jser = ccp4_hash_lookup(ibatch)
c Pointer to UB matrix
      kmat = jmtxbt(jserbt,irun)
c
c Phi matrix [P] and its inverse
      call rtzmat(phi*dtor, P)
      call rtzmat(-phi*dtor, Pinv)
c Original indices jhkl
      call asuget(ihkl, jhkl, isym)
      hkl(1) = jhkl(1)
      hkl(2) = jhkl(2)
      hkl(3) = jhkl(3)
c s(phi=0) = [D][UB] h
c Cases:
c  single axis (phi)  [D] = [Phi0]
c  3-axis:
c    omega scan   [D] = [E1-0][E2][E3] =
c                       [Omega0][Kappa][Phi] or [Omega0][Chi][Phi]
c    phi scan     [D] = [Phi0]
      call matvec(sphi0, dubmtxs(1,1,kmat), hkl)
      if (iabs(lxyflg) .eq. 1) then
c  Beam directions
         if (phi3scans(kmat) .ne. 0) then
c  3-axis phi scan, use [E1][E2] 
c s0(phi=0) = [Phi]^-1[E1E2]^-1 s0
            if (phi3scans(kmat) .lt. 0) then
c   negative e3 rotation
               call matmul(tmat, P, e1e2invmatxs(1,1,kmat))
               call ccpmvr(P, Pinv, 9)
               call ccpmvr(Pinv, tmat, 9)
            else 
               call matmul(tmat, Pinv, e1e2invmatxs(1,1,kmat))
               call ccpmvr(Pinv, tmat, 9)
            endif
c    else scan axis is outer rotation (1-axis or omega scan)
         else
            call matvec(x, P, sphi0)
         endif
c  s0(phi=0) = [Phi]^-1 s0
         call matvec(s0phi0, Pinv, s0xs(1,kmat))
c s2 = s' = s - s0 (in camera phi=0 frame)
         do i = 1,3
            s2(i) = sphi0(i) - s0phi0(i)
         enddo
c
         call ccpmvr(s2z, s2, 3)
         if (lxyflg .lt. 0) then
c Crystal frame
c Rotate back to permuted crystal frame
c s2(crystal) =  [DUQ]**-1 s2(phi=0)
            call matvec(s2z, DUQinv(1,1,irun), s2)
         endif            
c  z is polar axis
         rxy = sqrt(s2z(1)**2 + s2z(2)**2)
c   theta in range 0 -> pi
         xdn = atan2(rxy, s2z(3))
c   phi
         if (rxy .ne. 0.0) then
            ydn = atan2(s2z(2), s2z(1))
         else
            ydn = 0.0
         endif
      elseif (lxyflg .eq. +2) then
c  x = [Phi] s(phi=0)
         call matvec(xyz, P, sphi0)
         xdn = xyz(2)*0.5
         ydn = xyz(3)*0.5
      elseif (lxyflg .eq. +3) then
c  x = [Phi] s(phi=0)
         call matvec(xyz, P, sphi0)
         xdn = xyz(1)/wvlnxs(kmat)
         ydn = xyz(2)/wvlnxs(kmat)
         zdn = xyz(3)/wvlnxs(kmat)
         xsi = 0.0
         return
      endif
c
c Cylindrical radius from rotation axis
      xsi = sqrt(sphi0(1)**2 + sphi0(2)**2)
c
      if (mbatsc(irun) .ne. 0) then
         if (lbtslp(irun)) then
c  Sloped batch scale, store relative phi (0-1 within rotation range)
c     packed with batch number within run
            zdn = btslpf*jserbt +
     $           (phi - philim(1,jserbt,irun))/phirng(jserbt,irun)
         else
c  Plain batch scale
            zdn = jserbt
         endif
      else
         zdn = (phi - zlimrn(1,irun)) / spcscl(3,irun)
      endif
c
c      write (13, 6001) ibatch, zdn, ydn, xdn
c 6001 format (i6,5f10.3)

c
      return
      end
      subroutine dltexp(nobs, irank, prob, x)
c     ======================================
c
c     Calculate Delta(expected)
c
c On entry:
c  nobs         number of observations
c  irank        rank of this observation
c  prob(512)    table of p(x)
c
c On exit:
c  x            expected value
c
      integer nobs, irank
      real prob(512), x
c
      integer i, k, id
      real px, del
c
      px = abs(float(nobs - 2 * irank + 1) / float(nobs))
c
c     interpolate from probability table
c
      i = 256
      id =i / 2
c
c     start of binary search loop
c
      do 200 k=1,8
         del = px - prob(i)
         if (del .gt. 0.0) go to 290
         i = i - id
         go to 295
 290     i = i + id
 295     id = id / 2
 200  continue
      if (px .lt. prob(i)) i = i - 1
c
      del = (px - prob(i)) / (prob(i+1) - prob(i))
      x   = 0.7827789e-02 * (float(i-1) + del)
      if (irank .lt. nobs/2) x = -x
      return
      end
c
c
c
      subroutine dtsclrcc
c     ===================
c
c  Clear all sums for correlation coefficients between datasets
c  Set numbers of correlations ncca, nccd
c
      include 'parameter.fh'
      include 'datasets.fh'
      include 'crlncf.fh'
c
      integer i, j
c
      if (ndtsetsout .le. 0) then
         ncca = 0
         nccd = 0
         return
      endif
c Number of anomalous correlations
      ncca = ndtsetsout*(ndtsetsout - 1)/2
c Number of isomorphous|dispersive correlations
c Differences are referred to the base set,
c so number of differences = no. of datasets - 1
      nccd = ndtsetsout*(ndtsetsout - 1)/2
c
      do j = 1, maxcca
         do i = 1, maxres
            sumcca_x(i, j) = 0.0
            sumcca_y(i, j) = 0.0
            sumcca_x2(i, j) = 0.0
            sumcca_y2(i, j) = 0.0
            sumcca_xy(i, j) = 0.0
            nsumcca(i, j) = 0
         enddo
      enddo
      do j = 1, maxccd
         do i = 1, maxres
            sumccd_x(i, j) = 0.0
            sumccd_y(i, j) = 0.0
            sumccd_x2(i, j) = 0.0
            sumccd_y2(i, j) = 0.0
            sumccd_xy(i, j) = 0.0
            nsumccd(i, j) = 0
         enddo
      enddo
      do j = 1, maxsets
         do i = 1, maxres
            sumdiso(i, j)   = 0.0
            sumdiso2(i, j)  = 0.0
            nsumdiso(i, j)  = 0.0
            sumdanm(i, j)   = 0.0
            sumdanm2(i, j)  = 0.0
            nsumdanm(i, j)  = 0.0
         enddo
      enddo
      return
      end
c
      subroutine dtsadcc(ndts, mbn_s,
     $         danomdts, sddanomdts, disodts, sddisodts)
c     ==================
c
c Add contributions into sums for correlation coefficients
c
c On entry:
c  ndts          number of datasets (= ndtsetsout)
c  mbn_s            resolution bin number
c  danomdts(ndts)    anomalous differences for each dataset
c  sddanomdts(ndts) sd(anomalous differences) for each dataset
c  disodts(ndts)    isomorphous|dispersive difference from base dataset
c  sddisodts(ndts)  isomorphous|dispersive difference from base dataset
c
      integer ndts, mbn_s
      real danomdts(ndts), sddanomdts(ndts),
     $   disodts(ndts), sddisodts(ndts)
c
      include 'parameter.fh'
      include 'crlncf.fh'
      include 'datasets.fh'
c
      integer i, j, k
c
c Anomalous differences
c  Sums for correlation between D(i) and D(j) for datasets i,j = 1,ndts
c  (j .gt. i)
      k = 0
      do j = 1, ndts-1
         do i = j+1, ndts
            k = k+1
            if (sddanomdts(i) .gt. 0.0 .and.
     $         sddanomdts(j) .gt. 0.0) then
               nsumcca(mbn_s, k) = nsumcca(mbn_s, k) + 1
               sumcca_x(mbn_s, k) =
     $            sumcca_x(mbn_s, k) + danomdts(i)
               sumcca_y(mbn_s, k) =
     $            sumcca_y(mbn_s, k) + danomdts(j)
               sumcca_x2(mbn_s, k) =
     $            sumcca_x2(mbn_s, k) + danomdts(i)**2
               sumcca_y2(mbn_s, k) =
     $            sumcca_y2(mbn_s, k) + danomdts(j)**2
               sumcca_xy(mbn_s, k) =
     $            sumcca_xy(mbn_s, k) + danomdts(i) * danomdts(j)
            endif
         enddo
      enddo
      do i = 1, ndts
         if (sddanomdts(i) .gt. 0.0) then
            sumdanm(mbn_s, i)   = sumdanm(mbn_s, i)   +  danomdts(i)
            sumdanm2(mbn_s, i)  = sumdanm2(mbn_s, i)  +  danomdts(i)**2
            nsumdanm(mbn_s, i)  = nsumdanm(mbn_s, i)  +  1
         endif
      enddo
c
c Dispersive differences
c  Sums for correlation between D(i) and D(j) for datasets i,j = 1,ndts
c  (j .gt. i)
      k = 0
      do j = 1, ndts-1
         do i = j+1, ndts
            k = k+1
            if (sddisodts(i) .gt. 0.0 .and.
     $         sddisodts(j) .gt. 0.0) then
               nsumccd(mbn_s, k) = nsumccd(mbn_s, k) + 1
               sumccd_x(mbn_s, k) =
     $            sumccd_x(mbn_s, k) + disodts(i)
               sumccd_y(mbn_s, k) =
     $            sumccd_y(mbn_s, k) + disodts(j)
               sumccd_x2(mbn_s, k) =
     $            sumccd_x2(mbn_s, k) + disodts(i)**2
               sumccd_y2(mbn_s, k) =
     $            sumccd_y2(mbn_s, k) + disodts(j)**2
               sumccd_xy(mbn_s, k) =
     $            sumccd_xy(mbn_s, k) + disodts(i) * disodts(j)
               
            endif
         enddo
      enddo
      do i = 1, ndts
         if (sddisodts(i) .gt. 0.0) then
            sumdiso(mbn_s, i)   = sumdiso(mbn_s, i)   +  disodts(i)
            sumdiso2(mbn_s, i)  = sumdiso2(mbn_s, i)  +  disodts(i)**2
            nsumdiso(mbn_s, i)  = nsumdiso(mbn_s, i)  +  1
         endif
      enddo
c
      end
c
c
c
      subroutine dtsprcc(nsrmax, reso)
c     ===============================
c
c Print correlation tables between datasets
c
c On entry:
c  nsrmax     maximum resolution bin
c  reso       resolution limits for printing
c
      integer nsrmax
      real reso(nsrmax)
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'crlncf.fh'
      include 'datasets.fh'
c
      integer i, j, k, l, m, nsum, misod(maxccd), nisod, ndif, k2
      logical lisod(maxccd)
      real shbb
      double precision
     $   sum_x, sum_y,
     $   sum_x2, sum_y2,
     $   sum_xy, denom, sumn
c
      character line*500, line2*500, linlab*50,
     $   lab_i(maxcca)*16, lab_j(maxcca)*16,
     $   ldts(maxcca)*16, mark*8, brknum*8, cnumber*20,
     $   labdts(maxsets)*8, ctc(maxcca)*8, cns(maxcca)*8
      integer lenstr
      external lenstr
c
      write (lunout, '(/a,a/a,a/)')
     $   'Correlation coefficients for anomalous & dispersive ',
     $   'differences between different datasets',
     $   '====================================================',
     $   '======================================'
c
      write (lunout, '(/a/)') 'Datasets:-'
      k = 0
      linlab = ' '
      do j = 1, ndtsetsout
         call mkdtslab(j, line)
         if (j .eq. kdtsbase) then
            mark = 'base'
         else
            mark = ' '
         endif 
         write (lunout, '(i5,1x,a4,4x,a)')
     $      j, mark, line(1:lenstr(line))
         k = k+1
         labdts(k) = dname_out(j)
C  Construct a line for Graph label - maximum 50 characters
         l = lenstr(linlab)
         if (l .le. 45 .and. mark .eq. ' ') then
            linlab = linlab(1:l)//' '//labdts(k)(1:4)
         elseif (l .le. 39 .and. mark .eq. 'base') then
            linlab = linlab(1:l)//' '//labdts(k)(1:4)//'(base)'
         endif
      enddo
      write (lunout, '(/a//)')
     $   '====== Correlation of anomalous differences'
c
      k = 0
      do j = 1, ndtsetsout-1
         do i = j+1, ndtsetsout
            k = k+1
            sum_x  = 0.0
            sum_y  = 0.0
            sum_x2 = 0.0
            sum_y2 = 0.0
            sum_xy = 0.0
            nsum = 0
            lab_i(k) = dname_out(i)
            lab_j(k) = dname_out(j)
            write (line, '(i5,a,i5)') j, '-', i
            call squeez(line, ldts(k))
            do l = 1, nsrmax
               ccanom(l,k) = 0.0
               if (nsumcca(l, k) .gt. 0) then
                  sumn = nsumcca(l, k)
                  denom =
     $               (sumcca_x2(l,k) - sumcca_x(l,k)**2/sumn)*
     $               (sumcca_y2(l,k) - sumcca_y(l,k)**2/sumn)
                  if (denom .gt. 0.000001) then
                     ccanom(l,k) = (sumcca_xy(l,k) -
     $                  sumcca_x(l,k)*sumcca_y(l,k)/sumn)
     $                  /dsqrt(denom)
                  endif
                  sum_x  = sum_x  + sumcca_x(l,k)
                  sum_y  = sum_y  + sumcca_y(l,k)
                  sum_x2  = sum_x2  + sumcca_x2(l,k)
                  sum_y2  = sum_y2  + sumcca_y2(l,k)
                  sum_xy  = sum_xy  + sumcca_xy(l,k)
                  nsum = nsum + nsumcca(l, k)
               endif
            enddo
            ccanomov(k) = 0.0
            nsanomov(k) = nsum
            if (nsum .gt. 0) then
               sumn = nsum
               denom = (sum_x2 - sum_x**2/sumn)*
     $            (sum_y2 - sum_y**2/sumn)
               if (denom .gt. 0.000001) then
                  ccanomov(k) = (sum_xy -
     $               sum_x*sum_y/sumn)/dsqrt(denom)
               endif
            endif
         enddo 
      enddo
c
c Write out as table against resolution
c
      call htm_stgraph
      write (lunout, 6001) 
 6001 format (/
     $'$TABLE: Correlation of Anomalous Differences between datasets:')
      write (lunout, 6002)
     $   linlab(1:lenstr(linlab))
 6002 format (
     $  '$GRAPHS: Anom CCs v resln - ',a)

      line = ' :N:2,'
      do  i = 1, 2*ncca, 2
         write (line2, 6003) i+3
 6003    format(i3,',')
         l = lenstr(line)
         m = lenstr(line2)
         line = line(1:l)//line2(1:m)
         if (i .lt. 2*ncca-1 .and.  l .gt. 110) then
            l = lenstr(line)
            write (lunout, '(a)') line(1:l)
            line = ''
         endif
      enddo
      l = lenstr(line)
c     Overwrite last comma
      line(l:) = ': $$'
      l = lenstr(line)
      write (lunout, '(a/)') line(1:l)
c
      write (lunout, '(a,4x,14a8,100(/21x,14a8))')
     $   ' N 1/resol^2 dmax',('CC'//ldts(k), 'N'//ldts(k), k = 1,ncca)
      write (lunout, '(a,/,a,100(9x,6(a16)))') ' $$',
     $   '1st dataset',(lab_j(k), k = 1, ncca)
      write (lunout, '((a,100(9x,6(a16)/)))')
     $   '2nd dataset',(lab_i(k), k = 1, ncca)
      write (lunout, '(a)') ' $$'
c
      do l = 1,nsrmax
         shbb = 1./reso(l)**2
         do k = 1, ncca
            call tformat(ccanom(l,k), ctc(k), 8,3)
            call iformat(nsumcca(l,k), cns(k), 8)
         enddo
         write (lunout, '(i3,f8.4,f6.2,6(a8,a8),:
     $      ,100(/17x,6(a8,a8)))')
     $      l, shbb, reso(l), (ctc(k), cns(k), k = 1, ncca)
      enddo
      write (lunout, '(a)') ' $$'
      write (lunout, '(a,9x, 6(f8.3,i8),:,
     $   100(/17x,6(f8.3,i8)))')
     $   ' Overall', (ccanomov(k), nsanomov(k), k = 1, ncca)
      write (lunout, '(/)')
      call htm_endgraph
c
c Print cross correlation table
      write (lunout, 6010)
 6010 format (/'Correlation of Anomalous Differences between datasets'/
     $   6x,'(Numbers in brackets)'/)
      write (lunout, '(13x,100a8)')
     $   (labdts(i), i = 2, ndtsetsout)
      mark = '    *   '
      write (lunout, 6011) (mark, i = 2, ndtsetsout)
 6011 format (10x,100a)
      k  = 0
      k2 = 0
      do j = 1, ndtsetsout-1
         k2 = k2 + 1
         line = labdts(k2)(1:8)
         line(10:10) = '*'
         l = k2*8 + 2
         l = l+1
         line2 = ' '
         do i = j+1, ndtsetsout
            k = k+1
            write (line(l:), '(f8.3)') ccanomov(k)
            write (cnumber, '(a,i8,a)') '(',nsanomov(k),')'
            call squeez(cnumber, mark)
            call rjstfy(brknum, mark)
            line2(l+1:) = brknum
            l = l+8
         enddo
         write (lunout, '(a/a)') line(1:l), line2(1:l)
      enddo
      write (lunout, '(/)')
c
c========================================================================
c
c Dispersive differences
c
c Make names of differences, store in common block for later use
      do i = 1, ndtsetsout
         if (i .eq. kdtsbase) then
            disod_name(i) = 'base'
         else
            line2 = dname_out(i)//'-'//dname_out(kdtsbase)
            call squeez(line2, disod_name(i))
         endif
      enddo
c
      if (ndtsetsout .le. 2) return
c
      call mkdtslab(kdtsbase, line)
      write (lunout, '(a//a,a/)')
     $   '====== Correlation of dispersive (isomorphous) differences',
     $ 'All dispersive differences are relative to the "base" dataset ',
     $   line(1:lenstr(line))      
c
c
      k = 0
      m = 0
      do j = 1, ndtsetsout-1
         do i = j+1, ndtsetsout
            k = k+1
            if (i .ne. kdtsbase .and. j .ne. kdtsbase) then
               m = m+1
               misod(m) = k
c
               sum_x = 0.0
               sum_y = 0.0
               sum_x2 = 0.0
               sum_y2 = 0.0
               sum_xy = 0.0
               nsum = 0
               lab_i(k) = disod_name(i)
               lab_j(k) = disod_name(j)
c               write (line, '(i5,a,i5)') j, '-', i
c               call squeez(line, ldts(k))
               write (line, '(i4,i4,a,i4,i4,a)')
     $            j, kdtsbase, '-', i, kdtsbase
               call squeez(line, ldts(k))
               do l = 1, nsrmax
                  ccisod(l,k) = 0.0
                  if (nsumccd(l, k) .gt. 0) then
                     sumn = nsumccd(l, k)
                     denom =
     $                  (sumccd_x2(l,k) - sumccd_x(l,k)**2/sumn)*
     $                  (sumccd_y2(l,k) - sumccd_y(l,k)**2/sumn)
                     if (denom .gt. 0.000001) then
                        ccisod(l,k) = (sumccd_xy(l,k) -
     $                     sumccd_x(l,k)*sumccd_y(l,k)/sumn)/
     $                     dsqrt(denom)
                     endif
                     sum_x = sum_x + sumccd_x(l,k)
                     sum_y = sum_y + sumccd_y(l,k)
                     sum_x2 = sum_x2 + sumccd_x2(l,k)
                     sum_y2 = sum_y2 + sumccd_y2(l,k)
                     sum_xy = sum_xy + sumccd_xy(l,k)
                     nsum = nsum + nsumccd(l, k)
                  endif
               enddo
               ccisodov(k) = 0.0
               nsisodov(k) = nsum
               if (nsum .gt. 0) then
                  sumn = nsum
                  denom = (sum_x2 - sum_x**2/sumn)*
     $               (sum_y2 - sum_y**2/sumn)
                  if (denom .gt. 0.000001) then
                     ccisodov(k) = (sum_xy -
     $                  sum_x*sum_y/sumn)/dsqrt(denom)
                  endif
               endif
               lisod(k) = .true.
            else
c  One of datasets is the base dataset itself
               ccisodov(k) = 0.0
               lisod(k) = .false.
            endif
         enddo 
      enddo
      nisod = m
c
c Write out as table against resolution
c
      call htm_stgraph
      write (lunout, 6021) 
 6021 format (/
     $'$TABLE: Correlation of Dispersive Differences between datasets:')

      write (lunout, 6022)
     $   linlab(1:lenstr(linlab))
cccc , (i+3, i = 1, 2*nisod, 2)
 6022 format (
     $   '$GRAPHS: Dispersive CCs v resln - ',a)
c
      line = ' :N:2,'
      do  i = 1, 2*nisod, 2
         write (line2, 6003) i+3
         l = lenstr(line)
         m = lenstr(line2)
         line = line(1:l)//line2(1:m)
         if (i .lt. 2*nisod-1 .and.  l .gt. 110) then
            l = lenstr(line)
            write (lunout, '(a)') line(1:l)
            line = ''
         endif
      enddo
      l = lenstr(line)
c     Overwrite last comma
      line(l:) = ': $$'
      l = lenstr(line)
      write (lunout, '(a/)') line(1:l)
c
      write (lunout, '(a,4x,14(a8,'' ''),100(/21x,14(a8,'' '')))')
     $   ' N 1/resol^2 dmax',
     $   ('CC'//ldts(misod(m)), 'N'//ldts(misod(m)), m = 1,nisod)
      write (lunout, '(a,/a,100(6x,6(a16,'' '')))') ' $$',
     $   '1st difference',(lab_j(misod(m)), m = 1, nisod)
      write (lunout, '((a,100(6x,6(a16,'' '')/)))')
     $   '2nd difference',(lab_i(misod(m)), m = 1, nisod)
      write (lunout, '(a)') ' $$'
c
      do l = 1,nsrmax
         shbb = 1./reso(l)**2
         do m = 1, nisod
            call tformat(ccisod(l, misod(m)), ctc(m), 8,3)
            call iformat(nsumccd(l,misod(m)), cns(m), 8)
         enddo
         write (lunout, '(i3,f8.4,f6.2,6(a8,a8),:,
     $      100(/17x,6(a8,a8)))')
     $      l, shbb, reso(l),
     $      (ctc(m), cns(m), m = 1, nisod)
      enddo
      write (lunout, '(a)') ' $$'
      write (lunout, '(a,9x, 6(f8.3,i8),:,
     $   100(/17x,6(f8.3,i8)))')
     $   ' Overall',
     $   (ccisodov(misod(m)), nsisodov(misod(m)), m = 1, nisod)
      write (lunout, '(/)')
      call htm_endgraph
c
c Print cross correlation table
c  List of datasets which are not the base dataset
      j = 0
      do i = 1, ndtsetsout
         if (i .ne. kdtsbase) then
            j = j+1
            ldts(j) = disod_name(i)
         endif
      enddo
      ndif = j
      write (lunout, 6030)
 6030 format (/
     $   'Correlation between datasets of Dispersive Differences ',
     $   'from base set '/
     $   6x,'(Numbers in brackets)'/)
      write (lunout, '(21x,100a16)')
     $   (ldts(j), j = 2, ndif)
      mark = '    *   '
      write (lunout, 6031) (mark, i = 2, ndif)
 6031 format (18x,100(a8,8x))
      m = 0
      do j = 1, ndif-1
         line = ldts(j)(1:16)
         line(18:18) = '*'
         l = j*16 + 2
         l = l+1
         line2 = ' '
         do i = j+1, ndif
            m = m+1
            write (line(l:), '(f8.3,8x)') ccisodov(misod(m))
            write (cnumber, '(a,i8,a)') '(',nsisodov(misod(m)),')'
            call squeez(cnumber, mark)
            call rjstfy(brknum, mark)
            line2(l+1:) = brknum
            l = l+16
         enddo
         write (lunout, '(a/a)') line(1:l), line2(1:l)
      enddo
      write (lunout, '(/)')

      end
c
c
c
      subroutine rjstfy(a,b)
c     ======================
c
c Right justify string b into a
c
      character*(*) a,b
      integer i,j,la,lb,ma,lenstr
      external lenstr
c
      la=len(a)
      lb=lenstr(b)
      a = ' '
c
      ma = max(1,la-lb+1)
      j = lb
      do 1, i=la,ma,-1
         a(i:i) = b(j:j)
         j = j-1
 1    continue
      return
      end
c
c
c
      subroutine dtsbin(nfl, deliovf, idts, ndts)
c     ==========================================
c
c  Analyse list of (I+ - I-)/<F>
c
c  this is test stuff!
c
c
c On entry:
c  nfl      number of entries
c  deliovf  (I+ - I-)/<F>
c  idts     dataset number
c  ndts     number of datasets
c
c
      integer nfl, idts(nfl), ndts
      real deliovf(nfl)
c
      include 'parameter.fh'
      include 'inout.fh'
c
      integer maxbin
      parameter (maxbin = 25)
c
      integer i, l, n, nbin
c
      real wid, av, sd, bin 
      integer histo(-maxbin:maxbin, maxsets)
      real deli(maxsets), f(maxsets), wd(maxsets)
      integer nd
c
c Bin width in sd multiples
      data wid /0.25/
c
c Don't do anything
      return

c Get overall mean & sd
      call avgsd(nfl, deliovf, idts, 0, av, sd, n)
      if (n .eq. 0) then
         return
      endif
c      write (lunout, '(/a,2f10.3)')
c     $   ' Distribution of DelI/F: overall mean & sd = ', av, sd
      do l = 1, ndts
         call avgsd(nfl, deliovf, idts, l, av, sd, n)
c         write (lunout, '(/a,i4,a,2f10.3)')
c     $   ' For dataset ',l,'  mean & sd = ', av, sd
      enddo
c
c Work out binning for histogram
c
      nbin = 2*maxbin + 1
      call ccpzi(histo, nbin*maxsets)
      bin = sd * wid
c      call ccpdpn(13, 'HISTO', 'NEW', 'F', 0,0)
c
c      call dtsinitget
c 10   call dtsgetdeli(deli, f, wd, nd)
c      if (nd .lt. 0) go to 20
c      do l = 1, nd
c         if (wd(l) .gt. 0.0) then
c            if (f(l) .gt. 0.00001) then
c               write (13, '(2f10.2, 2f10.4)')
c     $            f(l), deli(l), deli(l)/f(l), deli(l)/(f(l)**2)
c            endif 
c         endif
c      enddo 
c      go to 10
c 20   call dtsinitget
c
      do i = 1, nfl
         if (deliovf(i) .ne. 0.0) then
            n = nint(deliovf(i) / bin)
            n = min(maxbin, max(-maxbin, n))
            histo(n, idts(i)) = histo(n, idts(i)) + 1
            
         endif
      enddo
c
c      do n = -maxbin, maxbin
c         write (13, '(20i6)') n, (histo(n,i), i = 1,  ndts)
c      enddo
c      close (unit=13)
      return
      end
c
c
c
      subroutine avgsd(n, y, ifl, lfl, av, sd, nn)
c     ===========================================
c
c  Work out mean and standard devaiation of list of numbers
c
c On entry:
c  n      number of entries
c  y(n)   list of numbers
c  ifl(n) flag array
c  lfl    flag to match ifl to include, = 0 means include all
c
c On exit:
c  av     average
c  sd     standard deviation
c  nn     number of data points
c
      integer n, ifl(n), lfl, nn
      real    y(n), av, sd
c
c
      integer i, k
c
      double precision sum_x, sum_x2, sn
c
      av = 0.0
      sd = 0.0
      nn = 0
      if (n .eq. 0) then
         return
      endif
c Get overall mean & sd
      sum_x = 0.0
      sum_x2 = 0.0
c
      k = 0
      do i = 1, n
         if (lfl .eq. 0 .or. ifl(i) .eq. lfl) then
            k = k+1
            sum_x  = sum_x  + y(i)
            sum_x2 = sum_x2 + y(i)**2
         endif
      enddo
      if (k .gt. 0) then
         sn = k
         av = sum_x/sn
         sd = sum_x2/sn - av**2
         if (sd .gt. 0.0) then
            sd = sqrt(sd)
         else
            sd = 0.0
         endif
      endif
      nn = k
      return
      end
c
c
c
      subroutine dtsdif(nrs, icntrc, jhkl, ffrf, sdffrf, wtrf, ghl,
     $   irunrf,  mrf, isttrf, isymrf, ndts,
     $   danomdts, sddanomdts, disodts, sddisodts, fdts,
     $   qihdts, qsdidts, qihpl, qsdipl, qihmn, qsdimn)
c     =============================================================
c
c
c  Calculate anomalous differences (I+ - I-) for each dataset
c  and isomorphous|dispersive differences relative to base dataset
c
c
c On entry:
c  nrs           number of observations
c  icntrc        centric reflection if icntrc = 1, else = 0
c  jhkl(3)       reflection indices
c  ffrf(nrs)     intensity 
c  sdffrf(nrs)   sd(intensity)
c  wtrf          weight for this observation, eg 1/sd(I)**2
c                  (note that this is unscaled, so needs to be
c                   multiplied by ghl**2)
c  ghl(nrs)      inverse scale
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isttrf(nrs)   status flag
c  isymrf(nrs)   symmetry number
c  ndts          number of datasets (= ndtsetsout)
c
c On exit:
c  danomdts(ndts)    anomalous differences (unweighted) for each dataset
c  sddanomdts(ndts) sd(anomalous differences) for each dataset
c  disodts(ndts)    isomorphous|dispersive difference from base dataset
c                      (including the base dataset itself)
c                   These differences are calculated from the
c                    unweighted means of I+ & I-
c  sddisodts(ndts)  isomorphous|dispersive difference from base dataset
c  fdts(ndts)       <F> = sqrt(<I>) (= 0.0 if  I .lt. 0)
c  qihdts(ndts), qsdidts(ndts)      Imean, sd(Imean) for each dataset
c  qihpl(ndts), qsdipl(ndts)        I+, sd(I+) for each dataset (= 0 for centric)
c  qihmn(ndts), qsdimn(ndts)        I-, sd(I-) for each dataset (= 0 for centric)
c
      integer nrs, icntrc, ndts
      integer jhkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs), isymrf(nrs)
      real  ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs),
     $   danomdts(ndts), sddanomdts(ndts),
     $   disodts(ndts), sddisodts(ndts), fdts(ndts),
     $   qihdts(ndts), qsdidts(ndts),
     $   qihpl(ndts), qsdipl(ndts),
     $   qihmn(ndts), qsdimn(ndts)
c
      include 'parameter.fh'
      include 'datasets.fh'
      include 'rundef.fh'
c
      real uh, aihdts(maxsets), sdihdts(maxsets),
     $   aihpls, sdihpl, aihmns, sdihms
      integer idts, janom, nu(maxsets), nupls, numns
      logical useflg(maxrfl)
c
c Loop datasets
      do idts = 1, ndts
         danomdts(idts) = 0.0
         sddanomdts(idts) = 0.0
         disodts(idts)    = 0.0
         sddisodts(idts)  = 0.0
         fdts(idts)       = 0.0
         if (icntrc .eq. 0) then
c  Means of I+ & I- 
            janom = 1
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, idts, .false., useflg)
            call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $         wtrf, aihpls, sdihpl, uh, nupls)
            janom = 2
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, idts, .false., useflg)
            call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $         wtrf, aihmns, sdihms, uh, numns)
            qihpl(idts) = aihpls
            qsdipl(idts) = sdihpl
            qihmn(idts) = aihmns
            qsdimn(idts) = sdihms
         else
c  Centric
            nupls = 0
            numns = 0
            qihpl(idts) = 0.0
            qsdipl(idts) = 0.0
            qihmn(idts) = 0.0
            qsdimn(idts) = 0.0
         endif
c  Anomalous difference (unweighted) & its sd
         if (nupls .gt. 0 .and. numns .gt. 0) then
            danomdts(idts)   = aihpls - aihmns
            sddanomdts(idts) = sqrt(sdihpl**2 + sdihms**2)
         else
c   missing or centric
            danomdts(idts)   = 0.0
            sddanomdts(idts) = 0.0
         endif
c <I>, unweighted mean of I+ & I- for acentric case
c   Centric or acentric
         janom = 0
         call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $      irefrn, idts, .false., useflg)
         call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $      wtrf, aihdts(idts), sdihdts(idts), uh, nu(idts))

         qihdts(idts) = aihdts(idts)
         qsdidts(idts) = sdihdts(idts)
c
         if (icntrc .eq. 0) then
c   Acentric
            if (nupls .gt. 0 .and. numns .gt. 0) then
               aihdts(idts) = 0.5*(aihpls + aihmns)
               sdihdts(idts) = sqrt(sdihpl**2 + sdihms**2)
               nu(idts) = 2
            else
               nu(idts) = 0
            endif
         endif
         if (aihdts(idts) .gt. 0.0) then
            fdts(idts) = sqrt(aihdts(idts))
         else
            fdts(idts) = 0.0
         endif
c
      enddo
c
      if (ndts .gt. 0) then
         do idts = 1, ndts
            if (kdtsbase .gt. 0 .and. idts .ne. kdtsbase) then
               if (nu(idts) .gt. 0 .and. nu(kdtsbase) .gt. 0) then
                  disodts(idts)   = aihdts(idts) - aihdts(kdtsbase)
                  sddisodts(idts) =
     $               sqrt(sdihdts(idts)**2 + sdihdts(kdtsbase)**2)
               else
                  disodts(idts)   = 0.0
                  sddisodts(idts) = 0.0
               endif
            else
               disodts(idts)   = 0.0
               sddisodts(idts) = 0.0
            endif
         enddo
      endif
c
      return
      end
c
c
c
      subroutine dtsprdif(nsrmax, reso)
c     ================================
c
c Print rms anomalous & dispersive differences etc
c
c On entry:
c  nsrmax     maximum resolution bin
c  reso       resolution limits for printing
c
      integer nsrmax
      real reso(nsrmax)
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'crlncf.fh'
      include 'datasets.fh'
c
      integer i, j, k, l, m, nsum, ndts, notbase(maxsets), nda, ndd
      real shbb
      double precision
     $   sum_x, sum_x2, sumn
      character line*500, line2*500, linlab*45,
     $          ldts(maxsets)*8, cnum*8, labdts(maxsets)*16,
     $          ctr(maxsets)*8, ctd(maxsets)*8,
     $          cnr(maxsets)*8, cnd(maxsets)*8

      integer lenstr
      external lenstr
c
      nda = 0
      ndd = 0
      linlab = ' '
c
      do j = 1,ndtsetsout
         sum_x  = 0.0
         sum_x2 = 0.0
         nsum = 0
         do l = 1, nsrmax
            if (nsumdanm(l, j) .gt. 0) then
               sumn = nsumdanm(l, j)
               sum_x  = sum_x  + sumdanm(l,j)
               sum_x2  = sum_x2  + sumdanm2(l,j)
               nsum = nsum + nsumdanm(l, j)
               rms_anom(l,j) = sqrt(sumdanm2(l,j)/sumn)
            else
               rms_anom(l,j) = 0.0
            endif
         enddo
         if (nsum .gt. 0) then
            sumn = nsum
            rms_anom_ov(j) = sqrt(sum_x2/sumn)
         else
            rms_anom_ov(j) = 0.0
         endif
         nsumanom_ov(j) = nsum
         nda = nda + 1
         labdts(nda) = dname_out(j)
C  Construct a line for Graph label - maximum 50 characters
         l = lenstr(linlab)
         if (l .le. 45 .and. j .ne. kdtsbase) then
            linlab = linlab(1:l)//' '//labdts(nda)(1:4)
         elseif (l .le. 39 .and. j .eq. kdtsbase) then
            linlab = linlab(1:l)//' '//labdts(nda)(1:4)//'(base)'
         endif
      enddo
c Dispersive|isomorphous
      do j = 1,ndtsetsout
         sum_x  = 0.0
         sum_x2 = 0.0
         nsum = 0
         do l = 1, nsrmax
            if (nsumdiso(l, j) .gt. 0) then
               sumn = nsumdiso(l, j)
               sum_x  = sum_x  + sumdiso(l,j)
               sum_x2  = sum_x2  + sumdiso2(l,j)
               nsum = nsum + nsumdiso(l, j)
               rms_isod(l,j) = sqrt(sumdiso2(l,j)/sumn)
            else
               rms_isod(l,j) = 0.0
            endif
         enddo
         if (nsum .gt. 0) then
            sumn = nsum
            rms_isod_ov(j) = sqrt(sum_x2/sumn)
         else
            rms_isod_ov(j) = 0.0
         endif
         nsumdiso_ov(j) = nsum
         if (j .ne. kdtsbase) ndd = ndd + 1
      enddo
c
c Write out as table against resolution
c
      call htm_stgraph
      write (lunout, 6001) 
 6001 format (/
     $'$TABLE: RMS Anomalous & Dispersive Differences:')
      ndts = ndtsetsout
      write (line, 6002)
     $   linlab(1:lenstr(linlab)), (i+3, i = 1, 2*nda, 2)
 6002 format (
     $ '$GRAPHS: RMS DelAnom v resln - ',a,' :N:2,',
     $   40(i2,','))
      l = lenstr(line)
      line(l:l) = ':'
C
      write (line2, 6003)
     $   linlab(1:lenstr(linlab)), (i+3, i = 2*nda+1, 2*(ndd+nda), 2)
 6003 format (
     $ ' : RMS Dispersive diffs v resln - ',a,' :N:2,',
     $   40(i2,','))
      l = lenstr(line2)
c     Overwrite last comma
      line2(l:) = ': $$'
      l = lenstr(line)
      write (lunout, '(a,/,a,/)') line(1:l),line2(1:lenstr(line2))
c
      k = 0
      do j = 1, ndtsetsout
         write (cnum, '(i4)') j
         call squeez(cnum, ldts(j))
         if (j .ne. kdtsbase) then
            k = k+1
            notbase(k) = j
         endif
      enddo
      write (lunout, '(a,3x,14a8,(/21x,14a8))')
     $   ' N 1/resol^2 dmax',
     $ ('Anom'//ldts(j), ' NA'//ldts(j), j = 1,nda),
     $ ('Disp'//ldts(notbase(k)), ' ND'//ldts(notbase(k)), k = 1,ndd)
      write (lunout, '(a)') ' $$'
      write (lunout, '(a8,13x,8(a16),:,100(/21x,8(a16)))') 
     $   ' Dataset',(labdts(j), j=1,nda),
     $   (disod_name(notbase(k)), k=1,ndd)

      write (lunout, '(a)') ' $$'
c
      do l = 1,nsrmax
         shbb = 1./reso(l)**2
         do j = 1, nda
            call tformat(rms_anom(l,j), ctr(j), 8,1)
            call iformat(nsumdanm(l,j), cnr(j), 8)
         enddo
         do  k = 1, ndd
            call tformat(rms_isod(l,notbase(k)), ctd(k), 8,1)
            call iformat(nsumdiso(l,notbase(k)), cnd(k), 8)
         enddo
         write (lunout, '(i3,f8.4,f6.2,8(a8,a8),:,(/18x,8(a8,a8)))')
     $      l, shbb, reso(l),
     $      (ctr(j), cnr(j), j = 1, nda),
     $      (ctd(k), cnd(k), k = 1, ndd)
      enddo
      write (lunout, '(a)') ' $$'
      write (lunout, '(a,9x, 8(f8.1,i8),:,(/18x,8(f8.1,i8)))')
     $   ' Overall', (rms_anom_ov(j), nsumanom_ov(j),
     $   j = 1, nda),
     $   (rms_isod_ov(notbase(k)),  nsumdiso_ov(notbase(k)),
     $   k = 1, ndd)
      write (lunout, '(/)')
      call htm_endgraph
c
      write (lunout, '(/)')
      end
c
c
c
      subroutine dtsscale
c     ===================
c
c  Scale anomalous & dispersive differences across datasets,
c for analysis of ratios of f1 and Delf'
c
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'datasets.fh'
      include 'crlncf.fh'
c
c Storage space for LSQ matrix etc
      integer maxmtxd
      parameter (maxmtxd = maxsets)
      double precision amatrxd(maxmtxd,maxmtxd),
     $   eigvald(maxmtxd), eigvecd(maxmtxd,maxmtxd), vecinvd(maxmtxd),
     $   workd(5*maxmtxd)
c      
c
      integer i, k, iparnorm, jparnorm, npar, nobspar(maxsets)
      real sclinv(maxsets), rmsmax, rmsratio(maxsets)
      logical ldata
      character line*80
c
      integer lenstr
      external lenstr

c Data retrieval routines
      external dtsgetanom, dtsgetisod
c
c Initialise retrieval of difference data
      call dtsinitget
c
c (a) anomalous differences
c  Dataset to set scale = 1.0 for
      iparnorm = kdtsbase
      npar = ndtsetsout
      call FoxHolmes(dtsgetanom, npar, iparnorm, sclinv, nobspar,
     $   maxmtxd, amatrxd, eigvald, eigvecd, vecinvd,  workd)
      do i = 1, ndtsetsout
         anom_ratio(i) = sclinv(i)
         if (i .eq. iparnorm) then
            rmsratio(i) = 1.0
         else
            if (rms_anom_ov(iparnorm) .eq. 0.0) then
               rmsratio(i) = 0.0
            else 
               rmsratio(i) =
     $            rms_anom_ov(i)/rms_anom_ov(iparnorm)
            endif
         endif
      enddo
      write (lunout, 6001)
     $   (dname_out(i), anom_ratio(i), nobspar(i),
     $   rmsratio(i), i = 1,  ndtsetsout)
 6001 format (/
     $   ' Relative anomalous differences for each dataset, ',
     $   'and ratio of rms values',/,
     $   '   These are approximately the ratios of f"',//,
     $   '                     Scale      NumObs  Ratio of RMS',/,
     $   100(a16,1x,f10.4,i10,f10.4/))
c
      if (ndtsetsout .gt. 2) then
c (b) dispersive differences
c  Dataset to set scale = 1.0 for is one with largest rms_isod
c    this is identified as
c        iparnorm   in the full list of datasets
c        jparnorm   in the condensed list omitting kdtsbase
c    since there are ndtsetsout - 1  parameters
         iparnorm = -1
         rmsmax = 0.0
         ldata = .true. 
         k = 0
         do i = 1, ndtsetsout
            if (nobspar(i) .le. 0) ldata = .false.
            if (i .ne. kdtsbase) then
               k = k+1
               if (rms_isod_ov(i) .gt. rmsmax) then
                  rmsmax = rms_isod_ov(i)
                  iparnorm = i
                  jparnorm = k
               endif
            endif
         enddo 
         if (ldata) then
            npar = ndtsetsout - 1
            call FoxHolmes(dtsgetisod, npar, jparnorm, sclinv, nobspar,
     $         maxmtxd, amatrxd, eigvald, eigvecd, vecinvd,  workd)
         endif
c
c  Scales returned are for each datasets excluding the base set
         k = 0
         do i = 1, ndtsetsout
            if (i .eq. kdtsbase) then
               disp_ratio(i) = 0.0
               rmsratio(i) = 1.0
            else
               k = k+1
               disp_ratio(i) = sclinv(k)
               if (i .eq. iparnorm) then
                  rmsratio(i) = 1.0
               else
                  if (rms_isod_ov(iparnorm) .eq. 0.0) then
                     rmsratio(i) = 0.0
                  else 
                     rmsratio(i) =
     $                  rms_isod_ov(i)/
     $                  rms_isod_ov(iparnorm)
                  endif
               endif
            endif
         enddo
c
         call mkdtslab(kdtsbase, line)
         write (lunout, 6010) line(1:lenstr(line))
 6010    format (/
     $      ' Relative dispersive differences from base dataset',
     $      ' for each dataset, ',
     $      'and ratio of rms values',/,
     $      '      Base dataset is  ',a/
     $'   These are approximately the ratios of the differences in f'''
     $      ,//, '                     Scale      NumObs  Ratio of RMS')
c
         do i = 1, ndtsetsout
            if (i .ne. kdtsbase) then
               write (lunout, 6011)
     $            disod_name(i), disp_ratio(i), nobspar(i), rmsratio(i)
 6011          format (a16,1x,f10.4,I10,f10.4)
            endif
         enddo
c
      endif
c
      write (lunout, 6101)
 6101 format (/1x,20('----')/)

c
      return
      end
c
c
c
      subroutine dtsinitstore
c     =======================
c
c Initialise storage of anomalous & dispersive differences for
c analysis of ratios
c
c Current implementation as scratch file, but could be array
c
c  dtsinitstore    initialise storage
c  dtsstore        store reflection
c  dtsinitget      initialise retrieval
c  dtsgetanom      retrieve reflection from store & return
c                    anomalous differences
c  dtsgetisod      retrieve reflection from store & return
c                    dispersive|isomorphous differences
c
      include 'parameter.fh'
      include 'datasets.fh'
      include 'dts_storage.fh'
c
c Open scratch file
c
      call qopen(mdts_index, 'SCRATCH11', 'SCRATCH')
      if (mdts_index .le. 0) then
         call ccperr(1,'*** dtsinitstore: cannot open scratch file ***')
      endif
c
      mdts_count = 0
      mdts_ndts = ndtsetsout
      mdts_recl = mdts_ndts*5 + 4
c
      return
      end
c
c
c
      subroutine dtsstore(icntrc, jhkl, 
     $   ndts, danomdts, sddanomdts, disodts, sddisodts, fdts)
c     ========================================================
c
c  Store away anomalous & dispersive differences for analysis of ratios
c
c  dtsinitstore    initialise storage
c  dtsstore        store reflection
c  dtsinitget      initialise retrieval
c  dtsgetanom      retrieve reflection from store & return
c                    anomalous differences
c  dtsgetisod      retrieve reflection from store & return
c                    dispersive|isomorphous differences
c
c  Write them to scratch
c
c On entry
c  jhkl(3)          indices
c  icntrc        centric reflection if icntrc = 1, else = 0
c  ndts             number of datasets (= ndtsetsout)
c  danomdts(ndts)    anomalous differences for each dataset
c  sddanomdts(ndts) sd(anomalous differences) for each dataset
c  disodts(ndts)    isomorphous|dispersive difference from base dataset
c                      (including the base dataset itself)
c  sddisodts(ndts)  isomorphous|dispersive difference from base dataset
c  fdts(ndts)       <F>
c
      integer icntrc, ndts
      integer jhkl(3)
      real  
     $   danomdts(ndts), sddanomdts(ndts),
     $   disodts(ndts), sddisodts(ndts), fdts(ndts)
c
      include 'parameter.fh'
      include 'dts_storage.fh'
      include 'datasets.fh'
c
      integer i, l
c
c Put dts_data into buffer & write it
      do i = 1,3
         dts_data(i) = jhkl(i)
      enddo
      dts_data(4) = icntrc
      i = 5
      do l = 1,ndts
         dts_data(i) = danomdts(l)
         dts_data(i+1) = sddanomdts(l)
         dts_data(i+2) = disodts(l)
         dts_data(i+3) = sddisodts(l)
         dts_data(i+4) = fdts(l)
         i = i+5
      enddo
      call qwritr(mdts_index, dts_data, mdts_recl)
      mdts_count = mdts_count + 1
      return
      end
c
c
c
      subroutine dtsinitget
c     =====================
c
c Initialise retrieval of anomalous & dispersive differences for
c analysis of ratios
c
c
c  dtsinitstore    initialise storage
c  dtsstore        store reflection
c  dtsinitget      initialise retrieval
c  dtsgetanom      retrieve reflection from store & return
c                    anomalous differences
c  dtsgetisod      retrieve reflection from store & return
c                    dispersive|isomorphous differences
c
      include 'parameter.fh'
      include 'dts_storage.fh'
c
c Rewind scratch file
      call qseek(mdts_index, 1, 1, mdts_recl)
      mdts_read = 0
      return
      end
c
c
c
      subroutine dtsgetanom(yd, wd, nd)
c     =================================
c
c Retrieve reflection from store & return anomalous differences
c
c On exit:
c  yd    anomalous differences for each dataset
c  wd    weights, if no data 0.0
c  nd    = number of parameters = number of datasets, = -1 for end of data
c
      real yd(*), wd(*)
      integer nd
c
      include 'parameter.fh'
      include 'dts_storage.fh'
c
      integer i, l, istat, icntrc
c
      mdts_read = mdts_read + 1
      if (mdts_read .gt. mdts_count) go to 100
c
      call qreadr(mdts_index, dts_data, mdts_recl, istat)
      if (istat .gt. 0) then
         call ccperr(1,'***  dtsgetstore: qread error ***')
      elseif (istat .lt. 0) then
         go to 100
      endif
c
      icntrc = nint(dts_data(4))
c
      i = 5
      do l = 1, mdts_ndts
         if (icntrc .eq. 1 .or. dts_data(i+1) .le. 0.0) then
c  Centric or missing data
            yd(l) = 0.0
            wd(l) = 0.0
         else
            yd(l) = dts_data(i)
            wd(l) = 1.0/(dts_data(i+1)**2)
         endif
         i = i+5
      enddo
      nd = mdts_ndts
      return
c  End of file, rewind for next time
 100  continue 
      call dtsinitget
      nd = -1
      return
      end

c
c
c
      subroutine dtsgetisod(yd, wd, nd)
c     =================================
c
c Retrieve reflection from store & return isomorphous differences
c
c On exit:
c  yd    isomorphous differences for each dataset, relative to base set
c  wd    weights, if no data 0.0
c  nd    = number of parameters = number of datasets - 1 (excluding base set)
c        = -1 for end of data
c
      real yd(*), wd(*)
      integer nd
c
      include 'parameter.fh'
      include 'dts_storage.fh'
      include 'datasets.fh'
c
      integer i, k, l, istat, icntrc
      logical first
c
      mdts_read = mdts_read + 1
      if (mdts_read .gt. mdts_count) go to 100
c
      call qreadr(mdts_index, dts_data, mdts_recl, istat)
      if (istat .gt. 0) then
         call ccperr(1,'***  dtsgetstore: qread error ***')
      elseif (istat .lt. 0) then
         go to 100
      endif
c
      icntrc = nint(dts_data(4))
c
      i = 7
      k = 0
      do l = 1, mdts_ndts
         if (l .ne. kdtsbase) then
            k = k+1
            if (dts_data(i+1) .le. 0.0) then
c  Missing data
               yd(k) = 0.0
               wd(k) = 0.0
            else
               yd(k) = dts_data(i)
               wd(k) = 1.0/(dts_data(i+1)**2)
            endif
         endif
         i = i+5
      enddo
      nd = k
      return
c  End of file, rewind for next time
 100  continue 
      call dtsinitget
      nd = -1
      return
      end
c
c
c
      subroutine dtsgetdeli(deli, f, wd, nd)
c     =====================================
c
c Retrieve reflection from store & return delI/F & F
c
c On exit:
c  deli     for each dataset, (anomalous difference)/<Fh>
c  f        for each dataset, <Fh>
c  wd       weights, if no data or centric 0.0
c  nd    = number of parameters = number of datasets, = -1 for end of data
c
      real deli(*), f(*), wd(*)
      integer nd
c
      include 'parameter.fh'
      include 'dts_storage.fh'
c
      integer i, l, istat, icntrc
c
      mdts_read = mdts_read + 1
      if (mdts_read .gt. mdts_count) go to 100

      call qreadr(mdts_index, dts_data, mdts_recl, istat)
      if (istat .gt. 0) then
         call ccperr(1,'***  dtsgetstore: qread error ***')
      elseif (istat .lt. 0) then
         go to 100
      endif
c
      icntrc = nint(dts_data(4))
c
      i = 5
      do l = 1, mdts_ndts
         if (icntrc .eq. 1 .or. dts_data(i+1) .le. 0.0
     $      .or. dts_data(i+4) .le. 0.0) then
c  Centric or missing data
            deli(l) = 0.0
            f(l) = 0.0
            wd(l) = 0.0
         else
            deli(l) = dts_data(i)/dts_data(i+4)
            f(l) = dts_data(i+4)
c    FIXME what should this weight be?
            wd(l) = 1.0/(dts_data(i+1)**2)
         endif
         i = i+5
      enddo
      nd = mdts_ndts
      return
c  End of file, rewind for next time
 100  continue 
      call dtsinitget
      nd = -1
      return
      end
      subroutine dumpsc
c     =================
c
c  Dump all scales to output file
c
      implicit none
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'column.fh'
      include 'refcon.fh'
      include 'params.fh'
      include 'sdfacc.fh'
      include 'dump.fh'
      include 'sphhrm.fh'
c
      integer ldum, ifail, i, j, irun
      logical first
      character date*8, time*8
c
      save first
c
      data first/.true./
c
c
      if (first) then
         ldum = 0
         ifail = 0
         call ccpdpn(lundmp, dumpfl, 'NEW', 'F', ldum, ifail)
         first = .false.
      else
         rewind lundmp
      endif
c
c
      call ccpdat(date)
      call utime(time)
      write (lundmp, 6000) date, time
 6000 format('# Scales from SCALA on ',a,' at ',a)
c
      write (lundmp, 6002) numrun
 6002 format('NUMRUN ',i6)
c
      do 10, irun = 1, numrun
         write (lundmp, 6010) irunum(irun), nbfact(irun), nbfpos(irun),
     $        (ncscls(i,irun), i = 1,4), nrntds(irun), nrnsph(irun)
 6010    format ('RUN ',I6,' NBFACT ',2I6,' NSCALES ',4I6,
     $      ' NTAILS ',I6, ' NSRF', I6)
c
         if (nbfact(irun) .gt. 0) then
            write (lundmp, 6012) spcbfc(irun)
 6012       format ('BSPACING ',f12.5)
            do 20, i = 1, nbfact(irun)
               write (lundmp, 6020)
     $            i, (bfactr(j,i,irun), sdbfac(j,i,irun),
     $            j=1,nbfpos(irun))
 6020          format('BFACTOR ', i3, 6f12.5)
 20         continue
         endif
c
         write (lundmp, 6030) (spcscl(i,irun), i = 1,3)
 6030    format('GSPACING', 3f12.5)
c
         do 30, j = 1,ncscls(2,irun)
            do 31, i = 1,ncscls(1,irun)
               write (lundmp, 6032) i,j,
     $              cscals(j,i,irun), sdcscl(j,i,irun)
 6032          format('SCALE ',2i4,2f12.5)
 31         continue
 30      continue
c Tails correction
         if (nrntds(irun) .gt. 0) then
            write (lundmp, 6041)
     $         (partds(i,irun), i = 1, nrntds(irun))
 6041       format ('TAILS ',5f10.5)
         endif
c
c  Surface spherical harmonic terms
         if (nrnsph(irun) .gt. 0) then
            write (lundmp, 6051)
     $         nrnsph(irun), lmxsph(irun), lodsph(irun),
     $         lsmode(irun), mhpole(irun),
     $         (sphclm(i,irun), i = 1,  nrnsph(irun))
 6051       format ('SURFACE ', 5i5,100(/8f10.5))
            write (lundmp, 6052) (
     $         (nsphith(i,j,irun), j = 1, mxsphi), i = 1, mxstheta)
 6052       format ('SRFCOUNT ', 500(/10i8))
         endif 
c
 10   continue
c
      return
      end
      subroutine endppr(iwrprb)
c     ========================
c
c  End previous plot in normal probability file
c
c *** For xmgr ***
c
      integer iwrprb
c
      write (iwrprb, '(a)') '&'
      return 
      end
      REAL FUNCTION ERFCX(X)
CD    DOUBLE PRECISION FUNCTION DERFCX(X)
C------------------------------------------------------------------
C
C This subprogram computes approximate values for exp(x*x) * erfc(x).
C   (see comments heading CALERF).
C
C   Author/date: W. J. Cody, March 30, 1987
C
C------------------------------------------------------------------
      INTEGER JINT
      REAL             X, RESULT
CD    DOUBLE PRECISION X, RESULT
C------------------------------------------------------------------
      JINT = 2
      CALL CALERF(X,RESULT,JINT)
      ERFCX = RESULT
CD    DERFCX = RESULT
      RETURN
C---------- Last card of DERFCX ----------
      END
c
c
c
      subroutine excinc(exclude, irun,
     $   ntok, n, cvalue, fvalue, ityp,
     $   llist, vlist, istat)
c    ===============
c
c Process Batch include/exclude command
c
c Sets array irunbt(i) where i is internal batch number
c            irunbt(i) = -1     exclude
c                      = irun   include (irun .gt. 0)
c
c On entry:
c     exclude   .true. for exclude, .false. include
c     irun      run number for include
c     ntok      number of tokens in line
c     itok      first one to use
c     cvalue    character values from Parser
c     fvalue    number values from Parser
c     ityp      number types from Parser
c
c On exit:
c     n         next token to use, = 0 if all tokens used
c     llist     .lt. 0 error, eg too few tokens for range
c     = 0  number range, returned in vlist(1) & (2)
c     .gt. 0, number list, llist entries in vlist
c     vlist     range or list
c     istat     =  0  no error
c               = -1  error
      include 'parameter.fh'
      include 'inout.fh'
      include 'nbtchc.fh'
      include 'rundef.fh'
c
      logical exclude
      integer irun, n, ntok, ityp(ntok), llist, istat
      real fvalue(ntok), vlist(*)
      character*(*) cvalue(ntok)
c
      integer i, ibatch, kbatch
c
      integer ccp4_hash_lookup
      external ccp4_hash_lookup
c
      istat = 0
c
      call lisran(ntok, n, cvalue, fvalue, ityp, llist, vlist)
      if (llist .lt. 0) then
         write (lunout, '(A)') ' ** Error in BATCH list **' 
         istat = -1
         return
c     Set all these batches to this run
      else if (llist .eq. 0) then
c              Range
         do 101, i = 1, nbtall
            ibatch = ibtall(i)
            if (ibatch .ge. nint(vlist(1)) .and.
     $         ibatch .le. nint(vlist(2))) then
c                    Exclude or include
               if (exclude) then
                  irunbt(i) = -1
               else
                  irunbt(i) = irun
               endif
            endif
 101     continue
      else
c              List
         do 102, ibatch = 1, llist
            kbatch = nint(vlist(ibatch))
            i = ccp4_hash_lookup(kbatch)
            if (i .gt. 0) then
               if (exclude) then
                  irunbt(i) = -1
               else
                  irunbt(i) = irun
               endif
            else
               write (lunout, '(a,i6,a)')
     $            ' WARNING: Batch ', kbatch,
     $            ' not found in file'
            endif
 102     continue
      endif
c
      return
      end
c
c
c
      subroutine findgap(
     $   ngroup, idtsgroup, ibgroup, ibgapgroup, gapgroup, istat)
c    ======================================================
c
c Check through all batches and try to divide tham up into
c possible runs, by looking for discontinuities in input dataset, 
c batch number and rotation
c
c Also fix up wrap-arounds on rotation angle "phi"
c
c On exit:
c  ngroup     number of "groups" found: a group is a candidate
c              for a run, and are separated by a gap either in
c              batch number or in phi (or both)
c  idtsgroup  input dataset number for group
c  ibgroup(2,) batch serial number for first and last
c                batches in group
c  ibgapgroup gap in batch number from previous group
c              (= -1000 for 1st group)
c  gapgroup   gap in phi from previous group
c              (= 0.0 for 1st group)
c   
c  istat   = 0 OK,  = -1 inconsistent header information
c
c  Also set in common block /rundef/:
c   For batch with serial number ib
c    phioffset(ib)      phi offset to cope with 360 degree wrap-around
c    phibat(3, ib)      phi1, phi2, delphi for batch, after wrap-around fix
c   For SMOOTH mode, these are reset later in s/r setscl
      include 'parameter.fh'
      integer istat
      integer ngroup, idtsgroup(maxgrp), ibgroup(2,maxgrp),
     $   ibgapgroup(maxgrp)
      real gapgroup(maxgrp)
c
      include 'inout.fh'
      include 'rundef.fh'
      include 'nbtchc.fh'
      include 'datasets.fh'
c
      integer i, j, k, l, ib, istat1, lastbatnum, lastdtsnum,
     $   phi3scan
      real    time1, time2, phi1, phi2, delphi, xydlim(2,2), ubmat(3,3),
     $   umat(3,3), bmat(3,3), s0(3), current_offset, philast,
     $   delphilast, gap, gapmod, dtmmat(3,3), wvlnth,
     $   e1e2mat(3,3), e1e2inv(3,3) 
      logical lphi, ldet, lmat, lnewgroup
c
c
      real tolerance
      data tolerance /0.01/

      istat = 0
      ngroup = 0
      current_offset = 0.0
c
c
      do ib = 1, nbtall
c  Get information for this batch
         call batinf(
     $      ibtall(ib), time1, time2, phi1, phi2, delphi,
     $      xydlim, dtmmat, e1e2mat, e1e2inv, phi3scan, ubmat,
     $      umat, bmat, wvlnth, s0,
     $      lphi, ldet, lmat, istat1)

c  We are only interested in phi1, phi2 & delphi
c  lphi .false. if no phi data
         if (istat1 .lt. 0 .or. .not.lphi) then
            phi1 = 0.0
            phi2 = 0.0
            delphi = 0.0
         endif
c
         if (ib .eq. 1) then
c First batch is by definition the beginning of a group
            ngroup = ngroup + 1
            idtsgroup(ngroup) = jdtsbt(ib)
            ibgroup(1,ngroup) = ib
            gapgroup(ngroup) = 0.0
            ibgapgroup(ngroup) = -1000
         else
c
c Compare this batch with previous one
c A group much have contiguous and increasing batch numbers
            lnewgroup = .false.
            if (jdtsbt(ib) .ne. lastdtsnum) then
               lnewgroup = .true.
            endif
            if (ibtall(ib) .ne. lastbatnum+1) then
               lnewgroup = .true.
            endif
c Fix up delphi (usually just phi2 - phi1)
            if (delphi .eq. 0.0) then
               delphi = phi2 - phi1
            else
c Delphi is explicit in the file, so check that it fits
               if (abs(delphi - (phi2-phi1)) .gt. tolerance) then
c   delphi .ne. phi2 - phi1
c        In this case, delphi from the file should bear some simple
c relationship to phi2-phi1
c           OK if phi1 + delphi = phi2 +- n 360
                  if (abs(mod(phi1 + delphi - phi2, 360.))
     $               .lt. tolerance) then
c               OK, fix up phi2 to match delphi
                     phi2 = phi1 + delphi
                  else
                     write (lunout, '(/a,3(a,f10.3)/)')
     $            ' **** Inconsistent rotation information in header:',
     $                  '      Phi1 :', phi1, '  Phi2 :', phi2,
     $                  '  Delphi :', delphi
                     istat = -1
                     return
                  endif
               endif
            endif
c
c  Check for phi gap from last batch
            gap = philast - phi1 
c  Force offset to be multiple of 360.0
            current_offset = float(nint(gap/360.))*360.
            if (abs(gap) .gt. tolerance) then
c   allow gap to be close to 360.0
               gapmod = gap - float(nint(gap/360.))*360.
c   check for 360 degree wrap-around
               if (abs(gapmod) .le. tolerance) then
c     wrap-around
                  gap = gapmod
               else
c     a real gap
                  lnewgroup = .true.
               endif
            endif
c
            if (lnewgroup) then
               ibgroup(2,ngroup) = ib - 1
               ngroup = ngroup + 1
               idtsgroup(ngroup) = jdtsbt(ib)
               ibgroup(1,ngroup) = ib
               gapgroup(ngroup) = gap
               ibgapgroup(ngroup) = ibtall(ib) - lastbatnum
               current_offset = 0.0
            endif
c store and apply offset
            phi1 = phi1 + current_offset
            phi2 = phi2 + current_offset
         endif
c store limit
         phioffset(ib) = current_offset
         phibat(1,ib) = phi1
         phibat(2,ib) = phi2
         phibat(3,ib) = delphi
         lastdtsnum = jdtsbt(ib)
         lastbatnum = ibtall(ib)
         philast = phi2
         delphilast = delphi
      enddo
      ibgroup(2,ngroup) = nbtall
c
      return
      end
      subroutine fitcrc(ncirpt, mcirpt, xycpt, circen, cirrad, rmsres)
c     ===============================================================
c
c  Fit circle to set of points
c
c On entry:
c     ncirpt               number of points in list
c     mcirpt               number of accepted points in list (excluding
c                            deleted points)
c     xycpt(2,ncirpt)     (pixel) coordinates of points (real)
c
c On exit:
c     circen(2)            coordinates of centre 
c                          (same coordinate frame as input)
c     cirrad               radius (= -1.0 if can't fit circle)
c     rmsres               rms deviation
c
c Fit coordinates to circle, NOT allowing for a scale factor in the
c     y direction relative to x
c
c     If x, y are vectors of the measured x & y coordinates, then we can
c     fit a circle, centre (p,q), radius r, with a relative yscale v
c     (ie y*v is the corrected y coordinate in mm). The error vector Eps
c     is given by
c
c     eps = x**2 + y**2 - 2px - 2qy + p**2 + q**2 - r**2
c      ~    ~      ~        ~     ~
c         = Rcalc**2 - r**2
c           ~
c
c         = A s  +  b
c             ~     ~
c
c where   A  =  ( x  y  1 )        s  =    ( -2p                )    
c                 ~  ~  ~          ~       ( -2q                )
c                                          ( p**2 + q**2 - r**2 )
c         b  =  x**2  + y**2             
c         ~     ~
c                                                    ~
c The least squares solution is given by minimizing eps eps
c                                                    ~   ~
c This is a linear least squares in the 3 components of s
c             ~        ~
c     s  = - (AA)**-1  A b
c     ~                  ~
c    ~
c    AA  = ( Sum x**2    Sum x y   Sum x     )
c          ( Sum x y     Sum y**2  Sum y     )
c          ( Sum x       Sum y     Sum 1     )
c    ~
c    Ab  = ( Sum x*(x**2 + y**2)  )
c          ( Sum y*(x**2 + y**2)  )
c          ( Sum (x**2 + y**2)    )
c
      implicit none
c
      integer ncirpt, mcirpt
      real circen(2),cirrad, yscale, xycpt(2,ncirpt), rmsres
c
      integer i,j
c
      integer MAXCPT
      parameter (MAXCPT = 200)
      real xc(MAXCPT), yc(MAXCPT)
c
      real sumx, sumy, sumx2, sumy2, sumxy, 
     $     sumx2py2, sumxx2py2, sumyx2py2
      real aa(3,3), ab(3), det, work1(3), work2(3), param(3)
      real x, y, r2, eps
c
      if (mcirpt .lt. 3) then
         cirrad = -1.0
         go to 900
      endif
c
      sumx = 0.0
      sumy = 0.0
      sumx2 = 0.0
      sumy2 = 0.0
      sumxy = 0.0
      sumx2py2 = 0.0
      sumxx2py2 = 0.0
      sumyx2py2 = 0.0
c
c--- Get scanner frame coordinates (ie convert to mm)
      do 10, i = 1, ncirpt
         x = xycpt(1,i)
         y = xycpt(2,i)
         xc(i) = x
         yc(i) = y
         sumx    = sumx + x
         sumy    = sumy + y
         sumx2   = sumx2 + x*x
         sumy2   = sumy2 + y*y
         sumxy   = sumxy + x*y
         sumx2py2 = sumx2py2 + x*x + y*y
         sumxx2py2 = sumxx2py2 + x*(x*x + y*y)
         sumyx2py2 = sumyx2py2 + y*(x*x + y*y)
 10   continue
c
      aa(1,1) = sumx2
      aa(1,2) = sumxy
      aa(1,3) = sumx
      aa(2,1) = sumxy
      aa(2,2) = sumy2
      aa(2,3) = sumy
      aa(3,1) = sumx
      aa(3,2) = sumy
      aa(3,3) = mcirpt
c
      ab(1) = -sumxx2py2
      ab(2) = -sumyx2py2
      ab(3) = -sumx2py2
c
      call minvn(aa, 3, det, work1, work2)
      if (det .eq. 0.0) then
         write (6, '(a)') ' ***FITCIR: singular matrix ***'
         cirrad = -1.0
         return
      endif
c aa is now inverse normal matrix
c
c param = aa**-1 * ab
      do 20, j = 1, 3
         param(j) = 0.0
         do 30, i = 1, 3
            param(j) = param(j) + aa(j,i) * ab(i)
 30      continue
 20   continue
c
c  param vector = ( -2 p               )
c                 ( -2 q               )
c                 ( p**2 + q**2 - r**2 )
c
c    where (p, q) is centre of circle (in mm), radius r (mm)
c
      circen(1) = -0.5 * param(1)
      circen(2) = -0.5 * param(2)
      cirrad = sqrt(circen(1)**2 + circen(2)**2 - param(3))
c
      sumx2 = 0.0
c--- Calculate rms residual
      do 40, i = 1, ncirpt
         eps = sqrt((xc(i) - circen(1))**2 +
     $      (yc(i) - circen(2))**2) - cirrad
         sumx2 = sumx2 + eps**2
 40   continue
      rmsres = sqrt(sumx2/float(mcirpt))
c
 900  return
      end
c
c
c
      subroutine fndepb(eacen, prob, ecen)
c     ====================================
c
c Find E for centric data corresponding to probability
c ie find inverse of p(E) = erfc(E/sqrt(2))
c
c This is done by a crude search procedure (but will only be done once)
c 
c
c On entry:
c  eacen        acentric E
c
c On exit:
c  eacen        acentric E  (possibly reset to maximum)
c  prob         probability
c  ecen         corresponding centric E (possibly reset to maximum)
c
      real eacen, prob, ecen
c
      real start, stope, step, tolerance, fac
      real e, p, eacmax, ecnmax
c
      real sc_erfc
      external sc_erfc
      data tolerance /0.00001/, fac /10.0/
      data eacmax, ecnmax /10.0, 12.0/
c
c Starting values
c
      eacen = min(eacen, eacmax)
      prob = exp(-eacen**2)
      start = 0.0
      stope = eacen*2
      step  = 1.0
      p = 0.0
 1    e = start
 2    if (e .le. stope) then
         p = sc_erfc(e/sqrt(2.))
         if (p .lt. prob) then
c  Close enough?
            if (step .lt. tolerance) then
               go to 10
            endif 
c  Reduce step
            start = e - step
            stope = e
            step  = step/fac
            go to 1
         endif
         e = e + step
         go to 2
      endif
c If we fall out iof the end, this answer is near enough
 10   ecen = e - 0.5*step
      ecen = min(ecen, ecnmax)
      return
c
      end
c
C
      SUBROUTINE CALERF(ARG,RESULT,JINT)
C------------------------------------------------------------------
C
C This packet evaluates  erf(x),  erfc(x),  and  exp(x*x)*erfc(x)
C   for a real argument  x.  It contains three FUNCTION type
C   subprograms: ERF, ERFC, and ERFCX (or DERF, DERFC, and DERFCX),
C   and one SUBROUTINE type subprogram, CALERF.  The calling
C   statements for the primary entries are:
C
C                   Y=ERF(X)     (or   Y=DERF(X)),
C
C                   Y=ERFC(X)    (or   Y=DERFC(X)),
C   and
C                   Y=ERFCX(X)   (or   Y=DERFCX(X)).
C
C   The routine  CALERF  is intended for internal packet use only,
C   all computations within the packet being concentrated in this
C   routine.  The function subprograms invoke  CALERF  with the
C   statement
C
C          CALL CALERF(ARG,RESULT,JINT)
C
C   where the parameter usage is as follows
C
C      Function                     Parameters for CALERF
C       call              ARG                  Result          JINT
C
C     ERF(ARG)      ANY REAL ARGUMENT         ERF(ARG)          0
C     ERFC(ARG)     ABS(ARG) .LT. XBIG        ERFC(ARG)         1
C     ERFCX(ARG)    XNEG .LT. ARG .LT. XMAX   ERFCX(ARG)        2
C
C   The main computation evaluates near-minimax approximations
C   from "Rational Chebyshev approximations for the error function"
C   by W. J. Cody, Math. Comp., 1969, PP. 631-638.  This
C   transportable program uses rational functions that theoretically
C   approximate  erf(x)  and  erfc(x)  to at least 18 significant
C   decimal digits.  The accuracy achieved depends on the arithmetic
C   system, the compiler, the intrinsic functions, and proper
C   selection of the machine-dependent constants.
C
C*******************************************************************
C*******************************************************************
C
C Explanation of machine-dependent constants
C
C   XMIN   = the smallest positive floating-point number.
C   XINF   = the largest positive finite floating-point number.
C   XNEG   = the largest negative argument acceptable to ERFCX;
C            the negative of the solution to the equation
C            2*exp(x*x) = XINF.
C   XSMALL = argument below which erf(x) may be represented by
C            2*x/sqrt(pi)  and above which  x*x  will not underflow.
C            A conservative value is the largest machine number X
C            such that   1.0 + X = 1.0   to machine precision.
C   XBIG   = largest argument acceptable to ERFC;  solution to
C            the equation:  W(x) * (1-0.5/x**2) = XMIN,  where
C            W(x) = exp(-x*x)/[x*sqrt(pi)].
C   XHUGE  = argument above which  1.0 - 1/(2*x*x) = 1.0  to
C            machine precision.  A conservative value is
C            1/[2*sqrt(XSMALL)]
C   XMAX   = largest acceptable argument to ERFCX; the minimum
C            of XINF and 1/[sqrt(pi)*XMIN].
C
C   Approximate values for some important machines are:
C
C                          XMIN       XINF        XNEG     XSMALL
C
C  CDC 7600      (S.P.)  3.13E-294   1.26E+322   -27.220  7.11E-15
C  CRAY-1        (S.P.)  4.58E-2467  5.45E+2465  -75.345  7.11E-15
C  IEEE (IBM/XT,
C    SUN, etc.)  (S.P.)  1.18E-38    3.40E+38     -9.382  5.96E-8
C  IEEE (IBM/XT,
C    SUN, etc.)  (D.P.)  2.23D-308   1.79D+308   -26.628  1.11D-16
C  IBM 195       (D.P.)  5.40D-79    7.23E+75    -13.190  1.39D-17
C  UNIVAC 1108   (D.P.)  2.78D-309   8.98D+307   -26.615  1.73D-18
C  VAX D-Format  (D.P.)  2.94D-39    1.70D+38     -9.345  1.39D-17
C  VAX G-Format  (D.P.)  5.56D-309   8.98D+307   -26.615  1.11D-16
C
C
C                          XBIG       XHUGE       XMAX
C
C  CDC 7600      (S.P.)  25.922      8.39E+6     1.80X+293
C  CRAY-1        (S.P.)  75.326      8.39E+6     5.45E+2465
C  IEEE (IBM/XT,
C    SUN, etc.)  (S.P.)   9.194      2.90E+3     4.79E+37
C  IEEE (IBM/XT,
C    SUN, etc.)  (D.P.)  26.543      6.71D+7     2.53D+307
C  IBM 195       (D.P.)  13.306      1.90D+8     7.23E+75
C  UNIVAC 1108   (D.P.)  26.582      5.37D+8     8.98D+307
C  VAX D-Format  (D.P.)   9.269      1.90D+8     1.70D+38
C  VAX G-Format  (D.P.)  26.569      6.71D+7     8.98D+307
C
C*******************************************************************
C*******************************************************************
C
C Error returns
C
C  The program returns  ERFC = 0      for  ARG .GE. XBIG;
C
C                       ERFCX = XINF  for  ARG .LT. XNEG;
C      and
C                       ERFCX = 0     for  ARG .GE. XMAX.
C
C
C Intrinsic functions required are:
C
C     ABS, AINT, EXP
C
C
C  Author: W. J. Cody
C          Mathematics and Computer Science Division
C          Argonne National Laboratory
C          Argonne, IL 60439
C
C  Latest modification: March 19, 1990
C
C------------------------------------------------------------------
      INTEGER I,JINT
      REAL
CD    DOUBLE PRECISION
     1     A,ARG,B,C,D,DEL,FOUR,HALF,P,ONE,Q,RESULT,SIXTEN,SQRPI,
     2     TWO,THRESH,X,XBIG,XDEN,XHUGE,XINF,XMAX,XNEG,XNUM,XSMALL,
     3     Y,YSQ,ZERO
      DIMENSION A(5),B(4),C(9),D(8),P(6),Q(5)
C------------------------------------------------------------------
C  Mathematical constants
C------------------------------------------------------------------
      DATA FOUR,ONE,HALF,TWO,ZERO/4.0E0,1.0E0,0.5E0,2.0E0,0.0E0/,
     1     SQRPI/5.6418958354775628695E-1/,THRESH/0.46875E0/,
     2     SIXTEN/16.0E0/
CD    DATA FOUR,ONE,HALF,TWO,ZERO/4.0D0,1.0D0,0.5D0,2.0D0,0.0D0/,
CD   1     SQRPI/5.6418958354775628695D-1/,THRESH/0.46875D0/,
CD   2     SIXTEN/16.0D0/
C------------------------------------------------------------------
C  Machine-dependent constants
C------------------------------------------------------------------
      DATA XINF,XNEG,XSMALL/1.70E+38,-9.344E0,5.96E-8/,
     1     XBIG,XHUGE,XMAX/9.194E0,2.90E3,4.79E37/
CD    DATA XINF,XNEG,XSMALL/1.79D308,-26.628D0,1.11D-16/,
CD   1     XBIG,XHUGE,XMAX/26.543D0,6.71D7,2.53D307/
C------------------------------------------------------------------
C  Coefficients for approximation to  erf  in first interval
C------------------------------------------------------------------
      DATA A/3.16112374387056560E00,1.13864154151050156E02,
     1       3.77485237685302021E02,3.20937758913846947E03,
     2       1.85777706184603153E-1/
      DATA B/2.36012909523441209E01,2.44024637934444173E02,
     1       1.28261652607737228E03,2.84423683343917062E03/
CD    DATA A/3.16112374387056560D00,1.13864154151050156D02,
CD   1       3.77485237685302021D02,3.20937758913846947D03,
CD   2       1.85777706184603153D-1/
CD    DATA B/2.36012909523441209D01,2.44024637934444173D02,
CD   1       1.28261652607737228D03,2.84423683343917062D03/
C------------------------------------------------------------------
C  Coefficients for approximation to  erfc  in second interval
C------------------------------------------------------------------
      DATA C/5.64188496988670089E-1,8.88314979438837594E0,
     1       6.61191906371416295E01,2.98635138197400131E02,
     2       8.81952221241769090E02,1.71204761263407058E03,
     3       2.05107837782607147E03,1.23033935479799725E03,
     4       2.15311535474403846E-8/
      DATA D/1.57449261107098347E01,1.17693950891312499E02,
     1       5.37181101862009858E02,1.62138957456669019E03,
     2       3.29079923573345963E03,4.36261909014324716E03,
     3       3.43936767414372164E03,1.23033935480374942E03/
CD    DATA C/5.64188496988670089D-1,8.88314979438837594D0,
CD   1       6.61191906371416295D01,2.98635138197400131D02,
CD   2       8.81952221241769090D02,1.71204761263407058D03,
CD   3       2.05107837782607147D03,1.23033935479799725D03,
CD   4       2.15311535474403846D-8/
CD    DATA D/1.57449261107098347D01,1.17693950891312499D02,
CD   1       5.37181101862009858D02,1.62138957456669019D03,
CD   2       3.29079923573345963D03,4.36261909014324716D03,
CD   3       3.43936767414372164D03,1.23033935480374942D03/
C------------------------------------------------------------------
C  Coefficients for approximation to  erfc  in third interval
C------------------------------------------------------------------
      DATA P/3.05326634961232344E-1,3.60344899949804439E-1,
     1       1.25781726111229246E-1,1.60837851487422766E-2,
     2       6.58749161529837803E-4,1.63153871373020978E-2/
      DATA Q/2.56852019228982242E00,1.87295284992346047E00,
     1       5.27905102951428412E-1,6.05183413124413191E-2,
     2       2.33520497626869185E-3/
CD    DATA P/3.05326634961232344D-1,3.60344899949804439D-1,
CD   1       1.25781726111229246D-1,1.60837851487422766D-2,
CD   2       6.58749161529837803D-4,1.63153871373020978D-2/
CD    DATA Q/2.56852019228982242D00,1.87295284992346047D00,
CD   1       5.27905102951428412D-1,6.05183413124413191D-2,
CD   2       2.33520497626869185D-3/
C------------------------------------------------------------------
      X = ARG
      Y = ABS(X)
      IF (Y .LE. THRESH) THEN
C------------------------------------------------------------------
C  Evaluate  erf  for  |X| <= 0.46875
C------------------------------------------------------------------
            YSQ = ZERO
            IF (Y .GT. XSMALL) YSQ = Y * Y
            XNUM = A(5)*YSQ
            XDEN = YSQ
            DO 20 I = 1, 3
               XNUM = (XNUM + A(I)) * YSQ
               XDEN = (XDEN + B(I)) * YSQ
   20       CONTINUE
            RESULT = X * (XNUM + A(4)) / (XDEN + B(4))
            IF (JINT .NE. 0) RESULT = ONE - RESULT
            IF (JINT .EQ. 2) RESULT = EXP(YSQ) * RESULT
            GO TO 800
C------------------------------------------------------------------
C  Evaluate  erfc  for 0.46875 <= |X| <= 4.0
C------------------------------------------------------------------
         ELSE IF (Y .LE. FOUR) THEN
            XNUM = C(9)*Y
            XDEN = Y
            DO 120 I = 1, 7
               XNUM = (XNUM + C(I)) * Y
               XDEN = (XDEN + D(I)) * Y
  120       CONTINUE
            RESULT = (XNUM + C(8)) / (XDEN + D(8))
            IF (JINT .NE. 2) THEN
               YSQ = AINT(Y*SIXTEN)/SIXTEN
               DEL = (Y-YSQ)*(Y+YSQ)
               RESULT = EXP(-YSQ*YSQ) * EXP(-DEL) * RESULT
            END IF
C------------------------------------------------------------------
C  Evaluate  erfc  for |X| > 4.0
C------------------------------------------------------------------
         ELSE
            RESULT = ZERO
            IF (Y .GE. XBIG) THEN
               IF ((JINT .NE. 2) .OR. (Y .GE. XMAX)) GO TO 300
               IF (Y .GE. XHUGE) THEN
                  RESULT = SQRPI / Y
                  GO TO 300
               END IF
            END IF
            YSQ = ONE / (Y * Y)
            XNUM = P(6)*YSQ
            XDEN = YSQ
            DO 240 I = 1, 4
               XNUM = (XNUM + P(I)) * YSQ
               XDEN = (XDEN + Q(I)) * YSQ
  240       CONTINUE
            RESULT = YSQ *(XNUM + P(5)) / (XDEN + Q(5))
            RESULT = (SQRPI -  RESULT) / Y
            IF (JINT .NE. 2) THEN
               YSQ = AINT(Y*SIXTEN)/SIXTEN
               DEL = (Y-YSQ)*(Y+YSQ)
               RESULT = EXP(-YSQ*YSQ) * EXP(-DEL) * RESULT
            END IF
      END IF
C------------------------------------------------------------------
C  Fix up for negative argument, erf, etc.
C------------------------------------------------------------------
  300 IF (JINT .EQ. 0) THEN
            RESULT = (HALF - RESULT) + HALF
            IF (X .LT. ZERO) RESULT = -RESULT
         ELSE IF (JINT .EQ. 1) THEN
            IF (X .LT. ZERO) RESULT = TWO - RESULT
         ELSE
            IF (X .LT. ZERO) THEN
               IF (X .LT. XNEG) THEN
                     RESULT = XINF
                  ELSE
                     YSQ = AINT(X*SIXTEN)/SIXTEN
                     DEL = (X-YSQ)*(X+YSQ)
                     Y = EXP(YSQ*YSQ) * EXP(DEL)
                     RESULT = (Y+Y) - RESULT
               END IF
            END IF
      END IF
  800 RETURN
C---------- Last card of CALERF ----------
      END
      REAL FUNCTION SC_ERF(X)
CD    DOUBLE PRECISION FUNCTION DERF(X)
C--------------------------------------------------------------------
C
C This subprogram computes approximate values for erf(x).
C   (see comments heading CALERF).
C
C   Author/date: W. J. Cody, January 8, 1985
C
C--------------------------------------------------------------------
      INTEGER JINT
      REAL             X, RESULT
CD    DOUBLE PRECISION X, RESULT
C------------------------------------------------------------------
      JINT = 0
      CALL CALERF(X,RESULT,JINT)
      SC_ERF = RESULT
CD    DERF = RESULT
      RETURN
C---------- Last card of DERF ----------
      END
      REAL FUNCTION SC_ERFC(X)
CD    DOUBLE PRECISION FUNCTION DERFC(X)
C--------------------------------------------------------------------
C
C This subprogram computes approximate values for erfc(x).
C   (see comments heading CALERF).
C
C   Author/date: W. J. Cody, January 8, 1985
C
C--------------------------------------------------------------------
      INTEGER JINT
      REAL             X, RESULT
CD    DOUBLE PRECISION X, RESULT
C------------------------------------------------------------------
      JINT = 1
      CALL CALERF(X,RESULT,JINT)
      SC_ERFC = RESULT
CD    DERFC = RESULT
      RETURN
C---------- Last card of DERFC ----------
      END
c
c
c
      subroutine FoxHolmes(fetch_obs, npar, iparnorm, sclinv, nobspar,
     $        maxmtx, amatrx, eigval, eigvec, vecinv,  work)
c     ========================================================
c
c Determine scales which relate npar sets of observations yd(npar)
c using the Fox & Holmes scaling algorithm. 
c
c  ie dividing scale factors gs(l) (l=1,npar) are determined from
c a series of sets of npar matching observations yd(l) (l=1,npar),
c Nobs sets, in order to minimize
c
c    Sum[n=1,Nobs]Sum[l=1,npar](wd(l)*(yd(l) - gs(l) * <y>)**2
c
c    where  <y> is the mean value of the yd(l) for the n'th observation
c
c    <y> = Sum[l=1,npar](wd(l) * gs(l) * yd(l))/Sum[l=1,npar](wd(l) * gs(l)**2)
c
c Observations are obtained from an external subroutine call
c
c    call fetch_obs(yd, wd, nd)
c      Returns:  yd(nd)  observations for each scale group 1->nd
c                wd(nd)  corresponding weights (= 0.0 for no observation)
c                nd      number of data in set, normally = npar
c                        = -1 for end of data
c
c Note that this routine only does the simple case where each member of a
c set of observations yd(l) has  single unique scale factor gs(l)
c associated with it. The more complicated case where the gs(l) are
c themselves functions of the actual parameters is not dealt with here.
c
c Internal arrays are dimensioned for a maximum of maxbat parameters
c
c
c On entry:
c
c  npar            number of parameters (= number of data in each set)
c  iparnorm        parameter number to normalise scale = 1.0
c
c  Matrix storage etc (see bigarg.fh)
c     maxmtx       dimension of matrix etc
c      double precision amatrx(maxmtx,maxmtx),
c     $     eigval(maxmtx), eigvec(maxmtx,maxmtx), vecinv(maxmtx),
c     $     work(5*maxmtx)
c
c On exit:
c  sclinv(npar)    dividing scale factors (sclinvi(parnorm) = 1.0)
c  nobspar(npar)   number of observations contributing to each parameter
c
      integer npar, iparnorm
      integer nobspar(npar)
      real    sclinv(npar)
c
      include 'parameter.fh'
      include 'bigarg.fh'
      include 'inout.fh'
c
c
      integer i, j, l, m, istat
      integer maxcyc, icyc, nobs, ni, i1, nt(maxbat)
      real gs(maxbat), grad(maxbat),
     $   swg2, swgy, ym, vmin1, vali,
     $   dglydg(maxbat), dymdg(maxbat), trshft(maxbat), shift(maxbat),
     $   convrg, scnorm
      double precision sumrsd, wd2nmm, wd2nmml, delwd2
c
      real yd(maxbat), wd(maxbat)
      integer nd
c
      if (npar .gt. maxbat) then
         write (lunout, '(/a,2i8/)')
     $      ' ERROR in subroutine FoxHolmes: too many parameters',
     $      npar, maxbat
         call dna_error('internal error')
         call ccperr(1,'*** Array overflow in subroutine FoxHolmes ***')
      endif
c
c Initialize all inverse scales = 1.0
      do l = 1, npar
         gs(l) = 1.0
      enddo
c
      maxcyc = 20
      icyc   = 0
      wd2nmml = 0.0
      convrg = 0.001
c---------- Loop iterations
 1    icyc = icyc+1
c initialise matrix & rhs
      do i = 1,npar
         do j = 1,npar
            amatrx(i,j) = 0.0
         enddo
         grad(i) = 0.0
         nobspar(i) = 0
      enddo
      nobs = 0
      sumrsd = 0.0
c
c.... loop all sets of observations
 10   call fetch_obs(yd, wd, nd)
c get observation set: nd = -1 marks end of data, else nd should = npar
      if (nd .eq. npar) then
c Determine mean <y> for this set
         swg2 = 0.0
         swgy = 0.0
         ni = 0
         do l = 1, npar
            if (wd(l) .ne. 0.0) then
               swg2 = swg2 + wd(l) * gs(l)**2
               swgy = swgy + wd(l) * gs(l) * yd(l)
               ni = ni+1
               nobs = nobs+1
               nobspar(l) = nobspar(l) + 1
            endif
         enddo
         if (ni .gt. 0) then
            if (swg2 .ne. 0.0) then
               ym = swgy / swg2
            else 
               ym = 0.0
            endif 
            do l = 1,npar
               if (wd(l) .ne. 0.0 .and. swg2 .ne. 0.0 ) then
c Dym/dgl = wdl (ydl - 2 ym gl)/Sum(wg^2)
                  dymdg(l) =
     $               wd(l) * (yd(l) - 2.*ym*gs(l))/swg2
               else
                  dymdg(l) = 0.0
               endif
            enddo
c . . .  loop groups l for observations
            do l = 1,npar
               if (wd(l) .ne. 0.0) then
                  sumrsd = sumrsd +
     $               wd(l)*(yd(l) - gs(l) * ym)**2
c . .. .. loop groups for parameters
                  do i = 1, npar
c  d(gl ym) / dgi
                     dglydg(i) = gs(l) *  dymdg(i)
                     if (i .eq. l) dglydg(i) = dglydg(i) + ym
                     if (dglydg(i) .ne. 0.0) then
c  rhs gradient(i) = - Sum[l](w (ydl - gl ym) d(gl ym) / dgi
                        grad(i) = grad(i)
     $                     + wd(l)*
     $                     (yd(l) - gs(l) * ym) * dglydg(i)
c Matrix element
                        do j = 1, i
c     Note should always have A(i,j) j .le. i for EA06C: = 'L' for dsyev
                           amatrx(i,j) = amatrx(i,j) +
     $                        wd(l) * dglydg(i) * dglydg(j)
                        enddo
                     endif
                  enddo
               endif 
            enddo
         endif
         go to 10
      endif
      if (nd .ge. 0) then
c nd should only ever be either = npar or = -1
         write (lunout, '(a,i4)') '**** FoxHolmes: nd error:', nd
         call dna_error('internal error')
         call ccperr
     $ (1,'*** Error in FoxHolmes: nd error return from fetch_obs ***')
      endif
c
      if (nobs-npar .gt. 0) then
         wd2nmm = sumrsd/float(nobs-npar)
      else 
         wd2nmm = 0.0
      endif
c Solve equations
c  get eigen values & eigenvectors of normal matrix
c      write (lunout, '(a,2i6)') ' FH matrix, npar, icyc', npar, icyc
c      do j = 1,npar
c         write (lunout, '(100g10.3)') (amatrx(i,j),i=1,npar)
c      enddo 
c       call ea06cd(
c     $   amatrx, eigval, eigvec, npar, maxmtx, maxmtx, work)
c
       call dsyev('V', 'L', npar, amatrx, maxmtx, eigval, work,
     $   5*maxmtx, istat)
       if (istat .ne. 0) then
          call dna_error('internal error')
          call ccperr(1,'*** Error in foxholmes/dsyev ***')
       endif
       do j = 1, npar
          do i = 1, npar
             eigvec(i,j) = amatrx(i,j)
          enddo
       enddo
c
c---- Find smallest eigenvalue
c
      vmin1 = 100000.0
      i1 = 1
c
      do i = 1,npar
         vali = eigval(i)
         if (vali.le.vmin1) then
            vmin1 = vali
            i1 = i
         end if
      enddo
c---- Now I1 points to smallest eigenvalue, I2 to next one
      do i = 1,npar
         if (i .eq. i1) then
            vecinv(i) = 0.0
         else
            if (eigval(i) .eq. 0.0) then
               vecinv(i) = 0.0
            else 
               vecinv(i) = 1./eigval(i)
            endif 
         endif
      enddo
c get transformed shiftss
      do j = 1,npar
         trshft(j) = 0.0
         do i = 1,npar
            trshft(j) = trshft(j) + eigvec(i,j) * grad(i)
         enddo
         trshft(j) = trshft(j) * vecinv(j)
      enddo
c Transform shifts back
      do j = 1, npar
         shift(j) = 0.0
         do i = 1, npar
            shift(j) = shift(j) + trshft(i) * eigvec(j,i)
         enddo
      enddo 
c
c Normalize on iparnorm'th & apply
      scnorm = gs(iparnorm) + shift(iparnorm)
      do j = 1, npar
         if (scnorm .ne. 0.0) then
            gs(j) = (gs(j) + shift(j))/scnorm
         else
            gs(j) = 0.0
         endif
      enddo
c
c
c Has it converged or run out of cycles? if not, go back
      if (wd2nmml .gt. 0.0) then
         delwd2 = wd2nmml - wd2nmm
         wd2nmml = wd2nmm
         if (wd2nmm .ne. 0.0) then
            if (delwd2/wd2nmm .gt. convrg .and. icyc .lt. maxcyc)
     $         go to 1
         endif
      else
         wd2nmml = wd2nmm
         if (icyc .lt. maxcyc) go to 1
      endif
c
c========================================================
c
c Finished
c
c
      do j = 1, npar
         sclinv(j) = gs(j)
      enddo
      return
      end
      subroutine fuldis(nrs, kanom, lselds, icntrc, jhkl,
     $   ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf,
     $   lanmtc, sdrej, sdrej2, sdreja, sdrej2a, krej2, 
     $   istrff, delfrf, swihf, swihfpl, swihfmn, nf, nfpl, nfmn)
c     =====================================================
c
c  Get discrepancies for fulls only
c    (this routine is based on choutl)
c
c On entry:
c  nrs           number of observations
c  kanom         anomalous flag, = 0 no anomalous
c  lselds        if .true., test discrepancies only within datasets
c  icntrc        centric reflection if icntrc = 1, else = 0
c  jhkl(3)       reflection indices
c  ffrf(nrs)     intensity 
c  sdffrf(nrs)   sd(intensity)
c  wtrf          weight for this observation, eg 1/sd(I)**2
c                  (note that this is unscaled, so needs to be
c                   multiplied by ghl**2)
c  ghl(nrs)      inverse scale
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isttrf(nrs)   status flag
c  isymrf(nrs)   symmetry number
c  lanmtc(nrs)   pointer to matched pair (=0 if none)
c  sdrej         rejection limit for reflections measured more than twice
c  sdrej2        rejection limit for reflections measured twice
c  sdreja        rejection limit for reflections measured more than twice,
c                  including all observations (I+ & I-) (.le. 0 for no test)
c                  (not used if kanom = 0)
c  sdrej2a       rejection limit for reflections measured twice,
c                  including all observations (I+ & I-) (.le. 0 for no test)
c                  (not used if kanom = 0)
c  krej2         what to do with 2 observations which disagree
c                = -1   reject both
c                =  0   keep both
c                = +1   reject larger
c                = +2   reject smaller
c
c On exit
c  istrff(nrs)    updated status flags (initially copied from isttrf,
c                 then outliers (from full only) flagged)
c  delfrf(nrs)    weighted deviation = deviation/sigma (= 0.0 if none)
c  swihf          mean intensity (fulls only) (weighted)
c  swihfpl        mean intensity I+ (fulls only) (weighted)
c  swihfmn        mean intensity I- (fulls only) (weighted)
c  nf             number of observations in swihf
c  nfpl           number of observations in swihfpl
c  nfmn           number of observations in swihfmn
c
c  
      integer nrs, kanom, icntrc, nuse, iselds, krej2, nf, nfpl, nfmn
      integer jhkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs), isymrf(nrs),
     $   istrff(nrs), lanmtc(nrs)
      real  ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs),delfrf(nrs),
     $   sdrej, sdrej2, sdreja, sdrej2a, swihf, swihfpl, swihfmn
      logical lselds
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'flags.fh'
      include 'datasets.fh'
c Locals
      integer nrej, nrejpl, nkept, janom, idts, ndts
      real    sdih, uh, delarf(maxrfl)
      logical useflg(maxrfl), lkfull, lprint, lrejct
c
      call ccpzr(delfrf,nrs)
c Fulls only in "outlier" check
      lkfull = .true.
      lprint = .false.
      lrejct = .true.
c Copy status flags
      call ccpmvi(istrff, isttrf, nrs)
c
      if (lselds) then
         ndts = ndtsetsout
      else
         ndts = 1
      endif
      nuse = 0
c
      nf = 0
      nfpl = 0
      nfmn = 0
c
      do 10, idts = 1, ndts
         if (lselds) then
            iselds = idts
         else
            iselds = 0
         endif
c
         if (kanom .eq. 0 .or. icntrc .eq. 1) then
c  No anomalous
            janom = 0
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, istrff, isymrf, lanmtc, 
     $         swihf, nrej,  sdrej, sdrej2, krej2, lqoutl,
     $         delfrf, nkept)
            swihfpl = 0.0
            swihfmn = 0.0
            nuse = nuse + nkept
            nf = nkept
         elseif (kanom .ge. 1) then
c  Anomalous
            swihf = 0.0
            nrej = 0
c   I+
            janom = 1
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, istrff, isymrf, lanmtc,
     $         swihfpl, nrejpl, sdrej, sdrej2, krej2,  lqoutl,
     $         delfrf, nkept)
            nfpl = nkept
c   I-
            janom = 2               
            call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $         lkfull, jhkl, ffrf, 
     $         sdffrf, wtrf, ghl, irunrf, mrf, istrff, isymrf, lanmtc,
     $         swihfmn, nrej,  sdrej, sdrej2, krej2,  lqoutl,
     $         delfrf, nkept)
            nfmn = nkept
c   All together
            if (sdreja .gt. 0.0) then
               janom = 0
               call outlyr(lprint, lrejct, nrs, nuse, janom, iselds,
     $           lkfull, jhkl, ffrf, 
     $           sdffrf, wtrf, ghl, irunrf, mrf, istrff, isymrf, lanmtc, 
     $           swihf, nrej,  sdreja, sdrej2a, krej2, lqoutlan,
     $            delarf, nkept)
c   Then recalculate means of I+, I-
               janom = 1
               call chkusf(nrs, janom, istrff, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swihfpl, sdih, uh, nuse)
               janom = 2
               call chkusf(nrs, janom, istrff, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swihfmn, sdih, uh, nuse)
            else
c  Overall mean after rejections (if no overall test)
               janom = 0
               call chkusf(nrs, janom, istrff, isymrf, irunrf, mrf,
     $            irefrn, iselds, lkfull, useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, swihf, sdih, uh, nuse)
            endif
         endif
 10   continue
c
      return
      end
      subroutine fulprf(nrs, ffrf, sdffrf, ffprrf, sdfprf, mrf,
     $   imid, ipower)
c     =================================================
c
c  For fulls, select profile-fitted intensity if imid .lt. 0
c
      integer nrs, mrf(nrs), imid, ipower
      real  ffrf(nrs), sdffrf(nrs), ffprrf(nrs), sdfprf(nrs)
c
      integer l
c
      if (imid .lt. 0) then
         do 10, l = 1, nrs
            if (mrf(l) .eq. 0) then
               call selint(
     $         ffrf(l), sdffrf(l), ffprrf(l), sdfprf(l), mrf(l),
     $            imid, ipower)
            endif
 10      continue
      endif
      return
      end
c
c
c
      subroutine get_anl_imean(yd, wd, nd)
c     ===================================
c
c Subroutine to be called from FoxHolmes to return data points for scaling
c
c Retrieve Imeans over resolution bins for runs or batches,
c from array aimean genereated in s/r anlini
c
c On exit:
c  yd    observations for each group
c  wd    weights, = 1.0 unless no data, in which case = 0.0
c  nd    = number of parameters, = -1 for end of data

      real yd(*), wd(*)
      integer nd
c
      include 'parameter.fh'
      include 'sclinimeans.fh'
c      
      integer i
c
      kmncount = kmncount + 1
      if (kmncount .gt. nmnresmax) then
c  End, reset count for re-entry
         nd = -1
         kmncount = 0
        return
      endif
c
      do i = 1, nmngroup
         if (nimean(kmncount, i) .gt. 0) then
            yd(i) = aimean(kmncount, i)
            wd(i) = 1.0
         else 
c  No data
            yd(i) = 0.0
            wd(i) = 0.0
         endif
      enddo
      nd =  nmngroup
c
      return
      end

         
      subroutine getdgi(lsdref,nparcy,nrs,aih,uh,nref,
     $     ffrf,sdffrf,wtrf,irunrf,isttrf,ghl,dghldc,dshldg,dshldq,
     $     dshldp,dIhdpi)
c     ======================================================
c     
c     Calculate d Ih / dp(i) for each parameter
c     
c     
c     Returns:
c     dghldc        d ghl / dp(k), set = 0.0 for sd factors (others left)
c     dshldp(k,l)   partial derivative d sd(Ihl) / dp(k) for reflection l
c     dIhdpi(i)     d Ih / dp(i), for i'th global parameter
c     
      implicit none
c     
      include 'parameter.fh'
      include 'refcon.fh'
      include 'params.fh'
      include 'flags.fh'
      include 'inout.fh'
      include 'glocon.fh'
c     
      logical lsdref
      integer nparcy, nrs, nref, irunrf(nrs), isttrf(nrs)
      real aih, uh, ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs),
     $     dghldc(maxpmr, maxrfl), dshldg(maxrfl),
     $     dshldq(2*maxsdf,maxrfl),
     $     dshldp(maxpmr, maxrfl), dIhdpi(maxpar)
c     
      integer i,k,l,m
      integer irun
c     
c     
      call ccpzr(dIhdpi, nparcy)
c     
      do 40, l = 1,nrs
         if (isttrf(l) .ge. lqOK) then
            irun = irunrf(l)
c     Loop parameters for this run
c     Scale parameters
            do 45, k = 1,nprrun(irun)
c     ipoint is pointer from run parameter to global parameter
               i = ipoint(k, irun)
               if (dghldc(k,l) .ne. 0.0)
     $              dIhdpi(i) = dIhdpi(i) +
     $              wtrf(l)*dghldc(k,l)*(ffrf(l)-2.0*ghl(l)*aih) / uh
c     
c*+d  
               if (kprint(1) .ge. 4) then
c     Debug
                  write (lunout,'(a,i3,a,4g12.4)')
     $               ' Contribution to dIhdpi(',i,')',
     $               dIhdpi(i), dghldc(k,l),
     $               wtrf(l)*(ffrf(l)-2.0*ghl(l)*aih)/uh,
     $               wtrf(l)*dghldc(k,l)*(ffrf(l)-2.0*ghl(l)*aih)/uh
               endif
c*-d  
               if (lsdref) then
c     store d sd(Ihl) / dp = (d sd(Ihl) / d ghl) * (d ghl / dp)
                  dshldp(k,l) = dshldg(l) * dghldc(k,l)
c     Contribution from d sd(Ihl) / dp
                  dIhdpi(i) = dIhdpi(i) +
     $                 2.0 * (wtrf(l) / sdffrf(l)) * ghl(l) *
     $                 dshldp(k,l) *
     $                 (ghl(l) * aih - ffrf(l)) / uh
               else
                  dshldp(k,l) = 0.0
               endif
 45         continue
c     
c     SD parameters
            if (lsdref .and. npqrun(irun) .gt. 0) then
               do 48, k = 1,npqrun(irun)
                  m = k + nprrun(irun)
                  i = ipoint(m,irun)
                  dshldp(m,l) = dshldq(k,l)
c*+d  
                  if (kprint(1) .ge. 4) then
cc                     if (mod(nref,kprint(2)) .eq. 0) then
c     Debug
                        write (lunout,'(a,i3,a,2g12.4)')
     $                       ' SD Contribution to dIhdpi(',i,')',
     $                       dIhdpi(i),
     $                       2.0 * (wtrf(l) / sdffrf(l)) * ghl(l) *
     $                       dshldp(m,l) *
     $                       (ghl(l) * aih - ffrf(l)) / uh
c                     endif
                  endif
c*-d  
                  dIhdpi(i) = dIhdpi(i) +
     $                 2.0 * (wtrf(l) / sdffrf(l)) * ghl(l) *
     $                 dshldp(m,l) *
     $                 (ghl(l) * aih - ffrf(l)) / uh
c
c Clear d ghl / dp for sd factors
                  dghldc(m,l) = 0.0
 48            continue
            endif
c     
         endif
 40   continue 
c     
      return
      end
c
c
c
      subroutine getdtsnum(xname, dname,
     $   xname_list, dname_list, ndtsets,
     $   idtsnum)
c     ===================================
c
c  Identify dataset in list of datasets
c
c On entry:
c  xname         crystal name (blank if undefined)
c  dname         dataset name (may be "xname/dname")
c  xname_list    list of crystal names
c  dname_list    list of dataset names
c  ndtsets       number of datasets in list
c
c On exit:
c  idtsnum    Serial number for dataset in list
c             =  0 no datasets
c             = -1 not found
c
      integer ndtsets, idtsnum
      character xname*(*), dname*(*),
     $   xname_list(ndtsets)*(*), dname_list(ndtsets)*(*)
      integer i, nf
c
c
      if (ndtsets .eq. 0) then
         idtsnum = 0
         return
      endif
c
c Separate xname/dname if necessary
      call spltdname(xname, dname)
c
      nf = 0
      do i = 1, ndtsets
         if (dname .eq. dname_list(i)) then
            nf = nf+1
c    Only check xname if not blank
            if (xname .eq. ' ') then
               go to 10
            else
               if (xname .eq. xname_list(i)) go to 10
            endif
         endif
      enddo
c Accept if dname recognized uniquely & xname not given
      if (xname .eq. ' ' .and. nf .eq. 1) go to 10
c Not found
      idtsnum = -1
      return
c Found
 10   idtsnum = i
      return
      end

      subroutine gethkl(maxnrs, ninref, lrefst, kpflag, lselip,
     $   eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $   ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $   isymrf, sclirf, sdscrf, lflgrf, bgpkrf, frclrf, clprf, isttrf,
     $   timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $   nmprrf, datarf, istat)
c     ===================
c
c  Read & store all (possibly selected) observations for the next unique hkl
c
c  Observations are not stored if:-
c   - they come from excluded batches (irun .lt. 0)
c
c On entry:
c  maxnrs        dimension of reflection stores
c  ninref        current running total of raw input reflections (to be updated)
c  lrefst        if .true., store observations from reference run
c  kpflag        = -1   accept fulls only, return m = -1 if rejected
c                =  0   accept fulls & previously summed partials (mpart=10)
c                = +1   accept fulls & partials
c  lselip        if .true., select IPR or I into ffrf, sdffrf according
c                   to setting of IMID (see s/r chkref)
c  eof           .true. if starting to read file, ie a new record should
c                   be read rather than using stored record
c              
c On exit:     
c  ninref        updated  running total of raw input reflections
c  eof           .true. if end-of-file found (istat = 0 in this case)
c  jhkl(3)       reflection indices
c  ss            4(sin theta/lambda)**2
c  stol          (sin theta/lambda)**2 (= ss/4)
c  nrs           number of observations stored
c  nuse          number of "useful" ones
c  nfull         number of fully recorded observations
c  lpart         .true. if any partials found
c  iserrf(nrs)   serial number in file for this observation
c  ffrf(nrs)     intensity (from column I, summation intensity)
c  sdffrf(nrs)   sd(intensity) (from column SIGI, summation intensity)
c  ffprrf(nrs)   intensity (from column IPR, profile-fitted I, if present)
c  sdfprf(nrs)   sd(intensity) (from column SIGIPR, if present)
c  ibatrf(nrs)   batch number
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial depending on kpflag
c                 = 0 for previously summed partial (mpart = 10)
c  mprf(nrs)     partial flag: number of images etc (from MPART if present
c                 = mrf if no MPART column)
c                 = 10 summed partial
c                 .gt. 200, number of images * 100 + image number
c                 .gt. 20, number of images * 10 + image number
c  isymrf(nrs)   symmetry number
c  sclirf(nrs)   scale factor read from SCALE column in input file,
c                  = 1.0 if none
c  sdscrf(nrs)   sd(scale factor) read from SIGSCALE column in input file,
c                  = 0.0 if none
c  lflgrf(nrs)   flag column from file (if present, else = 0)
c  bgpkrf(nrs)   bg/pk ratios from file (if present, else = 0)
c  frclrf(nrs)   calculated fraction (if present in file, else = 0.0)
c  clprf(nrs)    LP factor if present in file, else = 0.0
c  isttrf(nrs)   status flag  (see flags.fh)
c                 = 0 conditionally OK
c                 .gt.0 unconditionally OK
c                 .lt.0 rejected
c  timerf(nrs)   normalized "time" coordinate
c  xdrf(nrs), ydrf(nrs), zdrf(nrs)
c                normalized 3D coordinates, depending on mode
c  xsirf(nrs)    xsi coordinate, radius from rotation axis (=sqrt(x**2 + y**2)
c                 (only calculated for mode 2 or TAILS correction)
c  phirf(nrs)    Phi coordinate from ROT column in file (=0.0 if absent)
c  xdetrf(nrs), ydetrf(nrs)
c                XDET, YDET values from input file
c  nmprrf(nrs)   number of parts contributing to this observation
c                initialized here to 1 except for previously summed partial
c                  (MPART = 10), in which case = 2
c  datarf(maxcol,nrs) copy of all input records, for output
c
c  istat         return status
c                 =  0   OK
c                 = +1   reflection outside resolution limits (only if all
c                        observations are outside limits)
c
c Arguments
      integer maxnrs, ninref, kpflag
      logical eof, lpart, lrefst, lselip
      real    ss, stol
      integer jhkl(3), nrs, nuse, nfull, istat
c     Reflection stores
      include 'parameter.fh'
      integer iserrf(maxnrs),
     $   ibatrf(maxnrs), irunrf(maxnrs), mrf(maxnrs), mprf(maxnrs),
     $   isymrf(maxnrs), lflgrf(maxnrs), isttrf(maxnrs), nmprrf(maxnrs)  
      real ffrf(maxnrs), sdffrf(maxnrs), ffprrf(maxnrs), sdfprf(maxnrs),
     $   sclirf(maxnrs), sdscrf(maxnrs), frclrf(maxnrs), clprf(maxnrs),
     $   timerf(maxnrs), xdrf(maxnrs), ydrf(maxnrs), zdrf(maxnrs),
     $   xsirf(maxnrs), phirf(maxnrs), xdetrf(maxnrs), ydetrf(maxnrs),
     $   bgpkrf(maxnrs),
     $   datarf(maxcol,maxnrs)
c
      include 'inout.fh'
      include 'rfile.fh'
      include 'flags.fh'
      include 'scales.fh'
      include 'column.fh'
      include 'rundef.fh'
      include 'sphhrm.fh'
      include 'timecor.fh'
c Locals
      integer i, m, mpart, isym, ibatch, irun, jserbt, lrflag, istatr,
     $   irfcnt, llreso, lxyflg
      real    ff, sdff, ffpr, sdffpr, xd, yd, zd, time, sclinp, clp,
     $   sdscli, frccal, dum1, dum2, dum3, phi, bgpk
      logical first, lmxref
      real s, data(maxcol)
c
      integer ihkl(3)
c
      logical hkleq
      external hkleq
c
      save first, ihkl, data, s, irfcnt
      data first /.true./
c
c
      if (eof) first = .true.
      if (first) then
c  reflection count in file
         irfcnt = 0
      endif
      istat  = 0
      nrs    = 0
      nuse   = 0
      lpart  = .false.
      nfull  = 0
      lmxref = .false.
      llreso = 0
      if (.not. first) then
         jhkl(1) = nint(data(1))
         jhkl(2) = nint(data(2))
         jhkl(3) = nint(data(3))
c  Skip read on entry (data preserved from previous entry)
         go to 25
      endif
c
c============================================================
c     Loop observations
c     
 20   call lrrefl(1, s, data, eof)
      irfcnt = irfcnt + 1
 25   if (.not. eof) then
c     Indices
         ihkl(1) = nint(data(1))
         ihkl(2) = nint(data(2))
         ihkl(3) = nint(data(3))
         if (first) then
            jhkl(1) = ihkl(1)
            jhkl(2) = ihkl(2)
            jhkl(3) = ihkl(3)
            first = .false.
         endif
      endif
c     
c     --- Is this a new reflection ?
      if (eof .or. .not.hkleq(ihkl,jhkl)) then
c Set resolution rejection status iff all observations outside limit
         if (llreso .eq. -1) istat = +1
c  Finished, so exit, even if nrs = 0 (no observations kept)
c Keep ihkl, s & data for next entry ("save" statement above)
         return
c        ======
      endif
c
c     --- Extract information for this reflection from reflection record
      call chkref(
     $   lselip, kpflag, s, data, m, mpart, isym, ff, sdff,
     $   ffpr, sdffpr, ibatch, irun, jserbt,  phi, xd, yd, zd,
     $   time, sclinp, sdscli, frccal,  clp, lrflag, bgpk, istatr)
c This observation outside resolution limits, keep track of whether
c all observations are outside limits or not
      if (istatr .eq. lqreso) then
         if (llreso .eq. 0) then
c   1st observation outside limits
            llreso = -1
         endif
      else
         llreso = +1
      endif
c
      ninref = ninref + 1
c     
      if (irun .eq. irefrn) then
c  Reference run, omit or include?
         if (.not. lrefst) go to 20
      endif
c Omit excluded batches & ranges 
      if (irun .lt. 0) go to 20
c
      if (m .lt. 0) then
c Rejected partial
         go to 20
      endif
      if (m .gt. 0) then
         lpart = .true.
      endif
c     
      nrs = nrs + 1
c Count good ones
      if (istatr .ge. lqprt .and. irun .gt. 0) nuse = nuse + 1
c Count fulls 
      if (m .eq. 0 .and. irun .gt. 0) nfull = nfull+1
c
      if (nrs .gt. maxnrs) then
         if (.not.lmxref)
     $        write (lunout, '(a,i8,a,3i4,a)')
     $        ' *** More than ', maxnrs,
     $        ' observations of reflection ',ihkl,' ***'
         nrs = maxnrs
         lmxref = .true.
         go to 20
      endif
c     --- --- Store next measurement of this reflection
      stol = 0.25*s
      ss   = s
      iserrf(nrs) = irfcnt
      ffrf(nrs)  = ff
      sdffrf(nrs) = sdff
      ffprrf(nrs) = ffpr
      sdfprf(nrs) = sdffpr
      irunrf(nrs) = irun
      ibatrf(nrs) = ibatch
      isttrf(nrs) = istatr
      mrf(nrs)    = m
      mprf(nrs)   = mpart
      isymrf(nrs) = isym
      sclirf(nrs) = sclinp
      sdscrf(nrs) = sdscli
      lflgrf(nrs) = lrflag
      bgpkrf(nrs) = bgpk
c  Number of images contributing to this observation
      if (mpart .eq. 10) then
c  Previously summed partial, comes from two images
         nmprrf(nrs) = 2
      else
c    but everything else starts as 1 image
         nmprrf(nrs) = 1
      endif
c     
      do 90, i = 1,ncol
         datarf(i,nrs) = data(i)
 90   continue
c     
      frclrf(nrs) = frccal
      clprf(nrs) = clp
c     Store 3D coordinate
      if (lsmode(irun) .eq. 1) then
c     mode = 1, detector frame, xy = detector coordinates, z = rotation (phi)
         call detcdn(
     $      xd, yd, zd, irun,
     $      xdrf(nrs), ydrf(nrs), zdrf(nrs), jserbt)
c  for diffuse correction, we need xsi and phi
            if (ltdsrn(irun)) then
               call difcdn(0,
     $            ihkl, isym, irun, ibatch, phi,
     $            xsirf(nrs), dum1, dum2, dum3, jserbt)
            endif
      elseif (lsmode(irun) .eq. 2 .or. lsmode(irun) .eq. 3) then
c     mode = 2 or 3,  beam coordinates in diffractometer or crystal frame
c       Three options:
c         1) SECONDARY: lxyflg = +1, mode 2, diffractometer frame
c             returned values are polar angles of secondary beam
c                x = theta (radians), y = phi (radians)
c         2) ABSORPTION: lxyflg = -1, mode 3, crystal frame
c             returned values are polar angles of secondary beam
c                x = theta (radians), y = phi (radians)
c         3) ANISOTROPIC: lxyflg = +2
c        xy = projection of scattering vector perpendicular to beam,
c           = y,z components of scattering vector
c
c         z = rotation (phi)
         if (nrnsph(irun) .gt. 0) then
            if (lsmode(irun) .eq. 2) then
               lxyflg = +1
            else
               lxyflg = -1
            endif
         else
            lxyflg = 2
         endif
         call difcdn(lxyflg,
     $      ihkl, isym, irun, ibatch, phi,
     $      xsirf(nrs), xdrf(nrs), ydrf(nrs), zdrf(nrs), jserbt)
      elseif (lsmode(irun) .eq. 4) then
c    Crystal frame, calculate polar coordinate (xd = theta, yd = phi)
c in permuted crystal frame with defined pole along z
         call crycdn(
     $      ihkl, isym, irun, ibatch, phi,
     $      xsirf(nrs), xdrf(nrs), ydrf(nrs), zdrf(nrs), jserbt)
      endif
      phirf(nrs)  = phi
      xdetrf(nrs) = xd
      ydetrf(nrs) = yd
c     
c     Store normalized time coordinate
      if (lbfrun(irun) .or. ktimcor .ne. 0) then
         call timcdn(time, irun, timerf(nrs))
         if (mbatsc(irun) .gt. 0) then
            timerf(nrs) = jserbt
         endif
      else
         timerf(nrs) = 0.0
      endif
c Range of resolution for each run
      if (istatr .ne. lqreso) then
         sminfr(irun) = min(sminfr(irun), s)
         smaxfr(irun) = max(smaxfr(irun), s)
      endif
      go to 20
c
      end
      SUBROUTINE GN3CMP
C                     ! inputs: goniostat angles
     1 (ANGLE1, ANGLE2, ANGLE3
C                     ! inputs: goniostat axes at datum position
     2 , AXIS1, AXIS2, AXIS3
C                     ! input: maximum possible number of derivatives
     3 , NDERMX
C                     ! input: required number of derivatives
     4 , NDER
C                     ! outputs: rotation matrix and derivatives         
     5 , MATS)
C
CEE Composes the rotation matrix for a 3-circle goniostat and 
C   any number of derivatives with respect to the angles of rotation.
C Goniostat_3_compose
Created:      14-NOV-1986   D.J.Thomas, MRC Laboratory of Molecular Biology,
C                                       Hills Road, Cambridge, CB2 2QH, England
C Goniostat_3_compose
C
C                     ! 1st goniostat angle (usually omega) / radians
      REAL     ANGLE1
C                     ! 2nd goniostat angle (usually kappa/chi) / radians
      REAL     ANGLE2
C                     ! 3rd goniostat angle (usually phi) / radians
      REAL     ANGLE3
C                     ! dummy variable in statement function
      INTEGER  ORDER
C                     ! ORDER of derivative of product matrix wrt ANGLE1
      INTEGER  ORDER1
C                     ! ORDER of derivative of product matrix wrt ANGLE2
      INTEGER  ORDER2
C                     ! ORDER of derivative of product matrix wrt ANGLE3
      INTEGER  ORDER3
C                     ! number of derivatives required
      INTEGER  NDER
C                     ! maximum possible number of derivatives
      INTEGER  NDERMX
C                     ! statement function
      INTEGER  PRNPAL
C Goniostat_3_compose
C
C                     ! 1st axis (usually omega) (right-handed unit-vector)
      REAL AXIS1 (1:3)
C                     ! 2nd axis (usually kappa/chi) (right-handed unit-vector)
      REAL AXIS2 (1:3)   
C                     ! 3rd axis (usually phi) (right-handed unit-vector)
      REAL AXIS3 (1:3)
C                     ! overall rotation matrix and requested derivatives
      REAL MATS (1:3, 1:3, 0:NDERMX, 0:NDERMX, 0:NDERMX)
C                          ! 1st angle rotation matrix and requested derivatives
      REAL MATS1 (1:3,1:3,0:4)
C                          ! 2nd angle rotation matrix and requested derivatives
      REAL MATS2 (1:3,1:3,0:4)
C                          ! 3rd angle rotation matrix and requested derivatives
      REAL MATS3 (1:3,1:3,0:4)
C                      ! statement function to cycle round derivatives correctly
      PRNPAL(ORDER) = 1+MOD(ORDER-1,4)
C
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE1, AXIS1, MIN(NDER,4), MATS1)
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE2, AXIS2, MIN(NDER,4), MATS2)
C                   ! make rotation matrix and requested derivatives (not > 4th)
      CALL RTMATS(ANGLE3, AXIS3, MIN(NDER,4), MATS3)
C
      DO 100 ORDER1 = 0, NDER
         DO 100 ORDER2 = 0, NDER-ORDER1
            DO 100 ORDER3 = 0, NDER-ORDER1-ORDER2
C
C! calculate all of the derivatives of the rotation matrix
C!  up to the requested ORDER
C
               CALL ML3MAT(     
     2                     3, MATS1(1,1,PRNPAL(ORDER1)),   
     3                     3, MATS2(1,1,PRNPAL(ORDER2)),
     4                     3, MATS3(1,1,PRNPAL(ORDER3)),
     5                     3, MATS(1,1,ORDER1,ORDER2,ORDER3))
100         CONTINUE
         CONTINUE
      CONTINUE
C
      RETURN
      END
      integer function howmny(n, ntok, ityp)
c     ======================================
c
c  Count number of numeric fields starting at n, ending at next keyword
c or end of line
c
c On entry:
c  n           1st field to test
c  ntok        number of fields on line
c  ityp(ntok)  field types, = 1 non-numeric
c
c On exit:
c  howmny      number of numeric fields including field n
c
      integer n, ntok, ityp(ntok)
c
      integer i, k
c
      do 1, i = n, ntok
         if (ityp(i) .eq. 1) then
            k = i - n
            go to 10
         endif
 1    continue
      k = ntok - n + 1
 10   howmny = k
      return
      end 
c
c
c
      subroutine hstprt(nrs, isttrf, frclrf, kmptrf,nmprrf, mrf)
c    =====================================================
c
c
c  Add into sums for histogram of total fraction for summed partials
c
c On entry:
c     nrs           number of entries
c     isttrf(nrs)   status flag
c     frclrf(nrs)   total calculated fraction for summed
c                         partial
c     kmptrf(nrs) for first part of summed partial
c              = +1  partial with consistent MPART flags
c              = -1  partial with inconsistent MPART flags
c              =  0  other: eg full, other parts of partials,
c                    no MPART flags etc
c     nmprrf(nrs) number of parts summed into this slot
c              = 1 for full
c              = 0 unmatched partial or 2nd & subsequent parts of partial
c                    (ie those flagged as lquprt)
c              .gt. 1 for summed partial (1st slot)
c              negated for scaled partial, = - number of parts
c     mrf(nrs)  partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial depending on kpflag
c                 = 0 for previously summed partial (mpart = 10)
c
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
      include 'stats.fh'
c
      integer nrs
      real frclrf(nrs)
      integer isttrf(nrs), kmptrf(nrs), nmprrf(nrs),
     $   mrf(nrs)
c
      integer i, j, k, l
c
      do l = 1, nrs
         if (mrf(l) .eq. 1) then
c Get total fraction bin
            do j = 2, nhstbin
               if (frclrf(l) .lt. hprtbn(j)) go to 10
            enddo
            j =  nhstbin
 10         j = j-1
c Class for sums (principal part of partial only)
c  1.  unmatched partial (rejected)
c        (ie isttrf = lqnprt)
c  2.  MPART flags OK (kmptrf = +1) (accepted)
c  3.  MPART flags not OK (kmptrf = -1), accepted
            k = 0
            if (isttrf(l) .eq. lqnprt) then
               k = 1
            elseif (isttrf(l) .ge. lqOK) then
               if (kmptrf(l) .eq. +1) then
                  k = 2
               elseif (kmptrf(l) .eq. -1) then
                  k = 3
               endif            
            endif
            if (k .gt. 0) then
               nunmfr(k, j) =  nunmfr(k, j) + 1
            endif
c         
c    Histogram of number of parts, summed partials only
            if (nmprrf(l) .gt. 0 .and. isttrf(l) .ge. lqOK) then
               i = min(iabs(nmprrf(l)), maxrfl)
               nhsprt(i) = nhsprt(i) + 1
            endif
         endif
c
      enddo
      return
      end
c
c
c
      subroutine hstset
c    ===============
c
c Set up & clear partiality histogram stores
c
c
      include 'parameter.fh'
      include 'stats.fh'
c
      integer n
      real p
c
c  Bins will run from pmin to pmax, with  interval dbin1
c  except between plim1 & plim1, where the bin interval is dbin2
      real pmin, pmax, plim1, plim2, dbin1, dbin2
      data pmin, pmax, plim1, plim2, dbin1, dbin2
     $    / 0.0, 1.4,   0.7,   1.2,   0.05,  0.01/
c
c   Set up partiality bins (n+1 limits for n bins)
      p = pmin
      n = 0
 1    if (p .le. pmax) then
         n = n+1
         if (n .gt. maxhbn) then
            call dna_error('internal error')
            call ccperr(1,'*** HSTSET: increase maxhbn ***')
         end if
         hprtbn(n) = p
         if (p .ge. plim1 .and. p .lt. plim2-0.5*dbin2) then
            p = p + dbin2
         else 
            p = p + dbin1
         endif
         go to 1
      endif
c
      nhstbin = n
c
      call ccpzi(nunmfr, 3*maxhbn)
      call ccpzi(nhsprt, maxrfl)
c
      return
      end

 
c
c
c
      subroutine htm_add(text, name)
c     =============================
c
c  Add table of contents entry
c
c On entry
c  text     link text
c  name     link name
c
c
      character*(*) text, name
c
      include 'parameter.fh'
      include 'chtml.fh'
c
      character label*80
      integer lenstr
      external lenstr
c
c
      label = '#'//name
      call ccp4h_toc_ent(text, label(1:lenstr(label)))
c
      return
      end
c
c
c
      subroutine htm_doc(text, dest)
c     ==============================
c
c  Echo command line as link to documentation
c but store up all echoes until end: print only whe htm_endkey is called
c  If buffer fills up, remaining lines are ignored
c
      character*(*) text, dest
c
      include 'parameter.fh'
      include 'chtml.fh'
      include 'htmbuf.fh'
c
      if (npthbf .lt. maxhtb) then
         npthbf = npthbf + 1
         dsthtm(npthbf) = dest
         bufhtm(npthbf) = text
      else
         npthbf = maxhtb + 1
      endif
      end
c
c
c
      subroutine htm_endgraph
c     ======================
c
c End Jgraph
c
      include 'parameter.fh'
      include 'chtml.fh'
c
      if (lchtml) call ccp4h_graph_end()
      end
c
c
c
      subroutine htm_endkey()
c     =====================
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'chtml.fh'
      include 'htmbuf.fh'
c
      integer i
      character key*80, target*80
      integer lenstr
      external lenstr
c
      if (lchtml) then
         call ccp4h_header(
     $      ' Input keyworded commands (click for documentation):',
     $      'commands',3)
c
         do i = 1, min(npthbf, maxhtb)
            key = dsthtm(i)
            target = 'scala.html#'//key
            call ccplwc(target)
            call ccp4h_link(
     $         key(1:lenstr(key)), target(1:lenstr(target)))
            write (lunout, '(a)') bufhtm(i)(1:lenstr( bufhtm(i)))
         enddo
         if (npthbf .gt. maxhtb) then
            write (lunout, '(/a/)')
     $         ' ==== Too many input lines to echo, some omitted ===='
         endif
      endif
      end
c
c
c
      subroutine htm_index()
c     =====================
c
      include 'parameter.fh'
      include 'chtml.fh'
      include 'inout.fh'
c
      if (lchtml) then
         call ccp4h_pre_end()
         write (lunout, '(a)') '</br>'
         call ccp4h_link('Input commands:',
     $      '#commands')
         write (lunout, '(a)') '</br>'
         call ccp4h_link('Table of contents of logfile:','#toc')
         call ccp4h_pre_beg()
      endif
      end
      subroutine htm_start()
c     =====================
c
      include 'parameter.fh'
      include 'chtml.fh'
      character envvar
c
      call ugtenv('CCP_SUPPRESS_HTML',envvar)
      lchtml=(envvar.eq.' ')

      if (lchtml) then
         call ccp4h_init()
         call ccp4h_pre_beg()
      endif
      end
c
c
c
      subroutine htm_stgraph
c     ======================
c
c Start Jgraph
c
      include 'parameter.fh'
      include 'chtml.fh'
c
      if (lchtml) call ccp4h_graph_beg(700,300)
      end
c
c
c
      subroutine htm_stkey()
c     =====================
c
      include 'parameter.fh'
      include 'chtml.fh'
      include 'htmbuf.fh'
c
c  Clear buffer pointer
      npthbf = 0
c
      end
c
c
c
      subroutine htm_wrthdr(text, name)
c     =================================
c
c  Write HTML header using KC's libhtml
c
c On entry
c  text     link text
c  name     link name
c
c
      character*(*) text, name
c
      include 'parameter.fh'
      include 'chtml.fh'
c
      integer i
c
      if (.not. lchtml) return
c
c
 10   call ccp4h_header(text, name, 3)
      return 
      end
c
c
c
      subroutine htm_wrttoc(lscale, lmerge)
c     ====================================
c
c  Write table of contents things
c
c On entry:
c  lscale    .true. if scaling done
c  lmerge    .true. if merging done
c
      logical lscale, lmerge
c
      include 'parameter.fh'
      include 'chtml.fh'
c
c
      if (.not. lchtml) return
c
      call ccp4h_pre_end()
      call ccp4h_toc_beg()
c
      if (lscale) call htm_add('Final scale factors','finalscale')
      if (lmerge) call htm_add('Scale factors by batch','batchscale')
      if (lmerge) call htm_add('Agreement by batch','batchagreement')
      if (lmerge) call htm_add('Agreement by resolution','resagreement')
      if (lmerge) call htm_add(
     $   'Agreement by intensity','intensityagreement')
      if (lmerge) call htm_add(
     $   'Completeness & multiplicity','cmpltmult')
      if (lmerge) call htm_add('Axial reflections','axes')
      if (lmerge) call htm_add(
     $   'Analysis of standard deviations','erroranalysis')
c
      call ccp4h_toc_end()
      call ccp4h_pre_beg()
      return
      end
c
c
c
c
      logical function isunmt(a)
c     ==========================
c
c Returns .true. is 3x3 matrix a is unit
c
      real a(3,3)
c
      integer i,j
      real tol, del
      logical fail
      data tol /0.00001/
c
      fail = .false. 
      do 10, j = 1,3
         do 11, i = 1,3
            if (i .eq. j) then
               del = abs(a(i,j) - 1.0)
            else
               del = abs(a(i,j))
            endif
            if (del .gt. tol) then
               fail = .true.
            endif
 11      continue
 10   continue 
      isunmt = .not. fail
      return
      end
      subroutine keyin
c     ================
c
      implicit none
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'nbtchc.fh'
      include 'rundef.fh'
      include 'refcon.fh'
      include 'rfile.fh'
      include 'column.fh'
      include 'scales.fh'
      include 'sdfacc.fh'
      include 'dump.fh'
      include 'tie.fh'
      include 'outcon.fh'
      include 'anomtc.fh'
      include 'flow.fh'
      include 'glocon.fh'
      include 'params.fh'
      include 'errors.fh'
      include 'means.fh'
      include 'chtml.fh'
      include 'sphhrm.fh'
      include 'datasets.fh'
      include 'timecor.fh'
      include 'sdcdata.fh'
c-harvest
      include 'sharvest.fh'
C Things for PARSER ----
      integer maxtok
      parameter (maxtok=100)
      character key*4,cvalue(maxtok)*4,line*5000
      integer ibeg(maxtok),iend(maxtok),ityp(maxtok),
     .     idec(maxtok),ntok
      real    fvalue(maxtok)
      logical lend, lpprnt
c
      integer maxpnt
      parameter (maxpnt=30)
c
c Locals
      integer llist, ibatch, kbatch, icryst, lflag, jsdcor, istat
      integer i,j,k,m,n, jj,
     $   ierr, ikey, irun, nrun, idum, mbatdf, mbatm,
     $   mode, nbfac, nbfps, i1, i2, lreslm, lexcld,
     $   lchexc(maxrun), kinisc, klink(3,maxrun), nlink,
     $   ktsphm, lspmax, lspcen, lhpole, klmarc, idtsnum,
     $   ndtskyin, kexcld
      real vlist(maxtok), resmin, resmax, time, primry, secnd1, secnd2,
     $   sdrfac(3,2), fractn, prblim, srmin, srmax, 
     $   tmpmin, tmpmax, tmpabs, tlimit, sdrej, sdrej2, 
     $   sdreja, sdrej2a, temp, tmppnt(2,maxpnt), prob,
     $   circen(2), cirrad, phlmarc(2), dtor, wdprmx
      logical exclude, lscldf, lspac, lsdset, lallbf, lanybf, ltimst, 
     +   lbatsl, lfinal, lbatmd, ltdscr, lcorrc, lrotst, lfixx,
     $   llmrct, lbfac, lsdet, lprful, lprscl(2), lprtst, lsubky,
     $   l1, l2, lnktds, lnksph, lioarc, ldsrun, lsclnrm, lsmooth
c
c  Groups
      integer ngroup, idtsgroup(maxgrp), ibgroup(2,maxgrp),
     $   ibgapgroup(maxgrp)
      real gapgroup(maxgrp)
c
      character pname*(maxnamelen),
     $   xname*(maxnamelen), dname*(maxnamelen),
     $   pname_run*(maxnamelen),
     $   xname_run*(maxnamelen), dname_run*(maxnamelen),
     $   xname_base*(maxnamelen), dname_base*(maxnamelen)
c
c Resolution by dataset
      real smindts(maxinsets), smaxdts(maxinsets)
      character res_dname(maxinsets)*(maxnamelen)
      integer numresdts
c
      integer modedf, nbfcdf, nbfpdf, mrdrun, jflprt,
     $   lspmxdf, lspcndf
      real timedf, primdf, sec1df, sec2df, partdi(maxtds), rmsres
      logical ltdsdf, lflprt(2), lrundefined
      character axlabl(3,2)*1
c
c Functions
      integer ccp4_hash_lookup, lenstr, howmny
      logical ccponl
      external ccp4_hash_lookup, ccponl, lenstr, howmny
c
c Commands
      integer nkeys
c-harvest-mod
      parameter (nkeys=50)
      character*16 keys(nkeys)
      character keyword*80
c
      data keys/'RUN','INTENSITIES','CYCLES','FILTER','RESOLUTION',
     $   'EXCLUDE','SCALES','PRINT','BINS','SMOOTHING','INITIAL',
     $   'REJECT','DUMP','NODUMP','RESTORE','SDCORRECTION',
     $   'TIE','UNTIE',
     $   'OUTPUT','INSCALE','TITLE','HISTORY','OVERLAPMAP','FINAL',
     $   'NOSCALE','SKIP','FIX','UNFIX','ANOMALOUS','WIDTH',
     $   'LINK','UNLINK','ANALYSE','ONLYMERGE','XYBINS','PARTIALS',
     $   'DAMP','PNAME','DNAME','NAME','BASE','NORMALISE','ACCEPT',
     $   'ZERODOSE','CORNERCORRECT',
     +   'PRIVATE','USECWD','RSIZE','NOHARVEST',
     +   'END'/
c
c Defaults
c  Scale layout: 
c     mode = 2 (diffractometer frame)
      data modedf/1/
c     smooth scaling (ie not Batch)
      data mbatdf/0/
c     B-factors on, isotropic
      data nbfcdf/1/, nbfpdf/1/
c     spacing of time points = 20
      data timedf/20.0/
c     primary rotation interval 5 degrees
      data primdf/+5.0/
c     number of detector points = 1 x 1
      data sec1df, sec2df/-1.0, -1.0/
c     diffuse correction off
      data ltdsdf /.false./
c     Spherical harmonic maximum order 6, non-centrosymmetric
      data lspmxdf /6/, lspcndf/1/
c     Axis labels
      data axlabl/'H','K','L','A','B','C'/
c
c- Harvest initialisations
      call sharvest_init
c
      dtor = atan(1.0) / 45.0
c Initializations
      lscldf = .false.
      lfinal = .false.
c Program flow
      lanlin = .true.
      lscale = .true.
      lanout = .true.
c Resolution limits
      smaxlm = smaxf
      sminlm = sminf
      numresdts = 0
c  Intensity limits
      lexcld = 0
      dfsdmn = +3.0
      dfsdmx = 1.0e10
      dfabmx = 1.0e20
c Intensity bins
c   default type WILSON 
      kbnrtp = 3
c   no mid-point set yet
      binmid = -1000000.
c Number of bins
      ninbin = maxinb
c   no average I
      avgins = -1000000.
c Set no runs, all batches to reject
      numrun = 0
c Flag for explicit run definition
      lrundefined = .false. 
      do 1, i=1,maxbat
         irunbt(i) = 0
 1    continue
      do 2, j=1,maxrun
c No ranges rejected
         nptrun(j)=0
c No initial scales given
         sclset(j) = -10.0
c  Intensity limits
         sdImin(j) = dfsdmn
         sdImax(j) = dfsdmx
         abImax(j) = dfabmx
         lchexc(j) = 0
c No averaging of scales
         lavpsc(j) = .false.
c Initially no links (but see klink default)
         nlnkrn(j) = 0
         itdspt(j) = j
c Set default resolution limits = file limits
         smin(j) = sminf
         smax(j) = smaxf
         sminfr(j) =  10000.
         smaxfr(j) = -10000.
c Rectangles & arcs on detector
         nrjrct(j) = 0
         nrjarc(j) = 0
c Clear flags for all anomalous match matrices
         kanmtc(j) = -1
c
         mhpole(j) = -1
c No links
         isphpt(j) = j
c  Default SD correction factors
         jsdfac(j) = -1
 2    continue
c E limits
      eacmax = 10.0
      ecnmax = -1.0
      emnprb = -1.0
c Initially no means
      lqmnir = .false.
c Weighting scheme for scaling: variance weighting
      kscwgtyp = 2
c
      mrjrct = 0
      mrjarc = 0
c
c No reference set
      irefrn = 0
c No parameter sds available
      lsdpars = .false.
c
c  Default use profile intensities (if present)
      imid = -1
      ipower = 3
c  Always use integrated I for scaled partials
      lsclpr = .false.
c No anomalous
      kanoms = 0
      kanomo = 0
      anmphd = 0.0
      anmtmd = 0.0
      lanmtcrn = .false.
c Corner correction
      lcorner = .false.
      cornerfile = 'CORNERCORRECT'
c
c=== Flags for usage of partials
c  Note that PARTIALS command sets CHECK, TEST, SCALE parameters for both
c scaling and merging steps
c
c No scaling of partials
      lprscl(1) = .false.
      lprscl(2) = .false.
c No selection of fulls/partials for both scaling and merging steps yet
      lprful = .false.
c
c-  (a) In scaling:-
c  Default do use partials, if fractioncalc is present
      if (ncfrcl .gt. 0) then
         kprsum = +1
      else
         kprsum =  0
      endif
c  If summing partials, do check on correct MPART flags if present
c  default value will be set at end if no values input, but set default
c  to CHECK if MPART column present
      if (ncmprt .eq. 0) then
         kprflg = +1000
      else
         kprflg = -1000
      endif
c Acceptance limits on total fraction
c   accept prlims -> prmaxs
      prlims = 0.95
c   maximum allow total fraction for scaling
      prmaxs = 1.05
c     threshold for scaling, none
      prthld = prlims
      lcorrc = .false.
c Maximum number of parts - set later to be equivalent to
c wdprmx degrees
      mxprwd = 1000
      wdprmx = 10.0
c Reject gaps
      kprmis =  0
c-  (b) in output
c   sum partials, check fractions if MPART present (as for scaling)
      kprout = +1
      kprflo = kprflg
c Acceptance limits on total fraction
c   accept prlims -> prmaxs
      prlimo = 0.95
c   maximum allow total fraction for merging
      prmaxo = 1.05
c   threshold for scaling, none
      prthou = prlimo
c Maximum number of parts
      mxprwo = mxprwd
c Accept gaps
      kprmso = +1
c===
c     
      kprint(1) = 1
      kprint(2) = 1
      kprint(3) = 0
c
      lreslm = -1
c
c No TAILS correction
      nprtds = 0
c No spherical harmonics
      nprsph = 0
      nplphi = mxsphi - 1
      npltheta = min(11, mxstheta)
      dplphi = 360./float(nplphi)
      dpltheta = 180./(float(npltheta - 1))
c  Initial values
      partdi(1) = 0.01
      partdi(2) = 0.0
      partdi(3) = 10
c  Store  values for resetting if go below minimum, or above maximum
      partdf(1,1) = 0.004
      partdf(1,2) = 0.0
      partdf(1,3) = 5.0
      partdf(2,1) = 0.1
      partdf(2,2) = 10.0
      partdf(2,3) = 80.0
c  Minimum & maximum values
      partmn(1,1) = 0.002
      partmn(1,2) = 0.0
      partmn(1,3) = 0.0
      partmn(2,1) = 0.1
      partmn(2,2) = 10.0
      partmn(2,3) = 100.0
c  TAILS parameters A1, V free, A0 fixed
      lptfix(1) = .false.
      lptfix(2) = .true.
      lptfix(3) = .false.
c
c  Rejection limits
c   scaling
      sdrejr  = 6.0
      sdrejr2 = 6.0
c   merging
      sdrejm  = 6.0
      sdrejm2 = 6.0
c    all including I+ & I-
      sdrejma = -8.0
      sdrejm2a=  0.0
c In output, what to do with 2 observations which disagree: keep both
      krej2o = 0
c
c  don't write overlap map
      lovmap = .false.
c
c Dump/restore
      dumpfl = 'SCALES'
      rstrfl = 'SCALES'
      ldump = .true.
      lrstr = .false.
c
c Apply scale column if present
      lappsc = .true.
c
c Time correction off
      ktimcor = 0
c
c Scale weights
      Vwt  = 0.5
      Vwz  = 1.0
      Vwxy = 1.0
      prblt = -1.0
      prblz = -1.0
      prblxy = -1.0
c
c Tie defaults
      nties = 0
      ntiexy = 0
      ntiez = 0
      ntiesrf = 1
      ntiebf = 0
      ntieA1 = -1
      vtiexy = 0.2**2
      vtiez  = 0.3**2
      vtiesrf = 0.001**2
      vtiebf = 0.3**2
      vtieA1 = 4.0**2
      do i = 1,maxrun
         ttieA1(i) = 30.0
      enddo
c
c Scale & Bfactor normalisation
      knormk = -1000
      knormb = -1000
c Check for link set
      nlink  = 0
      lnktds = .false.
      lnksph = .false.
c
c Filter
      filter = 1.0E-6
      lfilt1 = .true.
c Damp, default  no damp
      damp   = 0.0
      ncycdamp = 0
c
      nsrang = 10
      ncycle = 10
c Convergence as fraction of standard deviation
      cnvfac = 0.3
c 1st cycle for sd refinement, outlier rejection
      jsdcyc = 1000
      jolcyc = -1000
c
c  Initial scales from mean intensities
      iniscl = 0
c Options on initial analysis pass
      kinisc = 0
c
      lsdmod = .false.
c Don't adjust sdcorrection factors after normal probability analysis
      ladjsd = .false.
c  INDIVIDUAL SD factors to be refined
      ksdtype = +1
c  Refine SD factors
      lsdfac = .true.
c  Don't fix SdB
      fixSdB = .false.
c Default values for SD factors
      sdfacdef(1) = 1.0
      sdfacdef(2) = 0.0
      sdfacdef(3) = 0.02
c Do normal probability analysis
      lnormn = .true.
      maxptd = 25
c Write it to output file
      iwrprb = 8
      iwrpra = 9
      ispprb = 0
c Flag for analysing deviations & rejections by dataset
c Default is to include all datasets in calculation (REJECT COMBINE)
c REJECT SEPARATE does outlier checks only within datasets
      lanbyd = .false.
c Analysis by detector position
      lxyanl = .true.
      nbinxy(1) = 20
      nbinxy(2) = 20
c
c  Output file control: defaults
c     write averaged output file (AVERAGE)
c     exclude reference data
c     reject reflections outside resolution range
c             (set at end depending on ifilou)
c     omit outliers 
c     keep partials
c     write rogues file
c     reduced indices
c     no SCALEPACK output
c
      ifilou = -1
      lrefou = .false.
      krsexc = -1
      lolexc = .true.
      nprexc = 0
      lwrrog = .true.
      lrdidx = .true.
      mbeams = 0
      isclpk = 0
c OUTPUT SPLIT|TOGETHER option: initially -1000 to default later
      kfilou = -1000
c
c Global errors
      nerpdc = 0
      lerpdc = .true.
c ACCEPT on Mosflm flags
      ltsmfl = .false.
      bgrlim = -1.0
      pkrlim = -1.0
      grdlim = -1.0
      lovld = .false.
      ledge = .false.
c  Output title & history
      outitl = ' '
      histry = ' '
C Dataset initialisations
      ndtskyin = 0
      pname = ' '
      xname = ' '
      dname = ' '
c No base dataset defined (reset to -1 if defined on BASE command)
      kdtsbase = 0
c
      nrfskp = -1
      ncyskp = 0
c Flag to check if any run are BATCH mode (ie .true.)
      lbatmd = .false.
c Final analysis by batch rather than rotation
      lanlbt = .true.
c
      call htm_stkey()
      lpprnt = .true.
c
c  Set input error counter
 9    ierr=0
C
C
C ---- Loop to read control input
C
 10   line=' '
      ntok=maxtok
      call parser(
     .   key,line,ibeg,iend,ityp,fvalue,cvalue,idec,ntok,lend,lpprnt)
      if(lend) go to 1000
      if(ntok.eq.0) go to 10
      call ccpupc(key)
      call chkkey(key,keys,nkeys,ikey)
      if (ikey .gt. 0) then
c OK
         keyword =  keys(ikey)
      elseif (ikey .eq. 0) then
         write(lunout,fmt='(a/a)') line(1:lenstr(line)),
     $      ' ** Unrecognized keyword **'
         if(ccponl(idum)) then
            goto 10
         else
            ierr=ierr+1
         endif
         keyword = 'Unrecognized: '//line(1:iend(1))
      elseif (ikey .lt. 0) then
         write(lunout,fmt='(a/a)') line(1:lenstr(line)),
     $      ' ** Ambiguous keyword **'
         if(ccponl(idum)) then
            goto 10
         else
            ierr=ierr+1
         endif
         keyword = 'Ambiguous: '//line(1:iend(1))
      endif
c
c If HTML is switched on, echo command line as link to documentation
c Principal keyword is full length and lowercase for documentation,
c converted inside routine
      call htm_doc(line(iend(1)+1:), keyword)
c
c Echo line 
c
      do 11, i=1,ntok
         call ccpupc(cvalue(i))
 11   continue
c-harvest-mod
      go to (100,110,120,130,140,150,160,170,180,190,200,210,220,225,
     $     230,240,250,255,260,270,280,290,300,310,320,330,340,345,350,
     $     360,370,375,380,390,400,410,420,430,440,450,460,470,480,490,
     $     500,
     $     901,902,905,906,1000), ikey
c------------------------------------------------------------------------
c
c  == RUN == command
c  RUN nrun 
c     [ALL]       include all batches
c     [BATCH   | b1 b2 b3 ... | b1 TO b2 |]
c     [CRYSTAL xtal_name]   must preceed DATASET if given
c     [DATASET dataset_name or xtal_name/dataset_name]
c     [INCLUDE|EXCLUDE] set include/exclude flag
c     [RANGE  r1 TO r2 ] rotation range (include or exclude)
c     [REFERENCE] this run is the reference set
c
c Run definitions must precede SCALES definitions
 100  if (lscldf) then
         write (lunout, '(/a/)')
     $      ' **** RUN command must come before SCALES definition ****'
         ierr = ierr+1
      endif
c  Define a "run" : nrun is the external Run number (arbitrary);
c     irun is the internal number (1 -> numrun)
c     external number is stored as irunum(irun) = nrun
      nrun = -1
      call gttint(2,nrun,lflag,ntok,ityp,fvalue)
      if (lflag .ne. 0) go to 990
      if (numrun .gt. 0) then
c Is this extending existing run definition?
         do 108, i = 1, numrun
            if (nrun .eq. irunum(i)) then
               irun = i
               go to 107
            endif
 108     continue
      endif
c New run
      numrun = numrun+1
      lrundefined = .true. 
      if (numrun .gt. maxrun) then
         write (lunout, '(a,i4,a)')
     $        ' **** Too many runs defined, maximum = ', maxrun,
     $        ' ****'
         call dna_error('too many runs')
         call ccperr(1,'*** Too many runs ***')
      endif
      irun = numrun
      irunum(irun) = nrun
c Set default scale layout definitions for this run
      call setrsc(
     $     irun, modedf, nbfcdf, nbfpdf, timedf,
     $   primdf, sec1df, sec2df)
c Default flag for BatchMode
      mbatsc(irun) = mbatdf
      lbtslp(irun) = .false.
c  Packing factor for zd coordinate in sloped scaling
c     (see s/r detcdn, sclder)
      btslpf = 100.0
c
c Default to include 
 107  exclude = .false.
      n = 3
      xname_run = ' '
      dname_run = ' '
c Next keyword N
 109  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'CRYS') then
c     CRYSTAL keyword, followed by a crystal_name
            if (n .ge. ntok) then
               xname_run = ' '
            else 
               xname_run = line(ibeg(n+1):iend(n+1))
            endif
            n = n+2
         elseif (cvalue(n) .eq. 'DATA') then
c     DATASET keyword, followed by a dataset_name
            if (n .ge. ntok) then
               dname_run = ' '
            else 
               dname_run = line(ibeg(n+1):iend(n+1))
            endif
            n = n+2
c Identify input dataset number (serial number in input list)
            call getdtsnum(xname_run, dname_run,
     $         xname_in, dname_in, ndatasets, idtsnum)
            if (idtsnum .le. 0) then
               write (lunout, '(a)')
     $            ' ** Dataset not found or ambiguous **'
               go to 999
            endif
c  Set all batches in this dataset to be in this run (irun)
            do i = 1, nbtall
               if (jdtsbt(i) .eq. idtsnum) then
                  irunbt(i) = irun
               endif
            enddo
c
         else if (cvalue(n) .eq. 'REFE') then
c  Define reference set
            if (irefrn .ne. 0) then
               write (lunout, '(a)')
     $              ' ** You can only define one reference set **'
               go to 999
            endif
            irefrn = irun
c    One scale only for reference set
            call setrsc(irun, modedf, +0, 1, 0.0, -1.0, -1.0, -1.0)
c Default flag for BatchMode
            mbatsc(irun) = +1
            lbtslp(irun) = .false.
            n = n+1
c
         else if (cvalue(n) .eq. 'INCL') then
c  Set Include flag
            exclude = .false.
            n = n+1
         else if (cvalue(n) .eq. 'EXCL') then
c  Set Exclude flag
            exclude = .true.
            n = n+1
         else if (cvalue(n) .eq. 'RANG') then
c     RANGE keyword, rotation range for include/exclude
            if (nptrun(irun) .ge. maxlmr) then
               write (lunout, '(a,i6)') ' ** Too many Ranges: ',maxlmr
               go to 999
            endif
            n = n+1
            call lisran(ntok, n, cvalue, fvalue, ityp, llist, vlist)
            if (llist .ne. 0) then
               write (lunout, '(A)') ' ** Error in RANGE list **' 
               go to 999
            else if (llist .eq. 0) then
               nptrun(irun) = nptrun(irun)+1
               zmnrun(nptrun(irun), irun) = vlist(1)
               zmxrun(nptrun(irun), irun) = vlist(2)
c              exclude/include flag
               lptrun(nptrun(irun), irun) = exclude
            endif
c
         else if (cvalue(n) .eq. 'ALL') then
c     Keyword ALL, use all batches
c      this is discounted as an explicit run definition 
c      (lrundefined .false.)
            do 106, i = 1, nbtall
               irunbt(i) = irun
 106        continue
            lrundefined = .false.
            n = n+1
         else
c     BATCH keyword (optional), followed by batch list or range
            if (cvalue(n) .eq. 'BATC') then
               n = n+1
            endif
c  process list or range
            call  excinc(exclude, irun,
     $         ntok, n, cvalue, fvalue, ityp,
     $         llist, vlist, istat)
            if (istat .lt. 0) go to 999
         endif
         go to 109
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c  == INTENSITIES == command
c
c  INTEGRATED | PROFILE | PR_PART {***| COMBINE [imid POWER ipower]***}
c                 [[NO]ANOMALOUS]
c		 [PARTIALS [[NO]CHECK | 
c                        [NO]TEST [<lower_limit> <upper_limit>] |
c			CORRECT <minimum_fraction>] 
c                 [[NO]GAP]
c                 [MAXWIDTH  <maximum_width>] |
c                  SCALE_PARTIAL <minimum_fraction>]
c
c  Select which intensity to use in all operations:
c   INTEGRATED   integrated intensity I                          (imid=0)
c   PROFILE      profile-fitted intensity IPR                    (imid=-1)
c   PR_PART      profile IPR for fullys, integrated for partials (imid=-2)
c   COMBINE      weighted mean after combining partials
c       combine integrated intensity Iint and
c       profile-fitted intensity Ipr as
c         I = w * Ipr  +  (1-w) * Iint
c       where w = 1/(1+(Iint/IMID)**IPOWER)
c       IMID is the point of equal weight, and
c       should be about the mean intensity.
c   ANOMALOUS    keep I+ & I- reflections separate in the scaling
c
c  Define treatment of partials in scaling
c   FULLS        use fully recorded observations
c                     (& previously summed partials) only
c   ONLYFULLS    fulls only (no previously summed partials)
c   PARTIALS     use summed partials as well as fulls
c       [NO]CHECK 
c                do [not] check for consistency of MPART flags
c                (if present) 
c       [NO]TEST [<lower_limit> <upper_limit>]
c                 do [not] accept partials only if total fraction (from 
c                 FRACTIONCALC column) is in range lower_limit -> upper_limit
c                 [default if no MPART flag]
c       CORRECT [<minimum_fraction>]
c                 Scale partials in range minimum_fraction -> lower_limit,
c                 predicted total fraction (needs reliable FRACTIONCALC)
c       [NO]GAP   do [not] accept partials with a gap in, eg a partial
c                 over 3 parts with the middle one missing. GAP implies
c                 NOCHECK and TEST: CORRECT may also be set.
c       MAXWIDTH  <maximum_width> maximum number of parts for an
c                 acceptable summed partial
c
c   SCALE_PARTIALS  use scaled partials greater than Minimum_fraction
c	        in the scaling. Only use this if the FRACTIONCALC
c
 110  n = 2
 111  if (n .le. ntok) then
         if (cvalue(n) .eq. 'FULL') then
            kprsum =  0
         elseif (cvalue(n) .eq. 'PART') then
            kprsum = +1
         elseif (cvalue(n) .eq. 'ANOM') then
            kanoms = +1
         elseif (cvalue(n) .eq. 'NOAN') then
            kanoms =  0
         elseif (cvalue(n) .eq. 'ONLY') then
            kprsum = -1
         elseif (cvalue(n) .eq. 'CHEC') then
            kprflg = -min(iabs(kprflg), 10)
         elseif (cvalue(n) .eq. 'NOCH') then
            kprflg = +min(iabs(kprflg), 10)
         elseif (cvalue(n) .eq. 'NOTE') then
            kprflg = sign(+1, kprflg)
         elseif (cvalue(n) .eq. 'GAP') then
            kprmis = +1
         elseif (cvalue(n) .eq. 'NOGA') then
            kprmis =  0
         elseif (cvalue(n) .eq. 'TEST') then
            if (kprsum .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword TEST must follow PARTIAL'
               go to 999
            endif
            kprflg = sign(+2, kprflg)
            call gttrea(n+1, prlims, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
            call gttrea(n+1, prmaxs, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
         elseif (cvalue(n) .eq. 'CORR') then
            if (kprsum .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword CORRECT must follow PARTIAL'
               go to 999
            endif
            kprflg = sign(+2, kprflg)
            call gttrea(n+1, prthld, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
            lcorrc = .true.
         elseif (cvalue(n) .eq. 'MAXW') then
            if (kprsum .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword MAXWIDTH must follow PARTIAL'
               go to 999
            endif
            call gttint(n+1, mxprwd, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
         elseif (cvalue(n) .eq. 'SCAL') then
            lprscl(1) = .true.
            call gttrea(n+1, prthld, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then
c  A number read
               lcorrc = .true.
               n = n+1
            endif
c----
c-- profile-fitted|summation integrated observations, set imid flag
         elseif (cvalue(n) .eq. 'INTE') then
c     Integrated intensities
            imid = 0
         elseif (cvalue(n) .eq. 'PROF') then
c     Use profile-fitted intensity
            imid = -1
         else if (cvalue(n) .eq. 'PR_P') then
            imid = -2
         else if (cvalue(n) .eq. 'COMB') then
            imid = +1
            if (ntok .gt. n+1) then
               n = n+1
               if (ityp(n) .eq. +2) then
                  call gttint(n, imid, lflag, ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
               endif
               ipower = 3
               if (ntok .gt. n .and. cvalue(n) .eq. 'POWE') then
                  n = n+1
                  call gttint(n, ipower, lflag, ntok,ityp,fvalue)
                  if (lflag .gt. 0) go to 990
               endif
            endif
         endif
         if (imid .ne. 0) then
c  Asked to use profile-fitted intensities 
c     Is there a column for these?
            if (ncip .le. 0 .or. ncsdip .le. 0) then
c     No, so just use column I
               ncip = 0
               ncsdip = 0
               imid = 0
               write (lunout, '(/a,a)')
     $            ' ==== WARNING: no column IPR of profile-fitted ',
     $            'intensities available: intensity column I used ===='
            endif
         endif
         n = n+1
         go to 111
      endif
c If no CORRECT keyword given, set scale threshold = lower TEST limit
      if (.not.lcorrc .and. kprsum .eq. +1) then
         prthld = prlims
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c  == CYCLES == command
c     
c Define number of refinement cycles,
c     and convergence limit (multiple of sd(param))
c
c    [NUMBER]   maximum number of cycles
c     CONVERGE  convergence limit
c     SDFACTOR  1st cycle number for refinement of SD correction factors
c     REJECT    1st cycle number for rejection of outliers
c     WEIGHTS [UNIT|VARIANCE|SQRTSCALE]  weighting scheme
 120  n = 2
c  try to read the 2nd field as a number
      if (ityp(2) .eq. 2) then
         call gttint(2, ncycle, lflag, ntok,ityp,fvalue)
         if (lflag .eq. 0) then
            n = n+1
         endif
      endif
 121  if (n .le. ntok) then
         if (cvalue(n) .eq. 'NUMB') then
            call gttint(n+1, ncycle, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            if (ncycle .le. 0) lscale = .false.
         elseif (cvalue(n) .eq. 'CONV') then
            call gttrea(n+1, cnvfac, lflag, ntok,ityp,fvalue) 
            if (lflag .gt. 0) go to 990
         elseif (cvalue(n) .eq. 'SDFA') then
            call gttint(n+1, jsdcyc, lflag, ntok,ityp,fvalue) 
            if (lflag .gt. 0) go to 990
         elseif (cvalue(n) .eq. 'REJE') then
            call gttint(n+1, jolcyc, lflag, ntok,ityp,fvalue) 
            if (lflag .gt. 0) go to 990
         elseif (cvalue(n) .eq. 'WEIG') then
            if (cvalue(n+1) .eq. 'UNIT') then
               kscwgtyp = 1
            elseif (cvalue(n+1) .eq. 'VARI') then
               kscwgtyp = 2
            elseif (cvalue(n+1) .eq. 'SQRT') then
               kscwgtyp = 3
            else
               go to 980
            endif
         else
            go to 980
         endif
         n = n+2
         go to 121
      endif
c     
      go to 10
c
c------------------------------------------------------------------------
c
c  == FILTER == command
c Define filter level, & damp level
 130  call gttrea(2, filter, lflag, ntok,ityp,fvalue)
      if (lflag .gt. 0) go to 990
      call gttrea(3, damp, lflag, ntok,ityp,fvalue)
      if (lflag .gt. 0) go to 990
      if (lflag .eq. 0) then
c  Set to damp on all cycles
         ncycdamp = -1
      endif
      go to 10
c
c
c------------------------------------------------------------------------
c
c  == RESOLUTION == command
c  Set resolution limits, optionally by run or datasets
c  nrun returns = 0 if not set, = -1 if datasetname is set
c  If run is specified, MUST come after definition of specified run
 140  call rdresr(line,ibeg,iend,2,ityp,fvalue,cvalue,ntok,
     $     resmin,resmax,srmin,srmax,nrun,dname_run,istat)
      if (istat .lt. 0) then
         if (istat .eq. -1) go to 980
         if (istat .eq. -2) go to 990
      elseif (istat .eq. 0) then
c     
c     Reset unset values to current limit values (initially file values)
c     and limit to file values
         if (srmin .lt. 0.0) srmin = sminlm
         if (srmax .lt. 0.0) srmax = smaxlm
         srmin = max(srmin, sminf)
         srmax = min(srmax, smaxf)
c
c Set limits for each run, if set
c     lreslm is a flag as to what was done:-
c       = -1  unset
c       =  0  overall resolution limit only
c       = +1  individual run limits
         if (nrun .gt. 0) then
            if (numrun .gt. 0) then
               call chkrun(nrun, irun)
               if (irun .le. 0) then
                  go to 999
               endif
            else
               write (lunout, '(/a/)')
     $              ' **** No runs have been defined ****'
               go to 999
            endif
            i1 = irun
            i2 = irun
            lreslm = +1
         else if (nrun .eq. 0) then
            i1 = 1
            i2 = maxrun
            if (lreslm .lt. 0) lreslm = 0
c Update current overall limits, if run not specified
            sminlm = srmin
            smaxlm = srmax
         else if (nrun .lt. 0) then
c Resolution set for dataset, store for later use, after runs have
c been defined
            numresdts = numresdts + 1
            if (numresdts .gt. maxinsets) then
               write (lunout, '(/a/a,i6/)')
     $            ' **** Too many datasets in RESOLUTION ****',
     $            ' Maximum', maxinsets
               go to 999
            endif
            smindts(numresdts) = srmin
            smaxdts(numresdts) = srmax
            res_dname(numresdts) = dname_run
            lreslm = +1
         endif
c  Don't set resolution limits for dataset yet
         if (nrun .ge. 0) then
            do 144, irun = i1, i2
               smin(irun) = srmin
               smax(irun) = srmax
 144        continue
         endif
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c  == EXCLUDE == command
c     RUN <RunNumber> define run number (else applies to all runs)
c     SDMIN   minimum sd multiple for inclusion
c     SDMAX   maximum sd multiple for inclusion
c     ABSMAX  maximum absolute value
c     EMAX    <maximum E value (acentric)> | EPROB <minimum probability>
c     RECTANGLE <xmin> <xmax> <ymin> <ymax>
c             exclude rectangular area of detector
c     ARC     INSIDE | OUTSIDE x1,y1, x2,y2, x3,y3, ... xN,yN
c             exclude area of detector defined by arc
c             first & last points are x1,y1, x2,y2
c             INSIDE  exclude if radius .lt. arc
c             OUTSIDE exclude if radius .gt. arc
c     BATCH   | b1 b2 b3 ... | b1 TO b2 |
c
 150  n = 2
      kexcld = 0
      nrun  = -1
      tlimit = -1.0e10
      tmpmin = tlimit
      tmpmax = tlimit
      tmpabs = tlimit
      tmppnt(1,1) = -1.0e10
      llmrct = .false.
      klmarc = 0
      xname_run = ' '
      dname_run = ' '
c  Default ARC INSIDE
      lioarc = .true. 
 151  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'RUN') then
c  Define run number
c   read run number
            call gttint(n+1, nrun, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+2
c  Get internal run number irun
            if (numrun .gt. 0) then
               call chkrun(nrun, irun)
               if (irun .le. 0) then
                  go to 999
               endif
            else
               write (lunout, '(/a/)')
     $              ' **** No runs have been defined ****'
               go to 999
            endif
            kexcld = 2
         elseif (cvalue(n) .eq. 'SDMI') then
            call gttrea(n+1, tmpmin, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+2
            kexcld = 1
         else if (cvalue(n) .eq. 'SDMA') then
            call gttrea(n+1, tmpmax, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+2
            kexcld = 1
         else if (cvalue(n) .eq. 'ABSM') then
            call gttrea(n+1, tmpabs, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+2
            kexcld = 1
         else if (cvalue(n) .eq. 'RECT') then
            if (ntok .lt. n+4) go to 990
            call gttrea(n+1, tmppnt(1,1), lflag, ntok,ityp,fvalue)
            call gttrea(n+2, tmppnt(2,1), lflag, ntok,ityp,fvalue)
            call gttrea(n+3, tmppnt(1,2), lflag, ntok,ityp,fvalue)
            call gttrea(n+4, tmppnt(2,2), lflag, ntok,ityp,fvalue)
            n = n+5
            llmrct = .true.
            kexcld = 1
         else if (cvalue(n) .eq. 'ARC') then
            kexcld = 1
c  Next field may be INSIDE or OUTSIDE (= INSIDE if missing)
            if (ntok .lt. n+1) go to 990
            n = n+1
            if (cvalue(n) .eq. 'INSI') then
               lioarc = .true.
               n = n+1
            elseif (cvalue(n) .eq. 'OUTS') then
               lioarc = .false.
               n = n+1
            endif
c  At least 3 points needed
            k = ntok - n + 1
            klmarc = k/2
            if (klmarc .lt. 3 .or. k .ne. klmarc*2 .or.
     $         klmarc .gt. maxpnt) go to 990
            do i = 1,klmarc
               call gttrea(n,   tmppnt(1,i), lflag, ntok,ityp,fvalue)
               if (lflag .gt. 0) go to 990
               call gttrea(n+1, tmppnt(2,i), lflag, ntok,ityp,fvalue)
               if (lflag .gt. 0) go to 990
               n = n+2
            enddo
         else if (cvalue(n) .eq. 'EMAX') then
            call gttrea(n+1, eacmax, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+2
         else if (cvalue(n) .eq. 'EPRO') then
            call gttrea(n+1, emnprb, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+2
         else if (cvalue(n) .eq. 'NOEM') then
            eacmax = -1.0
            emnprb = -1.0
            n = n+1
         else if (cvalue(n) .eq. 'BATC') then
c   Exclude batch range
            n = n+1
            exclude = .true.
            irun = -1
c     set irunbt for excluded batches
            call  excinc(exclude, irun,
     $         ntok, n, cvalue, fvalue, ityp,
     $         llist, vlist, istat)
            if (istat .lt. 0) go to 999
         else if (cvalue(n) .eq. 'CRYS') then
c     CRYSTAL keyword, followed by a crystal_name
            xname_run = line(ibeg(n+1):iend(n+1))
            n = n+2
         else if (cvalue(n) .eq. 'DATA') then
c     DATASET keyword, followed by a dataset_name
            dname_run = line(ibeg(n+1):iend(n+1))
            n = n+2
c Identify input dataset number (serial number in input list)
            call getdtsnum(xname_run, dname_run,
     $         xname_in, dname_in, ndatasets, idtsnum)
            if (idtsnum .le. 0) then
               write (lunout, '(a)')
     $            ' ** Dataset not found or ambiguous **'
               go to 999
            endif
c  Set all batches in this dataset to be excluded
            do i = 1, nbtall
               if (jdtsbt(i) .eq. idtsnum) then
                  irunbt(i) = -1
               endif
            enddo
         else
            go to 980
         endif
         go to 151
      endif
c
      if (kexcld .gt. 0) then
         lexcld = max(lexcld, kexcld)
         if (nrun .lt. 0) then
            i1 = 1
            i2 = maxrun
         else
            i1 = irun
            i2 = irun
         endif
         if (klmarc .gt. 0) then
c Process arc data
            call mklarc(klmarc, tmppnt, circen, cirrad, rmsres, phlmarc)
            if (cirrad .lt. 0.0) then
               write (lunout, '(/a/)')
     $            '**** Cannot fit arc to points ****'
               go to 990
            endif
            write (lunout, '(a,f10.1)')
     $         '  Arc fitted to points with rms deviation', rmsres
         endif
         do 154, irun = i1, i2
            lchexc(irun) = 1
            if (tmpmin .gt. 0.9*tlimit) sdImin(irun) = tmpmin
            if (tmpmax .gt. 0.9*tlimit) sdImax(irun) = tmpmax
            if (tmpabs .gt. 0.9*tlimit) abImax(irun) = tmpabs
c Store rectangles
            if (llmrct) then
               nrjrct(irun) = nrjrct(irun) + 1
               rejrct(1,1,nrjrct(irun),irun) = tmppnt(1,1)
               rejrct(2,1,nrjrct(irun),irun) = tmppnt(2,1)
               rejrct(1,2,nrjrct(irun),irun) = tmppnt(1,2)
               rejrct(2,2,nrjrct(irun),irun) = tmppnt(2,2)
c mrjrct = 0, no rectangles,
c        = -1 same for all runs,
c        = +1 different for some runs
               if (mrjrct .le. 0) then
                  if (nrun .lt. 0) then
                     mrjrct = -1
                  else
                     mrjrct = +1
                  endif
               endif
            endif
c Store arcs
            if (klmarc .gt. 0) then
               nrjarc(irun) = nrjarc(irun) + 1
c   Arc centre
               rcrarc(1,nrjarc(irun),irun) = circen(1)
               rcrarc(2,nrjarc(irun),irun) = circen(2)
c   Arc radius
               rcrarc(3,nrjarc(irun),irun) = cirrad
c   Phi limits
               phiarc(1,nrjarc(irun),irun) = phlmarc(1)
               phiarc(2,nrjarc(irun),irun) = phlmarc(2)
c   Inside/outside flag
               linarc(nrjarc(irun),irun) = lioarc
c mrjarc = 0, no arcs,
c        = -1 same for all runs,
c        = +1 different for some runs
               if (mrjarc .le. 0) then
                  if (nrun .lt. 0) then
                     mrjarc = -1
                  else
                     mrjarc = +1
                  endif
               endif
            endif
 154     continue
      endif 
c
      go to 10
c
c------------------------------------------------------------------------
c
c  == SCALES == command
c   Define layout of scales (MUST come after definition of runs)
c  RUN  run_number          (else applies to all runs)
c  BATCH                 set to "batch" mode, no interpolation along
c                         rotation axis
c  SLOPE                 "sloped" batch mode (implies BATCH)
c  SMOOTH  <spacing>     smoothed batch mode:
c                          treat batch number as rotation angle
c                           <spacing> is number of batches to
c                          smooth over
c  BFACTOR ON(ISOTROPIC)|ANISOTROPIC|OFF
c                        switch Bfactors on or off
c                        Anisotropic option forces DIFFRACTOMETER mode
c  TAILS [<v>] [<a0>] [<a1>]
c                         correction for diffuse scattering (reflection tails),
c                         must use with summed partials.
c                         The parameters are the half-width of the tails
c                         in A**-1, and the slope against
c                         (sin theta/lambda)**2 of the fraction of total
c                         intensity in the diffuse peak 
c
c   BROTATION|TIME, ROTATION, DETECTOR keywords specify number of scales or
c        spacing on this coordinate (if SPACING keyword present)
c  BROTATION|TIME    [SPACING]  Ntime|delta_time [time defaults to rotation]
c  ROTATION    [SPACING]  Nscales|delta_rotation
c  DETECTOR  [SPACING]  Nscales_Xdet [Nscales_Ydet] | delta_Xdet [delta_Ydet]
c  CONSTANT  just one scale for each run (as in reference batch)
c  SECONDARY|ABSORPTION|SURFACE   <Lmax>
c                      spherical harmonic expansion in secondary
c                      beam, crystal or HKL frame. Lmax is maximum order of
c                      spherical harmonics Yml
c  CENTROSYMMETRIC     make spherical harmonic correction centrosymmetric
c  POLE <h|k|l|a|b|c>  define polar axis for crystal (SURFACE) frame
c                      defaults: (a) closest to rotation axis if known
c                                (b) k for monoclinic spacegroups,
c                                    l for others
c
c Hidden options:-
c  MODE DETECTOR|DIFFRACTOMETER|CRYSTAL|SURFACE
c                        define Detector, Diffractometer or Crystal frame
c  
c
 160  nrun  = -1
      mode  = modedf
      mbatm = mbatdf
      lbatsl= .false.
      nbfac = nbfcdf
      lbfac = .false.
      nbfps = nbfpdf
      time   = timedf
      lrotst = .false.
      ltimst = .false.
      primry = primdf
      secnd1 = sec1df
      secnd2 = sec2df
      ltdscr = ltdsdf
      lsdet  = .false.
      ktsphm = 0
      lspmax = lspmxdf
      lspcen = lspcndf
      lhpole = -1
      lsmooth = .false.
      n = 2
c Next keyword N
 161  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'RUN') then
c  Define run number
c   read run number
            call gttint(n+1, nrun, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+2
c  Get internal run number irun
            if (numrun .gt. 0) then
               call chkrun(nrun, irun)
               if (irun .le. 0) then
                  go to 999
               endif
            else
               write (lunout, '(/a/)')
     $              ' **** No runs have been defined ****'
               go to 999
            endif
c
         else if (cvalue(n) .eq. 'BATC') then
            if (ltimst) then
c  BROTATION|TIME already set
               mbatm = -1
            elseif (lrotst) then
c  ROTATION set, convert that to BROTATION
               mbatm = -1
               time = primry
               ltimst = .true.
               lrotst = .false.
            else
               mbatm = +1
            endif
            n = n+1
         else if (cvalue(n) .eq. 'SLOP') then
            mbatm = +1
            lbatsl= .true.
            n = n+1
         else if (cvalue(n) .eq. 'TAIL') then
            ltdscr = .true.
            HLdiffuse = .true.
c  read optional initial values for v (width in degrees), alpha0, alpha1
            n = n+1
            if (ityp(n) .eq. 2) then
               call gttrea(n, partdi(1), lflag, ntok,ityp,fvalue)
               if (lflag .eq. 0) then
                  n = n+1
                  if (ityp(n) .eq. 2) then
                     call gttrea(n, partdi(2), lflag, ntok,ityp,fvalue)
                     if (lflag .eq. 0) then
                        n = n+1
                        if (ityp(n) .eq. 2) then
                           call gttrea(
     $                        n, partdi(3), lflag, ntok,ityp,fvalue)
                           if (lflag .eq. 0) then
                              n = n+1
                           endif
                        endif
                     endif
                  endif
               endif
            endif
         else if (cvalue(n) .eq. 'MODE') then
            if (cvalue(n+1) .eq. 'DETE') then
               mode = 1
            else if (cvalue(n+1) .eq. 'DIFF') then
               mode = 2
            else if (cvalue(n+1) .eq. 'CRYS') then
               mode = 3
            else if (cvalue(n+1) .eq. 'SURF') then
               mode = 4
            else
               write (lunout,'(A,A,A)') ' **** MODE keyword ',
     $              cvalue(n+1),' not recognized ****'
               go to 999
            endif
            n = n+2
c
         else if (cvalue(n) .eq. 'BFAC') then
            if (cvalue(n+1) .eq. 'ON') then
               nbfac = 1
            else if (cvalue(n+1) .eq. 'ISOT') then
               nbfac = 1
            else if (cvalue(n+1) .eq. 'OFF') then
               nbfac = 0
            else if (cvalue(n+1) .eq. 'ANIS') then
               nbfac = 1
               nbfps = 3
c      force DIFFRACTOMETER mode if anisotropic
               mode = 2
               HLaniso = .true.
            else
               write (lunout,'(A,A,A)') ' **** BFACTOR keyword ',
     $              cvalue(n+1),' not recognized ****'
               go to 999
            endif
            lbfac = .true.
            n = n+2
c
         else if (cvalue(n) .eq. 'TIME' .or.
     $            cvalue(n) .eq. 'BROT') then
            if (mbatm .gt. 0) then
c  BATCH already specified, set smooth scaling on Bfactor
               mbatm = -1
            else
c    else smooth on both k & B
               mbatm =  0
            endif
            n = n+1
            nbfac = 1
            if (cvalue(n) .eq. 'SPAC') then
c  Define spacing rather than number
               lspac = .false.
               n = n+1
            else
               lspac = .true.
            endif
            call gttrea(n, time, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
c    treat time 0 == Bfactor off
            if (time .eq. 0.0) then
               nbfac = 0
               lbfac = .true.
            else if (lspac) then
               time = -time
            endif
            ltimst = .true.
            n = n+1
c
         else if (cvalue(n) .eq. 'ROTA') then
            if (mbatm .gt. 0) then
c  if BATCH already specified treat this as if BROTATION
               mbatm = -1
            endif
            n = n+1
            if (cvalue(n) .eq. 'SPAC') then
c  Define spacing rather than number
               lspac = .false.
               n = n+1
            else
               lspac = .true.
            endif
            call gttrea(n, temp, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            if (lspac) temp = -temp
            if (mbatm .ne. 0) then
               time = temp
               ltimst = .true.
            else
               primry = temp
               lrotst = .true.
            endif
            n = n+1
c
         else if (cvalue(n) .eq. 'DETE') then
            n = n+1
            if (cvalue(n) .eq. 'SPAC') then
c  Define spacing rather than number
               lspac = .false.
               n = n+1
            else
               lspac = .true.
            endif
            call lisran(ntok, n, cvalue, fvalue, ityp, llist, vlist)
c    1 or 2 numbers
            if (llist .gt. 2) go to 990
            if (llist .lt. 1) then
               secnd1 = 1
            else
               secnd1 = vlist(1)
            endif 
            if (llist .le. 1) then
               secnd2 = secnd1
            else
               secnd2 = vlist(2)
            endif
c     Flag lsdet set .true. if more than one detector area defined:
c     if "spacing" , we can't easily tell here whether more there is than
c one area, so assumed there are
            if (lspac) then
               secnd1 = -secnd1
               secnd2 = -secnd2
               lsdet = .true.
            else
               if (secnd1*secnd2 .gt. 1.01) then
                  lsdet = .true.
               endif
            endif
         else if (cvalue(n) .eq. 'CONS') then
            n = n+1
            primry = -1.0
            secnd1 = -1.0
            secnd2 = -1.0
            nbfac = 0
            lbfac = .true.
c SECONDARY or SURFACE options, spherical harmonic expansions
         else if (cvalue(n) .eq. 'SECO' .or. cvalue(n) .eq. 'ABSO'
     $         .or. cvalue(n) .eq. 'SURF') then
            if (cvalue(n) .eq. 'SECO') then
               mode = 2
               ktsphm = 1
            else if (cvalue(n) .eq. 'ABSO') then
               mode = 3
               ktsphm = 2
            else if (cvalue(n) .eq. 'SURF') then
               mode = 4
               ktsphm = 3
            endif
            n = n+1
c Read maximum order, if present
            if (n .le. ntok) then
               if (ityp(n) .eq. 2) then
                  call gttint(n, lspmax, lflag, ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
               endif
            endif
         else if (cvalue(n) .eq. 'CENT') then
            lspcen = 2
         else if (cvalue(n) .eq. 'NONC') then
            lspcen = 1
         else if (cvalue(n) .eq. 'POLE') then
c Set pole for crystal frame (SURFACE)
c  accept H,K,L,A,B,C  (cvalue already uppercased here) (allow a* etc)
            n = n+1
            if (n .gt. ntok .or. (iend(n)-ibeg(n)) .gt. 1) then
               go to 990
            endif
            do j = 1,2
               do i = 1,3
                  if (cvalue(n)(1:1) .eq. axlabl(i,j)) go to 168
               enddo
            enddo
            go to 990
 168        lhpole = i
            n = n+1
c
         else if (cvalue(n) .eq. 'SMOO') then
c SMOOTH
            mbatm = 0
            n = n+1
c  Always define spacing rather than number
            if (cvalue(n) .eq. 'SPAC') then
               n = n+1
            endif
            call gttrea(n, temp, lflag, ntok,ityp,fvalue)
            primry = temp
            lsmooth = .true. 
            n = n+1
         else
            go to 980
         endif
         go to 161
      endif
c
c Is this batch mode? Set flag for at least one RUN in BATCH mode
      if (mbatm .ne. 0) lbatmd = .true.
c Store things
      if (nrun .lt. 0) then
         i1 = 1
         i2 = maxrun
      else
         i1 = irun
         i2 = irun
      endif
      if (.not. ltimst .and. nct .eq. 0) then
c If TIME flag not set explicitly, and rotation value is used as time,
c     then copy time setting from rotation setting
         time = primry
      endif
c Set nbfac flag if not set explicitly
      if (.not. lbfac) then
         if (lsdet) then
c   by default, bfactor off if more than one scale across the detector
c  Bfactor on may still be set explicitly, but a warning will be printed
c in s/r setscl
            nbfac = 0
         else
            nbfac = nbfcdf
         endif
      endif
c
      do 164, irun = i1, i2
         if (irun .eq. irefrn) then
c  Force reference run to CONSTANT            
            call setrsc(
     $         irun, modedf, 0, nbfpdf, timedf, -1.0, -1.0, -1.0)
            mbatsc(irun) = 0
            lbtslp(irun) = .false.
            lbtsmooth(irun) = .false.
            ltdsrn(irun) = .false.
            ltdsrn(irun) = .false.
            nrntds(irun) = 0
            nrnsph(irun) = 0
         else 
            call setrsc(
     $         irun, mode, nbfac, nbfps, time, primry, secnd1, secnd2)
            mbatsc(irun) = mbatm
            lbtslp(irun) = lbatsl
            lbtsmooth(irun) = lsmooth
            ltdsrn(irun) = ltdscr
            if (ltdscr) then
               ltdsrn(irun) = .true.
c--- Check valid TDS parameters
               if (partdi(1) .le. 0.00001) then
                  partdi(1) = 0.01
                  write (lunout, '(a,f8.3,a)')
     $               ' *** Initial Tails parameter V reset to ',
     $               partdi(1), ' ****'
               endif
               if (partdi(2) .lt. -0.001) then
                  partdi(2) = 0.0
                  write (lunout, '(a,f8.3,a)')
     $               ' *** Initial Tails parameter A0 reset to ',
     $               partdi(2), ' ****'
               endif
               if (partdi(3) .le. 0.0) then
                  partdi(3) = 10.0
                  write (lunout, '(a,f8.1,a)')
     $               ' *** Initial Tails parameter A1 reset to ',
     $               partdi(3), ' ****'
               endif
            
c Set initial values
               do 165, i = 1, maxtds
                  partds(i,irun) = partdi(i)
 165           continue
c Store A1 (parameter 3) as target for possible TIE A1
               ttieA1(irun) = partds(3,irun)
c
               nrntds(irun) = maxtds
c   just a flag for now, counted later
               nprtds = +1
            else
               ltdsrn(irun) = .false.
               nrntds(irun) = 0
            endif
c--- Spherical harmonic stuff
            if (ktsphm .eq. 0) then
               nrnsph(irun) = 0
            else
c   initially no links
               isphpt(irun) = irun
c   maximum order
               lmxsph(irun) = lspmax
c   centrosymmetric flag
               lodsph(irun) = lspcen
c   pole for hkl
               mhpole(irun) = lhpole
c  Number of parameters: don't count first constant 1.0 coefficient
               nrnsph(irun) =
     $            (lspmax + 1)*((lspmax + lspcen)/lspcen) - 1
c  Overall flag (counted later)
               nprsph = +1
            endif 
         endif
c       
 164  continue
      lscldf = .true.
      go to 10
c
c------------------------------------------------------------------------
c
c   == PRINT ==
c     define amount of printing
c     NONE    almost none
c     BRIEF   some more (default)
c     CYCLES  more information about each minimization cycle
c     FULL    quite a lot
c     DEBUG  [reflection_interval]   far too much: also define
c            reflection interval for printing
c     ALLOVERLAP  full print of overlap matrix
c     OVERLAP      print  overlap matrix
c     NOOVERLAP    no  print of overlap matrix
 170  n = 2
      kprint(1) = 2
c
 172  if (n .le. ntok) then
         if (cvalue(n) .eq. 'NONE') then
            kprint(1) = 0
         elseif (cvalue(n) .eq. 'BRIE') then
            kprint(1) = 1
         elseif (cvalue(n) .eq. 'CYCL') then
            kprint(1) = 2
         elseif (cvalue(n) .eq. 'FULL') then
            kprint(1) = 3
         elseif (cvalue(n) .eq. 'DEBU') then
            kprint(1) = 4
            kprint(2) = 1
            call gttint(n+1, kprint(2), lflag, ntok,ityp,fvalue)
            if (lflag .eq. 0) n = n+1
         elseif (cvalue(n) .eq. 'ALLO') then
            kprint(3) = +2
         elseif (cvalue(n) .eq. 'OVER') then
            kprint(3) = +1
         elseif (cvalue(n) .eq. 'NOOV') then
            kprint(3) = 0
         endif
         n = n+1
         go to 172
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == BINS ==
c   Number of resolution bins
 180  call gttint(2, nsrang, lflag, ntok,ityp,fvalue)
      nsrang = min(max(nsrang,1), maxres)
      go to 10
c
c------------------------------------------------------------------------
c
c   == SMOOTHING ==
c     Set smoothing factors
c
c     Subkeywords: TIME, ROTATION, DETECTOR   smoothing factors
c                  PROB_LIMIT [3 numbers]
 190  n = 2
 191  if (n .le. ntok) then
         if (cvalue(n) .eq. 'TIME') then
            call gttrea(n+1, vwt, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         elseif (cvalue(n) .eq. 'ROTA') then
            call gttrea(n+1, vwz, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         elseif (cvalue(n) .eq. 'DETE') then
            call gttrea(n+1, vwxy, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         elseif (cvalue(n) .eq. 'PROB') then
            call gttrea(n+1, prblt, lflag, ntok,ityp,fvalue)
            call gttrea(n+2, prblz, lflag, ntok,ityp,fvalue)
            call gttrea(n+3, prblxy, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+2
         else
            go to 980
         endif
         n = n+2
         go to 191
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == INITIAL ==
c      Choose initial scales
c     MEAN   from mean intensities by rotation range (default)
c     UNITY  set all scales = 1.0
c     RUN <RunNumber>  <InitialScale>  set initial scale factor
c     NONE   skip initial pass
c     ANALYSE force initial analysis pass even if RESTORE is used
c
 200  n = 2
 201  if (n .le. ntok) then
         if (cvalue(n) .eq. 'MEAN') then
            if (iniscl .eq. 2) then
               write (lunout, '(a,a)')
     $              ' **** INITIAL MEAN option incompatible with ',
     $              'initial scale factor for RUN ****'
               go to 999
            endif
            iniscl = 0
         elseif (cvalue(n) .eq. 'NONE') then
            kinisc = -1
            iniscl = 1
         elseif (cvalue(n) .eq. 'ANAL') then
            kinisc = +1
         elseif (cvalue(n) .eq. 'UNIT') then
            if (iniscl .eq. 2) then
               write (lunout, '(a,a)')
     $              ' **** INITIAL MEAN option incompatible with ',
     $              'initial scale factor for RUN ****'
               go to 999
            endif
            iniscl = 1
         elseif (cvalue(n) .eq. 'RUN') then
c   read run number
            call gttint(n+1, nrun, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+2
c  Get internal run number irun
            if (numrun .gt. 0) then
               call chkrun(nrun, irun)
               if (irun .le. 0) then
                  go to 999
               endif
            else
               write (lunout, '(/a/)')
     $              ' **** No runs have been defined ****'
               go to 999
            endif
            call gttrea(n, sclset(irun), lflag,ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            if (sclset(irun) .le. 0.0) then
               write (lunout, '(/a/)')
     $              ' **** Input scale must be POSITIVE ****'
               go to 999
            endif
            iniscl = 2
         else
            go to 980
         endif
         n = n+2
         go to 201
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == REJECT ==
c    Outlier rejection criteria
c    [SCALE|MERGE]  use these values for scaling|merging steps
c              if not specified, use for both
c    [COMBINE] compare observations across all datasets [default]
c    [SEPARATE]  outlier checks only within datasets
c     sdrej    sd multiplier for maximum deviation from scale-weighted mean I
c    [sdrej2]  special value for reflections measured twice
c    [REJECT|KEEP|SMALLER|LARGER]] these flags control what to do in the 
c              merging stage if there are two observations which disagree
c              In scaling, both are always rejected.
c              REJECT      reject both
c              KEEP        keep both [default]
c              LARGER      reject larger
c              SMALLER     reject smaller
c    [ALL sdreja [sdrej2a]]
c              if ALL is present and the ANOMALOUS option is on, then
c              test all data (ie all I+ & I-) values against these limits
c              after testing within the I+ & I- set
c              NOALL (== ALL 0) switches off this test
c              Only applies to merging step (scaling step checks all anyway)
c              
c
 210  n = 2
      k = 0
      j = 1
      m = 1
 211  if (n .le. ntok) then
         if (cvalue(n) .eq. 'SCAL') then
            k = 1
            j = 1
         elseif (cvalue(n) .eq. 'MERG') then
            k = 2
            j = 1
         elseif (cvalue(n) .eq. 'COMB') then
            lanbyd = .false.
         elseif (cvalue(n) .eq. 'SEPA') then
            lanbyd = .true.
         elseif (cvalue(n) .eq. 'REJE') then
            krej2o = -1
         elseif (cvalue(n) .eq. 'KEEP') then
            krej2o =  0
         elseif (cvalue(n) .eq. 'LARG') then
            krej2o = +1
         elseif (cvalue(n) .eq. 'SMAL') then
            krej2o = +2
         elseif (cvalue(n) .eq. 'ALL') then
            m = 2
            j = 1
         else
            if (m .eq. 1) then
c   Normal limits (within I+ & I-)
               if (j .eq. 1) then
                  call gttrea(n, sdrej, lflag, ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  sdrej2 = sdrej
                  j = 2
c  set sdrejr[2] for scaling, and|or sdrejm[2] for merging
                  if (k .le. 1) then
                     sdrejr  = sdrej
                     sdrejr2 = sdrej2
                  endif
                  if (k .ne. 1) then
                     sdrejm  = sdrej
                     sdrejm2 = sdrej2
                  endif
               else
                  call gttrea(n, sdrej2, lflag, ntok,ityp,fvalue)
                  if (k .le. 1) then
                     sdrejr2 = sdrej2
                  endif
                  if (k .ne. 1) then
                     sdrejm2 = sdrej2
                  endif
               endif
            else
c   Limits over all all including I+ & I-
               if (j .eq. 1) then
                  call gttrea(n, sdreja, lflag, ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  sdrej2a = sdreja
                  j = 2
c Values including both I+ and I-
c  set sdrejm[2]a for merging
                  if (k .ne. 1) then
                     sdrejma  = sdreja
                     sdrejm2a = sdrej2a
                  endif
               else
                  call gttrea(n, sdrej2a, lflag, ntok,ityp,fvalue)
                  if (k .ne. 1) then
                     sdrejm2a = sdrej2a
                  endif
               endif
            endif
         endif
         n = n+1
         go to 211
      endif
c
      continue
      go to 10     
c
c------------------------------------------------------------------------
c
c   == DUMP ==
c
c  Dump scales to output file, optionally read file name (default SCALES)
c
 220  ldump = .true.
      if (ntok .gt. 1) then
         dumpfl = line(ibeg(2):)
      endif
      go to 10
c------------------------------------------------------------------------
c
c   == NODUMP ==
c
c  Turn of dump of scales
c
 225  ldump = .false.
      go to 10
c
c------------------------------------------------------------------------
c
c   == RESTORE ==
c
c  Restore scales from file, optionally read file name (default SCALES)
c
 230  lrstr = .true.
      if (ntok .gt. 1) then
         rstrfl = line(ibeg(2):)
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == SDCORRECTION ==
c
c  Set SD correction factors
c
c  RUN  run_number          (else applies to all runs except reference)
c  [NO]ADJUST          controls automatic adjustment of sdfac from normal
c                      probability plot
c  [NO]REFINE          refinement of SD parameters by making SD plot flat
c  UNIFORM             same SD parameters for all runs & full/partials
c  INDIVIDUAL [default]   different SD parameters for all runs & full/partials 
c  COMMON              same SdB & SdAdd all runs & full/partials, different SdFac
c  FIXSDB              fix SdB paramter
c  FULL|PART|BOTH      following values apply to fulls|partials|both
c                       [default is both]
c  [VALUE] Sdfac1 [Sdfac2] Sdfac3


 240  nrun  = -1
      lsdset = .false.
c  Default to values applying to both fulls & partials
      jflprt = 0
      lflprt(1) = .false.
      lflprt(2) = .false.
      n = 2
c Next keyword N
 241  if (n .le. ntok) then
         if (cvalue(n) .eq. 'RUN') then
c  Define run number
c   read run number
            call gttint(n+1, nrun, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+2
            if (numrun .gt. 0) then
               do 242, irun = 1, numrun
                  if (irunum(irun) .eq. nrun) then
c        OK
                     go to 241
                  endif
 242           continue
            else
c     No run defined, 1 by default
               if (nrun .eq. 1) then
c        OK
                  irun = 1
                  go to 241
               endif
            endif
c   This run is not defined
            write (lunout, '(A,I4,A)') ' **** Run number', nrun,
     $         ' has not been defined ****'
            go to 999

         else if (cvalue(n) .eq. 'ADJU') then
            ladjsd = .true.
            n = n+1
         else if (cvalue(n) .eq. 'NOAD') then
            ladjsd = .false.
            n = n+1
         else if (cvalue(n) .eq. 'REFI') then
            lsdfac = .true.
            n = n+1
         else if (cvalue(n) .eq. 'NORE') then
            lsdfac = .false.
            n = n+1
         else if (cvalue(n) .eq. 'UNIF') then
            ksdtype = 0
            n = n+1
         else if (cvalue(n) .eq. 'INDI') then
            ksdtype = +1
            n = n+1
         else if (cvalue(n) .eq. 'COMM') then
            ksdtype = +2
            n = n+1
         else if (cvalue(n) .eq. 'FIXS') then
            fixSdB = .true.
            n = n+1
         else if (cvalue(n) .eq. 'FULL') then
            jflprt = 1
            n = n+1
         else if (cvalue(n) .eq. 'PART') then
            jflprt = 2
            n = n+1
         else if (cvalue(n) .eq. 'BOTH') then
            jflprt = 0
            n = n+1
         else
            if (cvalue(n) .eq. 'VALU') then
               n = n+1
            endif
c  Read up to maxsdf numbers
c   How many numbers are there before next keyword or end of line?
            k = howmny(n, ntok, ityp)
            if (k .gt. maxsdf) then
               write (lunout,'(A,A)') ' **** SDCORRECTION:',
     $           ' too many values given'
               go to 990
            elseif (k .le. 0) then
               write (lunout,'(A,A)') ' **** SDCORRECTION:',
     $           ' too few values given'
               go to 990
            endif
c  i1 = 1 (BOTH or FULL), or 2 (PART), index for temporary store of values
            i1 = max(1, jflprt)
            do 243, i = 1, k
               call gttrea(n, sdrfac(i,i1), lflag,ntok,ityp,fvalue)
               if (lflag .ne. 0) go to 990
               n=n+1
 243        continue
            if (k .lt. maxsdf) then
c Not all values given, rearrange them
               do 246, i = k+1, maxsdf
                  sdrfac(i,i1) = 0.0
 246           continue
               if (maxsdf .eq. 3) then
c  This should always be the case: CHANGE THIS if more than 3 sd parameters
                  sdrfac(3,i1) = sdrfac(2,i1)
                  sdrfac(2,i1) = 0.0
               endif
            endif
            lsdset = .true.
            lsdmod = .true.
            lflprt(i1) = .true.
         endif
         go to 241
      endif
c Store things: temporarily stored in either sdrfac(,1) or sdrfac(,2),
c as indicated by flags in lflprt
      if (nrun .lt. 0) then
         i1 = 1
         i2 = maxrun
      else
         i1 = irun
         i2 = irun
      endif
      do 244, irun = i1, i2
         if (lsdset .and.
     $      (irun .ne. irefrn .or. nrun .eq. irefrn)) then
            jsdfac(irun) = +1
c Don't change sd parameters for reference run unless explicitly given
            do 245, i = 1,maxsdf
               if (jflprt .eq. 0) then
c  BOTH, values stored in slot 1
                  sdfact(i,irun,1) = sdrfac(i,1)
                  sdfact(i,irun,2) = sdrfac(i,1)
               else
c  FULL and/or PART
                  if (lflprt(1)) sdfact(i,irun,1) = sdrfac(i,1)
                  if (lflprt(2)) sdfact(i,irun,2) = sdrfac(i,2)
               endif
 245        continue
         endif
 244  continue
      go to 10
c
c
c------------------------------------------------------------------------
c
c   == [UN]TIE ==
c
c   Define ties between parameters
c     [ROTATION  [sd]]   tie along rotation axis
c     [DETECTOR [sd]]    tie in detector plane
c     [SURFACE [sd]]     tie surface parameters to 0.0
c                          (ie restrain to spherical correction]
c     [BFACTOR  [sd]]    tie on Bfactor
c     [A1 [sd]]          tie on tails parameter A1
c
c  TIE
 250  n = 2
      l1 = .true.
      go to 251
c  UNTIE
 255  n = 2
      l1 = .false.
 251  if (n .le. ntok) then
         if (cvalue(n) .eq. 'ROTA') then
            if (l1) then
               ntiez = -1
               if (n+1 .le .ntok .and. ityp(n+1) .eq. 2) then
c   read number
                  call gttrea(n+1,vtiez, lflag,ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
                  vtiez =  vtiez**2
               endif
            else
               ntiez = 0
            endif 
         elseif (cvalue(n) .eq. 'DETE') then
            if (l1) then
               ntiexy = -1
               if (n+1 .le .ntok .and. ityp(n+1) .eq. 2) then
c   read number
                  call gttrea(n+1,vtiexy, lflag,ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
                  vtiexy =  vtiexy**2
               endif
            else
               ntiexy = 0
            endif 
         elseif (cvalue(n) .eq. 'SURF') then
            if (l1) then
               ntiesrf = -1
               if (n+1 .le .ntok .and. ityp(n+1) .eq. 2) then
c   read number
                  call gttrea(n+1,vtiesrf, lflag,ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
                  vtiesrf =  vtiesrf**2
               endif
            else
               ntiesrf = 0
            endif 
         elseif (cvalue(n) .eq. 'BFAC') then
            if (l1) then
               ntiebf = -1
               if (n+1 .le .ntok .and. ityp(n+1) .eq. 2) then
c   read number
                  call gttrea(n+1,vtiebf, lflag,ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
                  vtiebf =  vtiebf**2
               endif
            else
               ntiebf = 0
            endif 
         elseif (cvalue(n) .eq. 'A1') then
            if (l1) then
               ntieA1 = -1
               if (n+1 .le .ntok .and. ityp(n+1) .eq. 2) then
c   read number
                  call gttrea(n+1,vtieA1, lflag,ntok,ityp,fvalue)
                  if (lflag .ne. 0) go to 990
                  n = n+1
                  vtieA1 =  vtieA1**2
               endif
            else
               ntieA1 = 0
            endif 
         else
            go to 980
         endif
         n = n+1
         go to 251
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == OUTPUT ==
c  Options for output file
c
c  File format options:
c	NONE    no output file written
c	AVERAGE [default] output averaged intensities, <I+> & <I-> 
c                for each hkl (equivalent to the output from Agrovata)
c       SEPARATE output observations as input, but with
c		added columns for SCALE etc. This file may be input to
c		Agrovata for averaging
c	UNMERGED sum or scale partials, reject outliers, but do not
c		average observations
c       POLISH  write "Scalepack"-style file in addition to MTZ file
c       SPLIT   write different datasets to separate files, even if
c               main output option is not AVERAGE (in which case SPLIT
c               is the default)
c       TOGETHER output a single file even if multiple datasets are
c               present. This is the default for SEPARATE & UNMERGED options
c 
c    AVERAGE & UNMERGED may be combined to give both files (ifilou = +5)
c
c
c  Other options:
c
c    For SEPARATE option only:-
c	REFERENCE  write reference batch (if present) to output file
c	NOREFERENCE [default] omit reference batch (if present) from
c		output file
c
c	KEEP [default] keep reflections outside resolution limits. The
c	        SCALE column will be set = 0.0, and these reflections will
c                be rejected by Agrovata
c	KEEP SCALE keep reflections outside resolution limits, and
c		calculate scales for them. This is dangerous unless
c		the proportion of reflections omitted from scaling is small
c	EXCLUDE exclude reflections outside resolution limits
c
c	OMIT OUTLIERS  omit rejected outliers from output file.
c		In this case a ROGUES file is written (see below)
c	        [default keep them in, but flagged in the FLAG column]
c	OMIT PARTIALS [RUN <Nrun>] omit partially recorded reflections
c		from output file. If no run number is given, then it
c		applies to all runs. Multiple runs may specified on
c		successive OUTPUT OMIT PARTIALS RUN commands
c
c	ROGUES  write a list of rejected reflections is written
c		to the file ROGUES. This may be assigned on the
c		command line. A ROGUES file is always written for the
c		AVERAGE & UNMERGED options.
c		[for SEPARATE, default no ROGUES file written unless
c		OMIT OUTLIERS option used]
c
c	POSTREF append columns for POSTREF. This option implies SEPARATE
c		The added columns are IMEAN SIGIMEAN ISUM SIGISUM
c		IMEAN    mean of fully-recorded reflections
c		ISUM     summed partials (partials only)
c
c       REDUCED output reduced indices hkl (normal)
c    For UNMERGED option only:-
c       ORIGINAL output origina indices hkl (UNMERGED only)
c       BEAMS    output direction cosines of incident and
c                 diffracted beams, in orthogonalised crystal frame
c
 260  n = 2
      mrdrun = -1
 261  if (n .le. ntok) then
         if (cvalue(n) .eq. 'NONE') then
            ifilou = 0
         elseif (cvalue(n) .eq. 'AVER') then
            if (ifilou .eq. +4) then
c     AVERAGE & UNMERGED
               ifilou = +5
            else 
               ifilou = +1
            endif
         elseif (cvalue(n) .eq. 'SEPA') then
            ifilou = +2
c  Set defaults for SEPARATE or POSTREF, to be optionally overwritten
c by later switches
            call setodf
         elseif (cvalue(n) .eq. 'POST') then
            ifilou = +3
            call setodf
         elseif (cvalue(n) .eq. 'UNME') then
            if (ifilou .eq. +1) then
c     AVERAGE & UNMERGED
               ifilou = +5
            else 
               ifilou = +4
            endif
         elseif (cvalue(n) .eq. 'POLI') then
            isclpk = 3
         elseif (cvalue(n) .eq. 'SPLI') then
c   may be reset later
            kfilou = +1
         elseif (cvalue(n) .eq. 'TOGE') then
            kfilou = -1
c
c The following are only valid with UNMERGED
         elseif (cvalue(n) .eq. 'REDU') then
            lrdidx = .true.
         elseif (cvalue(n) .eq. 'ORIG') then
            lrdidx = .false.            
         elseif (cvalue(n) .eq. 'BEAM') then
            mbeams = +1
         elseif (cvalue(n) .eq. 'DBEA') then
            mbeams = -1
c The following are only valid with SEPARATE or POSTREF (no check made here)
         elseif (cvalue(n) .eq. 'REFE') then
            lrefou = .true.
         elseif (cvalue(n) .eq. 'NORE') then
            lrefou = .false.
         elseif (cvalue(n) .eq. 'KEEP') then
            krsexc = +1
         elseif (cvalue(n) .eq. 'SCAL') then
            krsexc = +2
         elseif (cvalue(n) .eq. 'EXCL') then
            krsexc = 0
         elseif (cvalue(n) .eq. 'OMIT') then
            continue 
         elseif (cvalue(n) .eq. 'OUTL') then
            lolexc = .true.
            lwrrog = .true.
         elseif (cvalue(n) .eq. 'PART') then
            nprexc = nprexc+1
            n = n+1
            mrdrun = 0
            if (n .le. ntok) then
               if (cvalue(n) .eq. 'RUN') then
                  n = n+1
                  if (n .le. ntok) then
                     call gttint(n, nrun,
     $                    lflag, ntok,ityp,fvalue)
                     if (lflag .ne. 0) go to 990
                     mrdrun = +1
                  else
                     go to 990
                  endif
               endif
            endif
         elseif (cvalue(n) .eq. 'ROGU') then
            lwrrog = .true.
         else
            go to 980
         endif
         n = n+1
         go to 261
      endif
c
      if (mrdrun .gt. 0) then
c OMIT PARTIALS RUN <n>
c  Get internal run number irun
         if (numrun .gt. 0) then
            call chkrun(nrun, irun)
            if (irun .le. 0) then
               go to 999
            endif
         else
            write (lunout, '(/a/)')
     $           ' **** No runs have been defined ****'
            go to 999
         endif
c  Store internal run number
         kprexc(nprexc) = irun
      elseif (mrdrun .eq. 0) then
c OMIT PARTIALS, for all runs
         if (numrun .gt. 0) then         
            nprexc = -1
         else
            write (lunout, '(/a/)')
     $           ' **** No runs have been defined ****'
            go to 999
         endif
      endif
c
      go to 10
c
c------------------------------------------------------------------------
c
c   == INSCALE ==
c Switch OFF or ON (default) application of input scale column
c
 270  if (cvalue(2) .eq. 'OFF') then
         lappsc = .false.
      elseif (cvalue(2) .eq. 'ON') then
         lappsc = .true.
      else
         go to 980
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c   == TITLE ==
c Read title to replace input title
 280  if (ntok .gt. 1) outitl = line(ibeg(2):)
      go to 10
c------------------------------------------------------------------------
c
c   == HISTORY ==
c Read new history line to add to file
 290  if (lenstr(histry) .gt. 0) then
         write (lunout, '(a,a)')
     $        ' ==== WARNING: only one new history line may be',
     $        ' added to file ===='
      else
         histry = line(ibeg(2):)
      endif
      go to 10
c------------------------------------------------------------------------
c
c   == OVERLAPMAP ==
c Set flag to write overlap array to MAPOUT
 300  lovmap = .true.
      go to 10
c------------------------------------------------------------------------
c
c  == FINAL == 
c		 [PARTIALS [[NO]CHECK | 
c                        [NO]TEST [<lower_limit> <upper_limit>] |
c			CORRECT <minimum_fraction>] 
c                 [[NO]GAP]
c                 [MAXWIDTH  <maximum_width>] |
c                 SCALE_PARTIAL <Minimum_fraction>]
c  Control usage of partials in final pass
c
c   NONE         no final analysis/output pass
c   FULLS         use fulls only [default if no FRACTIONCALC column]
c   ONLYFULLS    fulls only (no previously summed partials)
c   PARTIALS     use summed partials in final analysis (if present) 
c	        [this is the default if FRACTIONCALC column is present]
c       [NO]CHECK   do [not] check for consistency of MPART flags
c                (if present) [default do]
c       [NO]TEST [<lower_limit> <upper_limit>]
c                 do [not] accept partials only if total fraction (from 
c                 FRACTIONCALC column) is in range lower_limit -> upper_limit
c                 [default if no MPART flag, limits 0.95, 1.2]
c       CORRECT [<minimum_fraction>]
c                 Scale partials in range minimum_fraction -> lower_limit,
c                 predicted total fraction (needs reliable FRACTIONCALC)
c                 [default no correction]
c       [NO]GAP   do [not] accept partials with a gap in, eg a partial
c                 over 3 parts with the middle one missing. GAP implies
c                 NOCHECK and TEST: CORRECT may also be set.
c       MAXWIDTH  <maximum_width> maximum number of parts for an
c                 acceptable summed partial
c   SCALE_PARTIALS  use scaled partials greater than Minimum_fraction
c	        in the scaling. Only use this if the FRACTIONCALC
c	        column contains a good estimate of the partiality.
C
 310  n = 2
      lfinal = .true.
 301  if (n .le. ntok) then
         if (cvalue(n) .eq. 'PART') then
            kprout = +1
         elseif (cvalue(n) .eq. 'NONE') then
            lanout = .false.
         elseif (cvalue(n) .eq. 'FULL') then
            kprout = 0
         elseif (cvalue(n) .eq. 'ONLY') then
            kprout = -1
         elseif (cvalue(n) .eq. 'SCAL') then
            lprscl(2) = .true.
            call gttrea(n+1, prthou, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
c  A number read
               n = n+1
               lcorrc = .true.
            endif
         elseif (cvalue(n) .eq. 'CHEC') then
            kprflo = -min(iabs(kprflo), 10)
         elseif (cvalue(n) .eq. 'NOCH') then
            kprflo = +min(iabs(kprflo), 10)
         elseif (cvalue(n) .eq. 'NOTE') then
c      NOTEST
            kprflo = sign(+1, kprflo)
         elseif (cvalue(n) .eq. 'GAP') then
            kprmso = +1
         elseif (cvalue(n) .eq. 'NOGA') then
            kprmso =  0
         elseif (cvalue(n) .eq. 'TEST') then
            if (kprout .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword TEST must follow PARTIAL'
               go to 999
            endif
            kprflo = sign(+2, kprflo)
            call gttrea(n+1, prlimo, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
            call gttrea(n+1, prmaxo, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
         elseif (cvalue(n) .eq. 'CORR') then
            if (kprout .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword CORRECT must follow PARTIAL'
               go to 999
            endif
            kprflo = sign(+2, kprflo)
            call gttrea(n+1, prthou, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
            lcorrc = .true.
         elseif (cvalue(n) .eq. 'MAXW') then
            if (kprout .ne. 1) then
               write (lunout, '(a)')
     $            ' Subkeyword MAXWIDTH must follow PARTIAL'
               go to 999
            endif
            call gttint(n+1, mxprwo, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            endif
         endif
         n = n+1
         go to 301
      endif
c If no CORRECT keyword given, set scale threshold = lower TEST limit
      if (.not.lcorrc .and. kprout .eq. +1) then
         prthou = prlimo
      endif
      go to 10
c------------------------------------------------------------------------
c  == NOSCALE ==
c  no scaling, set cycles = 0
c
 320  ncycle = 0
      lscale = .false.
      go to 10
c------------------------------------------------------------------------
c  == SKIP ==
c  <N_skip>     skip every N_skip'th reflection
c  [FOR]  <N_skip_cycles>   number of cycles to use subset of reflections
c
 330  nrfskp = 0
      if (ntok .gt. 1) then
         call gttint(2, nrfskp, lflag, ntok,ityp,fvalue)
         if (lflag .ne. 0) go to 990
         if (ntok .gt. 2) then
            call gttint(ntok, ncyskp, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         endif
      endif
      go to 10
c------------------------------------------------------------------------
c  == [UN]FIX ==
c     [V]  [A0]  [A1]
c
c  Optionally fix TAILS parameters
c
 340  lfixx = .true. 
 341  n = 2      
c  do while (n .le. ntok)
 342  if (n .le. ntok) then
         if (cvalue(n) .eq. 'V') then
            lptfix(1) = lfixx
         elseif (cvalue(n) .eq. 'A0') then
            lptfix(2) = lfixx
         elseif (cvalue(n) .eq. 'A1') then
            lptfix(3) = lfixx
         else
            write (lunout, '(/a/)')
     $      ' **** Unrecognized parameter name on [UN]FIX command ****'
            go to 999
         endif
         n = n+1
         go to 342
      endif
      go to 10
c UNFIX
 345  lfixx = .false.
      go to 341
c------------------------------------------------------------------------
c  == ANOMALOUS ==
c  [ON|ALL]  [OFF] [RUN <run number>] [MATCH [SPINDLE|INVERT|<hkl symmetry>]]
c   [PHIDIF <maximum phi difference>]
c
c Switches for treatment of anomalous scattering information
c
c  OFF [default]    no anomalous used, I+ & I- observations averaged together
c  ON|ALL           separate anomalous observations in the final output
c                   pass, for statistics & merging: this is also selected
c                   the keyword ANOMALOUS on its own
c  RUN <run number>  set run for this MATCH option to apply to, else
c                   applies to all runs
c  MATCH            use only matching I+ & I- pairs in merging
c                   Matching pairs are
c                   (a) in same run (if INRUN given)
c                   (b) related by defined symmetry (if given)
c                   (c) not more than DeltaPhi aprt (if given)
c  Definition of symmetry:-
c    SPINDLE        related by negation of reciprocal index closest to spindle
c    INVERT         -h, -k, -l
c    <hkl symmetry> specified hkl symmetry (eg -h, -k, l)
c
c  INRUN            matching pairs must in same run (default no check)
c
c  PHIDIF <maximum phi difference>
c                   maximum difference in Phi (ROT) between matching pairs
c                   [default no check]
c  TIMEDIF <maximum time difference>
c                   maximum difference in TIME between matching pairs
c                   [default no check]
 350  n = 2
c  jj is flag for s/r strmtc
      jj = 0
      j = -1
      k = -1
      nrun = -1
      if (ntok .eq. 1) kanomo = +1
 351  if (n .le. ntok) then
         lsubky = .false.
         if (cvalue(n) .eq. 'RUN') then
c  Define run number
c   read run number
            call gttint(n+1, nrun, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            n = n+1
            if (numrun .gt. 0) then
               do 352, irun = 1, numrun
                  if (irunum(irun) .eq. nrun) then
c        OK
                     go to 353
                  endif
 352           continue
            else
c     No run defined, 1 by default
               if (nrun .eq. 1) then
c        OK
                  irun = 1
                  go to 353
               endif
            endif
c   This run is not defined
            write (lunout, '(A,I4,A)') ' **** Run number', nrun,
     $         ' has not been defined ****'
            go to 999
c
 353        lsubky = .true.
         elseif (cvalue(n) .eq. 'OFF') then
            kanomo = 0
            lsubky = .true.
         elseif (cvalue(n) .eq. 'ALL' .or. cvalue(n) .eq. 'ON') then
            kanomo = +1
            lsubky = .true.
         elseif (cvalue(n) .eq. 'MATC') then
c  MATCH, set flag
            kanomo = +2
            lsubky = .true.
         elseif (cvalue(n) .eq. 'INRU') then
c  INRUN
            lanmtcrn = .true.
            lsubky = .true.
         elseif (cvalue(n) .eq. 'NOIN') then
c  NOINRUN
            lanmtcrn = .false.
            lsubky = .true.
         elseif (cvalue(n) .eq. 'SPIN') then
c  SPINDLE, implies MATCH
            kanomo = +5
            jj = +1
            lsubky = .true.
         elseif (cvalue(n) .eq. 'INVE') then
c  INVERT, implies MATCH
            kanomo = +4
            jj = +2
            lsubky = .true.
c  PHIDIF <DelPhiMax>
         elseif (cvalue(n) .eq. 'PHID') then
            lsubky = .true.
            call gttrea(n+1, anmphd, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            else
               write (lunout, '(/a/)')
     $            ' **** PHIDIF must be followed by a value ***'
               go to 990
            endif
c  TIMEDIF <DelTimeMax>
         elseif (cvalue(n) .eq. 'TIME') then
            if (nct .eq. 0) then
               write (lunout, '(/a/)')
     $            ' **** Cannot give TIMEDIF without TIME column'
               go to 990
            endif
            lsubky = .true.
            call gttrea(n+1, anmtmd, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
            else
               write (lunout, '(/a/)')
     $            ' **** TIMEDIF must be followed by a value ***'
               go to 990
            endif
         endif
         if (lsubky) then
c Keyword found, flag end of symmetry if symmetry operator being read
            if (j .gt. 0) then
               k = ibeg(n) - 1
c  Putative symmetry now in line(j:k)
               if (k .lt. j) then
                  go to 980
               endif
            endif
         else
c Keyword not recognized, start symmetry operator, record 1st character,
c   but only if MATCH is set
            if (kanomo .lt. +2) then
c  else it is an error
               go to 980
            else
               if (j .le. 0) then
                  j = ibeg(n)
               endif
            endif
         endif
         n = n+1
         go to 351
      endif
c Process symmetry operator for match, if required
      if (nrun .lt. 0) then
         i1 = 1
         i2 = maxrun
      else
         i1 = irun
         i2 = irun
      endif
      if (kanomo .ge. +3 .or. j .gt. 0) then
         if (jj .eq. 0) kanomo = +3
         if (j .le. 0) j = ibeg(2)
         if (k .le. 0) k = iend(ntok)
         call strmtc(jj, line(j:k), i1, i2, istat)
         if (istat .ne. 0) go to 990
      endif
      go to 10
c
c------------------------------------------------------------------------
c  == WIDTH ==
c  WILSON|LINEAR]|SQUARE  [NBINS <number of bins>]  [<mid-point>]
c
c  Select binning mode on intensity
c  WILSON  [default]   exponential bins 
c  LINEAR              linear bins
c  SQUARE              quadratic bins
c
c  In each case, <mid-point> is the limit for the middle bin
 360  n = 2
      if (ntok .eq. 1) kbnrtp = 3
 361  if (n .le. ntok) then
         if (cvalue(n) .eq. 'WILS') then
            kbnrtp = 3
            n = n+1
         elseif (cvalue(n) .eq. 'LINE') then
            kbnrtp = 1
            n = n+1
         elseif (cvalue(n) .eq. 'SQUA') then
            kbnrtp = 2
            n = n+1
         elseif (cvalue(n) .eq. 'NBIN') then
            call gttint(n+1, ninbin, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            ninbin = max(min(ninbin, maxinb), 1)
            n = n+2
         else 
            if (ntok .ge. 2) then
               call gttrea(n, binmid, lflag, ntok,ityp,fvalue)
               n = n+1
            endif
         endif
         go to 361
      endif
      go to 10
c------------------------------------------------------------------------
c  == LINK ==
c [TAILS] <run_number2> TO <run_number1> | [ALL]
c [SURFACE] <run_number2> TO <run_number1> | [ALL]
c
c  Make tails or surface (secondary or crystal) parameters common to
c runs run_number2 & run_number1
c
 370  jj = 1
      j  = 1
 372  n = 2
      k = 1
      i1 = -1
      i2 = -1
c  jj = +1 link, -1 unlink
c   j = 1 TAILS, = 2 SURFACE
 371  if (n .le. ntok) then
         if (cvalue(n) .eq. 'TAIL') then
c  Link type
            j = sign(+1,jj)
         elseif (cvalue(n) .eq. 'SURF') then
c  Link type
            j = sign(+2,jj)
         elseif (cvalue(n) .eq. 'ALL') then
            k = -1
         elseif (cvalue(n) .eq. 'TO') then
            k = 2
         elseif (k .eq. 1) then
            call gttint(n,i1,lflag,ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         elseif (k .eq. 2) then
            call gttint(n,i2,lflag,ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         endif
         n = n+1
         go to 371
      endif
      if (i1 .lt. 0 .and. i2 .lt. 0) then
         k = -1
      elseif ((i1 .lt. 0 .or. i2 .lt. 0) .and. k .ge. 0) then
         go to 990
      endif
c  Store link for later processing
      if (k .lt. 0) i1 = -1
      if (nlink .lt. 0) nlink = 0
      nlink = nlink+1
      klink(1,nlink) = i1
      klink(2,nlink) = i2
      klink(3,nlink) = j
c flag to say that this type of link has been set
      if (iabs(j) .eq. 1) then
         lnktds = .true.
      elseif (iabs(j) .eq. 2) then
         lnksph = .true.
      endif
      go to 10
c-----
c  == UNLINK ==
c  Opposite of LINK
 375  jj = -1
      j  = -1
      go to 372
c------------------------------------------------------------------------
c  == ANALYSE ==
c [NO]NORMAL    do [not] do normal probability analysis
c [NO]PLOT      do [not] output normal probability plot
c MAXDENSITY    maximum point density for plot
c
 380  n = 2
 381  if (n .le. ntok) then
         if (cvalue(n) .eq. 'NORM') then
            lnormn = .true.
         elseif (cvalue(n) .eq. 'NONO') then
            lnormn = .false.
         elseif (cvalue(n) .eq. 'PLOT') then
            iwrprb = 8
         elseif (cvalue(n) .eq. 'NOPL') then
            iwrprb = 0
         elseif (cvalue(n) .eq. 'MAXD') then
            n = n+1
            call gttint(n,maxptd,lflag,ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         endif
         n = n+1
         go to 381
      endif
      go to 10
c------------------------------------------------------------------------
c  == ONLYMERGE ==
c  Merge step only, equivalent to INITIAL NONE; NOSCALE
 390  ncycle = 0
      lscale = .false.
      kinisc = -1
      iniscl = 1
      go to 10   
c------------------------------------------------------------------------
c  == XYBINS ==
c  Set number of bins for xy analysis
 400  if (ntok .gt. 1) then
         call gttint(2,nbinxy(1),lflag,ntok,ityp,fvalue)
         if (lflag .ne. 0) go to 990
         if (ntok .gt. 2) then
            call gttint(3,nbinxy(2),lflag,ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
         else
            nbinxy(2) = nbinxy(1)
         endif
      endif
      go to 10   
c------------------------------------------------------------------------
c
c  == PARTIALS == command
c
c                 [NO]CHECK
c                 [NO]TEST [<lower_limit> <upper_limit>]
c                  CORRECT [<minimum_fraction>]
c                 [NO]GAP
c                  MAXWIDTH  <maximum_width>
c                  SCALE_PARTIAL <minimum_fraction>
c                  USE_PROFILE
c
c  Define treatment of partials in both scaling & merging
c  These settings may be overridden by the INTENSITIES or FINAL commands
c
c   [NO]CHECK 
c            do [not] check for consistency of MPART flags
c            (if present) 
c   [NO]TEST [<lower_limit> <upper_limit>]
c             do [not] accept partials only if total fraction (from 
c             FRACTIONCALC column) is in range lower_limit -> upper_limit
c             [default if no MPART flag]
c   CORRECT [<minimum_fraction>]
c             Scale partials in range minimum_fraction -> lower_limit,
c             predicted total fraction (needs reliable FRACTIONCALC)
c   [NO]GAP   do [not] accept partials with a gap in, eg a partial
c             over 3 parts with the middle one missing. GAP implies
c             NOCHECK and TEST: CORRECT may also be set.
c   MAXWIDTH  <maximum_width> maximum number of parts for an
c             acceptable summed partial
c   SCALE_PARTIALS  use scaled partials greater than Minimum_fraction
c	        in the scaling. Only use this if the FRACTIONCALC
c   USE_PROFILE   use profile-fitted intensity even for scaled partials
c
 410  n = 2
      lfinal = .true.
      lprtst = .false.
c lprscl is flag that SCALE_PARTIAL has been set
c
c Set flags for scaling, and for merging 
 411  if (n .le. ntok) then
         if (cvalue(n) .eq. 'CHEC') then
            kprflg = -min(iabs(kprflg), 10)
            kprflo = kprflg
         elseif (cvalue(n) .eq. 'NOCH') then
            kprflg = +min(iabs(kprflg), 10)
            kprflo = kprflg
         elseif (cvalue(n) .eq. 'NOTE') then
            kprflg = sign(+1, kprflg)
            kprflo = kprflg
         elseif (cvalue(n) .eq. 'GAP') then
            kprmis = +1
            kprmso = kprmis
         elseif (cvalue(n) .eq. 'NOGA') then
            kprmis =  0
            kprmso = kprmis
         elseif (cvalue(n) .eq. 'TEST') then
            kprflg = sign(+2, kprflg)
            kprflo = kprflg
            lprtst = .true.
            call gttrea(n+1, prlims, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               prlimo = prlims
               n = n+1
            endif
            call gttrea(n+1, prmaxs, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               prmaxo = prmaxs
               n = n+1
            endif
         elseif (cvalue(n) .eq. 'CORR') then
            kprflg = sign(+2, kprflg)
            kprflo = kprflg
            call gttrea(n+1, prthld, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               prthou = prthld
               n = n+1
            endif
            lcorrc = .true.
         elseif (cvalue(n) .eq. 'MAXW') then
            call gttint(n+1, mxprwd, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               mxprwo = mxprwd
               n = n+1
            endif
         elseif (cvalue(n) .eq. 'SCAL') then
            lprscl(1) = .true.
            lprscl(2) = lprscl(1)
            call gttrea(n+1, prthld, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then
c  A number read
               prthou = prthld
               n = n+1
            endif
            lcorrc = .true.
         elseif (cvalue(n) .eq. 'USE_') then
c  Use profile-fitted intensities for scaled partials (only if present)
            if (ncip .gt. 0 .and. ncsdip .gt. 0) then
               lsclpr = .true.
            endif
         endif
         n = n+1
         go to 411
      endif
c If no CORRECT keyword given, set scale threshold = lower TEST limit,
c if this has been set
      if (.not.lcorrc .and. .not.lprscl(1) .and. lprtst) then
         prthld = prlims
         prthou = prlimo
      endif
      go to 10
c------------------------------------------------------------------------
c   DAMP NONE | <damp factor> <NcycDamp>
c
c Read damping factor & number of cycles for damping
c If NcycDamp not given, damp on all cycles
 420  n = 2
      if (cvalue(n) .eq. 'NONE') then
         damp = 0.0
         ncycdamp = 0
      else
         if (ntok .ge. n) then
            call gttrea(n, damp, lflag, ntok,ityp,fvalue)
            if (lflag .gt. 0) go to 990
            n = n+1
            if (ntok .ge. n) then
               call gttint(n, ncycdamp, lflag, ntok,ityp,fvalue)
               if (lflag .gt. 0) go to 990
            else
               ncycdamp = -1
            endif
         endif
      endif
      go to 10
c------------------------------------------------------------------------
C
C---- PNAME [pname]  <string>
C     =====
C     
C             if given with DATASET then harvest will o/p a file
C             this project_name should be always used for the
C             one structure determination
C             no default
C  Obsolete: use DATASET
C
 430  pname = line(ibeg(2) :iend(2))
      xname = pname
 431  ndtskyin = 1
      call nospch(pname_out(ndtskyin), pname)
      call nospch(xname_out(ndtskyin), xname)
      call nospch(dname_out(ndtskyin), dname)
c  Applies to all runs
      nrundts(ndtskyin) = -1
      write (lunout, '(a/a)')
     $   ' ==== WARNING: PNAME/DNAME commands are obsolete ====',
     $   ' ==== Please use the NAME command             ===='
      go to 10
c------------------------------------------------------------------------
c
c---- DNAME  [dname]  <string>
c     =====
c     
c             if given with PROJECT then harvest will o/p a file
c             this dataset_name is the name of one of the diffraction
c             data sets used in a particular project
c             no default
c  Obsolete: use DATASET
c
 440  dname = line(ibeg(2) :iend(2))
      call spltdname(xname, dname)
      go to 431
c------------------------------------------------------------------------
c
c  == NAME ==
c
c  [Re]define dataset information for datasets, to be used for output
c
c  RUN | n1 n2 n3 .. | n1 TO n2 |  select run[s].
c          The new dataset will include all
c          data in the selected run[s]. If this is omitted, it applies
c          to all runs
c  PROJECT   project_name   must preceed DATASET if given
c  CRYSTAL   xtal_name      must preceed DATASET if given
c  DATASET   dataset_name or xtal_name/dataset_name
c
 450  n = 2
      llist = -1
      l1 = .false.
c
 451  if (n .le. ntok) then
         if (cvalue(n) .eq. 'RUN') then
c   read run number[s]
            n = n+1
            call lisran(ntok, n, cvalue, fvalue, ityp, llist, vlist)
            if (llist .lt. 0) then
               write (lunout, '(A)') ' ** Error in RUN list **' 
               go to 999
            endif
         elseif (cvalue(n) .eq. 'PROJ') then
c  set current pname
            pname = line(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (cvalue(n) .eq. 'CRYS') then
c  set current xname
            xname = line(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (cvalue(n) .eq. 'DATA') then
            dname = line(ibeg(n+1):iend(n+1))
            n = n+2
            l1 = .true.
         else
            go to 990
         endif
         go to 451
      endif
c
      if (.not. l1) then
         write (lunout, '(/a/)') ' **** No Dataset Name given ****'
         go to 999
      endif
c Separate xname/dname if necessary
      call spltdname(xname, dname)
c Check if this is a new given dataset
      if (ndtskyin .gt. 0) then
         call nospch(xname, xname)
         call nospch(dname, dname)
         call getdtsnum(xname, dname, xname_out, dname_out, ndtskyin,
     $      idtsnum)
      else 
         idtsnum = 0
      endif
      if (idtsnum .le. 0) then
c  This is a new dataset, create it
         ndtskyin = ndtskyin + 1
         If (ndtskyin .gt. maxsets) then
            write (lunout, '(/a,i4/)')
     $         ' ***** Too many datasets defined, maximum = ', maxsets
            go to 999
         endif
         idtsnum = ndtskyin
c  Store names
         call nospch(pname_out(idtsnum), pname)
         call nospch(xname_out(idtsnum), xname)
         call nospch(dname_out(idtsnum), dname)
c  Initialise run count
         nrundts(idtsnum) = 0
      endif
c Set run list
      if (llist .lt. 0) then
c  No run specified, so apply to all: set number of runs = -1
         nrundts(idtsnum) = -1
      elseif (llist .eq. 0) then
c  Run range given, enter all in
         do i = nint(vlist(1)), nint(vlist(2))
            nrundts(idtsnum) = nrundts(idtsnum) + 1
c      krundts is run serial number
            call chkrun(i, irun)
            if (irun .le. 0) then
               go to 999
            endif
            krundts(nrundts(idtsnum), idtsnum) = irun
         enddo
      else
c  Run list
         do i = 1, llist
            nrundts(idtsnum) = nrundts(idtsnum) + 1
            call chkrun(nint(vlist(i)), irun)
            if (irun .le. 0) then
               go to 999
            endif
            krundts(nrundts(idtsnum), idtsnum) = irun
         enddo
      endif 
      go to 10
c------------------------------------------------------------------------
c
c  == BASE ==
c Set "base" dataset 
c     [CRYSTAL xtal_name]   must preceed DNAME if given
c     [DATASET dataset_name or xtal_name/dataset_name]
c
 460  xname_base = ' '
      dname_base = ' '
      n = 2
c Next keyword N
 461  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'CRYS') then
c     XNAME keyword, followed by a crystal_name
            xname_base = line(ibeg(n+1):iend(n+1))
            n = n+2
         elseif (cvalue(n) .eq. 'DATA') then
c     DNAME keyword, followed by a dataset_name
            dname_base = line(ibeg(n+1):iend(n+1))
            n = n+2
            kdtsbase = -1
         else
            go to 980
         endif
         go to 461
      endif
      go to 10
c------------------------------------------------------------------------
c
c  == NORMALISE ==
c Set normalisation option
c   SCALES      next keyword applies to scale
c                (by default just Bfactors)
c   BFACTOR     next keyword applies to bfactos
c                (default)
c   REFERENCE   normalise on reference dataset (k & B)
c                default if reference dataset present
c   FIRST       normalise on first point of first run (k & B)
c                 default for scales if no reference dataset present
c   RUN <n>     normalise on first point of run n (k & B)
c   BEST        normalise B-factors on best bit
c                 default for Bfactors if no reference dataset
c   
 470  n = 2
      lsclnrm = .false. 
c Next keyword N
 471  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'SCAL') then
            lsclnrm = .true. 
         elseif (cvalue(n) .eq. 'BFAC') then
            lsclnrm = .false. 
         elseif (cvalue(n) .eq. 'REFE') then
            if (lsclnrm) then
               knormk = 0
            else
               knormb = 0
            endif
         elseif (cvalue(n) .eq. 'FIRS') then
            if (lsclnrm) then
               knormk = +1
            else
               knormb = +1
            endif
         elseif (cvalue(n) .eq. 'RUN') then
            n = n+1
            call gttint(n, m, lflag, ntok,ityp,fvalue)
            if (lflag .ne. 0) go to 990
            call chkrun(m, irun)
            if (irun .gt. 0) then
               if (lsclnrm) then
                  knormk = +irun
               else
                  knormb = +irun
               endif
            else
               write (lunout, '(/a,i5,a/)')
     $            ' Run number', m,' is undefined'
               go to 990
            endif
         elseif (cvalue(n) .eq. 'BEST') then
            if (lsclnrm) then
c  scales best not implemented
               knormk = -1
               write (lunout, '(/a/)')
     $            ' ERROR: unimplemented option NORMALISE SCALES BEST'
               go to 999
            else
               knormb = -1
            endif
         endif
c
         n = n+1
         go to 471
      endif
      go to 10
c------------------------------------------------------------------------
c
c  == ACCEPT ==
c
c Set acceptance conditions for Mosflm FLAG column, merge step only
c By default all FLAG .ne. 0 observations are omitted.
c They are always omitted from the scale step
c
c  Subkeys:-
c    BGRATIO <bgrlim>    accept background ratio .lt. bgrlim
c                        .le. 0.0 no test
c    PKRATIO <pkrlim>    accept peak ratio .lt. pkrlim
c                        .le. 0.0 no test
c    GRADIENT <grdlim>   accept background gradient .lt. grdlim
c                        .le. 0.0 no test
c    OVERLOADS           accept profile-fitteed overloads
c    EDGE                accept edge reflections
c
 480  n = 2
c Next keyword N
 481  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'BGRA') then
            call gttrea(n+1, bgrlim, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
               ltsmfl = .true.
            endif
         elseif (cvalue(n) .eq. 'PKRA') then
            call gttrea(n+1, pkrlim, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
               ltsmfl = .true.
            endif
         elseif (cvalue(n) .eq. 'GRAD') then
            call gttrea(n+1, grdlim, lflag, ntok,ityp,fvalue) 
            if (lflag .eq. 0) then 
               n = n+1
               ltsmfl = .true.
            endif
         elseif (cvalue(n) .eq. 'OVER') then
            lovld = .true.
            ltsmfl = .true.
         elseif (cvalue(n) .eq. 'EDGE') then
            ledge = .true.
            ltsmfl = .true.
         endif
         n = n+1
         go to 481
      endif
      go to 10
c
c------------------------------------------------------------------------
c
c  == ZERODOSE ==
c   [NO]DAMP   default damp
c
 490  ktimcor = +2
      n = 2
 491  if (n .gt. 0 .and. n .le. ntok) then
         if (cvalue(n) .eq. 'DAMP') then
            ktimcor = +2
            n = n+1
         elseif (cvalue(n) .eq. 'NODA') then
            ktimcor = +1
            n = n+1
         endif
         go to 491
      endif
      go to 10
c------------------------------------------------------------------------
c
c  == CORNERCORRECT ==
c   read filename for correction table (or from logical name CORNERCORRECT
c
 500  lcorner = .true.
      if (ntok .gt. 1) then
         cornerfile = line(ibeg(2):)
      endif
      go to 10      
c------------------------------------------------------------------------
c-harvest
C
C  == PRIVATE   [priv]   flag only
C     =======
C                 if present sets dir access for first
C                 open of $HOME/DepositFiles to    drwx------
C                                       default is drwxr-xr-x 
C
C
 901  private = .true.
      go to 10
C
C  == USECWD [usec]   Flag only
C     ======
C                 if present file is opened in current working directory
C                 default is o/p file to
C                 $HOME/DepositFiles/ProjectName/DataSetName.ProgramName
C
 902  usecwd = .true.
      go to 10
C
C  == ROWLIMIT   [rsize]  <int>
C     ========
C     
C              if given sets the number of characters in a row
C              for the output file, default is 132
C              TRUE mmcif default is 80
C
 905  nrowlimit = 132
      if (ntok.ge.2) 
     +    call gttint(2, nrowlimit, lflag, ntok,ityp,fvalue) 
      go to 10
C
C  == NOHARVEST
C     ========
C
C              disable writing of harvest files
C
 906  NOHARVEST = .true.
      GO TO 10
c-harvest
c------------------------------------------------------------------------
c------------------------------------------------------------------------
c------------------------------------------------------------------------
c
 1000 continue
      call htm_endkey()
c Check & fail if any errors
C  Fatal errors
      if (iniscl .eq. 2 .and. lrstr) then
         write (lunout, '(//a,a/a,a//)')
     $        ' **** You have asked for scales to be restored,',
     $        ' AND set initial scales ****',
     $        ' **** These instructions are contradictory     ',
     $        '                        ****'
         ierr = 1
      endif
c If SCALE_PARTIAL selected on PARTIALS command, set flag for merge stage,
c but for scaling stage only if partials selected
      if (lprscl(1)) then
         kprsum = +2
      endif
      if (lprscl(2)) then
         kprout = +2
      endif
c PARTIAL CHECK requires MPART column
c Set default value of kprflg & kprflo, if not set
      call chkkpf(kprflg, ncmprt, lunout)
      call chkkpf(kprflo, ncmprt, lunout)
c
c Set usage of partials in final analysis if not set explicitly
      if (.not. lfinal) then
c   default to sum partials if FRACTIONCALC column present
         if (ncfrcl .gt. 0) then
            kprout = +1
         else
            kprout = 0
         endif
      endif
c Check for partials
      if (kprsum .eq. +2 .or. kprout .eq. +2) then
         if (ncfrcl .le. 0) then
            write (lunout,'(/a,a/)')
     $           ' **** You must have a FRACTIONCALC column to ',
     $           'scale partials ****'
            ierr = 1
         else
c Check sensible FRACTIONCALC range (not very rigorous check)
            if (frclrg(2) .lt. 0.01) then
               write (lunout, '(/a,a,a,2f8.3,a/)')
     $              ' **** The range of the FRACTIONCALC',
     $              ' does not seem sensible, and you have asked',
     $              ' for scaled fractions, range = ',frclrg,
     $              ' ****'
               ierr = 1
            endif
         endif
      endif
c SCALES TAILS requires INTENSITIES PARTIAL
      if (nprtds .gt. 0) then
         if (kprsum .ne. 1) then
            write (lunout, '(/a,a,a/)')
     $         ' **** Correction for diffuse scattering ',
     $         '(SCALES TAILS) can only be done with summed partials',
     $         ' (INTENSITIES PARTIAL) ****'
            write (lunout, '(a/)')
     $           ' **** INTENSITIES PARTIAL option switched on ****'
            kprsum = +1
         endif

      endif
c--- End of most fatal checks
      if (ierr .gt. 0) then
         call dna_error('error in input')
         call ccperr(1,'***** Error in input *****')
      endif
      if (imid .ne. 0) then
c Use profile fitted intensities if available
         if (ncip .eq. 0 .or. ncsdip .eq. 0) then
c Not available, use column I
            ncip = 0
            ncsdip = 0
            imid = 0
         endif
      endif
c
c
c No scaling if no cycles
      if (ncycle .le. 0) then
         lscale = .false.
      endif
c
c--- Write table of contents
      call htm_wrttoc(lscale,lanout)
c---
c  Default OUTPUT AVERAGE
      if (ifilou .lt. 0) ifilou = +1
c  Setup switches for number of output files
      if (kfilou .lt. -999) then
c  set defaults based on output file options
         if (ifilou .eq. 0) then
c   no output
            kfilou = 0
         elseif (ifilou .eq. +1 .or. ifilou .eq. +5) then
c   AVERAGE, default SPLIT, number set below
            kfilou = +1
         elseif (ifilou .eq. +2 .or. ifilou .eq. +3) then
c   SEPARATE, POSTREF, default TOGETHER
            kfilou = 0
         elseif (ifilou .eq. +4 .or. ifilou .eq. +5) then
c   UNMERGED, default SPLIT
            kfilou = +1
         endif
      endif
c
      call findgap(
     $   ngroup, idtsgroup, ibgroup, ibgapgroup, gapgroup, istat)
c If no run defined, include all batches for each dataset as single run
c This is done also for "RUN 1 ALL"
      if (.not. lrundefined) then
          call setrun(ngroup, idtsgroup, ibgroup, ibgapgroup, gapgroup)
c   Check for multiple runs assumed to be references runs, reset to none
         if (irefrn .lt. 0) then
            write (lunout, '(//a,a/)')
     $         '==== WARNING: multiple datasets consist of single',
     $         ' batches, so none can be considered as reference runs'
            irefrn = 0
         endif
c
      endif
c If scaling on summed partials, and run is in BATCH mode,
c then set flag to average scales, weighted by fractioncalc
      if (kprsum .eq. +1) then
         do 1005, irun = 1, numrun
            if (mbatsc(irun) .ne. 0) then
               lavpsc(irun) = .true.
            endif
 1005    continue
         if (lbatmd) then
c  Must have Fractioncalc column
            if (ncfrcl .le. 0) then
               write (lunout,'(/a,a/)')
     $            ' **** You must have a FRACTIONCALC column to ',
     $            'use summed partials in BATCH mode ****'
               ierr = 1
            else
c Check sensible FRACTIONCALC range (not very rigorous check)
               if (frclrg(2) .lt. 0.01) then
                  write (lunout, '(/a,a,a,2f8.3,a/)')
     $               ' **** The range of the FRACTIONCALC',
     $               ' does not seem sensible, and you have asked',
     $               ' for summed partials in BATCH mode, range = ',
     $               frclrg, ' ****'
                  ierr = 1
               endif
            endif
         endif
      endif
c
c If ONLYMERGE set, & no RESTORE, then set SCALE CONSTANT, even if
c SCALES command is given
c   lscale  .true. to do scaling (.false. for onlymerge == noscale)
c   kinisc  -1 for no initial pass
c   lrstr   .true. for restore
      if (.not.lscale .and. kinisc .lt. 0 .and. .not.lrstr) then
         do 1014, irun = 1, numrun
            call setrsc(irun, modedf, +0, 1, 0.0, -1.0, -1.0, -1.0)
c     Default flag for BatchMode
            mbatsc(irun) = mbatdf
            lbtslp(irun) = .false.
c     No diffuse (tails) correction
            ltdsrn(irun) = .false.
 1014    continue
      else
         if (.not.lscldf) then
c  If no SCALE definitions have been given, then set default for all runs
            do 1015, irun = 1, numrun
c  Just one scale if reference run, 
c    or if final analysis is done without any scale application
               if (irun .eq. irefrn .or.
     $            (.not.lanlin .and. .not.lscale .and. .not.lrstr)) then
                 call setrsc(irun, modedf, +0, 1, 0.0, -1.0, -1.0, -1.0)
               else
c     Set default scale layout definitions for this run
                  call setrsc(
     $               irun, modedf, nbfcdf, nbfpdf, timedf,
     $               primdf, sec1df, sec2df)
c     Default flag for BatchMode
                  mbatsc(irun) = mbatdf
                  lbtslp(irun) = .false.
c     No diffuse (tails) correction
                  ltdsrn(irun) = .false.
               endif
 1015       continue
         endif
      endif
c
c Print & check run definitions
      lallbf = .true.
      lanybf = .false.
      do 1020, irun = 1, numrun
c Reset reference run to CONSTANT in case it has not been done
         if (irun .eq. irefrn) then
            call setrsc(irun, modedf, +0, 1, 0.0, -1.0, -1.0, -1.0)
         endif
         if (.not.lbfrun(irun)) lallbf = .false.
         if (lbfrun(irun)) lanybf = .true.
         nrun = irunum(irun)
         write (lunout, 61020) nrun
61020    format(//' Run number ',I4,' consists of batches :-')
         k = 0

         call dna_table_start('batches')
         call dna_list_start('batch_numbers')

         do 1021, i = 1, nbtall
            if (irunbt(i) .eq. irun) then
c        This batch is in this run nrun
               k = k+1
               jbatrn(k,irun) = ibtall(i)
c        Set batch serial number within run
               isrrbt(i) = k
c        Clear matrix pointer
               jmtxbt(k,irun) = -1
            endif
 1021    continue
         nbatrn(irun) = k
         if (k .le. 0) then
            write (lunout, '(/a,i5/)')
     $         '**** No batches in run ', nrun
            call ccperr(1,'*** Empty run ***')
         endif

         call dna_integer_item('run_number', irun)
         do i = 1, k
            call dna_integer_item('batch', jbatrn(i,irun))
         end do
         call dna_list_end
         call dna_table_end         

         write (lunout, 61021) (jbatrn(i,irun), i=1,k)
61021    format(500(1x,16i7/))
c
         if (nptrun(irun) .gt. 0) then
            do 1022, j = 1, nptrun(irun)
c Exclusion/inclusion ranges
               if (lptrun(j, irun)) then
c         Exclude
                  write (lunout, 61023) 'Exclude',
     $                 zmnrun(j,irun), zmxrun(j,irun)
61023             format(1x,a,' rotation range ',f9.3,' to',f9.3)
               else
c         Include
                  write (lunout, 61023) 'Include',
     $                 zmnrun(j,irun), zmxrun(j,irun)
               endif
 1022       continue
         endif
         if (ltdsrn(irun)) then
c set default link pointer for diffuse correction parameters
            itdspt(irun) = irun
         endif
c
 1020 continue
c Set normalisation defaults
      if (knormk .lt. -999) then
         if (irefrn .gt. 0) then
            knormk = 0
         else 
            knormk = +1
         endif
      endif
      if (knormb .lt. -999) then
         if (irefrn .gt. 0) then
            knormb = 0
         else 
            knormb = -1
         endif
      endif
      if (HLaniso) then
c Anisotropic B, don't allow NORMALISE BEST
         if (knormb .lt. 0) knormb = +1
      endif
      if (knormb .gt. 0 .and. lanybf) then
c  Check that normalisation batch has B-factors on
         if (.not.lbfrun(knormb)) then
c  find another one which does have B-factors
            knormb = -1
            do irun = 1, numrun
               if (lbfrun(irun)) then
                  knormb = irun
               endif 
            enddo 
         endif
      endif
c
c Setup output dataset definitions
      call setdtsout(ndtskyin, istat)
      if (istat .gt. 0) then
         write (lunout, '(/a/a/)')
     $      '**** WARNING: no dataset defined in the output file ****',
     $      '**** You should use the NAME command to define one  ****'
      elseif (istat .lt. 0) then
         call dna_error('error in input')
         call ccperr(1,'***** Error in input *****')
      endif
c
c Reset number of output files if SPLIT, = number of output datasets
c  ndtsetsout is normally = ndatasets - 1 (ie excluding HKL_base), often = 1
c  ndtsetsout does not includes any reference set
      if (kfilou .gt. 0) then
         kfilou = ndtsetsout
      endif
      if (ndtsetsout .le. 1) then
         kfilou = 0
         ndtsetsout = 1
      endif
      if (kdtsbase .lt. 0) then
c Set base dataset if defined on BASE command
c If not set explicitly (kdtsbase = 0), it will be set later if required
         if (ndtsetsout .gt. 1) then
            call getdtsnum(xname_base, dname_base,
     $         xname_out, dname_out, ndtsetsout,
     $         kdtsbase)
            if (kdtsbase .lt. 0) then
               write (lunout, '(/a/)')
     $      '**** ERROR: dataset defined on BASE command does not exist'
               call ccperr(1,'*** Error in BASE command ***')
            endif
         endif
      else
         if (ndtsetsout .gt. 1) then
c  Define base dataset if not explicitly given
            call setbasedts
            if (kdtsbase .le. 0) then
               write (lunout, '(/a/a/a/)')
     $         '**** WARNING: unable to set BASE dataset automatically',
     $            '****        Please set it using the BASE command',
     $            '****        It has been set to dataset 1 by default'
               kdtsbase = 1
               call ccperr
     $            (2,'***  Base dataset defaulted to dataset 1 ***')
            endif
         endif
      endif
c
      if (kdtsbase .gt. 0) then
         call mkdtslab(kdtsbase, line)
         write (lunout, '(/a,a)')
     $      ' Base dataset for dispersive differences is ',
     $      line(1:lenstr(line))
      endif
c
c Make all runs active (this active flag is used in refout to select datasets)
      call setacr(0)
c
c Check consistency of Anomalous flags
      if (kanoms .gt. 0 .and. kanomo .eq.  0) then
         write (lunout, '(/a,a,a/a//)')
     $      ' ==== WARNING: you are keeping I+ and I- separate in ',
     $      'scaling, but ignoring anomalous differences in analysis',
     $      ' ====', ' ==== This is not sensible'
      endif
c
      if (imid .eq. 0) then
         write (lunout, '(/a/)')
     $        ' INTEGRATED intensities will be used'
         lsclpr = .false.
      elseif (imid .eq. -1) then
         write (lunout, '(/a/)')
     $        ' PROFILE_FITTED intensities will be used'
      elseif (imid .eq. -2) then
         write (lunout, '(/a/)')
     $        ' PROFILE_FITTED intensities will be used for fulls,',
     $        ' INTEGRATED intensities will be used for partials'
      elseif (imid .gt. 0) then
         write (lunout, '(/a,a/a/a,i2,a)')
     $      ' COMBINED intensities will be used: weighted mean of',
     $      ' profile-fitted & integrated intensities',
     $      '    I = w * Ipr + (1-w) * Iint',
     $      '    w = 1/(1+(Iint/Imean)**',ipower,')'
         if (imid .gt. +1) then
            write (lunout, '(a,i9)')
     $      '    Imean = ',imid
         endif
         write (lunout, '(/)')
      endif
      if (kprsum .eq. -1) then
         write (lunout, '(/a/)')
     $        ' Only fully-recorded reflections will be used in scaling'
      elseif (kprsum .eq. 0) then
         write (lunout, '(/a,a/)')
     $        ' Only fully-recorded reflections ',
     $      '(and previously summed partials) will be used in scaling'
      elseif (kprsum .eq. 1) then
         write (lunout, '(/a/)')
     $        ' Fully-recorded and summed partial reflections'//
     $        ' will be used in scaling'
         if (kprflg .lt. 0) then
            write (lunout, '(4x,a)')
     $     ' Summed partials will be checked for consistent MPART flags'
         elseif (ncmprt .gt. 0) then
            write (lunout, '(4x,a)')
     $ ' Summed partials will NOT be checked for consistent MPART flags'
         endif
         if (mxprwd .lt. 999) then
            write (lunout, '(4x,a,i4,a)')
     $   ' Summed partials must contain not more than',mxprwd,' parts'
         else
            write (lunout, '(4x,a,f5.1,a)')
     $   ' Maximum number of parts in summed partials equivalent to',
     $         wdprmx, ' degrees'
         endif
         if (iabs(kprflg) .eq. 1) then
            write (lunout, '(4x,a)')
     $       ' No test on total calculated fraction for summed partials'
         elseif (iabs(kprflg) .eq. 2) then
            if (prthld .lt. prlims) then
               write (lunout,
     $            '(4x,a,a,f8.3,a,f8.3/4x,a,f8.3,a,f8.3)')
     $        ' Summed partials accepted for scaling if total fraction',
     $            ' lies between',prlims,' and',prmaxs,
     $            ' scaled by total fraction if between', prthld,
     $            ' and', prlims
            else
               write (lunout, '(4x,a,a,f8.3,a,f8.3)')
     $        ' Summed partials accepted for scaling if total fraction',
     $            ' lies between', prlims,' and', prmaxs
            endif
            if (lsclpr) then
               write (lunout, '(/a/)')
     $   ' Profile-fitted intensities will be used for scaled partials'
            endif
         endif
         if (kprmis .ne. 0) then
            write (lunout, '(4x,a,a)')
     $         ' Partials with missing parts in the middle will',
     $         ' be accepted'
         else
            write (lunout, '(4x,a,a)')
     $         ' Partials with missing parts in the middle will',
     $         ' be rejected'
         endif
      elseif (kprsum .eq. 2) then
         write (lunout, '(/a,f10.4/)')
     $        ' Fully-recorded and scaled partial reflections'//
     $        ' will be used in scaling, with a minimum fraction = ',
     $        prthld
         if (lsclpr) then
            write (lunout, '(/a/)')
     $   ' Profile-fitted intensities will be used for scaled partials'
         endif
      endif
c
      if (kanoms .ge. +1) then
         write (lunout, '(/a/)')
     $      ' I+ and I- observations will be kept separate in scaling'
      endif
c
c Set resolution limits for different datasets, if specified
      if (numresdts .gt. 0) then
         do i = 1, numresdts
c Identify input dataset number (serial number in input list)
            xname_run = ''
            call getdtsnum(xname_run, res_dname(i),
     $         xname_in, dname_in, ndatasets, idtsnum)
            if (idtsnum .le. 0) then
               write (lunout, '(a)')
     $            ' ** Dataset not found or ambiguous **'
               go to 999
            endif
c   Set limits for all runs in that dataset
            do j=1,nrundts(idtsnum)
               irun = krundts(j,idtsnum)
               smin(irun) = smindts(i)
               smax(irun) = smaxdts(i)
            enddo
         enddo
      endif
c
c
c---- Resolution ranges, now set for all runs
c (default to file limits), get global limits
      smaxlm = 0.0
      sminlm = 10000.
      do 1100, irun = 1, numrun
         smaxlm = max(smaxlm, smax(irun))
         sminlm = min(sminlm, smin(irun))
 1100 continue
      srange = smaxlm/nsrang
c
      resmin = 1./sqrt(sminlm)
      resmax = 1./sqrt(smaxlm)
      write (lunout,'(/a,2f10.4/)')
     $   ' Overall resolution limits: ',resmin, resmax

      call dna_table_start('resolution_limits')
      call dna_list_start('overall')
      call dna_real_item('maximum', resmax)
      call dna_real_item('minimum', resmin)
      call dna_list_end
      call dna_table_end

C----harvest
C
       HRes1over = 1.0/sqrt(sminlm)
       HRes2over = 1.0/sqrt(smaxlm)
       HRh1 = sqrt(1.0/sminf)
       HRh2 = sqrt(1.0/smaxf)
C
      if (lreslm .gt. 0) then
         do 1110, irun = 1, numrun
cc            if (smin(irun) .ne. sminf .or. smax(irun) .ne. smaxf) then
            resmin = 1./sqrt(smin(irun))
            resmax = 1./sqrt(smax(irun))
            write (lunout,'(a,i3,a,2f10.4)')
     $         ' Resolution limits for run ',irunum(irun),' : ',
     $         resmin, resmax
cc            endif
 1110    continue
         write (lunout, '(/)')
      endif
c---- Print filter limits
      if (lscale) then
         write (lunout, '(//a/(7x,a,f12.6))')
     $        ' In solving least-squares equations:-',
     $        '               Eigenvalue filter limit:', filter
         if (ncycdamp .eq. 0 .or. damp .eq. 0.0) then
            write (lunout, '(7x,a/)')
     $         '              No damping of shifts'
         elseif (ncycdamp .lt. 0) then
            write (lunout, '(7x,a,f12.6/)')
     $         'Damping value added to all eigenvalues:', damp
         else
            write (lunout, '(7x,a,f12.6,a,i4,a/)')
     $         'Damping value added to all eigenvalues:', damp,
     $         '  for the first', ncycdamp, ' cycles'
         endif
c
c  Filter at least 2 eigenvalues if & only iff all runs have B-factors refined
         if (lallbf) then
            lfilt1 = .false.
            write (lunout, '(/a/)')
     $           ' At least TWO eigenvalues will be filtered'
         else
            lfilt1 = .true.
            write (lunout, '(/a/)')
     $           ' At least ONE eigenvalue will be filtered'
         endif
c Reflection skip
         if (nrfskp .gt. 1) then
            if (ncyskp .le. 0) ncyskp = max(ncyskp, ncycle-2)
            write (lunout, '(/a,i4,a,i4,a/)')
     $           ' For the first ', ncyskp,
     $           ' cycles, use only every ', nrfskp,
     $           '''th reflection (SKIP)'
         endif
c Weighting scheme
         if (kscwgtyp .eq. 1) then
            write (lunout, '(/a)')
     $         ' UNIT weights will be used in scaling'
         elseif (kscwgtyp .eq. 2) then
            write (lunout, '(/a)')
     $         ' VARIANCE weights will be used in scaling'
         elseif (kscwgtyp .eq. 3) then
            write (lunout, '(/a)')
     $         ' SQRTSCALE weights will be used in scaling'
         else
            write (lunout, '(/a)') ' **** Unrecognized weight type ****'
            call ccperr(1,'**** Unrecognized weight type ****')
         endif
      endif
c
c Set probability limits if required
      fractn = 1.1
      prblim = 3.0
      if (prblt  .lt. 0.0) then
         prblt  = max(fractn / Vwt , prblim)
      else
         if (prblt .lt. fractn / Vwt) then
            prblt = fractn / Vwt
            write (lunout, '(a,f10.4,a,a)')
     $           ' ==== WARNING: Prob_Limit_t reset to ',prblt,
     $           ' to avoid problem along Time direction',
     $           '(Rotation) ===='
         endif
      endif
      if (prblz  .lt. 0.0) then
         prblz  = max(fractn / Vwz , prblim)
      else
         if (prblz .lt. fractn / Vwz) then
            prblz = fractn / Vwz
            write (lunout, '(a,f10.4,a,a)')
     $           ' ==== WARNING: Prob_Limit_z reset to ',prblz,
     $           ' to avoid problem along Rotation direction',
     $           '(Rotation) ===='
         endif
      endif
      if (prblxy  .lt. 0.0) then
         prblxy  = max(sqrt(2.0) * fractn / Vwxy , prblim)
      else
         if (prblxy .lt. sqrt(2.0) * fractn / Vwxy) then
            prblxy = sqrt(2.0) * fractn / Vwxy
            write (lunout, '(a,f10.4,a,a)')
     $           ' ==== WARNING: Prob_Limit_xy reset to ',prblxy,
     $           ' to avoid problem in Detector direction',
     $           '(detector) ===='
         endif
      endif
c
c
c
      write (lunout, '(/5x,a,3(/5x,a,f12.2,14x,f10.4))')
     $ '              Smoothing factors   Maximum fractional distance',
     $     '        Time:',Vwt, prblt,
     $     '    Rotation:',Vwz, prblz,
     $     '    Detector:',Vwxy, prblxy
c
      if (lexcld .gt. 0) then
         if (lexcld .eq. 1) then
            i1 = 1
            i2 = 1
         elseif (lexcld .eq. 2) then
            i1 = 1
            i2 = numrun
         endif
         do 1200, i = i1, i2
c Exclusion limits set for all runs
            if (lexcld .eq. 1) then
               write (lunout, '(//a/)')
     $   '   Reflections will be excluded from scaling in all runs if:'
            else
               if (lchexc(i) .ne. 0) then
                  write (lunout, '(//a,i4,a/)')
     $   '   Reflections will be excluded from scaling in run',
     $           irunum(i),' if:'
               endif
            endif
c
            write (lunout,'(a,f10.3)')
     $           '     I .lt. sd(I) * ', sdImin(i)
            if (abs((sdImax(i)/dfsdmx)-1.0) .gt. 0.01)
     $         write (lunout,'(a,f10.3)')
     $         '     I .gt. sd(I) * ', sdImax(i)
            if (abs((abImax(i)/dfabmx)-1.0) .gt. 0.01)
     $         write (lunout,'(a,f10.3)')
     $         '     I .gt. ', abImax(i)
 1200    continue
      else
         write (lunout, '(//a/)')
     $    '   Reflections will be excluded from scaling in all runs if:'
         i = 1
         write (lunout,'(a,f10.3)')
     $      '     I .lt. sd(I) * ', sdImin(i)
      endif
c
      if (.not. lrdidx .and. ifilou .lt. +4) then
         write (lunout,'(a,a)')
     $  ' **** ORIGINAL indices only valid with UNMERGED, ignored ****'
         lrdidx = .true.
      endif
      if (mbeams .ne. 0 .and. ifilou .lt. +4) then
         write (lunout,'(a,a)')
     $  ' **** BEAMS option only valid with UNMERGED, ignored ****'
         mbeams = 0
      endif
c Output exclude defaults
      if (krsexc .lt. 0) then
         if (ifilou .eq. +1 .or. ifilou .eq. +5) then
            krsexc =  0
         else
            krsexc = +1
         endif
      endif
c
      if (eacmax .gt. 0.0 .or. emnprb .gt. 0.0) then
c E limits
         if (emnprb .gt. 0.0) then
c  E probability set by user: set E limit
c    Allow inverse probability (.gt. 1), invert it
            if (emnprb .gt. 1.0) emnprb = 1.0/emnprb
            eacmax = sqrt(-alog(emnprb))
            call fndepb(eacmax, prob, ecnmax)
         else
c  Emax set by user
c   Get equivalent for centric data
            call fndepb(eacmax, prob, ecnmax)
            emnprb = exp(-eacmax**2)
         endif
         write (lunout, 6201) eacmax, ecnmax, emnprb
 6201    format(/,
     $      ' Reflections judged implausibly large will be rejected',
     $      ' from scaling and merging',/,
     $      '     Maximum normalised F (ie E) for acentric reflection',
     $      f10.3,/,
     $      '     Maximum normalised F (ie E) for centric reflection ',
     $      f10.3,/,
     $      '     Minimum probability before reflection is rejected  ',
     $      e10.3/)
      endif
c FLAG acceptances
      if (ncflag .gt. 0 .and. ltsmfl) then
         write (lunout, '(/a,a)')
     $      'Conditional acceptance in merging step of observations ',
     $      ' with error FLAG column set'
         if (bgrlim .gt. 0.0) then
            bgrlim = min(bgrlim, 9.9)
            write (lunout, '(5x,a,f8.3)')
     $         ' ** Accept if BGration   < ', bgrlim
         endif
         if (pkrlim .gt. 0.0) then
            write (lunout, '(5x,a,f8.3)')
     $         ' ** Accept if PKration   < ', pkrlim
         endif
         if (grdlim .gt. 0.0) then
            grdlim = min(grdlim, 0.99)
            write (lunout, '(5x,a,f8.3)')
     $         ' ** Accept if BGgradient < ', grdlim
         endif
         if (lovld) write (lunout, '(5x,a)')
     $         ' ** Accept profile-fitted overloads'
         if (ledge) write (lunout, '(5x,a)')
     $         ' ** Accept edge reflections'
      endif
c  Rejection limits
c
c Check for sensible values of "ALL" limits: if set, these should be
c larger than the limits within I+ & I-
c Only relevent if anomalous on
      write (lunout, '(/a/a/)')
     $   ' Outlier rejection limits',
     $   ' ========================'
c Scaling limits
      write (lunout, '(a)') ' --| In scaling |--'
      if (kanoms . ne. 0) then
         write (lunout, '(a)')
     $      '   ==== Within each I+ & I- set ===='
      else
         write (lunout, '(a)')
     $      '   ==== For all observations including I+ & I-  ===='
      endif 
      write (lunout, 6210) sdrejr, sdrejr2
 6210 format (
     $   '    Reflections measured 3 or more times:', f10.3,
     $   '   maximum deviation from weighted mean of all other',
     $   ' observations'/
     $   '    Reflections measured twice:', f10.3,
     $   '   maximum deviation from weighted mean')
c
      if (jolcyc .lt. 0) then
         if (lrstr) then
c  If restoring scales, outlier rejection from start
            jolcyc = 1
         else
c   else no outlier rejection on first cycle
            jolcyc = 2
         endif
      endif
      write (lunout, '(1x,i10,a)') jolcyc,
     $   '   first refinement cycle to reject outliers'
c Rejection limits for merging step
      write (lunout, '(/a)') ' --| In merging |--'
      if (kanomo .ne. 0 .and. sdrejma .ne. 0.0) then
c  Reset 2-observation value if unset
         if (sdrejm2a .eq. 0.0) sdrejm2a = sdrejma
c  Anomalous on, possible extra rejection (if sdrejma .gt. 0)
         if (abs(sdrejma)  .lt. sdrejm .or.
     $      abs(sdrejm2a) .lt. sdrejm2) then
            write (lunout, 6120)
 6120        format(
     $         ' ==== WARNING: rejection limit for all data is ',
     $         'tighter than than within I+|I- sets',/,
     $         ' ==== This is not sensible'/)
         endif
         write (lunout, 6121) sdrejm, sdrejm2
 6121    format (
     $      '   ==== Within each I+ & I- set ====',/,
     $      '    Reflections measured 3 or more times:', f10.3,
     $      '   maximum deviation from weighted mean of all other',
     $      ' observations'/
     $      '    Reflections measured twice:', f10.3,
     $      '   maximum deviation from weighted mean')
         if (sdrejma .lt. 0.0) then
            if (sdrejm2a .gt. 0.0) then
c  Reset negative (not normal case)
               sdrejm2a = -abs(sdrejm2a)
            endif 
            write (lunout, 6122)
 6122       format (
     $      '   ==== For all observations including I+ & I- ====',/,
     $      '    Maximum deviation will be determined from observed',
     $      ' anomalous differences, see below'/)
         else
            write (lunout, 6123) sdrejma, sdrejm2a
 6123       format (
     $      '   ==== For all observations including I+ & I- ====',/,
     $      '    Reflections measured 3 or more times:', f10.3,
     $      '   maximum deviation from weighted mean of all other',
     $      ' observations'/
     $      '    Reflections measured twice:', f10.3,
     $      '   maximum deviation from weighted mean'/)
         endif
      else
         if (kanomo . ne. 0) write (lunout, '(a)')
     $      '   ==== Within each I+ & I- set ===='
         write (lunout, 6124) sdrejm, sdrejm2
 6124    format (
     $      '    Reflections measured 3 or more times:', f10.3,
     $      '   maximum deviation from weighted mean of all other',
     $      ' observations'/
     $      '    Reflections measured twice:', f10.3,
     $      '   maximum deviation from weighted mean')
         if (kanomo . ne. 0) write (lunout, '(a/)')
     $      '   ==== No outlier check will be made between I+ & I- ===='
      endif
c  REJECT SEPARATE makes no sense if there is only one dataset
      if (ndtsetsout .eq. 1) lanbyd = .false.
      if (lanbyd) then
         write (lunout, '(/a,a/)')
     $      ' Deviations and outliers will be checked ',
     $      'only within each dataset'
      endif
c-harvest-mod
      Hsdrej = sdrejm
      Hsdrej2 = sdrejm2
      Hsdreja = sdrejma
      Hsdrej2a = sdrejm2a
c
c Set up links (if present)
      if (nprtds .gt. 0) then
         if (.not. lnktds) then
c  Tails link not set, add default LINK TAILS ALL
            nlink = nlink+1
            klink(1,nlink) = -1
            klink(2,nlink) = -1
            klink(3,nlink) = +1
         endif
      endif
      if (nprsph .gt. 0) then
c On SURFACE, default is to link runs belonging to the same dataset
         if (.not. lnksph) then
            do j = 1, ndtsetsout
               if (nrundts(j) .gt. 1) then
                  do i = 1, nrundts(j)
                     if (nrnsph(krundts(i,j)) .gt. 0) then
                        k = i
                        go to 1250
                     endif 
                  enddo 
 1250             do i = k+1, nrundts(j)
c    Link each run to the first for the dataset
c which has SURFACE defined
                     if (nrnsph(krundts(i,j)) .gt. 0) then
                       nlink = nlink+1
                       klink(1,nlink) = irunum(krundts(i,j))
                       klink(2,nlink) = irunum(krundts(k,j))
                       klink(3,nlink) = +2
                    endif
                  enddo
               endif
            enddo
         endif
      endif
c
      if (nlink .gt. 0) then
         call setlnk(nlink, klink, istat)
         if (istat .ne. 0) then
            call ccperr(1,'**** Invalid LINK command ****')
         endif
      endif
      if (nprtds .gt. 0) then
         if  (lptfix(1)) write (lunout, '(a)')
     $      ' TAILS parameter  V fixed'
         if  (lptfix(2)) write (lunout, '(a)')
     $      ' TAILS parameter A0 fixed'
         if  (lptfix(3)) write (lunout, '(a)')
     $      ' TAILS parameter A1 fixed'
      endif
c Setup scale layout
c   nprtds counted in s/r setscl
      nprtds = 0
      nprsph = 0
      call setscl(istat)
      if (istat .ne. 0) then
         call dna_error('internal error')
         call ccperr(1,'**** Failed in SETSCL ****')
      endif
c Maximum width for partials, 
c  set maximum number of parts = Max width / Min slice 
      if (phirngmin .gt. 0.001) then
         n = nint(wdprmx / phirngmin)
         if (mxprwd .gt. 999) mxprwd = n
         if (mxprwo .gt. 999) mxprwo = n
      endif
c Detector exclusions
      if (mrjrct .ne. 0 .or. mrjarc .ne. 0) then
         if (.not. lcddet) then
            write (lunout, '(/a,a/)')
     $         ' **** EXCLUDE RECTANGLE|ARC command requires valid ',
     $         'XDET, YDET columns in file ****'
            call ccperr(1,'*** Invalid EXCLUDE RECTANGLE|ARC ***')
         endif
c No exclusion in reference run
         if (irefrn .gt. 0) then
            nrjrct(irefrn) = 0
            nrjarc(irefrn) = 0
         endif
         if (mrjrct .ne. 0) then
            write (lunout, '(/a/13x,a/)')
     $   ' Exclude the following detector areas from all calculations:',
     $         '      Xmin      Xmax      Ymin      Ymax'
            if (mrjrct .lt. 0) then
               irun = 1
               if (irun .eq. irefrn) irun = numrun
               if (nrjrct(irun) .le. 0) then
c              shouldn't happen
                  call ccperr(1,'*** No rectangles defined ***')
               endif
               do 1080, k = 1,nrjrct(irun)
                  write (lunout, '(13x,4f10.1)')
     $               ((rejrct(i,j,k,irun),i=1,2),j=1,2)
 1080          continue
            else
               do 1081, irun = 1, numrun
                  if (irun .ne. irefrn .and. nrjrct(irun) .gt. 0) then
                     do 1082, k = 1,nrjrct(irun)
                        write (lunout, '(2x,a,i4,3x,4f10.1)')
     $                     'Run ', irunum(irun),
     $                     ((rejrct(i,j,k,irun),i=1,2),j=1,2)
 1082                continue
                  endif 
 1081          continue
            endif
         endif
         if (mrjarc .ne. 0) then
            write (lunout, '(/a/13x,a/)')
     $   ' Exclude the following detector areas from all calculations:',
     $         '     Xcen      Ycen     Radius      Phi1      Phi2'
            if (mrjarc .lt. 0) then
               irun = 1
               if (irun .eq. irefrn) irun = numrun
               if (nrjarc(irun) .le. 0) then
c              shouldn't happen
                  call ccperr(1,'*** No arcs defined ***')
               endif
               do k = 1,nrjarc(irun)
                  if (linarc(k,irun)) then
                     line = 'exclude inside radius of arc'
                  else
                     line = 'exclude outside radius of arc'
                  endif
                  write (lunout, '(13x,5f10.1,4x,a)')
     $               (rcrarc(i,k,irun),i=1,3),
     $               (phiarc(i,k,irun)/dtor,i=1,2),
     $               line(1:lenstr(line))
               enddo
            else
               do irun = 1, numrun
                  if (irun .ne. irefrn .and. nrjarc(irun) .gt. 0) then
                     do k = 1,nrjarc(irun)
                        if (linarc(k,irun)) then
                           line = 'exclude inside radius of arc'
                        else
                           line = 'exclude outside radius of arc'
                        endif
                        write (lunout, '(2x,a,i4,3x,5f10.1,4x,a)')
     $                     'Run ', irunum(irun),
     $                     (rcrarc(i,k,irun),i=1,3),
     $                     (phiarc(i,k,irun)/dtor,i=1,2),
     $                     line(1:lenstr(line))
                     enddo
                  endif 
               enddo
            endif
c  Square radius
            do irun = 1, numrun
               if (irun .ne. irefrn .and. nrjarc(irun) .gt. 0) then
                  do k = 1,nrjarc(irun)
                     rcrarc(3,k,irun) = rcrarc(3,k,irun)**2
                  enddo
               endif
            enddo
         endif
      endif
c
c
c---- SD corrections
      write (lunout, 6109)
 6109 format (//
     $     ' Standard deviations will be adjusted to ',
     $   'SdFac * Sqrt[sd(I)**2 + SdB I + (SdAdd I)**2] as follows:-'/
     $     '                        Fulls        ',
     $     '                  Partials'/
     $     '        Run    SdFac       SdB     SdAdd',
     $     '        SdFac      SdB      SdAdd'/)
      do 1090, irun = 1, numrun
         write (lunout, '(1x,i10,3f10.5,3x,3f10.5)')
     $        irunum(irun), ((sdfact(i,irun,j), i = 1,maxsdf),j=1,2)
 1090 continue
      write (lunout, '(/)')
c
c Print layout
      call prtsly(istat)
      if (istat .ne. 0) then
         call dna_error('internal error')
         call ccperr(1,
     $      '**** Failed in setting up corner correction ****')
      endif
c
      if (ktimcor .gt. 0) then
         if (numrun .gt. 1) then
            write (lunout, 6111)
 6111       format (' **** ERROR: zerodose correction cannot be ',
     $         'made with more than one run ****')
            call ccperr(1,'*** multiple runs in zerodose mode ***')
         endif
         write (lunout, 6110)
 6110    format (//' Intensities will be extrapolate back to zero time')
         if (ktimcor .eq. +2)
     $      write (lunout, '(a)') '  with damped correction factors'
         write (lunout, '(/)')
      endif
c
c Set up ties (if any)
      if (lscale .and.
     $   (ntiez .ne. 0 .or. ntiexy .ne. 0 .or. ntiesrf .ne. 0 
     $   .or. ntiebf .ne. 0 .or. ntieA1 .ne. 0)) then
         call settie
      endif
c
c Set flag for initial analysis
c kinisc =  0  do initial analysis unless RESTORE
c        = -1  don't do initial analysis
c        = +1  do initial analysis even if RESTORE
      if (kinisc .eq. +1) then
         lanlin = .true.
      elseif (kinisc .eq. -1) then
         lanlin = .false.
      else
         if (lrstr) lanlin = .false.
      endif
c
      if (lrstr) then
c   Restore scales
         call rstrsc
      endif
c
      return
c
c
c
 980  write (lunout, '(a,a/)')
     $     ' **** Unrecognized or ambiguous sub-keyword:',
     $     line(ibeg(n):iend(n))
      go to 999
c
 990  write (lunout,'(A/)') ' ** Missing or illegal value **'
 999  ierr = ierr+1
      go to 10

      end
      subroutine linprb(iwrprb, linnum, range)
c     ==================================
c
c  Draw lines
c
c linnum      line number of this dataset
c range       maximum/minimum value
c
c *** For xmgr ***
c
      integer iwrprb, linnum
      real range
c
      if (linnum .lt. 10) then
         write (iwrprb, 6001) linnum
 6001    format ('@ s',i1,' color 1')
      else
         write (iwrprb, 6002) linnum
 6002    format ('@ s',i2,' color 1')
      endif
      call strppr(iwrprb)
      write (iwrprb, 6010) -range,-range,range,range
 6010 format (2f7.1/2f7.1)
      call endppr(iwrprb)
      return
      end
      subroutine lisran(ntok,itok,cvalue,fvalue,ityp,llist,vlist)
c     ===========================================================
c     
c     Parse number list or number range
c     
c     On entry:
c     ntok      number of tokens in line
c     itok      first one to use
c     cvalue    character values from Parser
c     fvalue    number values from Parser
c     ityp      number types from Parser
c     
c     On exit:
c     itok      next token to use, = 0 if all tokens used
c     llist     .lt. 0 error, eg too few tokens for range
c     = 0  number range, returned in vlist(1) & (2)
c     .gt. 0, number list, llist entries in vlist
c     vlist     range or list
c     
      integer ntok,itok,ityp(ntok),llist
      real fvalue(ntok), vlist(*)
      character*(*) cvalue(ntok)
c     
      integer i
      logical range
c     
      range = .false.
      if (itok+2 .le. ntok) then
         if (cvalue(itok+1) .eq. 'TO') then
c     Number range
            if (ityp(itok) .ne. 2 .or. ityp(itok+2) .ne. 2) then
c     Illegal number
               llist = -1
               return
            else
               vlist(1) = fvalue(itok)
               vlist(2) = fvalue(itok+2)
               llist = 0
               itok = itok + 3
               range = .true.
            endif
         endif
      endif
      if (.not.range) then
c     Number list
         llist = 0
         do 10, i = itok, ntok
            if (ityp(i) .ne. 1) then
c     Store number
               llist = llist+1
               vlist(llist) = fvalue(i)
            else
c     String found, this terminates list
               itok = i
               if (llist .le. 0) llist = -1
               return
            endif
 10      continue
         itok = 0
      endif
      return
      end
c
c
c
      subroutine lnkrun(lnktyp, link, irunm1, irunm2, istat)
c     =====================================================
c
c LINK or UNLINK parameters for 2 runs
c
c On entry:
c  lnktyp           link type = 1 TAILS = 2 SURFACE
c  link             .true. to link, .false. to unlink
c  irunm1, irunm2   run numbers to [un]link: link irunm1 to  irunm2
c
c On exit:
c  istat            =  0  OK
c                   = -1  run identification or no tails failure
c                   = -2  illegal type
c                   = +1  warning on invalid UNLINK
c                   = +2  ignored LINK to reference run
c
      logical link
      integer lnktyp, irunm1, irunm2, istat
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'scales.fh'
      include 'sphhrm.fh'
c
      integer ir, irun1, irun2
      character*8 ltypes(2)
      data ltypes/'TAILS', 'SURFACE'/
c
      istat = 0
      if (lnktyp .lt. 1 .or. lnktyp .gt. 2) then
         istat = -2
         return
      endif
c Check & get internal run numbers
      call chkrun(irunm1, irun1)
      if (irun1 .le. 0) then
         istat = -1
         return
      endif
      if (irun1 .eq. irefrn) then
         istat = +2
         return
      endif
      if ((lnktyp .eq. 1 .and. nrntds(irun1) .le. 0) .or.
     $   (lnktyp .eq. 2 .and. nrnsph(irun1) .le. 0)) then
         write (lunout, '(/a,i4,a,a,a/)') 
     $      ' **** LINK attempted with no relevent parameters, run',
     $      irun1,', type ',ltypes(lnktyp),' ****'
         istat = -1
      endif
      call chkrun(irunm2, irun2)
      if (irun2 .le. 0) then
         istat = -1
         return
      endif
      if (irun2 .eq. irefrn) then
         istat = +2
         return
      endif
      if ((lnktyp .eq. 1 .and. nrntds(irun2) .le. 0) .or.
     $   (lnktyp .eq. 2 .and. nrnsph(irun2) .le. 0)) then
         write (lunout, '(/a,i4,a,a,a/)') 
     $      ' **** LINK attempted with no relevent parameters, run',
     $      irun2,', type ',ltypes(lnktyp),' ****'
         istat = -1
         return
      endif
      if (istat .ne. 0) return
c Enforce irun1 .gt. irun2
      if (irun1 .lt. irun2) then
         ir = irun1
         irun1 = irun2
         irun2 = ir
      endif
c
      if (lnktyp .eq. 1) then
c TAILS
         if (link) then
c Link: set pointer
            itdspt(irun1) = irun2
         else
c UnLink
            if (itdspt(irun1) .eq. irun2) then
               itdspt(irun1) = irun1
            else
               istat = +1
            endif
         endif
      elseif (lnktyp .eq. 2) then      
c SURFACE (spherical harmonics)
         if (link) then
c Link: set pointer
            isphpt(irun1) = irun2
         else
c UnLink
            if (isphpt(irun1) .eq. irun2) then
               isphpt(irun1) = irun1
            else
               istat = +1
            endif
         endif
      endif
      return
      end
c
c  map_sdparam.f
c
c Setup SD correction parameters
c Parameters used by s/r corvar are in common /sdfacc/
c   sdfact(1, irun, m)  Sdfac
c   sdfact(2, irun, m)  SdB
c   sdfact(3, irun, m)  SdAdd
c  m = 1 full, = 2 partial
c Set common block parameters ready for corvar
c
c  Mapping types:
c   lsdtype =  0    same parameters for all runs, fulls & partials
c           = +1    independent parameters for each run,
c                    and for fulls & partials if present
c           = +2    same SdB, SdAdd for all, different Sdfac
c
      subroutine map_sdparam(params, nparam, istat)
c    ====================
c
c  Set sdfact parameters in common /sdfacc/ from params array
c in refinement
c
c Return istat = -1 if any parameter is out of range
c  else istat = 0
c
c NOTE that s/r setsdstart essentially does the inverse mapping, so
c both routines must be kept in step in any alterations
c
      implicit none
      integer nparam, istat
      double precision params(nparam)

      include 'parameter.fh'
      include 'sdcdata.fh'
      include 'sdfacc.fh'
      include 'rundef.fh'

      integer i,k,l,nfac,irun,k1,k2

      istat = 0

      if (lsdtype .eq. 0) then
c      =  0   same for all runs & fulls & partials
         nfac = 3
         do irun=1,numrun
            do k = 1,2
               l = 1
               do i=1,nfac
                  if (i .eq. 1 .and. params(l) .le. 0.0) then
c                 Sdfac must be > 0
                     istat = -1
                     return
                  endif 
                  if (.not. (i .eq. 2 .and. fixSdB)) then
                     sdfact(i,irun,k) = params(l)
                     l = l+1
                  endif
               enddo
            enddo
         enddo
      else if (lsdtype .eq. +1) then
c      = +1  3 independent parameters for each run &
c            full/partial
         l = 1
         nfac = 3
         do irun=1,numrun
            if (kfullpart(irun) .eq. 0) then
c Fulls only
               k1 = 1
               k2 = 1
            else if (kfullpart(irun) .eq. +1) then
c Partials only
               k1 = 2
               k2 = 2
            else if (kfullpart(irun) .eq. +2) then
c Fulls & partials
               k1 = 1
               k2 = 2
            endif
            do k = k1,k2
               do i=1,nfac
                  if (i .eq. 1 .and. params(l) .le. 0.0) then
c                 Sdfac must be > 0
                     istat = -1
                     return
                  endif 
                  if (.not. (i .eq. 2 .and. fixSdB)) then
                     sdfact(i,irun,k) = params(l)
                     l = l+1
                  endif
               enddo
            enddo
         enddo
      else if (lsdtype .eq. +2) then
c           = +2    same SdB, SdAdd for all, different Sdfac
         l = 1
         nfac = 3
         do irun=1,numrun
            if (kfullpart(irun) .eq. 0) then
c Fulls only
               k1 = 1
               k2 = 1
            else if (kfullpart(irun) .eq. +1) then
c Partials only
               k1 = 2
               k2 = 2
            else if (kfullpart(irun) .eq. +2) then
c Fulls & partials
               k1 = 1
               k2 = 2
            endif
            do k = k1,k2
               do i=1,nfac
                  if (i .eq. 1) then
                     if (params(l) .le. 0.0) then
c                 Sdfac must be > 0
                        istat = -1
                        return
                     endif 
                     sdfact(i,irun,k) = params(l)
                     l = l+1
                  else if (irun .eq. 1) then
c  SdB & SdAdd for run 1
                     if (.not. (i .eq. 2 .and. fixSdB)) then
                        sdfact(i,irun,k) = params(l)
                        l = l+1
                     endif
                  else
c     ... and the same for subsequent runs
                     sdfact(i,irun,k) = sdfact(i,1,k)
                  endif
               enddo
            enddo
         enddo
      else
         call ccperr(1,'*** Illegal lsdtype in map_sdparam ***')
      endif
      return
      end
c
c
c
      subroutine setsdstart(points, parinc)
c     =====================
c
c Set start points for optmisation
c
c On entry:
c  parinc(3)     increments for each parameter from start values
c
c On exit:
c  points        parameter array for simplex
c
      implicit none
      double precision points(*)
      real parinc(3)

      include 'parameter.fh'
      include 'sdcdata.fh'
      include 'rundef.fh'

      integer irun, k
c DEBUG
c      write (6,*) 'FixSdB',fixsdb

      if (lsdtype .eq.  0) then
         nsdparam = 3
         if (fixSdB) nsdparam = 2
      else if (lsdtype .eq. +1 .or. lsdtype .eq. +2) then
c Work out number of parameters
         nsdparam = 0
         do irun = 1, numrun
            if (kfullpart(irun) .eq. 0) then
c Fulls only
               k = 1
            else if (kfullpart(irun) .eq. +1) then
c Partials only
               k = 1
            else if (kfullpart(irun) .eq. +2) then
c Fulls & partials
               k = 2
            endif
            if (irun .eq. 1 .or. lsdtype .eq. +1) then
               if (fixSdB) then
c               Just SdFac & SdAdd
                  nsdparam = nsdparam + 2 * k
               else 
                  nsdparam = nsdparam + 3 * k
               endif
            else
               nsdparam = nsdparam + k
            endif
         enddo
      else 
         call ccperr(1,'*** Illegal lsdtype in setsdstart ***')
      endif
      call setsdstart2(points, nsdparam, parinc)
      return
      end
c
c
c
      subroutine setsdstart2(points, nsdp, parinc)
c     ======================
c
c Set up nsdp+1 set of parameters to use as start points for minimisation
c
c On entry:
c  nsdp    number of parameters
c  parinc(3)     increments for each parameter from start values
c
c On exit:
c  points(nsdp, nsdp+1)     nsdp+1 sets
c
      implicit none
      integer nsdp
      double precision points(nsdp, nsdp+1)
      real parinc(3)
c
      include 'parameter.fh'
      include 'sdcdata.fh'
      include 'rundef.fh'
      include 'sdfacc.fh'
c
      integer i,j,k,l,n, irun, nprun, nps
c
      real sdf(maxsdf), sdfinc(maxsdf)
c
c  Allowed types are 0 -> MAXTYPE
      integer MAXTYPE
      data MAXTYPE/2/
c
      if (lsdtype .lt. 0 .or. lsdtype .gt. MAXTYPE) then 
         call ccperr(1,'*** Illegal lsdtype in setsdstart ***')
      endif

c   steps in each direction (SdFac, SdB, SdAdd) for points
c  all multiplied by scale to allow restart
      sdfinc(1) = parinc(1)
      sdfinc(2) = parinc(2)
      if (fixSdB) sdfinc(2) = 0.0
      sdfinc(3) = parinc(3)
      nps = 3
c
c Total number of parameters should be a multiple of nps
      n = nps
      if (fixSdB) n = 2
      if (mod(nsdparam,n) .ne. 0) then
         write (6,'(/a,i8/)')
     $      '*** Error in number of parameters in setsdstart2 ***',
     $      nsdparam
         call ccperr(1,'*** Error in setsdstart2 ***')
      endif
      
      if (lsdtype .eq. 0) then
c ----------------------  type 0 ----------------------------------
c  Type 0, same values for all runs etc
c  Average current (input or default) SD corrections as start points
         call ccpzr(sdf, maxsdf)
         n = 0
         do irun = 1,numrun
            do i = 1,maxsdf
               if (kfullpart(irun) .ne. +1) then
c  Fulls
                  sdf(i) = sdf(i) + sdfact(i,irun,1)
                  n = n+1
               endif
               if (kfullpart(irun) .ge. +1) then
c  Partials
                  sdf(i) = sdf(i) + sdfact(i,irun,2)
                  n = n+1
               endif
            enddo
         enddo 
         if (n .eq. 0) then
            sdf(1) = 1.0
         else
            do i = 1,maxsdf
               sdf(i) = sdf(i)/float(n/maxsdf)
            enddo
         endif
c
c first point is just the base (starting) parameters
         do i=1,nsdparam
            k = mod(i-1,nps)+1
            if (i .eq. 2 .and. fixSdB) k = k+1
            points(i,1) = sdf(k)
         enddo
         l=1
c DEBUG
c         write (6,'(a,i4,100f10.2)') 'l, points',
c     $      l, (points(i,l), i=1,nsdparam)
c Remaining points are generated by adding one increment at a time
         do j=1,nsdparam
c  l'th point
            l = j+1
            do i=1,nsdparam
               k = mod(i-1,nps)+1
               if (i .eq. 2 .and. fixSdB) k = k+1
               points(i,l) = sdf(k)
               if (i .eq. j) then
                  points(i,l) = points(i,l) + sdfinc(k)
               endif
            enddo
c DEBUG
c            write (6,'(a,i4,100f10.2)') 'l, points',
c     $         l, (points(i,l), i=1,nsdparam)
         enddo 
      else
c ----------------------  type 1 or 2 --------------------------
c first point is just the base (starting) parameters
c Remaining points are generated by adding one increment at a time
c 
c DEBUG
c         write (6,*) 'nsdparam', nsdparam
         do l=1,nsdparam+1
            j = 1
            do irun = 1,numrun
               n = maxsdf
               if (lsdtype .eq. 2 .and. irun .gt. 1) then
c  Type 2, change only SdFac for runs after 1st
                  n = 1
               endif 
c  Fulls
               if (kfullpart(irun) .ne. +1) then
                  do i = 1,n
                     if (.not. (i .eq. 2 .and. fixSdB)) then
                        points(j,l) = sdfact(i,irun,1)
                        if (j .eq. l-1) then
c      The one to increment
c       cannot be first point (l-1 = 0 .ne. j)
                           points(j,l) = points(j,l) + sdfinc(i)
                        endif
                        j = j+1
                     endif
                  enddo
               endif
c  Partials
               if (kfullpart(irun) .ge. +1) then
                  do i = 1,n
                     if (.not. (i .eq. 2 .and. fixSdB)) then
                        points(j,l) = sdfact(i,irun,2)
                        if (j .eq. l-1) then
c      The one to increment
c       cannot be first point (l-1 = 0 .ne. j)
                           points(j,l) = points(j,l) + sdfinc(i)
                        endif
                        j = j+1
                     endif
                  enddo
               endif
            enddo
c DEBUG
c            write (6,'(a,i4,100f10.2)') 'l, points',
c     $         l, (points(i,l), i=1,nsdparam)
         enddo
      endif
      return
      end
      subroutine menint(nrs, ffrf, sdffrf, ghl, useflg,
     $     wtrf, aih, sdih, uh, nused)
c     =============================================
c
c  Calculate mean intensity aih &  uh = Sum(w * g**2)
c
c
c  On entry:
c     nrs       number of observations
c     ffrf      intensities
c     sdffrf    corrected sd(Ihl)
c     ghl       dividing scales
c     wtrf      weights eg = 1 /  variance
c     useflg    .true. to use this observation
c
c  On exit:
c     aih       mean intensity Ih
c     sdih      sd (mean intensity Ih)
c     uh        Sum(w * g**2)
c     nused     number of observations used in mean
c
      implicit none
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
c
      integer nrs, nused
      real ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs),
     $     aih, sdih, uh
      logical useflg(nrs)
c
      integer l
      real    sumwgI, varI
c
c
      uh  = 0.0
      aih = 0.0
      sdih = 0.0
      nused = 0
      if (nrs .le. 0) return
c
      sumwgI = 0.0
c
      do 10, l = 1, nrs
c
         if (useflg(l)) then
            sumwgI= sumwgI + wtrf(l) * ghl(l) * ffrf(l)
            uh  =  uh + wtrf(l) * (ghl(l)**2)
            nused = nused + 1
         endif
 10   continue
      if (uh .ne. 0.0) then
         aih = sumwgI/uh
         sdih = sqrt(1./uh)
      endif
c
      return
      end
      SUBROUTINE MKBMAT(CELL, BMAT, LAMDA, LAXV)
C     ==========================================
C
C   Makes reciprocal cell orthogonalization matrix BMAT from
C   direct cell CELL and wavelength LAMDA, using permutation
C   flags LAXV
C
C Based on subroutines CONDTR & MATFST
C     A. Messerschmidt
C     Max-Planck-Institut fuer Biochemie
C     D8033 Martinsried
C     West Germany
C
C     FORTRAN 77: No exceptions.
C
C  Phil Evans  April 1988
C
C  Dummy variables
      REAL CELL(6), BMAT(3,3), LAMDA
      INTEGER LAXV(3)
C
C Local variables
      INTEGER LAXS(3)
C
      REAL           PARM(9), STAR(9), WINKEL(3)
C
      REAL AA,BB,CC,CA,CB,CG,SA,SB,SG
      REAL AS,BS,CS,CAS,CBS,CGS,SAS,SBS,SGS
      EQUIVALENCE    (STAR(1), AS), (STAR(2), BS), (STAR(3), CS),
     *             (STAR(4),CAS), (STAR(5),CBS), (STAR(6),CGS),
     *             (STAR(7),SAS), (STAR(8),SBS), (STAR(9),SGS)
C
      REAL SINE, COSANG, PV, PVSTAR, DG2RD
      INTEGER I, LAX, LUP, LIN
C
C     Multiply by this to convert degrees to radians:
C
      PARAMETER (DG2RD = 3.141592654 / 180.)
C
C     **** Inline function definitions ****
C
      SINE(COSANG) = SQRT( 1. - COSANG * COSANG)
C
      AA = CELL(1)
      BB = CELL(2)
      CC = CELL(3)
      CA = COS ( CELL(4) * DG2RD )
      CB = COS ( CELL(5) * DG2RD )
      CG = COS ( CELL(6) * DG2RD )
      SA = SIN ( CELL(4) * DG2RD )
      SB = SIN ( CELL(5) * DG2RD )
      SG = SIN ( CELL(6) * DG2RD )
C
C     Look at those equivalences above!
C
      PV = SQRT(1. - CA * CA - CB * CB - CG * CG + 2. * CA * CB * CG)
C
C * AS,BS,CS,CAS,CBS,CGS * RECIPROCAL LATTICE (A*,B*,C*,COS(ALPHA*),...)
C
      AS = SA / (AA * PV)
      BS = SB / (BB * PV)
      CS = SG / (CC * PV)
      CAS = (CB * CG - CA) / (SB * SG)
      CBS = (CG * CA - CB) / (SG * SA)
      CGS = (CA * CB - CG) / (SA * SB)
      DO 200 I = 4, 6
C
         STAR(I+3) = SINE(STAR(I))
200   CONTINUE
C
      DO 300 I = 1, 3
         WINKEL(I) = ACOS(STAR(I+3)) / DG2RD
C
C        * Normalize Ewald-sphere, radius = 1
C
         STAR(I) = STAR(I) * LAMDA
         PARM(I+3) = STAR(I)
         PARM(I+6) = WINKEL(I) * DG2RD
C
300   CONTINUE
C
      PVSTAR = SQRT( 1. -CAS*CAS - CBS*CBS - CGS*CGS+ 2. *CAS*CBS*CGS)
C
      LAX = IABS(LAXV(1))
      LUP = IABS(LAXV(2))
      LIN = IABS(LAXV(3))
      DO 305 I = 1, 3
         LAXS(I)=ISIGN(1,LAXV(I))
305   CONTINUE
C
C     Form orientation matrix of reciprocal lattice
C
      BMAT(2, LAX) = 0.
      BMAT(3, LAX) = 0.
      BMAT(3, LUP) = 0.
      BMAT(1, LAX) = STAR(LAX) * LAXS(1)
      BMAT(1, LUP) = STAR(LUP) * STAR(3 + LIN) * LAXS(1)
      BMAT(1, LIN) = STAR(LIN) * STAR(3 + LUP) * LAXS(1)
      BMAT(2, LUP) = STAR(LUP) * STAR(6 + LIN) * LAXS(2)
      BMAT(2, LIN) =-STAR(LIN) * LAXS(2) / STAR(6 + LIN)
     *        * ( STAR(3 + LUP) * STAR(3 + LIN) - STAR(3 + LAX) )
      BMAT(3, LIN) = STAR(LIN) / STAR(6 + LIN) * LAXS(3)
     *        * PVSTAR
      RETURN
C     ======
C
      END
c
c
      subroutine mkdtslab(idts, dtslab)
c     =================================
c
c make dataset label in format pname/xname/dname 
c
c On entry:
c  idts    output dataset number
c
c On exit:
c  dtslab   combined label
c
      integer idts
      character dtslab*(*)
c
      include 'parameter.fh'
      include 'datasets.fh'
c
      integer lenstr
      external lenstr
c
      if (idts .le. 0) then
         dtslab = ' '
      else
         dtslab = pname_out(idts)(1:lenstr(pname_out(idts)))//'/'//
     $      xname_out(idts)(1:lenstr(xname_out(idts)))//'/'//
     $      dname_out(idts)(1:lenstr(dname_out(idts)))
      endif
      return
      end
c
c
c
      subroutine mkfilname(lname, jfilout, kfiles, lunmrg, fname)
c     ==================================================
c
c Make suitable filename for output file, but only change it
c if there are multiple output files
c
c WARNING if you change the syntax of the output file name,
c   the CCP4 GUI (cpp4i) must be changed to match
c
c On entry:
c  lname      logical name of file eg HKLOUT
c  jfilout    serial number of this file
c  kfiles     .le. +1 one output file (name just = lname)
c             .gt. 1 make new filename
c  lunmrg     if .true. append"_unmerged" to filename
c
c On exit:
c  fname      new filename or HKLOUT
c
c
      integer jfilout, kfiles
      logical lunmrg
      character*(*) lname, fname
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'datasets.fh'
c
      integer i, j, l, l2
      integer MAXAPPUNMRG
      parameter (MAXAPPUNMRG = 16)
      character appstr*(maxnamelen+MAXAPPUNMRG)
c
      integer lenstr
      external lenstr
c
c
      call ugtenv(lname, fname)
      if (lenstr(fname) .eq. 0) fname = lname

      if (kfiles .le. +1 .and. .not. lunmrg) then
c  no  processing needed
         return
      endif

c Multiple output files, make up name by inserting "_dname" or "_n" on filename
c &/or optionally "_unmerged"
c
c Make string to conditionally append to filename
      appstr = ''
      if (kfiles .gt. 0) then
         call mkdtsstring(dname_out(jfilout), jfilout, appstr)
      endif
      if (lunmrg) then
         appstr = appstr(1:lenstr(appstr))//"_unmerged"
      endif
c
      if (lenstr(appstr) .gt. 0) then
c Append string before any extension
c 
c Search backwards to find if there is a extension
c
         l = lenstr(fname)
         do j = l, 1, -1
            if (fname(j:j) .eq. '.') then
c  found "."
               go to 20
            elseif (fname(j:j) .eq. '/') then
c  found "/" abandon search for "."
               go to 10
            endif
         enddo
c No "." found, append string or number to end
 10      j = l+1
c j points to first position for appending
 20      l2 = lenstr(appstr)
c move rest of string along if necessary (ie if there is an extension)
         if (j .le. l) then
            fname(j+l2:) = fname(j:l)
         endif
c  append it
         fname(j:j+l2-1) = appstr(1:l2)
      endif
c
c DEBUG
cc      write (6,*) 'New filename: ', fname(1:lenstr(fname))
c
      return
      end

      subroutine mkdtsstring(dname, jfilout, appstr)
c    ========================================
c
c Make appending string from dataset name (or number)
c
c On entry:
c  dname    dataset name
c  jfilout  dataset number in case of blank string
c
c On exit:
c  appstr string to append, "_DatasetName"
c
      character*(*) dname, appstr
      integer jfilout

      character line*8

      if (dname .ne. ' ') then
c  Use dname
         appstr = '_'//dname
      else
c  or just the number
         write (line, '(a1,i3)') '_',jfilout
         call squeez(line, appstr)
      endif
      return 
      end
c
c
c
      subroutine mklarc(npoint, points, circen, cirrad, rmsres, philim)
c     ================================================================
c
c Generate arc limit on detector from list of points
c
c On entry:
c  npoint    number of points given (.ge. 3)
c  points(2,npoints)  coordinates of points (in pixels)
c            points 1 & 2 define the ends of the arc
c            if point 1 = point 2 then define a complete circle
c
c On exit:
c  circen(2)  centre of arc
c  cirrad     radius of arc (= -1.0 if fails)
c  rmsres     rms deviation from circle
c  philim(2)  polar angles of beginning and end of arc (radians)
c             Note that arc is forced to run "anti-clockwise"
c             with increasing angles, so that Phi1 < Phi2, and
c             Phi3 (not returned) lies between them, ie the 1st & 2nd
c             points which defined the ends of the arc may be
c             interchanged from the input order
c             In the special case of a complete circle, then Phi1 = Phi2 = 0
c
      integer npoint
      real points(2,*)
      logical lioarc
c
      real circen(2), cirrad, rmsres, philim(2)
c
c Locals
      integer i
      real phi(3), twopi, phi3
      logical tried
c
      twopi = 8.*atan(1.)

c Fit points to a circle
      call fitcrc(npoint, npoint, points, circen, cirrad, rmsres)
      if (cirrad .lt. 0.0) return
c
c Get phi angles of first three points
      do i = 1,3
         phi(i) =
     $      atan2((points(2,i)-circen(2)), (points(1,i)-circen(1)))
      enddo
c
c Anticlockwise & .lt. 2pi is either 1,3,2 or 2,3,1
c (ie do we need to swap points 1 & 2?)
c (a) 1,3,2
      philim(1) = phi(1)
      philim(2) = phi(2)
      phi3 = phi(3)
c  Complete circle?
      if (abs(philim(1)-philim(2)) .lt. 0.001) then
         philim(1) = 0.0
         philim(2) = 0.0
         return
      endif
c
      tried = .false.
c
 10   if (phi3 .lt. philim(1)) then
         phi3 = phi3 + twopi
      endif
      if (philim(2) .lt. phi3) then
         philim(2) = philim(2) + twopi
      endif
c    If phi2 - phi1 > 2 pi, or phi3 still not between phi1 & phi2
c    then this doesn't work, so try the other possibility
      if (philim(2)-philim(1) .gt. twopi .or.
     $   phi3 .gt.philim(2)) then
         if (tried) then 
c      Shouldn't happen
            write (6, '(/a,3f8.4/)')
     $         '**** Arc fit failed (mklarc): ',phi
            call ccperr(1,'*** MKLARC failure ***')
         endif
         philim(2) = phi(1)
         philim(1) = phi(2)
         phi3 = phi(3)
         tried = .true.
         go to 10
      endif
c
      return
      end

      SUBROUTINE MOVEIT(A,B,N)
C     ========================
C
C  Copy N items from A to B
C
      INTEGER I,N
      REAL A(N),B(N)
C
      DO 1,I=1,N
1     B(I)=A(I)
      RETURN
      END
c
c
c
       subroutine mtcanm(l, m, nrs, kanom, jhkl, irunrf, mrf,
     $   isymrf, phirf, timerf, istat)
c     ======================================================
c
c Check if 2 observations form a matching anomalous pair
c Only valid observations (isttrf .ge. lqOK) allowed
c
c On entry:
c  l,m           index numbers of observations to test
c  nrs           number of observations
c  kanom         anomalous option flag (.ge. 2)
c             = +2   anomalous match, no symmetry (ie run match only)
c             = +3   anomalous match, symmetry operator given
c             = +4   anomalous match, INVERT, allow different runs
c             = +5   anomalous match, SPINDLE (from 1st matrix in run)
c  jhkl(3)       h, k, l
c  irunrf(nrs)   run number
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isymrf(nrs)   symmetry number
c  phirf(nrs)    Phi (ROT)
c  timerf(nrs)   Time (if present)
c
c
c On exit:
c  istat         =  0  yes they are matched
c                .gt. 0  not matched
c
      integer l, m, nrs, kanom, istat
      integer jhkl(3), irunrf(nrs), mrf(nrs), isymrf(nrs)
      real phirf(nrs), timerf(nrs)
c
      include 'parameter.fh'
      include 'anomtc.fh'
c
      real delphi, deltim
c
      istat = 0
c
      if (lanmtcrn) then
         if (irunrf(l) .ne. irunrf(m)) then
c  Check  be same run if required
            istat = +1
            goto 100
         endif
      endif 
c
      if (kanom .ge. +3) then
c Check matching symmetry numbers
         if (mtcsym(isymrf(l),irunrf(l)) .ne. isymrf(m)) then
            istat = +2
            goto 100
         endif
      endif
c
      if (anmphd .ne. 0.0) then
c Check PHIDIF
         delphi = abs(phirf(l) - phirf(m))
         if (delphi .gt. 180.0) delphi = 360. - delphi
         if (delphi .gt.anmphd) then
            istat = +3
         endif
      endif
      if (anmtmd .ne. 0.0) then
c Check TIMEDIF
         deltim = abs(timerf(l) - timerf(m))
         if (deltim .gt.anmtmd) then
            istat = +4
         endif
      endif
c
 100  return
      end
      subroutine mtxsiz(maxmtx)
c     =========================
c
c  Determine required size of main matrix arrays
c
c 1. needed for s/r anlini for overlap matrix
c 2. nparam*nparam for scaling
c
      integer maxmtx
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'refcon.fh'
      include 'inout.fh'
      include 'flow.fh'
      include 'rundef.fh'
c
      integer irun, nrrnge, nmtrng
c
      nmtrng = 0
      if (lanlin) then
c-- This code mimics some code in anlini
c  The overlap matrix nrovlp has a row & a column for each rotation
c     range for each run. nmtrng is the number of rotation ranges in all runs
         do 1, irun = 1, numrun
            if (mbatsc(irun) .ne. 0) then
               if (lbtslp(irun)) then
c     Slope
                  nrrnge = max(1, ncscls(1,irun)/2)
               else
c     Plain batch
                  nrrnge = max(1, ncscls(1,irun))
               endif
            else
               nrrnge = max(1, ncscls(1,irun) - 1)
            endif
            nmtrng = nmtrng + nrrnge
 1       continue
      endif
c
      maxmtx = max(nmtrng, nparam)
      write (lunout, '(/a,i10/)')
     $   ' Working array size = ', maxmtx
c
      if (maxmtx .le. 0) then
         write (lunout, '(//a//a//)')
     $      '****** ERROR: no scale parameters ******',
     $      '****** Something horribly wrong   ******'
         call ccperr(1,'**** No scale parameters ****')
      endif 


      if (maxmtx .gt. maxpar) then
         write (lunout, '(/a,a,3(/a,i10))')
     $      ' Working array size too large, reset maxpar or reduce',
     $      'number of parameters:',
     $      '   array size for initial analysis = ',nmtrng,
     $      '            array size for scaling = ',nparam,
     $      '          current maximum (maxpar) = ',maxpar
         call ccperr(1,'**** Array size error ****')
      endif
      return
      end
      subroutine newlpr(iwrprb, linnum, lintyp, linlgn)
c     ================================================
c
c *** For xmgr ***
c
c  Write headers for new line
c
c On entry:
c  iwrprb          output stream: only write if .gt. 0
c  linnum          line number, used to label dataset &
c                     to choose colour (0 to maxlin)
c  lintyp          line type, used to control shape (0 to maxtyp)
c                  = 1 fulls, = 2 summed partials, = 3 scaled partials
c                  = 4 anomalous differences, = 5 other
c  linlgn          legend for line
c
      integer linnum, lintyp, iwrprb
      character linlgn*(*)
c
      integer maxlin, maxtyp
      parameter (maxlin=14, maxtyp=5)
c
      character set*3
      integer lsymbl(maxtyp), lcolor(0:maxlin), l, linnmm
c
      integer lenstr
      external lenstr
c  Symbols:
c   2  circle, 5 triangle up, 9 plus, 3 square, 10 x
      data lsymbl/2,5,9,3,10/
c  Colours: (maximum 15 in xmgr) 6 & 7 are not very good
      data lcolor/1,2,3,4,5,8,9,10,11,12,13,14,15,6,7/
c
      if (iwrprb .le. 0) return
c
      linnmm = mod(linnum, maxlin+1)
c
      if (linnmm .lt. 0 .or. linnmm .gt. maxlin) then
         call ccperr(1,'*** NEWLPR: illegal linnum ***')
      endif
      if (lintyp .lt. 1 .or. lintyp .gt. maxtyp) then
         call ccperr(1,'*** NEWLPR: illegal lintyp ***')
      endif
c
c Dataset number
      if (linnmm .lt. 10) then
         write (set, 6001) linnmm
 6001    format ('s',i1)
      else
         write (set, 6002) linnmm
 6002    format ('s',i2)
      endif
c Constant part of symbol definition, no line, symbol size etc
      write (iwrprb, 6100) set,set,set,set
 6100 format (
     $   '@  ',a,' type xy'/
     $   '@  ',a,' symbol size 0.270000'/
     $   '@  ',a,' symbol fill 0'/
     $   '@  ',a,' linestyle 0')
c Symbol & colour
      write (iwrprb, 6200) set, lsymbl(lintyp), set, lcolor(linnmm)
 6200 format (
     $   '@  ',a,' symbol ',i3/
     $   '@  ',a,' symbol color ',i3)
c Legend
      l = lenstr(linlgn)
      if (l .gt. 0) then
         write (iwrprb, 6300) linnmm, linlgn(1:l)
 6300    format(
     $      '@  legend string ',i3,' "',a,'"')
      endif
      return
      end
c
c
c
      subroutine noslash(text)
c------replace "/" by "_" in string
c
      implicit none
c
      character text*(*)
c     
      integer i,l
c     
      integer lenstr
      external lenstr
c     
      l = lenstr(text)
c     
      do 10 i = 1,l
         if (text(i:i) .eq. '/') then
            text(i:i) = "_"
         endif
 10   continue
c      
      return
      end
c
c
c
      subroutine nospch(txtout, txtin)
c
c------replace special characters by "_" in string
c  this is for project & dataset names, to avoid problems
c  Note that txtin & txtout may be the same array in the call
c
c Acceptable characters are letters, numbers
c  "-_.""
c
      implicit none
c
      character txtin*(*), txtout*(*)
c
c  Acceptable special characters
      integer nspch
      parameter (nspch = 3)
      character*1 spch(nspch)
      data spch / '-','_','.'/
c     
      integer i,j,l
c     
      integer lenstr
      external lenstr
c     
      txtout = txtin
      l = lenstr(txtout)
c     
      do 10 i = 1,l
c Check letters
         if (lge(txtout(i:i), 'A') .and. lle(txtout(i:i),'Z')) go to 10
         if (lge(txtout(i:i), 'a') .and. lle(txtout(i:i),'z')) go to 10
c  numbers
         if (lge(txtout(i:i), '0') .and. lle(txtout(i:i),'9')) go to 10
         do j = 1, nspch
            if (txtout(i:i) .eq. spch(j)) go to 10
         enddo 
c  Fails, replace character
         txtout(i:i) = '_'
 10   continue
c      
      return
      end
      subroutine nrmprb(iwrprb, maxptd, delta, ndelta, dltlim,
     $    prblim, obslim, slope, sintct, nused)
c     ================================================
c
c Normal probability analysis of list of normalized deltas (delta/sd(delta))
c
c David Smith and Lynne Howell ( J.Appl. Cryst (1992) 25, 81-86) 
c David Smith, CCP4 Study Weekend (1993), page 99-106
c
c On entry:
c  iwrprb          if .gt. 0 write deltas to file on this stream
c                         (already opened)
c  maxptd          maximum point density for plot: above this number,
c                    only a sample of points will be written to the plot
c                    file
c  delta(ndelta)   list of deltas (all non-zero)
c  dltlim          probability limit for central part of distribution,
c                    used for slope 
c  prblim          probability limit to eliminate the worst outliers
c
c On exit:
c  obslim          the observed delta limit corresponding to the prblim
c  slope(2)        slope of probability plot (should = 1.0):
c                   (1) all data
c                   (2) central part of data with delta(expected) < dltlim
c  sintct(2)       intercept of probability plot (should = 0.0):
c                   (1) all data
c                   (2) central part of data with delta(expected) < dltlim
c  nused(2)        number used in each plot
      integer iwrprb, maxptd, ndelta, nused(2)
      real    delta(ndelta)
      real    dltlim, slope(2), sintct(2), xmin, xmax, prblim, obslim
c
      include 'inout.fh'
c
      integer maxsmp
      parameter (maxsmp = 30)
      integer i, k, klmflg, irnlo, irnhi, nexpct, idummy
      real    x, y, expctd, w, s, obslo, obshi, dx, xsmmax
      real    sumw(2), sumwx(2), sumwy(2), sumwxx(2), sumwxy(2),
     $   sample(0:maxsmp), rrt2pi, ptdmax, rdmnum
      logical lsmple, lwrite

c Probability array P(x) 
      real prob(512)
c
      real rndmnm
      external rndmnm
      logical lsample
      external lsample
c
      if (ndelta .le. 0) return
c
c
      if (iwrprb .gt. 0) then
         call strppr(iwrprb)
c Because the plot can contain a ridiculously large number of points,
c if there are a lot, only a sample are output. maxptd is the maximum
c point density, above this a random number is used to select points.
c Setup up sampling array at intervals of dx from -xsmmax to +xsmmax
c Outside this range, all points will be used
         xsmmax = +3.0
         dx = xsmmax/float(maxsmp)
         lsmple = lsample(ndelta, maxptd, maxsmp, dx, sample)
      endif
c
c Set up P(x) table for x = 0 to 4.0 in increments of 0.007827789.
      call setprob(prob)
c
c Sort delta array
      call qsortr(ndelta, delta)
c Calculate expected delta for each point from rank,
c and fit to a straight line
c (1) all data
c (2) data with expected delta .lt. dltlim
      do 2, k = 1,2
         sumw(k) = 0.0
         sumwx(k) = 0.0
         sumwy(k) = 0.0
         sumwxx(k) = 0.0
         sumwxy(k) = 0.0
         nused(k) = 0
 2    continue
c
      klmflg = -1
      obshi = 0.0
      do 10, i = 1, ndelta
         call dltexp(ndelta, i, prob, expctd)
c
c We want the values of delta at which exptcd = +/- prblim
         if (klmflg .lt. 0) then
c  below -prblim, check for expctd .gt. -prblim
            if (expctd .gt. -prblim) then
               obslo = delta(i)
               irnlo = i
               klmflg = 0
            endif
         elseif (klmflg .eq. 0) then
c  above -prblim, check for expctd .gt. +prblim
            if (expctd .gt. +prblim) then
               obshi = delta(i)
               irnhi = i
               klmflg = +1
            endif
         endif
c  Loop sums for all data & selected data
         do 11, k = 1,2
            if (k .eq. 1 .or.
     $         (k .eq. 2 .and. abs(expctd) .le. dltlim)) then
               nused(k) = nused(k) + 1
               w = 1.0
               y = delta(i)
               x = expctd
               sumw(k) = sumw(k) + w
               sumwx(k) = w*x + sumwx(k)
               sumwy(k) = w*y + sumwy(k)
               sumwxx(k) = w*x*x + sumwxx(k)
               sumwxy(k) = w*x*y + sumwxy(k)
            end if
 11      continue
c Write to output file if required
         if (iwrprb .gt. 0) then
c option to write out only a sample
            lwrite = .true. 
            if (lsmple) then
               if (abs(expctd) .lt. xsmmax) then
c  Test a random number
                  k = int(abs(expctd)/dx)
                  rdmnum = rndmnm(idummy)
c    sample(k) is the desired sampling probability for this range
c expected values
                  if (rdmnum .gt. sample(k)) lwrite = .false.
               endif
            endif
            if (lwrite) write (iwrprb, '(2f10.4)') expctd, delta(i)
         endif
 10   continue
c
c
      if (obshi .eq. 0.0) obshi = delta(ndelta)
      obslim = min(abs(obslo), abs(obshi))
c      write (lunout, '(/a,f8.3/2(a,f9.3,a,i8/),a,f9.3/)')
c     $   ' Observed limits for expected probability limit ', prblim,
c     $   '     Low limit: ', obslo, ' at number', irnlo,
c     $   '    High limit: ', obshi, ' at number', irnhi,
c     $   ' Overall limit: ', obslim
c
c
c Fit a straight line to the values:
c   first for all data
c   then only for values of expected delta(i) exptcd(i) > dltlim
      do 15, k = 1,2
         if (sumw(k).ne.0.0 .and. nused(k).ge.2) then
            s = sumw(k)*sumwxx(k) - sumwx(k)*sumwx(k)
            if (s .ne. 0.0) then
               slope(k)  = (sumw(k)*sumwxy(k)-sumwx(k)*sumwy(k))/s
               sintct(k) = (sumwxx(k)*sumwy(k)-sumwx(k)*sumwxy(k))/s
            else
               slope(k)  = 0.0
               sintct(k) = 0.0
            endif
         else
            slope(k)  = 0.0
            sintct(k) = 0.0
         end if
 15   continue
c A plot of observed delta(i) against expected delta exptcd(i) should
c be a straight line with slope = 1.0, if the sds are correct.
      if (iwrprb .gt. 0) then
         call endppr(iwrprb)
      endif
      return
      end
c
c
c
      logical function lsample(npoints, maxptd, maxsmp, dx,
     $   sample)
c    =================================================
c
c  Set up sampling for plot
c
c On entry:
c  npoints    number of points to sample
c  maxptd     maximum point density for plot: above this number,
c                only a sample of points will be written to the plot
c                file
c  maxsmp     number of sampling bins
c  dx         width of sample bins
c
c On exit:
c  lsample  returns true if sampling, false if no sampling
c  sample(0:maxsmp)
c
      integer npoints, maxptd, maxsmp
      real  dx, sample(0:maxsmp)
c
      real ptdmax, rrt2pi, x
      integer i, nexpct
c
      ptdmax = float(maxptd)
c 1/sqrt(2 pi)
      rrt2pi = 1./sqrt(8.*atan(1.))
      if (npoints .lt. maxptd) then
c  Few points, no sampling needed
         lsample = .false. 
      else
         lsample = .true.
         do 1, i = 0, maxsmp
            x = float(i) * dx
c Expected number for this x = dP(x)/dx
            nexpct = nint(
     $         dx * float(npoints) * rrt2pi * exp( - 0.5 * x**2))
            if (nexpct .gt. maxptd) then
               sample(i) = ptdmax / float(nexpct)
            else
               sample(i) = 1.0
            endif
 1       continue
      endif
      end
c
c
c
      subroutine  nxtrfl(k,
     $   nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $   mrfl, nrs, ffrf, sdffrf, ghl, irunrf, mrf, idxrf, clprf,
     $   isttrf, isymrf, icntrc)
c
c
c Extract observations for next reflection from internal arrays
c
c Ignore singletons (nrs = 1)
c
c On entry:
c  k             index of next observation
c  nrfgar        number of observations in file (array dimension)
c  iingar        integer array
c  aregar        real array
c
c   iingar(,1)  (a) for first observation in each reflection,
c                  = number of observations (negated if I-)
c               (b) for subsequent observations = +/- 1
c               .gt. 0 for I+ (or centric), .lt. 0 I-
c   iingar(,2) packed run number, partial status, centric flag 
c
c   aregar(,1)  Ihl
c   aregar(,2)  sd(Ihl)
c   aregar(,3)  ghl  scale
c   aregar(,4)  LP
c  irfidx       number of observations in arrays
c  mrfl         array dimension for reflection arrays
c
c On exit:
c  k       index of next observation
c            -1 for last reflection (ie data exhausted)
c  nrs     number of observations found
c  ffrf(nrs)     intensity 
c  sdffrf(nrs)   sd(intensity)
c  ghl(nrs)      inverse scale
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial
c  idxrf(nrs)    index into arrays for this observation
c  clprf(nrs)    LP
c  isttrf(nrs)   status flag = lqOK
c  isymrf(nrs)   symmetry number = 1 for I+, = 2 for I-
c  icntrc        centric flag, = 1 centric else = 0
c                  
c
      integer nrfgar, ningar, nregar, irfidx, mrfl
      integer iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
      integer k, kwgtyp
      integer nrs, icntrc
      real ffrf(mrfl), sdffrf(mrfl), ghl(mrfl), clprf(mrfl)
      integer  irunrf(mrfl), mrf(mrfl), isttrf(mrfl),
     $   isymrf(mrfl), idxrf(mrfl)
c
      include 'flags.fh'
c
      integer l, nmpr
c
c Variance weights
      kwgtyp = 2
c
 1    if (k .gt. irfidx) then
         k = -1
         return
      endif
c
      nrs = iabs(iingar(k, 1))
      if (nrs .le. 1) then
c  Ignore singletons (nrs=1), nrs = 0 shouldn't happen
         k = k+1
         go to 1
      endif
c
      do l = 1, nrs
         if (k .gt. irfidx) then
            k = -1
            return
         endif
         ffrf(l)   = aregar(k, 1)
         sdffrf(l) = aregar(k, 2)
         ghl(l)    = aregar(k, 3)
         clprf(l)  = aregar(k, 4)
         idxrf(l)  = k
c
c   Run, full|partial, centric
c        (ignore nmpr, flag for scaled partial)
         call unpckf(irunrf(l), mrf(l), nmpr, icntrc, iingar(k,2))
c
         if (iingar(k, 1) .gt. 0) then
c I+
            isymrf(l) = 1
         else
c I-
            isymrf(l) = 2
         endif
c List only contains accepted observations
         isttrf(l) = lqOK
c+DEBUG
c         if (k .lt. 100) then
c            write (6,'(i7,2f9.2,2f9.4,3i4)')
c     $         k,(aregar(k,i),i=1,4), irunrf(l), mrf(l),
c     $         nmpr
c         else if (k .eq. 100) then
c            write (6,*)
c         endif
c-DEBUG
c
         k = k+1
      enddo
c         
      return
      end

      subroutine opnopf(jfilout, jdataset)
c     ==============================
c
c Open output file
c 
c Note that now (v3.3.3) the output MTZ files are opened on stream
c 2 (& 3 for the optional additioanl UNMERGED output) rather than
c on stream 1 which is the input file, to avoid confusion.
c This means that all information must be explicitly copied from
c the input MTZ file on stream 1
c
c On entry:
c  jfilout    serial number of file to output
c             (multiple files may be written for multiple datasets)
c  jdataset   dataset for this file, = 0 for all or only dataset
c
      implicit none
c
      integer jfilout, jdataset
c
      include 'parameter.fh'
      include 'column.fh'
      include 'outcon.fh'
      include 'sdfacc.fh'
      include 'rfile.fh'
      include 'symmty.fh'
      include 'datasets.fh'
      integer JJcol
c
      integer i, j, k, i1, ncnew, iappnd, icol, jcol, ifail, nlprgo,
     $   j1, j2, kout, jsort(5)
      character line*80, date*8, time*8, fname*200, appstr*21
c
c Column dataset names
      character pname_col(maxcolo)*(maxnamelen),
     $   xname_col(maxcolo)*(maxnamelen),
     $   dname_col(maxcolo)*(maxnamelen)
c
      integer lenstr
      external lenstr
c
c New labels
c  (0) general, space for copies
      integer NEWLAC
      parameter (NEWLAC = 9)
      character labeln(NEWLAC)*30, ctypen(NEWLAC)*1
      character base_xdname*30
c  (1) AVERAGE
      integer NEWLAA, DTSLAA
      parameter (DTSLAA = 6, NEWLAA = 3+DTSLAA)
      character labela(NEWLAA)*30, ctypea(NEWLAA)*1
      character labelt(maxcolo)*30, ctypet(maxcolo)*1
c  (2) SEPARATE
      integer NEWLAS
      parameter (NEWLAS = 5)
      character labele(NEWLAS)*30, ctypee(NEWLAS)*1
c  (3) POSTREF
      integer NEWLAP
      parameter (NEWLAP = 4)
      character labelp(NEWLAP)*30, ctypep(NEWLAP)*1
c  (4) UNMERGED
      integer NEWLAU
      parameter (NEWLAU = 22)
      character labelu(NEWLAU)*30, ctypeu(NEWLAU)*1
      character labels(NEWLAU)*30, ctypes(NEWLAU)*1
      integer ncolum
c
c  Label for base dataset
      data base_xdname/'HKL_base'/
c  (1) AVERAGE
      data labela /'H','K','L','IMEAN','SIGIMEAN',
     $   'I(+)','SIGI(+)','I(-)','SIGI(-)'/
c      data labela /'H','K','L','IMEAN','SIGIMEAN',
c     $   'I+','SIGI+','I-','SIGI-'/
c      data ctypea /'H','H','H','J',   'Q', 'J',    'Q', 'J',    'Q'/
      data ctypea /'H','H','H','J',   'Q', 'K',    'M', 'K',    'M'/
c      data labela/'H','K','L','IMEAN',
c     +     'SIGIMEAN','IW+','SIGIW+','I+','SIGI+','IW-','SIGIW-',
c     +     'I-','SIGI-'/
c      data ctypea /'H','H','H','J','Q','J','Q','J','Q','J','Q',
c     $   'J','Q'/
c  (2) SEPARATE
      data labele /'SCALE','SIGSCALE','SIGIC','SIGIPRC','FLAG'/
      data ctypee /    'R',       'Q',    'Q',      'Q',   'I'/
c  (3) POSTREF
      data labelp /'IMEAN','SIGIMEAN','ISUM','SIGISUM'/
      data ctypep /    'J',       'Q',   'J',      'Q'/
c  (4) UNMERGED
c  First ncolum columns are compulsory
c  others are only present if present in input file
      data labelu /'H','K','L','M/ISYM','BATCH','I','SIGI',
     $   'SCALEUSED','SIGSCALEUSED',
     $   'NPART','TIME','XDET','YDET','ROT','FRACTIONCALC','LP',
     $   'S0X','S0Y','S0Z','S2X','S2Y','S2Z'/
      data ctypeu /'H','H','H',   'Y',     'B', 'J',   'Q',
     $       'R',       'Q',
     $        'I',   'R',   'R',   'R',  'R',           'R', 'R',
     $   6*'R'/
c Number of compulsory output columns
      data ncolum /9/
c
c
      koutstr1 = 2
      koutstr2 = 0
      kout = koutstr1
c Output stream 3 if AVERAGE output as well
      if (ifilou .eq. +5) then
         koutstr2 = 3
         kout = koutstr2
      elseif (ifilou .eq. +2 .or. ifilou .eq. +3) then
c SEPARATE or POSTREF, use same stream as input file
         koutstr1 = 1
         kout = koutstr1
      endif

c  Primary output file
      call mkfilname('HKLOUT', jfilout, kfilou, .false., fname)
c Store filename
      dfilename_out(max(1,jdataset)) = fname 
      dfilename2_out(max(1,jdataset)) = '' 
      call lwopen(koutstr1, fname)
c  Secondary output file: UNMERGED if AVERAGE is also selected
      if (koutstr2 .ne. 0) then
         call mkfilname('HKLOUT', jfilout, kfilou, .true., fname)
         dfilename2_out(max(1,jdataset)) = fname 
         call lwopen(koutstr2, fname)
      endif
c
      nlprgo = 0
c
c Different output options:
c  ifilou =  0   no output hkl file
c         = +1   AVERAGE
c         = +2   SEPARATE
c         = +3   POSTREF
c         = +4   UNMERGED
c         = +5   AVERAGE & UNMERGED
c -------------------------------------------------------------------------
      if (ifilou .eq. +4 .or. ifilou .eq. +5) then
c UNMERGED (do this first before AVERAGE so that cpybth works properly,
c since the AVERAGE bit resets MTX stream 1 to be merged file)
c
c  If writing to stream 2, copy symmetry from common block
         if (kout .eq. koutstr2) then
            call lwsymm(kout,
     $         nsym,nsymp,rsym,lattyp,nspgrp,spgnam,pgname)
         endif 
c  Column numbers for compulsory columns
         ncoi   =  6
         ncosdi =  7
         ncosc  =  8
         ncosds =  9
c  Column numbers for optional columns
         nconpt = 0
         ncotim = 0
         ncoxdt = 0
         ncoydt = 0
         ncorot = 0
         ncofrc = 0
         ncolp  = 0
         ncobeam = 0
c Start column count
         ncolou = ncolum
c Copy compulsory columns for assignment
         do 40, icol = 1, ncolum
            labels(icol) = labelu(icol)
            ctypes(icol) = ctypeu(icol)
 40      continue
         icol = ncolum
         jcol = icol
c Optional columns only if present in input file
c   (or for NPART, if there are partials)
c- NPART
         jcol = jcol+1
         if (lprtfl) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, nconpt)
         endif

c- TIME
         jcol = jcol+1
         if (nct .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncotim)
         endif
c- XDET
         jcol = jcol+1
         if (ncx .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncoxdt)
         endif
c- YDET
         jcol = jcol+1
         if (ncy .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncoydt)
         endif
c- ROT
         jcol = jcol+1
         if (ncz .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncorot)
         endif
c- FRACTIONCALC
         jcol = jcol+1
         if (ncfrcl .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncofrc)
         endif
c- LP
         jcol = jcol+1
         if (nclp .gt. 0) then
            call adcllb(
     $         icol, jcol, ncolou,
     $         labels, labelu, ctypes, ctypeu, ncolp)
         endif
c- BEAMS, output diffracted beams (6 columns)
         if (mbeams .ne. 0) then
c   set first output column number
            ncobeam = ncolou+1
            do i = 1, 6
               jcol = jcol+1
               call adcllb(
     $            icol, jcol, ncolou,
     $            labels, labelu, ctypes, ctypeu, k)
            enddo
         else
            jcol = jcol+6
         endif
c
         call lwclab(kout, labels, ncolou, ctypes, 0)
c Put all columns into base dataset
         do jjcol = 1, ncolou
            xname_col(jjcol) = base_xdname
            dname_col(jjcol) = base_xdname
         enddo
         call lwidasx(kout,ncolou,xname_col,dname_col,0)
c
c Dataset headers for this or all datasets
         if (jdataset .eq. 0) then
c All
            j1 = 1
            j2 = ndtsetsout
         else
            j1 = jdataset
            j2 = jdataset
         endif
         do j = j1, j2
c*4*            call lwidc(1, pname_out(j), dname_out(j),
c*4*     $      dcell_out(1,j), dwavel_out(j))
            call lwidx(kout, pname_out(j), xname_out(j), dname_out(j),
     $      dcell_out(1,j), dwavel_out(j))
         enddo
c
c  Copy batch headers, except for excluded batches (& reference batch
c     if not output)
         call cpybth(jdataset, kout)
      endif
c -------------------------------------------------------------------------
      if (ifilou .eq. 1 .or. ifilou .eq. 5) then
c  (1) AVERAGE
c   Clear batch titles
         i1 = 0
         call lwbtit(koutstr1, i1, line)
c
         if (kfilou .ge. 0) then
c . . . . . . .
c    SPLIT or one dataset, just one dataset in (each) output file
            ncolou = newlaa
            call lwclab(koutstr1, labela, ncolou, ctypea, 0)
c Set column datasets, all the same
            do jjcol = 1, ncolou
               pname_col(jjcol) = pname_out(jfilout)
               xname_col(jjcol) = xname_out(jfilout)
               dname_col(jjcol) = dname_out(jfilout)
            enddo
c  Cell and wavelength have been averaged from those in batch headers
c Add dataset definition, names, cell, wavelength
            j = max(jdataset, 1)
            call lwidx(koutstr1,
     $         pname_out(jfilout), xname_out(jfilout),
     $         dname_out(jfilout),
     $         dcell_out(1,j), dwavel_out(j))
            call lwidasx(koutstr1,ncolou,xname_col,dname_col,0)
c Store cell for harvesting & summary
            do i=1,6
               cell(i) = dcell_out(i,j)
            enddo
         else
c . . . . . . .
c  TOGETHER, multiple datasets
c Number of columns 3 (H,K,L) + 6*Number of output datasets
            ncolou = 3 + 6*ndtsetsout
c    HKL labels
            do i=1,3
               labelt(i) = labela(i)
               ctypet(i) = ctypea(i)
               pname_col(i) = 'HKL_base'
               xname_col(i) = 'HKL_base'
               dname_col(i) = 'HKL_base'
            enddo

            i1 = 3
            call ccpzr(cell,6)
c - loop datasets
            do k=1,ndtsetsout
c      store first column number for this dataset
               ncodts(k) = i1+1
c      get string to append to column labels
               call mkdtsstring(dname_out(k), k, appstr)
               do i = 1,DTSLAA
                  i1 = i1+1
                  line = labela(i+3)//appstr
                  call squeez(line, labelt(i1))
                  ctypet(i1) = ctypea(i+3)
c      datasets for each column
                  pname_col(i1) = pname_out(k)
                  xname_col(i1) = xname_out(k)
                  dname_col(i1) = dname_out(k)
               enddo
c  add dataset to output
               call lwidx(koutstr1,
     $            pname_out(k), xname_out(k),
     $            dname_out(k),
     $            dcell_out(1,k), dwavel_out(k))
c  Store average cell for harvesting & summary
               do i=1,6
                  cell(i) = cell(i) + dcell_out(i,k)
               enddo
            enddo
c  Store average cell for harvesting & summary
            do i=1,6
               cell(i) = cell(i)/float(ndtsetsout)
            enddo
c  Store labels
            call lwclab(koutstr1, labelt, ncolou, ctypet, 0)
c  Store datasets for columns
            call lwidasx(koutstr1,ncolou,xname_col,dname_col,0)
         endif
c -------------------------------------------------------------------------
      elseif (ifilou .eq. +2 .or. ifilou .eq. +3) then
c  (2) SEPARATE or (3) POSTREF
         ncolou = ncol
         i1 = 0
c
c Set up new column labels
c     Add Scale column if not already present
c
         if (ncsc .eq. 0) then
            ncolou = ncolou+1
            ncosc = ncolou
            i1 = i1+1
            labeln(i1) = labele(1)
            ctypen(i1) = ctypee(1)
         else
            ncosc = ncsc
         endif
c  sd(scale)
         if (ncsdsc .eq. 0) then
            ncolou = ncolou+1
            ncosds = ncolou
            i1 = i1+1
            labeln(i1) = labele(2)
            ctypen(i1) = ctypee(2)
         else
            ncosds = ncsdsc
         endif
c  Output modified SD columns only if SDs changed
         ncosdi = 0
         ncosdp = 0
         if (lsdmod) then
c  Modified sd(I): Integrated I
            if (ncsdic .eq. 0) then
               ncolou = ncolou+1
               ncosdi = ncolou
               i1 = i1+1
               labeln(i1) = labele(3)
               ctypen(i1) = ctypee(3)
            else
               ncosdi = ncsdic
            endif
c  Profile-fitted I
            if (ncip .ne. 0) then
               if (ncsdpc .eq. 0) then
                  ncolou = ncolou+1
                  ncosdp = ncolou
                  i1 = i1+1
                  labeln(i1) = labele(4)
                  ctypen(i1) = ctypee(4)
               else
                  ncosdp = ncsdpc
               endif
            else
               ncosdp = 0
            endif
         endif
c
c  Status flag
         if (ncflag .ne. 0) then
            ncoflg = ncflag
         else
            ncolou = ncolou + 1
            ncoflg = ncolou
            i1 = i1+1
            labeln(i1) = labele(5)
            ctypen(i1) = ctypee(5)
         endif
c
c  ..... 
         if (ifilou .eq. +3) then
c  POSTREF
            if (.not.lpstrf) then
c    Append Postref columns
               do 30, i = 1, newlap
                  ncolou = ncolou + 1
                  if (i .eq. 1) ncopsf = ncolou
                  i1 = i1+1
                  labeln(i1) = labelp(i)
                  ctypen(i1) = ctypep(i)
 30            continue
            endif
         endif
c Put all columns into base dataset
         do jjcol = 1, ncolou
            xname_col(jjcol) = base_xdname
            dname_col(jjcol) = base_xdname
         enddo
c  existing columns first (before adding new columns)
         call lwidasx(koutstr1,ncol,xname_col,dname_col,0)
c  Append to existing column labels
         ncnew = ncolou - ncol
         if (ncnew .gt. 0) then
            iappnd = 1
            call lwclab(koutstr1, labeln, i1, ctypen, iappnd)
            nlprgo = i1 + ncol + nlprgo
         endif
         call lwidasx(koutstr1,ncolou,xname_col,dname_col,0)
c
c
c  Copy batch headers, except for excluded batches (& reference batch
c     if not output)
         call cpybth(jdataset, koutstr1)
      endif 
c
c  >>> Common bit ...
c Copy symmetry from common block
      call lwsymm(koutstr1,
     $   nsym,nsymp,rsym,lattyp,nspgrp,spgnam,pgname)
c Sort order
      do i=1,5
         jsort(i) = i
      enddo
      call lwsort(koutstr1,jsort)
c History
      call ccpdat(date)
      call utime(time)
      write (line, '(a,a,a,a)') 'From SCALA: run at ', time,' on ', date
      call lwhist(koutstr1, line, 1)
c
c Optional addition of new history record
      if (lenstr(histry) .gt. 0) then
         call lwhist(koutstr1, histry, 1)
      endif
c
c Replace title if given
      if (lenstr(outitl) .gt. 0) then
         call lwtitl(koutstr1, outitl, 0)
      endif
c   ...  but duplicate for 2nd file if needed
      if (koutstr2 .ne. 0) then
c Sort order
         call lwsort(koutstr2,jsort)
c History
         call ccpdat(date)
         call utime(time)
         write (line, '(a,a,a,a)')
     $      'From SCALA: run at ', time,' on ', date
         call lwhist(koutstr2, line, 1)
c
c Optional addition of new history record
         if (lenstr(histry) .gt. 0) then
            call lwhist(koutstr2, histry, 1)
         endif
c
c Replace title if given
         if (lenstr(outitl) .gt. 0) then
            call lwtitl(koutstr2, outitl, 0)
         endif
      endif
c <<<
c
c Open output file for Scalepack style, if required
      if (ifilou .gt. 0 .and. isclpk .gt. 0) then
         call mkfilname('SCALEPACK', jfilout, kfilou, .false., fname)
         ifail = 0
         call ccpdpn(isclpk, fname, 'UNKNOWN', 'F', i, ifail)
c
         if (ifilou .eq. 1) then
c Merged output
c  Header:
c   two lines with some number or other on them
c   cell and spacegroup
            i = 1
            write (isclpk, '(i5/i5)') i, i
            write (isclpk, '(6f10.3,x,a)') cell, spgnam
         else
c Unmerged output
c  Header:
c   Number of symops, spacegroup name
c   symmetry operators
            write (isclpk, '(i5,x,a)') nsym, spgnam
            do k = 1, nsym
               write (isclpk, '(9i3/3i3)')
     $            ((nint(rsym(i,j,k)), j=1,3),i=1,3),
     $            (nint(12.0*rsym(i,4,k)),i=1,3)
            enddo
         endif
      endif
c -------------------------------------------------------------------------
      return
      end

      SUBROUTINE ORDR3(KNTRL,Q,NQ)
C
C*************************************************************
C       Find NQ(1) so that |Q(NQ(1)| is biggest Q.
C       If KNTRL=1, make NQ(1),NQ(2),NQ(3) a positive permutn.
C       If KNTRL= anything else,
C       find nos. NQ() so that|Q(NQ(1)|>|Q(NQ(2))|>|Q(NQ(3))|
C
C*************************************************************
        REAL Q(3)
        INTEGER NQ(3),KNTRL,J
C
C       Find no. NQ(1) of Q() with largest |Q()|.
          NQ(1)=1
          IF (ABS(Q(2)).GT.ABS(Q(1))) NQ(1)=2
          IF (ABS(Q(3)).GT.ABS(Q(NQ(1)))) NQ(1)=3
C       Find nos. for other two Q()'s & order so |Q(NQ(2))|>|Q(NQ(3))|
          NQ(2)=MOD(NQ(1),3)+1
          NQ(3)=MOD(NQ(2),3)+1
          IF (KNTRL.EQ.1) GO TO 100
            IF (ABS(Q(NQ(3))).GT.ABS(Q(NQ(2)))) THEN
                J=NQ(3)
                NQ(3)=NQ(2)
                NQ(2)=J
            END IF
 100    RETURN
        END
      subroutine outlyr(lprint, lrejct, nrs, nuse, janom, iselds, 
     $   lufull, ihkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $   isymrf, lanmtc, wmih, nrej, sdrej, sdrej2, krej2, loutlfl,
     $   delrf, nkept)
c     ==========================================================
c
c Reject outliers. Reflections from a reference run are excluded from the
c     outlier test. Option to select only I+ or I- observations, depending
c     on argument janom
c Always calculate weighted deviation (Delta(hl) below) for all "used"
c observations
c
c Algorithm:
c   (1) if there are 2 observations (left), then
c        (a) for each observation Ihl, test deviation 
c              Delta(hl) =
c                 (Ihl - ghl Iother) / sqrt[sigIhl**2 + (ghl*sdIother)**2]
c            against sdrej2, where Iother = the other observation
c        (b) if either Delta(hl) > sdrej2, then reject reflection 
c             according to fkag krej2 (qv)
c   (2) if there 3 or more observations left, then
c        (a) for each observation Ihl, 
c            (i)   calculate weighted mean of all other 
c                  observations <I>n-1 & its sd(<I>n-1)
c            (ii)  deviation Delta(hl) =
c                   (Ihl - ghl <I>n-1>) / sqrt[sigIhl**2 + (ghl*sd(<I>n-1))**2]
c            (iii) find largest deviation max|Delta(hl)|
c            (iv)  count number of observations for which
c                   Delta(hl) .ge. 0 (ngt), & for which
c                   Delta(hl) .lt. 0 (nlt)
c        (b) if max|Delta(hl)| > sdrej, then reject one observation, but
c             which one?
c            (i) if ngt == 1 .or. nlt == 1, then one observation is a long
c                 way from the others, and this one is rejected
c           (ii) else reject the one with the worst deviation max|Delta(hl)|
c    (3) iterate from beginning
c
c On entry:
c     lprint     if .true., print outliers
c     lrejct     if .true., reject outliers, else just calculate deviations
c     nrs        number of observations
c     nuse       number of observations used (ie excluding partial halves etc)
c     janom      = 0 use all valid observations
c                = 1 use I+ only (isym odd)
c                = 2 use I- only (isym even)
c     iselds     if .gt. 0, use only observations from this dataset
c     lufull     .true. to use fulls only, .false. partials as well
c     ihkl       hkl indices (for printing)
c     ffrf       intensities
c     sdffrf     corrected standard deviations of intensities
c     wtrf       weight for this observation, eg 1/sd(I)**2
c                  (note that this is unscaled, so needs to be
c                   multiplied by ghl**2)
c     ghl        inverse scale factors
c     irunrf     run number
c     mrf        full/partial flag (= 0 full, = 1 partial)
c     isttrf     status flags
c     isymrf     symmetry numbers (odd for I+, even for I-)
c     lanmtc     pointer to matched pair (=0 if none). Matched pairs
c                are rejected together
c     sdrej      rejection limit for reflections measured more than twice
c     sdrej2     rejection limit for reflections measured twice
c     krej2      what to do with 2 observations which disagree
c                = -1   reject both
c                =  0   keep both
c                = +1   reject larger
c                = +2   reject smaller
c     loutlfl    flag to use to mark rejected outliers
c               (typically = lqoutl but see flags.fh for other outlier flags)
c
c On exit:
c     isttrf     updated status flags (loutlfl for rejected outliers)
c     wmih       overall weighted intensity
c     nrej       number rejected: this may be .gt. 0 even if lrejct .true.
c                if observations are flagged as rejected outiers on entry,
c                in array isttrf
c     delrf      for each observation, the deviation/sd
c                   (left unchanged if not used)
c     nkept      number used in wmih, ie count of selected reflections
c                  which are not rejected
c
      implicit none
c
      logical lprint, lrejct, lufull
      integer nrs, nuse, ihkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs),
     $   nrej, krej2, janom, nkept, isymrf(nrs), iselds, lanmtc(nrs),
     $   loutlfl
      real ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs), wmih, 
     $   delrf(nrs), sdrej, sdrej2
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
      real wgirf(maxrfl), wg2rf(maxrfl)
      real sumwgI, sumwg2, worst, others, sdoth, small, varoth
      integer l, lworst, nkeep, lsmall, nlt, ngt, kgt, klt, i,
     $   irn, krej2l
      logical reject, lnegsq, useflg(maxrfl), lrejrf(maxrfl)
      character*8 pflag
      character*1 cjunk(maxrfl)
c
      nrej = 0
      nkeep = 0
      wmih = 0.0
c Calculate weighted mean 
      sumwgI = 0.0
      sumwg2 = 0.0
c
c  Set flag useflg for whether to use observation:
c   accept (.true.) if all of
c   (a) isstrf .ge. lqOK
c   (b) irunrf .ne. irefrn (if irefrn .ne. 0)
c   (c) isymrf appropriate for janom flag
c   (d) irunrf belongs to dataset iselds if iselds .gt. 0
c   (e) mrf .eq. 0  if lufull .true. (fulls only)
      call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf, 
     $   irefrn, iselds, lufull, useflg)
c
c
      do 10, l = 1,nrs
         if (useflg(l)) then
            wgirf(l) = wtrf(l) * ghl(l)* ffrf(l)
            wg2rf(l) = wtrf(l) * ghl(l)**2
            sumwgI = sumwgI + wgirf(l)
            sumwg2 = sumwg2 + wg2rf(l)
            nkeep = nkeep+1
         endif
c Initialise flag for rejected in this pass
         lrejrf(l) = .false.
 10   continue
c
      if (sumwg2 .eq. 0.0) then
         nuse = 0
         nkept = 0
         return
      endif
c
      wmih = sumwgI / sumwg2
c
c--- If not rejecting observations, just count any previous outlier rejects
      if (.not. lrejct) then
         do l = 1, nrs
            if (useflg(l) .and. isttrf(l) .le. lqoutl) then
               nrej = nrej + 1
            endif
         enddo
      endif
c Outlier rejection
c   return here after rejection for further test
c
c Calculate all delrf (Delta(hl)) values
 30   if (nkeep .le. 1) go to 100
      do 35, l = 1, nrs
         if (useflg(l) .and. .not.lrejrf(l)) then
            if (sumwg2 - wg2rf(l) .le. 0.0) then
               others  = 0.0
               varoth = 0.0
            else
c  weighted mean of other intensities
               others = (sumwgI - wgirf(l)) / (sumwg2 - wg2rf(l))
               varoth = 1./(sumwg2 - wg2rf(l))
            endif
            if (sdffrf(l) .eq. 0.0) then
               write (lunout,'(a,3i5)')
     $            ' *** OUTLYR: zero standard deviation '//
     $            'in reflection ',ihkl
               delrf(l) = 0.0
            else
               delrf(l) = (ffrf(l) - ghl(l) * others) /
     $            sqrt(sdffrf(l)**2 + ghl(l)**2 * varoth)
c DEBUG
c               write (6,*) 'OL', ffrf(l),sdffrf(l), ghl(l),others,varoth
            endif
         endif
 35   continue
c--- Check rejects
      if (lrejct .and. nkeep .gt. 1) then
c---------------------------------------  2 observations
         if (nkeep .eq. 2) then
c
c Two observations
c  check deviations |Ihl - ghl Iother| / sqrt[sigIhl**2 + (ghl*sdIother)**2]
c  against sdrej2
            reject = .false.
            krej2l = krej2
            do 40, l = 1, nrs
               if (useflg(l) .and. .not.lrejrf(l)) then
                  if (abs(delrf(l)) .gt. sdrej2) then
                     reject = .true.
                     if (lanmtc(l) .gt. 0 .and. janom .eq. 0) then
c  Reject both if matched pairs
                        krej2l = -1
                     endif
                  endif
               endif
 40         continue
c
            if (reject .and. krej2l .ne. 0) then
c   Rejection flagged and allowed, but which one?
c   Find smaller scaled I = Ihl/ghl
               if (krej2l .gt. 0) then
                  small = 1.0e+20
                  do 41, l = 1, nrs
                     if (useflg(l) .and. .not.lrejrf(l)) then
                        if (ffrf(l)/ghl(l) .lt. small) then
                           small = ffrf(l)/ghl(l)
                           lsmall = l
                        endif
                     endif
 41               continue
               endif
c           Recalculate weighted mean 
               sumwgI = 0.0
               sumwg2 = 0.0
               do 45, l = 1, nrs
                  if (useflg(l) .and. .not.lrejrf(l)) then
c Should we reject this one?
c     krej2      = -1   reject both
c                =  0   keep both
c                = +1   reject larger
c                = +2   reject smaller
                     if (krej2l .eq. -1 .or.
     $                  (krej2l .eq. +1 .and. l .ne. lsmall) .or.
     $                  (krej2l .eq. +2 .and. l .eq. lsmall)) then
c  yes reject it
                        isttrf(l) = loutlfl
                        lrejrf(l) = .true.
                        nrej = nrej + 1
                        nkeep = nkeep - 1
                     else
c    keep it
                        sumwgi = sumwgi + wgirf(l)
                        sumwg2 = sumwg2 + wg2rf(l)
                     endif
                  endif
 45            continue
c  Recalculate mean and deviations
               if (sumwg2 .gt. 0.0) then
c   One rejected
                  wmih = sumwgI / sumwg2
               else
c   Both rejected
                  if (nkeep .ne. 0) then
                     call ccperr(1,'**** Help: OUTLYR nkeep .ne. 0')
                  endif
               endif
               go to 30
            endif
c
         else
c------------------------------------------------- More than 2 observations
c 
c Check each one against weighted mean of others, find worst
c
            worst = -1000000.
            lworst = 0
            ngt = 0
            nlt = 0
            do 50, l = 1,nrs
               if (useflg(l) .and. .not.lrejrf(l)) then
                  if (delrf(l) .ge. 0.0) then
                     ngt = ngt + 1
                     kgt = l
                  else
                     nlt = nlt + 1
                     klt = l
                  endif
                  if (abs(delrf(l)) .gt. worst) then
                     worst = abs(delrf(l))
                     lworst = l
                  endif
               endif
 50         continue
c
            if (worst .gt. sdrej) then
c Reject one, but which one?
c   Is one on opposite side of weighted mean from all others?
c   If so, chuck that one, else chuck the worst deviant
               if (nlt .eq. 1) then
                  lworst = klt
               elseif (ngt .eq. 1) then
                  lworst = kgt
               endif
c   Reject lworst
               nrej = nrej+1
               nkeep = nkeep - 1
               isttrf(lworst) = loutlfl
               lrejrf(lworst) = .true.
               sumwgI = sumwgI - wgirf(lworst)
               sumwg2 = sumwg2 - wg2rf(lworst)
c   Was the rejected one one of a matched pair?
c If so reject the other one as well
               if (lanmtc(lworst) .gt. 0 .and. janom .eq. 0) then
                  l = lanmtc(lworst)
                  if (lanmtc(l) .ne. lworst) then
                     write (lunout, '(/a/a/)')
     $      ' **** OUTLYR: ambiguity in ANOMALOUS MATCH flags! ****',
     $      ' **** You cannot use this option in this case     ****'
                     write (6,*) 'lworst, nrs, nkeep,ihkl',
     $                  lworst, nrs, nkeep, ihkl
                     write (6,*)  'lanmtc', lanmtc
                     call ccperr(1,'*** Anomalous Match problem ***')
                  endif
                  nrej = nrej+1
                  nkeep = nkeep - 1
                  isttrf(l) = loutlfl
                  lrejrf(l) = .true.
                  sumwgI = sumwgI - wgirf(l)
                  sumwg2 = sumwg2 - wg2rf(l)
               endif
               if (sumwg2 .ne. 0.0) then
                  wmih = sumwgI / sumwg2
               else
                  wmih = 0.0
               endif
               go to 30
            endif
         endif
      endif
c------------------------------------------------------------------------
c Finished rejections, recalculate deviations for rejected reflections
c (not strictly necessary unless more than rejected, but doesn't hurt)
 100  if (nrej .gt. 0) then
         if (sumwg2 .gt. 0.0) then
            varoth = 1. / sumwg2
         else
            varoth = 0.0
         endif
         do l = 1, nrs
            if (lrejrf(l)) then
c   this one rejected in this entry to s/r
               delrf(l) = (ffrf(l) - ghl(l) * wmih) /
     $            sqrt(sdffrf(l)**2 + ghl(l)**2 * varoth)
            endif
         enddo
      endif
      if (nrej .gt. 0 .and. lprint) then
c Print outliers
         write (lunout, 6100) ihkl, wmih
 6100    format(' Outlier in ', 3i4, ' weighted mean intensity',
     $      f10.1)
         do 110, l = 1, nrs
            if (isttrf(l) .eq. loutlfl) then
               pflag = '  Reject'
            else
               pflag = ' '
            endif
            if (useflg(l)) then
               if (ghl(l) .eq. 0.0) then
                  write (lunout, 6105) ihkl, l, isttrf(l)
 6105             format(' OUTLYR: zero scale factor for hkl',3i4,
     $               ', measurement ',i5,' status', i5)
               else
                  write (lunout, 6110) pflag,
     $               delrf(l), ffrf(l)/ghl(l), sdffrf(l)/ghl(l),
     $               irunrf(l), isymrf(l)
 6110             format(4x,a8,f9.1,2f10.1,2i8)
               endif
            endif
 110     continue
      endif
c
      nkept = nkeep
      return
      end
      subroutine partls(ltdspr,
     $   jhkl, kprsal, lpart, stol, nrs, nprtds, ltdsrn,
     $   ffrf, sdffrf, ffprrf, sdfprf,
     $   ghl, Dfhl, xdrf, ydrf, zdrf, phirf, xsirf,
     $   frclrf, kmptrf, isttrf, mrf, mprf, isymrf, 
     $   ibatrf, irunrf, nmprrf, jmprrf, kprfll, kprmss, mxprwl, imidl,
     $   nrejpt, nuse, prthll, prliml, prmaxl, dghldc, lavpsp, lavpsc)
c     ===============================================================
c
c  Sum or scale partials, calculate Tails correction if required
c
c On entry:
c  ltdspr      if .true., calculate Tails correction (if switched on)
c                for unmatched partials, else don't
c  nrs         number of observations
c
c On exit:
c  ibatrf(nrs) batch number updated for summed partials, so that summed
c              partial is attributed to the "central" batch
c  nuse        number of accepted observations (ie less unmatched or
c                summed partials)
c  Dfhl(nrs)   Tails correction, = 1.0 if not otherwise set
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'rfile.fh'
c
      integer jhkl(3), kprsal, nprtds, istat
      logical ltdspr, lpart, ltdsrn(maxrun)
      real stol
      integer nrs
      real    ffrf(nrs), sdffrf(nrs), ffprrf(nrs), sdfprf(nrs),
     $   ghl(nrs), Dfhl(nrs), xdrf(nrs), ydrf(nrs), zdrf(nrs), 
     $   phirf(nrs), xsirf(nrs), frclrf(nrs)
      integer isttrf(nrs), mrf(nrs), mprf(nrs), isymrf(nrs), 
     $   ibatrf(nrs), irunrf(nrs), nmprrf(nrs), jmprrf(nrs),
     $   kmptrf(nrs)
      integer kprfll, kprmss, mxprwl, imidl, nrejpt(6), nuse
      real prthll, prliml, prmaxl, dghldc(maxpmr,nrs)
      logical lavpsp, lavpsc(maxrun)
c
c locals
      integer l, kl, ibatch, nppart, istat1
      real frccal
c jmidsl stores the mid-slot number of summed partials: for consistency,
c this is set = slot number for fulls as well
      integer jmidsl(maxrfl)
c
c Set jmprrf flags to OK for fulls, all others to reject (-2) to start
c  If kprsal = -1 (ONLYFULLS), reject previously summed partials (mprf = 10)
c  jmprrf may be reset in sumprt or sclprt
      do 1, l = 1,nrs
         if (isttrf(l) .ge. lqOK .and.
     $      ((kprsal .lt. 0 .and. mprf(l) .ne. 10) .or.
     $       (kprsal .ge. 0 .and. mprf(l) .eq. 0))) then
            jmprrf(l) =  0
         else
            jmprrf(l) = -2
         endif
         jmidsl(l) = l
 1    continue
c     Sum partials if required (using selected I), update nuse
c     Here, nuse includes fulls & partials
      if (lpart) then
         if (kprsal .eq. +1) then
c     Sum partials: summed partials returned in ffrf, sdffrf even
c     if IPR values (ffprrf, sdfprf) are used
            call sumprt(jhkl, nrs, ffrf, sdffrf, ffprrf, sdfprf, ghl,
     $         isttrf, mrf, mprf, isymrf, ibatrf, irunrf,
     $         xdrf, ydrf, zdrf, phirf, frclrf, kmptrf, nmprrf, jmprrf,
     $         jmidsl, kprfll, kprmss, prthll, prliml, prmaxl, mxprwl,
     $         imidl, ipower,
     $         lsclpr, nrejpt, nuse, dghldc, lavpsp, lavpsc)
         elseif (kprsal .eq. +2) then
c     Scale partials: ffprrf never used
            call sclprt(nrs, ffrf, sdffrf, ffprrf, sdfprf, frclrf,
     $         isttrf, nmprrf, jmprrf, mrf, prthll, nrejpt, nuse,
     $         imidl, ipower, lsclpr)
         endif
      endif
c     For fulls, get profile-fitted value if needed
      call fulprf(nrs, ffrf, sdffrf, ffprrf, sdfprf, mrf, imidl, ipower)
c
c
      if (nprtds .gt. 0) then
c Diffuse correction (Tails)
         do 40, l = 1,nrs
c   Batch of first image for this observation: only different from
c   this batch number for 2nd & subsequent parts of summed partial,
c   for which the 1st slot number is stored in jmprrf
c   For unmatched partials jmprrf .lt. 0, use to original slot number
c   in TDS calculation
            if (ltdsrn(irunrf(l)) .and.
     $           (ltdspr .or. isttrf(l) .ge. lqOK)) then
               if (isttrf(l) .eq. lquprt) then
                  kl = jmprrf(l)
                  if (kl .le. 0) then
                     kl = l
                  endif 
                  ibatch = ibatrf(kl)
                  nppart = iabs(nmprrf(kl))
               else
                  ibatch = ibatrf(l)
                  nppart = max(1,iabs(nmprrf(l)))
               endif
c  For now, treat all reflections as full, even for unmatched partials,
c ie set fraction = 1.0
c  This should be fixed sometime!
               frccal = 1.0
               call dffscn(
     $            irunrf(l), stol, phirf(l), xsirf(l),
     $            frccal, ibatch, nppart,
     $            ghl(l), Dfhl(l), dghldc(1,l), istat1)
               if (istat1 .lt. 0) then
                  call ccperr(1,
     $               '**** Partial beyond end of run ****')
               endif
            else
               Dfhl(l) = 1.0
            endif
 40      continue
      endif
c
c For summed partials, reset batch number to central slot number
      do l = 1,nrs
         if (jmidsl(l) .ne. l .and. jmidsl(l) .gt. 0) then
            ibatrf(l) = ibatrf(jmidsl(l))
         endif
      enddo
c
      return
      end



      subroutine parupd(lsdref,shift,sdpar,parold,parnew,parnam,istat)
c     =======================================================
c
c  Apply shifts to parameters, and renormalize
c
c On entry:
c     lsdref         .true. if SD factors are being refined
c     shift          parameter shifts
c     sdpar          standard deviations
c
c On exit: 
c     (on exit, scale parameters are scale K rather than inverse G)
c
c     sdpar          standard deviations (modified for scale)
c     shift          shift in renormalized parameters
c     parold         old parameters
c     parnew         new parameters
c     parnam         parameter name
c     istat          =  0  OK
c                    = +1  parameter reset
c                    = -1  at least one scale is negative
c                           in this case no parameters are altered
c
      implicit none
c
      integer istat
      logical lsdref
      real shift(*), sdpar(*), parold(*), parnew(*)
      character*(*) parnam(*)
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'sdfacc.fh'
      include 'rundef.fh'
      include 'params.fh'
      include 'inout.fh'
      include 'sphhrm.fh'
      include 'refcon.fh'
c
      integer i, j, k, l, irun, jnorm, jnormb, irunmax, kbfacmax
      real bnorm, scnorm, scale, bf(3), stovl, xd, yd, zd,
     $   bfac, bfacmax, time, ghl, Thl, Chl, Shl, dummy(maxpmr),
     $   scale0
      character*30 name, ptype*2
      logical lallbf
c
      integer lenstr
      external lenstr
c
c-save-      save /scales/,/sdfacc/
c
      istat = 0
c
c Check if any of the scale parameters are going to go
c negative before doing anything else, so that original
c parameters can be retained if necessary
      do irun=1,numrun
         l = nbfact(irun)
         do k = 1,ncscls(1,irun)
            do j = 1,ncscls(2,irun)
               l = l+1
               i = ipoint(l,irun)
               write(name,'(a,i4,a,i4,a,i4)')
     $              'K',irunum(irun),'.',k,'.',j
               call squeez(name, parnam(i))
               scale = cscals(j,k,irun) + shift(i)
               if (scale .lt. 0.0) then
                  scale0 = cscals(j,k,irun)
                  write (lunout, '(/a,a,a,f10.3,a,f10.3,a/)')
     $               ' #### WARNING: shifted scale factor ',
     $               parnam(i), ' is negative (shift = ',
     $               1./scale - 1./scale0,
     $               ', scale = ', 1./scale0, ')'
                  istat = -1
                  return
               endif
            enddo
         enddo
      enddo

c
c Which is the normalization set? set runnumber jnorm depending
c on flag knormk
      if (knormk .eq. 0 .and. irefrn .ne. 0 ) then
         jnorm = irefrn
      else
c Not reference run
c    if not specified, use 1st run
         jnorm = knormk
      endif
c
c Apply B-factor shifts without normalisation first,
c so that we can find largest later
      do irun = 1,numrun
         l = 0
c  B-factors
         if (nbfact(irun) .gt. 0) then
            do k = 1, nbfact(irun)
               if (nbfpos(irun) .eq. 1) then
c    Isotropic
                  l = l+1
                  i = ipoint(l,irun)
                  parold(i) = bfactr(1,k,irun)
                  parnew(i) = bfactr(1,k,irun) + shift(i)
                  bfactr(1,k,irun) = parnew(i)
                  sdbfac(1,k,irun) = sdpar(i)
               else
c    Anisotropic
                  do j = 1, nbfpos(irun)
                     l = l+1
                     i = ipoint(l,irun)
                     parold(i) = bfactr(j,k,irun)
                     parnew(i) = bfactr(j,k,irun) + shift(i)
                     bfactr(j,k,irun) = parnew(i)
                     sdbfac(j,k,irun) = sdpar(i)
                  enddo
               endif
            enddo
         endif
      enddo
c Bfactor defaults
      jnormb = knormb
      Bnorm = 0.0
c Do all runs have B-factors?
      lallbf = .true. 
      do 10, irun = 1, numrun
         if (.not.lbfrun(irun)) lallbf = .false.
 10   continue
c   if lallbf .false. then some batches have no B-factors refined,
c so we don't need to normalize them
      if (.not. lallbf) then
         jnormb = 0
      else
c
         if (knormb .eq. 0 .and. irefrn .ne. 0) then
c  If there is a reference set, Bs are automatically normalized to it
            jnormb = irefrn
            Bnorm = 0.0
         elseif (knormb .gt. 0) then
c  knormb has already been checked in s/r keyin for validity
            if (.not. lbfrun(jnormb)) then
c    so this shouldn't happen
               write (lunout, '(/a/)')
     $            ' **** PARUPD: no Bfactor on normalisation run ***'
               call ccperr(1,
     $            '*** PARUPD Bfactor nomralisation error ***')
            endif
c   Set normalisation run
            jnormb = knormb
         elseif (knormb .lt. 0) then
c  Find "best" B-factor to normalise on
            jnormb = -1
         endif
      endif
c
c Find the normalization shift for Bfactor
      if (jnormb .gt. 0) then
c  Normalisation run
         if (nbfpos(jnormb) .le. 1) then
c  Isotropic
            i = ipoint(1,jnormb)
            Bnorm = bfactr(1,1,jnormb)
         else
c  Anisotropic, make sum of eigenvalues = 0.0 after applying shifts
            do 21, j = 1, nbfpos(jnormb)
               i = ipoint(j, jnormb)
               bf(j) = bfactr(j,1,jnormb)
 21         continue
            call bfnorm(bf, Bnorm)
c     Bnorm/2 will be subtracted from bfactr(1,)  & bfactr(3,)
         endif
      elseif (jnormb .lt. 0) then
c  Mode BEST, Find "best" B-factor to normalise on
c   This works for isotropic B only
         stovl = 0.0001
         xd = 0.0
         yd = 0.0
         zd = 1.0
         l  = 0
         bfacmax = -1000000.
         do irun = 1, numrun
            if (nbfact(irun) .gt. 0) then
               do i = 1, nbfact(irun)
c "Time" runs from 0 for smooth scaling, from 1 for batch scaling
                  if (mbatsc(irun) .gt. 0) then
                     time = i
                  else
                     time = i - 1
                  endif
                  call sclder(
     $               irun, stovl, time, xd, yd, zd,
     $               ghl,  Thl, Chl,  Shl, dummy)
                  bfac = alog(Thl)/(2.0*stovl)
                  if (bfac .gt. bfacmax) then
                     bfacmax = bfac
                     irunmax = irun
                     kbfacmax = i
                  endif
               enddo
            endif
         enddo
         Bnorm = bfacmax
         write (lunout, '(a,i5,a,i6)')
     $      ' B-factors normalised on point', kbfacmax,
     $      ' in run ',irunum(irunmax)
      endif
c
c
c Pick an area near middle of detector for normalization
      k = ((ncscls(4,jnorm)+1)/2 - 1) * ncscls(3,jnorm) +
     $     (ncscls(3,jnorm)+1)/2
      i = ipoint(k + nbparm(jnorm),jnorm)
      scnorm = cscals(k,1,jnorm) + shift(i)
c
c
      do 100, irun = 1,numrun
         l = 0
c  B-factors: shifts already applied, normalise
         if (nbfact(irun) .gt. 0) then
            do 110, k = 1, nbfact(irun)
               if (nbfpos(irun) .eq. 1) then
c    Isotropic
                  l = l+1
                  i = ipoint(l,irun)
                  parnew(i) = bfactr(1,k,irun) - Bnorm
                  bfactr(1,k,irun) = parnew(i)
                  shift(i)  = parnew(i) - parold(i)
                  write(name,'(a,i4,a,i4)') 'B',irunum(irun),'.',k
                  call squeez(name, parnam(i))
               else
c    Anisotropic
                  do 111, j = 1, nbfpos(irun)
                     l = l+1
                     i = ipoint(l,irun)
                     parnew(i) = bfactr(j,k,irun)
                     if (j .ne. 2) then
c        Normalization only of Byy & Bzz, leave off-diagonal term alone
                        parnew(i) = parnew(i) - Bnorm*0.5
                     endif
                     bfactr(j,k,irun) = parnew(i)
                     shift(i)  = parnew(i) - parold(i)
                     write(name,'(a,i4,a,i4,a,i1)')
     $                  'B',irunum(irun),'.',k,':',j
                     call squeez(name, parnam(i))
 111              continue
               endif
 110        continue
         endif
c
         do 120, k = 1,ncscls(1,irun)
            do 121, j = 1,ncscls(2,irun)
               l = l+1
               i = ipoint(l,irun)
               write(name,'(a,i4,a,i4,a,i4)')
     $              'K',irunum(irun),'.',k,'.',j
               call squeez(name, parnam(i))
               parold(i) = cscals(j,k,irun)
               scale = (cscals(j,k,irun) + shift(i))/scnorm
c               if (scale .lt. 0.0) then
cc  Don't allow scale factor to go negative
c                  write (lunout, '(a,a,a)')
c     $                 ' ||| WARNING ||| Shifted scale factor ',
c     $                 parnam(i),
c     $                 ' is negative: reset to sd(scale)'
c                  scale = sdpar(i)
c               endif
               cscals(j,k,irun) = scale
               sdcscl(j,k,irun) = sdpar(i)
c
c  Return parameters & shifts as scale rather than inverse scale
               if (scale .ne. 0.0 .and. parold(i) .ne. 0.0) then
                  parold(i) = 1.0 / parold(i)
                  parnew(i) = 1.0 / scale
                  shift(i)  = parnew(i) - parold(i)
                  sdpar(i)  = sdpar(i) / scale**2
               else
                  parold(i) = 0.0
                  parnew(i) = 0.0
                  shift(i)  = 0.0
                  sdpar(i)  = 0.0
               endif
 121        continue
 120     continue
c
c  Diffuse correction (no normalization)
c  No update here of linked parameters
         if (ltdsrn(irun) .and. itdspt(irun) .eq. irun) then
            do 140, k = 1, nrntds(irun)
               l = l+1
               i = ipoint(l,irun)
c Parameters:
c   1.  v
c   2.  alpha0
c   3.  alpha1
               if (k .eq. 1) then
                  ptype = 'V'
               elseif (k .eq. 2) then
                  ptype = 'A0'
               else
                  ptype = 'A1'
               endif
               write(name,'(a,a,i4)')
     $              ptype,'.',irunum(irun)
               call squeez(name, parnam(i))
               parold(i) = partds(k,irun)
               parnew(i) = parold(i) + shift(i)
               if (parnew(i) .lt. partmn(1,k))  then
                  write (lunout, '(a,a,a,f8.4,a,f8.4,a)')
     $               ' >>>> WARNING: parameter ',
     $               parnam(i)(1:lenstr(parnam(i))),
     $               ' is below minimum (', partmn(1,k),
     $               '): reset to ', partdf(1,k),' and fixed'
                  partds(k,irun) = partdf(1,k)
                  lptfix(k) = .true.
                  istat = +1
               elseif (parnew(i) .gt. partmn(2,k))  then
                  write (lunout, '(a,a,a,f8.4,a,f8.4,a)')
     $               ' >>>> WARNING: parameter ',
     $               parnam(i)(1:lenstr(parnam(i))),
     $               ' is above maximum (', partmn(2,k),
     $               '): reset to ', partdf(2,k),' and fixed'
                  partds(k,irun) = partdf(2,k)
                  lptfix(k) = .true.
                  istat = +1
               else
c  OK
                  partds(k,irun) = parnew(i)
               endif
 140        continue 
         else
            l = l + nrntds(irun)
         endif
c
c  SURFACE correction (no normalization)
c  No update here of linked parameters
         if (nrnsph(irun) .gt. 0 .and. isphpt(irun) .eq. irun) then
            do k = 1, nrnsph(irun)
               l = l+1
               i = ipoint(l,irun)
               write(name,'(a,i4,a,i4)') 'Y',irunum(irun),'.',k
               call squeez(name, parnam(i))
               parold(i) = sphclm(k,irun)
               parnew(i) = parold(i) + shift(i)
               sphclm(k,irun) = parnew(i)
            enddo
         endif
c
 100  continue
c
c      if (lsdref) then
cc SD corrections (no normalization)
cc  FIXME  this code needs updating to cope with separate sdfact for
cc fulls & partials !!!
c         do 200, irun = 1,numrun
c            if (jsdfac .ne. 0) then
c               k = nprrun(irun)
c               do 210, j = 1,npqrun(irun)
c                  k = k+1
c                  i = ipoint(k,irun)
c                  parold(i) = sdfact(j,irun,1)
c                  parnew(i) = (sdfact(j,irun,1) + shift(i))
c                  sdfact(j,irun,1) = parnew(i)
c                  shift(i)  = parnew(i) - parold(i)
c                  write(name,'(a,i4,a,i4)') 'S',irunum(irun),'.',k
c                  call squeez(name, parnam(i))
c 210           continue
c            endif
c 200     continue
c      endif       
c
      return
      end
      subroutine pckflg(irun, m, nmpr, icntrc, ifrflg)
c     ================================================
c
c Pack things into reflection flag
c
c On entry:
c  irun       run number
c  m          full/partial flag, = 0 full
c  nmpr       partial scale flag, .lt. 0 for scaled partial
c  icntrc     centric flag, = 1 centric else = 0
c
c On exit:
c  ifrflg     packed flag
c
      integer irun, m, nmpr, icntrc, ifrflg
c
      ifrflg = irun
      if (m .ne. 0) then
c  Partial
         if (nmpr .lt. 0) then
c    Scaled Partial
            ifrflg = ifrflg + 2000
         else
c    Summed Partial
            ifrflg = ifrflg + 1000
         endif
      endif
      if (icntrc .ne. 0) then
         ifrflg = ifrflg + 10000
      endif
      return
      end
c
c
c
      subroutine pltrog(krogplt,
     $   nrs, jhkl, isttrf, isymrf, irunrf, ibatrf,
     $   xdetrf, ydetrf, phirf)
c    ======================================
c
c  Write rogues to (xmgrace formatted) "plot" file by
c virtual detector position
c
c On entry:
c  krogplt      stream number for output (=0 none)
c  nrs         number of observations
c  jhkl(3)       reflection indices
c  isttrf(nrs)   status flag  (see flags.fh)
c  isymrf(nrs)   symmetry number
c  irunrf(nrs)   run number, = -1 if rejected
c  ibatrf(nrs)   batch number
c  phirf(nrs)    Phi coordinate from ROT column in file (=0.0 if absent)
c
      include 'parameter.fh'
      integer krogplt, nrs, jhkl(3)
      integer isttrf(maxrfl), isymrf(maxrfl), irunrf(maxrfl),
     $   ibatrf(maxrfl)
      real xdetrf(maxrfl), ydetrf(maxrfl), phirf(maxrfl)

      include 'flags.fh'
      include 'rundef.fh'

      integer l
      integer jser, jserbt
      real xsi, xdn, ydn, zdn, dstar, c2theta

      integer ccp4_hash_lookup
      external ccp4_hash_lookup

      if (krogplt .le. 0) return

      do l=1,nrs
         if (isttrf(l) .eq. lqoutl) then
c     internal batch serial number jser
            jser = ccp4_hash_lookup(ibatrf(l))
            jserbt = isrrbt(jser)
            if (jmtxbt(jserbt, irunrf(l)) .gt. 0) then
c
c  Calculate ideal detector coordinates ydn, zdn (1/A units)
               call difcdn(3,
     $            jhkl, isymrf(l), irunrf(l), ibatrf(l), phirf(l),
     $            xsi, xdn, ydn, zdn, jserbt)
c
c Virtual detector coordinates are y/cos(2theta), z/cos(2theta)
               dstar = sqrt(xdn**2 + ydn**2 + zdn**2)
               c2theta = cos(2.*asin(0.5*dstar))
               if (c2theta .gt. 0.0) then
                  ydn = ydn/c2theta
                  zdn = zdn/c2theta
                  write (krogplt, '(2f10.4)')
     $               zdn, ydn
               endif
            endif
         endif
      enddo
      return
      end
c
c
      subroutine stwrrp(krogplt, title, smax)
c    =================================
c
c Start rogue plot file
c
c  krogplt             stream number
c  title               subtitle
c  smax                maximum s = 4(sin theta/lambda)**2
c                      = (d*max)**2 = 1/dmin**2
c
c Note radius on "detector" = d* cos theta / cos 2theta 
c
      integer krogplt
      character title*(*)
      real smax

      integer l, iset
      real dstar, theta, ctovc2t, radius, radi
      character set*2

      integer  lenstr
      external lenstr

c
      if (krogplt .le. 0) return

      call ccpdpn(krogplt, 'ROGUEPLOT', 'NEW', 'F', 0,0)

      dstar = sqrt(smax)
      theta = asin(0.5*dstar)
c  cos theta/cos 2theta
      ctovc2t = cos(theta)/cos(2.*theta)
      radius = dstar * ctovc2t
      
c      write (krogplt, 6001)
c 6001 format('$TABLE:Outliers on virtual detector',/,
c     $   '$SCATTER:Outliers:A:1,2:$$',/,
c     $   '   Y        Z      $$ $$')    
c
      write (krogplt, '(a)') '@version 40102'
      write (krogplt, '(a)')
     $   '@ title "Outliers on detector'//
     $   ' (horizontal rotation axis)"'
      l = lenstr(title)
      if (l .gt. 0) write (krogplt, '(a,a,a)')
     $   '@ subtitle "',title(1:l),'"'
c
      write (krogplt, 6000) -radius, radius, -radius, radius
 6000 format(
     $   '@    world xmin ',f8.2/
     $   '@    world xmax ',f8.2/
     $   '@    world ymin ',f8.2/
     $   '@    world ymax ',f8.2)

      write (krogplt, 6001) 
 6001 format (
     $   '@with g0'/
     $   '@g0 on'/
     $   '@g0 label off'/
     $   '@    title size 1.2'/
     $   '@    view xmin 0.150000'/
     $   '@    view xmax 0.750000'/
     $   '@    view ymin 0.150000'/
     $   '@    view ymax 0.850000'/
     $   '@    xaxis  tick off'/
     $   '@    xaxis  label ""'/
     $   '@    xaxis  ticklabel off'/
     $   '@    yaxis  tick off'/
     $   '@    yaxis  label ""'/
     $   '@    yaxis  ticklabel off'/
     $   '@G0 ON')

      write (krogplt, 6005) radius*0.5, -radius*0.92
 6005 format(
     $   '@  legend x1 ', f8.3/
     $   '@  legend y1 ', f8.3)

c Axis lines
      write (krogplt, '(a)') '@ s0 color 1'
      write (krogplt, '(2f9.4)') -radius, 0.0
      write (krogplt, '(2f9.4)')  radius, 0.0
      write (krogplt, '(a)') '&'

      write (krogplt, '(a)') '@ s1 color 1'
      write (krogplt, '(2f9.4)') 0.0, -radius
      write (krogplt, '(2f9.4)') 0.0,  radius
      write (krogplt, '(a)') '&'
c Outer circle
      write (krogplt, '(a)') '@ s2 color 1'
      call drwcirc(krogplt, radius, 100)
      write (krogplt, '(a)') '&'

c ice rings at 3.90, 3.67, 3.44
      nset = 3
c      radi = ctovc2t/3.90
      dstar = 1./3.90
      theta = asin(0.5*dstar)
      ctovc2t = cos(theta)/cos(2.*theta)
      radi = dstar * ctovc2t

      if (radi .lt. radius) then
         write (set, 6006) nset
 6006    format('s',i1)
         nset = nset+1
         write (krogplt, '(a,a,a)') '@ ',set,' color 2'
         call drwcirc(krogplt, radi, 100)
         write (krogplt, '(a)') '&'
      endif 
c      radi = ctovc2t/3.67
      dstar = 1./3.67
      theta = asin(0.5*dstar)
      ctovc2t = cos(theta)/cos(2.*theta)
      radi = dstar * ctovc2t
      if (radi .lt. radius) then
         write (set, 6006) nset
         nset = nset+1
         write (krogplt, '(a,a,a)') '@ ',set,' color 2'
         call drwcirc(krogplt, radi, 100)
         write (krogplt, '(a)') '&'
      endif
c      radi = ctovc2t/3.44
      dstar = 1./3.44
      theta = asin(0.5*dstar)
      ctovc2t = cos(theta)/cos(2.*theta)
      radi = dstar * ctovc2t
      if (radi .lt. radius) then
         write (set, 6006) nset
         nset = nset+1
         write (krogplt, '(a,a,a)') '@ ',set,' color 2'
         call drwcirc(krogplt, radi, 100)
         write (krogplt, '(a)') '&'
      endif

      write (set, 6006) nset
      write (krogplt, 6003) set, set, set, set, set, set
 6003 format (
     $   '@    ',a,' type xy'/
     $   '@    ',a,' symbol 2'/
     $   '@    ',a,' symbol size 0.2'/
     $   '@    ',a,' symbol fill 1'/
     $   '@    ',a,' symbol color 1'/
     $   '@    ',a,' linestyle 0'/
     $   '@TYPE xy')
      write (krogplt, 6007) 
 6007 format (
     $   '@  legend string   3 "Ice rings"'/
     $   '@  legend loctype world'/
     $   '@  legend color 2'/
     $   '@  legend font 4'/
     $   '@  legend char size 0.8'/
     $   '@  legend on'/)

      return
      end
c
c
c
      subroutine clwrrp(krogplt)
c    =======================
c
      integer krogplt
c
      if (krogplt .le. 0) return
      write (krogplt, 6002)
 6002 format('&')
      close (unit=krogplt)
c 6002 format('$$'/)

      return
      end

      subroutine drwcirc(krogplt, radius, npt)
c    ==================================
c
c  Draw lines for circle (xmgrace format)
c
c  krogplt   stream
c  radius    radius
c  npt       number of sample points
c
      integer krogplt, npt
      real radius
c
      real delang
      integer np, i

      integer maxpt
      parameter (maxpt=200)
      real x, y, a


      np = min(max(16, npt), maxpt)
      delang = 8.*atan(1.0)/float(np)

      do i=1,np+1
         a = delang * (i-1)
         write (krogplt,'(2f8.4)') radius*cos(a), radius*sin(a)
      enddo

      return
      end
c
c
c
      subroutine pltsph
c     =================
c
c  Plot calculated surface correction values
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'inout.fh'
      include 'outcon.fh'
      include 'sphhrm.fh'
c
      real thet1(mxstheta), phi1(mxsphi), scalc(mxstheta,mxsphi)
c
      character  title*80, type*80
c
      real tmax,theta,vwport(2,2),xlow,xhigh,ylow,yhigh,thmin,thmax,
     .   x(mxsphi),y(mxsphi),phimin,phimax,dlthet,rtod,
     $   rep1, dash1
      integer i,j,k,it,ip,nxtick,nxstck,nytick,nystck,iobs
      logical first, lomit(mxsphi), ldash
c
      real phi, cth, sth, ylm(maxspt), Shl
      integer irun
c
      integer lenstr
      external lenstr
c
c  VWPORT defines the area of the graph (whole page = 0 to 1 )
      data vwport/0.2,0.2,0.9,0.9/
c  rep1, dash1 are repeat & dash parameters (fractions of 1 as whole page)
c              for omitted observations
      data rep1,dash1/0.01,0.004/

c Start plot
      call pltini
      first = .true. 
c
      rtod=45./atan(1.)
c
c  Get ranges for scaling
c
c  All angles in degrees
      thmin = 0.0
      thmax = 180.0
      phimin = 0.0
      phimax = 360.0
c Set up all theta, phi values
      do j = 1, npltheta
         thet1(j) = float(j-1) * dpltheta
      enddo
      do i = 1, nplphi
         phi1(i) = float(i-1) * dplphi
      enddo
c
c-------------------  Loop runs to plot, one / page
      do irun = 1, numrun
         if (nrnsph(irun) .gt. 0 .and. isphpt(irun) .eq. irun) then
            if (.not. first) call pltpic
            first = .false.
c Calculate all corrections
            tmax=-1000000.
            do j = 1, npltheta
c theta, phi in radians
               theta = thet1(j) / rtod
               do i = 1, nplphi
                  phi = phi1(i) / rtod
c
                  cth = cos(theta)
                  sth = sin(theta)
                  call sphhmc(
     $               sth, cth, phi, lmxsph(irun), lodsph(irun), ylm)
c  sum Ylm * Clm  using linked parameters if required
c  First coefficient is a fixed constant 1.0, & is not used,
c  hence offset by +1 here
                  Shl = 1.0
                  do k = 1, nrnsph(irun)
                     Shl = Shl + ylm(k+1) * sphclm(k, isphpt(irun))
                  enddo
                  if (Shl .lt. 0.0) then
cc                     write (lunout, '(a)') 'Help'
                     Shl = 0.0
                  endif
                  scalc(j,i) = Shl
                  tmax = max(tmax, Shl)
               enddo
            enddo
c
c Draw axes.  Assume equal ranges for theta.  Each theta step will be divided
c  into a range of TMAX intensity
            xlow=phimin
            xhigh=phimax
            nxtick=-5
            nxstck=2
            dlthet=abs(thet1(2)-thet1(1))
            ylow=thmin*rtod
            yhigh=thmax + dlthet
            nytick=-(npltheta+1)
            nystck=0
            call pltaxs(
     $         vwport,xlow,xhigh,nxtick,nxstck,+1,'Phi (polar)',
     .         ylow,yhigh,nytick,nystck,-1,'Theta (polar)')
c
            if (lenstr(outitl) .gt. 0) then
               title = outitl
            else 
               title = 'Surface correction'
            endif
            call pltctx(title,0.55,0.98,1.3,2,0.0)
            if (lsmode(irun) .eq. 2) then
               type = ' Secondary beam correction'
            elseif (lsmode(irun) .eq. 3) then
               type = ' Absorption correction'
            elseif (lsmode(irun) .eq. 4) then
               type = ' Surface correction'
            else
               type = ' '
            endif
            write (title, '(a,i3,a,a,i3)')
     $         'Run ',irunum(irun), type(1:lenstr(type)),
     $         ', order',lmxsph(irun)
            call pltctx(title,0.55,0.94,1.3,2,0.0)
c
c  Loop graphs for each theta value
            iobs=0
            do 10,it=1,npltheta
               theta=thet1(it)
c
c Draw horizontal separating line except for bottom
               if(theta.ne.thmin) then
                  call pltmvu(phimin,theta)
                  call pltdwu(phimax,theta)
               endif
c Plot calculated values
               do 30,ip=1,nplphi
                  x(ip)=phi1(ip)
                  y(ip)=scalc(it,ip)*dlthet/tmax+theta
                  if (nsphith(it,ip,irun) .le. 0) then
                     lomit(ip) = .false.
                  else
                     lomit(ip) = .true.
                  endif
 30            continue
c Duplicate 1st point as last
               ip=nplphi
               if(x(ip).ne.x(1)) then
                  ip=nplphi+1
                  x(ip)=x(1)+sign(360.,x(nplphi))
                  y(ip)=y(1)
                  if (nsphith(it,ip,irun) .le. 0) then
                     lomit(ip) = .false.
                  else
                     lomit(ip) = .true.
                  endif
               endif
c
               j=1
               ldash=lomit(1)
               do 25,i=2,ip
                  if(lomit(i)) then
C new point is solid
                     if(ldash) then
C   so is old point, so OK
                        continue
                     else
C   was dashed, so plot dashed to this point
                        call pltdsh(1,rep1,dash1,0.)
                        call pltlin(x(j),y(j),i-j+1)
                        j=i
                     endif
                     ldash=.true.
                  else
C new point is dashed
                     if(ldash) then
C   was solid, so plot solid to previous point
                        if(i.gt.2) then
                           call pltdsh(0,0.,0.,0.)
                           call pltlin(x(j),y(j),i-j)
                           j=i-1
                        endif
                     else
C   was dashed so OK
                        continue
                     endif
                     ldash=.false.
                  endif
 25            continue
C
C plot to last point
               if(ldash) then
                  call pltdsh(0,0.,0.,0.)
               else
                  call pltdsh(1,rep1,dash1,0.)
               endif
               i=ip
               call pltlin(x(j),y(j),i-j+1)
c
 10         continue
         endif
      enddo
c
c End plot
      call pltstp
c
      return
      end
      subroutine prapsc(lanlbr, ibtanl, nsrmax, reso, label)
c     ======================================================
c
c Print applied scale by rotation/batch & resolution
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
      integer nsrmax, ibtanl
      real reso(nsrmax)
      logical lanlbr(maxrun)
      character*(*) label(2,nmtrng)
c
      real     sumkr(maxrtr), totalk, totaln
      integer  i, k, nobskr(maxrtr), i1, i2, nsmax
c
      write (lunout, 6102)
 6102 format (//1x,20('====')//
     $   ' Analysis of applied scale factors (including input scale)'/)
c
      if (ibtanl .gt. 0) then
         write (lunout, 6100)
 6100    format(//1x,
     $   'Mean scale factors analysed by Batch for each run',
     $     ' and resolution'/)
      elseif (ibtanl .lt. 0) then
         write (lunout, 6101)
 6101    format(//1x,
     $   'Mean scale factors analysed by rotation range for each run',
     $     ' and resolution'/
     $     ' Each group is labelled "run_number.range"'/
     $     ' NOTE: the number of ranges is one less than the',
     $     ' number of scale factors'/
     $     ' NOTE: a value of zero means there are no observations',
     $     ' in that range'//)
      else
         write (lunout, 6103)
 6103    format(//1x,
     $   'Mean scale factors analysed by batch or rotation range',
     $      'for each run and resolution'/
     $     ' Each rotation group is labelled "run_number.range"'/
     $     ' NOTE: the number of ranges is one less than the',
     $     ' number of scale factors'/
     $     ' NOTE: a value of zero means there are no observations',
     $     ' in that range'//)
      endif
c     
      call ccpzr(sumkr, nmtrng)
      call ccpzi(nobskr, nmtrng)
      totalk = 0.0
      totaln = 0.0
c     
c Find any trailing empty resolution ranges, search from end
      nsmax = nsrmax
      do k = nsrmax, 1, -1
         do i = 1, nmtrng
            if (n_a_k_ts(i,k) .gt. 0) then
               go to 10
            endif
         enddo
c Empty resolution range, store
         nsmax = k-1
 10      continue
      enddo

      i1 = 1
 120  i2 = min(nmtrng, i1+17)
      write (lunout, 6120) (label(2,i), i = i1,i2)
 6120 format(/1x,'Rotation  ',20a6)
      write (lunout, '(a)') '  dmax'
c     
      do 130, k = 1, nsmax
         do 134, i = i1, i2
            sumkr(i)  =  sumkr(i) + sum_a_k_ts(i,k)
            nobskr(i) = nobskr(i) + n_a_k_ts(i,k)  
            totalk = totalk +  sum_a_k_ts(i,k)
            totaln = totaln +  n_a_k_ts(i,k)
            if (n_a_k_ts(i,k) .gt. 0) then
               sum_a_k_ts(i,k) = sum_a_k_ts(i,k)/n_a_k_ts(i,k)
            else
               sum_a_k_ts(i,k) = 0.0
            endif
 134     continue
         write (lunout, 6130) reso(k), (sum_a_k_ts(i,k), i = i1,i2)
 6130    format(1x,f6.2,1x,18f6.2)
 130  continue
c     
      do 136, i = i1,i2
         if (nobskr(i) .gt. 0) then
            avg_scl_o_t(i) = sumkr(i) / nobskr(i)
            totalk = totalk + sumkr(i)
            totaln = totaln + nobskr(i)
         else
            avg_scl_o_t(i) = 0.0
         endif
 136  continue
      write (lunout, '(/1x,a7,18f6.2)')
     $     'Overall', (avg_scl_o_t(i), i = i1,i2)
c     
      i1 = i2+1
      if (i1 .le. nmtrng) go to 120
c
      if (totaln .ne. 0.0) then
         avg_scl_o = totalk/totaln
      else
         avg_scl_o = 0.0
      endif
c
      return
      end
      subroutine prares(labelr, nrunp1, reso, nsrmax, labdts)
c     ======================================================
c
      integer nrunp1, nsrmax
      real reso(nsrmax)
      character*(*) labelr(nrunp1), labdts
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
      integer i,j,k,l, nrun, irun, nobs, lr
      character line*500, cr_dr(maxrun)*8, csumdkr*8
      real sumdkr, sumikr, rtotal, atotal, r_dr(maxrun),
     $   sumdrr(maxrun), sumirr(maxrun), shbb
      integer lenstr
      external lenstr
c
c  Print analyses against resolution by run
      write (lunout, 6140) 
 6140 format(//' Rmerge against resolution for each run',
     $   ' (deviation from mean I+ or I-)'/)
      call htm_stgraph
      write (lunout, 6142) labdts(1:lenstr(labdts))
 6142  format (
     $     ' $TABLE: Rmerge for each run',a,' :')
      write (line, 6141) (i+3,i=1,nrunp1)
 6141 format(
     $     ' $GRAPHS:Rmerge v. resolution:N: 2,',(140(i2,',')))
      l = lenstr(line)
c     Overwrite last comma
      line(l:) = ': $$'
      l = lenstr(line)
      write (lunout, '(a/)') line(1:l)
c     
      write (lunout, '(1x,a,14a8,(/19x,14a8))')
     $     ' N 1/resol^2 dmax  ',(labelr(irun),irun = 1, nrunp1)
      write (lunout, '(a,a)') ' $$ $$'

      call ccpzr(sumdrr, numrun)
      call ccpzr(sumirr, numrun)
      nobs = 0
      rtotal = 0.0
      atotal = 0.0
c     
      do 140, k = 1,nsrmax
         lr = 0
         sumdkr = 0.0
         sumikr = 0.0
         shbb = 1./reso(k)**2
         do 142, irun = 1,numrun
            if (irun .ne. irefrn .and. lactvrun(irun)) then
               lr = lr+1
               if (n_m_del_rs(irun,k) .gt. 0) then
                  sumdrr(lr) = sumdrr(lr) + sum_m_del_rs(irun,k)
                  sumirr(lr) = sumirr(lr) + sum_m_i_rs(irun,k)
                  sumdkr = sumdkr + sum_m_del_rs(irun,k)
                  sumikr = sumikr + sum_m_i_rs(irun,k)
                  nobs = nobs + n_m_del_rs(irun,k)
                  rtotal = rtotal + sum_m_del_rs(irun,k)
                  atotal = atotal + sum_m_i_rs(irun,k)
                  r_dr(lr) = sum_m_del_rs(irun,k)/sum_m_i_rs(irun,k)
               else
                  r_dr(lr) = 0.0
               endif
            endif
 142     continue
         if (sumikr .ne. 0.0) then
            sumdkr = sumdkr / sumikr
         else
            sumdkr = 0.0
         endif
         nrun = lr
         do lr=1,nrun
            call tformat(r_dr(lr), cr_dr(lr), 8,3)
         enddo
         call tformat(sumdkr, csumdkr, 8,3)
         write (lunout, '(1x,i3,f8.4,f6.2,14a8,(/18x,14a8))')
     $      k, shbb, reso(k), (cr_dr(lr), lr = 1,nrun), csumdkr
 140  continue
c     
c     
      lr = 0
      do 146, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then
            lr = lr+1
            if (sumirr(lr) .ne. 0.0) then
               sumdrr(lr) = sumdrr(lr) / sumirr(lr)
            else
               sumdrr(lr) = 0.0
            endif
         endif
 146  continue
c     
      if (atotal .ne. 0.0) then
         rtotal = rtotal / atotal
      else
         rtotal = 0.0
      endif
c     
      write (lunout, '(/a)') ' $$'
      call htm_endgraph
      write (lunout, '(/1x,a8,9x,14f8.3,(/18x,14f8.3))')
     $     ' Overall',(sumdrr(lr), lr = 1,nrun), rtotal
      write (lunout, '(1x,a,14a8,(/19x,14a8))')
     $     '                 ',(labelr(irun),irun = 1, nrunp1)
c
c
      return
      end
      subroutine prarot(label, labdts)
c     ===============================
c
c Print deviations against batch/rotation
c  labdts          label for this dataset, blank if only one
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
      character*(*) label(2,maxrun*maxrot), labdts
c
      integer i, k, irun, ntot, ntm, nto, nfull, npart,
     $   nttrjs, nrtrjs, ntrejs, ntrejm, nrrejs
      real avgI, sigma, sigm0, rfact, frcsig, sumI, sumdel,
     $   summsg, sumosg, fracrej, fracrejmax, fracrejlim
      logical lwarn
      integer lenstr
      external lenstr
c
c Warn on .gt. 2% rejection
      data fracrejlim/0.02/
c
      call htm_wrthdr('Agreement by batch','batchagreement')
c
      write (lunout,fmt=6000)
 6000 format (/
     + 36x,'Agreement between batches',/
     $ 36x,'-------------------------',//,
     +//' RMS scatters are shown as SIGMA and SIGM0 for differences',/,
     + '  from Mn(I+),Mn(I-) and Mn(Imean) respectively.',//,
     + ' Rmerge in this table is the difference from Mn(Imean),',/,
     + '  but in later tables Rmerge is the difference ',
     $   'from Mn(I+),Mn(I-).',/)
      write (lunout,fmt=6002)
 6002 format (/,
     + ' NFR,NPR count those (mixed & unmixed sets) included in SIGMA',
     +/' NR  those in SIGM0 and Rmerge ',/,
     + '     (mixed & unmixed FULLY/PARTIAL SETS)',//,
     + ' NBREJ  number of reflections rejected for bad agreement',//,
     + ' NBRJM  number of reflections rejected for M .gt. 1 ',/,
     +'        (neither valid fully nor partial)',//)
      call htm_stgraph

      call dna_table_start('analysis_against_batch')

      write (lunout,fmt=6004) labdts(1:lenstr(labdts))
 6004 format (/,
     $ ' $TABLE : Analysis against all Batches for all runs ',a,' :'/,
     $ ' $GRAPHS',
     $ ' :Rmerge v Batch for all runs :N:1,9: ',
     $ ' :Imean & RMS Scatter:N:1,3,4,7:',
     $ ' :Imean/RMS scatter:N:1,8:',/
     $ ' :Number of rejections v Batch:N:1,11,12: $$',/,
     $ ' N_batch Batch_number Mn(Imean) Scatter_from_I+/- ',/,
     $ ' Number_of_fulls Number_of_partials ',
     $ ' Scatter_from_Imean Imean/Scatter_from_Imean Rmerge ',/,
     $ ' Number_in_Rmerge  Number_rejected Number_invalid ',
     $ ' Fraction_rejected $$',///,
     + '     N   BATCH   Mn(Imean)     SIGMA   ',
     + '  NFR     NPR    SIGM0  Imean/SIGM0  Rmerge   ',
     + '   NR  NBREJ  NBRJM  FracRejtd'/' $$')
c
c
      ntot = 0
      sumI = 0.0
      sumdel = 0.0
      summsg = 0.0
      ntm = 0
      sumosg = 0.0
      nto = 0
      nfull = 0
      npart = 0
      nttrjs = 0
      ntrejs = 0
      ntrejm = 0
      fracrejmax = 0.0
      lwarn = .false.
c
      i = 0
      do 10, irun = 1, numrun
         if (irun .ne. irefrn) then
            nrtrjs = 0
            nrrejs = 0
            do 20, k = 1, nrrang(irun)
               i = i+1
               avgI = 0.0
               sigma = 0.0
               sigm0 = 0.0
               frcsig = 0.0
               rfact = 0.0
c
               if (n_o_i_t(i) .gt. 0) then
                  avgI = sum_o_i_t(i)/n_o_i_t(i)
                  rfact = sum_o_del_t(i)/sum_o_i_t(i)
                  ntot = ntot + n_o_i_t(i)
                  sumI = sumI + sum_o_i_t(i)
                  sumdel = sumdel + sum_o_del_t(i)
               endif
               if (n_m_sig_t(i) .gt. 0) then
                  sigma = sqrt(sum_m_sig_t(i)/n_m_sig_t(i))
                  summsg = summsg + sum_m_sig_t(i)
                  ntm = ntm + n_m_sig_t(i)
               endif
               if (n_o_sig_t(i) .gt. 0) then
                  sigm0 = sqrt(sum_o_sig_t(i)/n_o_sig_t(i))
                  if (sigm0 .eq. 0.0) then
                     frcsig = 0.0
                  else
                     frcsig = avgI/sigm0
                  endif
                  sumosg = sumosg + sum_o_sig_t(i)
                  nto = nto + n_o_sig_t(i)
               endif
               nfull = nfull + n_m_full_t(i)
               npart = npart + n_m_part_t(i)
               nttrjs = nttrjs + ntotrjs(k,irun)
               ntrejs = ntrejs + nrejs(k,irun)
               nrtrjs = nrtrjs + ntotrjs(k,irun)
               nrrejs = nrrejs + nrejs(k,irun)
               ntrejm = ntrejm + nrejm(k,irun)
               if (ntotrjs(k,irun) .gt. 0) then
                  fracrej = float(nrejs(k,irun))/float(ntotrjs(k,irun))
               else
                  fracrej = 0.0
               endif
c

               call dna_ilist_start('batch', i)
               call dna_integer_item('batch', i)
               call dna_character_item('label',
     +              label(2, i)(1:lenstr(label(2, i))))
               call dna_real_item('mean_intensity', avgI)
               call dna_real_item('mean_sigma', sigma)
               call dna_integer_item('number_full', n_m_full_t(i))
               call dna_integer_item('number_partial', n_m_part_t(i))
               call dna_real_item('mean_intensity_sigma', frcsig)
               call dna_real_item('rmerge', rfact)
               call dna_integer_item('n_ref', n_o_del_t(i))
               call dna_integer_item('n_rejected', nrejs(k,irun))
               call dna_integer_item('n_invalid', nrejm(k,irun))
               call dna_real_item('fraction_rejected', fracrej)
               call dna_list_end

               write (lunout, 6101) i, label(2,i), avgI, sigma,
     $            n_m_full_t(i), n_m_part_t(i),
     $            sigm0, frcsig, rfact, n_o_del_t(i),
     $            nrejs(k,irun), nrejm(k,irun), fracrej
 6101          format
     $          (1x,i5,3x,a7,f10.1,f10.1,2i8,f10.1,f9.2,f11.3,
     $            i8,2i7,f9.3)
 20         continue
            if (nrtrjs .gt. 0) then
               fracrej = float(nrrejs)/float(nrtrjs)
            else
               fracrej = 0.0
            endif
            fracrejmax = max(fracrej, fracrejmax)
            if (fracrej .gt. fracrejlim) then
               lwarn = .true.
            endif
         endif
 10   continue
c
      avgI = 0.0
      sigma = 0.0
      sigm0 = 0.0
      frcsig = 0.0
      rfact = 0.0
      if (ntot .gt. 0) then
         avgI = sumI/float(ntot)
         rfact = sumdel/sumI
      endif
      if (ntm .gt. 0) then
         sigma = sqrt(summsg/ntm)
      endif
      if (nto .gt. 0) then
         sigm0 = sqrt(sumosg/nto)
         if (sigm0 .eq. 0.0) then
            frcsig = 0.0
         else
            frcsig = avgI/sigm0
         endif
      endif
c
      if (nttrjs .gt. 0) then
         fracrej = float(ntrejs)/float(nttrjs)
      else
         fracrej = 0.0
      endif
      fracrejmax = max(fracrej, fracrejmax)
      if (fracrej .gt. fracrejlim) then
         lwarn = .true.
      endif
      write (lunout, '(/a)') ' $$'

      call dna_table_end
      
      call htm_endgraph
      write (lunout, 6110) avgI, sigma,
     $   nfull, npart,
     $   sigm0, frcsig, rfact, ntot,
     $   ntrejs, ntrejm, fracrej
 6110 format (
     $   ' Overall        ',
     $   f10.1,f10.1,2i8,f10.1,f9.2,f11.3,i8,2i7,f9.3)
      write (lunout, 6111)
 6111 format (
     + '                 Mn(Imean)     SIGMA  ',
     + '   NFR     NPR    SIGM0 Imean/SIGM0   Rmerge   ',
     + '   NR  NBREJ  NBRJM  FracRejtd')
c
c
      if (lwarn) then
         write (lunout, 6001) fracrejmax, fracrejlim
 6001    format (//
     $      ' ==== WARNING ==== WARNING ==== WARNING ==== WARNING ====',
     $    //' You are rejecting a high proportion of reflections,',
     $      ' either in one run or overall',/
     $      ' Maximum fraction rejected : ', f9.3,/,
     $      '             Warning limit : ', f9.3,//,
     $      ' Are your rejection limits set too low?'//
     $      ' ==== WARNING ==== WARNING ==== WARNING ==== WARNING ===='
     $      //)
      endif
C
      return
      end
c
c
c
      subroutine praxrf(labdts)
c     ========================
c
c  Print axial reflections
c
      character*(*) labdts
c
      include 'parameter.fh'
      include 'axes.fh'
      include 'inout.fh'
c
      real aiovsd, aimax, aismax
      integer i, j, l, indmax
      integer lenstr
      external lenstr

      character hkl(3)*1, dna_label*80
      data hkl /'h','k','l'/
c
      call htm_wrthdr('Axial reflections','axes')
c Loop axes
      do 10, j = 1,3
c  Get largest index
         do 11, l = maxhkl, 1, -1
            if (sdiaxs(l,j) .gt. 0.0) then
               indmax = l
               go to 12
            endif 
 11      continue
         indmax = 0
c
 12      if (indmax .gt. 0) then
c  Get limits on I etc
            aimax  = 0.0
            aismax = 0.0
            do 20, l = 1, indmax
               if ( sdiaxs(l,j) .gt. 0.000001) then
                  aiovsd = aihaxs(l,j) / sdiaxs(l,j)
                  aimax = max(aimax, aihaxs(l,j))
                  aismax = max(aismax, aiovsd)
               endif
 20         continue
            if (aimax .gt. 0.00001 .and. aismax .gt. 0.00001) then
               aimax = 1.1*aimax
               aismax = 1.1*aismax
               call htm_stgraph

                              
               dna_label = ' '
 1009          format('axial_reflections_', a)
               write(dna_label, 1009) hkl(j)
               call dna_table_start(dna_label(1:lenstr(dna_label))) 

               write (lunout, 6001) hkl(j), labdts(1:lenstr(labdts)),
     $            hkl(j), indmax, aismax,
     $            hkl(j), indmax, aimax, hkl(j)
 6001          format (//
     $            ' $TABLE: Axial reflections, axis ',a,a,' :'/,
     $            ' $GRAPHS: ',
     $            'I/sigI vs. ',a,':0|',i6,'x0|',f10.2,':1,4:',/,
     $            6x,': I vs. ',a,':0|',i6,'x0|',f10.2,':1,2:$$'/,/,
     $            5x,a,'         I      sigI    I/sigI',10x,'  $$ $$'/)
               do 25, l = 1, indmax
                  if ( sdiaxs(l,j) .gt. 0.000001) then
                     aiovsd = aihaxs(l,j) / sdiaxs(l,j)

                     call dna_ilist_start('index', l)
                     call dna_integer_item('index', l)
                     call dna_real_item('intensity', aihaxs(l,j))
                     call dna_real_item('sigma', sdiaxs(l,j))
                     call dna_real_item('i_over_sig', aiovsd)
                     call dna_list_end


                     write (lunout, '(1x,i5,2f10.0,f10.3)')
     $                  l, aihaxs(l,j), sdiaxs(l,j), aiovsd 
                  endif
 25            continue

               call dna_table_end
               
               write (lunout, '(a)') ' $$'
               call htm_endgraph
            endif
         endif
 10   continue
c
      end
      subroutine prbfrc(nfrclbin, frclbin)
c     ==============================
c
c  Print partial bias v. fractioncalc
c
c On entry:
c  nfrclbin   number of bins
c  frclbin    bin width
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
c
      integer nfrclbin
      real frclbin
c
      integer ib, n
      real binlow, binhigh, bias, avgI, fracbias,
     $   bias2, avgI2, fracbias2
c
c
      write (lunout, 6000)
 6000 format (//
     $   'Analysis of full/partial bias against calculated fraction',
     $   ' of larger part of two: '/
     $   '   AbsBias :- Mean( Mn(If) - Ip )',/,
     $   '   FrcBias :- Mean( Mn(If) - Ip )/Mean( Mn(I) )',/,12x,
     $   'for Nbias mixed sets with fulls and 2-part partials only'//
     $   10x,' NbiasT  AbsBiasT FrcBiasT are the same for top',
     $   ' intensity bin only'/)
      write (lunout, 6001)
 6001 format (/'  Fraction range     Nbias  AbsBias FrcBias',
     $   '     NbiasT  AbsBiasT FrcBiasT'/)
c
      n = 0
      do ib = 1, nfrclbin
         if (n_b_bias_c(ib, 1) .gt. 0) then
            binlow = float(ib) * frclbin + 0.5
            binhigh = float(ib) * frclbin + 0.5
            bias = sum_b_bias_c(ib, 1)/float(n_b_bias_c(ib, 1))
            avgI = sum_b_i_c(ib, 1)/float(n_b_bias_c(ib, 1))
            fracbias = bias/avgI
            if (n_b_bias_c(ib, 2) .gt. 0) then
               bias2 = sum_b_bias_c(ib, 2)/float(n_b_bias_c(ib, 2))
               avgI2 = sum_b_i_c(ib, 2)/float(n_b_bias_c(ib, 2))
               fracbias2 = bias2/avgI2
            else
               bias2 = 0.0
               avgI2 = 0.0
               fracbias2 = 0.0
            endif
c
            write (lunout, 6002) binlow, binhigh,
     $         n_b_bias_c(ib, 1), bias, fracbias,
     $         n_b_bias_c(ib, 2), bias2, fracbias2
 6002       format(2f7.2,2(4x,i8,f8.0,f8.3))
            n = n+1
         endif
      enddo 

      if (n .eq. 0) write (lunout, '(/a)')
     $   'No mixed sets with fulls and 2-part partials'

      write (lunout, '(/)')
c
      end

      subroutine prcmpl(reso, nsrmax, smaxal, labdts, dcell)
c    ===============================================
c
c Print completeness & multiplicity table
c
      integer nsrmax
      real reso(nsrmax), smaxal, dcell(6)
      character*(*) labdts
c
      include 'parameter.fh'
C  Symmetry, cell
      include 'symmty.fh'
      include 'rfile.fh'
      include 'inout.fh'
      include 'stats.fh'
c-harvest-mod
      include 'sharvest.fh'
c
      integer i, n, nrunq1, nrunqd, nindep, ncent, naindep,
     $   ntacen, ntcent, ntanom, nrqdrs(maxres), nrqdra(maxres),
     $   nsmax
      real    ttd, shbb, thbb, complt,
     $   ccmplt, amltpl, tt1, tt2, tg, tt
      real rmeas, rmeaso, pcv, pcvo, twdl, tsi, twdo, tsio, twsg, tsim,
     $   twso, tsmo, an, acmplt, anomfrc, anmltpl,
     $   tndl, tni, tndo, tnio, rpim, rpimo
      double precision sna
      character ccomplt*7, cacmplt*7, camltpl*6, canomfrc*7, canmltpl*7,
     $   crmeas*7, crmeaso*7, ctg*7, crpim*7, crpimo*7, cpcv*7, cpcvo*7

      integer lenstr
      external lenstr
      real trnprn
      external trnprn
c
c
      twdl = 0.0
      tsi = 0.0
      twdo = 0.0
      tsio = 0.0
      tndl = 0.0
      tni = 0.0
      tndo = 0.0
      tnio = 0.0
      twsg = 0.0
      tsim = 0.0
      twso = 0.0
      tsmo = 0.0
      tt1 = 0.0
      tt2 = 0.0
c
      call ncmplt(nsrmax, srange, smaxal, dcell, nrqdrs, nrqdra)
c
      n = 0
      nindep = 0
      ncent = 0
      sna = 0.0
      naindep = 0
      call htm_wrthdr('Completeness & multiplicity','cmpltmult')
      call dna_table_start('completeness_multiplicity_vs_resolution')
      write (lunout,fmt=6134)
 6134 format(//
     $     '  Completeness and multiplicity, including reflections',
     $     ' measured only once'/
     $     '  ====================================================',
     $     '===================')
      write (lunout,fmt=6200)
 6200 format(//
     $   ' %poss is completeness in the shell, Cm%poss in cumulative ',
     $   'to that resolution',/,
     $   ' The anomalous completeness values (AnomCmpl) are',
     $   ' the percentage of possible anomalous differences measured',/,
     $   ' AnomFrc is the % of measured',
     $   ' acentric reflections for which an anomalous difference has',
     $   ' been measured'/)
c
      write (lunout,fmt=6235)
 6235 format(/
     $ ' Rmeas: redundancy-independent (multiplicity-weighted) Rmerge ',
     $   ' or R(r.i.m.) (Rmeas is relative ',
     $   'to I+ or I-, RmeasO is relative to overall mean)'/
     $   ' PCV:   pooled coefficient of variation (PCV is relative ',
     $   'to I+ or I-, PCV0 is relative to overall mean)'//
     $   ' PCV is a multiplicity-weighted RMS Rmerge'//
     $ ' Rpim: precision-indicating (multiplicity-weighted) Rmerge ',
     $   ' (Rpim is relative ',
     $   'to I+ or I-, RpimO is relative to overall mean)'/
     $   ' See:- "Improved R-factors for diffraction data analysis in',
     $   ' macromolecular crystallography"'/
     $   '       Kay Diederichs & P. Andrew Karplus, Nature Structural',
     $   ' Biology, 4, 269-275 (1997)',/,
     $   '       "On the use of the merging R factor as a quality',
     $   'indicator for X-ray data"',/,
     $   '       Manfred Weiss & Rolf Hilgenfeld, J.Appl.Cryst. 30, ',
     $   '203-205 (1997)',/,
     $   '       "Global Indicators of X-ray data quality"',/,
     $   '       Manfred Weiss, J.Appl.Cryst. 34, 130-135 (2001)',
     $   //)
      call htm_stgraph
      write (lunout, fmt=6133) labdts(1:lenstr(labdts))
 6133 format(
     $   ' $TABLE: Completeness, multiplicity, Rmeas v. resolution',
     $   a,' :'/
     $   ' $GRAPHS:Completeness v Resolution :N:2,7,8,10,11:',/,
     $   ' :Multiplicity v Resolution :N:2,9,12:',/,
     $   ' :Rpim (precision R) v Resolution :N:2,16,17:',/,
     $   ' :Rmeas, Rsym & PCV v Resolution :N:2,13,14,15,18,19:  ',
     $   '  $$'/1x,
     $   'N 1/resol^2 Dmin   Nmeas    Nref  Ncent  %poss C%poss ',
     $   'Mlplct AnoCmp AnoFrc AnoMlt Rmeas Rmeas0  (Rsym)',
     $   '  Rpim  RpimO    PCV   PCV0',/,
     $   ' $$  $$')
c
      ttd = 0.0
      ntacen = 0
      ntcent = 0
      ntanom = 0
      nrunq1 = 0

c  Find last range with any contributions to eliminate empty ranges
      call findlastrange(n_o_meas_s, nsrmax, nsmax)
c
      do 40 i = 1,nsmax
c
         shbb = 1./reso(i)**2
c
         thbb  = reso(i)
c
         nrunqd = nrqdrs(i)
         nrunq1 = nrunq1 + nrunqd
         complt = 0.0
c  nrunqd is the calculated number of reflections in this shell of the
c complete sphere
         if(nrunqd .gt. 0)
     $      complt = 100.*float(n_o_totl_s(i))/float(nrunqd)
c Overall completeness
         ttd = ttd + n_o_totl_s(i)
c Cumulative completeness
         ccmplt  = 100.*ttd/float(nrunq1)
c  Anomalous completeness
c   (1) proportion with measured DelAnom
         an = float(nrqdra(i))
         if (an .ne. 0.0) then
            acmplt = 100.*float(n_a_anom_s(i))/an
         else
            acmplt = 0.0
         endif
c   (2) proportion of measured acentrics with DelAnom
         if (n_a_acen_s(i) .ne. 0) then
            anomfrc = 100.*float(n_a_anom_s(i))/float(n_a_acen_s(i))
         else
            anomfrc = 0.0
         endif
c Totals
         ntacen = ntacen + n_a_acen_s(i)
         ntcent = ntcent + n_a_cent_s(i)
         ntanom = ntanom + n_a_anom_s(i)
c
         amltpl = 0.0
         if (n_o_uniq_s(i) .gt. 0)
     +      amltpl = float(n_o_meas_s(i))/float(n_o_uniq_s(i))
c  Mean multiplicity
         n = n + n_o_meas_s(i)
         nindep = nindep + n_o_uniq_s(i)
         ncent = ncent + n_o_cent_s(i)
c
         anmltpl = 0.0
         if (n_m_uniq_s(i) .gt. 0.0)
     +      anmltpl = sn_m_meas_s(i)/dble(n_m_uniq_s(i))
c  Mean multiplicity for anomalous
         sna = sna + sn_m_meas_s(i)
         naindep = naindep + n_m_uniq_s(i)
c Rmeas
c   relative to I+, I-
         rmeas = 0.0
         if (sum_m_i_s(i) .ne. 0.0) then
            rmeas = sum_m_wdl_s(i)/sum_m_i_s(i)
            rmeas = trnprn(rmeas, -9.99, 99.999)
            twdl = twdl + sum_m_wdl_s(i)
            tsi  = tsi  + sum_m_i_s(i)
         endif
c   relative to overall <I>
         rmeaso = 0.0
         if (sum_o_i_s(i) .ne. 0.0) then
            rmeaso = sum_o_wdl_s(i)/sum_o_i_s(i)
            rmeaso = trnprn(rmeaso, -9.99, 99.999)
            twdo = twdo + sum_o_wdl_s(i)
            tsio = tsio + sum_o_i_s(i)
         endif
c Rpim
c   relative to I+, I-
         rpim = 0.0
         if (sum_m_i_s(i) .ne. 0.0) then
            rpim = sum_m_ndl_s(i)/sum_m_i_s(i)
            rpim = trnprn(rpim, -9.99, 99.999)
            tndl = tndl + sum_m_ndl_s(i)
            tni  = tni  + sum_m_i_s(i)
         endif
c   relative to overall <I>
         rpimo = 0.0
         if (sum_o_i_s(i) .ne. 0.0) then
            rpimo = sum_o_ndl_s(i)/sum_o_i_s(i)
            rpimo = trnprn(rpimo, -9.99, 99.999)
            tndo = tndo + sum_o_ndl_s(i)
            tnio = tnio + sum_o_i_s(i)
         endif
c-- Rsym (as in prdres)
         tg = 0.0
         if (sum_m_i_s(i) .ne. 0.0) then
            tg = sum_m_del_s(i)/sum_m_i_s(i)
            tg = trnprn(tg, -9.99, 99.999)
            tt1 = tt1 + sum_m_del_s(i)
            tt2 = tt2 + sum_m_i_s(i)
         endif
c PCV
c   relative to I+, I-
         pcv = 0.0
         if (sum_m_im_s(i) .ne. 0.0) then
            pcv = sum_m_wsg_s(i) / sum_m_im_s(i)
            pcv = trnprn(pcv, -9.99, 99.999)
            twsg = twsg + sum_m_wsg_s(i)
            tsim = tsim + sum_m_im_s(i)
         endif
c   relative to overall <I>
         pcvo = 0.0
         if (sum_o_im_s(i) .ne. 0.0) then
            pcvo = sum_o_wsg_s(i) / sum_o_im_s(i)
            pcvo = trnprn(pcvo, -9.99, 99.999)
            twso = twso + sum_o_wsg_s(i)
            tsmo = tsmo + sum_o_im_s(i)
         endif

         call dna_ilist_start('shell', i)
         call dna_real_item('inv_d_sq', shbb)
         call dna_real_item('d_min', thbb)
         call dna_real_item('percent_possible', complt)
         call dna_real_item('percent_c_possible', ccmplt)
         call dna_real_item('rmeas', rmeas)
         call dna_real_item('rsym', tg)
         call dna_real_item('multiplicity', amltpl)
         call dna_list_end

         call tformat(complt, ccomplt, 7,1)
         call tformat(acmplt, cacmplt, 7, 1)
         call tformat(amltpl, camltpl, 6,1)
         call tformat(anomfrc, canomfrc, 7, 1)
         call tformat(anmltpl, canmltpl, 7, 1)
         call tformat(rmeas, crmeas, 7,3)
         call tformat(rmeaso, crmeaso, 7,3)
         call tformat(tg, ctg, 7,3)
         call tformat(rpim, crpim, 7,3)
         call tformat(rpimo, crpimo, 7,3)
         call tformat(pcv, cpcv, 7,3)
         call tformat(pcvo, cpcvo, 7,3)

         write(lunout,fmt=6135) i,shbb,thbb,
     $      n_o_meas_s(i), n_o_uniq_s(i), n_o_cent_s(i),
     $      ccomplt, ccmplt, camltpl, cacmplt, canomfrc, canmltpl,
     $      crmeas, crmeaso, ctg, crpim, crpimo, cpcv, cpcvo
 6135    format(i3,f6.3,f7.2,i9,i8,i7,
     $      a7,f7.1,a6,10a7)
         HRes2Shell(i) = reso(i)
         if (i .eq. 1) then
            hres1shell(i) = sqrt(1.0/sminlm)
         else
            hres1shell(i) = reso(i-1)
         end if
         NHall(i)  = n_o_meas_s(i)
         NHuniq(i) = n_o_uniq_s(i)
         NHcent(i) = n_o_cent_s(i)
         Hcomplt(i) = complt
         Hccmplt(i) = ccmplt
         Hamltpl(i) = amltpl
         Hpcvo(i)   = pcvo
         Hpcv(i)    = pcv
         Hrmeas(i)  = rmeas
         Hrmeaso(i) = rmeaso
         Hrpim(i)  = rpim
         Hrpimo(i) = rpimo
         Hanomfrc(i) = anomfrc
         Hanomcmp(i) = acmplt
         Hanommlt(i) = anmltpl
         NShell = i
 40   continue
      if (NShell .lt. 0) NShell = 1
c
      write (lunout, fmt='(A)') ' $$'

      call dna_table_end

      call htm_endgraph
c
      tt = float(n)/float(nindep)
      anmltpl = 0.0
      if (naindep .gt. 0) then
         anmltpl = sna/dble(naindep)
      endif
      complt = 100.*ttd/float(nrunq1)
      rmeas  = 0.0
      rmeaso = 0.0
      rpim  = 0.0
      rpimo = 0.0
      pcv  = 0.0
      pcvo = 0.0
      if (tsi .ne. 0.0) rmeas = twdl/tsi
      if (tsio .ne. 0.0) rmeaso = twdo/tsio
      if (tni .ne. 0.0) rpim = tndl/tni
      if (tnio .ne. 0.0) rpimo = tndo/tnio
      if (tsim .ne. 0.0) pcv = twsg/tsim
      if (tsmo .ne. 0.0) pcvo = twso/tsmo
      if (tt2 .ne. 0.0) tg = tt1/tt2
      tg = trnprn(tg, -9.99, 99.999)
      an = 0.0
      do i = 1, nsrmax
         an = an + float(nrqdra(i))
      enddo
      if (an .ne. 0.0) then
         acmplt = 100.*float(ntanom)/an
      else
         acmplt = 0.0
      endif
      if (ntacen .ne. 0) then
         anomfrc = 100.*float(ntanom)/float(ntacen)
      else
         anomfrc = 0.0
      endif
      call tformat(acmplt, cacmplt, 7, 1)
      call tformat(anomfrc, canomfrc, 7, 1)
      call tformat(anmltpl, canmltpl, 7, 1)
      write(lunout,fmt=6136) n, nindep, ncent, complt, complt, tt,
     $   cacmplt, canomfrc, canmltpl, rmeas, rmeaso, tg, rpim, rpimo,
     $   pcv, pcvo
 6136 format(/1x,'       Overall ',i9,i8,i7,
     $      f7.1,f7.1,f6.1,3a7,
     $      f7.3,f7.3,f7.3,f7.3,f7.3,f7.3,f7.3)
      write (lunout, 6137)
 6137 format (
     $   '                    Nmeas    Nref  Ncent  %poss C%poss ',
     $   'Mlplct AnoCmp AnoFrc AnoMlt Rmeas Rmeas0  (Rsym)',
     $   '  Rpim  RpimO    PCV   PCV0',/)
c
c-harvest-mod
      NHovermeas = n
      NHoveruniq = nindep
      NHovercent = ncent
      HcompltALL= complt
      HmulityALL = tt
      HpcvALL = pcv
      HpcvoALL = pcvo
      HrmeasALL = rmeas
      HrmeasoALL = rmeaso
      HrpimALL = rpim
      HrpimoALL = rpimo
      HanomfrcALL = anomfrc
      HanomcmpALL = acmplt
      HanommltALL = anmltpl
c
      end
c
c
c
      subroutine ncmplt(nsrmax, srange, smaxal, cell, nrqdrs, nrqdra)
c    =======================================================
c
c Calculate number of reflections in each resolution bin,
c for complete sphere, by counting all lattice points
c
c On entry:
c  nsrmax     number of bins
c  srange     bin width on 4(sin theta/lambda)**2
c  smaxal     maximum resolution in 4(sin theta/lambda)**2
c  cell(6)    unit cell
c
c On exit:
c  nrqdrs(nsrmax)  number of reflections in each bin 
c  nrqdra(nsrmax)  number of acentric reflections in each bin 
c
      integer nsrmax, nrqdrs(nsrmax), nrqdra(nsrmax)
      real srange, smaxal, cell(6)
c
      integer hmax, kmax, lmax, h, k, l, mbn_s, ihkl(3), icntrc
      real resmax, ss, epsiln
      logical absnt
c
      external lstlsq
      real     lstlsq
c
c
      resmax = 1./sqrt(smaxal)
      hmax = int(cell(1)/resmax)+1
      kmax = int(cell(2)/resmax)+1
      lmax = int(cell(3)/resmax)+1
c
      call ccpzi(nrqdrs, nsrmax)
      call ccpzi(nrqdra, nsrmax)
c Set  up lattice absence test
      call setlat
c
      do h = -hmax, hmax
         ihkl(1) = h
         do k = -kmax, kmax
            ihkl(2) = k
            do l = -lmax, lmax
               ihkl(3) = l
c
c Use mtzlib routine to calculate 4(sin theta/lambda)**2
c This should be already primed with the right cell
               ss = 4.0 * lstlsq(1,h,k,l)
               if (ss .le. smaxal) then
c Lattice absence test
                  call latabs(ihkl, absnt)
                  if (.not. absnt) then
c Bin number
                     mbn_s = min(int((ss-0.0000001)/srange) + 1, nsrmax)
                     nrqdrs(mbn_s) = nrqdrs(mbn_s) + 1
c  Centric if icntrc = 1, else = 0
                     call centr(ihkl, icntrc)
                     if (icntrc .eq. 0)
     $                  nrqdra(mbn_s) = nrqdra(mbn_s) + 1
                  endif
               endif
            enddo
         enddo
      enddo
      return 
      end
c
c
      subroutine setlat
c     =================
c
c Set up lattice symmetry test
c  R rhombaohedral lattice will be considered as hexagonal 
c    with -h+k+l = 3n if the number of primitive operators is equal total
c
c From common block
c  lattyp    single character lattice type, = P, A, B, C, R, I
c
c On exit, sets in common
c  ksysts(3)  test vector
c  isysts     test divisor
c
      include 'parameter.fh'
      include 'symmty.fh'
c
c
      ksysts(1) = 1
      ksysts(2) = 1
      ksysts(3) = 1
      isysts = 2
      if (lattyp.eq.'A') then
         ksysts(1) = 0
      else if (lattyp.eq.'B') then
         ksysts(2) = 0
      else if (lattyp.eq.'C') then
         ksysts(3) = 0
      else if (lattyp.eq.'I') then
         isysts = 2
      else if (lattyp.eq.'R') then
c Rhombohedral cell: either 
c     (a) primitive no absences, rhombohedral indexing (rare)
         if (nsym .eq. nsymp) then
            isysts = 0
         else
c or  (b) hexagonal with -h+k+l = 3n 
            isysts = 3
            ksysts(1) = -1
         endif
      else if (lattyp.eq.'H') then
c Rhombohedral cell: 
c    hexagonal with -h+k+l = 3n 
         isysts = 3
         ksysts(1) = -1
      else if (lattyp.eq.'F') then
         isysts = 4
      else if (lattyp.eq.'P') then
         isysts = 0
      else 
         call ccperr(1,'*** SETLAT: Unrecognized Lattice Type ***')
      end if
      return 
      end
c
      subroutine latabs(h, absnt)
c     ===========================
c
c Test reflection for lattice absence
c
c On entry:
c  h(3)       h,k,l
c
c On exit:
c
c  absnt     .true. if absent
c
      integer h(3)
      logical absnt
c
      include 'parameter.fh'
      include 'symmty.fh'
c
      integer i,j
c     ..
c     .. intrinsic functions ..
      intrinsic mod
c
      if (isysts.eq.0) then
c  No test needed, no absences
         absnt = .false.
         return
c
      elseif (isysts.eq.4) then
c
c---- face centred lattice
c
        absnt = .true.
c
c No need to test h+l to check for h,k,l all even or all odd
        do 10 i = 1,2
          if (mod(h(i)+h(i+1),2).ne.0) return
   10   continue
c
c
        absnt = .false.
      else
c
c---- this version for a,b,c,r,i & f lattice absences
c
        absnt = .false.
        j = 0
c
c
        do 20 i = 1,3
          j = h(i)*ksysts(i) + j
   20   continue
c
c
        if (mod(j,isysts).ne.0) absnt = .true.
      end if
c
c
      end
      subroutine prdfsc(nsrmax, reso, labelr, nrunp1)
c     ==============================================
c
c Print applied tails correction by run & resolution
c
      integer nsrmax, nrunp1
      real reso(nsrmax)
      character*(*) labelr(nrunp1)
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
c
      integer i, j, k, irun, i1, i2, lr, nrun
      real sum_r(maxrun), sum_d, avgdhl(maxrun), runlim(2,maxrun),
     $   dhlmin, dhlmax
      integer n_r(maxrun), n_d
c
      write (lunout, 6001)
 6001 format (//
     $   ' Analysis of TAILS correction (mean and range) by run ',
     $   'and resolution'//)
      write (lunout, '(1x,a,16a8/)')
     $     '  N    dmax  ',(labelr(irun),irun = 1, nrunp1)
c

      call ccpzr(sum_r, numrun)
      call ccpzi(n_r, numrun)
c     
      do 10, k = 1,nsrmax
         lr = 0
         sum_d = 0.0
         n_d   = 0
         do 20, irun = 1,numrun
            if (irun .ne. irefrn .and. lactvrun(irun)) then
               lr = lr+1
               if (n_a_t_rs(irun,k) .gt. 0) then
                  sum_r(lr) = sum_r(lr) + sum_a_t_rs(irun,k)
                  n_r(lr)    = n_r(lr)    + n_a_t_rs(irun,k)
                  sum_d = sum_d + sum_a_t_rs(irun,k)
                  n_d   = n_d   + n_a_t_rs(irun,k)
c     
                  avgdhl(lr) = sum_a_t_rs(irun,k)/n_a_t_rs(irun,k)
               else
                  avgdhl(lr) = 0.0
               endif
            endif
 20      continue
         nrun = lr
         if (n_d .gt. 0) sum_d = sum_d/n_d
         write (lunout, '(1x,i3,f8.2,15f8.3)')
     $      k, reso(k), (avgdhl(lr), lr = 1,nrun), sum_d
 10   continue
c
      sum_d = 0.0
      n_d   = 0
c
      do 30, lr = 1, nrun
         sum_d = sum_d + sum_r(lr)
         n_d   = n_d   + n_r(lr)
         if (n_r(lr) .gt. 0) then
            sum_r(lr) = sum_r(lr)/n_r(lr)
         else
            sum_r(lr) = 0.0
         endif
 30   continue
      if (n_d .gt. 0) sum_d = sum_d/n_d
c
      lr = 0
      dhlmin = +1.0e10
      dhlmax = -1.0e10
      do 40, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then
            lr = lr+1
            runlim(1,lr) = exm_t_r(1,irun)
            runlim(2,lr) = exm_t_r(2,irun)
            dhlmin = min(exm_t_r(1,irun), dhlmin)
            dhlmax = max(exm_t_r(2,irun), dhlmax)
         endif
 40   continue
c
      write (lunout, '(/1x,a8,3x,15f8.3)')
     $          ' Overall',(sum_r(lr), lr = 1,nrun), sum_d
      write (lunout, '(1x,a8,3x,15f8.3)')
     $          ' Minimum',(runlim(1,lr), lr = 1,nrun), dhlmin
      write (lunout, '(1x,a8,3x,15f8.3)')
     $          ' Maximum',(runlim(2,lr), lr = 1,nrun), dhlmax
      return
      end
      subroutine prdint(kanom, labdts)
c     ===========================
c
c  Print statistics against intensity
c
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'outcon.fh'
      include 'sharvest.fh'
c
      integer kanom
      character*(*) labdts
c
      integer i,j,k, npb, ndli, nuqi, ncni, nai, npbt
      real shbb, tg, aa, ranom, tn, ts, fsigi, td,
     $   tt1, tt2, tpbias, tpmean, fpbias, rfl, qrf1, qrf2
      real qdel, qimn, qadl, qasm, qsmi, qnmi, qssi, qnsi, qssd,
     $   qnsd, qtpb, qtpm
      character crfl*7, cranom*7
      integer lenstr
      external lenstr
      real trnprn
      external trnprn
c
      call htm_wrthdr(
     $   'Agreement by intensity','intensityagreement')
c
      if (kanom .gt. 0) then
         write (lunout,fmt=6036)
 6036    format (//,
     +  ' BY  INTENSITY ranges (all statistics use Mn(I+),Mn(I-)etc)',
     +      /,' ',31 ('__'))
      else
         write (lunout,fmt=6037)
 6037    format (//,
     +  ' BY  INTENSITY ranges (all statistics relative to Mn(I))',/,
     +      ' ',31 ('__'))
      endif
      call htm_stgraph
      write (lunout,fmt=6040) labdts(1:lenstr(labdts))
 6040 format (/,
     $  ' $TABLE: Analysis against intensity',a,' :'/
     $  ' $GRAPHS:Rmerge v Intensity :N:1,2,3,4:'/' $$'/
     +'     Imax   Rmrg  Rfull  Ranom   Nanom   Av_I    SIGMA ',
     +' I/Sigma     sd    Nmeas    Nref  Ncent   FRCBIAS',
     +'  Nbias   Abs.BIAS'/' $$  $$')
c
      qdel = 0.0
      qimn = 0.0
      qadl = 0.0
      qasm = 0.0
      qsmi = 0.0
      qnmi = 0.0
      qssi = 0.0
      qnsi = 0.0
      qssd = 0.0
      qnsd = 0.0
      qtpb = 0.0
      qtpm = 0.0
      qrf1 = 0.0
      qrf2 = 0.0
      ndli = 0
      nuqi = 0
      ncni = 0
      nai = 0
      npbt = 0
      HrmergeItop = 0.0

c
      do 10, k = 1, ninbin
         tg = 0.0
         ranom = 0.0
         rfl = 0.0
         tn = 0.0
         ts = 0.0
         fsigi = 0.0
         td = 0.0
         fpbias = 0.0
         ndli = ndli + n_m_del_i(k)
         nuqi = nuqi + n_uniq_i(k)
         ncni = ncni + n_cent_i(k)
c
c-- Rfactor
         if (sum_m_i_i(k) .ne. 0.0) then
            tg = sum_m_del_i(k)/sum_m_i_i(k)
            tg = trnprn(tg, -9.99, 99.999)
            qdel = qdel + sum_m_del_i(k)
            qimn = qimn + sum_m_i_i(k)
         endif
c-- Ranom
         if (sum_anom_sum_i(k) .ne. 0.0) then
            ranom = sum_anom_dif_i(k)/sum_anom_sum_i(k)
            ranom = trnprn(ranom, -9.99, 99.999)
            qadl = qadl + sum_anom_dif_i(k)
            qasm = qasm + sum_anom_sum_i(k)
            nai = nai + n_anom_i(k)
         endif
c-- Rfull
         if (sum_f_i_i(k) .ne. 0.0) then
            rfl  = sum_f_del_i(k)/sum_f_i_i(k)
            rfl = trnprn(rfl, -9.99, 99.999)
            qrf1 = qrf1 + sum_f_del_i(k)
            qrf2 = qrf2 + sum_f_i_i(k)
         endif
c-- Average I
         if (n_m_i_i(k) .gt. 0) then
            tn = sum_m_i_i(k)/n_m_i_i(k)
            qsmi = qsmi + sum_m_i_i(k)
            qnmi = qnmi + n_m_i_i(k)
         endif
c-- RMS scatter (Sigma)
         if (n_m_sig_i(k) .gt. 0) then
            ts = sqrt(sum_m_sig_i(k)/n_m_sig_i(k))
            if (ts .eq. 0.0) then
               fsigi = 0.0
            else
               fsigi = tn/ts
            endif
            qssi = qssi + sum_m_sig_i(k)
            qnsi = qnsi + n_m_sig_i(k)
         endif
c-- Mean sd (Sd)
         if (n_m_sd_i(k) .gt. 0) then
            td = sum_m_sd_i(k)/n_m_sd_i(k)
            qssd = qssd + sum_m_sd_i(k)
            qnsd = qnsd + n_m_sd_i(k)
         endif
c-- Fractional partial bias
         tpbias = 0.0
         tpmean = 0.0
         npb = 0
         do 20, i = 1, numrun
            if (i .ne. irefrn) then
               if (n_b_bias_ri(i,k) .gt. 0) then
                  tpbias = tpbias + sum_b_bias_ri(i,k)
                  tpmean = tpmean + sum_b_i_ri(i,k)
                  npb = npb + n_b_bias_ri(i,k)
               endif
            endif
 20      continue
         qtpb = qtpb + tpbias
         qtpm = qtpm + tpmean
         npbt = npbt + npb
         if (npb .ne. 0) then
            if (tpmean .ne. 0.0) then
               fpbias = tpbias / tpmean
            else
               fpbias = 0.0
            endif
            tpbias = tpbias / float(npb)
         else
            fpbias = 0.0
         endif
c
c  Rmerge in top bin
         if (k .eq. ninbin) then
            HrmergeItop = tg
         endif
c
         call tformat(rfl, crfl, 7,3)
         call tformat(ranom, cranom, 7,3)
         write (lunout,fmt=6038) binrng(k),tg,crfl,cranom,n_anom_i(k),
     $      tn,ts,fsigi,td,n_m_del_i(k),n_uniq_i(k),n_cent_i(k),
     $      fpbias,npb,tpbias
 6038    format (
     $      1x,f8.0,f7.3,2a7,i7,f9.0,3f8.1,i9,i8,i7,f10.3,i7,f11.1)
 10   continue
c
      write (lunout, '(a)') ' $$'
      call htm_endgraph
c
      tg = 0.0
      ranom = 0.0
      rfl = 0.0
      tn = 0.0
      ts = 0.0
      fsigi = 0.0
      td = 0.0
      fpbias = 0.0
      tpbias = 0.0
      if (qimn .ne. 0.0) tg = qdel/qimn
      if (qasm .ne. 0.0) ranom = qadl/qasm
      if (qrf2 .ne. 0.0) rfl = qrf1/qrf2
      if (qnmi .ne. 0.0) tn = qsmi/qnmi
      tg = trnprn(tg, -9.99, 99.999)
      ranom = trnprn(ranom, -9.99, 99.999)
      rfl = trnprn(rfl, -9.99, 99.999)
      if (qnsi .ne. 0.0) then
         ts = sqrt(qssi/qnsi)
         if (ts .eq. 0.0) then
            fsigi = 0.0
         else
            fsigi = tn/ts
         endif
      endif
      if (qnsd .ne. 0.0) td = qssd/qnsd
      if (qtpm .ne. 0.0) fpbias = qtpb/qtpm
      if (npbt .ne. 0) tpbias = qtpb / float(npbt)
      call tformat(rfl, crfl, 7,3)
      call tformat(ranom, cranom, 7,3)
      write (lunout, 6100) tg, crfl, cranom, nai, tn, ts, fsigi, td, 
     $   ndli, nuqi, ncni, fpbias, npbt, tpbias
 6100 format (/' Overall ',
     $      f7.3,2a7,i7,f9.0,3f8.1,i9,i8,i7,f10.3,i7,f11.1)
      write (lunout, 6101)
 6101 format (
     +'            Rmrg  Rfull  Ranom   Nanom   Av_I    SIGMA ',
     +' I/Sigma     sd    Nmeas    Nref  Ncent   FRCBIAS',
     +'  Nbias   Abs.BIAS'/)
      return
      end
      subroutine prdres(reso, nsrmax, kanom, labdts)
c     =============================================
c
c  Print statistics against resolution
c
      integer nsrmax, kanom
      real reso(nsrmax)
      character*(*) labdts
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
c-harvest-mod
      include 'sharvest.fh'
c
      integer i,j,k, npb
      real shbb, tg, aa, ranom, tn, ts, fsigi, td,
     $   tt1, tt2, tpbias, tpmean, fpbias, signal, rfl, trf1, trf2
      real    tano1, tano2, tavi, tsig, tsd, tios, tbias, tbmean
      integer nano, navi, nsig, nsd, nios, ntb, nmeas, nuniq, ncent,
     $   nsmax
      character ctg*7, crfl*7, cranom*7, caa*7,cfsigi*6, csignal*6,
     $   cfpbias*8, cts*9, ctd*7
      integer lenstr
      external lenstr
      real trnprn
      external trnprn
c
c
      write (lunout,fmt=6032)
 6032 format (/10x,
     + 'Rcum    :- R-merge up to this range,',/10X,
     + 'Ranom   :- Sum |Mn(I+) - Mn(I-)| / Sum (Mn(I+) + Mn(I-))',/10X,
     + 'Rfull   :- R-merge for fulls only,',/10X,
     + 'Nref    :- number of independent hkl''s',/10X,
     + 'Av_I    :- unmerged Ihl averaged in bin <Ihl>',/10x,
     + 'SIGMA   :- rms scatter of observations',/10x,
     + 'I/sigma :- <Ihl> / rms scatter  = AvI/SIGMA',/10x,
     + 'sd      :- average standard deviation derived from ',
     + 'experimental SDs, after',/10x,
     + '        :- application of SdFac SdB SdAdd',/10x,
     + 'Mn(I/sd):- average < merged<Ih>/sd(<Ih>) > ~= signal/noise',/10x, 
     + 'FRCBIAS :- Mean( Mn(If) - Ip )/Mean( Mn(I) )',/22X,
     +'for Nbias mixed sets only (If is a full if present, else the',
     $   /22x,'partial with the smallest number of parts)')

      call htm_wrthdr('Agreement by resolution','resagreement')
      if (kanom .gt. 0) then
         write (lunout, 6101)
 6101    format (////
     +    ' By  4SINTH/LASQ bins (all statistics use Mn(I+),Mn(I-)etc)',
     +      /,' ',31 ('__'),/)
      else
         write (lunout, 6102)
 6102    format (////
     +    ' By  4SINTH/LASQ bins (all statistics relative to Mn(I))',/,
     +      ' ',31 ('__'),/)
      endif
      call htm_stgraph

      call dna_table_start('analysis_against_resolution')      

      write (lunout,fmt=6033) labdts(1:lenstr(labdts))
 6033 format(
     $   ' $TABLE: Analysis against resolution ',a,' :'/
     $   ' $GRAPHS:',
     $   ' I/sigma, Mean Mn(I)/sd(Mn(I)):N:2,11,13:'/
     $   ': Rmerge v Resolution:N:2,4,5,7:',/,
     $   ' : Average I,sd and Sigma :A:2,9,10,12:',
     $   ' : Fractional bias :A:2,17: $$'
     $   /' N 1/resol^2 Dmin(A) Rmrg',
     +'  Rfull   Rcum  Ranom  Nanom    Av_I  SIGMA I/sigma',
     +'   sd Mn(I/sd)  Nmeas  Nref  Ncent FRCBIAS  Nbias $$',//
     $     /'   N 1/d^2 Dmin(A) Rmrg',
     +'  Rfull   Rcum  Ranom  Nanom    Av_I  SIGMA I/sigma',
     +'   sd Mn(I/sd)  Nmeas   Nref  Ncent FRCBIAS  Nbias',/
     $     ' $$')
c
c
      aa = 0.0
      tt1 = 0.0
      tt2 = 0.0
      tano1 = 0.0
      tano2 = 0.0
      nano = 0
      trf1 = 0.0
      trf2 = 0.0
      tavi = 0.0
      navi = 0
      tsig = 0.0
      nsig = 0
      tsd = 0.0
      nsd = 0
      tios = 0.0
      nios = 0
      tbias = 0.0
      tbmean = 0.0
      ntb = 0
      nmeas = 0
      nuniq = 0
      ncent = 0
c  Find last range with any contributions to eliminate empty ranges
      call findlastrange(n_m_del_s, nsrmax, nsmax)

c
      do 10, k = 1, nsmax
         tg = 0.0
         ranom = 0.0
         rfl = 0.0
         tn = 0.0
         ts = 0.0
         fsigi = 0.0
         td = 0.0
         fpbias = 0.0
         shbb = 1./reso(k)**2
c
c-- Rmerge
         if (sum_m_i_s(k) .ne. 0.0) then
            tg = sum_m_del_s(k)/sum_m_i_s(k)
            tg = trnprn(tg, -9.99, 99.999)
c  cumulative
            tt1 = tt1 + sum_m_del_s(k)
            tt2 = tt2 + sum_m_i_s(k)
            if (tt2 .ne. 0.0) then
               aa = tt1/tt2
            endif
            aa = trnprn(aa, -9.99, 99.999)
         endif
c-- Ranom
         if (sum_anom_sum_s(k) .ne. 0.0) then
            ranom = sum_anom_dif_s(k)/sum_anom_sum_s(k)
            ranom = trnprn(ranom, -9.99, 99.999)
            tano1 = tano1 + sum_anom_dif_s(k)
            tano2 = tano2 + sum_anom_sum_s(k)
            nano = nano + n_anom_s(k)
         endif
c-- Rfull
         if (sum_f_i_s(k) .ne. 0.0) then
            rfl  = sum_f_del_s(k)/sum_f_i_s(k)
            rfl = trnprn(rfl, -9.99, 99.999)
            trf1 = trf1 + sum_f_del_s(k)
            trf2 = trf2 + sum_f_i_s(k)
         endif
c-- Average I
         if (n_m_i_s(k) .gt. 0) then
            tn = sum_m_i_s(k)/n_m_i_s(k)
            tavi = tavi + sum_m_i_s(k)
            navi = navi + n_m_i_s(k)
         endif
c-- RMS scatter (Sigma)
         if (n_m_sig_s(k) .gt. 0) then
            ts = sqrt(sum_m_sig_s(k)/n_m_sig_s(k))
            if (ts .eq. 0.0) then
               fsigi = 0.0
            else
               fsigi = tn/ts
            endif
            tsig  = tsig  + sum_m_sig_s(k)
            nsig  = nsig  + n_m_sig_s(k)
         endif
c-- Mean sd (Sd)
         if (n_m_sd_s(k) .gt. 0) then
            td = sum_m_sd_s(k)/n_m_sd_s(k)
            tsd = tsd + sum_m_sd_s(k)
            nsd = nsd + n_m_sd_s(k)
         endif
c-- signal = mean <I>/sd<I>
         if (n_iovrsd_s(k) .gt. 0) then
            signal = sum_iovrsd_s(k) / n_iovrsd_s(k)
            tios = tios + sum_iovrsd_s(k) 
            nios = nios + n_iovrsd_s(k)
         else
            signal = 0.0
         endif
         nmeas = nmeas + n_m_del_s(k)
         nuniq = nuniq + n_uniq_s(k)
         ncent = ncent + n_cent_s(k)
c-- Fractional partial bias
         tpbias = 0.0
         tpmean = 0.0
         npb = 0
         do 20, i = 1, numrun
            if (i .ne. irefrn) then
               if (n_b_bias_rs(i,k) .gt. 0) then
                  tpbias = tpbias + sum_b_bias_rs(i,k)
                  tpmean = tpmean + sum_b_i_rs(i,k)
                  npb = npb + n_b_bias_rs(i,k)
               endif
            endif
 20      continue
         if (tpmean .ne. 0.0) then
            fpbias = tpbias / tpmean
            tbias  = tbias + tpbias
            tbmean = tbmean + tpmean
            ntb = ntb + npb
         else
            fpbias = 0.0
         endif
c

         call dna_ilist_start('shell', k)

         call dna_real_item('inv_d_sq', shbb)
         call dna_real_item('d_min', reso(k))
         call dna_real_item('rmerge', tg)
         call dna_real_item('rfull', rfl)
         call dna_real_item('mean_intensity', tn)
         call dna_real_item('mean_sigma', ts)
         call dna_real_item('mean_intensity_sigma', fsigi)
         call dna_real_item('signal',  signal)
         call dna_real_item('fractional_bias', fpbias)
         call dna_list_end

         call tformat(tg, ctg, 7,3)
         call tformat(aa, caa, 7,3)
         call tformat(rfl, crfl, 7, 3)
         call tformat(ranom, cranom, 7, 3)
         call iformat(nint(ts), cts, 7)
         call tformat(fsigi, cfsigi, 6,1)
         call iformat(nint(td), ctd, 7)
         call tformat(signal, csignal, 6,1)
         call tformat(fpbias, cfpbias, 8,3)
c
         write (lunout,fmt=6034)
     $      k,shbb,reso(k),
     $      ctg,crfl,caa,cranom,n_anom_s(k),nint(tn),
     $      cts,cfsigi,ctd,csignal,n_m_del_s(k),n_uniq_s(k),
     $      n_cent_s(k),cfpbias,npb
 6034    format (1x,i2,f7.4,f6.2,
     $      a7,a7,a7,a7,i7,i9,
     $      a7,a6,a7,a6,i9,i8,
     $      i6,a8,i7)
c 6034    format (1x,i2,f7.4,f6.2,
c     $      f7.3,a7,f7.3,a7,i7,i9,
c     $      i7,f6.1,i7,f6.1,i8,i7,
c     $      i6,f8.3,i7)
c-harvest-mod
         NHanom(k) = n_anom_s(k)
         HsigI(k)  = fsigi
         HRmerge(k) = tg
         HRcum(k)   = aa
         HRanom(k) = ranom
         Hsdall(k) = signal
         Hfbias(k)  = fpbias
         NHnpb(k)  = npb
 10   continue
c
      write (lunout, '(a)') ' $$'

      call dna_table_end

      call htm_endgraph
c
      ranom = 0.0
      rfl = 0.0
      tn = 0.0
      ts = 0.0
      fsigi = 0.0
      td = 0.0
      signal = 0.0
      fpbias = 0.0
      if (tano2 .gt. 0.0) ranom = tano1/tano2
      if (trf2 .gt. 0.0) rfl = trf1/trf2
      if (navi .gt. 0) tn = tavi/navi
      if (nsig .gt. 0) then
         ts = sqrt(tsig/nsig)
         if (ts .eq. 0.0) then
            fsigi = 0.0
         else
            fsigi = tn/ts
         endif
      endif
      if (nsd .gt. 0) td = tsd/nsd
      if (nios .gt. 0) signal = tios/nios
      if (tbmean .gt. 0.0) fpbias = tbias/tbmean
c
      call tformat(rfl, crfl, 7, 3)
      call tformat(ranom, cranom, 7, 3)

      write (lunout, 6040)
     $   aa,crfl,aa,cranom,nano,nint(tn),nint(ts),fsigi,
     $   nint(td),signal, nmeas, nuniq, ncent, fpbias, ntb
 6040 format (/' Overall:',7x,
     $   f7.3,a7,f7.3,a7,i7,i9,i7,f6.1,i7,f6.1,i9,i8,i6,f8.3,i7)
      write (lunout, 6041)
 6041 format (
     $      '                   Rmrg',
     +'  Rfull   Rcum  Ranom  Nanom    Av_I  SIGMA I/sigma',
     +'   sd Mn(I/sd)  Nmeas   Nref  Ncent FRCBIAS  Nbias'/)
c
      NHoverano  = nano
      Hoverfsigi = fsigi
      HRIover    = aa
      HRIAover   = ranom
      HSDover    = signal
      HoverBias  = fpbias
      NHoverntb   =ntb
      return
      end
c
c
c
      subroutine presult(lresult)
c    ================
c
c Print short summary of what was done
c Print as result block for multiple datasets
c
c On entry:
c  lresult  .true. is print as result block
      implicit none

      logical lresult

      include 'parameter.fh'
      include 'sharvest.fh'
      include 'inout.fh'
      include 'datasets.fh'
      include 'outcon.fh'

      character ctftype*16, ctftype2*16, dtslab*100, fname*200
      integer idts
      integer lenstr
      external lenstr


      if (lresult) write (lunout, 6002)
 6002 format ('$TEXT:Result: $$ $$')

      if (ndtsetsout .gt. 1) then
         write (lunout, 6010) ndtsetsout
 6010    format(/i4, '  datasets processed'/)
      endif

c File output type
      if (ifilou .gt. 0) then
         if (ifilou .eq. 1 .or. ifilou .eq. 5) then
c   AVERAGE
            ctftype = 'averaged'
         else if (ifilou .eq. 4) then
c   UNMERGED
            ctftype = 'unmerged'
         else if (ifilou .gt. 1) then
c   SEPARATE or POSTREF
            ctftype = 'separate'
         endif
         if (ndtsetsout .eq. 1) then
c  Output file name
            call ugtenv('HKLOUT', fname)
            if (fname .eq. '') fname = 'HKLOUT'
            call mkdtslab(1, dtslab)
            write (lunout, 6020) dtslab(1:lenstr(dtslab)),
     $         ctftype(1:lenstr(ctftype)),
     $         dfilename_out(1)(1:lenstr(dfilename_out(1))),
     $         dresol_out(1)
 6020       format('Dataset: ',a,/,9x,'written as ',a,
     $         ' data to output file ',a//
     $         14x,'Maximum resolution: ',f8.2, 'A'/)
            if (ifilou .eq. 5) then
               ctftype2 = 'unmerged'
               write (lunout, 6020) dtslab(1:lenstr(dtslab)),
     $         ctftype2(1:lenstr(ctftype)), 
     $         dfilename2_out(1)(1:lenstr(dfilename2_out(1))),
     $         dresol_out(1)
            endif 
         else if (ndtsetsout .gt. 1) then
c  Multiple datasets
            if (kfilou .gt. 0) then
               do idts=1,ndtsetsout
                  call mkdtslab(idts, dtslab)
c  Multiple output files
                  write (lunout, 6020) dtslab(1:lenstr(dtslab)),
     $               ctftype(1:lenstr(ctftype)), 
     $               dfilename_out(idts)(1:lenstr(dfilename_out(idts))),
     $               dresol_out(idts)
                  if (ifilou .eq. 5) then
                     ctftype2 = 'unmerged'
                     write (lunout, 6030) 
     $                  ctftype2(1:lenstr(ctftype)), 
     $                  dfilename2_out(idts)
     $                  (1:lenstr(dfilename2_out(idts)))
                  endif
               enddo
            else
c  Single output file
               write (lunout, 6030) 
     $            ctftype(1:lenstr(ctftype)), 
     $            dfilename_out(1)(1:lenstr(dfilename_out(1)))
 6030          format('Data written as ',a,
     $            ' data to output file ',a/)
               if (ifilou .eq. 5) then
                  ctftype2 = 'unmerged'
                  write (lunout, 6030) 
     $               ctftype2(1:lenstr(ctftype)), 
     $               dfilename2_out(1)(1:lenstr(dfilename2_out(1)))
               endif
               do idts=1,ndtsetsout
                  call mkdtslab(idts, dtslab)
                  write (lunout, 6031)
     $               dtslab(1:lenstr(dtslab)), dresol_out(idts)
 6031             format('  Dataset: ',a,6x,
     $               'Maximum resolution:',f6.2, 'A'/)
               enddo
            endif
         endif
      endif

      if (lresult) then
         write (lunout, '(a/a)')
     $      'See Summary Tables in logfile for details',
     $      '$$'
      endif
      return
      end
c
c
c
      subroutine provlm(amatrx, maxmtx, nmtrng, label, iprint, istat)
c     ==============================================================
c
c Print overlap matrix, either in full or in condensed form
c
c On entry:
c  amatrx(maxmtx, maxmtx)    count of overlaps
c  nmtrng                    number of batches/rotation ranges
c  label                     label for each range
c  iprint                    print flag, = 0 none, = +1 condensed, = +2 full
c
c On exit:
c  istat                     =  0    OK
c                            .gt. 0  at least one batch has no overlaps
c                                    istat = row number of this batch
c                            = -1    no overlaps at all
c
      integer maxmtx, nmtrng, iprint, istat
      double precision amatrx(maxmtx, maxmtx)
      character*(*) label(nmtrng)
c
      include 'inout.fh'
c
      integer maxlev
      parameter (maxlev=11)
      integer i1, i2, i, j, k, l, n, nmax, nrovlp(50), limits(maxlev+1)
      real aimax, range, a, al
      integer lenstr
      external lenstr
      character levels(maxlev)*1, line*150
      data levels/' ','.','-','/','+','o','x','X','$','#','*'/
c
      istat = 0
      if (nmtrng .le. 1) return
      aimax = 0.0
      do 10, j = 1, nmtrng
         k = 0
         do 11, i = 1, nmtrng
            if (i .ne. j) then
               a = amatrx(i,j)
               aimax = max(aimax, a)
               if (a .ne. 0.0) k = k+1
            endif
 11      continue
         if (k .eq. 0) then
c  Empty row found
            istat = j
            write (lunout, '(/a,a/)')
     $         ' **** WARNING: no accepted overlaps with batch ',
     $         label(j)
         endif
 10   continue
      if (aimax .le. 0.0) then
         write (lunout, '(/a/)')
     $      ' **** ERROR: no overlaps anywhere ****'
         istat = -1
         return
c        ======
      endif       
      if (iprint .le. 0) then
         return
c        ======
      elseif (iprint .eq. +1) then
c Condensed print
         range = max(alog(aimax)/10., 1.0)
         limits(1) = 0
         limits(2) = 1
         do 12, i = 2, maxlev
            limits(i+1) = int(exp(float(i-1)*range))
 12      continue
         write (lunout, '(a,a/a/)')
     $      ' Condensed printout of overlap matrix on log scale: ',
     $      'use command PRINT ALLOVERLAP to get numbers',
     $      ' Scale:'
         write (lunout, 6011) (levels(i), i=1,maxlev),
     $      (limits(i),i = 1,maxlev+1)
 6011    format (' Symbol   ', 11(7x,a1)/
     $           ' Count', 12i8//
     $      1x,25('====')/)
         i1 = 1
         nmax = 100
 20      if (i1 .le. nmtrng) then
            i2 = min(nmtrng, i1+nmax-1)
            do 21, j = 1,6
               write (line, 6020) (label(i)(j:j), i = i1,i2)
 6020          format(1x,6x,1x,100a1)
               if (lenstr(line) .gt. 0) then
                  write (lunout, '(a)') line(1:lenstr(line))
               endif
 21         continue
            write (lunout, '(1x,a6,1x,100a1)')
     $         'Range', ('_', i = i1,i2)
            line = ' '
            do 30, k = 1, nmtrng
               j = 0
               do 35, i = i1, i2
                  a = amatrx(i,k)
                  if (a .eq. 0.0) then
                     l = 1
                  else
                     al = alog(a)
                     n = int(al/range) + 2
                     l = min(n, maxlev)
                  endif
                  j = j+1
                  line(j:j) = levels(l)
 35            continue
               write (lunout, 6030) label(k), line(1:j)
 6030          format(1x,a6,'|',a)
 30         continue
            i1 = i2+1
            go to 20
         endif
c
      elseif (iprint .eq. +2) then
         write (lunout, '(/a/)')
     $      ' To get condensed printout, use PRINT CONDENSED command'
c Full print
         i1 = 1
 120     if (i1 .le. nmtrng) then
            i2 = min(nmtrng, i1+17)
            write (lunout, 6120) (label(i), i = i1,i2)
 6120       format(/1x,6x,4x,18a6)
            write (lunout, '(1x)')
            do 130, k = 1, nmtrng
               j = 0
               do 135, i = i1, i2
                  j = j+1
                  nrovlp(j) = nint(amatrx(i,k))
 135           continue
               write (lunout, 6130) label(k), (nrovlp(i), i = 1,j)
 6130          format(1x,a6,1x,18i6)
 130        continue
            i1 = i2+1
            go to 120
         endif
      endif
c     
      return
      end
      subroutine prscrn(lanlbr, ibtanl, label, timfac, labdts)
c     =======================================================
c
c  Print B-factor & scale as function of rotation/batch
c
c On entry:
c  lanlbr(irun)    .true. if analysis is by Batch
c                 .false. if by rotation range
c  ibtanl = +1 if all Batch, = -1 if all by rotation range, = 0 if mixed
c  label(1,) if always run.rotation
c  label(2,) is batch if batch analysis selected
c  timfac          time is related to zd by 
c                          timfac = nbfact(irun) / ncscls(1,irun)
c                           = 1.0 usually
c  labdts          label for this dataset, blank if only one
c
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'scales.fh'
      include 'column.fh'
c
      logical lanlbr(maxrun)
      integer ibtanl
      character*(*) label(2,nmtrng), labdts
      real timfac(maxrun)
c
      integer i, k, n, irun, nstot, nrjtot, lr, mbatch
      real    btotal, stotal, stota1, stovl, scal0, scal1,
     $   xd, yd, zd, phi, Bfac, bfcprn(2), rotang, tim, time,
     $   ghl, Thl, Chl, Shl, dummy(maxpmr), bfc(3)
      logical lslope, ldone(2)
      integer lenstr
      external lenstr
c
      call htm_wrthdr('Scale factors by batch','batchscale')
c
      if (ibtanl .gt. 0) then
         write (lunout, 6100)

         call dna_table_start('scale_factors_by_batch')

 6100    format(//
     $      ' Scale factors analysed by Batch for each run'/
     $      ' ============================================')
      elseif (ibtanl .lt. 0) then
         write (lunout, 6101)

         call dna_table_start('scale_factors_by_rotation')

 6101    format(//
     $      ' Scale factors analysed by rotation range for each run'/
     $      ' ====================================================='//
     $      ' Each group is labelled "run_number.range"'/
     $      ' NOTE: the number of ranges is one less than the',
     $      ' number of scale factors'/)
      else
         write (lunout, 6103)

         call dna_table_start('scale_factors_by_batch_or_rotation')

 6103    format(//
     $      ' Scale factors analysed by batch or rotation range for',
     $      'each run'/
     $      ' Each rotation group is labelled "run_number.range"'/
     $      ' NOTE: the number of ranges is one less than the',
     $      ' number of scale factors'/)
      endif
c     
c Do any runs have the SLOPE option? If so the format of the table changes
c
      lslope = .false.
      do 200, irun = 1, numrun
         if (lbtslp(irun)) lslope = .true.
 200  continue
c
c
      write (lunout, 6200)
 6200 format(//
     $   ' Note that 0k below is calculated for the centre of each',
     $   ' rotation range,'/
     $   '   at theta = 0 (for the B-factor) and at ',
     $   'the centre of the detector:'/
     $   ' This will be inappropriate if the detector is not ',
     $   'centred on the beam, and the scale varies across the',
     $   ' detector'//
     $   ' Mn(k) is average applied scale, including any input scale'/
     $   '  0k is the scale calculated from this scaling run'//)
c
      call htm_stgraph
      if (lslope) then
         if (nct .gt. 0) then
            write (lunout, 6201) labdts(1:lenstr(labdts))
 6201       format(
     $         ' Bfactor and mean scale Mn(k) for each rotation range'
     $         //' $TABLE: >>> Scales v rotation range',a,' :',
     $         /' $GRAPHS',
     $         ' :Mn(k) & 0k (at each end of rotation, at theta = 0) ',
     $         'v  range:N:1,7,8,9:'/
     $         ' :B  v  range:A:1,6:',/
     $         ' :Number rejected v range:N:1,11:',
     $         '  $$'/
     $         /'    N  Run.Rot     MidPhi   MidTime   ',
     $         'Batch    Bfactor   Mn(k)',
     $         '       0k.1      0k.2     Number   NumReject   $$ $$')
         else
            write (lunout, 6202) labdts(1:lenstr(labdts))
 6202       format(
     $         ' Bfactor and mean scale Mn(k) for each rotation range'
     $         //' $TABLE: >>> Scales v rotation range',a,' :',
     $         /' $GRAPHS',
     $         ' :Mn(k) & 0k (at each end of rotation, at theta = 0) ',
     $         'v  range:N:1,6,7,8:'/
     $         ' :B  v  range:A:1,5:',/
     $         ' :Number rejected v range:N:1,10:',
     $         '  $$'/
     $         /'    N  Run.Rot     MidPhi   Batch    Bfactor   Mn(k)',
     $         '       0k.1      0k.2     Number   NumReject   $$ $$')
         endif
      else
         if (nct .gt. 0) then
            write (lunout, 6203) labdts(1:lenstr(labdts))
 6203       format(
     $         ' Bfactor and mean scale Mn(k) for each rotation range'
     $         //' $TABLE: >>> Scales v rotation range',a,' :',
     $         /' $GRAPHS',
     $         ':Mn(k) & 0k (at theta = 0) v range:N:1,7,8:',/,
     $         ' :B  v range:A:1,6:',/
     $         ' :Number rejected v range:N:1,10:',
     $         '  $$'/
     $         /'    N  Run.Rot     MidPhi   MidTime   ',
     $         'Batch    Bfactor   Mn(k)',
     $         '        0k     Number   NumReject   $$ $$')
         else
            write (lunout, 6204) labdts(1:lenstr(labdts))
 6204       format(
     $         ' Bfactor and mean scale Mn(k) for each rotation range'
     $         //' $TABLE: >>> Scales v rotation range',a,' :',
     $         /' $GRAPHS',
     $         ':Mn(k) & 0k (at theta = 0) v range:N:1,6,7:',/,
     $         ' :B  v range:A:1,5:',/
     $         ' :Number rejected v range:N:1,9:',
     $         '  $$'/
     $         /'    N  Run.Rot     MidPhi   Batch    Bfactor   Mn(k)',
     $         '        0k     Number   NumReject   $$ $$')
         endif
      endif
c     
      btotal = 0.0
      stotal = 0.0
      stota1 = 0.0
      nstot  = 0
      nrjtot = 0
      i = 0
      n = 0
      lr = 0
      stovl = 0.00001
c
      do 210, irun = 1, numrun
         scal1 = 0.0
         if (irun .ne. irefrn .and. lactvrun(irun)) then
            lr = lr+1
            do 220, k = 1, nrrang(irun)
c  Loop batches/ranges for this run
               i = i+1
c B-factors, and scales at infinite resolution
               if (ncscls(2,irun) .eq. 1) then
                  xd = 0.0
                  yd = 0.0
               else
                  xd = 0.5 * (dlimrn(1,1,irun) + dlimrn(2,1,irun))
                  xd = (xd - dlimrn(1,1,irun)) / spcscl(1,irun)
                  yd = 0.5 * (dlimrn(1,2,irun) + dlimrn(2,2,irun))
                  yd = (yd - dlimrn(1,2,irun)) / spcscl(2,irun)
               endif
               ldone(1) = .false.
               ldone(2) = .false.
               mbatch = -1000
c
               if (lanlbr(irun)) then
c-- analysis by batch 
                  if (mbatsc(irun) .ne. 0) then
c---  Batch scales
c    and actually batch scales, so it's fairly simple
c (with the complication of smooth Bfactors
                     if (mbatsc(irun) .gt. 0) then
c  Bfactors by batch
                        if (nbfpos(irun) .eq. 1) then
c             isotropic
                           Bfac = bfactr(1,k,irun)
                        else
c             anisotropic
c                 get principle components
                           call bfrots(bfactr(1,k,irun), bfcprn, rotang)
c                 average Bfac = mean of principle components
                           Bfac = 0.5*(bfcprn(1)+bfcprn(2))
                        endif
                        ldone(1) = .true.
                     endif
c  Scale factors by batch
                     if (ncscls(2,irun) .le. 1) then
c       Simple case, no variation on detector, so just pick up scales
                        if (lbtslp(irun)) then
                           scal0 = 0.0
                           scal1 = 0.0
                           if (cscals(1,2*k-1,irun) .ne. 0.0)
     $                        scal0 = 1.0/cscals(1,2*k-1,irun)
                           if (cscals(1,2*k,irun) .ne. 0.0)
     $                        scal1 = 1.0/cscals(1,2*k,irun)
                        else
                           scal0 = 0.0
                           if (cscals(1,k,irun) .ne. 0.0) then
                              scal0 = 1.0/cscals(1,k,irun)
                           endif
                        endif
                     else
c  Variation across detector, so have to use sclder
c  Use sclder
                        if (lbtslp(irun)) then
                           zd = btslpf*k
                        else
                           zd = k
                        endif
                        time = zd * timfac(irun)
                        call sclder(
     $                     irun, stovl, time, xd, yd, zd,
     $                     ghl,  Thl, Chl, Shl, dummy)
                        scal0 = 0.0
                        if (Chl .ne. 0.0) then
                           scal0 = 1.0 / Chl
                        endif
                        if (lbtslp(irun)) then
                           zd = btslpf*k + 1.0
                           call sclder(
     $                        irun, stovl, time, xd, yd, zd,
     $                        ghl,  Thl, Chl,  Shl, dummy)
                           scal1 = 1.0 / Chl
                        endif
                     endif
                     ldone(2) = .true.
                  endif
                  mbatch = jbatrn(k,irun)
                  phi = 0.5*(philim(1,k,irun)  + philim(2,k,irun))
                  tim = 0.5*(timlim(1,k,irun)  + timlim(2,k,irun))
                  if (mbatsc(irun) .le. 0) then
                     zd  = (phi - zlimrn(1,irun)) / spcscl(3,irun)
                     call timcdn(tim, irun, time)
                  else
                     time = tim
                  endif
               endif
c
               if (.not.ldone(1) .or. .not.ldone(2)) then
c  Smoothed scales
                  if (.not.lanlbr(irun)) then
c     analysis by smoothed ranges
                     zd = k - 0.5
                     phi =
     $                  zd*spcscl(3,irun)+zlimrn(1,irun)
c       time is related to zd by timfac = nbfact(irun) / ncscls(1,irun)
c                                       = 1.0 usually
                     time = zd * timfac(irun)
                     tim = time
                  endif
                  if (mbatch .lt. -999) mbatch = irunum(irun)
                  if (ldone(2)) zd = 1.0
c Scale
                  call sclder(
     $               irun, stovl, time, xd, yd, zd,
     $               ghl,  Thl, Chl,  Shl, dummy)
                  if (.not. ldone(1)) then
c Bfactor
                     if (nbfpos(irun) .eq. 1) then
c             isotropic
                        Bfac = alog(Thl)/(2.0*stovl)
                     else
c             anisotropic
c                 interpolate Bs on rotation
                        call bfcitp(irun, time, bfc)
c                 get principle components
                        call bfrots(bfc, bfcprn, rotang)
c                 average Bfac = mean of principle components
                        Bfac = 0.5*(bfcprn(1)+bfcprn(2))
                     endif
                  endif
                  if (.not. ldone(2)) then
                     scal0 = 0.0
                     if (Chl .ne. 0.0) then
                        scal0 = 1.0 / Chl
                     endif
                  endif
               endif
c     
               if (n_o_t(i) .gt. 0) then
                  btotal = btotal + Bfac * n_o_t(i)
                  stotal = stotal + scal0
                  stota1 = stota1 + scal1
                  nstot  = nstot  + 1
                  n = n + n_o_t(i)
               endif
               nrjtot = nrjtot + nrejs(k,irun)
c     
               if (lslope) then
                  if (nct .gt. 0) then
                     write (lunout,
     $               '(1x,i4,3x,a6,3x,f8.2,f10.2,i8,f10.3,3f10.4,2i10)')
     $                  i, label(1,i), phi, tim, mbatch, Bfac,
     $                  avg_scl_o_t(i), scal0, scal1, n_o_t(i),
     $                  nrejs(k,irun)
                  else 
                     write (lunout,
     $                  '(1x,i4,3x,a6,3x,f8.2,i8,f10.3,3f10.4,2i10)')
     $                  i, label(1,i), phi, mbatch, Bfac,
     $                  avg_scl_o_t(i), scal0, scal1, n_o_t(i),
     $                  nrejs(k,irun)
                  endif
               else
                  if (nct .gt. 0) then
                     write (lunout,
     $               '(1x,i4,3x,a6,3x,f8.2,f10.2,i8,f10.3,2f10.4,2i10)')
     $                  i, label(1,i), phi, tim, mbatch, Bfac,
     $                  avg_scl_o_t(i), scal0, n_o_t(i),
     $                  nrejs(k,irun)
                  else

                     call dna_ilist_start('batch', i)
                     call dna_integer_item('batch_number', i)
                     call dna_character_item('batch_name', 
     +                    label(1, i)(1:lenstr(label(1, i))))
                     call dna_real_item('mid_phi', phi)
                     call dna_real_item('b_factor', bfac)
                     call dna_real_item('mean_k', avg_scl_o_t(i))
                     call dna_real_item('zero_k', scal0)
                     call dna_integer_item('n_ref', n_o_t(i))
                     call dna_integer_item('n_rejected', nrejs(k,irun))
                     call dna_list_end                     

                     write (lunout,
     $                  '(1x,i4,3x,a6,3x,f8.2,i8,f10.3,2f10.4,2i10)')
     $                  i, label(1,i), phi, mbatch, Bfac,
     $                  avg_scl_o_t(i), scal0, n_o_t(i),
     $               nrejs(k,irun)
                  endif
               endif
 220        continue
         endif
 210  continue
      write (lunout, '(1x,a)') '$$'

      call dna_table_end

      call htm_endgraph
c     
      if (n .gt. 0) then
         btotal = btotal/n
      else
         btotal = 0.0
      endif
      if (nstot .gt. 0) then
         stotal = stotal/nstot
         stota1 = stota1/nstot
      else
         stotal = 0.0
      endif
c     
      if (lslope) then
         if (nct .gt. 0) then
            write (lunout,
     $         '(/1x,6x,a6,30x,f10.3,3f10.4,2I10////)')
     $         ' Total',btotal, avg_scl_o, stotal, stota1, n,
     $         nrjtot
            write (lunout, 6211)
 6211       format (
     $         /47x, 'Bfactor   Mn(k)',
     $         '       0k.1      0k.2     Number   NumReject')
         else
            write (lunout,
     $         '(/1x,6x,a6,20x,f10.3,3f10.4,2I10////)')
     $         ' Total',btotal, avg_scl_o, stotal, stota1, n,
     $         nrjtot
            write (lunout, 6212)
 6212       format (
     $         /37x, 'Bfactor   Mn(k)',
     $         '       0k.1      0k.2     Number   NumReject')
         endif
      else
         if (nct .gt. 0) then
            write (lunout,
     $         '(/1x,6x,a6,30x,f10.3,2f10.4,2I10)')
     $         ' Total',btotal, avg_scl_o, stotal, n,
     $         nrjtot
            write (lunout, 6213)
 6213       format (
     $         /47x,'Bfactor   Mn(k)',
     $         '        0k     Number   NumReject'//)
         else
            write (lunout,
     $         '(/1x,6x,a6,20x,f10.3,2f10.4,2I10)')
     $         ' Total',btotal, avg_scl_o, stotal, n,
     $         nrjtot
            write (lunout, 6214)
 6214       format (
     $         /37x,'Bfactor   Mn(k)',
     $         '        0k     Number   NumReject'//)
         endif
      endif
c
      return
      end
      subroutine prsdan(ntfull, ntpart, labdts)
c     ========================================
c
c Print  sd analysis
c
c Sums from s/r ad2sts
c
c  Del = Ihl - <Ih>
c  FrcDel = Del/sdI
c
c  sum_m_isq_irf   = sum(I**2)         [by intensity, run & full/part]
c  sum_m_frd_irf   = sum(FrcDel)
c  sum_m_frcsq_irf = sum(FrcDel**2)
c  n_m_sd_irf      = n
c After this routine, slot for run 1 contains overall totals
c 
c  k = int((Frcdel+5)*2)+1     [ie deviation bins in steps of 1/2]
c  n_m_sdhist(k,Mfull) counts histogram
c  
c  
      integer ntfull, ntpart
      character*(*) labdts
c  
      include 'parameter.fh'
      include 'stats.fh'
      include 'outcon.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'sdfacc.fh'
c-harvest-mod
      include 'sharvest.fh'
c
      integer i, j, k, l, n, nfllpt, jj, nss(2), llab, ir, irun,
     $   nr(2), nrfull, nrpart, irntotal
      real  s1, s2, aa, d
      integer is1, is2
      real  t1(2), t2(2), t3(2), t4(2), t5(2), t6(2),
     $   t8(2), t9(2), cmin(2), cmax(2)
      double precision t7(2)
      integer it1(2), nt(2)
      character last*1, label*80
      character  ct2(2)*6, ct3(2)*6, ct7(2)*6
c
      integer lenstr
      external lenstr
c
      data last/'*'/
c
      if (ntfull .eq. 0 .and. ntpart .eq.0) return
c
      call htm_wrthdr(
     $   'Analysis of standard deviations','erroranalysis')

      call dna_table_start('analysis_of_standard_deviations')

c
      write (lunout,fmt=6084)
 6084 format (///31x,'ANALYSIS OF STANDARD DEVIATIONS',/31x,
     +'===============================',///,
     +   ' This plots the distribution of the number of observations',/,
     +'with Chi = (Ihl - Mn(Iothers) )/sqrt[sd(Ihl)**2 + sd(Mn(I))**2]',
     $   ' in ranges from -5 to +5 .',//,
     +  ' If the SD is a true estimate of the error, this distribution',
     +   ' should have Mean=0.0 and Sigma=1.0 ',/,
     $   'for all ranges of intensity.',//,
     +   ' The analysis is repeated for ranges of increasing Imean.',
     +   ' The Mean is expected to increase with Imean',/,
     $   'since the latter is a weighted mean and sd(Ihl) & Ihl are',
     $   ' correlated.',//,
     +   ' If the Sigma increases with Imean, increase the value ',
     $   'of SdAdd.'//
     $   ' In the table, Cmin, Cmax & Cmean are respectively the ',
     $   'minimum, maximum and mean ',/,
     $   'of the applied SD correction, ie SDcorrected/SDoriginal'//)
      write (lunout, 6109)
 6109 format (
     $     ' Current values of SD corrections are:-',
     $     'SdFac * Sqrt[sd(I)**2 + SdB I + (SdAdd I)**2]'/
     $     '                        Fulls        ',
     $     '                  Partials'/
     $     '        Run    SdFac       SdB     SdAdd',
     $     '        SdFac      SdB      SdAdd'/)
      do 9, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then

            call dna_ilist_start('run', irunum(irun))
            call dna_integer_item('run', irunum(irun))
            call dna_real_item('sdfac_full', sdfact(1, irun, 1))
            call dna_real_item('sdb_full', sdfact(2, irun, 1))
            call dna_real_item('sdadd_full', sdfact(3, irun, 1))
            call dna_real_item('sdfac_partial', sdfact(1, irun, 2))
            call dna_real_item('sdb_partial', sdfact(2, irun, 2))
            call dna_real_item('sdadd_partial', sdfact(3, irun, 2))
            call dna_list_end

            write (lunout, '(1x,i10,3f10.5,3x,3f10.5)')
     $         irunum(irun), ((sdfact(i,irun,j), i = 1,maxsdf),j=1,2)
         endif
 9    continue
      write (lunout, '(/)')
c
      nfllpt = 2
      if (ntpart.eq.0) nfllpt = 1
c
c  Accumulate totals in last slot, if needed and enough space
      irntotal = 0
      if (nactvrun .gt. 1 .and. numrun .lt. maxrun) then
         irntotal = maxrun
c  Clear totals
         do j = 1,2
            do i = 1, ninbin
               n_m_sd_irf(i,irntotal,j) = 0
               sum_m_isq_irf(i,irntotal,j) = 0.0
               sum_m_frd_irf(i,irntotal,j) = 0.0
               sum_m_frdsq_irf(i,irntotal,j) = 0.0
               sum_a_i_rf(i,irntotal,j) = 0.0
               n_a_i_rf(i,irntotal,j) = 0
               smax_a_i_rf(i,irntotal,j) = -1000000.
               smin_a_i_rf(i,irntotal,j) = +1000000.
            enddo
         enddo
      endif
      do j=1,2
c  Minimum & maximum SD corrections
         cmin(j) = +1000000.
         cmax(j) = -1000000.
      enddo
      HsdCminFull = 0.0
      HsdCmaxFull = 0.0
      HsdCminPart = 0.0
      HsdCmaxPart = 0.0
c
      do 5, ir = 1, numrun + 1
         if (ir .eq. irefrn) go to 5
         if (ir .le. numrun .and. .not. lactvrun(ir)) go to 5
c
         if (ir .le. numrun) then
            write (label, '(a,i5)') 'Run ', irunum(ir)
            irun = ir
         else
c Last one, for totals
            if (irntotal .eq. 0) go to 5
            write (label, '(a)') 'All runs'
            irun = irntotal
         endif
         llab = lenstr(label)
         if (numrun .gt. 1) then
            write (lunout, '(/a,a,a/)') 
     $         ' ========= ', label(1:llab), ' ========='
         endif
c
         do 10 j = 1,nfllpt
            t4(j) = 0.0
            t5(j) = 0.0
            t6(j) = 0.0
            nss(j) = 0
            t7(j) = 0.0
            nt(j) = 0
            t8(j) =  100000.
            t9(j) = -100000.
 10      continue
c
c   Which of fulls & partials is more complete?
         nrfull = 0
         nrpart = 0
         do i = 1, ninbin
c  count empty slots
            if (n_m_sd_irf(i,irun,1) .gt. 0) nrfull = nrfull + 1
            if (n_m_sd_irf(i,irun,2) .gt. 0) nrpart = nrpart + 1
         enddo
c
         call dna_table_start('standard_deviation_vs_intensity')
         

         call htm_stgraph
         if (nfllpt .eq. 1) then
            write (lunout,fmt=6086)
     $         label(1:llab), labdts(1:lenstr(labdts)),
     $         label(1:llab), label(1:llab), label(1:llab)
 6086       format (/,
     $     ' $TABLE:',a,', standard deviation v. Intensity',a,' :'/
     $         ' $GRAPHS: Sigma(scatter/SD) ',a,':N:4,7: ',
     $         ': Mean(scatter/SD) ',a,':A:4,6:',/,
     $         ' : MeanCorrection ',a,':N:4,10: ',
     $         '$$',//,
     $         ' Range Imin Imax Irms Number MeanFull SigmaFull ',
     $         'MinCorFull MaxCorFull MeanCorFull    $$ ',/,
     +         /39x,'Fully_recordeds'/,
     $         ' Range   Imin     Imax       ',
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean'
     $         /,'$$')


         else
c   Which Irms column to use? Use partial one unless incomplete
            if (nrpart .eq. ninbin) then
               j = 11
            else
               j = 4
            endif
            write (lunout,fmt=6087)
     $         label(1:llab), labdts(1:lenstr(labdts)),
     $         label(1:llab), j, label(1:llab), j, label(1:llab), j
 6087       format (/,
     $     ' $TABLE: ',a,', standard deviation v. Intensity',a,' :'/
     $         ' $GRAPHS: Sigma(scatter/SD) ',a,':N:',i2,',7,14: ',
     $         ' : Mean(scatter/SD) ',a,':A:',i2,',6,13: ',
     $         ' : MeanCorrection ',a,':N:',i2,',10,17: ',
     $         '$$',//,
     $         ' Range Imin Imax Irms Number MeanFull SigmaFull ',
     $         'MinCorFull MaxCorFull MeanCorFull Irms ',/
     $         ' Number MeanPart ',
     $        'SigmaPartial MinCorPartial MaxCorPartial MeanCorPartial',
     $         '  $$ ',/,
     +         /39x,'Fully_recordeds',37x,'Partially_recordeds',/,
     $         ' Range   Imin     Imax       ',
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean',8x,
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean'
     $         /,'$$')
         endif
c
c
         do 20 i = 1, ninbin
            if (i .eq. 1) then
               s1 = aihmin
            else
               s1 = binrng(i-1)
            endif
            if (i .eq. ninbin) then
               s2 = aihmax
            else
               s2 = binrng(i)
            endif
c
c
            do j=1,2
               t1(j) = 0.0
               t2(j) = 0.0
               t3(j) = 0.0
            enddo
            do 30 j = 1, nfllpt
               if (n_m_sd_irf(i,irun,j).gt.1) then
                  t4(j) = t4(j) + sum_m_isq_irf(i,irun,j)
                  t5(j) = t5(j) + sum_m_frd_irf(i,irun,j)
                  t6(j) = t6(j) + sum_m_frdsq_irf(i,irun,j)
                  t1(j) = sum_m_isq_irf(i,irun,j)/n_m_sd_irf(i,irun,j)
                  t1(j) = sign(sqrt(abs(t1(j))), t1(j))
                  aa = sum_m_frd_irf(i,irun,j)/n_m_sd_irf(i,irun,j)
                  t2(j) = aa
                  d  = (sum_m_frdsq_irf(i,irun,j)/
     $                   n_m_sd_irf(i,irun,j)-aa*aa)
                  t3(j) = sqrt(max(0.0,d))
                  nss(j) = nss(j) + n_m_sd_irf(i,irun,j)
               end if
               nr(j) = n_m_sd_irf(i,irun,j)
c
c Accumulate totals into slot for totals
               if ((ir .ne. irntotal) .and. (irntotal .gt. 0)) then
                  n_m_sd_irf(i,irntotal,j) = n_m_sd_irf(i,irntotal,j) +
     $               n_m_sd_irf(i,irun,j)
                  sum_m_isq_irf(i,irntotal,j) =
     $               sum_m_isq_irf(i,irntotal,j) +
     $               sum_m_isq_irf(i,irun,j)
                  sum_m_frd_irf(i,irntotal,j) =
     $               sum_m_frd_irf(i,irntotal,j) +
     $               sum_m_frd_irf(i,irun,j)
                  sum_m_frdsq_irf(i,irntotal,j) =
     $               sum_m_frdsq_irf(i,irntotal,j) +
     $               sum_m_frdsq_irf(i,irun,j)
                  smax_a_i_rf(i,irntotal,j) =
     $               max(smax_a_i_rf(i,irntotal,j),
     $               smax_a_i_rf(i,irun,j))
                  smin_a_i_rf(i,irntotal,j) =
     $               min(smin_a_i_rf(i,irntotal,j),
     $               smin_a_i_rf(i,irun,j))
                  sum_a_i_rf(i,irntotal,j) =
     $               sum_a_i_rf(i,irntotal,j) +sum_a_i_rf(i,irun,j)
                  n_a_i_rf(i,irntotal,j) =
     $               n_a_i_rf(i,irntotal,j) + n_a_i_rf(i,irun,j)
               endif
 30         continue
            is1 = nint(s1)
            is2 = nint(s2)

            do j=1,2
               it1(j) = nint(t1(j))
c
c  Average and range of correction factors
               if (n_a_i_rf(i,irun,j) .gt. 0) then
                  t7(j) = t7(j) + sum_a_i_rf(i,irun,j)
                  nt(j) = nt(j) + n_a_i_rf(i,irun,j)
                  t8(j) = min(t8(j), smin_a_i_rf(i,irun,j))
                  t9(j) = max(t9(j), smax_a_i_rf(i,irun,j))
                  sum_a_i_rf(i,irun,j) =
     $               sum_a_i_rf(i,irun,j)/float(n_a_i_rf(i,irun,j))
               else
                  smax_a_i_rf(i,irun,j) = 1.0
                  smin_a_i_rf(i,irun,j) = 1.0
               endif
               call tformat(t2(j), ct2(j), 6, 2)
               call tformat(t3(j), ct3(j), 6, 2)
               call tformat(real(sum_a_i_rf(i,irun,j)), ct7(j), 6, 2)
            enddo

            write (lunout,fmt=6088) i,is1,is2,
     +         (it1(j),nr(j),ct2(j),ct3(j),
     $         smin_a_i_rf(i,irun,j), smax_a_i_rf(i,irun,j),
     $         ct7(j),  j=1,nfllpt)
 6088       format (i5,2i8,2(4x,i8,i9,2a6,1x,2f8.2,a6))

            call dna_ilist_start('intensity_bin', i)
            call dna_real_item('min_intensity', s1)
            call dna_real_item('max_intensity', s2)
            if (nfllpt .eq. 1) then
               call dna_real_item('rms_intensity_full', t1(1))
               call dna_integer_item('number_full', nr(1))
               call dna_real_item('mean_full', t2(1))
               call dna_real_item('sigma_full', t3(1))
            else
               call dna_real_item('rms_intensity_full', t1(1))
               call dna_integer_item('number_full', nr(1))
               call dna_real_item('mean_full', t2(1))
               call dna_real_item('sigma_full', t3(1))
               call dna_real_item('rms_intensity_partial', t1(2))
               call dna_integer_item('number_partial', nr(2))
               call dna_real_item('mean_partial', t2(2))
               call dna_real_item('sigma_partial', t3(2))
            end if
            call dna_list_end
            

 20      continue
c
c
         write (lunout,fmt='(a)') ' $$'

         call dna_table_end

         call htm_endgraph
         write (lunout,fmt='(/a)') ' TOTALS:'
         i = 0
c
c
         do 50 j = 1, 2
            if (abs(nss(j)).eq.0) then
               t1(j) = 0.0
               t2(j) = 0.0
               t3(j) = 0.0
            else
               t1(j) = t4(j)/nss(j)
               t1(j) = sign(sqrt(abs(t1(j))), t1(j))
               aa = t5(j)/nss(j)
               t2(j) = aa
               t3(j) = sqrt(t6(j)/nss(j)-aa*aa)
            end if
            if (nt(j) .gt. 0) then
               t7(j) = t7(j)/float(nt(j))
               cmin(j) = min(cmin(j), t8(j))
               cmax(j) = max(cmax(j), t9(j))
            else
               t8(j) = 1.0
               t9(j) = 1.0
            endif
 50      continue
c
         do j =1,2
            call tformat(t2(j), ct2(j), 6, 2)
            call tformat(t3(j), ct3(j), 6, 2)
            call tformat(real(t7(j)), ct7(j), 6, 2)
         enddo
c
         write (lunout,fmt=6088) i, nint(aihmin), nint(aihmax),
     +      (nint(t1(j)),nss(j),ct2(j),ct3(j),
     $      t8(j),t9(j),ct7(j),j=1, nfllpt)
         if (nfllpt .eq. 1) then
            write (lunout,fmt=6186)
 6186       format (
     $         ' Range   Imin     Imax       ',
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean',
     +         /39x,'Fully_recordeds',/)
         else
c            if (ntfull .gt. 0) then
c               j = 4
c            else
c               j = 8
c            endif
            write (lunout,fmt=6187)
 6187       format (
     $         ' Range   Imin     Imax       ',
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean',8x,
     $         'Irms       N   Mean Sigma     Cmin   Cmax  Cmean'
     +         /39x,'Fully_recordeds',37x,'Partially_recordeds',/)
         endif
c-harvest-mod
        IF (irun .eq. irntotal .or. irntotal .eq. 0) THEN
           HminI = aihmin
           HmaxI = aihmax
           Hfull = t1(1)
           HFFscatter = t2(1)
           HSFscatter = t3(1)
           Hpart     = t1(2)
           HFPscatter = t2(2)
           HSPscatter = t3(2)
           IF (nfllpt .eq. 1) HLNPART = .true.
           if (ntfull .gt. 0) then
              HsdCminFull = cmin(1)
              HsdCmaxFull = cmax(1)
           endif
           if (ntpart .gt. 0) then
              HsdCminPart = cmin(2)
              HsdCmaxPart = cmax(2)
           endif
        END IF
 5    continue

c      write (lunout,fmt=6090)
c 6090 format (////,
c     +' For all REFLECTIONS, histogram of mean fractional ',/,
c     +'  deviations:  FULLYS first and then PARTIALS',//,
c     +' DEL(I)/SD  NUMBER in bin with upper limit DEL(I)/SD as shown')
c      k = 0
c      do 55, i = 1,21
c         k = max(n_m_sdhist(i,1), n_m_sdhist(i,2),k)
c 55   continue
c      k = k/100
c      if (k.le.0) k = 1
c
c
c      do 70 j = 1, nfllpt
c         write (lunout,fmt=6092)
c 6092    format (//' -5.0   %')
c         do 60 i = 1,21
c            d = real(i-10)*0.5
c            jj = n_m_sdhist(i,j)/k
c            jj = min(jj,100)
c            n = 0
c            if (nss(j).ne.0) n = nint(100.0*n_m_sdhist(i,j)/nss(j))
c            write (lunout,fmt=6094) d,n,n_m_sdhist(i,j), (last,l=1,jj)
c 6094       format (1x,f4.1,i4,i8,1x,100a1)
c 60      continue
c 70   continue


      end
c
c
c
      subroutine prsumry(jdataset, lresult)
c     ================================
c
c Write summary 
c
c On entry:
c  jdataset    dataset number  = 0 all or only dataset
c  lresult     true  if "Result" mark-up should be written (for single dataset only)
c
c Based on Harvesting routine
c
      integer jdataset
      logical lresult
c
      include 'parameter.fh'
      include 'sharvest.fh'
      include 'inout.fh'
      include 'symmty.fh'
      include 'datasets.fh'
      include 'anomtc.fh'
c
      character*(maxnamelen) pname, xname, dname
      integer i, jd
      integer lenstr
      external lenstr
c
      jd = max(1,jdataset)
      pname = pname_out(jd)
      xname = xname_out(jd)
      dname = dname_out(jd)
      
      call ccp4h_summary_beg()

      call dna_table_start('summary_data')
      call dna_list_start('overall')
      call dna_real_item('low_resolution_limit', HRes1over)
      call dna_real_item('high_resolution_limit', HRes2over)
      call dna_real_item('rmerge', HRIover)
      call dna_integer_item('total_observations', NHovermeas)
      call dna_integer_item('total_unique_observations', NHoveruniq)
      call dna_real_item('mean_i_over_sigma', HSDover)
      call dna_real_item('completeness', HcompltALL)
      call dna_real_item('multiplicity', HmulityALL)
      call dna_list_end
      call dna_list_start('outer_shell')
      call dna_real_item('low_resolution_limit', HRes1Shell(nshell))
      call dna_real_item('high_resolution_limit', HRes2Shell(nshell))
      call dna_real_item('rmerge', HRmerge(nshell))
      call dna_integer_item('total_observations', NHall(nshell))
      call dna_integer_item('total_unique_observations', 
     +     NHuniq(nshell))
      call dna_real_item('mean_i_over_sigma', hsdall(nshell))
      call dna_real_item('completeness', Hcomplt(nshell))
      call dna_real_item('multiplicity', Hamltpl(nshell))
      call dna_list_end
      call dna_table_end


      write (lunout, 6001)
 6001 format (/10('========')/)
      if (lresult) write (lunout, 6002)
 6002 format ('$TEXT:Result: $$ $$')
c
c
      write (lunout, '(3(a,a)//43x,a)')
     $   'Summary data for   Project: ', pname(1:lenstr(pname)),
     $   ' Crystal: ', xname(1:lenstr(xname)),
     $   ' Dataset: ', dname(1:lenstr(dname)),
     $   'Overall  InnerShell  OuterShell'
c
      write (lunout, 6010)
     $   '  Low resolution limit                  ',
     $   HRes1over,  HRes1Shell(1),  HRes1Shell(nshell),
     $   '  High resolution limit                 ',
     $   HRes2over,  HRes2Shell(1),  HRes2Shell(nshell),
     $   '  Rmerge                                ',
     $   HRIover,    HRmerge(1), HRmerge(nshell),
     $   '  Rmerge in top intensity bin           ',
     $   HrmergeItop,
     $   '  Rmeas (within I+/I-)                  ',
     $   HrmeasALL,  Hrmeas(1), Hrmeas(nshell),
     $   '  Rmeas (all I+ & I-)                   ',
     $   HrmeasoALL, Hrmeaso(1), Hrmeaso(nshell),
     $   '  Rpim (within I+/I-)                   ',
     $   HrpimALL,  Hrpim(1), Hrpim(nshell),
     $   '  Rpim (all I+ & I-)                    ',
     $   HrpimoALL, Hrpimo(1), Hrpimo(nshell),
     $   '  Fractional partial bias               ',
     $   HOverBias,  Hfbias(1), Hfbias(nshell),
     $   '  Total number of observations          ',
     $   NHovermeas, NHall(1), NHall(nshell),
     $   '  Total number unique                   ',
     $   NHoveruniq, NHuniq(1), NHuniq(nshell),
     $   '  Mean((I)/sd(I))                       ',
     $   HSDover, hsdall(1), hsdall(nshell),
     $   '  Completeness                          ',
     $   HcompltALL, Hcomplt(1), Hcomplt(nshell),
     $   '  Multiplicity                          ',
     $   HmulityALL, Hamltpl(1), Hamltpl(nshell)
 6010 format (2(a,3f10.2/),/,a,3f10.3/
     $   (a,f10.3,'        -         -'/),
     $   5(a,3f10.3/),2(a,3i10/),3(a,3f10.1/))
c
c Anomalous
c^      if (NHoverano .gt. 0) then
      write (lunout,6020)
     $   '  Anomalous completeness                ',
     $   HanomcmpALL, Hanomcmp(1), Hanomcmp(nshell),
     $   '  Anomalous multiplicity                ',
     $   HanommltALL, Hanommlt(1), Hanommlt(nshell),
     $   '  DelAnom correlation between half-sets ',
     $   HanoCCALL,   HanoCC(1),   HanoCC(nshell),
     $   '  Mid-Slope of Anom Normal Probability  ',
     $   HanoNPs
 6020 format (2(a,3f10.1/),a,3f10.3/,a,f10.3,
     $   '       -         -')
c^      endif
c
      if (kanomo .eq. 0) then
         write (lunout,'(/a,a)')
     $      'Outlier rejection and statistics assume that there is no',
     $      ' anomalous scattering'
      else
         write (lunout,'(/a,a)')
     $      'Outlier rejection and statistics assume that there is',
     $      ' anomalous scattering, ie I+ differs from I-'
      endif

      write (lunout, '(/a,3f8.2,4x,3f8.2/)')
     $   'Average unit cell: ',(cell(i), i=1,6)
      write (lunout, '(a,a/)')
     $   'Space group: ', spgnam
      write (lunout, '(a,f8.2/)')
     $   'Average mosaicity: ', dmosaic_out(jd)
c  Range of SD corrections
      if (Hfull .gt. 0 .and. Hpart .gt. 0) then
         write (lunout, 6030)
     $      HsdCminFull, HsdCmaxFull, HsdCminPart, HsdCmaxPart
 6030    format(
     $      'Minimum and maximum SD correction factors:',
     $      ' Fulls ',f6.2,f7.2,'  Partials ',f6.2,f7.2/)
      else if (Hpart .eq. 0) then
         write (lunout, 6031)
     $      HsdCminFull, HsdCmaxFull
 6031    format(
     $      'Minimum and maximum SD correction factors:',
     $      ' Fulls ',f6.2,f7.2/)
      else if (Hfull .eq. 0) then
         write (lunout, 6032)
     $      HsdCminPart, HsdCmaxPart
 6032    format(
     $      'Minimum and maximum SD correction factors:',
     $      ' Partials ',f6.2,f7.2/)
      endif
c
      if (lresult) then
         call presult(.false.)
         write (lunout, '(a)') '$$'
      endif
      write (lunout, 6001)      
      call ccp4h_summary_end()
c
      return
      end


c
c
c
      subroutine prtmtc(irun)
c     ======================
c
c Print matching symmetry operator pairs for run irun
c
c On entry:
c  irun    run number
c
      integer irun
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'anomtc.fh'
      include 'rundef.fh'
c
      integer i, j, l
c
c Number of characters in symmetry operator
      integer nchsym
      parameter (nchsym=14)
      character symstr(2*maxsym)*(nchsym), string*80
c
c
c Make string versions of all inverse symmetry operators
      do 10, l = 1, nisym
         call strrsm(rinsym(1,1,l), string)
c  Save only first nchsym characters
         symstr(l) = string(1:nchsym)
 10   continue
c
      write (lunout, '(/a/)')
     $   ' Isym1  Symmetry1     Isym2  Symmetry2 '
      do 20, l = 1,nisym
         write (lunout, '(1x,2(i4,3x,a))')
     $      l, symstr(l), mtcsym(l,irun), symstr(mtcsym(l,irun))
 20   continue
c
      return
      end
      subroutine prtrog(irogue, nrs, ihkl, isymrf, ffrf, sdffrf,
     $   ibatrf, frclrf, isttrf, mrf, ghl, clprf, delrf, ss,
     $   swih, swihpl, swihmn, xdetrf, ydetrf, phirf, sclirf, eorf)
c     =============================================================
c
c  Print ROGUES file
c
      implicit none
c
      integer irogue, nrs, ihkl(3)
      integer isymrf(nrs), ibatrf(nrs), isttrf(nrs), mrf(nrs)
      real ffrf(nrs), sdffrf(nrs), frclrf(nrs), ghl(nrs), clprf(nrs),
     $   delrf(nrs), ss, swih, swihpl, swihmn,
     $   xdetrf(nrs), ydetrf(nrs), phirf(nrs), sclirf(nrs), eorf(nrs)
c
      include 'flags.fh'
c
c Locals
      integer jhkl(3), l, iffr, isdffr, iwih, iwihpl, iwihmn
      real scale
      character flag*4, labfpr*1
c Blank line
      write (irogue, *)
c
      do 10, l = 1,nrs
c
         if (isttrf(l) .ge. lqOK .or.
     $      isttrf(l) .le. lqoutl) then
c Get original indices
            call asuget(ihkl,jhkl,isymrf(l))
            if (mod(isymrf(l), 2) .eq. 0) then
               flag = '  I-'
            else
               flag = '  I+'
            endif
            if (isttrf(l) .eq. lqoutl) then
               flag(2:2) = '*'
            elseif (isttrf(l) .eq. lqoutlan) then
               flag(2:2) = '@'
            elseif (isttrf(l) .eq. lqtoobig) then
               flag(2:2) = '#'
            else
               flag(2:2) = ' '
            endif
            if (abs(ghl(l)) .gt. 0.00001) then
               scale = 1.0/ghl(l)
               iffr = nint(ffrf(l)*scale)
               isdffr = nint(sdffrf(l)*scale)
            else
               iffr = 0
               isdffr = 0
               scale = 0.0
            endif
            if (mrf(l) .eq. 0) then
               labfpr = 'f'
            elseif (mrf(l) .eq. 1) then
               labfpr = 'p'
            else
               labfpr = 'x'
            endif
c
            scale = scale * sclirf(l)
c
            write (irogue, fmt=6022) jhkl, ihkl, ibatrf(l),
     $         iffr, isdffr, eorf(l), frclrf(l), labfpr, flag, scale,
     $         clprf(l), delrf(l), 1./sqrt(ss),
     $         xdetrf(l), ydetrf(l), phirf(l)
 6022       format (3i4,2x,3i4,i7,i7,i6,f6.2,f6.2,a1,a4,2f7.3,f7.1,f6.2,
     $         3f7.1)
         endif
 10   continue
c Scale weighted means
      iwih = nint(swih)
      iwihpl = nint(swihpl)
      iwihmn = nint(swihmn)
      if (iwihpl .eq. 0 .and. iwihmn .eq. 0) then
         write (irogue, 6030) iwih
 6030    format (19x,' Weighted mean', i7)
      else
         write (irogue, 6040) iwih, iwihpl, iwihmn
 6040    format (19x,' Weighted mean', i7,' I+ :',i7,' I- :',i7)
      endif
c
      return
      end
      subroutine prtscl
c     =================
c     
c     Print final parameters
c     
      implicit none
c     
      include 'parameter.fh'
      include 'scales.fh'
      include 'sdfacc.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'rfile.fh'
      include 'sphhrm.fh'
c     
      integer lenstr
      external lenstr
c     
      character line*200, line2*200, line3*200, blank*80,
     $   bfclab(3)*2
      integer nclwid, nclscl, nclpad, ncx, nset, irun, ncchar
      integer i, j, k, i1, i2, j1, j2, l, lz, l1, l2, iy, jphi, jj,
     $   ir
      real sclk(16), sdsclk(16), value(20), phi(50),
     $     stovl, time, xd, yd, zd, ghl, Thl, Chl, Shl, dummy(maxpmr),
     $     bfcprn(2,maxrot), rotang(maxrot), bfmean(maxrot),
     $     resmax, alpha
      logical lintrp
c
      data bfclab/'yy','yz','zz'/
c     
      call htm_wrthdr('Final scale factors','finalscale')
c
      write (lunout, 6001)
     $     ' Final scale factors:',
     $     ' ===================='
 6001 format (////1x,30('****')//a/a//)
c     
c     Page layout parameters
      nclwid = 120
c     Number of characters / scale (also in format statements below)
      nclscl = 7
      nclpad = 7
c     
      blank = ' '
      stovl = 0.0001
c     
      do 10, irun = 1, numrun
         write (lunout, 6010) irunum(irun)
 6010    format(//' ============ Run ',i4,' ============='//)
c     
         if (lbfrun(irun)) then
            write (lunout, '(a/)') ' Relative B-factors:'
c     
            ncx = (nclwid - 12) / nclscl
            xd = 0.0
            yd = 0.0
            zd = 1.0
            if (nbfpos(irun) .gt. 1) then
c  Anisotropic B: get principle components
               do 15, i = 1, nbfact(irun)
                  call bfrots(bfactr(1,i,irun), bfcprn(1,i), rotang(i))
                  bfmean(i) = 0.5*(bfcprn(1,i) + bfcprn(2,i))
 15            continue
            endif
c
            i1 = 1
 20         if (i1 .le. nbfact(irun)) then
               i2 = min(i1+ncx-1, nbfact(irun))
               if (mbatsc(irun) .gt. 0) then
                  write (lunout, '(/12x,18i7)')
     $                 (jbatrn(i,irun), i = i1,i2)
               else
                  write (lunout, '(/12x,18i7)') (i, i = i1,i2)
               endif
c Values of smoothed scale (for batch mode, no interpolation)
               if (mbatsc(irun) .le. 0) then
                  k = 0
                  do 22, i = i1,i2
                     k = k+1
                     time = i - 1
                     call sclder(
     $                    irun, stovl, time, xd, yd, zd,
     $                    ghl,  Thl, Chl,  Shl, dummy)
                     value(k) = alog(Thl)/(2.0*stovl)
 22               continue
c
                  write (lunout, 6020)
     $               ' B-smooth: ',(value(i), i = 1,k)
               endif
               if (nbfpos(irun) .eq. 1) then
                  write (lunout, 6020)
     $               ' B-factor: ',(bfactr(1,i,irun), i = i1,i2)
                  write (lunout, 6020)
     $               '       sd: ',(sdbfac(1,i,irun), i = i1,i2)
               else
c  Anisotropic
                  do 23, j = 1,nbfpos(irun)
                     write (lunout, 6021)
     $                  ' B-fac-',bfclab(j),': ',
     $                  (bfactr(j,i,irun), i = i1,i2)
                     write (lunout, 6021)
     $                  '    sd-',bfclab(j),': ',(
     $                  sdbfac(j,i,irun), i = i1,i2)
 6021                format(1x,a,a,a,18f7.1)
 23               continue
                  write (lunout, 6020)
     $               ' Bfac_min: ',(bfcprn(1,i), i = i1,i2)
                  write (lunout, 6020)
     $               ' Bfac_max: ',(bfcprn(2,i), i = i1,i2)
                  write (lunout, 6020)
     $               'Bfac_mean: ',(bfmean(i), i = i1,i2)
                  write (lunout, 6020)
     $               ' Bfac_rot: ',(rotang(i), i = i1,i2)
               endif
               write (lunout, 6020) ' Position: ',
     $              (posbfc(i,irun)*spcbfc(irun)+tlimrn(1,irun),
     $              i = i1,i2)
 6020          format(1x,a,18f7.1)
               i1 = i2 + 1
               go to 20
            endif
         endif
c     
         write (lunout, '(/a/)')
     $        ' Scales -- smoothed, individual, (sd):'
c     
         ncx = ncscls(3,irun)
c     How many sets across the page?
         ncchar = nclscl * ncx + nclpad
         nset = nclwid / ncchar
c
c  Interpolate if not batch mode or multiple scales on detector
         if (mbatsc(irun) .eq. 0 .or. ncscls(2,irun) .gt. 1) then
            lintrp = .true.
         else
            lintrp = .false.
         endif
c     
         time = 0.0
         lz = 1
         j1 = 1
c     
c  Loop groups of nset on zd
 120     if (lz .le. ncscls(1,irun)) then
            i1 = 1
            j2 = min(j1+nset-1, ncscls(1,irun))
c     
            line = ' '
            l1 = 3
            if (ncscls(4,irun) .eq. 1) l1 = 11
            do 125, j = j1, j2
               if (mbatsc(irun) .ne. 0) then
                  if (lbtslp(irun)) then
                     write (line(l1:), '(i7)') jbatrn((j+1)/2,irun)
                  else
                     write (line(l1:), '(i7)') jbatrn(j,irun)
                  endif
               else
                  write (line(l1:), '(i7)') j
               endif
               l1 = l1+ncchar
 125        continue
            l = lenstr(line)
            write (lunout, '(a)') line(1:l)
c     
c---- Loop yd
            do 130, iy = 1, ncscls(4,irun)
               yd = iy-1
               i2 = i1 + ncx - 1
               l1 = 3
               if (ncscls(4,irun) .eq. 1) l1 = 11
               line = ' '
               line2 = ' '
               line3 = ' '
c------  Loop zd
               do 135, j = j1, j2
                  zd = j-1
                  if (mbatsc(irun) .ne. 0) then
                     if (lbtslp(irun)) then
                        zd = btslpf*j
                     else 
                        zd = j
                     endif
                  endif
c-------- Loop xd
                  do 136, i = i1,i2
                     xd = mod(i-1, ncx)
                     k = i - i1 + 1
                     if (cscals(i,j,irun) .ne. 0.0) then
                        sclk(k)   = 1.0 / cscals(i,j,irun)
                        sdsclk(k) = sdcscl(i,j,irun) * sclk(k)**2
                     else
                        sclk(k)   = 0.0
                        sdsclk(k) = 0.0
                     endif
                     if (lintrp) then
c   Interpolate
                        call sclder(
     $                       irun, stovl, time, xd, yd, zd,
     $                       ghl,  Thl, Chl,  Shl, dummy)
                        value(k) = 1.0/Chl
                     endif
 136              continue
                  write (line(l1:), '(16f7.3)')
     $                 (sclk(i), i = 1,k)
                  l2 = l1+1
                  write (line2(l2:), '(16(a1,f5.3,a1))')
     $                 ('(',sdsclk(i),')', i = 1,k)
                  if (lintrp)
     $                 write (line3(l1:), '(16f7.3)')
     $                 (value(i), i = 1,k)
                  l1 = l1 + ncchar
 135           continue
               if (lintrp) then
                  l = lenstr(line3)
                  write (lunout, '(a)') line3(1:l)
               endif
               l = lenstr(line)
               write (lunout, '(a)') line(1:l)
               l = lenstr(line2)
               write (lunout, '(a/)') line2(1:l)
               i1 = i2+1
 130        continue
c     
            i1 = max(0, ncx*7 - 10 - 7)
            i2 = max(0, ncchar-7)
c     
            jj = 0
            do 140, j = j1, j2
               jj = jj+1
               if (mbatsc(irun) .ne. 0) then
                  if (lbtslp(irun)) then
                     jphi = (j+1)/2
                  else
                     jphi = j
                  endif
                  phi(jj) = 0.5 *
     $               (philim(1, jphi, irun) + philim(2, jphi, irun))
               else
                  phi(jj) =
     $                poscsr(j,irun)*spcscl(3,irun)+zlimrn(1,irun)
               endif
 140        continue
c            
            line = ' '
            write (line, '(a)')  ' Rotation'
            l = lenstr(line) + 1
            if (i1 .gt. 0) write (line(l:), '(a)') blank(1:i1)
            l = lenstr(line) + 1
            write (line(l:), 6130)
     $           (phi(jj), blank(1:i2), jj = 1, j2-j1+1)
 6130       format(1x,10(f7.2,a))
            l = lenstr(line)
            write (lunout, '(a//)') line(1:l)
c     
            lz = lz + nset
            j1 = j2 + 1
            go to 120
         endif
c
c Diffuse correction
         if (ltdsrn(irun)) then
c    Fraction alpha at maximum resolution (smax(irun))
            resmax = sqrt(1./smaxfr(irun))
            ir = itdspt(irun)
            alpha  =
     $         partds(2,ir) + partds(3,ir) * smaxfr(irun) * 0.25
            write (lunout, 6201)
     $         partds(1,ir), partds(2,ir), partds(3,ir),
     $         alpha, resmax
 6201       format (/' Diffuse scattering correction (TAILS)'/
     $         '               Half-width (A**-1): ', f12.3/
     $         '           Fraction at zero theta: ', f12.5/
     $         '                   Fraction slope: ', f12.5/
     $         '   Fraction at maximum resolution: ', f12.5/
     $         '     (resolution ',f8.3,'A )')
            if (ir .ne. irun) then
               write (lunout, '(a,i4)')
     $            ' Tails parameters linked to run ',ir
            endif
         endif
c
c SURFACE parameters, not for linked runs
         if (nrnsph(irun) .gt. 0 .and. isphpt(irun) .eq. irun) then
            if (lsmode(irun) .eq. 2) then
               line2 = 'SECONDARY beam'
            elseif (lsmode(irun) .eq. 3) then
               line2 = 'ABSORPTION'
            elseif (lsmode(irun) .eq. 4) then
               line2 = 'crystal (SURFACE)'
            endif

            write (lunout, 6301) line2(1:lenstr(line2)),
     $         (sphclm(i, irun), i = 1, nrnsph(irun))
 6301       format (' Spherical harmonic expansion coefficients in ',
     $         a,' frame',100(/5x, 10f7.3))
         endif
c     
 10   continue
c Plot Surface corrections
      if (nprsph .gt. 0) call pltsph

c     
      write (lunout, '(///)')
c     
c     %% sd factors now %%
      return
      end
      subroutine prtsly(istat)
c    ====================
c
c Print Scale LaYout
c
      implicit none
c
      integer istat
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'column.fh'
      include 'sphhrm.fh'
      include 'datasets.fh'
      include 'rfile.fh'

c
      character label*40, lpole(3)*1, dtslab*100
      integer irun, i
c
      integer lenstr
      external lenstr
c
      data lpole/'h','k','l'/
c
      istat = 0
c
      write (lunout, '(/a/a/)')
     $     ' Layout of scale factors',
     $     ' ======================='
c
      do 10, irun = 1, numrun
c  Make dataset label
         call mkdtslab(kdtsrn(irun), dtslab)
         write (lunout, '(/a,i4,2x,a,a/)') ' Run number ',irunum(irun),
     $      '  Dataset: ', dtslab(1:lenstr(dtslab))
c    
         if (irun .eq. irefrn) then
            write (lunout, '(a/)') ' Reference run'
         endif
c
         if (lbfrun(irun)) then
c B-factors
            if (nct .eq. 0) then
               label = 'rotation'
               if (negphi(irun)) then
                  label = 'rotation (negated to make increasing)'
               endif
            else
               label = 'time'
            endif
            if (lbtsmooth(irun)) then
               label = 'smoothed batch number'
            endif
c
            if (mbatsc(irun) .gt. 0) then
c Batch mode
               write (lunout, 6010) nbfact(irun)
 6010          format(
     $              ' Relative B-factor for each of ', i5, ' batches')
            else
               write (lunout, 6015) nbfact(irun), spcbfc(irun),
     $            label(1:lenstr(label))
 6015          format(' Relative B-factor: ',i5,
     $              ' factors at intervals of ',f8.2,' on ',a)
            endif
            if (nbfpos(irun) .gt. 1) then
               write (lunout, 6016)
 6016          format ('     Anisotropic B-factors')
            endif
         endif
c
         write (lunout, '(a)') ' Scales:'
         if (mbatsc(irun) .ne. 0) then
            if (lbtslp(irun)) then
               write (lunout, 6020) ncscls(1,irun)/2
               write (lunout, 6021)
 6021          format(5x,
     $      '  with different values at each end of range (SLOPE)')
            else
               write (lunout, 6020) ncscls(1,irun)
            endif
 6020       format(5x,'Along rotation axis: scales for each of',
     $           i5,' batches')
         else
            if (lbtsmooth(irun)) then
               write (lunout, 6024) ncscls(1,irun), spcscl(3,irun)
 6024          format(5x,'Along rotation axis: ',i5,
     $           ' scales at intervals of ',f8.2, ' batches')
            else
               write (lunout, 6025) ncscls(1,irun), spcscl(3,irun)
 6025          format(5x,'Along rotation axis: ',i5,
     $            ' scales at intervals of ',f8.2)
            endif
         endif
c
         if (ncscls(2,irun) .eq. 1) then
            write (lunout, '(5x,a)')
     $           'No variation of scale with detector coordinate'
         else
            write (lunout, 6035) (ncscls(i,irun),i=2,4),
     $           spcscl(1,irun), spcscl(2,irun)
 6035       format(5x,'On detector,',i4, ' scales: ',i3,' x ',i3,
     $           ' at spacings of ',f8.2,' x ',f8.2)
         endif
c
         if (nrnsph(irun) .gt. 0) then
            if (lodsph(irun) .eq. 1) then
               label = 'non-centrosymmetric'
            elseif (lodsph(irun) .eq. 2) then
               label = 'centrosymmetric'
            else
               label = 'WHAT?'
            endif
            if (lsmode(irun) .eq. 2) then
               write (lunout, '(a,a,i3,1x,a)')
     $            'Spherical harmonic correction on SECONDARY beam',
     $            ' direction in camera frame, l-order ', lmxsph(irun),
     $            label(1:lenstr(label))
            elseif (lsmode(irun) .eq. 3) then
               write (lunout, '(a,a,i3,/,a,a,a)')
     $            'Spherical harmonic ABSORPTION correction on ',
     $            'secondary beam direction in crystal frame, l-order ',
     $            lmxsph(irun),
     $            label(1:lenstr(label)),
     $            ', pole = ', lpole(mhpole(irun))
            elseif (lsmode(irun) .eq. 4) then
               write (lunout, '(a,a,i3,1x,a,a,a)')
     $            'Spherical harmonic SURFACEcorrection on scattering ',
     $            ' vector in crystal frame, l-order ', lmxsph(irun),
     $            label(1:lenstr(label)),
     $            ', pole = ', lpole(mhpole(irun))

            endif
         endif
c
         if (ltdsrn(irun)) then
            write (lunout, '(/a,a,5f10.5/)')
     $         ' Diffuse scattering correction (TAILS), ',
     $         'initial parameters: ',
     $         (partds(i,irun),i=1,nrntds(irun))
         endif
c
 10   continue
c
c Corner corretion
      if (lcorner) then
         call cnr_read_table(cornerfile, istat)
         if (istat .ne. 0) return
         call cnr_setup(1,
     $      nint(dlimrn(2,1,1)), nint(dlimrn(2,2,1)), istat)
         if (istat .ne. 0) return
      endif
c
      write (lunout, '(/a//)')
     $     ' ==========================================='
c
c
      return
      end
c
c
c
      subroutine prtsph
c    ===============
c
c  Print average surface correction values
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'inout.fh'
      include 'outcon.fh'
      include 'sphhrm.fh'
c
      integer nsmple, mtbphi
      parameter (nsmple = 2, mtbphi = (mxsphi+nsmple-1)/nsmple)
      real thet1(mxstheta), phi1(mxsphi), savg(mxstheta,mtbphi)
      integer navg(mxstheta,mtbphi)
c
      character  title*80, type*80
c
      integer i, j, irun, kip, n, ntbphi, it, ip
      real s, dtbphi
      logical lphi(mtbphi)
      character line*200
c
      integer lenstr
      external lenstr
c
c  All angles in degrees
c  Number of tabulated points on Phi, probably binning
c accumulated values
      ntbphi = nplphi/nsmple
      dtbphi = dplphi*float(nsmple)
c Set up all theta, phi values
      do j = 1, npltheta
         thet1(j) = float(j-1) * dpltheta
      enddo
      do i = 1, ntbphi
         phi1(i) = float(i-1) * dtbphi
      enddo
c
      do irun = 1, numrun
         if (lactvrun(irun) .and.
     $      nrnsph(irun) .gt. 0 .and. isphpt(irun) .eq. irun) then
            call ccpzr(savg, npltheta*ntbphi)
            call ccpzi(navg, npltheta*ntbphi)
c
            do kip =  1, ntbphi
               lphi(kip) = .false.
            enddo 
c
            do it=1,npltheta
c
               kip = 0
               do ip=1,ntbphi
                  s = 0.0
                  n = 0
                  do j = 1, nsmple
                     kip = kip+1
                     if (nsphith(it,kip,irun) .gt. 0) then
                        s = s + avphith(it,kip,irun)
                        n = n + nsphith(it,kip,irun)
                        avphith(it,kip,irun) =
     $                     avphith(it,kip,irun)/
     $                     float(nsphith(it,kip,irun))
                     else
                        avphith(it,kip,irun) = 0.0
                     endif 
                  enddo
                  if (n .gt. 0) then
                     navg(it,ip) = n
                     savg(it,ip) = s/float(n)
                     lphi(ip) = .true.
                  endif
               enddo
            enddo
c Print table for this run
           write (lunout, 6001)  irunum(irun), sphmnmx(1, irun),
     $         sphmnmx(2, irun) 
 6001      format (//' Spherical harmonic absorption correction',
     $        '(SECONDARY or ABSORPTION) for run ', i5/
     $        ' Minimum and maximum values: ', 2f10.3//
     $        'Average values by polar coordinates Phi & Theta'/)
c
           write (lunout, '(5x,a,20(f8.2))')
     $        '   Theta  ',(thet1(j), j = 1, npltheta)
           write (lunout, '(a)') '     Phi'
           do ip = 1, ntbphi
              if (lphi(ip)) then
                 write (line, 6010)
     $              phi1(ip), (savg(it, ip), it = 1, npltheta)
c  If you change this format statement, chenge the blanking below
 6010            format (f10.1,5x,11f8.3)
c Blank out null entries
                 do it = 1, npltheta
                    if (navg(it,ip) .eq. 0) then
c     column calculation depends on format statement above
                       j = (it-1)*8 + 16
                       line(j:j+7) = '        '
                    endif
                 enddo
                 write (lunout, '(a)') line(1:lenstr(line))
              endif
           enddo
         endif
      enddo
c
      return
      end
      subroutine prunmt
c     ==============
c
c  Print histogram of unmatched partials & number of parts, etc
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'inout.fh'
c
c
      integer i, k
c
      write (lunout, 6000)
 6000 format (//
     $   ' Analysis of total fraction for partials: ',/,
     $   '   1) rejected (unmatched) partials',/,
     $   '   2) accepted partials with consistent MPART flags',/,
     $   '   3) accepted partials with inconsistent MPART flags',//,
     $   '     Partiality      N1     N2     N3')
c
      write (lunout, 6001)
     $   (hprtbn(i), hprtbn(i+1),
     $   (nunmfr(k,i), k = 1,3), i = 1, nhstbin-1)
 6001 format(200(2f8.2, 3i7/))
c
      do k = maxrfl, 1, -1
         if (nhsprt(k) .gt. 0) go to 10
      enddo
      return
 10   write (lunout, 6010) (i, i = 1, k)
 6010 format (/
     $   ' Histogram of number of parts in accepted summed partials',/,
     $   1x,50i6)
      write (lunout, '(1x,50i6)') (nhsprt(i), i= 1,k)
c
      end
c
c
c
      subroutine prxyan
c     =================
c
c Print analyses on detector position
c
      include 'parameter.fh'
      include 'stats.fh'
      include 'outcon.fh'
      include 'inout.fh'
c

      integer i, j, k, i1, i2, i3, maxnum, jchi(30)
      real    pixel(maxxyn,2), chi(30)
      character label(maxxyn,2)*5

      maxnum = 22
c
      write (lunout, '(/a/a/)')
     $   ' In the following analyses, ',
     $  '    Chi = (Ihl - Mn(Iothers) )/sqrt[sd(Ihl)**2 + sd(Mn(I))**2]'
c
c Pixel coordinates
      do 1, j = 1,2
         do 2, i = 1, nbinxy(j)
            pixel(i,j) = float(i)*sbinxy(j) + rbinxy(j)
            if (sbinxy(j) .gt. 1.0) then
               k = pixel(i,j)
               write (label(i,j), '(i5)') k
            else
               write (label(i,j), '(f5.2)') pixel(i,j)
            endif
 2       continue
 1    continue
c
      write (lunout,'(//a//a/)')
     $   ' Chi**2 analysis by position of detector (Xdet, Ydet)',
     $   '    Each entry is 10 * Chi**2  (ie relative discrepancy **2)'
c
      i1 = 1
c
 120  if (i1 .le. nbinxy(1)) then
         i2 = min(nbinxy(1), i1 + maxnum - 1)
         i3 = min(i2, nbinxy(1)-1)
         write (lunout, 6120) (label(i,1), i = i1,i3)
 6120    format(/1x,'    Xdet',2x,22a5)
         write (lunout, '(1x,a)') ' Ydet'
         do 130, k = 1, nbinxy(2)
            j = 0
            do 135, i = i1, i2
               j = j+1
               if (n_m_frd_xy(i,k) .gt. 0) then
                  jchi(j) =
     $               nint(10.*sum_m_frdsq_xy(i,k)/n_m_frd_xy(i,k))
               else
                  jchi(j) = 0
               endif
 135        continue
            write (lunout, 6130) label(k,2), (jchi(i), i = 1,j)
 6130       format(1x,a5,1x,22i5)
 130     continue
         i1 = i2+1
         go to 120
      endif
c
      write (lunout,'(//a//a/)')
     $   ' Chi analysis by position of detector (Xdet, Ydet)',
     $'    Each entry is  10 * Mn(Chi) (ie signed relative discrepancy)'
c
      i1 = 1
c
 220  if (i1 .le. nbinxy(1)) then
         i2 = min(nbinxy(1), i1 + maxnum - 1)
         i3 = min(i2, nbinxy(1)-1)
         write (lunout, 6120) (label(i,1), i = i1,i3)
         write (lunout, '(1x,a)') ' Ydet'
         do 230, k = 1, nbinxy(2)
            j = 0
            do 235, i = i1, i2
               j = j+1
               if (n_m_frd_xy(i,k) .gt. 0) then
                  jchi(j) =
     $               nint(10.*sum_m_frd_xy(i,k)/n_m_frd_xy(i,k))
               else
                  jchi(j) = 0
               endif
 235        continue
            write (lunout, 6130) label(k,2), (jchi(i), i = 1,j)
 230     continue
         i1 = i2+1
         go to 220
      endif
c
      write (lunout,'(//a/)')
     $   ' Number of outlier rejections in each area'
c
      i1 = 1
c
 280  if (i1 .le. nbinxy(1)) then
         i2 = min(nbinxy(1), i1 + maxnum - 1)
         i3 = min(i2, nbinxy(1)-1)
         write (lunout, 6120) (label(i,1), i = i1,i3)
         write (lunout, '(1x,a)') ' Ydet'
         do k = 1, nbinxy(2)
            j = 0
            do i = i1, i2
               j = j+1
               jchi(j) = n_m_rej_xy(i,k)
            enddo 
            write (lunout, 6130) label(k,2), (jchi(i), i = 1,j)
         enddo
         i1 = i2+1
         go to 280
      endif
c
      write (lunout,'(//a/)')
     $   ' Number of observations in each area'
c
      i1 = 1
c
 320  if (i1 .le. nbinxy(1)) then
         i2 = min(nbinxy(1), i1 + maxnum - 1)
         i3 = min(i2, nbinxy(1)-1)
         write (lunout, 6120) (label(i,1), i = i1,i3)
         write (lunout, '(1x,a)') ' Ydet'
         do 330, k = 1, nbinxy(2)
            j = 0
            do 335, i = i1, i2
               j = j+1
               jchi(j) = min(n_m_frd_xy(i,k), 99999)
 335        continue
            write (lunout, 6130) label(k,2), (jchi(i), i = 1,j)
 330     continue
         i1 = i2+1
         go to 320
      endif
c
      return
      end

      subroutine qsortr(n,sarray)
c***********************************************************************
c
c*****sorts arrays sarray
c*****non-recursive version of quicksort algorithm (c.a.r. hoare)
c
c   n    - number of items to be sorted                          (given)
c sarray - real array to be sorted                             (updated)
c          array sarray is sorted to increasing values of sarray.
c
c Stolen from Wolfgang Kabsch
c
c***********************************************************************
      integer   istack(32,2),n,j,jl,jr,il,ir,k
      real      sarray(n),wk,wl,wr,skey
      if (n.lt.2)return
      j=1
      istack(1,1)=1
      istack(1,2)=n
c.....take top request from stack
10    jl=istack(j,1)
      jr=istack(j,2)
      j=j-1
c.....split jl...jr
20    il=jl
      ir=jr
      k=(jl+jr)/2
      wk=sarray(k)
30    wl=sarray(il)
      if (wl.ge.wk)go to 40
      il=il+1
      if (il.le.jr)go to 30
      il=jr
40    wr=sarray(ir)
      if (wk.ge.wr)go to 50
      ir=ir-1
      if (ir.ge.jl)go to 40
      ir=jl
50    if (il.gt.ir)go to 60
      skey=sarray(il)
      sarray(il)=sarray(ir)
      sarray(ir)=skey
      il=il+1
      ir=ir-1
      if (il.le.ir)go to 30
60    if ((ir-jl).ge.(jr-il))go to 80
c.....stack request for sorting right partition
      if (il.ge.jr)go to 70
      j=j+1
      if (j.gt.32)stop ' stack overflow in "qsort1"'
      istack(j,1)=il
      istack(j,2)=jr
c.....continue sorting left partition
70    jr=ir
      go to 100
c.....stack request for sorting left partition
80    if (jl.ge.ir)go to 90
      j=j+1
      if (j.gt.32)stop ' stack overflow in "qsort1"'
      istack(j,1)=jl
      istack(j,2)=ir
c.....continue sorting right partition
90    jl=il
100   if (jl.lt.jr)go to 20
      if (j.gt.0)go to 10
      return
      end
      subroutine rbvol(cel,vol)
c     =========================
c
c---- This subroutine works out unit CEL volume from CEL
c
c     .. Scalar Arguments ..
      real vol
c     ..
c     .. Array Arguments ..
      real cel(6)
c     ..
c     .. Local Scalars ..
      real alph,bet,conv,fct,gamm,sum,v
c     ..
c     .. Intrinsic Functions ..
      intrinsic sin,sqrt
c     ..
c
c
      conv = 3.14159/180.0
      fct = 8.0*3.14159*3.14159
      alph = cel(4)*conv
      bet = cel(5)*conv
      gamm = cel(6)*conv
      sum = (alph+bet+gamm)*0.5
      v = sqrt(sin(sum-alph)*sin(sum-bet)*sin(sum-gamm)*sin(sum))
      vol = cel(1)*2.0*cel(2)*cel(3)*v
c
c
      end
      subroutine rdresr(line,ibeg,iend,itok,itype,fvalue,cvalue,ntok,
     +                  resmin,resmax,smin,smax,irun,dname,istat)
c     =====================================================
c
c
c---- Read and decode resolution limits.
c     Subkeywords in CVALUE recognized:
c       LOW   read next number as low resolution limit
c       HIGH  read next number as high resolution limit
c       RUN   read next number as run number
c       DATASET read next field as dataset name
c
c
c     If LOW & HIGH are both present, the limits will still be swapped
c     to the correct order
c
c     On entry:
c
c     LINE,IBEG,IEND,ITYPE,FVALUE,NTOK,CVALUE contain information from Parser
c     ITOK  is first field to interpret
c
c     On exit:
c
c     RESMIN  Minimum resolution (in As) (ie low resolution)
c     RESMAX  Maximum resolution (in As) (ie high resolution)
c     SMIN    Minimum resolution ( 4sin**2/lambda**2) (ie low resolution)
c     SMAX    Maximum resolution ( 4sin**2/lambda**2) (ie high resolution)
c     IRUN    Run number, = 0 if unset, < 0 if dataset given
c     DNAME   Dataset name, if given
c     ISTAT   = 0, OK; = +1  no limits set
c             = -1 illegal subkeyword
c             = -2 illegal number (probably can't happen)
c
c     If only LOW or HIGH are given, the unset limit (ie either RESMAX, SMAX
c     or RESMIN, SMIN) will be set to -1.0. If only one number is given,
c     it is treated as a high resolution limit
c
      implicit none
c
      integer maxsbk
      parameter (maxsbk = 4)
c
c     .. Scalar Arguments ..
      real              resmax,resmin,smax,smin
      integer           itok,ntok,irun,istat
c     ..
c     .. Array Arguments ..
      real              fvalue(*)
      integer           itype(*), ibeg(*), iend(*)
      character*4       cvalue(*)
      character         line*(*), dname*(*)
c     ..
c     .. Local Scalars ..
      real              restem,stem
      integer           n, kmnmx, nset, lflag, nkeys, ikey, lsbkey
      logical           both, keywrd
      character*4 subkey(maxsbk)
c     ..
c     .. External Subroutines ..
      external          gttrea, ccpupc, chkkey
c     ..
c     .. Intrinsic Functions ..
      intrinsic         sqrt
c     ..
      data subkey /'LOW', 'HIGH', 'RUN', 'DATA'/
      data nkeys /maxsbk/
c
c---- Global defaults set here
c
      resmax = -1.0
      resmin = -1.0
      smin   = -1.0
      smax   = -1.0
      irun   = 0
      nset   = 0
      kmnmx  = 1
      istat  = 0
      both = .true.
      keywrd = .false.
      lsbkey = 0
      dname = ''
c
      n  = itok
c
 1    if (n .le. ntok) then
c
         if (itype(n) .eq. 1) then
c String
            call ccpupc(cvalue(n))
            call chkkey(cvalue(n),subkey,nkeys,ikey)
            if(ikey.le.0) then
c  Are we expecting a dataset name?
               if (lsbkey .eq. 3) then
                  dname = line(ibeg(n):iend(n))
                  irun = -1
               else
                  istat = -1
                  return
c                 ======
               endif
            elseif (ikey .eq. 1) then
c----- subkey LOW
               kmnmx = 1
               both = .not. both
               lsbkey = 1
               keywrd = .true.
            elseif (ikey .eq. 2) then
c----- subkey HIGH
               kmnmx = 2
               both = .not. both
               lsbkey = 2
               keywrd = .true.
            elseif (ikey .eq. 4) then
c----- subkey DATASET
               lsbkey = 3
            elseif (ikey .eq. 3) then
c----- subkey RUN
               lsbkey = 4
            endif
         else
c Number
            if (lsbkey .le. 3) then
c  Resolution key
               restem = 0.0
c                 ******************************************
               call gttrea(n,restem,lflag,ntok,itype,fvalue)
c                 ******************************************
               if (lflag .eq. 0) then
                  if (kmnmx .eq. 1) then
                     resmin = restem
                     nset   = nset+1
                     kmnmx  = 2
                  elseif (kmnmx .eq. 2) then
                     resmax = restem
                     nset = nset+1
                     kmnmx  = 1
                  endif
               else
                  istat = -2
               endif
            elseif (lsbkey .eq. 4) then
c   Run number
c                 ****************************************
               call gttint(n,irun,lflag,ntok,itype,fvalue)
c                 ****************************************
               if (lflag .ne. 0) then
                  istat = -2
               endif
               lsbkey = 0
            endif
         endif
         n = n+1
         go to 1
      endif
c
c  Have any numbers been set?
      if (nset .eq. 0) then
         istat = +1
         return
c        ======
      elseif (nset .eq. 1) then
c One only set, if no keywords have been defined, use single number as
c     high resolution limit
         if (both) then
            resmax = resmin
            resmin = -1.0
         endif
      endif
c
c---- option to read 4sin**2/lambda**2
      if (.not. keywrd .and. nset .eq. 2) then
         if (resmin .gt. 0.0 .and. resmin .le. 1.0 .and.
     $       resmax .gt. 0.0 .and. resmax .le. 1.0) then
c---- swap over SMIN and RESMIN
c
            smin = resmin
            resmin = sqrt(1.0/smin)
c---- swap over SMAX and RESMAX
            smax = resmax
            resmax = sqrt(1.0/smax)
         endif
      endif
c
      if (resmin .gt. 0.0) then
         smin = 1.0/resmin**2
      endif
c
      if (resmax .gt. 0.0) then
         smax = 1.0/resmax**2
      endif
c
c---- Check that they are in the correct order, if both limits read
c
      if (nset .eq. 2) then
         if (resmin.le.resmax) then
            restem = resmax
            resmax = resmin
            resmin = restem
         endif
         if (smin.gt.smax) then
            stem = smax
            smax = smin
            smin = stem
         endif
      endif
c
c
      return
      end
      subroutine refout(
     $   nrfgar, irjgar, ningar, iingar, nregar, aregar)
c     ================================================================
c     
c  Final pass(es) though data, merge & output stage
c     
c On entry:
c Global arrays with enough slots for each observation
c  nrfgar           number of observations in file
c  irjgar(nrfgar)   integer array (not set)
c  iirgar(nrfgar, ningar)   integer arrays (not set)
c  aregar(nrfgar, nregar)   real arrays  (not set)
c
      integer nrfgar, ningar, nregar
      integer irjgar(nrfgar), iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
c     
      include 'parameter.fh'
      include 'scales.fh'
      include 'sdfacc.fh'
      include 'inout.fh'
      include 'refcon.fh'
      include 'column.fh'
      include 'flags.fh'
      include 'params.fh'
      include 'rfile.fh'
      include 'rundef.fh'
      include 'outcon.fh'
      include 'anomtc.fh'
      include 'refflg.fh'
      include 'glocon.fh'
      include 'stats.fh'
      include 'symmty.fh'
      include 'errors.fh'
      include 'means.fh'
      include 'sphhrm.fh'
      include 'timecor.fh'
c-harvest-mod
      include 'sharvest.fh'
      include 'datasets.fh'
c     
c     
      integer jhkl(3), ihkl(3), irun, nrs, ninref, mbn_b, mbn_s
      integer i, j, k, l, m, n, istat, nuse, nrej, ibatch, nsrmax,
     $   mbn_t, nrejrs, noutrf, nunqrf, irfser, jfroff, iflprt,
     $   nrfrej, irogue, nstrej(maxflg), nrunp1, ibtanl, ip, irn, nmpr,
     $   nrfreja, nrfrjem, jfilout, noutfiles, iprint, ifail,
     $   ianmser, koutu
      integer ntuniq, ntotal, ntfull, ntsmpr, ntscpr, ntpart,
     $   krej2, ntonce, nfull, nrejpt(6), nppart,
     $   janom, nkept, mbn_i, nuall, nupls, numns, nrf, nrp, nfbias,
     $   icntrc, multcy, isysab, iselds, nused(2,4), igraph, igrapha,
     $   mbn_x, mbn_y, jbtpdc(maxbat), nf, nfpl, nfmn, nfif,
     $   nanmrj, nptmax, kwgtyp, icnflg, ispndle, iasym,
     $   jdataset, ipass, npass, nrejg, nrfldli, iwrpdi, minparts,
     $   mflcnt(2,maxmfl), krogplt, nfrclbin, maxmbi, nCC, nRCR
      real aih, aihpls, aihmns, uh, sdih, swih, varghl, delI,
     $   sdihpl,sdihms, reso(maxres), ss, stol, Thl, Chl, Shl, 
     $   timfac(maxrun), frccal, sumpart, sdsumpart, sdifull,
     $   sumg, aifull, aihl, sdihl, epsiln, swihpl, swihmn,
     $   sdidli, vardli, varipl, varims, varihm,
     $   slope(2,4,maxrun), sintct(2,4), dltlim, prblim,
     $   obslim(maxrun), aihf, aihfpl, aihfmn, DelIf, argar,
     $   sdmlfl, sdmlpt, smaxal, resmax,
     $   sumDI2o, sumDI2b, sumDI2p, sumDI2m, scatob, scatmn,
     $   danomdts(maxsets), sddanomdts(maxsets),
     $   disodts(maxsets), sddisodts(maxsets), fdts(maxsets),
     $   s0c(3), s2c(3), flmmax(3,2),  sclpkscl, delIoutlier, DelILimit,
     $   frclbin, bigpart, swihc, CCthrshld, RCRthrshld
c
      real
     $   qihdts(maxsets), qsdidts(maxsets),
     $   qihpl(maxsets), qsdipl(maxsets),
     $   qihmn(maxsets), qsdimn(maxsets)

      logical eof, lpart, lprint, lrejol, lunqdl, lanlbr(maxrun),
     $   lfull, lanlys, lptrog, lstats, wrtout, loutlchk,
     $   lrogfil, lccdts, lplnrm, lplanm, lplndi, legend, lresult
      character labfpt*80, pltitl*80, runlab*32,
     $   labdts*(maxnamelen), labxdts*(maxnamelen)
c     
c     
      real dummy(maxpmr), dummyr
      real datarf(maxcol,maxrfl), datavg(maxcolo), dataun(maxcolo)
c     
c     Reflection stores
      real ffrf(maxrfl), wtrf(maxrfl),
     $   sdffrf(maxrfl), timerf(maxrfl), frclrf(maxrfl),
     $   xdrf(maxrfl), ydrf(maxrfl), zdrf(maxrfl),
     $   ghl(maxrfl), dghldc(maxpmr, maxrfl), sdglrf(maxrfl),
     $   ffprrf(maxrfl), sdfprf(maxrfl), clprf(maxrfl),
     $   sclirf(maxrfl),  bgpkrf(maxrfl),
     $   sdscrf(maxrfl), delrf(maxrfl), xsirf(maxrfl),
     $   phirf(maxrfl), xdetrf(maxrfl), ydetrf(maxrfl),
     $   Dfhl(maxrfl), delfrf(maxrfl), eorf(maxrfl), sdffrf0(maxrfl)
      integer iserrf(maxrfl), irunrf(maxrfl), ibatrf(maxrfl),
     $   isttrf(maxrfl), mrf(maxrfl), isymrf(maxrfl), mprf(maxrfl),
     $   lflgrf(maxrfl), nmprrf(maxrfl), jmprrf(maxrfl),
     $   mbn_b_rf(maxrfl), mbn_t_rf(maxrfl), istrff(maxrfl),
     $   lanmtc(maxrfl), kmptrf(maxrfl)
      logical useflg(maxrfl), usefl2(maxrfl)
c     
      character label(2,maxrot*maxrun)*6, lab*6, 
     $   labelr(maxrun+1)*6, ibmode(3)*6,
     $   pname*(maxnamelen), xname*(maxnamelen), dname*(maxnamelen)
c
      integer ccp4_hash_lookup, lenstr
      external ccp4_hash_lookup, lenstr
c
      data ibmode/'LINEAR','SQUARE','WILSON'/
c
      nerpdc = 0
c Offset for scaled partial flag
      jfroff = 1000
c Variance weights
      kwgtyp = 2
c
      igrapha = 0
      igraph = 0
c     
      write (lunout, 6001)
 6001 format (//3(1x,20('====')/)/
     $   ' ------ Analysis Pass ------'//
     $   3(1x,20('====')/)/)
c
      NHallobs = nrfgar
c
      if (irefrn .gt. 0) then
         if (lrefou) then
            write (lunout, '(//a,a/)')
     $         ' ==== Reference run will be included in ',
     $         'statistics and output file ===='
         else
            write (lunout, '(//a,a/)')
     $         ' ==== Reference run will be omitted from ',
     $         'statistics and output file ===='
         endif
      endif
c
      if (kprout .eq. -1) then
         write (lunout, '(/a/)')
     $      ' Only fully-recorded reflections will be used in analysis'
      elseif (kprout .eq. 0) then
         write (lunout, '(/a,a/)')
     $      ' Only fully-recorded reflections ',
     $      '(and previously summed partials) will be used in analysis'
      elseif (kprout .eq. 1) then
         write (lunout, '(/a/)')
     $      ' Fully-recorded and summed partial reflections'//
     $      ' will be used in analysis'
         if (kprflo .lt. 0) then
            write (lunout, '(4x,a/4x,a)')
     $    ' Summed partials will be checked for consistent MPART flags',
     $         '    sets with consistent MPART flags will be accepted'
            if (iabs(kprflo) .eq. 2) then
               write (lunout, '(4x,a,a)')
     $   '    sets with inconsistent MPART flags will be tested on the',
     $            ' total fraction'
            else
               write (lunout, '(4x,a)')
     $         '    sets with inconsistent MPART flags will be rejected'
            endif
         elseif (ncmprt .gt. 0) then
            write (lunout, '(4x,a)')
     $ ' Summed partials will NOT be checked for consistent MPART flags'
         endif
         if (mxprwo .lt. 999) write (lunout, '(4x,a,i4,a)')
     $    ' Summed partials must contain not more than',mxprwo,' parts'
         if (iabs(kprflo) .eq. 1) then
            write (lunout, '(4x,a)')
     $       ' No test on total calculated fraction for summed partials'
         elseif (iabs(kprflo) .eq. 2) then
            if ((prlimo-prthou) .gt. 0.0001 ) then
               write (lunout,
     $            '(4x,a,a,f8.3,a,f8.3/4x,a,f8.3,a,f8.3)')
     $            ' Summed partials accepted if total fraction',
     $            ' lies between',prlimo,' and',prmaxo,
     $            ' scaled by total fraction if between', prthou,
     $            ' and', prlimo
               if (lsclpr) then
                  write (lunout, '(/a/)')
     $   ' Profile-fitted intensities will be used for scaled partials'
               endif
            else
               write (lunout, '(4x,a,a,f8.3,a,f8.3)')
     $            ' Summed partials accepted if total fraction',
     $            ' lies between', prlimo,' and', prmaxo
            endif
         endif
         if (kprmso .ne. 0) then
            write (lunout, '(4x,a,a)')
     $         ' Partials with missing parts in the middle will',
     $         ' be accepted'
         else
            write (lunout, '(4x,a,a)')
     $         ' Partials with missing parts in the middle will',
     $         ' be rejected'
         endif
      elseif (kprout .eq. 2) then
         write (lunout, '(/a,f10.4/)')
     $      ' Fully-recorded and scaled partial reflections'//
     $      ' will be used in analysis, with a minimum fraction = ',
     $      prthou
         if (lsclpr) then
            write (lunout, '(/a/)')
     $   ' Profile-fitted intensities will be used for scaled partials'
         endif
      endif
c
      if (lolexc) then
         write (lunout, '(/a)')
     $      ' Outliers will be omitted from output file'
      endif
c
      if (nprexc .lt. 0) then
         write (lunout, '(/a)')
     $      ' Partials will be omitted from output file'
      elseif (nprexc .gt. 0) then
         write (lunout, '(/a,25i3)')
     $      ' Partials will be omitted from the output file for runs',
     $      (irunum(kprexc(i)), i = 1,nprexc)
      endif
      if (lrdidx) then
         write (lunout, '(/a,a)')
     $      ' Reduced indices hkl will be output ',
     $      '(ie reduced to asymmetric unit as in input file)'
      else
         write (lunout, '(/a,a)')
     $      ' Original indices hkl will be output ',
     $      '(ie NOT reduced to asymmetric unit as in input file)'
      endif
c
c
c Reject outliers from statistics
      lrejol = .true.
c Options for badly agreement with 2 observations
      krej2 = krej2o
      if (krej2 .eq. 0) then
         write (lunout, '(/a/)')
     $      ' Outliers with two observations will be kept'
      elseif (krej2 .eq. -1) then
         write (lunout, '(/a/)')
     $      ' Outliers with two observations will be rejected'
      elseif (krej2 .eq. +1) then
         write (lunout, '(/a/)')
     $      ' For outliers with two observations reject the larger'
      elseif (krej2 .eq. +2) then
         write (lunout, '(/a/)')
     $      ' For outliers with two observations reject the smaller'
      endif
      if (kprint(1) .gt. 2) then
         lprint = .true.
         write (lunout, '(/a/)')
     $      ' Outliers:     DelI/sd       I       sd(I)     Run'
      else
         lprint = .false.
      endif
c     
      nsrmax = 0
      nptmax = 0
      nrejpt(1) = 0
      nrejpt(2) = 0
      nrejpt(3) = 0
      nrejpt(4) = 0
      nrejpt(5) = 0
      nrejpt(6) = 0
c
      scatob = 0.
      scatmn = 0.

      sclimax = 0.0

      call clrsts

c Outlier limit in multiples of RMS for unitary correlation analysis
c within datasets
      delIoutlier = 5.0
c
c----------------------------------------------------------------
c Go through all reflections working out various means, tabulating
c weighted deviations, & setting flags
      call rfltbl(nrfgar, ningar, iingar, nregar, aregar)
c----------------------------------------------------------------
c  Set up intensity bins
      if (binmid .lt. 0.0) then
c    Middle bin intensity not set, try to set it
         if (avgins .gt. 0.0) then
c    (a) use average intensity from initial or scaling pass, if available
            binmid = avgins
         else
c    (b) estimate from maximum intensity in file
            binmid = 0.02 * flmxin
         endif
      endif

      write (lunout, 6211)
 6211 format (/3(1x,20('====')/)/
     $   ' ------ Final Pass for statistics of merging ------'/
     $   3(1x,20('====')/)/)
c
      write (lunout, '(a,a,a,f10.0)')
     $   'Intensity bins, ',ibmode(kbnrtp),
     $   ' mode, mid-bin intensity = ', binmid
c Set intensity bin limits, top limit = file maximum
      call setbrn(kbnrtp, ninbin, binmid, ninbin/2, binrng)
      binrng(ninbin) = flmxin
      maxmbi = ninbin
c
c=== Open ROGUES file if omitting outliers
      irogue = 15
      if (lwrrog) call strogf(irogue)
c=== Open Normal Probability files if required
      lplanm = .false.
      lplnrm = .false.
      lplndi = .false.
      if (lenstr(outitl) .eq. 0) then
         pltitl = rtitle
      else
         pltitl = outitl
      endif
      if (lnormn) then
c   Probability limit 0.9, c. 63% of data
         dltlim = 0.9
cc   Probability limit 0.674, c. 50% of data
c         dltlim = 0.674
c  Limit to just exclude real outliers (98% of data)
         prblim = 2.5
c
c^         if (kanomo .ne. 0) then
            if (ndtsetsout .gt. 1) then 
               legend = .true.
            else 
               legend = .false.
            endif
c  Open file for anomalous plot
            call stwrpr(iwrpra, 'ANOMPLOT', legend, 
     $         'Anomalous differences', pltitl)
            lplanm = .true. 
            igrapha = 0
c^         endif
         call stwrpr(iwrprb, 'NORMPLOT', .true., ' ', pltitl)
         lplnrm = .true.
         igraph = 0
      endif
c Number of bins for bias analysis (range from 0.5 to 1.0)
      nfrclbin = maxfrc
      frclbin = 0.5/float(nfrclbin)
c
c Scale factor for Scalepack-style output to keep it within f8.1
      sclpkscl = 1.0
      if (isclpk .gt. 0) then
         sclpkscl = 999900.0
         if (sclimax .gt. sclpkscl) then
            sclpkscl = sclpkscl/sclimax
         else
            sclpkscl = 1.0
         endif 
      endif
c
      
      krogplt = 0
      if (wvlnmn .lt. wvlnmx+0.1 .and. nmatxs .gt. 0) then
c  ROGUEPLOT only if we have orientation data & wavelengths
         krogplt = 12
         call stwrrp(krogplt, pltitl, smaxlm)
      endif

c*c*c
c      if (lbeams) then
c         call ccpdpn(13, 'BEAMS', 'NEW', 'F', 0,0)
c      endif
c*c*c
c
c********************************************************************
c
      jfilout = 0
      ipass = 0
c 
c Flag for what to do about outliers, for multiple datasets
c loutlchk
c  =  .false.  check for outliers and set global outlier array irjgar
c              ie. irjgar array not set or not used
c  =  .true.   use previously-generated global outlier array irjgar
c irjgar is a global outlier array for each observation
c  =  0    not rejected, keep this observation (also initial setting)
c  .ne. 0  rejected
      loutlchk = .false.
c Clear outlier flags
      call ccpzi(irjgar, nrfgar)
c
c Set number of output files noutfiles &
c number of passes through data npass
      if (kfilou .ge. +1) then
c OUTPUT SPLIT, new file for each dataset (usually one only)
         noutfiles = kfilou
         npass = noutfiles
c    For multiple datasets, first pass looks at all datasets
c to set outlier rejections, so we need an  extra pass
         if (ndtsetsout .gt. 1) then
            npass = npass+1
         endif
      elseif (kfilou .lt. 0) then
c OUTPUT TOGETHER, multiple datasets in one file (ndtsetsout .gt. 1)
         noutfiles = 1
         npass = ndtsetsout + 1
      elseif (kfilou .eq. 0) then
c One dataset in & out, simple case
         npass = 1
         noutfiles = 1
      endif
c Resolution limits set overall for all datasets
c (all runs except reference)
      smaxal = smaxlm
      resmax = 1.0 / sqrt(smaxal)
      do 130, k = 1, nsrang
         reso(k) = max(resmax, 1.0 / sqrt(float(k) * srange))
 130  continue
c
c****************************************************************
c Loop for multiple datasets
c do while ipass .le. npass  (loop return at end of routine)
 10   ipass = ipass + 1
      lstats = .true.
      if (kfilou .eq. 0) then
c One data setin & out, simple case
         wrtout = .true.
         jfilout = jfilout + 1
         jdataset = 0
      elseif (kfilou .lt. 0) then
c OUTPUT TOGETHER, multiple datasets in one file
c   First pass for all datasets, write output file, no statistics
         if (ipass .eq. 1) then
            wrtout = .true.
            lstats = .false.
            jfilout = jfilout + 1
            jdataset = 0
         else
c   Last  ndtsetsout passes, statistics only, no output file
            wrtout = .false.
            lstats = .true.
            jdataset = ipass - 1            
c    Outliers already flagged
            loutlchk = .true.
            lrejol = .false.
         endif
      elseif (kfilou .ge. +1) then
c OUTPUT SPLIT, new file for each dataset (usually one only)
         if (ndtsetsout .gt. 1 .and. ipass .eq. 1) then
c    First outlier pass only
            wrtout = .false.
            lstats = .false.
            jdataset = 0
         else
c    This pass writing data
            wrtout = .true.
            lstats = .true.
            jfilout = jfilout + 1
            jdataset = jfilout
c    Outliers already flagged
            loutlchk = .true.
            lrejol = .false.
         endif
      endif
c
c lccdts .true. to generate correlation coefficients between datasets
      if (ndtsetsout .gt. 1 .and. .not.loutlchk) then
         lccdts = .true.
         call dtsclrcc
         call dtsinitstore
      else
         lccdts = .false. 
      endif
c
c Write rogues files only when actually checking outliers
      lrogfil = lwrrog .and. .not.loutlchk
c
c Set current P/Dname
      if (kfilou .eq. 0) then
         if (ndtsetsout .eq. 1) then
            pname = pname_out(1)
            xname = xname_out(1)
            dname = dname_out(1)
            labdts = ', '//dname
         else
            pname = ' '
            xname = ' '
            dname = ' '
            labdts = ' '
         endif
      else
         if (jdataset .gt. 0) then
            pname = pname_out(jdataset)
            xname = xname_out(jdataset)
            dname = dname_out(jdataset)
            labdts = ', '//dname
         else
            pname = ' '
            xname = ' '
            dname = ' '
            labdts = ' '
         endif
      endif
      
      write (lunout, 6333)
 6333 format (/2(1x,20('****')/))
c
      if (lstats) then
 9       if (jdataset .eq. 0) then
            write (lunout,'(a/)')
     $         '   Statistics for all datasets'
         else
            call mkdtslab(jdataset, labxdts)
            write (lunout,'(a,i4,6x,a/)')
     $         '   Statistics for dataset  ', jdataset,
     $         labxdts(1:lenstr(labxdts))
         endif
      endif
c Set flags for which runs are active
c This means that only observations from active runs will be used in
c any subsequent calculations
      call setacr(jdataset)
c
      if (wrtout) then
         if (ifilou .gt. 0) then
c     Open output file
            write (lunout, '(//)')
            call opnopf(jfilout, jdataset)
         else
            write (lunout, '(//a/)')
     $         ' ==== No output HKLOUT file written ===='
            wrtout = .false.
         endif
c
      endif
c..     
c---- Set up analysis bins on batch or rotation range
c
c  lanlbt  .true.  if analysis is done by Batch
c          .false. if analysis is done by rotation range
c                       (only if not Batch mode)
c  lanlbr(irun) .true. for batch analysis for this run
c
c     i1ovlp is a pointer to the 1st entry for
c     each run. nmtrng is the number of rotation bins in all runs
c      = -1 for inactive runs
c     No bins in reference run
      nmtrng = 1
      do 1, irun = 1, numrun
         if (irun .ne. irefrn) then
            if (lactvrun(irun)) then
               i1ovlp(irun) = nmtrng
               if (lanlbt .or. mbatsc(irun) .ne. 0) then
                  nrrang(irun) = nbatrn(irun)
                  lanlbr(irun) = .true.
               else
                  nrrang(irun) = max(1, ncscls(1,irun) - 1)
                  lanlbr(irun) = .false.
               endif
               nmtrng = nmtrng + nrrang(irun)
               timfac(irun) = float(nbfact(irun)) /
     $            float(ncscls(1,irun))
            else
               nrrang(irun) = 0
               timfac(irun) = 1.0
            endif
         else
            i1ovlp(irun) = -1
         endif
 1    continue
      nmtrng = nmtrng-1
c..
c
c  Clear all sums for statistics
      call clrsts
      if (lstats) then
         j = jdataset
         if (ndtsetsout .le. 1) j = -1
         call wdsclrcc(delIoutlier, j, 'DelAnom scatter plot', pltitl)
      endif
c--- Clear flags
      call ccpzi(iingar, nrfgar*2)
c     
      do 2, irun = 1, numrun
         sminfr(irun) = 10000.
         smaxfr(irun) = 0.0
 2    continue
      nrs = 0
      nuse = 0
      nfull = 0
      ninref = 0
      noutrf = 0
      nunqrf = 0
      nrejrs = 0
      nrfrej = 0
      nrfreja= 0
      nrfrjem = 0
      ntuniq = 0
      ntotal = 0
      ntfull = 0
      ntsmpr = 0
      ntscpr = 0
      ntonce = 0
      nanmrj = 0
      nrfldli = 0
      ianmser = 0
      do 18, i = 1,maxflg
         nstrej(i) = 0
 18   continue
      aihmin = +1.0e10
      aihmax = -1.0e10
      call ccpzi(mflcnt, 2*maxmfl)
      call ccpzr(flmmax, 3*2)
      call clr_tcr
c Analysis on detector
      mbn_x = 1
      mbn_y = 1
      if (.not. lcddet) then
         lxyanl = .false.
      elseif (lxyanl) then
c Set up xy bins
         call setsxy(istat)
         if (istat .ne. 0) lxyanl = .false.
      endif
c   Clear corner correction counts if file is written
      if (lstats .and. lcorner) call cnr_clear_histo
c
      call lrrewd(1)
      koutu = koutstr1
      if (koutstr2 .ne. 0) then
         koutu = koutstr2
      endif

c  Tell gethkl that we're starting to read the file
      eof = .true.
c     
cooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c   Get all measurements of this reflection: nuse is the number of useful
c ones (fulls and partials bits)
 20   call gethkl(maxrfl, ninref, lrefou, +1, .false.,
     $   eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $   ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $   isymrf, sclirf, sdscrf, lflgrf, bgpkrf, frclrf, clprf, isttrf,
     $   timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $   nmprrf, datarf, istat)
c Optionally exclude reflections outside resolution limits from output file
      if (krsexc .eq. 0) then
         if (istat .eq. +1) then
            nrejrs = nrejrs + 1
            go to 20
         endif
      endif
c    mbn_s is resolution bin number
      mbn_s = min(int((ss-0.0000001)/srange) + 1, nsrang)
      nsrmax = max(nsrmax, mbn_s)
c no need to check eof here
      if (nrs .gt. 0) then
c...................................................................
c     --- --- process all measurements of this reflection
c     
c  Count FLAGs, and accept if required
         if (ncflag .gt. 0)
     $      call tstmfl(nrs, lflgrf, bgpkrf, isttrf, mrf,
     $      mflcnt, flmmax)
         nunqrf = nunqrf+1
c  Centric if icntrc = 1, else = 0
         call centr(jhkl, icntrc)
c  Multiplicity: isysab = 1 for systematic absences
         call epslon(jhkl,epsiln,isysab)
c Multiplicity is number of times this reflection will occur in a
c     complete sphere of data. This is Nsym/Epsilon, multiplied by 2
c     for acentric reflections (since Nsym symmetry operations generate
c     a hemisphere of acentric data, but all centric reflections)
c NOTE this is not the same as the multiplicity used for weighting
c     <I> in Wilson plot calculations (see Iwasaki & Ito, Acta Cryst,
c     A33,227-229(1977))
         multcy = nint(float(nsym)/epsiln)
         if (icntrc .eq. 0) multcy = multcy*2
c     
c     Get scale for all observations, & their derivatives with respect to
c     the scale factors
         do 25, l=1,nrs
            call sclder(irunrf(l),
     $         stol, timerf(l), xdrf(l), ydrf(l), zdrf(l),
     $         ghl(l), Thl, Chl,  Shl, dghldc(1,l))
            Dfhl(l) = 1.0
            eorf(l) = 0.0
 25      continue
c
c     Process partials if required (using selected I), update nuse
c     After this call, nuse includes fulls & summed|scaled partials
c     (ie a summed partials counts as 1)
         call partls(.false.,
     $      jhkl, kprout, lpart, stol, nrs, nprtds, ltdsrn,
     $      ffrf, sdffrf, ffprrf, sdfprf,
     $      ghl, Dfhl, xdrf, ydrf, zdrf, phirf, xsirf,
     $      frclrf, kmptrf, isttrf, mrf, mprf, isymrf, ibatrf, irunrf, 
     $      nmprrf, jmprrf, kprflo, kprmso, mxprwo, imid,
     $      nrejpt, nuse, prthou, prlimo, prmaxo, dghldc, .false.,
     $      lavpsc)
         if (kprout .eq. 0) then
            nuse = nfull
         endif
c
c     Correct sds for selected I
c This is done after summing partials, but before outlier rejection
c Save original SDs
         do l = 1,nrs
            sdffrf0(l) = sdffrf(l)
         enddo
         call sdcorr
     $      (nrs, ffrf, sdffrf, clprf, ghl, isttrf, isymrf, irunrf, mrf,
     $      swihc)
c Set variance weights
         call setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c
c ***
c Extrapolate to zero dose if requested
         if (ktimcor .ne. 0) then
c  We don't need to select datasets (runs) here because inactive
c runs have already been removed by chkref/runget, so iselds = 0
            iselds = 0
            if (kanomo .ne. 0) then
c  Extrapolate I+ & I- seperately (anom on)
               janom = 1
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, .false., useflg)
               call timcor
     $            (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
c  
               janom = 2
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, .false., useflg)
               call timcor
     $            (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
            else
               janom = 0
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, .false., useflg)
               call timcor
     $            (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
            endif
         endif
c ***
c
c Anomalous match option: flag unmatched observations as rejected
         call selanm(icntrc, nrs, kanomo, jhkl, irunrf, mrf, isttrf,
     $      isymrf, phirf, timerf, lanmtc, nanmrj)
c
c Check Emax limits if set
         lptrog = .false.
         if (lqmnir) then
            call chemax(nrs, .true., 
     $         icntrc, epsiln,
     $         jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $         isymrf, ibatrf, mbn_s,
     $         eorf, nrej, istat)
            if (istat .ne. 0) then
               write (lunout, '(a,3i4,a)')
     $            ' WARNING: Emax test failed on reflection ',
     $            jhkl,'    see ROGUES file'
               lptrog = .true.
            endif
            if (nrej .gt. 0) then
               nrfrjem = nrfrjem + nrej
               lptrog = .true.
            endif 
         endif
c If outliers have already been flagged into the global irjgar array,
c copy outlier flags to current reflection arrays
         if (loutlchk) then
            call rejget(nrs, isttrf, iserrf, irjgar, nrfgar, nrejg)
            if (nrejg .ne. 0) then
               nrfrej = nrfrej + nrejg
            endif
         endif
c     
c Get discrepancies for fulls only (delfrf)
         call fuldis(nrs, kanomo, lanbyd, icntrc, jhkl,
     $      ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf, isymrf,
     $      lanmtc, sdrejm, sdrejm2, sdrejma, sdrejm2a, krej2,
     $      istrff, delfrf, aihf, aihfpl, aihfmn, nf, nfpl, nfmn)
c---
c Outlier rejection & discrepancies for all data
         call choutl(nrs, kanomo, lprint, lrejol, lanbyd, icntrc,
     $      jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $      isymrf, lanmtc, nuse, nrej,
     $      sdrejm, sdrejm2, sdrejma, sdrejm2a, krej2, delrf,
     $      swih, swihpl, swihmn)
         if (nrej .ne. 0) then
            nrfrej = nrfrej + nrej
            lptrog = .true.
         endif
         if (lptrog .and. lrogfil) then
            call prtrog(irogue, nrs, jhkl, isymrf, ffrf, sdffrf,
     $         ibatrf, frclrf, isttrf, mrf, ghl, clprf, delrf, ss,
     $       swih, swihpl, swihmn, xdetrf, ydetrf, phirf, sclirf, eorf)
         endif
c Plot rejects into file
         if (krogplt .gt. 0) then
            call pltrog(krogplt, nrs, jhkl, isttrf, isymrf, irunrf,
     $         ibatrf, xdetrf, ydetrf, phirf)
         endif 
c
c  Store discrepancy & flag for normal analysis    
c  Pack run number, partial status (full, summed|scaled partial),
c centric flag into iingar(,1) array
         do 29, l = 1,nrs
            irfser = iserrf(l)
            aregar(irfser,1) = delrf(l)
c  Discrepancy for fulls only
            aregar(irfser,3) = delfrf(l)
            call pckflg(
     $         irunrf(l), mrf(l), nmprrf(l), icntrc, iingar(irfser,1))
            if (.not. loutlchk) then
c Set global outlier reject flag if not already set
               if (isttrf(l) .le. lqoutl) then
                  irjgar(irfser) = isttrf(l)
               endif
            endif
 29      continue
c     
         nrf = 0
         nrp = 0
         do 30, l=1,nrs
            if (isttrf(l) .eq. lqreso) then
               if (krsexc .le. 1) then
c  Reflection rejected on resolution limits, set scale = 0.0 as reject flag
                  ghl(l) = 0.0
                  sdglrf(l) = 0.0
                  nrejrs = nrejrs + 1
                  go to 30
               endif
            endif
            if (irunrf(l) .gt. 0) then
               if (lsdpars) then
c     Get sd(ghl)
c     (diagonal approximation, full variance/covariance matrix is too
c     slow)
                  varghl = 0.0
                  irun = irunrf(l)
                  do 35, k = 1, nprrun(irun)
                     i = ipoint(k,irun)
                     varghl = varghl + (dghldc(k,l)*sdpars(i))**2
 35               continue
                  sdglrf(l) = sqrt(varghl)
               else
                  sdglrf(l) = 0.0
               endif
            else
               ghl(l) = 0.0
               sdglrf(l) = 0.0
            endif
c Count fulls & summed or scaled partials
            if (isttrf(l) .ge. lqOK) then
               if (mrf(l) .eq. 0) then
                  nrf = nrf+1
               else
                  nrp = nrp+1
               endif
            endif
 30      continue
c     
c     Calculate the mean intensity aih, weights wtrf,
c     uh = Sum(w * ghl**2)
c Exclude reference run from means
c  aih, sdih        Imean, sd(Imean)  all observations
c  aihpls, sdihpl   I+ , sd(I+)    }   if
c  aihmns, sdihms   I- , sd(I-)    }  kanomo .ge. 1
         nupls = 0
         numns = 0
c  Means of I+ & I- (even if anomalous off)
         janom = 1
c  We don't need to select datasets (runs) here because inactive
c runs have already been removed by chkref/runget, so iselds = 0
         iselds = 0
         call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $      irefrn, iselds, .false., useflg)
         call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $      wtrf, aihpls, sdihpl, uh, nupls)
         janom = 2
         call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $      irefrn, iselds, .false., useflg)
         call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $      wtrf, aihmns, sdihms, uh, numns)
         janom = 0
         call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $      irefrn, iselds, .false., useflg)
         call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $      wtrf, aih, sdih, uh, nuall)
c
c  Divide data randomly into two sets (if enough) for correlation
c analysis within dataset
c  Always analyse anomalous differences
         if (lstats) then
            call wdsanl(icntrc, nrs,  mbn_s, ffrf, sdffrf, isymrf, 
     $         ghl, useflg, nupls, numns)
         endif
         if (lccdts) then
c Correlating multiple datasets
c  Calculate anomalous differences and isomorphous|dispersive differences
c  for each dataset
c  Returns means for each datasets (qihdts etc)
c
            call dtsdif(nrs, icntrc, jhkl, ffrf, sdffrf, wtrf, ghl,
     $         irunrf, mrf, isttrf, isymrf, ndtsetsout,
     $         danomdts, sddanomdts, disodts, sddisodts, fdts,
     $         qihdts, qsdidts, qihpl, qsdipl, qihmn, qsdimn)
c  Add into sums for correlation
            call dtsadcc(ndtsetsout, mbn_s,
     $         danomdts, sddanomdts, disodts, sddisodts)
c  Store away for later determination of scale factors between 
c  anomalous differences and isomorphous|dispersive differences
            call dtsstore(icntrc, jhkl,  ndtsetsout,
     $         danomdts, sddanomdts, disodts, sddisodts, fdts)
c TEST   Store DelI/F for later analysis (but why?)
c            if (icntrc .eq. 0) then 
c               do l = 1, ndtsetsout
c                  if (fdts(l) .gt. 0.000001) then
c                     nrfldli = nrfldli + 1
c                     aregar(nrfldli,4) = danomdts(l) / fdts(l)
c                     iingar(nrfldli,2) = l
c                  endif
c               enddo
c            endif
         endif
c  useflg marks accepted reflections for Imean, excluding reference run
c
c "Partial" bias analysis:
c   extended to cope with the case of no fulls
c   Use sqrt-scale-weighted mean of observations with the minimum
c    number of parts
         nfbias = 0
         minparts = 1000
         n = 0
         m = 0
         aifull = 0.0
c
         do l = 1,nrs
            if (useflg(l) .and. nmprrf(l) .gt. 0) then
               minparts = min(minparts, nmprrf(l))
            endif
         enddo
         do l = 1,nrs
            if (useflg(l)) then
c Select only those valid observations with minparts parts
               if (nmprrf(l) .eq. minparts) then
                  usefl2(l) = .true.
                  m = m+1
               else
c count other valid observations
                  usefl2(l) = .false.
                  n = n+1
               endif 
            else
               usefl2(l) = .false.
            endif
         enddo
         if (n*m .gt. 0) then
c Set sqrt-scale weights
            call setwgt(3, nrs, ffrf, sdffrf, ghl, wtrf)
            call menint(nrs, ffrf, sdffrf, ghl, usefl2,
     $         wtrf, aifull, sdifull, uh, nfbias)
         endif
c     
c======================================================================
c  Accumulate statistics
c    analysis bins:-
c   1) by resolution[, by run]        mbn_s, irun
c   2) by intensity                   mbn_i
c   3) by batch or rotation range     mbn_b
c
         if (lstats) then
c Intensity range
            do 46, i = 1, ninbin
               if (aih .lt. binrng(i)) then
                  mbn_i = i
                  go to 47
               endif
 46         continue 
            mbn_i = ninbin
 47         continue
c----
c-- Anomalous differences (Ranom)
            if (icntrc .eq. 0) then
               varipl = sdihpl**2
               varims = sdihms**2
               if (nupls .gt. 0 .and. numns .gt. 0) then
c Store DelAnom/sd for normal analysis & increment count
                  ianmser = ianmser + 1
                  aregar(ianmser,2) =
     $               (aihpls - aihmns)/sqrt(varipl + varims)
                  if (kanomo .ne. 0) then
                     call ad4sts(aihpls, aihmns, mbn_s, mbn_i)
                  endif
c   Counts for anomalous multiplicity
c   Contributions to measured totals
c      (a) number of pairs = min(nupls, mumns)
c      (b) additional contribution for excess measurements
c          of I+ or I-, always .lt. 1
                  n_m_uniq_s(mbn_s) = n_m_uniq_s(mbn_s) + 1
                  sn_m_meas_s(mbn_s) =
     $               sn_m_meas_s(mbn_s) + dble(min(nupls, numns))
                  i = abs(nupls - numns)
                  if (i .gt. 0) sn_m_meas_s(mbn_s) =
     $               sn_m_meas_s(mbn_s) + dble(i)/dble(i+1)
               endif
c  Counts for anomalous completeness
               call anomcn(icntrc, nupls, numns, multcy, mbn_s)
            endif
            varihm = sdih**2
c-- < <I>/sd(<I>) > by resolution
            call ad5sts(aih, sdih, mbn_s)
c
c-- Counts by resolution bin for multiplicity
            if (nuall .gt. 0) then
c   Unique measured reflections
               n_o_uniq_s(mbn_s) = n_o_uniq_s(mbn_s) + 1
c   Total in sphere (allowing for multiplicity
               n_o_totl_s(mbn_s) = n_o_totl_s(mbn_s) + multcy
c   All measured
               n_o_meas_s(mbn_s) = n_o_meas_s(mbn_s) + nuall
               if (icntrc .ne. 0) then
                  n_o_cent_s(mbn_s) = n_o_cent_s(mbn_s) + 1
               endif
            endif
c
c-- Some overall statistics
            aihmax = max(aihmax, aih)
            aihmin = min(aihmin, aih)
c
            do 44, l = 1,nrs
               irun = irunrf(l)
c mbn_b is rotation bin in run
c mbn_t   is global bin
               if (lanlbr(irun)) then
                  mbn_b = min(isrrbt(
     $               ccp4_hash_lookup(ibatrf(l))), nrrang(irun))
               else
                  mbn_b = min(int(zdrf(l)) + 1, nrrang(irun))
               endif
               mbn_t = i1ovlp(irun) + mbn_b - 1
               mbn_b_rf(l) = mbn_b
               mbn_t_rf(l) = mbn_t
c--- Means of applied scale even for singletons
               if (useflg(l)) then
                  call adssts(sclirf(l)/ghl(l), Dfhl(l),
     $               nprtds, irun, mbn_t, mbn_s)
c    Statistics of SD correction (including singletons)
                  call adsdst(sdffrf0(l), sdffrf(l),
     $               mbn_i, irun, mrf(l), swihc)

               elseif (isttrf(l) .eq. lqoutl) then
                  nrejs(mbn_b, irun) = nrejs(mbn_b, irun) + 1
               elseif (isttrf(l) .eq. lqoutlan) then
                  nrejs(mbn_b, irun) = nrejs(mbn_b, irun) + 1
                  nrfreja = nrfreja + 1
               elseif (isttrf(l) .eq. lqrejm) then
                  nrejm(mbn_b, irun) = nrejm(mbn_b, irun) + 1
               endif
c
 44         continue
c Histograms of partiality etc
            call hstprt(nrs, isttrf, frclrf, kmptrf, nmprrf, mrf)
c
c
            if (nuall .gt. 1) then
c Statistics only if more than one observation for this reflection 
               ntuniq = ntuniq + 1
               lunqdl = .false.
               sumDI2o = 0.0
               sumDI2b = 0.0
               sumDI2p = 0.0
               sumDI2m = 0.0
               janom = 1
c  Loop each observation for statistics
               do 49, l = 1, nrs
c
                  irun = irunrf(l)
                  mbn_b = mbn_b_rf(l)
                  mbn_t = mbn_t_rf(l)
c
c mbn_x, mbn_y, bins on detector position
                  if (lxyanl) then
                     mbn_x =
     $                  int((xdetrf(l) - cbinxy(1,irun))/sbinxy(1))
     $                  + nbinxy(1)/2
                     mbn_x = min(max(mbn_x,1), nbinxy(1))
                     mbn_y =
     $                  int((ydetrf(l) - cbinxy(2,irun))/sbinxy(2))
     $                  + nbinxy(2)/2
                     mbn_y = min(max(mbn_y,1), nbinxy(2))
                  endif
c
                  if (useflg(l)) then
c   Counts, fulls & partials 
                     ntotal = ntotal + 1
                     if (mrf(l) .eq. 0) then
                        ntfull = ntfull + 1
                     else
                        if (nmprrf(l) .gt. 0) then
                           ntsmpr = ntsmpr + 1
                        elseif (nmprrf(l) .lt. 0) then
                           ntscpr = ntscpr + 1
                        endif
                        nptmax = max(nptmax, iabs(nmprrf(l)))
                     endif
c Scaled intensity & sd
                     aihl  = ffrf(l) / ghl(l)
                     sdihl = sdffrf(l) / ghl(l)
c-- Deviation from overall mean
                     delI = aihl - aih
c      Sums for deviation from overall mean
                     call ad1sts(nuall, DelI, aihl, sdihl, irun, 
     $                  mbn_b, mbn_t, mbn_s, mbn_i)
                     sumDI2o = sumDI2o + DelI**2
c-- Deviation from I+ or I- (or Imean if no anomalous)
                     delIf = 0.0
                     nfif = 0
                     lfull = mrf(l).eq.0
                     if (kanomo .eq. 0 .or. icntrc .eq. 1) then
                        janom = 0
                        nkept = nuall
                        vardli = varihm
                        if (lfull) then
                           DelIf = aihl - aihf
                           nfif = nf
                        endif
                        if (nkept .gt. 1) sumDI2b = sumDI2b + DelI**2
                     else
                        if (mod(isymrf(l), 2) .eq. 0) then
c  I-
                           delI = aihl - aihmns
                           nkept = numns
                           vardli = varims
                           if (lfull) then
                              DelIf = aihl - aihfmn
                              nfif = nfmn
                           endif
                           if (nkept .gt. 1) sumDI2m = sumDI2m + DelI**2
                        else
c  I+
                           delI = aihl - aihpls
                           nkept = nupls
                           vardli = varipl
                           if (lfull) then
                              DelIf = aihl - aihfpl
                              nfif = nfpl
                           endif
                           if (nkept .gt. 1) sumDI2p = sumDI2p + DelI**2
                        endif
                     endif
                     if (nkept .gt. 1) then
c  Add into sums for deviation relative to I+/- only if more than 1 observation
c   Must be more than one observation in fulls-only selection as well
                        if (nfif .le. 1) lfull = .false.
                        call ad2sts(
     $                     nkept, DelI, vardli, delrf(l), aihl, sdihl,
     $                     DelIf, lfull, irun, 
     $                     mrf(l), mbn_b, mbn_t, mbn_s, mbn_i,
     $                     mbn_x, mbn_y)
c DEBUG
c                        if (delrf(l) .ne. 0.0) then
c                           write (6,'(5i4,3f10.1,f10.4,f10.1,f10.4)')
c     $                        jhkl, nrs, l, ffrf(l), sdffrf(l),
c     $                        ghl(l), swih, delrf(l)
c                        endif
c-DEBUG
c  Flag for unique reflection count
                        lunqdl = .true.
                     endif
c-- Bias from mixed sets, fulls and partials or partials with different
c   numbers of parts
                     if (nfbias .gt. 0 .and.
     $                  nmprrf(l) .ne. minparts) then
c     deviation = <Ifull> - Ihl(partial) 
c        <Ifull> is from minimum number of parts, perhaps full
                        DelI = aifull - aihl
                        call ad3sts
     $                     (DelI, aih, irun, mbn_b, mbn_t, mbn_s, mbn_i)
                     endif
c
c-- Bias analysis between fulls and 2-part partials, if any
                     if (minparts .eq. 1 .and. nfbias .gt. 0) then
c     There are some fulls
                        if (nmprrf(l) .eq. 2) then
c      and this is a 2-part partial
c       The next part (l+1) should contain the other half, sanity check
                           if (l+1 .le. nrs .and.
     $                        isttrf(l+1) .eq. lquprt) then
                              bigpart = frclrf(l+1)
c        frclrf(l) contains the total fraction, choose largest bit
                              bigpart =
     $                           max((frclrf(l)-bigpart), bigpart)
                              mbn_f =
     $                           min(max(int((bigpart-0.5)/frclbin)+1,
     $                           1), nfrclbin)
                              call adfsts(
     $                           DelI, aih, mbn_f, mbn_i, maxmbi)
                           endif
                        endif
                     endif 
c                    
                     ntotrjs(mbn_b, irun) = ntotrjs(mbn_b, irun) + 1
                  else
c Count outliers by detector position
                     if (isttrf(l) .eq. lqoutl)
     $                  n_m_rej_xy(mbn_x, mbn_y) =
     $                  n_m_rej_xy(mbn_x, mbn_y) + 1
                  endif
 49            continue
c PCV sums, returns estimates of population sd scatob & sd of mean scatmn
c (= 0.0 if not defined)
               call ad6sts(sumDI2o, nuall, aih, janom, mbn_s,
     $            sumDI2b, sumDI2p, nupls, aihpls, sumDI2m, numns, 
     $            aihmns, scatob, scatmn)
c
               if (lunqdl) then
c Number of unique multiplet reflections by resolution & intensity bins
                  n_uniq_s(mbn_s) = n_uniq_s(mbn_s) + 1
                  n_uniq_i(mbn_i) = n_uniq_i(mbn_i) + 1
                  if (icntrc .ne. 0) then
                     n_cent_s(mbn_s) = n_cent_s(mbn_s) + 1
                     n_cent_i(mbn_i) = n_cent_i(mbn_i) + 1
                  endif
               endif
            elseif (nuse .eq. 1) then
               ntonce = ntonce + 1
            endif
         endif
c     
c==========================================================================
c  Output
         if (wrtout) then
c..........................................................................
            if (ifilou .eq. 1 .or. ifilou .eq. 5) then
c AVERAGE option, output averaged I
               if (nuall .gt. 0) then
                  do 60, i = 1,3
                     datavg(i) = jhkl(i)
 60               continue
                  if (kfilou .ge. 0) then
c OUTPUT SPLIT (or one dataset)
c   Imean
                     datavg(4) = aih
                     datavg(5) = sdih
                     datavg(6) = aihpls
                     datavg(7) = sdihpl
                     datavg(8) = aihmns
                     datavg(9) = sdihms
c                     datavg(10) = scatob
c                     datavg(11) = scatmn
c
                     call lwrefl(koutstr1, datavg)
                     noutrf = noutrf + 1
c Store axial reflections for analysis of systematic absences
                     call staxrf(jhkl, aih, sdih)
c
c Optional Scalepack-style output
                     if (isclpk .gt. 0) then
                        if (kanomo .eq. 0 .or. icntrc .eq. 1) then
c   No anomalous
                           write (isclpk, 6160)
     $                        jhkl, aih*sclpkscl, sdih*sclpkscl
 6160                      format (3i4, 4f8.1)
                        else
c   Anomalous
                           write (isclpk, 6160)
     $                        jhkl, aihpls*sclpkscl, sdihpl*sclpkscl,
     $                        aihmns*sclpkscl, sdihms*sclpkscl
                        endif
                     endif
                  else
c OUTPUT TOGETHER
                     do k = 1, ndtsetsout
                        datavg(ncodts(k)) = qihdts(k)
                        datavg(ncodts(k)+1) = qsdidts(k)
                        datavg(ncodts(k)+2) = qihpl(k)
                        datavg(ncodts(k)+3) = qsdipl(k)
                        datavg(ncodts(k)+4) = qihmn(k)
                        datavg(ncodts(k)+5) = qsdimn(k)
                     enddo 
c
                     call lwrefl(koutstr1, datavg)
                     noutrf = noutrf + 1
c Store axial reflections for analysis of systematic absences
                     call staxrf(jhkl, aih, sdih)
                  endif 
               endif
            elseif (ifilou .eq. +2 .or. ifilou .eq. +3) then
c..........................................................................
c SEPARATE or POSTREF options, output all observations
               do 70, l = 1, nrs
                  if (isttrf(l) .lt. 0) then
                     istat = iabs(isttrf(l))
                     nstrej(istat) = nstrej(istat) + 1
                  endif
c Option to omit outliers
                  if (lolexc) then
                     if (isttrf(l) .le. lqoutl .or.
     $                  isttrf(l) .eq. lqoutp) go to 70
                  endif
c Option to omit partials
                  if (nprexc .lt. 0) then
                     if (mrf(l) .ne. 0) go to 70
                  elseif (nprexc .gt. 0) then
                     if (mrf(l) .ne. 0) then
                        do 71, i = 1, nprexc
                           if (irunrf(l) .eq. kprexc(i)) go to 70
 71                     continue
                     endif
                  endif
c     Scale factor
                  if (ghl(l) .gt. 0.0) then
                     datarf(ncosc,l) = sclirf(l) / ghl(l)
                     datarf(ncosds, l) =
     $                  sqrt((sdglrf(l) / ghl(l)**2)**2 + sdscrf(l)**2)
c     Modified sd
                     if (ncosdi .gt. 0) then
                        call corvar(.false., irunrf(l), datarf(ncsdi,l),
     $                     clprf(l), ghl(l)*swih, mrf(l), 
     $                     datarf(ncosdi,l), dummy, dummyr)
                     endif
                     if (ncosdp .gt. 0) then
                        call corvar(
     $                     .false., irunrf(l), datarf(ncsdip,l),
     $                     clprf(l), ghl(l)*swih, mrf(l),
     $                     datarf(ncosdp,l), dummy, dummyr)
                     endif
                  else
c    No valid scale, clear columns to zero
                     datarf(ncosc,l) = 0.0
                     datarf(ncosds, l) = 0.0
c     Modified sd
                     if (ncosdi .gt. 0) datarf(ncosdi,l) = 0.0
                     if (ncosdp .gt. 0) datarf(ncosdp,l) = 0.0
                  endif
c   Set status flag
                  if (ncoflg .gt. 0) then
c     To start, set flags as from input file
c  just this for now
                     krflag = lflgrf(l)
cc     Unpack
c                     call xrfflg(maxrfg, lrfflg, krflag)
cc     Set flag for outlier
c                     if (isttrf(l) .le. lqoutl) lrfflg(lfgotl) = .true.
c     Repack
c                     call srfflg(maxrfg, lrfflg, krflag)
                     datarf(ncoflg,l) = krflag
                  endif
c Postref
                  if (ifilou .eq. +3) then
c  Imean
                     datarf(ncopsf  ,l) = aih
                     datarf(ncopsf+1,l) = sdih
                     if (nmprrf(l) .gt. 1) then
c  First slot of summed partial (not scaled), save Isum (scaled)
                        sumpart   = ffrf(l)/ghl(l)
                        sdsumpart = sdffrf(l)/ghl(l)
                     endif
                     if ((jmprrf(l) .eq. 0 .and. nmprrf(l) .gt. 1) .or.
     $                (jmprrf(l) .gt. 0 .and. nmprrf(jmprrf(l)) .gt. 1)
     $                  ) then
c  Any slot of summed partial , store Isum
                        datarf(ncopsf+2,l) = sumpart
                        datarf(ncopsf+3,l) = sdsumpart
                     else
                        datarf(ncopsf+2,l) = 0.0
                        datarf(ncopsf+3,l) = 0.0
                     endif
                  endif
c
                  call lwrefl(koutstr1, datarf(1,l))
                  noutrf = noutrf + 1
 70            continue
            endif
            if (ifilou .eq. +4 .or. ifilou .eq. +5) then
c..........................................................................
c  UNMERGED option
c   Output unmerged data, but with partials summed or scaled, and with
c outliers  rejected
               do 80, l = 1, nrs
                  if (isttrf(l) .lt. 0) then
                     istat = iabs(isttrf(l))
                     nstrej(istat) = nstrej(istat) + 1
                     go to 80
                  endif
c Option to omit partials
                  if (nprexc .lt. 0) then
                     if (mrf(l) .ne. 0) go to 80
                  elseif (nprexc .gt. 0) then
                     if (mrf(l) .ne. 0) then
                        do 81, i = 1, nprexc
                           if (irunrf(l) .eq. kprexc(i)) go to 80
 81                     continue
                     endif
                  endif
c
                  call ccpzr(dataun, ncolou)
c
c Restore original indices
                  call asuget(jhkl,ihkl,isymrf(l))
c
                  if (lrdidx) then
c Reduced hkl as in input
                     do 82, i = 1,3
                        dataun(i) = jhkl(i)
 82                  continue
c M/ISYM (M = 0)
                     dataun(4) = isymrf(l)
                  else
                     do 83, i = 1,3
                        dataun(i) = ihkl(i)
 83                  continue
c M/ISYM = 1 (M = 0)
                     dataun(4) = 1
                  endif
c BATCH
                  dataun(5) = ibatrf(l)
c Intensity (scaled)
                  dataun(ncoi) = ffrf(l) / ghl(l)
c sd(I) (scaled) 
                  dataun(ncosdi) = sdffrf(l) / ghl(l)
c scale
                  dataun(ncosc) = sclirf(l) / ghl(l)
c sd(scale)
                  dataun(ncosds) =
     $               sqrt((sdglrf(l) / ghl(l)**2)**2 + sdscrf(l)**2)
c Optional columns:
c  NPART number of parts
c              = 1 for full (as initial value)
c              = 0 unmatched partial or 2nd & subsequent parts of partial
c                    (ie those flagged as lquprt)
c              .gt. 1 for summed partial (1st slot)
c              negated for scaled partial, = - number of parts
c   
                  if (nconpt .gt. 0) then
                     if (nmprrf(l) .eq. 0) then
c   observations with nmprrf = 0 shouldn't be here
                        write (lunout, '(a,5f8.0)')
     $                     ' **** nmprrf = 0, ',(dataun(i), i=1,5)
                        go to 80
                     else
                        dataun(nconpt) = nmprrf(l)
                     endif
                  endif
c-
c  TIME
                  if (ncotim .gt. 0) then
c    Time should be averaged over summed partials, but probably isn't
c too critical
                     dataun(ncotim) = datarf(nct, l)
                  endif
c-
c  XDET
                  if (ncoxdt .gt. 0) then
                     dataun(ncoxdt) = datarf(ncx, l)
                  endif
c-
c  YDET
                  if (ncoydt .gt. 0) then
                     dataun(ncoydt) = datarf(ncy, l)
                  endif
c-
c  ROT
c   use averaged phi (ROT)
                  if (ncorot .gt. 0) then
                     dataun(ncorot) = phirf(l)
                  endif
c- 
c  FRACTIONCALC
c   total fraction
                  if (ncofrc .gt. 0) then
                     dataun(ncofrc) = frclrf(l)
                  endif
c-
c  LP
                  if (ncolp .gt. 0) then
                     dataun(ncolp) = clprf(l)
                  endif

c-
c  BEAMS
                  if (mbeams .ne. 0) then
                     call bemvec(mbeams, ihkl, isymrf(l), irunrf(l),
     $                  ibatrf(l), phirf(l),
     $                  s0c, s2c)
                     if (ncobeam .gt. 0) then
                        call moveit(s0c, dataun(ncobeam), 3)
                        call moveit(s2c, dataun(ncobeam+3), 3)
                     endif
                  endif
c
                  call lwrefl(koutu, dataun)
                  noutrf = noutrf + 1
c Optional SCALEPACK-style output (unmerged)
                  if (isclpk .gt. 0) then
c  icnflg = 0 centric, = 1 I+, = 2 I- 
                     if (icntrc .ne. 0) then
c    Centric
                        icnflg = 0
                     else
                        if (mod(isymrf(l), 2) .eq. 0) then
c    I-
                           icnflg = 2
                        else
c    I+
                           icnflg = 1
                        endif
                     endif
c  ispndle is dummy = 0 here
                     ispndle = 0
c  Symmetry number
                     iasym = (isymrf(l) - 1)/2 + 1
c
                     write (isclpk, 6180)
     $                  ihkl, jhkl, ibatrf(l), icnflg, ispndle, iasym,
     $                  dataun(ncoi), dataun(ncosdi)
 6180                format (3i4,3i4,i6,i2,i2,i3,2f8.1)
                  endif
 80            continue
            endif
         endif
      endif
c     
c End of file?
      if (.not. eof) go to 20
c     
c     
cooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c     End of all reflections
c     
 100  continue
      if (wrtout) then
         write (lunout, *)
         call lwclos(koutstr1,1)
         if (koutstr2 .ne. 0) call lwclos(koutstr2,1)
         if (isclpk .gt. 0)
     $      close (unit=isclpk)
      endif
c
c Reset maximum resolution for completeness calculation
      smaxal = 0.0
      do 137, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then
            smaxal = max(smaxal, smaxfr(irun))
         endif
 137  continue
      if (smaxal .gt. 0.0) then
         dresol_out(max(1,jdataset)) = 1.0 / sqrt(smaxal)
      endif
c
c print results of correlation analysis
      if (lccdts) then
         call dtsprcc(nsrmax, reso)
c Print rms differences etc
         call dtsprdif(nsrmax, reso)
c Scale differences to each other to get estimates of relative f" and Delf'
         call dtsscale
c         if (nrfldli.gt. 0) then
c            call dtsbin(nrfldli, ar4gar, iingar(1,2), ndtsetsout)
c         endif
      endif
c
      if (.not.lstats) go to 500
      write (lunout, '(/5(a,i8/),a,i8)')
     $   ' Number of observations read                      : ',
     $   ninref,
     $   ' Number of unique reflections read                : ',
     $   nunqrf,
     $   ' Number of observations output                    : ',
     $   noutrf,
     $   ' Number of outliers rejected                      : ',
     $   nrfrej,
     $   ' Number of observations rejected on Emax limit    : ',
     $   nrfrjem,
     $   ' Number of observations outside resolution limits : ',
     $   nrejrs
      if (nrfreja .gt. 0) then
         write (lunout, '(a,i8)')
     $      ' Number of outliers rejected between I+ & I-      : ',
     $      nrfreja
      endif
      if (ifilou .gt. 0) then
         if (krsexc .eq. 0) then
            write (lunout, '(a,a)')
     $         '   (observations outside resolution limits are ',
     $         'omitted from the output file)'
         elseif (krsexc .eq. +1) then
            write (lunout, '(a,a)')
     $         '   (observations outside resolution limits will have ',
     $         'SCALE = 0.0, so will be rejected from merging)'
         elseif (krsexc .eq. +2) then
            write (lunout, '(a,a)')
     $         '   (observations outside resolution limits will have ',
     $         'scales calculated in the output file)'
         endif
      endif
c
c Mosflm flags
      if (ncflag .gt. 0) then
         write (lunout, 6201)
     $      (mflcnt(i,1), i=1,2), flmmax(1,1), flmmax(1,2),
     $      (mflcnt(i,2), i=1,2), flmmax(2,1), flmmax(2,2),
     $      (mflcnt(i,3), i=1,2),
     $      (mflcnt(i,4), i=1,2), flmmax(3,1), flmmax(3,2),
     $      (mflcnt(i,5), i=1,2),
     $      (mflcnt(i,6), i=1,2)
 6201    format (/'Numbers of observations (including parts of ',
     $      'reflections) marked in the FLAG column',/,
     $      '    By default all flagged observations are rejected',/,
     $      '    Observations may be counted in more than one category',
     $      //,
     $      33x,'Flagged  Accepted   Maximum   MaxAccepted',/,
     $      7x,'BGratio too large       ',2i8,2f12.3/,
     $      7x,'PKratio too large       ',2i8,2f12.3/,
     $      7x,'Negative < 5sigma       ',2i8,/,
     $      7x,'BG gradient too large   ',2i8,2f12.3/,
     $      7x,'Profile-fitted overloads',2i8,/,
     $      7x,'Spots on edge           ',2i8,/)
      endif
c
      ntpart = ntsmpr + ntscpr
      NHfull = ntfull
      NHpart = ntpart
c
      if (kprout .eq. +1) then
         write (lunout, '(//a,i8,a,i8,a/12x,a,i8,a,i8,a,i8,a/)')
     $      ' Statistics below are accumulated from:-', ntuniq,
     $      ' unique reflections, ', ntotal,' observations','(of which',
     $      ntfull, ' are fully recorded, ',
     $      ntsmpr, ' are summed partials, ',
     $      ntscpr, ' are scaled partials)'
      elseif (kprout .le. 0) then
         write (lunout, '(//a,i8,a,i8,a/)')
     $      ' Statistics below are accumulated from:-', ntuniq,
     $      ' unique reflections, ', ntotal,' observations'
      elseif (kprout .eq. +2) then

         write (lunout, '(//a,i8,a,i8,a/12x,a,i8,a,i8,a/)')
     $      ' Statistics below are accumulated from:-', ntuniq,
     $      ' unique reflections, ', ntotal,' observations','(of which',
     $      ntfull, ' are fully recorded, ', ntscpr,
     $      ' are scaled partials)'
      endif
      if (ntonce .gt. 0) then
         write (lunout,'(a,i8,a//)')
     $      '     this excludes ', ntonce,
     $      ' good observations measured once only'
      endif
c
      if (nanmrj .gt. 0) then
         write (lunout, '(/1x,i10,a/)')
     $      nanmrj, ' unmatched anomalous pairs omitted'
      endif
      if (nptmax .gt. 0) then
         write (lunout, '(/a,i4/)')
     $  ' Maximum number of parts in a partially recorded reflection: ',
     $      nptmax
      endif
c
      if (nstrej(1)+nstrej(3)+nstrej(4)+nstrej(5)+nstrej(6)+
     $   nstrej(7)+nstrej(8)+nstrej(11)+nstrej(12)+nstrej(13) .gt. 0)
     $   write (lunout, '(a)')
     $   ' Observations omitted from statistics:'
 6091 format(9x,i8,4x,a)
      if (nstrej(1) .gt. 0) write (lunout, 6091)
     $   nstrej(1), 'unmatched or incomplete partials'
      if (nstrej(3) .gt. 0) write (lunout, 6091)
     $   nstrej(3),'rejected outliers'
      if (nstrej(4) .gt. 0) write (lunout, 6091)
     $   nstrej(4),'overload'
      if (nstrej(5) .gt. 0) write (lunout, 6091)
     $   nstrej(5),'resolution'
      if (nstrej(6) .gt. 0) write (lunout, 6091)
     $   nstrej(6),'zero input scale'
      if (nstrej(7) .gt. 0) write (lunout, 6091)
     $   nstrej(7),'zero standard deviation'
      if (nstrej(8) .gt. 0) write (lunout, 6091)
     $   nstrej(8),'M flag .gt. 2'
      if (nstrej(11) .gt. 0) write (lunout, 6091)
     $   nstrej(11),'excluded area of detector'
      if (nstrej(12) .gt. 0) write (lunout, 6091)
     $   nstrej(12),'unmatched Anomalous'
      if (nstrej(13) .gt. 0) write (lunout, 6091)
     $   nstrej(13),'non-zero error FLAG in input file'
c
      if (nrejpt(2) .gt. 0)
     $   write (lunout, 6091)
     $   nrejpt(2),'partial sets with too many parts'
      if (nrejpt(1) .gt. 0)
     $   write (lunout, 6091)
     $   nrejpt(1),'partial sets with gaps'
      if (nrejpt(3) .gt. 0 .or. nrejpt(4) .gt. 0) then
         write (lunout, 6091)
     $      nrejpt(3), 'partial sets with total fraction too small'
         write (lunout, 6091)
     $      nrejpt(4), 'partial sets with total fraction too large'
      endif 
      if (nrejpt(5) .gt. 0)
     $   write (lunout, 6091)
     $   nrejpt(5),'partial sets with inconsistent MPART flags'
      if (nrejpt(6) .gt. 0)
     $   write (lunout, 6091)
     $   nrejpt(6),'partial sets with error flags (FLAG)'
c
c Corner correction
      if (lcorner) call cnr_print_histo
c     
c--- Labels: rotation ranges or batches
c   label(1,) if always run.rotation
c   label(2,) is batch if batch analysis selected
      k = 0
      do 110, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then
c  Omit reference run from statistics
            do 115, i = 1, nrrang(irun)
               k=k+1
               write (lab, '(i2,a1,i3)') irunum(irun), '.', i
               call  squeez(lab, label(1,k))
               if (lanlbr(irun)) then
                  write (lab, '(i6)') jbatrn(i,irun)
               endif
               call  squeez(lab, label(2,k))
 115        continue
         endif
 110  continue
c-- Check if all runs are analysed by batch or rotation range
      k = 0
      ibtanl = +1
      do 120, irun = 1,numrun
         if (lanlbr(irun)) then
            if (k .lt. 0) then
               ibtanl = 0
            endif
            k = +1
         else
            if (k .gt. 0) then
               ibtanl = 0
            endif
            k = -1
         endif
 120  continue
      if (ibtanl .ne. 0) ibtanl = k
c ibtanl = +1 if all Batch, = -1 if all by rotation range, = 0 if mixed
c-    runs
      nrunp1 = 0
      do 138, irun = 1, numrun
         if (irun .ne. irefrn .and. lactvrun(irun)) then
            nrunp1 = nrunp1+1
            write (lab, '(a3,i3)') 'Run',irunum(irun)
            call squeez(lab, labelr(nrunp1))
         endif
 138  continue
      nrunp1 = nrunp1+1
      labelr(nrunp1) = 'AllRun'
c
c     
c.............................................................................
c--- Print analysis of applied scales
      call prapsc(lanlbr, ibtanl, nsrmax, reso, label)
      call prt_tcr
c
c--- Tails correction by run & resolution
      if (nprtds .gt. 0) then
         call prdfsc(nsrmax, reso, labelr, nrunp1)
      endif
c
      if (nprsph .gt. 0) then
         call prtsph
      endif

c
      call prscrn(lanlbr, ibtanl, label, timfac, labdts)
c     
c.............................................................................
c     
c---- Rmerge v rotation range
      call prarot(label, labdts)
      write (lunout, 6120)
 6120 format (/1x,20('====')/)
c---- Rmerge v. resolution by run
      if (nrunp1 .gt. 2) then
         call prares(labelr, nrunp1, reso, nsrmax, labdts)
      endif
c---- Rmerge & other deviations v. resolution
      call prdres(reso, nsrmax, kanomo, labdts)
c---- Rmerge v. intensity
      call prdint(kanomo, labdts)
c
c....................................................................
c--- Completeness & multiplicity
      call prcmpl(reso, nsrmax, smaxal, labdts,
     $   dcell_out(1,max(1,jdataset)))
c
c--- Correlations within dataset
      CCthrshld = 0.3
      RCRthrshld = 1.3
c  nCC      number of bins with CC  > CCthrshld
c  nRCR     number of bins with RCR > RCRthrshld
      call wdsprcc(nsrmax, reso, labdts,
     $   CCthrshld, RCRthrshld, nCC, nRCR)
c....................................................................
c--- Axial reflections
      call praxrf(labdts)
c....................................................................
c--- sd analysis
      call prsdan(ntfull, ntpart, labdts)
c....................................................................
c--- Histograms of partials
      if (kprint(1) .ge. 3) call prunmt
c....................................................................
c--- Partial bias v. fractioncalc
      if (kprint(1) .ge. 3) call prbfrc(nfrclbin, frclbin)
c....................................................................
c--- Print analyses on detector position
      if (lxyanl) call prxyan
c....................................................................
c  Errors:
c    differences in coordinates between partials
      if (nerpdc.gt.0) then
         write(lunout,'(/,a,a)')
     +      ' Batches with differences in detector ',
     +      'coordinates for partials:'
         k = 0
         do 300, i = 1,min(nerpdc,maxpdc)
            if (i.gt.1) then
               do 301, j = 1,min((i-1),maxpdc)
                  if (ibtpdc(i).eq.ibtpdc(j)) goto 305
 301           continue
               k = k+1
               if (k .lt. maxbat) then
                  jbtpdc(k) = ibtpdc(i)
               else
c                 Shouldn't happen
                  call ccperr(1,'*** Too many error nbatches ***')
               endif
 305           continue
            else
               k = k+1
               jbtpdc(k) = ibtpdc(i)
            endif
 300     continue
         if (k .gt. 0) then
            write (lunout, '((1x,10i5/))') (jbtpdc(i), i = 1,k)
            write(lunout,'(/)')
         endif
         if (nerpdc.ge.maxpdc) write(lunout,'(a,i8,a,/)') ' more than '
     +      ,maxpdc,' bad reflections!'
      endif
c.............................................................................
      HanoNPs = 0.0
      if (lnormn) then
c^         if (kanomo .ne. 0) then
c  Always do this
c++++++++++++++  1) Anomalous differences  ++++++++++++++++++++++ 
c
            write (lunout, '(/a/a)')
     $         ' ================================================',
     $         ' Normal probability analysis of anomalous differences'
c Close down array of normalized anomalous differences
            ip = 4
            labfpt = ' '
            if (ndtsetsout .gt. 1) then
               labfpt = labxdts
            endif
            call newlpr(iwrpra, igrapha, ip, labfpt)
            ip = 1
            call nrmprb(
     $         iwrpra, maxptd, aregar(1,2), ianmser, dltlim, prblim,
     $         obslim(irun), slope(1,ip,1), sintct(1,ip), nused(1,ip))
            labfpt = 'Anomalous differences'
            l = lenstr(labfpt)
            write (lunout, 6151) labfpt(1:l), nused(1,ip),
     $         slope(1,ip,1), sintct(1,ip), dltlim,
     $         nused(2,ip), slope(2,ip,1), sintct(2,ip)
 6151       format (
     $         /1x,'====== ',a,' ======'/
     $         35x,'Number     Slope Intercept'/
     $         '                        All data:',i8,2f10.3/
     $         ' Data within expected delta',f5.2,':',i8,2f10.3//)
            igrapha = igrapha + 1
            HanoNPs = slope(2,ip,1)
c^         endif
c++++++++++++++   2) Individual discrepancies ++++++++++++++++++++++ 
         k = 0
         if (ntfull .gt. 0) k = k+1
         if (ntsmpr .gt. 0) k = k+1
         if (ntscpr .gt. 0) k = k+1
         if (k .eq. 0) go to 460
         write (lunout, '(/a/a)')
     $      ' ================================================',
     $      ' Normal probability analysis, by run & partiality'
         do 400, irun = 1, numrun
            if (irun .eq. irefrn .or. .not. lactvrun(irun)) go to 400
            if (numrun .gt. 1) then
               write (runlab,'(a,i2,a)') 'Run',irunum(irun),': '
            else
               runlab = ' '
            endif
            l = lenstr(runlab)
c  Analysis on fulls & summed & scaled partials separately
c.................
            do ip = 1,4
               slope(1,ip,irun) = 0.0
               slope(2,ip,irun) = 0.0
            enddo
            do 402, ip = 1,4
               lanlys = .false.
               iwrprb = iabs(iwrprb)
               if (ip .eq. 1) then
                  labfpt = 'fulls'
                  if (ntfull .gt. 0) lanlys = .true. 
               elseif (ip .eq. 2) then
                  labfpt = 'summed partials'
                  if (ntsmpr .gt. 0) lanlys = .true. 
               elseif (ip .eq. 3) then
                  labfpt = 'scaled partials'
                  if (ntscpr .gt. 0) lanlys = .true. 
               elseif (ip .eq. 4) then
c     Fulls vs. fulls analysis only if both fulls & partials are present
                  labfpt = 'fulls against fulls only'
                  if (ntfull .gt. 0 .and.
     $               (ntsmpr .gt. 0 .or. ntscpr .gt. 0)) lanlys = .true.
c     No plot of fulls vs. fulls
                  iwrprb = -iabs(iwrprb)
               endif
               if (l .gt. 0) labfpt = runlab(1:l)//' '//labfpt
               k = 0
               if (lanlys) then
                  call newlpr(iwrprb, igraph, ip, labfpt)
                  do 401, j = 1, nrfgar
                     if (ip .eq. 4) then
                        argar = aregar(j,3)
                     else
                        argar = aregar(j,1)
                     endif
c Copy all non-zero deltas for run irun into array ar2gar
                     if (iingar(j,1) .ne. 0 .and. argar .ne. 0.0) then
                        call unpckf(
     $                     irn, m, nmpr, icntrc, iingar(j,1))
                        if (irn .eq. irun) then
                           if (m .eq. 0) then
c                            Full
                              iflprt = 1
c                            .. for fulls only
                              if (ip .eq. 4) iflprt = ip
                           elseif (nmpr .lt. 0) then
c                            Scaled partial
                              iflprt = 3
                           else
c                            Summed partial
                              iflprt = 2
                           endif
                           if (ip .eq. iflprt) then
                              k = k+1
                              aregar(k,2) = argar
                           endif
                        endif
                     endif
 401              continue
c Do normal probability analysis
                  call nrmprb(
     $               iwrprb, maxptd, aregar(1,2), k, dltlim, prblim,
     $               obslim(irun), slope(1,ip,irun), sintct(1,ip),
     $               nused(1,ip))
                  write (lunout, 6101)
     $               irunum(irun), labfpt(1:lenstr(labfpt)), k,
     $               slope(1,ip,irun), sintct(1,ip), dltlim,
     $               nused(2,ip), slope(2,ip,irun), sintct(2,ip)
 6101             format (
     $               /1x,'====== Run number:',i4,', ',a,' ======'/
     $               35x,'Number     Slope Intercept'/
     $               '                        All data:',i8,2f10.3/
     $               ' Data within expected delta',f5.2,':',i8,2f10.3/)
                  if (iwrprb .gt. 0) igraph = igraph+1
               endif
c.................
 402        continue
 400     continue
         write (lunout, '(a/)')
     $      ' =========================================='
c
         iwrprb = iabs(iwrprb)
c
c  List final SD multipliers
         write (lunout, 6111)
 6111    format (//' Final assessment of SDcorrection multipliers'//
     $      '     Run                  Fulls                     ',
     $      '      Partials'/
     $      '          SdFac_used  _corrected  _fullsonly ',
     $      '      SdFac_used  _corrected ')
         do 480, irun = 1, numrun
            if (irun .ne. irefrn .and. lactvrun(irun)) then
               sdmlfl = sdfact(1,irun,1)
               sdmlpt = sdfact(1,irun,2)
               write (lunout, '(1x,i6,3f12.4,5x,2f12.4)')
     $            irunum(irun),
     $            sdmlfl, sdmlfl*slope(2,1,irun),
     $            sdmlfl*slope(2,4,irun), 
     $            sdmlpt, sdmlpt*slope(2,2,irun)
            endif
 480     continue
c
      endif
c
 460  continue
      write (lunout, '(///)')
c-harvest-mod
      call sharvest(pname, xname, dname)
c Summary
      lresult = .true. 
      if (ndtsetsout .gt. 1) lresult = .false.
      call prsumry(jdataset, lresult)
c Print warning if ANOMALOUS OFF & big anomalous signal
      if (kanomo .eq. 0) then
c   ANOM OFF
c and either
c    AnomPlot slope > 1.3
c or CCanom > threshold (0.3) in 2 resolution bins
c or RCRanom > threshold (1.3) in 2 resolution bins
         if ((HanoNPs .gt. 1.3) .or. (nCC .gt. 2) .or.
     $      (nRCR .gt. 2)) then
            write (lunout, '(/a/)')
     $         'WARNING WARNING WARNING' 
            write (lunout, 6501)
 6501       format ('$TEXT:Warning:     $$ $$'///
     $    'This dataset appears to have a significant anomalous signal',
     $         ' but you have processed it ignoring this.'//
     $         'Important large anomalous differences may have been ',
     $         'rejected as outliers.'//
     $    'You maybe should rerun scaling with ANOMALOUS ON selected'/
     $ '(in ccp4i, "Separate anomalous pairs for merging statistics")'/
     $         '$$'//)
         endif
      endif 
c
 500  continue
      if (ipass .lt. npass) then
c Loop back for more output datasets
c Close & reopen input file to reinitialise header stuff
         call lrclos(1)
         iprint = 0
         ifail = 0
         call lropen(1,'HKLIN',iprint,ifail)
         go to 10
      endif
c Final "Result" summary
      if (ndtsetsout .gt. 1) call presult(.true.)
c Close normal probability files if required
      if (lplnrm) then
         if (iwrprb .gt. 0) then
            call linprb(iwrprb, igraph, 4.0)
         endif 
         close (unit=iwrprb)
      endif
      if (lplanm) then
         call linprb(iwrpra, igrapha, 4.0)
         close (unit=iwrpra)
      endif
      call clwrrp(krogplt)
c
      end
      subroutine refscl(maxmtx, amatrx, eigval, eigvec, vecinv, work)
c     ===============================================================
c
c     
c-- Declarations of arguments
      include 'bigarg.fh'
c     
c     Common blocks
c     
      include 'parameter.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'refcon.fh'
      include 'rfile.fh'
      include 'params.fh'
      include 'column.fh'
      include 'rundef.fh'
      include 'flags.fh'
      include 'dump.fh'
      include 'tie.fh'
      include 'glocon.fh'
      include 'errors.fh'
      include 'means.fh'
      include 'sphhrm.fh'
      include 'datasets.fh'
c     
c     Functions
      logical hkleq
      external hkleq
c     
c     Locals
      real w,
     $   aih, uh, summ(4,maxrun), sdih, swih, Rtie, delC
      real ubss(4,maxrun), resid, tdgidp,
     $   Thl, Chl, Shl, vmin1, vmin2, vali, wt,
     $   sumd, frccal, sumins, swihpl, swihmn,
     $   ss, stol, shfmen, shfmax, frcshf, dampnow, epsiln
      integer nfilter, istat, ip, nsumd, kshfmx, kwgtyp, mbn_s
      integer ninref, ninrfr, iaddr(maxpar),
     $   nparcm, iptpar(maxpar), nrdref, nrejpt(6), nsumin, nrfrjem
      integer nsum(4,maxrun), icycle, jhkl(3), nref, irun, nrs
      integer i, j, k, l, m, i1, i2, nparcy, k1, k2, nuse, nrej,
     $   nstrej(12), nolrej, nwsrej, nrfc(maxpar), nused, isysab
      integer ii, jj, jp, krej2, nfull, krfskp, ninskp,
     $   nrfc0(maxpar), janom, nkept, icntrc, iselds, kpflag, ltietp

      logical eof, convrg, lsdref, lpart, lprint, lrejol, lreset, lfail,
     $     lsanom
c     
      real simnsr(maxres, maxsets), swtsr(maxres, maxsets)
      real simnsa(maxres), swtsa(maxres), siunsc
      integer nimnsr(maxres, maxsets), niunsc, nextracycles
c
c     Reflection stores
      real ffrf(maxrfl), wtrf(maxrfl), 
     $   sdffrf(maxrfl), timerf(maxrfl),
     $   xdrf(maxrfl), ydrf(maxrfl), zdrf(maxrfl), frclrf(maxrfl),
     $   ffprrf(maxrfl), sdfprf(maxrfl),
     $   clprf(maxrfl), delrf(maxrfl), xsirf(maxrfl),
     $   phirf(maxrfl), xdetrf(maxrfl), ydetrf(maxrfl),
     $   ghl(maxrfl), dghldc(maxpmr, maxrfl), dgIdpi(maxpar),
     $   sclirf(maxrfl), sdscrf(maxrfl), datarf(maxcol,maxrfl),
     $   Dfhl(maxrfl), eorf(maxrfl)
      integer irunrf(maxrfl), ibatrf(maxrfl), mrf(maxrfl),
     $   isymrf(maxrfl), isttrf(maxrfl), mprf(maxrfl),
     $   nmprrf(maxrfl), jmprrf(maxrfl), lflgrf(maxrfl), iserrf(maxrfl),
     $   lanmtc(maxrfl), kmptrf(maxrfl)
      logical useflg(maxrfl)
c     Refinement
c     per reflection
      real dIhdpi(maxpar), delI, dshldq(2*maxsdf,maxrfl),
     $   dshldg(maxrfl), dshldp(maxpmr, maxrfl)
c     least squares
      real sumwdl, varfac, wdgdp
      real gradnt(maxpar)
c     parameters
      real trshft(maxpar), shift(maxpar), sdpar(maxpar),
     $   parold(maxpar), parnew(maxpar), ubs(maxpar)
      character*8 parnam(maxpar)
c     
c-save-      save /scales/
c     
c------------------------------------------------------------------
c     
c     Initializations
      nerpdc = 0
c     Don't refine sd corrections at first
      lsdref = .false.
      convrg = .false.
c     Outlier printing
      lprint = .false.
c     Outlier rejection
      lrejol = .false.
c     Reject outliers if 2 observations
      krej2 = -1
c     Ignore anomalous differences
      janom = 0
      iselds = 0
c Keep partials?
      if (kprsum .eq. -1) then
c  fulls only 
         kpflag = -1
      elseif (kprsum .eq. 0) then
c  fulls + previously scaled partials
         kpflag =  0
      else
c  everything
         kpflag = +1
      endif
      do 2, i = 1, nparam
         parnam(i) = ' '
 2    continue
      nrejpt(1) = 0
      nrejpt(2) = 0
      nrejpt(3) = 0
      nrejpt(4) = 0
      nrejpt(5) = 0
      nrejpt(6) = 0
c
c Weighting type
      kwgtyp = kscwgtyp
c
c Separate I+ & I- in scaling if kanoms = +1
      if (kanoms .ge. +1) then
         lsanom = .true.
      else
         lsanom = .false.
      endif
c
      do 1, irun = 1, numrun
         sminfr(irun) = 10000.
         smaxfr(irun) = 0.0
 1    continue
c
c Initialize skip option
c  ninskp    offset for skip, incremented each cycle
      ninskp = -1
      krfskp = nrfskp
c
c     
c     +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+
c     
c     Loop refinement cycles
c     
      icycle = 0
c     
 10   icycle = icycle + 1
c
c Reflection skip option
      if (nrfskp .gt. 1) then
         ninskp = ninskp+1
         if (icycle .le. ncyskp) then
            krfskp = nrfskp
            if (convrg) then
c  Refinement converged, but still skipping reflections: do some more cycles
c  without skipping (at least 2 more)
               krfskp = 0
               ncyskp = 0
               convrg = .false.
               write (lunout, '(/,a,i5,a,/,a)')
     $            ' Converged on cycle ', icycle-1,
     $            ' while still SKIPPING reflections',
     $            ' Reset convergence flag'
               if (ncycle .le. icycle+1) then
                  ncycle = icycle+1
                  write (lunout, '(a,i5)')
     $               ' Number of cycles reset to ', ncycle
               endif
            endif
         else
            krfskp = 0
         endif
      endif
c     Do next cycle or not?
c     ncycle is maximum number of cycles
      if (icycle .le. ncycle .and.
     $   .not. convrg) then
c     
c     Refine sd corrections only from cycle jsdcyc
         if (icycle .ge. jsdcyc) lsdref = .true.
c     
c     Reject outliers only from cycle jolcyc
         if (icycle .ge. jolcyc) lrejol = .true.
c     
c    Never print outliers unless PRINT DEBUG set
         if (kprint(1) .ge. 4) lprint = .true.
c     
c     Clear matrix & rhs
         do 15, j = 1, nparam
            gradnt(j) = 0.0
            nrfc(j) = 0
            do 16, i = 1, nparam
               amatrx(i,j) = 0.0
 16         continue
 15      continue
c     
         ninref = 0
         ninrfr = 0
         nrdref = 0
         nref = 0
         nwsrej = 0
         nolrej = 0
         nrfrjem = 0
c Sum for mean intensity
         sumins = 0.0
         nsumin = 0
         do 18, i = 1,12
            nstrej(i) = 0
 18      continue
         sumwdl = 0.0
         resid = 0.0
         do j = 1, maxsets
            do i = 1, maxres
               simnsr(i,j) = 0.0
               swtsr(i,j) = 0.0
               nimnsr(i,j) = 0
            enddo
         enddo
         siunsc = 0.0
         niunsc = 0
c
         call clrsts
c Matched pair pointers
         call ccpzi(lanmtc, maxrfl)
c     
c     Parameter count for this cycle, depending on whether refining
c     SD factors or not, also allow for LINKs
         nparcy = 0
         do 17, irun = 1,numrun
            if (lsdref) then
               npmrun(irun) = nprrun(irun) + npqrun(irun) - nlnkrn(irun)
            else
               npmrun(irun) = nprrun(irun) - nlnkrn(irun)
            endif
            nparcy = nparcy + npmrun(irun)
 17      continue
c     
         call lrrewd(1)
c  Tell sgetref that we're starting to read the file
         eof = .true.
c     
cooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c     Loop reflections
c   Get all observations of this reflection
c     
 20      call sgetref(lsanom, maxrfl, ninrfr, .true., kpflag, .false.,
     $      eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $      ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $      isymrf, sclirf, sdscrf, lflgrf, frclrf, clprf, isttrf,
     $      timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $      nmprrf, datarf, istat)
         if (istat .eq. +1) then
c  All observations beyond resolution limits
            nstrej(5) = nstrej(5) + nrs
            go to 90
         endif
         if (nrs .gt. 0) then
            nrdref = nrdref + 1
         endif
         if (nrs .gt. 1) then
c...................................................................
c  Centric if icntrc = 1, else = 0
            call centr(jhkl, icntrc)
c  Multiplicity: isysab = 1 for systematic absences
            call epslon(jhkl,epsiln,isysab)
c Reflection skip option
            if (krfskp .gt. 1) then
               if (mod((nrdref+ninskp), krfskp) .ne. 0) go to 90
            endif
c Don't do anything if .le. 2 fulls & only fulls used
            if (kprsum .eq. 0 .and. nfull .le. 1) go to 90
c     
c     --- --- Yes, process all measurements of this reflection
c     
c     Get scale for each observations, & their derivatives with respect to
c     the scale factors
            do 30, l=1,nrs
               if (isttrf(l) .eq. lqOK
     $            .or. isttrf(l) .eq. lqprt) then
                  call sclder(irunrf(l),
     $               stol, timerf(l), xdrf(l), ydrf(l), zdrf(l),
     $               ghl(l), Thl, Chl,  Shl, dghldc(1,l))
c Trap negative scales
                  if (ghl(l) .le. 0.0) then
                     write (lunout,
     $                  '(/a,f12.5,a,i3,a,3i4,a,i8,a,i4/)')
     $                  ' **** Negative scale factor',ghl(l),
     $                  ' for observation',
     $                  l,' of reflection',jhkl,', batch',ibatrf(l),
     $                  ' symmetry',isymrf(l)
                     write (lunout,6001)
 6001                format (/,
     $                  'This may arise from over-shifting scale ',
     $                  'factors, particularly if the scales have a ',
     $                  'wide range',/,
     $                  'Try damping shifts (eg DAMP 0.1 3) or unit ',
     $                  'weights (CYCLES WEIGHT UNIT)',/,/)
                     call ccperr(1,'*** Negative scale ***')
                  endif
               endif
 30         continue
c     
c Process partials if required
            nuse = nrs
            call partls(.true.,
     $         jhkl, kprsum, lpart, stol, nrs, nprtds, ltdsrn,
     $         ffrf, sdffrf, ffprrf, sdfprf,
     $         ghl, Dfhl, xdrf, ydrf, zdrf, phirf, xsirf,
     $         frclrf, kmptrf, isttrf, mrf, mprf, isymrf, ibatrf,
     $         irunrf, nmprrf, jmprrf, kprflg, kprmis, mxprwd, imid,
     $         nrejpt, nuse, prthld, prlims, prmaxs, dghldc, .true.,
     $         lavpsc)
            if (nuse .le. 1) go to 90
c Check fulls, summed or scaled partials against intensity limits
            call  chkilm(
     $         nrs, ffrf, sdffrf, isttrf, irunrf, nmprrf, sclirf, istat)
            if (istat .eq. lqweak) then
c     Reject all observations of this reflection if any is too weak
               nwsrej = nwsrej + 1
               go to 90
            elseif (istat .eq. lqstrn) then
c     too strong
               nstrej(12) = nstrej(12) + 1
               go to 90
            endif
c
c     Calculate sqrt-scale-weighted mean swih, for sd correction
c exclude reference run
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, iselds, .false., useflg)
            call setwgt(3, nrs, ffrf, sdffrf, ghl, wtrf)
            call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $         wtrf, swih, sdih, uh, nused)
            do 45, l = 1,nrs
c     Correct sds.
c     If refining sd factors, also calculate partial derivatives of sd(Ihl)
               call corvar(lsdref, irunrf(l), sdffrf(l), clprf(l),
     $            ghl(l)*swih, mrf(l), sdffrf(l),
     $            dshldq(1,l), dshldg(l))
 45         continue
c
c    mbn_s is resolution bin number
            mbn_s = min(int((ss-0.0000001)/srange) + 1, nsrang)
            if (lqmnir .and. eacmax .gt. 0.0) then
c Check Emax limits if set
               call chemax(nrs, lprint, 
     $            icntrc, epsiln,
     $            jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $            isymrf, ibatrf, mbn_s,
     $            eorf, nrej, istat)
c  returns istat = -1 if mean I is missing, but doesn't matter
               nrfrjem = nrfrjem + nrej
            endif
            if (lrejol) then
               call choutl(nrs, janom, lprint, .true., lanbyd, icntrc,
     $            jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $            isymrf, lanmtc, nuse, nrej,
     $            sdrejr, sdrejr2, 0.0, 0.0, krej2,
     $            delrf, swih, swihpl, swihmn)
c     For scaling, if any observation rejected, reject whole reflection
               if (nrej .ne. 0) then
                  nolrej = nolrej + 1
                  go to 90
               endif
            endif
c     
c     Calculate the variance-weighted mean intensity aih,
c     uh = Sum(w * ghl**2)
c     include reference run
            l = 0
c Flag observations to include
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         l, iselds, .false., useflg)
c Set weights
            call setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c Get mean
            call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $         wtrf, aih, sdih, uh, nused)
            sumins = sumins + aih
            nsumin = nsumin + 1
c Sums for mean I by run
c  Weight = 1 centric, 2 acentric
            wt = 2.0 - float(icntrc)
            do l = 1,nrs
               if (useflg(l)) then
                  k = kdtsrn(irunrf(l))
                  if (k .gt. 0) then
c  Sum(Imean)
                     simnsr(mbn_s, k) =
     $                  simnsr(mbn_s, k) +
     $                  ffrf(l)*wt/(ghl(l)*epsiln)
                     swtsr(mbn_s, k) =
     $                  swtsr(mbn_s, k) + wt
                     nimnsr(mbn_s, k) =
     $                  nimnsr(mbn_s, k) + 1
                     siunsc = siunsc + ffrf(l)
                     niunsc = niunsc + 1
                  endif
               endif
            enddo
c     
c     Calculate partial derivatives with respect to global parameters:
c     dshldp = d sd(Ihl) / dp; dIhdpi = d Ih / dp
c     Also set dghldc = 0.0 for sd factors
            call getdgi(lsdref,nparcy,nrs,aih,uh,nref,
     $         ffrf,sdffrf,wtrf,irunrf,isttrf,ghl,
     $         dghldc,dshldg,dshldq,dshldp,dIhdpi)
c     
            if (kprint(1) .ge. 4) then
               if (mod(nref,kprint(2)) .eq. 0) then
c     Debug
                  write (lunout,6491) jhkl
 6491             format(' *hkl:',3i4)
                  do l=1,nrs
                     if (isttrf(l) .eq. lqOK) then
                        write (lunout,6492)
     $                     ffrf(l),sdffrf(l),ghl(l),irunrf(l)
 6492                   format(10x,2f9.0,f8.4,i4)
                     endif
                  enddo
                  write (lunout,'(a,2f10.0)')
     $               '    Ih, sdIh',aih, sdih
                  write (lunout, '(a,100(/1x,12g10.3))')
     $               '    dIhdpi ',(dIhdpi(k), k=1,nparcy)
                  do 49, l = 1,nrs
                     if (isttrf(l) .eq. lqOK) then
                        irun = irunrf(l)
                        write (lunout,'(a,i4,a,100(1x,10g10.3/))')
     $                     ' l = ',l,' dghldc:',
     $                     (dghldc(k,l), k = 1,nprrun(irun))
                     endif
 49               continue
               endif
            endif
c     
c     Add components to normal matrix & gradient vector
c     Loop observations l
            do 50, l=1,nrs
               if (isttrf(l) .eq. lqOK) then
                  irun = irunrf(l)
                  delI = ffrf(l) - ghl(l)*aih
                  sumwdl = sumwdl + wtrf(l) * delI**2
                  resid = resid + log(sdffrf(l)) + wtrf(l) * delI**2
                  nref = nref+1
c     Loop parameters i(k)
                  ip = 0
                  do 60, ii = 1, nparcy
                     k = ipnloc(ii)
                     if (k .lt. 0) then
c          global parameter linked to different local parameters
c              k := 0 if global parameter does not apply to this run
                        k = iplloc(-k, irun)
                     else
c          unlinked parameter, just check run
                        if (ipnrun(ii) .ne. irun) k = 0
                     endif
c 
c     d (ghl * Ih) / dp = ghl * dIh/dp + Ih * dghl/dp
                     tdgIdp = ghl(l)*dIhdpi(ii)
c     Does ghl for this observation depend on this parameter?
c     It does if this parameter belongs to this run, directly or via link
                     if (k .gt. 0) then
                        if (dghldc(k,l) .ne. 0.0) then
                           tdgIdp = tdgIdp + aih*dghldc(k,l)
c     Count contributions
                           nrfc(ii) = nrfc(ii)+1
                        endif
                     endif
                     if (tdgIdp .ne. 0.0) then
                        ip = ip+1
                        iaddr(ip) = ii
                        dgIdpi(ip) = tdgIdp
c     Rhs gradient vector
                        gradnt(ii) = gradnt(ii) +
     $                     wtrf(l)*tdgIdp*delI
                        wdgdp = wtrf(l)*dgIdpi(ip)
c     Normal matrix
                        do 70, jp = 1,ip
                           jj = iaddr(jp)
c     Note should always have A(i,j) j .le. i for EA06C, but here build
c     i .le. j, and copy to other triangle later {was (i,j)}
c     Note that jj .le. ii
                           amatrx(jj,ii) = amatrx(jj,ii) +
     $                        wdgdp*dgIdpi(jp)
c%debug
c                              if (ii.eq.jj) then
c                                 write (lunout, '(a,2i4,3f12.5)')
c     $                                ' ip,jp,dgdpi,dgdpj,dgdpi*dgdpj',
c     $                                ip, jp, dgIdpi(ip), dgIdpi(jp),
c     $                                dgIdpi(ip)*dgIdpi(jp)
c                              endif
c%debug
 70                     continue
                     endif
c     
c-sd                        if (lsdref) then
c-sd                           ipaddr(ii) = 0
c-sdc     contribution from d sdI / dp
c-sd                           gradnt(ii) = gradnt(ii) +
c-sd     $                          dshldp(k,l) * (- 1.0/sdffrf(l) +
c-sd     $                          (wtrf(l)/sdffrf(l)) * delI**2)
c-sd                           if (tdgIdp .ne. 0.0 .or.
c-sd     $                          dshldp(k,l) .ne. 0.0) then
c-sd                              ipaddr(ii) = ip
c-sd                              p2 = dshldp(m,l) * dgIdpi(ip)
c-sd                              do 71, jj = 1,ii
c-sd                                 m = ipnloc(jj)
c-sd                                 jp = ipaddr(jj)
c-sd                                 if (jp .eq. 0) then
c-sd                                    p1 = p2 
c-sd                                 else
c-sd                                    p1 = dshldp(k,l) * dgIdpi(jp) + p2
c-sd                                 endif
c-sd                                 amatrx(ii,jj) = amatrx(ii,jj) +
c-sd     $                            wtrf(l) * dshldp(k,l) * dshldp(m,l) *
c-sd     $                                (3.0 * wtrf(l) * delI**2 - 1.0) +
c-sd     $                            2.0 * (wtrf(l)/sdffrf(l)) * delI * p1
c-sd 71                           continue
c-sd                           endif
c-sd                        endif
 60               continue
               endif
 50         continue
            ninref = ninref+1
         endif
c     
 90      if (.not. eof) go to 20
c     
c     
cooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c     End of all reflections, solve equations
c     
         continue
c  Add in Ties, ie restraints on scales
         Rtie = 0.0
         if (nties .gt. 0) then
            do 102, l = 1,nties
c  The first ntiexy ties are in the detector plane (for different variances),
c then on z (rotation, then surface, then bfactor
               if (l .le. ntiexy) then
                  ltietp = 1
                  w = 1.0 / vtiexy
               elseif (l .le. ntiexy + ntiez) then
                  ltietp = 2
                  w = 1.0 / vtiez
               elseif (l .le. ntiexy + ntiez + ntiesrf) then
                  ltietp = 3
                  w = 1.0 / vtiesrf
               elseif (l .le. ntiexy + ntiez + ntiesrf + ntiebf) then
                  ltietp = 4
                  w = 1.0 / vtiebf
               else 
                  ltietp = 5
                  w = 1.0 / vtieA1
               endif
               if (ltietp .ne. jties(4,l)) then
c  Wrong tie type: shouldn't happen
                  write (lunout, '(/a/)')
     $               ' **** ERROR: wrong tie type ****'
                  call dna_error('internal error')
                  call ccperr(1,'*** Wrong tie type: program bug ***')
               endif
               irun = jties(3,l)
               i = jties(1,l)
               j = jties(2,l)
               if (ltietp .le. 2) then
                  call unploc(irun, i, k, m, i1, k1)
                  call unploc(irun, j, k, m, i2, k2)
                  delC = (cscals(i1,k1,irun) - cscals(i2,k2,irun))
                  gradnt(i) = gradnt(i) - delC * w
                  gradnt(j) = gradnt(j) + delC * w
c         Note i .lt. j
                  amatrx(i,i) = amatrx(i,i) + w
                  amatrx(j,j) = amatrx(j,j) + w
                  amatrx(i,j) = amatrx(i,j) - w
               elseif (ltietp .eq. 3) then
c  Surface parameter
c    local pointer to coefficient array
                  k = ipnloc(i)
                  if (k .lt. 0) then
                     k = iplloc(-k,irun)
                  endif
                  i1 = k - ipnsph(irun) + 1
                  delC = sphclm(i1, irun)
                  gradnt(i) = gradnt(i) - delC * w
                  amatrx(i,i) = amatrx(i,i) + w
               elseif (ltietp .eq. 4) then
c  Bfactor
                  call unploc(irun, i, k, i1, m, k1)
                  call unploc(irun, j, k, i2, m, k2)
                  delC = (bfactr(1, i1, irun) - bfactr(1, i2, irun))
                  gradnt(i) = gradnt(i) - delC * w
                  gradnt(j) = gradnt(j) + delC * w
c         Note i .lt. j
                  amatrx(i,i) = amatrx(i,i) + w
                  amatrx(j,j) = amatrx(j,j) + w
                  amatrx(i,j) = amatrx(i,j) - w
               elseif (ltietp .eq. 5) then
c  Tails A1 parameter
                  k = ipnloc(i)
                  if (k .lt. 0) then
                     k = iplloc(-k,irun)
                  endif
                  delC = partds(3,irun) - ttieA1(irun)
                  gradnt(i) = gradnt(i) - delC * w
                  amatrx(i,i) = amatrx(i,i) + w
               endif                  
c
               Rtie = Rtie + (delC*w)**2
 102        continue
c
         endif
c
         do 108, irun = 1, numrun
            summ(1,irun) = 0.0
            summ(2,irun) = 0.0
            summ(3,irun) = 0.0
            summ(4,irun) = 0.0
            nsum(1,irun) = 0
            nsum(2,irun) = 0
            nsum(3,irun) = 0
            nsum(4,irun) = 0
            do i=1,4
               ubss(i,irun) = 0.0
            enddo
 108     continue
         sumd = 0.0
         nsumd = 0
c  Get scales UBS for matrix, to put B-factor terms on similar scale
c  to scale terms
         do 110, i = 1,nparcy
            if (nrfc(i) .gt. 0) then
c  iptyp(i) = 1 for B-factor parameter, = 2 scale parameter,
c  = 3 diffuse correction = 4 surface correction, = 5 sd correction parameter
c  Only average scale for types 1 & 2, but separately for each run
               irun = ipnrun(i)
               if (iptyp(i) .le. 2 .or. iptyp(i) .eq. 4) then
                  summ(iptyp(i),irun) =
     $               summ(iptyp(i),irun) + amatrx(i,i)
                  nsum(iptyp(i),irun) = nsum(iptyp(i),irun) + 1
               endif
            endif
 110     continue
         do 112, irun = 1, numrun
            if (nsum(1,irun) .gt. 0) then
               ubss(1,irun) =
     $            sqrt(summ(1,irun)/float(nsum(1,irun)))
            endif
            if (nsum(2,irun) .gt. 0) then
               ubss(2,irun) =
     $            sqrt(summ(2,irun)/float(nsum(2,irun)))
            endif
            if (nsum(4,irun) .gt. 0) then
               ubss(4,irun) =
     $            sqrt(summ(4,irun)/float(nsum(4,irun)))
            endif
 112     continue
c     
         if (kprint(1) .ge. 3) then
            write (lunout, '(a,i4)') ' Number of parameters:', nparcy
            write (lunout, '(a)') ' Sample of matrix before scaling:'
            do 111, i = 1, min(10, nparcy)
               write(lunout,'(1x,10g10.3)')
     $            (amatrx(i,j),j=1,min(10,nparcy))
 111        continue
            write (lunout, '(a)') ' Gradient before scaling:'
            write (lunout,'(100(1X,10G10.2/))') (gradnt(i),i=1,nparcy)
         endif
c     
         lfail = .false. 
c     Divide matrix & rhs by UBS = sqrt(matrix diagonal)
         do 120, i = 1,nparcy
            if (iptyp(i) .le. 2 .or. iptyp(i) .eq. 4) then
               ubs(i) = ubss(iptyp(i),ipnrun(i))
            else
               if (amatrx(i,i) .gt. 0.0) then
                  ubs(i) = sqrt(amatrx(i,i))
               else
                  ubs(i) = 1.0
               endif
            endif
            if (ubs(i) .eq. 0.0) then
               write (lunout, '(a,i6,a,i6)')
     $            ' No observations for parameter',i,' run',
     $            irunum(ipnrun(i))
               lfail = .true.
            else
               gradnt(i)= gradnt(i)/ubs(i)
c Copy to other triangle: here j .le. i, we have built A(jj,ii), jj .le.ii
c for speed
               do 125, j = 1,i
                  amatrx(i,j) = amatrx(j,i)/(ubs(i)*ubs(j))
                  if (j .ne. i) amatrx(j,i) = 0.0
 125           continue
               sumd = sumd + amatrx(i,i)
               nsumd = nsumd + 1
            endif
 120     continue 
c
         if (lfail) then
            write (lunout, '(/1x,a,i8)')
     $         'Total number of observations read = ', nrdref
            write (lunout, 6120) nwsrej, nolrej
 6120       format(' Whole reflections rejected (all observations):'/
     $         9x,I8,4x,'too weak'/
     $         9x,I8,4x,'outliers'/)
            call ccperr(1,'*** No observations ***')
         endif
c
         if (kprint(1) .ge. 3) then
c     Debug
            write (lunout, '(a)') 'Diagonal scaling matrix'
            write (lunout, '(10g10.3)') (ubs(i),i=1,nparcy)
            write (lunout, '(a, f12.5)')
     $         ' Mean diagonal of scaled matrix',sumd/float(nsumd)
            write (lunout, '(a)') ' Sample of matrix after scaling:'
            do 121, i = 1, min(10, nparcy)
               write(lunout,'(1x,10g10.3)')
     $            (amatrx(i,j),j=1,min(10,nparcy))
 121        continue
            write (lunout, '(a)') ' Scaled gradient:'
            write(lunout,'(100(1X,10G10.2/))') (gradnt(i),i=1,nparcy)
c     
         endif
c
c---- Compress normal matrix amatrx & gradient vector gradnt to  remove
c     any parameters which have no contributions (nrfc(i) = 0)
         call cmpmvc
     $      (maxmtx, amatrx, gradnt, nrfc, nparcy, nparcm, iptpar)
c The new number of parameters is in nparcm
         if (nparcm .ne. nparcy) then
c  Some parameters have no contributions, but this can be valid if they
c are FIXed: check this
            call chkfix(iptpar, nparcy, istat)
            if (istat .ne. 0) then
               write (lunout, '(/a/2(a,i6/)/)')
     $            ' ----| Some parameters have no contributions |----',
     $            '       Total number of parameters: ',nparcy,
     $            '  Number of determined parameters: ',nparcm
c  List them
               k = 0
               do 128, i = 1,nparcy
                  if (nrfc(i) .le. 0) then
                     k = k+1
                     nrfc0(k) = i
                  endif
 128           continue
c  Use names if set (ie after 1st cycle)
               if (parnam(nrfc0(1)) .eq. ' ') then
                  write (lunout, '(/a/100(1x,20i4/))')
     $               ' List of parameter numbers with no contribution:',
     $               (nrfc0(i), i=1,k)
               else
                  write (lunout, '(/a/1000(1x,i4,3x,a/))')
     $               ' List of parameter numbers with no contribution:',
     $               (nrfc0(i), parnam(nrfc0(i)), i=1,k)
               endif
c
c               write (lunout, '(1x,i8,a,i8,a)')
c     $            nref, ' observations from ', ninref,
c     $            ' independent reflections'
c               write (lunout, 6140) nwsrej, nolrej
c               if (nstrej(4)+nstrej(5)+nstrej(6)+nstrej(7)+
c     $            nstrej(8)+nstrej(12) .gt. 0)
c     $            write (lunout, '(a)') ' Observations rejected:'
c               if (nstrej(4) .gt. 0)
c     $            write (lunout, 6141) nstrej(4),'overload'
c               if (nstrej(5) .gt. 0)
c     $            write (lunout, 6141) nstrej(5),'resolution'
c               if (nstrej(6) .gt. 0)
c     $            write (lunout, 6141) nstrej(6),'zero input scale'
c               if (nstrej(7) .gt. 0)
c     $            write (lunout, 6141)
c     $            nstrej(7),'zero standard deviation'
c               if (nstrej(8) .gt. 0)
c     $            write (lunout, 6141) nstrej(8),'M flag .gt. 2'
c               if (nstrej(12) .gt. 0)
c     $            write (lunout, 6141) nstrej(12),'too strong'
cc               call ccperr(1,'*** Some parameters undetermined ***')
            endif
         endif
         if (nref .lt. nparcy) then
            write (lunout, '(/a/1x,i10,a,i10,a/)')
     $         ' **** Insufficient data to determine parameters ****',
     $         nref, ' reflections, ', nparcy, ' parameters'
            call ccperr(1,'*** Too few reflections ***')
         endif
c     
c     Get eigenvalues & eigenvectors
cDP         call ea06c(
cDP     $        amatrx, eigval, eigvec, nparcm, maxmtx, maxmtx, work)
c  Now double-precision
c         call ea06cd(
c     $      amatrx, eigval, eigvec, nparcm, maxmtx, maxmtx, work)
c
       call dsyev('V', 'L', nparcm, amatrx, maxmtx, eigval, work,
     $   5*maxmtx, istat)
       if (istat .ne. 0) then
          call ccperr(1,'*** Error in refscl/dsyev ***')
       endif
       do j = 1, nparcm
          do i = 1, nparcm
             eigvec(i,j) = amatrx(i,j)
          enddo
       enddo
c     
c
c---- Find two smallest eigenvalues
c
         vmin1 = 100000.0
         vmin2 = 100000.0
         i1 = 1
c
         do 129, i = 1,nparcm
            vali = eigval(i)
            if (vali.le.vmin1) then
               i2 = i1
               vmin2 = vmin1
               vmin1 = vali
               i1 = i
            else if (vali.le.vmin2) then
               vmin2 = vali
               i2 = i
            end if
 129     continue
c---- Now I1 points to smallest eigenvalue, I2 to next one
c---- Always filter smallest eigenvalue: filter 2nd if all runs are refining
c     B-factors
         if (lfilt1) then
c  Filter 1 only
            i2 = i1
         endif
c     
         if (icycle .le. ncycdamp .or. ncycdamp .lt. 0) then
            dampnow = damp
         else
            dampnow = 0.0
         endif
c     Set inverse vector of eigenvalues. Suppress very small eigenvalues,
c     & inflate others
         nfilter = 0
         eigmin = 1.0e+10
         eigflt = -1.0e+10
         do 130, i = 1, nparcm
            eigmin = min(eigmin, eigval(i))
            if (i .eq. i1 .or. i .eq. i2
     $         .or. eigval(i) .lt. filter) then
               nfilter = nfilter+1
               vecinv(i) = 0.0
               eigflt = max(eigval(i), eigflt)
            else
               if (abs(eigval(i) + dampnow) .gt. 0.000000001) then
                  vecinv(i) = 1.0/(eigval(i) + dampnow)
               else
                  vecinv(i) = 0.0
               endif
            endif
 130     continue
c     
c     Get transformed shift vector TRSHFT = L**-1 . AT. Gradnt
c     L = vecinv = inverse eigenvalues
c     AT = transposed matrix of eigenvectors
c     (ie rows of AT are eigenvectors)
c     
         do 140, j = 1, nparcm
            trshft(j) = 0.0
            do 145, i = 1, nparcm
               trshft(j) = trshft(j) + eigvec(i,j)*gradnt(i)
 145        continue
            trshft(j) = trshft(j)*vecinv(j)
 140     continue
c     
c     Rotate back by eigenvector matrix, & rescale by UBS
         convrg = .true.
         do 148, j= 1, nparcy
            shift(j) = 0.0
            sdpar(j) = 0.0
            sdpars(j) = 0.0
 148     continue
c
c     Factor for variance: Sum(w del**2)/(m-n)
         if (nref-nparcy .le. 0) then
            write (lunout, '(/a,i6,a,i6,a/a//)')
     $         ' **** You are trying to refine ', nparcy,
     $         ' parameters with ', nref, ' observations ****',
     $         ' **** This doesn''t work ****'
            call ccperr(1,'*** Too few observations ***')
         endif
         varfac = sumwdl/float(nref-nparcy)
c Mean and maximum fractional shift
         shfmen = 0.0
         shfmax = -1000.
         kshfmx = 0
c     
         do 150, j = 1, nparcy
            k = iptpar(j)
            if (k .gt. 0) then
               do 155, i = 1, nparcm
                  sdpar(j) = sdpar(j) +
     $               vecinv(i)*eigvec(k,i)*eigvec(k,i)
                  shift(j) = shift(j) + trshft(i)*eigvec(k,i)
 155           continue
c
               shift(j) = shift(j)/ubs(j)
               sdpar(j) = sqrt(sdpar(j)*varfac) / ubs(j)
c Store sd for refout & set flag
               sdpars(j) = sdpar(j)
               lsdpars = .true.
c     
            else
               shift(j)  = 0.0
               sdpar(j)  = 0.0
               sdpars(j) = 0.0
            endif
 150     continue
c    

         call dna_table_start('refinement_cycle')
         call dna_ilist_start('cycle', icycle)
         call dna_integer_item('cycle', icycle)
         call dna_real_item('residual_sum', sumwdl)
         call dna_real_item('restraint_residual', rtie)
         call dna_real_item('total_residual', resid)
 
         write (lunout,6130)
     $      '  =====  Cycle ', icycle,'  =====',
     $      ' Residual Sum( w Del**2) =',sumwdl,
     $      ', restraint residual =', Rtie,
     $      ', total residual = ', resid,
     $      ' Sum( w Del**2) / (m-n) = ', varfac,
     $      nfilter, ' eigenvalues filtered out, smallest =',
     $      eigmin, ', largest filtered = ', eigflt,
     $      'Damping factor = ', dampnow
 6130    format(
     $      //a,i5,a/a,g15.3,a,g15.3,a,g15.3/
     $      a,g15.4/1x,i4,a,g12.3,a,g12.3,3x,a,f7.3)
c     
         if (kprint(1) .ge. 3) then
c     Debug
            write (lunout, '(a)') ' Eigenvalues:'
            write(lunout,'(100(1X,10G12.3/))') (eigval(i),i=1,nparcm)
         endif

         call dna_integer_item('observations', nref)
         call dna_integer_item('independent_reflections', ninref)
         call dna_integer_item('rejected_too_weak', nwsrej) 
         call dna_integer_item('rejected_outlier', nolrej)

         write (lunout, '(1x,i8,a,i8,a)')
     $      nref, ' observations used from ', ninref,
     $      ' independent reflections'
         write (lunout, 6140) nwsrej, nolrej
 6140    format(' Whole reflections rejected (all observations):'/
     $      9x,I8,4x,'too weak'/
     $      9x,I8,4x,'outliers'/)
         if (nstrej(4)+nstrej(5)+nstrej(6)+nstrej(7)+
     $      nstrej(8)+nstrej(12) .gt. 0)
     $      write (lunout, '(a)') ' Observations rejected:'
 6141    format(9x,i8,4x,a)
         if (nstrej(4) .gt. 0)
     $      write (lunout, 6141) nstrej(4),'overload'
         if (nstrej(5) .gt. 0)
     $      write (lunout, 6141) nstrej(5),'resolution'
         if (nstrej(6) .gt. 0)
     $      write (lunout, 6141) nstrej(6),'zero input scale'
         if (nstrej(7) .gt. 0)
     $      write (lunout, 6141) nstrej(7),'zero standard deviation'
         if (nstrej(8) .gt. 0)
     $      write (lunout, 6141) nstrej(8),'M flag .gt. 2'
         if (nstrej(12) .gt. 0)
     $      write (lunout, 6141) nstrej(12),'too strong'
         if (nrfrjem .gt. 0)
     $      write (lunout, 6141) nrfrjem, 'Emax limit'
c     
c     Apply shifts
         call parupd(lsdref,shift,sdpar,parold,parnew,parnam,istat)
c      if any scale has gone negative, parameters are left unchanged
         if (istat .lt. 0) then
            if (ncycdamp .eq. 0) then
c  No damping set yet, so try setting some
               nextracycles = 2
               damp = 0.05
               ncycdamp = icycle + nextracycles
c  increase number of cycles
               ncycle = ncycle + nextracycles
               write (lunout, '(a,i3,a,i3,a/)')
     $            ' **** Damping switched on for',nextracycles,
     $            ' cycles, number of cycles increased by',
     $            nextracycles, ' ****'
               convrg = .false.
               go to 10
            else
c  Already damping shifts, give up

               write (lunout, '(/a/a/)')
     $            ' **** ERROR: damping has failed to prevent negative',
     $            ' scale factors ****',
     $            ' **** Giving up'
               call ccperr(1,'**** Negative scales ****')
            endif
         endif
c      force continuation if parameter reset
         if (istat .gt. 0) then
            convrg = .false.
            lreset = .true.
         else
            lreset = .false.
         endif
c
         ii = 0
         do 159, j = 1, nparcy
c     --- Check for convergence
            if (sdpar(j) .ne. 0.0) then
               frcshf = abs(shift(j)/sdpar(j))
            else
               frcshf = 0.0
            endif
            if (frcshf .gt. cnvfac) then
               convrg = .false.
            endif
            if (frcshf .gt. shfmax) then
               kshfmx = j
            endif
            shfmax = max(shfmax, frcshf)
            shfmen = shfmen + frcshf
            ii = ii + 1
 159     continue
         if (ii .gt. 0) shfmen = shfmen/float(ii)

         call dna_real_item('mean_shift_sd', shfmen)
         call dna_real_item('max_shift_sd', shfmax)

         call dna_list_end
         call dna_table_end

         write (lunout, 6131) shfmen, shfmax, kshfmx, parnam(kshfmx)
 6131    format (/' Mean and maximum shift/sd : ',2f10.3,
     $      ' (parameter ',i5,3x,a,')'/)
c     
c Print shifts on last cycle or if PRINT FULL
         if (icycle .ge. ncycle .or. convrg .or.
     $      kprint(1) .ge. 2) then
            write (lunout, '(//a,i5)')
     $         '=== Shifts for cycle',icycle
            do 160, irun = 1, numrun
               write (lunout, '(//a,i6)') ' Run number', irunum(irun)
c Set up pointers to global parameters for this run, allowing for linked
c parameters
               k1 = 10000
               k2 = 0
               do 162, k = 1, nprrun(irun)
                  if (ilnkrn(k, irun) .ge. 0) then
c this parameter not linked back
                     k1 = min(ipoint(k,irun), k1)
                     k2 = max(ipoint(k,irun), k2)
                  endif
 162           continue
               j = 0
c  Next statement begins a loop for 1) B & scale parameters 2) sd factors
c     -+-+
 165           continue
               i1 = k1
c     Page shifts
 161           if (i1 .le. k2) then
                  i2 = min(k2, i1+9)
                  write (lunout, 6200) (parnam(i), i=i1,i2)
 6200             format((/' Parameter:  ',10(2X,A8)))
                  write (lunout, 6210)
     $               'Old value:',(parold(i), i=i1,i2)
 6210             format(1X,A,10F10.3)
                  write (lunout, 6210)
     $               'Shift    :',(shift(i),  i=i1,i2)
                  write (lunout, 6210)
     $               'New value:',(parnew(i), i=i1,i2)
                  write (lunout, 6210)
     $               'Error    :',(sdpar(i), i=i1,i2)
                  write (lunout, 6220)
     $               'Number   :',(nrfc(i), i=i1,i2)
 6220             format(1X,A,10I10)
                  i1 = i2+1
                  go to 161
               endif
c     For sd factor refinement, now print these shifts
               if (lsdref .and. j .eq. 0) then
                  j = 1
                  k1 = ipoint(nprrun(irun)+1, irun)
                  k2 = ipoint(npmrun(irun), irun)
                  go to 165
c     -+-+
               endif
 160        continue
         endif
 200     continue
c     
c Dump scales
         if (ldump) call dumpsc
c
c Update store of mean intensity
         if (nsumin .gt. 0) avgins = sumins/float(nsumin)
         m = 0
         do k = 1, maxres
            simnsa(k) = 0.0
            swtsa(k) = 0.0
         enddo
         do j = 1, ndtsetsout
            do k = 1, maxres
               if (nimnsr(k,j) .gt. 0) then
c    Mean <>/epsilon for normalization
                  qimnsr(k,j) = simnsr(k,j)/swtsr(k,j) 
                  m = max(m, k)
                  simnsa(k) = simnsa(k) + simnsr(k,j)
                  swtsa(k)  = swtsa(k)  + swtsr(k,j)
               else
                  qimnsr(k,j) = 0.0 
               endif
            enddo
         enddo
         do k = 1, m
            if (swtsa(k) .gt. 0.0) then
               qimnsm(k) = simnsa(k)/swtsa(k)
            else
               qimnsm(k) = 0.0
            endif
c            write (lunout, '(25f8.0)')
c     $         (qimnsr(k, irun), irun = 1, numrun)
         enddo
c Overall unscaled intensity mean
         qsisu = 0.0
         nqsisu = niunsc
         if (niunsc .gt. 0) qsisu = siunsc/float(niunsc)
c
         lqmnir = .true.
c Update IMID if necessary on first cycle
         if (imid .eq. +1) then
            imid = max(+1, nint(qsisu))
            write (lunout, '(/a,a,i8/)')
     $         'COMBINE intensity option: IMID updated to mean',
     $         '  unscaled intensity ', IMID
         endif
c     End cycle loop
         if (nerpdc .gt. 0) lerpdc = .false.
         go to 10
      endif
c     
      if (.not. convrg) then
         write (lunout, 6900)
 6900    format(//2(1x,8('****')/))
         write (lunout, 6901) shfmen, shfmax, parnam(kshfmx)
 6901    format ('  WARNING WARNING WARNING '//
     $        ' Scaling has not converged!!'/
     $        ' Mean and maximum shift/sd : ',2f10.3,
     $        ' on parameter ',a)
         write (lunout, 6900)
      endif
c
      if (lreset) then
         write (lunout, 6900)
         write (lunout, 6910)
 6910    format(' ERROR ERROR ERROR '/
     $        ' Parameter was reset on last cycle, ',
     $        ' scaling has not converged')
         write (lunout, 6900)
         call ccperr(1,'*** Scaling has failed to converge ***')
      endif
      nparam = nparcy
c     
c     +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+
c     
c Print scale factors
      call prtscl
c
c Close SCALES file
      if (ldump) close (unit=lundmp)
c
c
      end
c
c
c
      subroutine rejget(nrs, isttrf, iserrf, irjgar, nrfgar, nrejg)
c     ============================================================
c
c Set outlier rejection flag for observation flagged as rejected in
c global outlier rejection array
c
c On entry:
c  nrs           number of observations
c  isttrf(nrs)   status flag
c  iserrf(nrs)   serial number in file for this observation
c  irjgar        flag array for all observations in file
c                =  0  accept,  .ne. 0 reject
c  nrfgar        array length
c
c On exit:
c  isttrf(nrs)   updated status flag
c  nrejg         number rejected
c
      integer nrs, nrfgar, nrejg      
      integer isttrf(nrs), iserrf(nrs), irjgar(nrfgar)
c
      include 'flags.fh'
c
      integer i
c
      nrejg = 0
      do i = 1, nrs
         if (irjgar(iserrf(i)) .ne. 0) then
            isttrf(i) = lqoutl
            nrejg = nrejg + 1
         endif
      enddo
c
      return
      end
      subroutine rfltbl(nrfgar, ningar, iingar, nregar, aregar)
c    =================================================
c
c
c   Set reflection flags & tabulate deviations, calculate various 
c useful means etc
c   Go through file in similar way to s/r refout
c
c On entry:
c  nrfgar        number of observations in file
c  iingar        integer work array, unset
c  aregar        real work array, unset
c
c On exit:
c  iingar(nrfgar,1)    reflection flag
c                      =  0  not used
c
c
      implicit none
      integer nrfgar, ningar, nregar
      integer iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
c
      include 'parameter.fh'
      include 'outcon.fh'
      include 'anomtc.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'rfile.fh'
      include 'refcon.fh'
      include 'rundef.fh'
      include 'means.fh'
      include 'sdfacc.fh'
      include 'glocon.fh'
      include 'errors.fh'
      include 'datasets.fh'
      include 'flags.fh'
      include 'column.fh'
      include 'timecor.fh'
      include 'sdcdata.fh'
c
c     Reflection stores
      real ffrf(maxrfl), wtrf(maxrfl),
     $   sdffrf(maxrfl), timerf(maxrfl), frclrf(maxrfl),
     $   xdrf(maxrfl), ydrf(maxrfl), zdrf(maxrfl),
     $   ghl(maxrfl), dghldc(maxpmr, maxrfl), sdglrf(maxrfl),
     $   ffprrf(maxrfl), sdfprf(maxrfl), clprf(maxrfl),
     $   sclirf(maxrfl), bgpkrf(maxrfl), 
     $   sdscrf(maxrfl), delrf(maxrfl), xsirf(maxrfl),
     $   phirf(maxrfl), xdetrf(maxrfl), ydetrf(maxrfl),
     $   Dfhl(maxrfl), datarf (maxcol, maxrfl), eorf(maxrfl)
      integer iserrf(maxrfl), irunrf(maxrfl), ibatrf(maxrfl),
     $   isttrf(maxrfl), mrf(maxrfl), isymrf(maxrfl), mprf(maxrfl),
     $   lflgrf(maxrfl), nmprrf(maxrfl), jmprrf(maxrfl), lanmtc(maxrfl),
     $   kmptrf(maxrfl)
      logical useflg(maxrfl)
      real
     $   danomdts(maxsets), sddanomdts(maxsets),
     $   disodts(maxsets), sddisodts(maxsets), fdts(maxsets)
      real
     $   qihdts(maxsets), qsdidts(maxsets),
     $   qihpl(maxsets), qsdipl(maxsets),
     $   qihmn(maxsets), qsdimn(maxsets)
c
c  Sums for means etc
      real    simnmn(maxres), swtimn(maxres), sdlimn(maxres, maxinb),
     $   sde2mn(maxres, maxrun), sdelmn(maxres, maxrun), sdl2mn(maxres),
     $   svarmn(maxres, maxrun), sdi2mn(maxres,maxrun), sstlmn(maxres),
     $   sddelr(maxres, maxrun)
      double precision sdelis(maxres, maxsets), sdelid(maxsets), sumins
      integer nimnmn(maxres), ndlimn(maxres,maxinb), ndl2mn(maxres),
     $   nrefmn(maxres, maxrun), nrdlmn(maxres, maxrun),
     $   nsdelis(maxres, maxsets), nsdelid(maxsets), nins
c Histogram
      integer maxchs
      parameter (maxchs=30)
      real    delmax
      parameter (delmax = 8.0)
      integer mbn_del, nchbin(-maxchs:+maxchs, maxrun)
      real    delbin
c
      integer maxdhs
      parameter (maxdhs = 20)
      integer ndlbin(-maxdhs:+maxdhs)
      real    dhsbin
c
      integer i, j, k, l, icntrc, nrej, krej2, nupls, numns, nuall,
     $   idts, kpass
      integer jhkl(3), ninref, nrs, nfull, istat, nuse, mbn_s,
     $   nsrmax, irun, irfidx, janom, mbn_i, nrejpt(6), 
     $   iselds, nomit, nomitu, nanmrj, isysab, kwgtyp,
     $   mflcnt(2,maxmfl), niunsc, nfulls(maxrun), npartial(maxrun)
      real    ss, stol, Thl, Chl, Shl, swih, swihpl, swihmn,
     $   aihpls, sdihpl, uh, aihmns, sdihms, aih, sdih, aihl, sdihl,
     $   delI, anorat, delr, slpanom(2), anomfac,
     $   epsiln, wt, flmmax(3,2), siunsc, swihc
      logical eof, lpart, lprint, ldtrun(maxrun), lrejct, lomit,
     $   lanom
c

c  One pass only
      kpass = 1
 10   continue

c Variance weights
      kwgtyp = 2
c 
      nerpdc = 0
      lprint = .false.
      lrejct = .false.
c
c *** Means invalid if zero-dose correction is used
      if (ktimcor .ne. 0) lqmnir = .false. 
c Flag for bad agreement with 2 observations
      krej2 = krej2o
      nanmrj = 0
      nsrmax = 0
      delbin = delmax/float(maxchs)
      dhsbin = 2.0
      nrejpt(1) = 0
      nrejpt(2) = 0
      nrejpt(3) = 0
      nrejpt(4) = 0
      nrejpt(5) = 0
      nrejpt(6) = 0
      siunsc = 0.0
      niunsc = 0
c
      do 1, irun = 1, numrun
         ldtrun(irun) = .false.
 1    continue
c--- Clear flags
      call ccpzi(iingar, nrfgar)
c
c--- Clear histograms, means etc
      call ccpzr(simnmn, maxres)
      call ccpzr(swtimn, maxres)
      call ccpzi(nimnmn, maxres)
      call ccpzr(sdl2mn, maxres)
      call ccpzi(ndl2mn, maxres)
      call ccpzr(sdlimn, maxres*maxinb)
      call ccpzi(ndlimn, maxres*maxinb)
      call ccpzi(nrefmn, maxres*maxrun)
      call ccpzi(nrdlmn, maxres*maxrun)
      call ccpzr(sde2mn, maxres*maxrun)
      call ccpzr(sdelmn, maxres*maxrun)
      call ccpzr(sdi2mn, maxres*maxrun)
      call ccpzi(nchbin, (2*maxchs+1)*maxrun)
      call ccpzi(ndlbin, 2*maxdhs+1)
      call ccpzr(svarmn, maxres*maxrun)
      call ccpzr(sstlmn, maxres)
      call ccpzi(mflcnt, 2*maxmfl)
      call ccpzr(flmmax, 3*2)
      call ccpzi(nsdelis, maxres*maxsets)
      call ccpzi(nsdelid, maxsets)
      call clr_tcr
      do j = 1, maxsets
         do i = 1, maxres
            sdelis(i,j) = 0.0
         enddo
         sdelid(j) = 0.0
      enddo
      do i=1,maxrun
         nfulls(i) = 0
         npartial(i) = 0
      enddo
      sumins = 0.0
      nins = 0
c
c=====================================================================
      call lrrewd(1)
c  Tell gethkl that we're starting to read the file
      eof = .true.
      nomit  = 0
      nomitu = 0
      ninref = 0
      irfidx = 0
c     
c  Loop all reflections
coooooooooooooooooooooooooooooooooooooooooooooooooooooooo
c     
c   Get all observations of this reflection
 300  call gethkl(maxrfl, ninref, lrefou, +1, .false.,
     $   eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $   ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $   isymrf, sclirf, sdscrf, lflgrf, bgpkrf, frclrf, clprf, isttrf,
     $   timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $   nmprrf, datarf, istat)
      if (nrs .le. 0) go to 390
c  Resolution limit test
      if (istat .ne. 0) go to 300
c  Count FLAGs, and accept if required
      if (ncflag .gt. 0) then
         call tstmfl(nrs, lflgrf, bgpkrf, isttrf, mrf,
     $      mflcnt, flmmax)
      endif
c  Centric if icntrc = 1, else = 0
      call centr(jhkl, icntrc)
c  Epsilon class: isysab = 1 for systematic absences
      call epslon(jhkl,epsiln,isysab)
c     Get scale for all observations, & their derivatives with respect to
c     the scale factors
      do 325, l=1,nrs
         call sclder(irunrf(l),
     $      stol, timerf(l), xdrf(l), ydrf(l), zdrf(l),
     $      ghl(l), Thl, Chl,  Shl, dghldc(1,l))
         Dfhl(l) = 1.0
 325  continue
c
c     Process partials if required (using selected I), update nuse
c     Here, nuse includes fulls & partials
      call partls(.false.,
     $   jhkl, kprout, lpart, stol, nrs, nprtds, ltdsrn,
     $   ffrf, sdffrf, ffprrf, sdfprf,
     $   ghl, Dfhl, xdrf, ydrf, zdrf, phirf, xsirf,
     $   frclrf, kmptrf, isttrf, mrf, mprf, isymrf, ibatrf, irunrf, 
     $   nmprrf, jmprrf, kprflo, kprmso, mxprwo, imid,
     $   nrejpt, nuse, prthou, prlimo, prmaxo, dghldc, .false.,
     $   lavpsc)
c
c     No correction of  sds for selected I
c     
      lrejct = .false.
c Set variance weights
      call setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c ***
c Extrapolate to zero dose if requested
      if (ktimcor .ne. 0) then
c  We don't need to select datasets (runs) here because inactive
c runs have already been removed by chkref/runget, so iselds = 0
         iselds = 0
         if (kanomo .ne. 0) then
c  Extrapolate I+ & I- separately (anom on)
            janom = 1
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, iselds, .false., useflg)
            call timcor
     $         (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
c  
            janom = 2
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, iselds, .false., useflg)
            call timcor
     $         (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
         else
            janom = 0
            call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $         irefrn, iselds, .false., useflg)
            call timcor
     $         (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
         endif
      endif
c ***
c Anomalous match option: flag unmatched observations as rejected
      call selanm(icntrc, nrs, kanomo, jhkl, irunrf, mrf, isttrf,
     $      isymrf, phirf, timerf, lanmtc, nanmrj)
c
c    mbn_s is resolution bin number
      mbn_s = min(int((ss-0.0000001)/srange) + 1, maxres)
c
      if (lqmnir) then
         call chemax(nrs, .false., 
     $      icntrc, epsiln,
     $      jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $      isymrf, ibatrf, mbn_s,
     $      eorf, nrej, istat)
      else 
         nrej = 0
      endif
c
c Calculate the mean intensity aih
c Exclude reference run from means
      iselds = 0
      janom = 0
      call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $   irefrn, iselds, .false., useflg)
      call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $   wtrf, aih, sdih, uh, nuall)
c
c Store accepted (and summed|scaled partials) I, sd(I), ghl
c  in arrays for later analysis
c Array contains only accepted observations
c  iingar(,1)  (a) for first observation in each reflection,
c                  = number of observations (negated if I-)
c              (b) for subsequent observations = +/- 1
c              .gt. 0 for I+ (or centric), .lt. 0 I-
c  iingar(,2) packed run number, partial status, centric flag 
c
c  aregar(,1)  Ihl
c  aregar(,2)  sd(Ihl)
c  aregar(,3)  ghl  scale
c  aregar(,4)  LP
c
      k = 0
      do 326, l = 1,nrs
         if (isttrf(l) .ge. lqOK) then
            irfidx = irfidx+1
            if (k .eq.  0) then
               j = irfidx
            endif
            k = k+1
c I+ or I-
c  Centric if icntrc = 1, else = 0
c  I+ if isym odd, I- even
            if (icntrc .gt. 0 .or. mod(isymrf(l), 2) .eq. 1) then
c  Centric or I+
               iingar(irfidx,1) = +1
            else
c  Acentric and I-
               iingar(irfidx,1) = -1
            endif
c  Pack run number, partial status (full, summed|scaled partial),
c centric flag into iingar(,2) array
            call pckflg(
     $         irunrf(l), mrf(l), nmprrf(l), icntrc, iingar(irfidx,2))
c  
            if (mrf(l) .eq. 0) then
c  Full
               nfulls(irunrf(l)) = nfulls(irunrf(l)) + 1
            else 
c  Partial
               npartial(irunrf(l)) = npartial(irunrf(l)) + 1
            endif
c
            aregar(irfidx,1) = ffrf(l)
            aregar(irfidx,2) = sdffrf(l)
            aregar(irfidx,3) = ghl(l)
            aregar(irfidx,4) = clprf(l)
c+DEBUG
c         if (irfidx .lt. 100) then
c            write (13,'(i7,2f9.1,2f9.3,3i4)')
c     $         irfidx,(aregar(irfidx,i),i=1,4), irunrf(l), mrf(l),
c     $         nmprrf(l)
c         endif
c-DEBUG

c
            siunsc = siunsc + ffrf(l)
            niunsc = niunsc + 1
c Maximum scaled unmerged intensity
            sclimax = max(sclimax, ffrf(l)/ghl(l))
         endif
 326  continue
c  Store observation count in first slot (which now contains +/- 1)
      if (k .gt. 0) then
         iingar(j, 1) = iingar(j, 1) * k
      endif
c
      if (nuall .gt. 0 .and. nrej .eq. 0) then
c---  Some statistics: exclude all observations if any are rejected
         nsrmax = max(nsrmax, mbn_s)
c
c Sums for mean I, by resolution but not by run
c  Weight = 1 centric, 2 acentric
         wt = 2.0 - float(icntrc)
c  Sum for mean s
         sstlmn(mbn_s) = sstlmn(mbn_s) + ss
c  Sum(Imean)
         simnmn(mbn_s) = simnmn(mbn_s) + wt*aih/epsiln
         swtimn(mbn_s) = swtimn(mbn_s) + wt
         nimnmn(mbn_s) = nimnmn(mbn_s) + 1
c
         sumins = sumins + aih
         nins = nins+1
c
c  Anomalous means
         if (icntrc .eq. 0) then
c  Calculate anomalous differences and isomorphous|dispersive differences
c  for each dataset
            call dtsdif(nrs, icntrc, jhkl, ffrf, sdffrf, wtrf, ghl,
     $         irunrf, mrf, isttrf, isymrf, ndtsetsout,
     $         danomdts, sddanomdts, disodts, sddisodts, fdts,
     $         qihdts, qsdidts, qihpl, qsdipl, qihmn, qsdimn)
c Sums for rms DelIanom
            do idts = 1, ndtsetsout
               sdelis(mbn_s, idts) = sdelis(mbn_s, idts) +
     $            danomdts(idts)**2
               nsdelis(mbn_s, idts) = nsdelis(mbn_s, idts) + 1
            enddo               
         endif
      endif
 390  if (.not.eof) go to 300
c========================================================================
c End of reflections
      if (nins .gt. 0) avgins = sumins/float(nins)
c Do normal probability analysis to get initial SDfac values
      if (lsdfac .or. ladjsd) then
         call sdanls(.true., .false.,
     $      nrfgar, ningar, iingar, nregar, aregar, irfidx, slpanom)
      endif
      if (lsdfac) then
         call sdcopt(nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $      nfulls, npartial)
         write (lunout, 6102)
 6102    format (/'===== Updated SDcorrection parameters:',//,
     $      7x,'Run              Fulls                 ',
     $      '         Partials'/)
         do irun = 1,numrun
            if (npartial(irun) .eq. 0) then
c  Fulls only
               write (lunout, 6104) irunum(irun),
     $            (sdfact(i,irun,1),i=1,3)
 6104          format(5x,i5,3x,f9.3,f9.2,f9.4)
            else if (nfulls(irun) .eq. 0) then
c  Partials only
               write (lunout, 6105) irunum(irun),
     $            (sdfact(i,irun,2),i=1,3)
 6105          format(5x,i5,3x,32x,f9.3,f9.2,f9.4)
            else
c  Both
               write (lunout, 6103) irunum(irun),
     $            ((sdfact(i,irun,j),i=1,3),j=1,2)
 6103          format(5x,i5,3x,f9.3,f9.2,f9.4,5x,f9.3,f9.2,f9.4)
            endif
         enddo
      endif
c
      lanom = .false.
      if (kanomo .ne. 0) lanom = .true.
      if (lnormn) call sdanls(.false., lanom,
     $   nrfgar, ningar, iingar, nregar, aregar, irfidx, slpanom)
c
c Adjust SD test for I+/- if required
      if (kanomo .ne. 0 .and. sdrejma .lt. 0.0) then
c  FIXME the factor 3.7 here is _really_ arbitrary!
         anomfac = 3.7
         sdrejma  = abs(sdrejma)  + anomfac * slpanom(2)
         sdrejm2a = abs(sdrejm2a) + anomfac * slpanom(2)
c
         write (lunout, 6121) anomfac, slpanom(2)
 6121    format (' Outlier rejection limits limits for I+ v I- ',
     $      'have been adjusted by a factor ',f7.3,' * ',f7.3,
     $      ' (AnomProbSlope)'/)
         write (lunout, 6122) sdrejma, sdrejm2a
 6122    format (
     $      '   ==== For all observations including I+ & I- ====',/,
     $      '    Reflections measured 3 or more times:', f10.3,
     $      '   maximum deviation from weighted mean of all other',
     $      ' observations'/
     $      '    Reflections measured twice:', f10.3,
     $      '   maximum deviation from weighted mean')
      endif 
c Means etc
      do k = 1, nsrmax
         nrefsm(k) = nimnmn(k)
         if (nimnmn(k) .gt. 0) then
c    Mean <>/epsilon for normalization
            qimnsm(k) = simnmn(k)/swtimn(k) 
            qtolsm(k) = sstlmn(k)/float(nimnmn(k))
         else
            qimnsm(k) = 0.0
            qtolsm(k) = 0.0
         endif
c Populate means for all datasets
         do i = 1, ndtsetsout
            qimnsr(k,i) = qimnsm(k)
c  rms DelIanom
            if (nsdelis(k,i) .gt. 0) then
               qsctsr(k,i) = sqrt(sdelis(k,i)/dble(nsdelis(k,i)))
               nrefsr(k,i) = nsdelis(k,i)
               sdelid(i) = sdelid(i) + sdelis(k,i)
               nsdelid(i) = nsdelid(i) + nsdelis(k,i)
            else
               qsctsr(k,i) = 0.0
               nrefsr(k,i) = 0
            endif
         enddo
      enddo
      do i = 1, ndtsetsout
         if (nsdelid(i) .gt. 0) then
            qsctsa(i) = sqrt(sdelid(i)/dble(nsdelid(i)))
            nrefsa(i) = nsdelid(i)
         else
            qsctsa(i) = 0.0
            nrefsa(i) = 0
         endif
      enddo
c Overall unscaled intensity mean
      qsisu = 0.0
      nqsisu = niunsc
      if (niunsc .gt. 0) qsisu = siunsc/float(niunsc)
      lqmnir = .true.

c Update IMID if necessary on first cycle
      if (imid .eq. +1) then
         imid = max(+1, nint(qsisu))
         write (lunout, '(/a,a,i8/)')
     $      'COMBINE intensity option: IMID updated to mean',
     $      '  unscaled intensity ', IMID
      endif
c Overall averages (not by run)
      if (kprint(1) .ge. 3) then
         write (lunout, 6300)
 6300    format (//
     $      ' Average Intensities in 4(sin theta/lambda)**2 bins',//,
     $      '    N   1/resol^2   Mn(I/Eps) '/)
         do 1300, k = 1,nsrmax
            if (nrefsm(k) .gt. 0) then
               write (lunout, 6310)
     $            k, qtolsm(k), qimnsm(k)
 6310          format (1x,i4,f10.4,f10.0)
            endif
 1300    continue
         write (lunout, '(//)')
      endif
      call prt_tcr
c
      if (nerpdc .gt. 0) lerpdc = .false.
      return
      end
      real function rndmnm(idummy)
c     ==========================
c
c  Returns a random number between 0 & 1, using ccplib routine to fill
c a buffer of random numbers
c
      integer nrandm, nrand1, idummy
      parameter (nrandm=1000, nrand1=nrandm+1)
      real rannum(nrandm)
      integer irandm
      save irandm, rannum
      data irandm/nrand1/
c
      if (irandm .gt. nrandm) then
c Fill buffer
         call ranmar(rannum, nrandm)
         irandm = 1
      endif
c Next number from buffer
      rndmnm = rannum(irandm)
      irandm = irandm+1
      return
      end
      subroutine rstrsc
c     =================
c     
c     Restore all scales from SCALES file
c   2.6.2  allow expansion of number of scales by copying last one
c     
      implicit none
c     
      include 'parameter.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'column.fh'
      include 'refcon.fh'
      include 'params.fh'
      include 'sdfacc.fh'
      include 'dump.fh'
      include 'sphhrm.fh'
c     
      integer ldum, ifail, i, j, irun, nrun, nbf, nbfp, ncs(4), n,
     $   ii, jj, k, ir, nskip, ntds, nsph,
     $   nrnsphr, lmxsphr, lodsphr, lsmoder, mhpoler

      real spc, spcs(3)
      character date*8, time*8, line*80
      logical lfail, lselect, lfound(maxrun)
c     
c-save-      save /scales/
c     
c     
c     
      lselect = .false.
      ldum = 0
      ifail = 0
      call ccpdpn(lunrst, rstrfl, 'READONLY', 'F', ldum, ifail)
c     
c     
      read (lunrst, '(a)') line
      date = line(24:31)
      time = line(36:43)
cc 6000 format('# Scales from SCALA on ',a,' at ',a)
      write (lunout, '(4a)')
     $     ' Restoring scales from ',time,' on ',date
c     
      read (lunrst, 6002) n
 6002 format(7x,i6)
cc 6002 format('NUMRUN ',i6)
      if (n .eq. numrun) then
         numrun = n
      elseif (n .gt. numrun) then
c Select a subset of runs
         lselect = .true.
         do 2, irun = 1, numrun
            lfound(irun) = .false.
 2       continue
         write (lunout,'(a,30i3)')
     $        ' Selecting scales only for runs ',
     $        (irunum(i), i = 1,numrun)
      else
         write (lunout, '(/a,i6,a,i6/)')
     $        ' **** ERROR **** Scale file has ',n,
     $        ' runs defined instead of ',numrun
         go to 999
      endif
c     
      irun = 0
      do 10, ir = 1, n
         read (lunrst, 6010) nrun, nbf, nbfp, (ncs(i), i = 1,4),
     $      ntds, nsph
 6010    format (4x,I6,8x,2I6,9x,4I6,8x,I6,5x,I6)
cc 6010    format ('RUN ',I6,' NBFACT ',2I6,' NSCALES ',4I6,
cc     $      ' NTAILS ',I6, ' NSRF', I6)
c     
c
         if (lselect) then
c Select scales for just some of the runs
            do 11, j = 1, numrun
               if (irunum(j) .eq. nrun) go to 14
 11         continue
c Skip this one
            nskip = ncs(1)*ncs(2)+1
            if (nbf .gt. 0) nskip = nskip + nbf + 1
            if (ntds .gt. 0) nskip = nskip+1
            if (nsph .gt. 0) then
               nskip = nskip + (nsph-1)/8 + 2 +
     $            ((mxsphi*mxstheta)-1)/10+3
            endif
            do 13, i = 1, nskip
               read (lunrst,'(1x)')
 13         continue
            go to 10
c
 14         irun = irun+1
            lfound(irun) = .true.
         else
            if (nrun .ne. irunum(ir)) then
               write (lunout, '(/a,a,2i6/)')
     $          ' **** ERROR **** Scale file has different run number',
     $          ' definition: ', nrun, irunum(irun)
               go to 999
            endif
            irun = ir
         endif
         if (nbf .ne. nbfact(irun)) then
c     Change of number of Bfactors
            if (nbf .lt. nbfact(irun)) then
               write (lunout, '(/a,a,2i6/)')
     $            ' **** WARNING **** Scale file has fewer',
     $            ' Bfactors, new ones will be extrapolated:',
     $            nbf, nbfact(irun)
            endif
         endif
c     
         lfail = .false.
         do 12, i = 1,4
            if (ncs(i) .ne. ncscls(i,irun)) then
               lfail = .true.
            endif
 12      continue
         if (lfail) then
c     Change of number of scales, but allow input of fewer factors
            if (ncs(2) .gt. 1 .and. ncscls(2,irun) .ne. ncs(2)) then
               write (lunout, '(/a,a,8i4/)')
     $            ' **** ERROR **** Scale file has different number',
     $            ' of scale factors:', (ncs(i), i = 1,4),
     $            (ncscls(i,irun), i = 1,4)
               go to 999
            else
               write (lunout, '(/a,a,8i4/)')
     $            ' **** WARNING **** Scale file has fewer',
     $            ' scale factors, new ones will be extrapolated:',
     $            (ncs(i), i = 1,4),
     $            (ncscls(i,irun), i = 1,4)
               
            endif
         endif
c     
         k = 0
         if (nbf .gt. 0) then
            read (lunrst, 6012) spc
 6012       format (9x,f12.5)
cc 6012       format ('BSPACING ',f12.5)
            do 20, i = 1, nbf
               read (lunrst, 6020)
     $          n, (bfactr(j,i,irun), sdbfac(j,i,irun), j=1,nbfp)
 6020          format(8x, i3, 6f12.5)
cc 6020          format('BFACTOR ', i3, 6f12.5)
 20         continue
c     
c     Expand if necessary from last one given
            if (nbf .lt. nbfact(irun)) then
               ii = nbf
               do 22, i = nbf+1, nbfact(irun)
                  do 23, j = 1, nbfpos(irun)
                     if (nbfp .eq. nbfpos(irun)) then
                        jj = j
                     else
                        jj = 1
                     endif
                     bfactr(j,i,irun) = bfactr(jj,ii,irun)
                     sdbfac(j,i,irun) = sdbfac(jj,ii,irun)
 23               continue
 22            continue
            endif
c Set sdpars array for refout
            do 25, i = 1, nbfact(irun)
               do 26, j = 1,nbfpos(irun)
                  k = k+1
                  sdpars(ipoint(k,irun)) = sdbfac(j,i,irun)
 26            continue
 25         continue
            lsdpars = .true.
         endif
c     
         read (lunrst, 6030) (spcs(i), i = 1,3)
 6030    format(8x, 3f12.5)
cc 6030    format('GSPACING', 3f12.5)
c     
         do 30, j = 1,ncs(2)
            do 31, i = 1,ncs(1)
               read (lunrst, 6032) ii,jj,
     $              cscals(j,i,irun), sdcscl(j,i,irun)
 6032          format(6x,2i4,2f12.5)
cc 6032          format('SCALE ',2i4,2f12.5)
 31         continue
 30      continue
c     Expand if necessary
c      along Rot
         if (ncs(1) .lt. ncscls(1,irun)) then
            ii = ncs(1)
            do 35, i = ncs(1)+1, ncscls(1,irun)
               do 36, j = 1,ncs(2)
                  cscals(j,i,irun) = cscals(j,ii,irun)
                  sdcscl(j,i,irun) = sdcscl(j,ii,irun)
 36            continue
 35         continue
         endif
         if (ncs(2) .lt. ncscls(2,irun)) then
c     across detector
            do 37, j = 1, ncscls(2,irun)
               do 38, i = 1,ncs(1)
                  cscals(j,i,irun) = cscals(1,i,irun)
                  sdcscl(j,i,irun) = sdcscl(1,i,irun)
 38            continue
 37         continue
         endif
c Copy to sdpars array
        do 41, j = 1,ncscls(1,irun)
           do 42, i = 1, ncscls(2, irun)
              k = k+1
              sdpars(ipoint(k, irun)) = sdcscl(i,j,irun)
 42        continue
 41     continue
c
c Tails correction
        if (ntds .gt. 0) then
           if (nrntds(irun) .eq. ntds) then
              read (lunrst, 6041)
     $           (partds(i,irun), i = 1, nrntds(irun))
 6041         format (6x,5f10.5)
cc 6041         format ('TAILS ',5f10.5)
           elseif (nrntds(irun) .eq. 0) then
c  ignore TAILS line if not wanted
              read (lunrst, '(1x)')
           else
              write (lunout, '(/a,i5,a,i5,a,i5,a/)')
     $           ' *** ERROR: in Scales file for run ',irunum(irun),
     $           ', ', ntds,' TAILS parameters found where ',
     $           nrntds(irun), ' expected ***'
              ifail = ifail+1
           endif
        endif
c Surface stuff
        if (nsph .gt. 0) then
           if (nrnsph(irun) .eq. nsph) then
              read (lunrst, 6051)
     $           nrnsphr, lmxsphr, lodsphr,
     $           lsmoder, mhpoler,
     $           (sphclm(i,irun), i = 1,  nrnsph(irun))
 6051       format (8x, 5i5,100(/8f10.5))
cc 6051       format ('SURFACE ', 5i5,100(/8f10.5))
              if (
     $           nrnsphr .ne. nrnsph(irun) .or.
     $           lmxsphr .ne. lmxsph(irun) .or.
     $           lodsphr .ne. lodsph(irun) .or.
     $           lsmoder .ne. lsmode(irun) .or.
     $           mhpoler .ne. mhpole(irun)) then
                 write (lunout, '(/a,i5,a)')
     $              ' *** ERROR: in Scales file for run ',irunum(irun),
     $         ' SURFACE or SECONDARY parameters changed from dump ***'
                 ifail = ifail+1
              endif            
              read (lunrst, 6052) (
     $           (nsphith(i,j,irun), j = 1, mxsphi), i = 1, mxstheta)
 6052         format (9x, 500(/10i8))
cc 6052         format ('SRFCOUNT ', 500(/10i8))
           elseif (nrnsph(irun) .eq. 0) then
c  ignore SURFACE lines if not wanted
              read (lunrst, 6051)
     $           nrnsphr, lmxsphr, lodsphr,
     $           lsmoder, mhpoler,
     $           (sphclm(i,irun), i = 1,  nrnsph(irun))
              read (lunrst, 6052) (
     $           (nsphith(i,j,irun), j = 1, mxsphi), i = 1, mxstheta)
           else
              write (lunout, '(/a,i5,a,i5,a,i5,a/)')
     $           ' *** ERROR: in Scales file for run ',irunum(irun),
     $           ', ', nsph,' SURFACE parameters found where ',
     $           nrnsph(irun), ' expected ***'
              ifail = ifail+1
           endif
        endif
c     
 10   continue
c
      if (lselect) then
c Check scales for all required runs have been found
         ifail = 0
         do 101, irun = 1, numrun
            if (.not.lfound(irun)) then
               write (lunout, '(a,i5,a)')
     $              ' *** ERROR: run ', irunum(irun),
     $              ' not found in Scales file'
               ifail = ifail+1
            endif
 101     continue
         if (ifail .ne. 0) call ccperr
     $        (1, '*** Missing runs in scale file ***')
      endif
c     
      return
c     
 999  call ccperr(1,'** Incompatible scale file **')
      end
      SUBROUTINE  RTMATS
C                 ! input: rotation angle
     1   (ANGLE
C                 ! input: rotation axis
     2   ,AXIS
C                 ! input: required number of derivatives
     3   ,NDER
C                 ! output: rotation matrix and up to 4 requested derivatives
     4   ,MATS)
C
CEE Creates a rotation matrix and optional derivatives up to the 4th with 
CEE respect to the angle of rotation.  The direction of
CEE the input vector is taken as the axis down which the rotation represented 
CEE by the matrix is seen to be clockwise,
CEE and the angle of rotation is taken to be in radians.
C rotation_matrices
Created: 14-NOV-1986 D.J.Thomas,   MRC Laboratory of Molecular Biology, 
C                                  Hills Road, Cambridge, CB2 2QH, England
C    - from dRotation_matrix routine
C rotation_matrices
C
C                 ! rotation angle / radians
      REAL   ANGLE     
C                 ! cos(rotation angle)
      REAL   CSR     
C                 ! loop counters
      INTEGER   I, J     
C                 ! number of derivatives required (on [0,4])
      INTEGER   NDER    
C                 ! sin(rotation angle)
      REAL   SNR     
C                 ! 1-cos(rotation angle)
      REAL   VCR     
C
C                 ! rotation axis (unit vector)
      REAL   AXIS  (1:3)   
C                 ! required rotation matrix and requested derivatives
      REAL   MATS (1:3,1:3,0:*)  
C
      CSR = COS(ANGLE)
      VCR = 1.0 - CSR
      SNR = SIN(ANGLE)
C
      IF (NDER .LT. 0) THEN
C                  ! warning message, no other action necessary
         PRINT *, 'RTMATS: negative number of derivatives?' 
      ENDIF
C
C                  ! evaluate the rotation matrix explicitly
      MATS(1,1,0) = VCR * AXIS(1) * AXIS(1)  +  CSR   
      MATS(2,1,0) = VCR * AXIS(2) * AXIS(1)  +  SNR * AXIS(3)
      MATS(3,1,0) = VCR * AXIS(3) * AXIS(1)  -  SNR * AXIS(2)
      MATS(1,2,0) = VCR * AXIS(1) * AXIS(2)  -  SNR * AXIS(3)
      MATS(2,2,0) = VCR * AXIS(2) * AXIS(2)  +  CSR
      MATS(3,2,0) = VCR * AXIS(3) * AXIS(2)  +  SNR * AXIS(1)
      MATS(1,3,0) = VCR * AXIS(1) * AXIS(3)  +  SNR * AXIS(2)
      MATS(2,3,0) = VCR * AXIS(2) * AXIS(3)  -  SNR * AXIS(1)
      MATS(3,3,0) = VCR * AXIS(3) * AXIS(3)  +  CSR
C
C                  ! evaluate the (4n+1)th derivative matrix explicitly
      IF (NDER .GE. 1) THEN     
         MATS(1,1,1) = SNR * AXIS(1) * AXIS(1)  -  SNR
         MATS(2,1,1) = SNR * AXIS(2) * AXIS(1)  +  CSR * AXIS(3)
         MATS(3,1,1) = SNR * AXIS(3) * AXIS(1)  -  CSR * AXIS(2)
         MATS(1,2,1) = SNR * AXIS(1) * AXIS(2)  -  CSR * AXIS(3)
         MATS(2,2,1) = SNR * AXIS(2) * AXIS(2)  -  SNR
         MATS(3,2,1) = SNR * AXIS(3) * AXIS(2)  +  CSR * AXIS(1)
         MATS(1,3,1) = SNR * AXIS(1) * AXIS(3)  +  CSR * AXIS(2)
         MATS(2,3,1) = SNR * AXIS(2) * AXIS(3)  -  CSR * AXIS(1)
         MATS(3,3,1) = SNR * AXIS(3) * AXIS(3)  -  SNR
C
C                   ! evaluate the (4n+2)th derivative matrix explicitly
         IF (NDER .GE. 2) THEN     
            MATS(1,1,2) = CSR * AXIS(1) * AXIS(1)  -  CSR
            MATS(2,1,2) = CSR * AXIS(2) * AXIS(1)  -  SNR * AXIS(3)
            MATS(3,1,2) = CSR * AXIS(3) * AXIS(1)  +  SNR * AXIS(2)
            MATS(1,2,2) = CSR * AXIS(1) * AXIS(2)  +  SNR * AXIS(3)
            MATS(2,2,2) = CSR * AXIS(2) * AXIS(2)  -  CSR
            MATS(3,2,2) = CSR * AXIS(3) * AXIS(2)  -  SNR * AXIS(1)
            MATS(1,3,2) = CSR * AXIS(1) * AXIS(3)  -  SNR * AXIS(2)
            MATS(2,3,2) = CSR * AXIS(2) * AXIS(3)  +  SNR * AXIS(1)
            MATS(3,3,2) = CSR * AXIS(3) * AXIS(3)  -  CSR
C
C                    ! evaluate the (4n+3)th derivative matrix
            IF (NDER .GE. 3) THEN     
               DO 100 J = 1, 3
                  DO 100 I = 1, 3
C                    ! (4n+3)th derivative is minus (4n+1)th derivative
100                  MATS(I,J,3) =  -  MATS(I,J,1)
                  CONTINUE
               CONTINUE
C
C                    ! evaluate the (4n+4)th derivative matrix
               IF (NDER .GE. 4) THEN    
                  DO 200 J = 1, 3
                     DO 200 I = 1, 3
C                       ! (4n+4)th derivative is minus (4n+2)th derivative
200                     MATS(I,J,4) =  -  MATS(I,J,2)
                     CONTINUE
                  CONTINUE
C
                  IF (NDER .GE. 5) THEN 
                     PRINT *, 'RTMATS: > 4 derivatives requested!'
                     PRINT *, ' Only the first 4 have been evaluated '
                     PRINT *, ' since they repeat after this.'
                     PRINT *, ' Additionally, d3=-d1, d4=-d2.'
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDIF
C
      RETURN
      END
      subroutine rtzmat(rz, rot)
c     ==========================
c
c  Get rotation matrix around z
c
c On entry:
c  rz        rotation angle in radians
c
c On exit:
c  rot(3,3)  rotation matrix
c
      real rz, rot(3,3)
c
c
c
      rot(1,1) = cos(rz)
      rot(1,2) = -sin(rz)
      rot(1,3) = 0.0
      rot(2,1) = sin(rz)
      rot(2,2) = cos(rz)
      rot(2,3) = 0.0
      rot(3,1) = 0.0
      rot(3,2) = 0.0
      rot(3,3) = 1.0
      return
      end
      subroutine runget(ibatch, zd, irun, jser, jserbt)
c     ==========================================
c     
c     Determine run number for this batch
c     
c     On entry:
c     ibatch     batch number
c     zd         rotation angle
c     
c     On exit:
c     irun       run number, = -1 if rejected or run inactive
c     jser       overall batch serial number
c     jserbt     batch serial number within run (for SCALE BATCH)
c     
      implicit none
c     
      include 'parameter.fh'
      include 'rundef.fh'
c     
      integer ibatch, irun, jser, jserbt
      real zd
c     
      integer j
      integer ccp4_hash_lookup
      external ccp4_hash_lookup
c     
c     
c     internal batch serial number jser
      jser = ccp4_hash_lookup(ibatch)
      irun = irunbt(jser)
      jserbt = isrrbt(jser)
      if (irun .le. 0) then
         irun = -1
      else
c  Is run currently active?
         if (lactvrun(irun)) then
c     
c     Any includes or excludes?
            if (nptrun(irun) .gt. 0) then
               do 10, j = 1,nptrun(irun)
                  if (zd .ge. zmnrun(j,irun) .and.
     $               zd .le. zmxrun(j,irun)) then
c     within defined range
                     if (lptrun(j,irun)) then
c     exclude
                        irun = -1
                        go to 100
                     else
                        go to 100
                     endif
                  endif
 10            continue
            endif
         else
c inactive
            irun = -1
         endif
      endif
c     
 100  return
      end
      subroutine scala_main(
     $   nrfgar, irjgar,
     $   niagar, iingar,
     $   nragar, aregar,
     $   maxmt1, amatrx, maxmtx, eigval, maxmt3, eigvec, maxmt4,
     $   vecinv, maxmt5, work
     $   )
c     ==================================================
c
c  Call main routines with allocated arrays, for each reflection in file
c  Note that array irjgar has the length = number of reflections in file
c
c  iinger & aregar are 2D arrays: their second dimension is set in the
c  main program and must match here  
      integer  ningar, nregar
      parameter  (ningar=5, nregar=6)
c  matrix work array length is taken from maxmtx (length of eigval)
c      (other length arguments are ignored)
      integer nrfgar
      integer irjgar(nrfgar), iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
c Dummy lengths
      integer  niagar, nragar, maxmt1, maxmt3, maxmt4, maxmt5
c Declarations
      include 'bigarg.fh'
c
      include 'flow.fh'
      include 'inout.fh'
c
      if (nrfgar*ningar .ne. niagar .or. nrfgar*nregar .ne. nragar) then
         write (lunout, '(/a,5i10)')
     $      ' **** Incorrect dummy array dimensions in scala_main: ',
     $      nrfgar, ningar, niagar, nregar, nragar
         call ccperr(1, '*** Wrong arrays: fix source! ***')
      endif
c
c Initial analysis
      if (lanlin)
     $   call anlini(maxmtx, amatrx, eigval, eigvec, vecinv, work)
c
c Refine
      if (lscale)
     $   call refscl(maxmtx, amatrx, eigval, eigvec, vecinv, work)
c
c Analyse & output
      if (lanout)
     $   call refout(nrfgar, irjgar, ningar, iingar, nregar, aregar)
c
      call dna_end
      call ccperr(0,'** Normal termination **')
      end

      subroutine sclder(
     $     irun, stovl, time, xd, yd, zd, ghl,  Thl, Chl, Shl, dghldc)
c     ===============================================================
c     
c     Calculate scale ghl & derivatives dghldc. This excludes the diffuse
c  scattering correction, which can only be calculated after summing partials
c     
c     On entry:
c     irun     internal run number
c     stovl    (sin theta/lambda)**2
c     time     time (may be actually rotation)
c     xd, yd   x & y coordinates (detector coordinates if detector frame),
c              or (theta,phi) radians for polar correction
c     zd       z coordinate (rotation)
c               (if mbatsc(irun) .ne. 0, zd = batch serial number within run)
c     
c     On exit:
c     ghl           dividing scale factor (= Thl * Chl)
c     Thl           B-factor component
c     Chl           scale component
c     Shl           surface component (spherical harmonics)
c     dghldc(npar)  partial derivatives d(ghl)/d(param)
c                   for all scale parameters for this run
c                   (excluding diffuse scattering correction)
c
c  3/11/95  2.1.1  anisotropic B options
c 16/ 2/94  added lbtslp option
c  4/11/93  rewritten to speed up for Batch mode PRE
c     
      implicit none
c     
      include 'parameter.fh'
c     
c     Arguments
c     
      integer irun
      real stovl, time, xd, yd, zd, ghl, Thl, Chl, Shl,
     $   dghldc(maxpmr)
c     
c     Commons
      include 'scales.fh'
      include 'params.fh'
      include 'refcon.fh'
      include 'glocon.fh'
      include 'inout.fh'
      include 'sphhrm.fh'
c     
c     Locals
      real p(maxrot), q(maxxys), dThldb(maxpmr),
     $   dChldc(maxxys*maxrot),
     $   sumq, sumqsc, sump, sumpsc, del, del2, phi, sumqbf(maxbfc),
     $   ylm(maxspt), cth, sth, expmin
      integer i,j,n,k,k0,l, i1, i2, np
c
      data expmin/0.000001/
c
c     
cc      write (lunout, '(a,i3,a,3f10.4)')
cc     $     ' SCLDER: Run ',irun,'xyz',xd,yd,zd
c     
c     ---- B-factor component ----
      if (lbfrun(irun)) then
         sumq   = 0.0
         do 9, j = 1,nbfpos(irun)
            sumqbf(j) = 0.0
 9       continue
c
c     Thl is B-factor component of scale, dThldb are derivatives
         if (nbfact(irun) .eq. 1) then
c B1. No interpolation on "time" (angle)
            if (nbfpos(irun) .eq. 1) then
c    Isotropic
               Thl = max(exp(+2.0*stovl*bfactr(1,1,irun)), expmin)
               dThldb(1) = +Thl*2.0*stovl
            else
c    Anisotropic
               Thl = exp(+2.0*(bfactr(1,1,irun)*xd*xd
     $                       + bfactr(2,1,irun)*xd*yd*2.0
     $                       + bfactr(3,1,irun)*yd*yd))
               dThldb(1) = +Thl*2.0*xd*xd
               dThldb(2) = +Thl*4.0*xd*yd
               dThldb(3) = +Thl*2.0*yd*yd
            endif
         else if (mbatsc(irun) .gt. 0) then
c B2. Batch scaling, no interpolation on zd
            do 5, j=1, nbparm(irun)
               dThldb(j) = 0.0
 5          continue
            j = nint(time)
            if (nbfpos(irun) .eq. 1) then
c    Isotropic
               Thl = exp(+2.0*stovl*bfactr(1,j,irun))
               dThldb(j) = +Thl*2.0*stovl
            else
c    Anisotropic
               Thl = exp(+2.0*(bfactr(1,j,irun)*xd*xd
     $            + bfactr(2,j,irun)*xd*yd*2.0
     $            + bfactr(3,j,irun)*yd*yd))
               k = (j-1)*nbfpos(irun)
               dThldb(k+1) = +Thl*2.0*xd*xd
               dThldb(k+2) = +Thl*4.0*xd*yd
               dThldb(k+3) = +Thl*2.0*yd*yd
            endif
         else
c B3. interpolate on rotation
c     (NB this bit of code is replicated in s/r bfcitp
c.c               do 10, i = 1, nbfact(irun)
c.c                  del = (posbfc(i,irun) - time)**2/Vwt
c.c                  if (del .lt. prblt) then
c.c                     p(i) = exp(-del)
c.c                     do 11, j = 1,nbfpos(irun)
c.c                        sumqbf(j) = sumqbf(j) + p(i)*bfactr(j,i,irun)
c.c 11                  continue
c.c                     sumq   = sumq + p(i)
c.c                  else
c.c                     p(i) = 0.0
c.c                  endif
c.c 10            continue

c B3. interpolate on rotation
c     (NB this bit of code is replicated in s/r bfcitp
            do i=1,nbfact(irun)
               p(i) = 0.0
            enddo
c *** use 3 points unless fewer
            np = min(3, nbfact(irun))
            i1 = min(max(1, nint(time)), nbfact(irun)-np+1) 
            i2 = i1 + np - 1
            do 10, i = i1, i2
               del = (posbfc(i,irun) - time)**2/Vwt
               p(i) = exp(-del)
               do 11, j = 1,nbfpos(irun)
                  sumqbf(j) = sumqbf(j) + p(i)*bfactr(j,i,irun)
 11            continue
               sumq   = sumq + p(i)
 10         continue
c     
            if (sumq .le. 0.0) then
               Thl = 1.0
            else
c     Thl is B-factor component of scale, dThldb are derivatives
               if (nbfpos(irun) .eq. 1) then
c    Isotropic
                  Thl = exp(+2.0*stovl*sumqbf(1)/sumq)
                  do 20, i = 1, nbfact(irun)
                     dThldb(i) = +Thl*2.0*stovl*p(i)/sumq
 20               continue
               else
c    Anisotropic
                  Thl = exp(+2.0*(xd*xd*sumqbf(1)/sumq
     $               + 2.0*xd*yd*sumqbf(2)/sumq
     $               +     yd*yd*sumqbf(3)/sumq))
                  k = 0
                  do 21, i = 1, nbfact(irun)
                     dThldb(k+1) = +Thl*2.0*xd*xd*p(i)/sumq
                     dThldb(k+2) = +Thl*4.0*xd*yd*p(i)/sumq
                     dThldb(k+3) = +Thl*2.0*yd*yd*p(i)/sumq
                     k = k+3
 21               continue
               endif
            endif
            if (kprint(1) .ge. 4) then
               write (lunout, '(a,f10.4,100(10f10.5/))')
     $            ' Bfac # ',Thl, (dThldb(i), i=1,nbfact(irun))
            endif
         endif
c     
      else
c     no B-factor component
         Thl = 1.0
      endif
c     ----
c     
c     ---- Scale component ----
c     
      Chl = 1.0
      n = ncscls(1,irun)*ncscls(2,irun)
      do 90, k = 1, n
         dChldc(k) = 0.0
 90   continue
c     
C  Probabilities on z
      sump   = 0.0
      if (mbatsc(irun) .ne. 0) then
c Batch scaling, no interpolation on zd
         do 95, j=1,ncscls(1,irun)
            p(j) = 0.0
 95      continue
         if (lbtslp(irun)) then
c   "sloped" scaling, use 2 consecutive scale factors for this batch
            j = nint(zd/btslpf)
            phi = zd - float(j)*btslpf
            j = 2*(j-1) + 1
            p(j) = 1.0 - phi
            p(j+1) = phi
            sump = p(j) + p(j+1)
         else
c  plain batch scaling            
            j = nint(zd)
            p(j) = 1.0
            sump = 1.0
         endif
      else
cxc
         do 101, j = 1, ncscls(1,irun)
            p(j) = 0.0
 101     continue 
c *** use 3 points unless fewer
         np = min(3, ncscls(1,irun))
         i1 = min(max(1, nint(zd)), ncscls(1,irun)-np+1) 
         i2 = i1 + np - 1
         do 100, j=i1,i2
            if (ncscls(1,irun) .eq. 1) then
               del = 0.0
               p(j) = 1.0
            else
               del = (zd - poscsr(j,irun))**2/Vwz
               p(j) = exp(-del)
            endif
            sump = sump + p(j)
 100     continue

c.c         do 100, j = 1, ncscls(1,irun)
c.c            if (ncscls(1,irun) .eq. 1) then
c.c               del = 0.0
c.c            else
c.c               del = (zd - poscsr(j,irun))**2/Vwz
c.c            endif
c.c            if (del .lt. prblz) then
c.c               p(j) = exp(-del)
c.c               sump = sump + p(j)
c.c            else
c.c               p(j) = 0.0
c.c            endif
c.c 100     continue
      endif
c
c  Probabilities on xy
      sumq   = 0.0
      if (ncscls(2,irun) .eq. 1) then
         q(1) = 1.0
         sumq = 1.0
      else
         do 110, i = 1, ncscls(2,irun)
            if (ncscls(3,irun) .eq. 1) then
               del2 = 0.0
            else
               del2 = (xd - poscsx(1,i,irun))**2
            endif
            if (ncscls(4,irun) .gt. 1) then
               del2 = del2 + (yd - poscsx(2,i,irun))**2
            endif
            del2 = del2/Vwxy
            if (del2 .lt. prblxy) then
               q(i) = exp(-del2)
               sumq   = sumq   + q(i)
            else
               q(i) = 0.0
            endif
 110     continue
      endif
c
c
      sumpsc = 0.0
      if (sumq .gt. 0.0) then
         k = 0
         if (mbatsc(irun) .ne. 0 .and. .not.lbtslp(irun)) then
c  Batch scaling option
            j = nint(zd)
            k = (j-1) * ncscls(2,irun)
            if (ncscls(2,irun) .eq. 1) then
               Chl = cscals(1,j,irun)
               dChldc(k+1) = 1.0
            else
               sumqsc = 0.0
               do 115, i = 1, ncscls(2,irun)
                  k = k+1
                  if (q(i) .gt. 0.000001) then
                     sumqsc = sumqsc + q(i) * cscals(i,j,irun)
c     
                     if (kprint(1) .ge. 4) then
                        write (lunout, '(a,2i3,a,f10.4,a,2f10.5)')
     $                       ' Scale # ',i,j,':',cscals(i,j,irun),
     $                       '  probability ',p(j),q(i)
                     endif
                     dChldc(k) = q(i)/sumq
                  endif
 115           continue
               Chl = sumqsc/sumq
            endif
         else
c   Smooth scaling
            do 120, j = 1, ncscls(1,irun)
               k0 = k+1
               if (p(j) .gt. 0.000001) then
                  sumqsc = 0.0
                  do 130, i = 1, ncscls(2,irun)
                     k = k+1
                     if (q(i) .gt. 0.000001) then
                        sumqsc = sumqsc + q(i) * cscals(i,j,irun)
c     
                        if (kprint(1) .ge. 4) then
                           write (lunout, '(a,2i3,a,f10.4,a,2f10.5)')
     $                          ' Scale # ',i,j,':',cscals(i,j,irun),
     $                          '  probability ',p(j),q(i)
                        endif
                        dChldc(k) = p(j)*q(i)
                     endif
 130              continue
                  sumpsc = sumpsc + p(j)*sumqsc/sumq
                  do 140, l = k0, k
                     dChldc(l) = dChldc(l)/sumq
 140              continue
               else
                  k = k + ncscls(2,irun)
               endif
 120        continue
c     
            if (sump .gt. 0.0) then
               Chl = sumpsc / sump
            else
               Chl = 1.0
            endif
            do 150, k = 1,n
               if (sump .gt. 0.0) then
                  dChldc(k) = dChldc(k) / sump
               else
                  dChldc(k) = 0.0
               endif
 150        continue
         endif
      endif
c     
c     ----
      if (Chl .eq. 0.0) then
         write (lunout, '(a,100f8.3)')
     $        ' Zero scale: p = ',(p(j),j=1,ncscls(1,irun))
         write (lunout, '(a,100f8.3)')
     $        '             q = ',(q(i),i=1,ncscls(2,irun))
         write (lunout, '(a, 3f12.4)')
     $        ' xd, yd, zd = ', xd, yd, zd
      endif
c
c------------------------------------------
c  Spherical harmonic (SECONDARY or SURFACE) scale
      if (nrnsph(irun) .gt. 0) then
c  get spherical harmonic coefficients for this theta & phi
c (polar angles, not to be confused with spindle phi!)
         cth = cos(xd)
         sth = sin(xd)
         phi = yd
         call sphhmc(sth, cth, phi, lmxsph(irun), lodsph(irun), ylm)
c
c  sum Ylm * Clm  using linked parameters if required
c  First coefficient is a fixed constant 1.0, & is not used,
c  hence offset by +1 here
         Shl = 1.0
         do i = 1, nrnsph(irun)
            Shl = Shl + ylm(i+1) * sphclm(i, isphpt(irun))
         enddo
         if (Shl .lt. 0.0) then
cc            write (lunout, '(a)') 'Help'
            Shl = 0.0
         endif
C         write (lunout, '(a,3f6.3,/,50(12f6.3))')
c     $      ' Shl, th, ph',  Shl, xd, yd,(ylm(i+1), i=1, nrnsph(irun))
c  Count & average
         call cntsph(xd, yd, isphpt(irun), Shl)
c
      else
         Shl = 1.0
      endif
c
c     Total scale 
      ghl = Thl * Chl * Shl
c
      if (kprint(1) .ge. 4) then 
         write (lunout, '(a,i6,2f8.1,f8.3,a,4f10.5/)')
     $        ' irun, xd, yd, zd', irun, xd, yd, zd,
     $        ' Thl, Chl, Shl, ghl: ', Thl, Chl, Shl, ghl
      endif
c     
c     Derivatives
      k = 0
      if (lbfrun(irun)) then
         do 200, i = 1, nbparm(irun)
            k = k+1
            dghldc(k) = Chl * dThldB(i)
 200     continue
      endif
c     
      k = ipnscl(irun) - 1
      do 210, i = 1,n
         k = k+1
         dghldc(k) = Thl * dChldc(i)
 210  continue
c
      if (nrnsph(irun) .gt. 0) then
         k = ipnsph(irun) - 1
         do i = 1, nrnsph(irun)
            k = k+1
            dghldc(k) = Thl * Chl * ylm(i+1)
         enddo
      endif         
c     
      return
      end
      subroutine sclprt(nrs, 
     $   ffrf, sdffrf, ffprrf, sdfprf, frclrf, isttrf, nmprrf, 
     $   jmprrf, mrf, prthld, nrejpt, nuse, imid, ipower, lsclpr)
c     ==========================================================
c
c Scale partials
c
c  On entry:
c     nrs          number of observations
c     ffrf(nrs)    intensity
c     sdffrf(nrs)  sd( intensity)
c     ffprrf(nrs)  profile-fitted intensity
c     sdfprf(nrs)  sd(profile-fitted intensity)
c     frclrf(nrs)  fraction calculated
c     isttrf(nrs)  status flags
c     nmprrf(nrs)  number of parts summed into this slot, initially = 1 for all
c     jmprrf(nrs)  status of this slot
c              = -2 for otherwise rejected, initially set to this
c     mrf(nrs)     partiality flags
c     prthld       partiality threshold
c     nrejpt(6)    current rejection numbers
c     imid         flag for selection of profile-fitted|summation intensity
c                  if imid = 0, then ffprrf is ignored
c     ipower       power for combine intensities option (imid .gt. 0)
c     lsclpr       if .false. always use integrated I for scaled partials
c                  .true. allow profile-fitted I to be used for
c                      scaled partials
c
c  On exit:
c     ffrf     scaled partials where appropriate
c     sdffrf   sds
c     isttrf   modified status flag, lqOK for scaled partials
c     nmprrf(nrs) number of parts summed into this slot
c              =  1 for full (as initial value)
c              =  0 unscaled partial 
c                    (ie those flagged as lqnprt)
c              = -1  scaled partial, = - number of parts
c     jmprrf(nrs)  status of this slot
c              =  0  principal slot (ie scaled partial)
c              = -1 for unscaled partial 
c              = -2 for otherwise rejected
c     nrejpt(6)  number rejected: 
c              (1) rejected reflections with GAP (GAP not set)
c              (2) too wide (.gt. maxpwd parts)
c              (3) total fraction too small
c              (4) total fraction too large
c              (5) inconsistent MPART flags
c              (6) some parts flagged as rejected
c     nuse     number of reflections remaining,
c                number of fulls + number of accepted scaled partials
c
c
      implicit none
c
      integer nrs, nuse
      real ffrf(nrs), sdffrf(nrs), ffprrf(nrs), sdfprf(nrs),
     $   frclrf(nrs), prthld
      integer isttrf(nrs), nmprrf(nrs), jmprrf(nrs), mrf(nrs),
     $   nrejpt(5), imid, ipower
      logical lsclpr
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
c
      integer l
c
      nuse = 0
cc
      do 10, l = 1,nrs
         if (isttrf(l) .eq. lqprt) then
c this is a partial
            if (lsclpr .and. imid .ne. 0) then
               call selint(
     $            ffrf(l), sdffrf(l), ffprrf(l), sdfprf(l),
     $            mrf(l), imid, ipower)
            endif

c Is it above threshold?
            if (frclrf(l) .gt. prthld) then
               ffrf(l) = ffrf(l)/frclrf(l)
               sdffrf(l) = sdffrf(l)/frclrf(l)
c  Change flags
               isttrf(l) = lqOK
               nmprrf(l) = -1
               jmprrf(l) = 0
            else
               nrejpt(3) = nrejpt(3) + 1
               isttrf(l) = lqnprt
               nmprrf(l) = 0
               jmprrf(l) = -1
            endif
         endif
c Count useful observations
         if (isttrf(l) .ge. lqOK) nuse = nuse+1
 10   continue
c
c
      return
      end
      SUBROUTINE SCNDER(X,Y,N,Y2)
C--------------------------------
C Returns second derivatices (Y2) of the interpolating function for y(x)
C whose N values Y are tabulated for points X.
C
      INTEGER NMAX,N,K,I
      PARAMETER (NMAX=100)
      REAL X(N),Y(N),Y2(N),U(NMAX),UN,QN,P,SIG
C
      Y2(1)=0.
      U(1)=0.
C
      DO 11 I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.
        Y2(I)=(SIG-1.)/P
        U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))
     *      /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
11    CONTINUE
C
      QN=0.
      UN=0.
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)
C
      DO 12 K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
12    CONTINUE
C
      RETURN
      END
c
c
c
      subroutine sdanls(ldonorm, ldoanom,
     $   nrfgar, ningar, iingar, nregar, aregar,
     $   irfidx, slpanom)
c    ========================================================
c
c
c On entry:
c  ldonorm       .true. to do normal porbability analysis &
c                  adjust SDcorrections
c  ldoanom       .true. to do anomalous analysis
c  nrfgar        number of observations in file
c  iingar        integer array
c  aregar        real array
c
c   iingar(,1)  (a) for first observation in each reflection,
c                  = number of observations (negated if I-)
c               (b) for subsequent observations = +/- 1
c               .gt. 0 for I+ (or centric), .lt. 0 I-
c   iingar(,2) packed run number, partial status, centric flag 
c
c   aregar(,1)  Ihl
c   aregar(,2)  sd(Ihl)
c   aregar(,3)  ghl  scale
c   aregar(,4)  LP
c  irfidx       number of observations in arrays
c
c On exit:
c  slpanom(2)   slopes of anomalous normal probability analysis
c                (1) all data (2) central part of distribution 
c
      logical ldonorm, ldoanom
      integer nrfgar, ningar, nregar, irfidx
      integer iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
      real slpanom(2)
c
      include 'parameter.fh'
      include 'outcon.fh'
      include 'inout.fh'
      include 'anomtc.fh'
      include 'sdfacc.fh'
      include 'glocon.fh'
      include 'rundef.fh'
c
      real ffrf(maxrfl), wtrf(maxrfl),
     $   sdffrf(maxrfl), ghl(maxrfl), delrf(maxrfl), clprf(maxrfl)
      integer irunrf(maxrfl), mrf(maxrfl), isttrf(maxrfl),
     $   isymrf(maxrfl), lanmtc(maxrfl), idxrf(maxrfl)
      logical useflg(maxrfl)
c
      integer nrs, icntrc, jhkl(3), nuse, nrej, krej2, nupls, numns,
     $   kwgtyp
      logical lprint, lrejct
      real swih, swihpl, swihmn, slope(2,2), dltlim, prblim,
     $   obslim(maxrun),  sintct(2,2), aihpls, sdihpl, uh,
     $   aihmns, sdihms, varipl, varims
      integer i, j, k, l, jrf, irun, ip, nused(2,2), iselds, janom, nr
      character label*8, labfpt*80
c
      integer lenstr
      external lenstr
c
      lprint = .false.
      lrejct = .false.
      krej2 = krej2o
      kwgtyp = 2
      call ccpzi(jhkl, 3)
      call ccpzi(lanmtc, maxrfl)
      call ccpzr(slpanom,2)
c   Probability limit 0.9, c. 63% of data
      dltlim = 0.9
cc   Probability limit 0.674, c. 50% of data
c         dltlim = 0.674
c============================================
c Normal probability analysis of deltas, by run
c**** Extract data from arrays for each reflection, calculate 
c Deltas 
c
      if (ldonorm) then
         k = 1
         jrf = 0
         nr = 1
c
 10      call nxtrfl(k,
     $      nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $      maxrfl, nrs, ffrf, sdffrf, ghl, irunrf, mrf, idxrf, 
     $      clprf, isttrf, isymrf, icntrc)
c  do while k .le. irfidx (essentially)
         if (k .gt. 0) then
c  Apply current SD correction
            call sdcorr(nrs, ffrf, sdffrf, clprf, ghl, isttrf,
     $         isymrf, irunrf, mrf, swih)
c Set variance weights
            call setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c Calculate Delta (Chi) for each observation
c No rejection of outliers (lrejct .false.)
c Just calculate deviations
c  Always do it only within datasets
            call choutl(nrs, kanomo, lprint, lrejct, .true., icntrc,
     $         jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $         isymrf, lanmtc, nuse, nrej,
     $         sdrejm, sdrejm2, sdrejma, sdrejm2a,
     $         krej2, delrf,
     $         swih, swihpl, swihmn)
c Store deviations etc
c  Note that initial sd corrections have already been added to sd(I)
            do l = 1,nrs
               jrf = jrf + 1
c   Store delta, & run number, negate run for partials
c   Note that delrf = 0.0 for rejected observations
               aregar(jrf, 5) = delrf(l)
               if (mrf(l) .eq. 0) then
                  iingar(jrf,3) = irunrf(l)
               else
                  iingar(jrf,3) = -irunrf(l)
               endif
c    Array index of this observation
               iingar(jrf,4) = idxrf(l)
c
            enddo
            nr =nr+1
            go to 10
         endif
c  Limit to just exclude real outliers (98% of data)
         prblim = 2.5
         write (lunout, '(/a/a)')
     $      ' ===========================',
     $      ' Normal probability analysis'
         do 400, irun = 1, numrun
            if (irun .eq. irefrn) go to 400
            k = 0
            slope(2,1) = 1.0
            slope(2,2) = 1.0
c  Analysis on fulls & partials separately
c.................
            do 402, ip = 1,2
               do 401, j = 1, jrf
c Copy all non-zero deltas for run irun
                  if (
     $               iingar(j,3) .ne. 0 .and. aregar(j,5) .ne. 0.0) then
                     if (iabs(iingar(j,3)) .eq. irun) then
                        if ((ip .eq. 1 .and. iingar(j,3) .gt. 0) .or.
     $                     (ip .eq. 2 .and. iingar(j,3) .lt. 0)) then
                           k = k+1
c                    Delta
                           aregar(k,6) = aregar(j,5)
c                    Index
                           iingar(k,5) = iingar(j,4)
                        endif
                     endif
                  endif
 401           continue
c Do normal probability analysis
               if (k .gt. 0) then
                  call nrmprb(0, maxptd, aregar(1,6), k, dltlim, prblim,
     $               obslim(irun), slope(1,ip), sintct(1,ip),
     $               nused(1,ip))
                  if (ip .eq. 1) then
                     label = 'Fulls'
                  else
                     label = 'Partials'
                  endif
                  write (lunout, 6101) irunum(irun), label, k,
     $               slope(1,ip), sintct(1,ip), dltlim,
     $               nused(2,ip), slope(2,ip), sintct(2,ip)
 6101             format (/1x,'====== Run number:',i4,', ',a,' ======'/
     $               35x,'Number     Slope Intercept'/
     $               '                        All data:',i8,2f10.3/
     $               ' Data within expected delta',f5.2,':',i8,2f10.3/)
               endif
c.................
 402        continue
            if (nused(2,1) .gt. 0 .or. nused(2,2) .gt. 0) then
c  Use slope(2) to update SDcorrection multiplier
               write (lunout, 6102) ((sdfact(i,irun,j),i=1,3),j=1,2)
 6102          format (
     $            ' Update of SDcorrection parameters:'/
     $            15x,'            Fulls               ',
     $            '         Partials'/
     $            1x,' initial values:',3f8.4,5x,3f8.4)
               if (slope(2,1) .ne. 0.0)
     $            sdfact(1,irun,1) = sdfact(1,irun,1) * slope(2,1)
               if (slope(2,2) .ne. 0.0)
     $            sdfact(1,irun,2) = sdfact(1,irun,2) * slope(2,2)
               write (lunout, 6103) (sdfact(1,irun,j),j=1,2)
 6103          format ('      changed to:', f8.4,21x,f8.4/)
            else
               write (lunout, '(a)')
     $            ' SDcorrection parameters not updated: no data'
            endif
 400     continue
c  End run loop for normal probability analysis & updating SDcorr
         write (lunout, '(a/)')
     $      ' =========================================='
      endif
c
c============================================
c Now do analysis of anomalous differences using updated Sdfactors
      if (ldoanom) then
         k = 1
         jrf = 0
c
 100     call nxtrfl(k,
     $      nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $      maxrfl, nrs, ffrf, sdffrf, ghl, irunrf, mrf, idxrf,
     $      clprf, isttrf, isymrf, icntrc)
c  do while k .le. irfidx (essentially)
         if (k .gt. 0) then
            if (icntrc .eq. 0) then
c     Correct sds for selected I
c  Apply current SD correction
               call sdcorr(nrs, ffrf, sdffrf, clprf, ghl, isttrf,
     $            isymrf, irunrf, mrf, swih)
c Set variance weights
               call setwgt(2, nrs, ffrf, sdffrf, ghl, wtrf)
c  We don't need to select datasets (runs) here because inactive
c runs have already been removed
               iselds = 0
               janom = 1
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, .false., useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, aihpls, sdihpl, uh, nupls)
               janom = 2
               call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $            irefrn, iselds, .false., useflg)
               call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $            wtrf, aihmns, sdihms, uh, numns)
c
               if (nupls .gt. 0 .and. numns .gt. 0) then
                  varipl = sdihpl**2
                  varims = sdihms**2
c Store DelAnom/sd for normal analysis & increment count
                  jrf = jrf + 1
                  aregar(jrf,5) =
     $               (aihpls - aihmns)/sqrt(varipl + varims)
               endif
            endif 
            go to 100
         endif
c
         write (lunout, '(/a/a)')
     $      ' ================================================',
     $      ' Normal probability analysis of anomalous differences'
         ip = 1
         call nrmprb(
     $      0, maxptd, aregar(1,5), jrf, dltlim, prblim,
     $      obslim(1), slope(1,ip), sintct(1,ip), nused(1,ip))
         labfpt = 'Anomalous differences'
         l = lenstr(labfpt)
         write (lunout, 6151) labfpt(1:l),
     $      nused(1,ip), slope(1,ip), sintct(1,ip), dltlim,
     $      nused(2,ip), slope(2,ip), sintct(2,ip)
 6151    format (
     $      /1x,'====== ',a,' ======'/
     $      35x,'Number     Slope Intercept'/
     $      '                        All data:',i8,2f10.3/
     $      ' Data within expected delta',f5.2,':',i8,2f10.3//)
c
         slpanom(1) =  slope(1,ip)
         slpanom(2) =  slope(2,ip)
      endif
c
      return
      end
c


      subroutine sdcopt(nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $   nfulls, npartial)
c    =====================
c
c Optimise SD correction parameters
c
c On entry:
c  nrfgar        number of observations in file
c  iingar        integer array
c  aregar        real array
c
c   iingar(,1)  (a) for first observation in each reflection,
c                  = number of observations (negated if I-)
c               (b) for subsequent observations = +/- 1
c               .gt. 0 for I+ (or centric), .lt. 0 I-
c   iingar(,2) packed run number, partial status, centric flag 
c
c   aregar(,1)  Ihl
c   aregar(,2)  sd(Ihl)
c   aregar(,3)  ghl  scale
c   aregar(,4)  LP
c  irfidx       number of observations in arrays
c  nfulls(irun) number of fulls in run
c  npartial(irun) number of partials in run
c
c On exit:
c  Updated SD correction parameters in common /sdfacc/
c
      implicit none
      integer nrfgar, ningar, nregar, irfidx
      integer iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)
      integer nfulls(*), npartial(*)
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'sdcdata.fh'
      include 'sdfacc.fh'
      include 'rundef.fh'
      include 'rfile.fh'
      include 'outcon.fh'
c
      double precision sdcresid, value
      external  sdcresid
c
      integer maxsdpar, i, ntfp(2)
      parameter (maxsdpar = maxrun*maxsdf*2)
      integer irun, lcntarr(6), maxcyc, istat, k,
     $   kpass, npass, kfp(maxrun), kextra
      real f, rcntarr(1) 
      real parinc(maxsdpar)
      double precision fail, tolerance
      double precision points(maxsdpar*(maxsdpar+1))
      logical gradient, lfew, lfp, copypar

c  Criteria for only a "few" fulls or partials
c  "Few" if either 
c     1. fraction (f = full/partial or 1/f)  .lt. FMINFEW
c     2. number .lt. MINFEW   and f .lt. 0.5
c
c  In the case of "few", the minority groups is reoptimised on its own
c in a last pass , fixing the parameters for the majority
c  NB the "few" options are currently inactivated, so this isn't done
c  actions commented out below
      real    FMINFEW
      integer MINFEW
c
c  IBINSIZE desired minimum average number in intensity bins
c  MINIBINS minimum number of intensity bins 
      integer IBINSIZE, MINIBINS, MAXIBINS
c
      data FMINFEW /0.03/
      data MINFEW  /100/
      data IBINSIZE/2000/, MINIBINS/10/,
     $   MAXIBINS/max_ibin/

c
c Count total fulls & partials
      ntfp(1) = 0
      ntfp(2) = 0
      lfew = .false.
c Check for runs with very few fulls (or partials)
      do irun = 1, numrun
         ntfp(1) = ntfp(1) + nfulls(irun)
         ntfp(2) = ntfp(2) + npartial(irun)
c   kfp(irun) =  0   fulls & partials
c             = -2   only fulls
c             = -1   few fulls
c             = +1   few partials
c             = +2   only partials
c   kfullpart(irun)
c             =  0  fulls only
c             = +1  partials only
c             = +2  partials + fulls
         if (npartial(irun) .eq. 0) then
            kfp(irun) = -2
            kfullpart(irun) = 0
         else if (nfulls(irun) .eq. 0) then            
            kfp(irun) = +2
            kfullpart(irun) = +1
         else
            kfullpart(irun) = +2
            kfp(irun) = 0
            f = nfulls(irun)/float(npartial(irun))
            if (f .lt. FMINFEW .or.
     $         (nfulls(irun) .lt. MINFEW .and. f .lt. 0.5)) then
c      Few fulls
               kfp(irun) = -1
               lfew = .true.
            else if (1.0/f .lt. FMINFEW .or.
     $            (npartial(irun) .lt. MINFEW .and. f .lt. 0.5)) then
c      Few partials
               kfp(irun) = +1
               lfew = .true.
            endif
         endif
      enddo 

c  lfp .true. if both fulls & partials
      lfp = .true.
      if (ntfp(1) .eq. 0 .or. ntfp(2) .eq. 0) lfp = .false. 

c  Set up intensity bins
      if (avgins .gt. 0.0) then
c    (a) use average intensity from initial or scaling pass, if available
         binmid = avgins
      else
c    (b) estimate from maximum intensity in file
         binmid = 0.02 * flmxin
      endif

c  Set number of intensity bins taking into account total,
c fulls & partials separately
c Number of observations  = ntfull or ntpart
c  Normally use the maximum, but reduce it if there are few observations
      do i=1,2
         nibin(i) =
     $      min(max(ntfp(i)/(numrun*IBINSIZE), MINIBINS), MAXIBINS)
c Set intensity bin limits, top limit = file maximum
         call setbrn(kbnrtp, nibin(i), binmid, nibin(i)/2,
     $      bin_ibin(1,i))
         bin_ibin(nibin(i),i) = flmxin
c DEBUG
c         do k=1,nibin(i)
c            write (6,'(a,2i6,f12.1)')
c     $         'Intensity range', i, k, bin_ibin(k,i)
c         enddo
c-DEBUG
      enddo

      fail = -999.
      maxcyc = 200
c      gradient = .false.
c  Kevin's weighted shift
      gradient = .true. 
      lcntarr(2) = nrfgar
      lcntarr(3) = ningar
      lcntarr(4) = nregar
      lcntarr(5) = irfidx
c  Print flag
      lcntarr(6) = 0

c How many passes?
c
c  1st pass as specified on input (default INDIVIDUAL)
c  2nd pass (if fulls & partials) for the worst of full/partial for each run
c  3rd pass (if > 1 run && not UNIFORM) do it again
c  if UNIFORM then one pass only
      npass = 2
      if (lfp) then
         npass = npass + 1
      endif
c  Input option to use UNIFORM parameters
      if (ksdtype .eq. 0) npass = 1

c NoFewOption      if (lfew) npass = npass+1

      write (lunout, 6010)
 6010 format (/'===== Refinement of SD correction parameters')

      write (lunout, '(/a,f10.0,a,i3,3x,i3/)')
     $   'Intensity bins: mid-bin intensity = ', binmid,
     $   '  number of bins ', nibin(1), nibin(2)

      do irun=1,numrun
         if (kfp(irun) .eq. 0) then
            write (lunout, 6022) irunum(irun), nfulls(irun),
     $         npartial(irun)
 6022       format (' Run ',i4,' has fulls & partials, ',2i8)
         else if (kfp(irun) .eq. -1) then
            write (lunout, 6020) irunum(irun), nfulls(irun)
 6020       format (' Run ',i4,' has relatively few fulls, ',i5)
         else if (kfp(irun) .eq. +1) then
            write (lunout, 6021) irunum(irun), npartial(irun)
 6021       format (' Run ',i4,' has relatively few partials, ',i5)
         else if (kfp(irun) .eq. -2) then
            write (lunout, 6023) irunum(irun)
 6023       format (' Run ',i4,' has fulls only, ')
         else if (kfp(irun) .eq. +2) then
            write (lunout, 6024) irunum(irun)
 6024       format (' Run ',i4,' has partials only, ')
         endif
      enddo

c - - - - - - - - - - - - Loop passes
c Possibly 1,2 or 3 passes
c  Pass 1: always UNIFORM ie just one set of parameters
c  If fulls & partials present, last pass is just for the worst class
c  If more tha one run, 2nd pass uses different parameters for each run
c   (INDIVIDUAL)
c       
      do kpass=1,npass
         tolerance = 0.0001
c         tolerance = 0.0000001
         lsdtype = ksdtype
c
c DEBUG
c         write (6,*) 'ksdtype, lsdtype', ksdtype, lsdtype
c Flag for printing something special on last pass
c  kextra  =  0 no print
c          = -1 last cycle refining fulls only on all runs
c          = +1 last cycle refining partials only on all runs
c          = +2 last cycle refining fulls only on some runs
         kextra = 0
c  Print flag
         lcntarr(6) = 0
c  Flag to copy undetermined parameters from determined ones
         copypar = .true.
         if (kpass .eq. 2 .and. lfp) then
c  Second pass and both fulls & partials
c   Optimise full/partial class with worst residual 
c  Don't upset previously determined parameters
            copypar = .false.
            do irun=1,numrun
               if (residrun(irun,1) .gt. residrun(irun,2)) then
c    bad fulls, refine fulls only
                  kfullpart(irun) = 0
                  if (kextra .le. 0) then
                     kextra = -1
                  else
                     kextra = +2
                  endif
               else
c    bad partials, refine partials only
                  kfullpart(irun) = +1
                  if (kextra .ge. 0) then
                     kextra = +1
                  else
                     kextra = +2
                  endif
               endif
            enddo
         else if (kpass .eq. npass) then
c  Last pass, reset kfullpart flags
            kextra = 0
            do irun=1,numrun
               if (npartial(irun) .eq. 0) then
                  kfullpart(irun) = 0
               else if (nfulls(irun) .eq. 0) then            
                  kfullpart(irun) = +1
               else
                  kfullpart(irun) = +2
               endif
            enddo
         endif
c NoFewOption: didn't seem to work very well
cNF         if (lfew .and. kpass .eq. npass) then
cNFc  Last pass if at least one run was flagged as having "few"
cNFc  fulls or partials
cNF            do irun=1,numrun
cNF               if (kfp(irun) .eq. -1) then
cNFc    few fulls, refine fulls only
cNF                  kfullpart(irun) = 0
cNF                  kextra = -1
cNF               else if (kfp(irun) .eq. +1) then
cNFc    few partials, refine partials only
cNF                  kfullpart(irun) = +1
cNF                  kextra = +1
cNF               endif
cNF            enddo
cNF         endif

c Increments on each parameter from starting values, to start simplex
         if (kpass .eq. 1) then
            parinc(1) = 1.0
            parinc(2) = 10
            parinc(3) = 0.1
         else
            parinc(1) = 0.3
            parinc(2) = 3
            parinc(3) = 0.02
         endif

c  First point will be at "current" parameter values  
         call setsdstart(points, parinc)
         lcntarr(1) = nsdparam

c Initial residual
         resid = sdcresid(points,
     $      lcntarr, rcntarr, iingar, aregar)
         write (lunout, '(/a,i2,a,f10.5,5x,a)')
     $      '>>>>> Pass ', kpass,
     $      '    Initial residual: ', resid, '<<<<<<'
         call prtrunresid
         if (numrun .gt. 1) then
            if (lsdtype .eq. 0) then
               write (lunout, '(/a,a/)') '   Option: ',
     $          'UNIFORM       SD parameters for all runs are the same'
            else if (lsdtype .eq. +1) then
               write (lunout, '(/a,a/)') '   Option: ',
     $     'INDIVIDUAL    SD parameters are different for each run'
            else if (lsdtype .eq. +2) then
               write (lunout, '(/a,a/)') '   Option: ',
     $       'COMMON        SdFac parameters for all runs are the same'
            endif
         else
            if (lsdtype .eq. 0) then
               write (lunout, '(/a,a,a/)') '   Option: ',
     $            'UNIFORM       SD parameters for fulls and partials',
     $            ' are the same'
            else if (lsdtype .eq. +1) then
               write (lunout, '(/a,a,a/)') '   Option: ',
     $            'INDIVIDUAL    SD parameters are different for ',
     $            'fulls and partials'
            endif
         endif
         if (kextra .lt. 0) then
            write (lunout, 6030) 'fulls'
         else if (kextra .eq. +1) then
            write (lunout, 6030) 'partials'
         else if (kextra .eq. +2) then
            write (lunout, 6030) 'fulls or partials'
 6030       format (' 2nd pass, optimisation of parameters for ',
     $         a,' only')
         endif
c DEBUG
c         write (lunout, '(a,100f8.3)')
c     $      'Parameters-before:',(points(i), i=1,nsdparam)
c-DEBUG         
         call simplex(nsdparam, sdcresid,
     $      lcntarr, rcntarr, iingar, aregar, fail, tolerance, maxcyc,
     $      gradient, points, value)
         
         write (lunout, 6001) value, maxcyc
 6001    format (' Best residual',f10.5,' after',i4,' cycles') 
c DEBUG
c         write (lunout, '(a,100f8.3)')
c     $      'Parameters-after: ',(points(i), i=1,nsdparam)
c         write (6,*) 'kfullpart',(kfullpart(irun),irun=1,numrun)
c-DEBUG

c Store optimised SD correction parameters
         call map_sdparam(points, nsdparam, istat)
         if (copypar) then
c Copy new parameters to  full/partial slots which were not refined
            do irun = 1, numrun
               if (kfullpart(irun) .eq. 0) then
c   refined fulls only, copy to partial
                  do i = 1,maxsdf
                     sdfact(i,irun,2) = sdfact(i,irun,1)
                  enddo
               else if (kfullpart(irun) .eq. +1) then
c   refined partials only, copy to full
                  do i = 1,maxsdf
                     sdfact(i,irun,1) = sdfact(i,irun,2)
                  enddo
               endif
            enddo
         endif

c Recalculate residual
c  Update parameters
cNF         do irun=1,numrun
cNF            if (kfp(irun) .lt. 0) then
cNFc    fulls only
cNF               kfullpart(irun) = 0
cNF            else if (kfp(irun) .gt. 0) then
cNFc    partials only
cNF               kfullpart(irun) = +1
cNF            else
cNF               kfullpart(irun) = +2
cNF            endif
cNF         enddo
         parinc(1) = 0.0
         parinc(2) = 0.0
         parinc(3) = 0.0
         call setsdstart(points, parinc)
         lcntarr(1) = nsdparam
c  Print flag
         lcntarr(6) = 1
         resid = sdcresid(points,
     $      lcntarr, rcntarr, iingar, aregar)
         call prtrunresid
c  Print SD tables on last pass (DEBUG)
c         if (kpass .eq. npass) call prtsdp(kfp)

c   end pass loop
      enddo         
      end


      subroutine  prtrunresid
c     =======================
c
c  Print run residuals
c
c
      implicit none
      include 'parameter.fh'
      include 'inout.fh'
      include 'sdcdata.fh'
      include 'rundef.fh'
c
      integer NRUNPERLINE
      integer i,j,k,l, nsplit, kmax
      if (numrun .le. 1) return

      NRUNPERLINE = 5
      nsplit = (numrun-1)/NRUNPERLINE + 1
      k = 1
      do j = 1,nsplit
         kmax = min(numrun, k+NRUNPERLINE-1)
         write (lunout, 6001) (irunum(i), i=k,kmax)
 6001    format ('       Run: ',10(7x,I6,5x))
         write (lunout, 6002) ((residrun(i,l), l=1,2), i=k,kmax)
 6002    format ( ' Residuals: ',10(2f8.5,2x))
         k = k+ NRUNPERLINE
      enddo
      return 
      end


      subroutine prtsdp(kfp)
c    ===============
c
c Print SD tables at end of refinement
c   kfp(irun) =  0   fulls & partials
c             = -2   only fulls
c             = -1   few fulls
c             = +1   few partials
c             = +2   only partials
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'sdcdata.fh'
      include 'rundef.fh'
c
      integer kfp(maxrun)
c
      integer i,j,k,irun, mibin


      mibin = max(nibin(1), nibin(2))

      do irun = 1,numrun
         write (lunout, 6001) irunum(irun), irunum(irun)
 6001    format (/'SD plot after refinement (for debugging): run',i5//
     $      '$TABLE: Run ',i4,' SD v intensity:'/)

         if (abs(kfp(irun)) .le. 1) then
c Fulls & partials
            write (lunout, 6002)
6002       format ('$GRAPHS:Sigma(scatter/SD):A:1,2,4: $$'//
     $    '     Range  SdFulls  Nfulls SdPartial NPartial $$  $$'/)
            k1 = 1
            k2 = 2
         else if (kfp(irun) .lt. 0) then
c Fulls only
            write (lunout, 6003) 'SdFulls  Nfulls'
 6003       format ('$GRAPHS:Sigma(scatter/SD):A:1,2: $$'//
     $         '     Range  ',a,'  $$  $$')
            k1 = 1
            k2 = 1
         else 
c  Partials only
            write (lunout, 6003) 'SdPartial NPartial'
            k1 = 2
            k2 = 2
         endif
         do i = 1, mibin
            write (lunout, 6010) i, (sddelta(irun,i,j),
     $         nsddelta(irun,i,j), j=k1,k2)
 6010       format (5x,i4,2(f9.2,i9))
         enddo
         write (lunout, '(a/)') '$$'
      enddo
      return
      end
      subroutine sdcorr
     $   (nrs, ffrf, sdffrf, clprf, ghl, isttrf, isymrf, irunrf, mrf,
     $   swih)
c     ========================================================
c
c     Correct sds for selected I
c This is done after summing partials, but before outlier rejection
c
c On entry:
c  mrf(l) = 0 full, = +1 partial
c
c On exit:
c  sdffrf(nrs)    corrected standard deviations
c  swih           intensity used for correction
c
      integer nrs
      real ffrf(nrs), sdffrf(nrs), ghl(nrs), clprf(nrs), swih
      integer isttrf(nrs), isymrf(nrs), irunrf(nrs), mrf(nrs)
c
      include 'parameter.fh'
      include 'rundef.fh'
c
      integer l, janom, iref, nused, iselds
      real dummy(1), dummyr, sdih, uh
c
      real wtrf(maxrfl)
      logical useflg(maxrfl)
c
c Calculate sqrt-scale-weighted mean to use for correcting sds,
c exclude reference run
c    
      janom = 0
      iselds = 0
c Set usage flags
      call chkusf(nrs, janom, isttrf, isymrf, irunrf, mrf,
     $   irefrn, iselds, .false., useflg)
c Set sqrt-scale weights
      call setwgt(3, nrs, ffrf, sdffrf, ghl, wtrf)
c Get mean intensity
      call menint(nrs, ffrf, sdffrf, ghl, useflg,
     $   wtrf, swih, sdih, uh, nused)
c Correct sds
      do 50, l = 1, nrs
         call corvar(.false., irunrf(l),  sdffrf(l), 
     $      clprf(l), ghl(l)*swih, mrf(l), sdffrf(l), dummy, dummyr)
 50   continue
c
      return
      end
      double precision function sdcresid
     $   (params, lcntarr, rcntarr, intgarr, realarr)
c    ===============
c
c Routine to evaluate residual to be optimised for SD correction
c
c On entry:
c   params   array of nparam parameters
c   lcntarr  integer array of control information
c            (1) nparam  number of parameters
c            (2) nrfgar  1st dimension of data arrays
c            (3) ningar  2nd dimension of intgarr
c            (4) nregar  2nd dimension of realgarr
c            (5) irfidx  number of observations in data arrays
c            (6) iprint  .gt. 0 for extra printing
c   rcntarr  real array of control information
c   intgarr  integer array of data
c            2D array of dimensions (nrfgar,ningar)
c   realarr  real array of data
c            2D array of dimensions (nrfgar,nregar)
c
c  Return -1000 if parameters out of range
c
      implicit none
      double precision params(*)
      integer lcntarr(*), intgarr(*)
      real rcntarr(*), realarr(*)

      double precision sdcresid2
      external sdcresid2
      
      sdcresid = sdcresid2(params, lcntarr(1),
     $   lcntarr(2), lcntarr(3), intgarr, lcntarr(4), realarr,
     $   lcntarr(5), lcntarr(6))

      end


      double precision function sdcresid2(params, nparam,
     $   nrfgar, ningar, iingar, nregar, aregar, irfidx, iprint)
c    ==================
c 
c  nrfgar        number of observations in file (array dimension)
c  iingar        integer array
c  aregar        real array
c
c   iingar(,1)  (a) for first observation in each reflection,
c                  = number of observations (negated if I-)
c               (b) for subsequent observations = +/- 1
c               .gt. 0 for I+ (or centric), .lt. 0 I-
c   iingar(,2) packed run number, partial status, centric flag 
c
c   aregar(,1)  Ihl
c   aregar(,2)  sd(Ihl)   initial correction applied
c   aregar(,3)  ghl  scale
c   aregar(,4)  LP
c    Note that no outlier rejection has been done on these data
c  irfidx       number of observations in arrays
      implicit none
      integer nparam, nrfgar, ningar, nregar, irfidx, iprint
      double precision  params(nparam)
      integer iingar(nrfgar, ningar)
      real    aregar(nrfgar, nregar)

      include 'parameter.fh'
      include 'outcon.fh'
      include 'flags.fh'
      include 'anomtc.fh'
      include 'sdcdata.fh'
      include 'rundef.fh'
      include 'sdfacc.fh'

      integer i, j, k, l, nrs, jhkl(3), mbn_i(2), krej2, nuse, nrej, j1,
     $   irun, mp, k1, k2, kwgtyp, nn, nrt
      real swih, swihpl, swihmn
      double precision dn, sd, w, sw
      real ffrf(maxrfl), sdffrf(maxrfl), wtrf(maxrfl), ghl(maxrfl),
     $   clprf(maxrfl), delrf(maxrfl),
     $   sd0(maxrfl)
      integer irunrf(maxrfl), mrf(maxrfl), idxrf(maxrfl),
     $   isttrf(maxrfl), isymrf(maxrfl), lanmtc(maxrfl)
      integer icntrc, istat
      logical lrejct, lprint, ldebug
c
      data lprint/.false./
      data ldebug/.true./
      data nn /0/
      save ldebug, nn

c Sums etc
      double precision sum_delta(maxrun,max_ibin,2),
     $   sum_delta2(maxrun,max_ibin,2)
      integer n_delta2(maxrun,max_ibin,2), nr(maxrun,2),
     $   n_allruns(max_ibin,2)
c Variance weights
      kwgtyp = 2

c DEBUG
c      write (6,'(a,100f10.2)') 'StartResidParams',(params(i),i=1,nparam)

c Setup SD correction parameters
c Parameters used by s/r corvar are in common /sdfacc/
c   sdfact(1, irun, m)  Sdfac
c   sdfact(2, irun, m)  SdB
c   sdfact(3, irun, m)  SdAdd
c  m = 1 full, = 2 partial
c Set common block parameters ready for corvar
c  Return istat = -1 if params are out of range
      call map_sdparam(params, nparam, istat)
      if (istat .lt. 0) then
         sdcresid2 = -1001.
         return 
      endif
c DEBUG      
c      do irun=1,numrun
c         write (6, 6103) irunum(irun),
c     $      ((sdfact(i,irun,j),i=1,3),j=1,2)
c 6103    format(5x,i5,3x,3f8.4,5x,3f8.4)
c      enddo
c- DEBUG
      k = 1
      lrejct = .true.
      call ccpzi(jhkl,3)
      krej2 = krej2o
      call ccpzi(lanmtc, maxrfl)

      call ccpzd(sum_delta, maxrun*max_ibin*2)
      call ccpzd(sum_delta2, maxrun*max_ibin*2)
      call ccpzi(n_delta2, maxrun*max_ibin*2)
      call ccpzi(nr, maxrun*2)
      call ccpzi(n_allruns, max_ibin*2)
      call ccpzr(sddelta, maxrun*max_ibin*2)
      call ccpzi(nsddelta, maxrun*max_ibin*2)
c
 10   call nxtrfl(k,
     $   nrfgar, ningar, iingar, nregar, aregar, irfidx,
     $   maxrfl, nrs, ffrf, sdffrf, ghl, irunrf, mrf, idxrf, 
     $   clprf, isttrf, isymrf, icntrc)
c  do while k .le. irfidx (essentially)
      if (k .gt. 0) then
c DEBUG
c         call ccpmvr(sd0, sdffrf, nrs)
c  Apply current SD correction
         call sdcorr(nrs, ffrf, sdffrf, clprf, ghl, isttrf, isymrf,
     $      irunrf, mrf, swih)
c Set variance weights
         call setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c Calculate Delta (Chi) for each observation
c Reject of outliers (lrejct .true.)
c  and calculate deviations
c  Always do it only within datasets
         call choutl(nrs, kanomo, lprint, lrejct, .true., icntrc,
     $      jhkl, ffrf, sdffrf, wtrf, ghl, irunrf, mrf, isttrf,
     $      isymrf, lanmtc, nuse, nrej,
     $      sdrejm, sdrejm2, sdrejma, sdrejm2a,
     $      krej2, delrf,
     $      swih, swihpl, swihmn)
c
         if (nuse .lt. 2) go to 10
c  Intensity bins, fulls & partials from swih
         do j=1,2
            do i = 1, nibin(j)
               if (swih .lt. bin_ibin(i,j)) then
                  mbn_i(j) = i
                  go to 47
               endif
            enddo 
            mbn_i(j) = ninbin
 47         continue
         enddo
c Sum deviations
c Binned by run, intensity and full/partial
c  Note that initial sd corrections have already been added to sd(I)
         do l = 1,nrs
            if (isttrf(l) .eq. lqOK .and. irunrf(l) .gt. 0) then
               if (delrf(l) .ne. 0.0) then
                  mp = mrf(l)+1
                  sum_delta(irunrf(l),mbn_i(mp),mp) =
     $               sum_delta(irunrf(l),mbn_i(mp),mp) + delrf(l)
                  sum_delta2(irunrf(l),mbn_i(mp),mp) =
     $               sum_delta2(irunrf(l),mbn_i(mp),mp) + delrf(l)**2
                  n_delta2(irunrf(l),mbn_i(mp),mp) =
     $               n_delta2(irunrf(l),mbn_i(mp),mp) + 1
                  nr(irunrf(l),mp) = nr(irunrf(l),mp) +1
c DEBUG
c                  if (iprint .gt. 0 .and. k .lt. 5) then
c                  if (ldebug) then
c                  if (nn .eq. 0) then
cc                     write (6,'(i8,5i4,3f10.1,f10.4,f10.1,f10.4)')
c                     write (13,'(i8,5i4,3f10.0,f10.2,f10.0,f10.2)')
c     $                  k, jhkl, nrs, l, ffrf(l), sd0(l), sdffrf(l),
c     $                  ghl(l), swih, delrf(l)
c                  endif
c-DEBUG
               endif
            endif
         enddo
         go to 10
      endif
c DEBUG
c      do irun=1,numrun
c         write (6,'(a,i5,2i9)') 'Numbers in runs ',
c     $      irun, (nr(irun,j),j=1,2)
c      enddo 
c-DEBUG
c
c Residual is Sum[w *(1-SD)**2] but what weight?
c  Sum numbers of all runs
      do irun = 1,numrun 
         if (kfullpart(irun) .eq. 0) then
c Fulls only
            k1 = 1
            k2 = 1
         else if (kfullpart(irun) .eq. +1) then
c Partials only
            k1 = 2
            k2 = 2
         else if (kfullpart(irun) .eq. +2) then
c Fulls & partials
            k1 = 1
            k2 = 2
         endif
         do j = k1,k2
            do i = 1,nibin(j)
               n_allruns(i,j) = n_allruns(i,j) +
     $            n_delta2(irun,i,j)
            enddo
         enddo
      enddo

      resid = 0.0
      sw = 0.0
      nrt = 0
      do irun = 1,numrun 
         residrun(irun,1) = 0.0
         residrun(irun,2) = 0.0
         swrun(irun,1) = 0.0
         swrun(irun,2) = 0.0
         if (kfullpart(irun) .eq. 0) then
c Fulls only
            k1 = 1
            k2 = 1
         else if (kfullpart(irun) .eq. +1) then
c Partials only
            k1 = 2
            k2 = 2
         else if (kfullpart(irun) .eq. +2) then
c Fulls & partials
            k1 = 1
            k2 = 2
         endif
         do j = k1,k2
            do i = 1,nibin(j)
               if (n_delta2(irun,i,j) .gt. 0) then
                  dn = dfloat(n_delta2(irun,i,j))
c  Unit weights
c                  w = 1.0
c                  w = dn
c N weights over all runs
                  w = dfloat(n_allruns(i,j))
c Sqrt(N) weights
                  w = sqrt(w)
c                  w = sqrt(dn)
c 1/N weights
c                  w = 1./dn
                  sd = sum_delta2(irun,i,j)/dn -
     $               (sum_delta(irun,i,j)/dn)**2
c
                  if (sd .ge. 0.0) then
                     sd = sqrt(sd)
c DEBUG
c                     if (iprint .gt. 0) then
c                        write (6,
c     $                  '(a,i5,f10.1,f10.3,i6,2i5,f10.1,f10.2)')
c                     write (13,
c     $                  '(a,i5,f10.1,f10.2,i6,2i5,f10.1,f10.1)')
c     $                  'jI, ibin, BinSD, n, run, FP, w',
c     $                  i, bin_ibin(i,j), sd, n_delta2(irun,i,j),
c     $                  irun, j, w, w*((1.0 - sd)**2)
c                     endif
c-DEBUG
cc Try unweighted resid sum over runs
cc                     resid = resid + w*(1.0 - sd)**2
cc                     sw = sw + w
                     residrun(irun,j) = residrun(irun,j) +
     $                  w*((1.0 - sd)**2)
                     swrun(irun,j) = swrun(irun,j) + w
                     if (i.eq.1) nrt = nrt + nr(irun,j)
                  endif
                  sddelta(irun,i,j)  = sd
                  nsddelta(irun,i,j) = n_delta2(irun,i,j)
               endif
            enddo
         enddo
      enddo 

cc      if (sw .gt. 0.000001) resid = resid/sw
      resid = 0.0
      sw = 0.0
      do irun=1,numrun
         sd = 0.0
         w = 0.0
         if (swrun(irun,1) .gt. 0.000001) then
            sd = sd + residrun(irun,1)
            w = w + swrun(irun,1)
            residrun(irun,1) = residrun(irun,1)/swrun(irun,1)
         endif
         if (swrun(irun,2) .gt. 0.000001) then
            sd = sd + residrun(irun,2)
            w = w + swrun(irun,2)
            residrun(irun,2) = residrun(irun,2)/swrun(irun,2)
         endif
         if (w .gt. 0.000001) then
c DEBUG
c            if (iprint .gt. 0) then
c               write (6,'(a,i5,2f10.1,f10.4)')
c     $            'Resid for run ',irun,sd,w,sd/w
c            endif
c-DEBUG
            resid = resid + sd/w
            sw = sw + 1.0
         endif
      enddo
      if (sw .gt. 0.000001) resid = resid/sw
      nn = nn+1
c DEBUG
c      write (6,'(i5,a,f15.5,i8)') nn, ' Resid:',resid, nrt

      sdcresid2 = resid
      ldebug = .false.
      end


      SUBROUTINE CCPZD (ARR1,NUM)
C    ===========================
C
C  This routine assigns zero to ARR1 using NUM words
C
C Arguments:
C
C    ARR1 (O)   DOUBLE PRECISION ARRAY(*): array to be zeroed
C     NUM (I)   INTEGER: Number of words
C
C  Arguments ..........
      INTEGER NUM
      DOUBLE PRECISION ARR1(*)
C
      INTEGER J
C
      DO 10 J=1,NUM
   10 ARR1(J)=0.0
      END
c
c
c
      subroutine selanm(icntrc, 
     $   nrs, kanom, jhkl, irunrf, mrf, isttrf, isymrf, phirf, timerf,
     $   lanmtc, nanmrj)
c     ===============================================================
c
c
c  Match anomalous pairs & flag unmatched ones as rejected
c Acentric reflections only
c
c
c On entry:
c  icntrc        = 1 centric, else = 0
c  nrs           number of observations
c  kanom         anomalous flag, = 0 no anomalous
c                = +1   use all anomalous observations
c                = +2   anomalous match <symmetry operation>
c  jhkl(3)       reflection indices
c  irunrf(nrs)   run number, = -1 if rejected
c  mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c                 = -1 for ignored partial if lpflag = .true.
c                 =  1 for previously summed partial (mpart = 10)
c  isttrf(nrs)   status flag
c  isymrf(nrs)   symmetry number
c  phirf(nrs)    Phi (ROT)
c  timerf(nrs)   Time (if present)
c  nanmrj        current count of rejected observations
c
c On exit:
c  isttrf(nrs)   updated status flag
c  lanmtc(nrs)   pointer to matching pair, = 0 if none
c  nanmrj        updated count of rejected observations
c
      integer icntrc, nrs, kanom, nanmrj
      integer jhkl(3), irunrf(nrs), mrf(nrs), isttrf(nrs), isymrf(nrs),
     $   lanmtc(nrs)
      real phirf(nrs), timerf(nrs)
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'flags.fh'
      include 'glocon.fh'
      include 'inout.fh'
c
      integer i, j, k, l, m, istat
      logical luse(maxrfl)
c
c Initialize all use flags to .false., & all pointers null
      do 1, l = 1, nrs
         luse(l) = .false.
         lanmtc(l) = 0
c Don't reject reflections from reference runs
c (they mostly are not used anyway)
         if (irunrf(l) .eq. irefrn) luse(l) = .true.
 1    continue
c
c Don't do anything else if no anomalous match or centric
      if (kanom .le. +1 .or. icntrc .eq. 1) return
c
c-- Outer loop over observations
      do 10, l = 1,nrs-1
         if (isttrf(l) .ge. lqOK .and. .not.luse(l)) then
c 
c---- Inner loop over observations
            do 20, m = l+1, nrs
               if (isttrf(m) .ge. lqOK .and.  .not.luse(m)) then
c Do observations l & m match?
                  call mtcanm(
     $               l, m, nrs, kanom, jhkl, irunrf, mrf,
     $               isymrf, phirf, timerf, istat)
                  if (istat .eq. 0) then
c Yes, flag both as OK
                     luse(l) = .true.
                     luse(m) = .true.
c  Set pointers (so that matched pair can be rejected)
                     lanmtc(l) = m
                     lanmtc(m) = l
                     go to 10
                  endif
               endif
 20         continue
         endif
 10   continue
c
c--
c Update status flags for all observations not accepted
      do 50, l = 1, nrs
         if (isttrf(l) .ge. lqOK) then
            if (.not.luse(l)) then
               isttrf(l) = lquman
               nanmrj = nanmrj + 1
            endif
         endif
 50   continue
c
      if (kprint(1) .ge. 4) then
         write (lunout, '(a)')
     $      'Selanm: h   k   l  Isym  Irun  Ist  lanmtc'
         do 60, l = 1, nrs
            write (lunout, '(i4,3i4,4i5)')
     $         l, jhkl, isymrf(l), irunrf(l), isttrf(l), lanmtc(l)
 60      continue
      endif
c
      return
      end
      subroutine selint(ff, sdff, ffpr, sdffpr, m, imid, ipower)
c     ==================================================
c
c  Select either summed intensity or profile-fitted intensity as
c appropriate
c
c On entry:
c  ff, sdff      summed intensity & sd
c  ffpr, sdffpr  profile-fitted intensity & sd, sdffpr = 0.0 if missing
c  m             partial flag, = 0 full, .ge. 1 partial
c  imid          selection option:
c                 =  0  select summed intensity (ie leave as is)
c                 = -1  select profile-fitted IPR if present
c                 = -2  select profile-fitted IPR for fulls, summed for partials
c                 .gt. 0  use weighted mean between IPR & IINT
c                               I = w * Ipr  +  (1-w) * Iint
c                         where w = 1/(1+(Iint/IMID)**IPOWER)
c
c On exit:
c  ff, sdff      selected intensity & sd
c
      real ff, sdff, ffpr, sdffpr
      integer m, imid, ipower
c
      real w
c
c Select which intensity to use if both are present
      if (imid .eq. -1) then
c   Profile-fitted intensity
         if (sdffpr .gt. 0.000001) then
            ff = ffpr
            sdff = sdffpr
         endif
      elseif (imid .eq. -2) then
c   Profile-fitted intensity for fulls only
         if (m .eq. 0) then
            if (sdffpr .gt. 0.000001) then
               ff = ffpr
               sdff = sdffpr
            endif
         endif
      elseif (imid .gt. 0) then
c   Combined mean
         if (sdffpr .gt. 0.000001) then
            w = 1.0/(1.0 + (abs(ff)/float(imid))**ipower)
            ff = w * ffpr + (1.0-w) * ff
            sdff = sqrt(w * sdffpr**2 + (1.0-w) * sdff**2)
         endif
      endif
      return
      end
c
c
c
      subroutine setacr(jflag)
c     ========================
c
c Set up which runs are active
c Reference run is always active
c
c On entry:
c  jflag        select option
c               = 0 make all runs active
c               .gt. 0 make just runs which belong to this dataset active
c               (using dataset serial number in output list, not dataset ID)
c
      integer jflag
c
      include 'parameter.fh'
      include 'rundef.fh'
      include 'datasets.fh'
c
      integer irun

      if (jflag .le. 0) then
         do irun = 1, numrun
            lactvrun(irun) = .true.
         enddo
         if (irefrn .eq. 0) then
            nactvrun = numrun
         else
            nactvrun = numrun - 1
         endif
         return
      else
         nactvrun = 0
         do irun = 1, numrun
            if (irun .eq. irefrn) then
c  Reference run active
               lactvrun(irun) = .true.
            else
c   Test dataset number
               if (kdtsrn(irun) .eq. jflag) then
c     This run belongs to selected dataset
                  lactvrun(irun) = .true.
                  nactvrun = nactvrun + 1
               else
                  lactvrun(irun) = .false.
               endif
            endif
         enddo
      endif
      end

c
c
c
      subroutine setbasedts
c     =====================
c
c Choose "base" dataset kdtsbase, if not set explicitly on input
c Dispersive differences for analysis are relative to the base dataset
c
c Choose shortest wavelength, if known
c
c kdtsbase = 0 if unset (in common /datasets/)
c On exit: kdtsbase = output dataset serial number for base dataset
c           = -2 if invalid wavelength (zero or negative)
c
      include 'parameter.fh'
      include 'datasets.fh'
c
      integer idts
      real wavel
c
      if (kdtsbase .eq. 0) then
c Not set, so set it here
         wavel = 1000000.
         do idts = 1, ndtsetsout
            if (dwavel_out(idts) .lt. wavel) then
               wavel = dwavel_out(idts)
               kdtsbase = idts
            endif
         enddo
      end if
c
      if (kdtsbase .gt. 0) then
c Check for valid wavelength
         if (dwavel_out(kdtsbase) .lt. 0.001) then
            kdtsbase = -2
         endif
      endif
c
      return
      end
      subroutine setbrn(kbnrtp, nbnlim, binpar, nbnpar, binrng)
c     =========================================================
c
c  Set up binning for intensity analysis
c
c On entry:
c     kbnrtp     binning type
c                = 1   linear
c                = 2   square
c                = 3   exponential
c     nbnlim     number of bin limits
c     binpar     binning parameter: this is the desired value of the
c                bin limit at point nbnpar (ie binrng(nbnpar) = binpar)
c     nbnpar     bin number for input limit binpar
c
c On exit:
c     binrng(nbnlim) upper limits of bins
c                   (top bin, ie binrng(nbnlim), is set = 0.0
c                    this needs to be set outside this routine to the
c                    maximum possible intensity)
c
      implicit none
c
      integer kbnrtp, nbnlim, nbnpar
      real    binpar, binrng(nbnlim)
c
      integer j, nb
      real a, s
c
c
      if (kbnrtp .eq. 1) then
c Linear
         a = binpar / float(nbnpar)
         do 10, j = 1, nbnlim-1
            binrng(j) = a * j
 10      continue
c
      elseif (kbnrtp .eq. 2) then
c Square
         a = sqrt(binpar) / float(nbnpar)
         do 20, j = 1, nbnlim-1
            binrng(j) = (a * float(j))**2
 20      continue
      elseif (kbnrtp .eq. 3) then
c Wilson distribution
c  f(I) = p(I) dI = (1/S) exp ( -I/S)
c  then f(0) = 1/S
c Dividing f(I) into N = NBNLIM divisions we have for the j'th limit
c p(I)j = (1/S)[1 - j/N] =  (1/S) exp ( -I/S)
c  ie   ln(1-j/N) = -I/S
c If B = BINPAR is the value of I at j = j0 = NBNPAR
c  S = - B / (ln(1 - j0/N))
c & Ij = - S ln(1-j/N)
c
         nb = nbnlim
         s = - binpar / alog(1.0 - float(nbnpar)/float(nb))
         do 30, j = 1, nbnlim-1
            binrng(j) = - s * alog(1.0 - float(j)/float(nb))
 30      continue
      endif
      binrng(nbnlim) = 0.0
      return
      end
      subroutine setcry
c     =================
c
c   set up list of datasets for all batches
c (& crystal ids)
c
c
      implicit none
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'nbtchc.fh'
      include 'orient.fh'
      include 'datasets.fh'
c     ..
      integer i, j, ibatch, iprint, kbsetid
c
      do j = 1, ndatasets
         nbtchdts(j) = 0
      enddo
c
      do 10, i  = 1, nbtall
         ibatch = ibtall(i)
c Position header read to batch IBATCH
         call lrbres(1, ibatch)
         iprint = 0
c     Read orientation block
         call lrbat(1,j,nwords,btitle,iprint)
         if (nwords .gt. 0) then
c Check right one (I'm not sure this can ever be wrong)
            if (j .ne. ibatch) then
               write (lunout, '(A,I8,A,I8)')
     $              ' !!!! Orientation data for batch',j
     $              ,' found when expecting data for batch',ibatch
               call ccperr(1,'*** Invalid orientation data ***')
            endif
         else
            ncryst = 0
         endif
c     .. Get Dataset ID from file
         call lrbsetid(1, ibatch, kbsetid)
         if (kbsetid .ne. lbsetid) then
c  Unnecessary safety check
            write (lunout, '(a,3i8)')
     $         ' **** WARNING: dataset inconsistency for batch',
     $         ibatch, kbsetid, lbsetid
         endif
         if (kbsetid .gt. 0) then
            idtsbt(i) = kbsetid
c Count batches
            nbtchdts(kbsetid) = nbtchdts(kbsetid) + 1
c Get serial number for this dataset ID
            do j = 1, ndatasets
               if (idtsets(j) .eq. kbsetid) go to 5
            enddo
c    Shouldn't happen if hklin file is correct
            write (lunout, '(a,i5,a,a,i6)')
     $        ' Batch Dataset ID ', kbsetid, ' not found in input file',
     $        ' for batch ', ibatch
            call ccperr(1,'*** Invalid batch dataset number ***')
         else
c  Dataset ID unset, set it to 1 (ie everything is one dataset)
            if (ndatasets .gt. 1) then
c    Can't do this if there is more than one dataset
               write (lunout, '(/a,a/)')
     $            ' ERROR: the file contains multiple datasets',
     $            ' but batches are not assigned to datasets (SETCRY)'
               call ccperr
     $            (1, '*** batches not assigned to datasets ***')
            endif
            j = 1
            idtsbt(i) = j
         endif
c
 5       jdtsbt(i) = j
c  Default output list = input list
         kdtsbt(i) = j
c FIXME  set crystal ID properly - here ncryst = 0
         icrybt(i) = ncryst
c
 10   continue
c
      end
c
c
c
      subroutine setdtsout(ndtskyin, istat)
c     ======================
c
c Set up output dataset definitions
c These are driven by the run definitions
c
c On entry:
c  ndtskyin          number of datasets defined on command input
c                         = 0 if none
c
c On exit:
c  istat             exit status
c                      =  0  no error
c                      = +1  no datasets defined on input or output
c                      = -1  at least one run contains batches assigned
c                             to different datasets
c                      = -2  "all runs"  assigned to multiple datasets
c                      = -3  a run has not been assigned to a dataset
c                      = -4  reference & non-reference runs mixed in dataset
c
c  Any reference runs are eliminated from output dataset list
c
      integer ndtskyin, istat
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'nbtchc.fh'
      include 'datasets.fh'
c
      integer i, j, k, id, jd, irun, jdataset, id1, id2
      integer jdtsrn(maxrun), koutdts(maxsets), ndtsused(maxsets),
     $   nindts(maxsets), kindts(maxinsets, maxsets)
      real waveln, cell(6)
      logical ldtsrun, lfound, lerrwvl, lerrcell, ldifpname, ldifxname
      character dtslab*100
      character pname*(maxnamelen), xname*(maxnamelen),
     $   dname*(maxnamelen)
      integer lenstr
      external lenstr
c
c  Tolerance for wavelength, cell length, cell angle
c before warning is printed
      real tolwvl, tolcll, tolcng
      data tolwvl, tolcll, tolcng /0.0001, 0.01, 0.05/
c
c========================================================================
      istat = 0
      pname = ' '
      xname = ' '
      dname = ' '
c  Clear all dataset assignments to batches
      do i = 1, nbtall
         kdtsbt(i) = -1
      enddo

      if (ndtskyin .le. 0) then
c
c  No datasets defined on command input
c  Check each run for all batches coming from the same dataset
c  If not fail
         if (ndatasets .le. 0) then
c   None defined on input either
            ndtsetsout = 1
            nrundts(1) = numrun
c   Set up one dummy output dataset (1) + reference dataset if present (2)
            k = 0
            do irun = 1, numrun
               if (irun .ne. irefrn) then
                  k = k+1
                  kdtsrn(irun) = 1
                  krundts(k, 1) = irun
               endif
            enddo
c
            istat = 0
            return
         else
c
c  No runs defined on input but one or more present in input file
            ndtsetsout = -1
         endif
      else 
c
c Runs defined on input
c Check for and eliminate reference dataset
         if (irefrn .gt. 0) then
            k = 0
            do id = 1, ndtskyin
               lfound = .false.
               if (nrundts(id) .gt. 0) then
c   only need to check explicit runs
                  do i = 1, nrundts(id)
                     irun = krundts(i, id)
                     if (irun .eq. irefrn) then
                        lfound = .true.
                     endif
                  enddo
                  if (.not.lfound) then
c    this dataset is not reference, close down list if necessary
                     k = k+1
                     if (k .lt. id) then
                        nrundts(k) = nrundts(id)
                        do i = 1, nrundts(id)
                           krundts(i, k) = krundts(i, id)
                        enddo 
                        pname_out(k) = pname_out(id)
                        xname_out(k) = xname_out(id)
                        dname_out(k) = dname_out(id)
                     endif 
                  endif  
               endif 
            enddo 
            ndtsetsout = k
         else 
            ndtsetsout =  ndtskyin
         endif
      endif
c----
      if (ndtsetsout .lt. 0) then
c  No explicit dataset assignments
c Go through run list to check input dataset assignments
         if (ndatasets .gt. maxsets) then
            write (lunout, '(/a,2i8,a/)')
     $         '*** Too many datasets', ndatasets, maxsets,
     $         ' increase MAXSETS'
            call ccperr(1,
     $         '*** Too many datasets, increase MAXSETS ***')
         endif 

         ldtsrun = .false.
         do i = 1, ndatasets
            ndtsused(i) = 0
         enddo
c
c   Loop runs
         do irun = 1, numrun
            jdataset = -1
c     Loop all batches checking if they belong to this run
            do i = 1, nbtall
               if (irunbt(i) .eq. irun) then
c      This batch is in this run: jdataset here is input dataset number
                  if (jdataset .lt. 0) then
                     jdataset = jdtsbt(i)
                  else
                     if (jdataset .ne. jdtsbt(i)) then
c    this batch is in different dataset from first one in run
                        ldtsrun = .true.
                     endif
                  endif
               endif
            enddo
c
            if (ldtsrun) then
               write (lunout, '(/a,i5,a/a//)')
     $            '==== ERROR: in run ', irunum(irun),
     $            ' not all batches belong to the same dataset ====',
     $            '     This is not allowed'
               istat = -1
               return
            endif
            if (irun .eq. irefrn) then
               jdtsrn(irun) = -1
               if (ndtsused(jdataset) .eq. 0) ndtsused(jdataset) = -1
            else 
c  jdataset is input dataset number for this run
               jdtsrn(irun) = jdataset
c    Reset used flag if it has been set to reference
               if (ndtsused(jdataset) .lt. 0) ndtsused(jdataset) = 0
               ndtsused(jdataset) = ndtsused(jdataset) + 1
            endif
         enddo
c
c  Check input dataset list for duplicates: project & crystal name 
c are ignore, only the dataset name counts to identify unique dataset
c ie bits of the same dataset from different crystals are recognised
c as belonging to the same dataset
         ndtsetsout = 0
         do id = 1, ndatasets 
            if (ndtsused(id) .gt. 0) then
               call nospch(pname, pname_in(id))
               call nospch(xname, xname_in(id))
               call nospch(dname, dname_in(id))
c Have we got this dataset already?
               if (ndtsetsout .gt. 0) then
                  lfound = .false.
                  do k = 1, ndtsetsout
                     if (dname .eq. dname_out(k)) then
c   Yes we have it already
                        lfound = .true.
                        jd = k
                     endif
                  enddo
               else
c  1st one
                  lfound = .false.
               endif
c
               if (.not. lfound) then
c   Add this dataset to output list
                  ndtsetsout = ndtsetsout + 1         
                  jd = ndtsetsout
                  pname_out(jd) = ' '
                  xname_out(jd) = ' '
                  dname_out(jd) = dname
               endif
               koutdts(id) = jd
               nrundts(jd) = 0
            endif
         enddo
c Set output dataset numbers for each run, and record runs in this dataset
         do irun = 1, numrun
            if (irun .eq. irefrn) then
               kdtsrn(irun) = -1
            else
               k = koutdts(jdtsrn(irun))
               kdtsrn(irun) = k
               nrundts(k) = nrundts(k) + 1
               krundts(nrundts(k), k) = irun
c   Batches assigned to this run/dataset
               do i = 1, nbtall
                  if (jdtsbt(i) .eq. jdtsrn(irun)) then
                     kdtsbt(i) = kdtsrn(irun)
                  endif
               enddo
            endif
         enddo
      endif
c
c========================================================================
c  All runs must be assigned to a dataset
c
c  Number of dataset assigned is ndtsetsout
c  Already assigned for each dataset above or in keyin:-
c    p/x/dname_out     names
c    nrundts           number of runs in this dataset,
c                         = -1 if all runs (excluding reference run)
c    krundts           list of runs input
c
      call ccpzi(kdtsrn, numrun)
      if (irefrn .gt. 0) kdtsrn(irefrn) = -1

c Loop input assignments
      do id = 1, ndtsetsout
         if (nrundts(id) .le. 0) then
c  All runs
c   Only one dataset can contain all runs
            if (ndtsetsout .gt. 1) then
               write (lunout, '(/a/)')
     $            ' **** Only one dataset may contain ALL runs ****'
               istat = -2
               return
            endif
c        enumerate all runs in this (sole) dataset, but omit 
c        reference set
            nrundts(id) = 0
            do irun = 1, numrun
               if (irun .eq. irefrn) then
                  kdtsrn(irun) = -1
               else
c        add run to list
                  nrundts(id) = nrundts(id) + 1
                  krundts(nrundts(id), id) = irun
                  kdtsrn(irun) = id
               endif
            enddo
         else
c   
c   List of runs
            k  = 0
            do i = 1,  nrundts(id)
               irun = krundts(i, id)
               if (irun .eq. irefrn) then
                  kdtsrn(irun) = -1
                  if (nrundts(id) .gt. 1) then
                     write (lunout, '(/a,i5,a,a/)')
     $                  ' **** Reference run ',irunum(irun),
     $                  ' may not be put into the same dataset as ',
     $                  'other runs ****'
                     istat = -4
                  endif
               else
                  k = k+1
c   Set index to dataset from run
                  kdtsrn(irun) = id
               endif 
            enddo
            nrundts(id) = k
         endif
      enddo
      if (istat .lt. 0) return
c
c  Are all runs included somewhere (excluding reference)?
      do irun = 1, numrun
c     Note kdtsrn = -1 for reference run
         if (kdtsrn(irun) .eq. 0) then
            write (lunout, '(/a,i5,a/)')
     $         ' **** Run ',irunum(irun),
     $         ' has not been assigned to a dataset ****'
            istat = -3
         endif
      enddo
      if (istat .lt. 0) return
c
c  Assign dataset numbers to each batch
      call ccpzi(nindts, maxsets)
      call ccpzi(kindts, maxinsets*maxsets)
c
      do i = 1, nbtall
         if (irunbt(i) .gt. 0) then
c    Output dataset number
            id = kdtsrn(irunbt(i))
            kdtsbt(i) = id
            if (id .gt. 0) then
c    Input dataset number
               jdataset = jdtsbt(i)
c    Do we have this one already?
               lfound = .false.
               if (nindts(id) .gt. 0) then
                  do j = 1, nindts(id)
                     if (kindts(j, id) .eq. jdataset) then
                        lfound = .true.
                     endif
                  enddo
               endif
c   For each output dataset, list input datasets which contribute to it,
c   but only if not already found
               if (.not.lfound .and. jdataset .gt. 0) then
                  nindts(id) = nindts(id) + 1
                  kindts(nindts(id), id) = jdataset
               endif
            endif
         endif
      enddo
c
      do jd = 1, ndtsetsout
         waveln = 0.0
         call ccpzr(cell, 6)
         if (nindts(jd) .gt. 0) then
            waveln = dwavel_in(kindts(1, jd))
            call ccpmvr(cell, dcell_in(1,jd), 6)
            if (pname_out(jd) .eq. ' ') then
               pname = pname_in(kindts(1,jd))
            else if (pname .eq. ' ') then
               pname = pname_out(jd)
            endif
            if (xname_out(jd) .eq. ' ') then
               xname = xname_in(kindts(1,jd))
            else if (xname .eq. ' ') then
               xname = xname_out(jd)
            endif
            if (nindts(jd) .gt. 1) then
               lerrwvl  = .false.
               lerrcell = .false.
               ldifpname = .false.
               ldifxname = .false.
               do id = 2, nindts(jd)
c    Multiple input datasets correspond to this output dataset
c    Check for consistency of cell & wavelength & names
                  if (abs(dwavel_in(kindts(id, jd)) - waveln)
     $               .gt. tolwvl) lerrwvl = .true.
                  do i = 1,3
                     if (abs(dcell_in(i, kindts(id, jd)) - cell(i))
     $                  .gt. tolcll) lerrcell = .true.
                  enddo
                  do i = 4,6
                     if (abs(dcell_in(i, kindts(id, jd)) - cell(i))
     $                  .gt. tolcng) lerrcell = .true.
                  enddo
                  if (pname_in(kindts(id,jd)) .ne. pname)
     $               ldifpname = .true. 
                  if (xname_in(kindts(id,jd)) .ne. xname)
     $               ldifxname = .true. 
               enddo
c
               call mkdtslab(jd, dtslab)
               if (lerrwvl) then
                  write (lunout, '(/a,a,a)')
     $               ' WARNING: output dataset ',
     $               dtslab(1:lenstr(dtslab)),
     $             ' contains input datasets with different wavelengths'
               endif
               if (lerrcell) then
                  write (lunout, '(/a,a,a)')
     $               ' WARNING: output dataset ',
     $               dtslab(1:lenstr(dtslab)),
     $               ' contains input datasets with different cells'
               endif
               if (ldifpname) then
                  write (lunout, '(/a,a,a)')
     $               ' WARNING: output dataset ',
     $               dtslab(1:lenstr(dtslab)),
     $          ' contains input datasets with different Project Names'
                  pname = 'MultiProject'
               endif
               if (ldifxname) then
                  write (lunout, '(/a,a,a)')
     $               ' WARNING: output dataset ',
     $               dtslab(1:lenstr(dtslab)),
     $          ' contains input datasets with different Crystal Names'
                  xname = 'MultiCrystal'
               endif

            endif
c Store wavelength and cell & names, but not if explicit names are given on input
            if (ndtskyin .le. 0) then
               dwavel_out(jd) = waveln
               call ccpmvr(dcell_out(1,jd), cell, 6)
            else
c    in this case leave them to be determined from batch headers
               dwavel_out(jd) = 0.0
               call ccpzr(dcell_out(1,jd), 6)
            endif

c
            if (pname_out(jd) .eq. ' ')
     $         call nospch(pname_out(jd), pname)
            if (xname_out(jd) .eq. ' ')
     $         call nospch(xname_out(jd), xname)
c
         endif
      enddo
c
c========================================================================
c
c Print dataset information
c
      istat = 0
      do id = 1, ndtsetsout
c Does this have a valid cell? If not try to extract it from batch header data
         call mkdtslab(id, dtslab)
         write (lunout,'(/a,a)')
     $      '===== Dataset: ',dtslab(1:lenstr(dtslab))
         write (lunout, '(a,(30i4))')
     $      '     Run(s): ', (krundts(i, id), i = 1, nrundts(id))
         call chkclw(id)
         write (lunout, '(a,f9.6,a,6f10.3)')
     $      '  Wavelength: ', dwavel_out(id), '  Cell: ',
     $      (dcell_out(i, id), i = 1,6)
c
      enddo
c Fail if non-unique dataset names
      do id1 = 1, ndtsetsout-1
         do id2 = id1+1, ndtsetsout
            if (dname_out(id2) .eq. dname_out(id1)) then
               write (lunout, 6101)
     $            dname_out(id1)(1:lenstr(dname_out(id1)))
 6101            format(//
     $            '**** ERROR **** dataset name must be unique: ',
     $            'change or add NAME command',a//)
                call ccperr
     $              (1,'**** ERROR ****  non-unique dataset name')
            endif
            
         enddo
      enddo 


      return
      end
      subroutine setlnk(nlink, klink, istat)
c     =====================================
c
c  Set up links between parameters
c
c On entry:
c  nlink            number of links
c  klink(3,nlink)   (3,) link type,
c                      = 1 for TAILS parameters
c                      = 2 for spherical harmonic parameters
c                   run number klink(1,) is linked to run number klink(2,)
c                   (these are external not internal run numbers)
c                   klink(1,) = -1 means LINK ALL (ie all runs with TAILS)
c                   link type klink(3,) .lt. 0 means unlink this group
c
c On exit:
c  nlink, klink     possibly updated
c  istat            =  0  OK
c                   = -1  failure
c
c  itdspt (common /scales/) set as pointer from run 1 to run 2
c                   keyin has previously set itdspt(irun) = irun for all runs
c  isphpt (common /scales/) set as pointer from run 1 to run 2
c                   keyin has previously set isphpt(irun) = irun for all runs
c
      integer nlink, klink(3,nlink), istat
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'scales.fh'
      include 'sphhrm.fh'
c
      integer k, irun, irunm1, irunm2
      logical link
c
      istat = 0
c
      do 10, k = 1, nlink
         if (iabs(klink(3,k)) .lt. 1 .or. iabs(klink(3,k)) .gt. 2) then
            write (lunout, '(/a/)')
     $         ' **** LINK type not recognized ****'
            istat = -1
            return
         endif
c LINK or UNLINK
         if (klink(3,k) .gt. 0) then
            link = .true.
         else
            link = .false.
         endif
         if (klink(1,k) .lt. 0) then
c [UN]LINK ALL (but only those runs with TAILS or SURFACE defined)
            if (numrun .gt. 1) then
               irunm2 = -1
               do 20, irun = 1, numrun
                  if (irun .ne. irefrn) then
                     irunm1 = irunum(irun)
                     if (
     $                (klink(3,k) .eq. 1 .and. nrntds(irun) .gt. 0) .or.
     $                (klink(3,k) .eq. 2 .and. nrnsph(irun) .gt. 0)
     $                  ) then
c    TAILS or SURFACE switched on for this run
                        if (irunm2 .lt. 0) then
c        Store first one to link all others to
                           irunm2 = irunm1
                           go to 20
                        else
c        Link irunm1 & irunm2
                           call lnkrun(
     $                        klink(3,k), link, irunm1, irunm2, istat)
                           if (istat .gt. 0) then
                              istat = 0
                           endif
                           if (istat .ne. 0) then
                              go to 999
                           endif
                        endif
                     endif
                  endif
 20            continue
            endif
         else
c [UN]LINK <irunm1> to <irunm2>
            call lnkrun(klink(3,k), link, klink(1,k), klink(2,k), istat)
            if (istat .gt. 0) then
c  Print warning only for specific UNLINK, not for UNLINK ALL
               write (lunout, '(/a,i6,a,i6,a/)')
     $            ' WARNING: Invalid UNLINK run ', klink(1,k),
     $            ' to run', klink(2,k),' |===='
               istat = 0
            endif
            if (istat .ne. 0) then
               go to 999
            endif
         endif
 10   continue
c
c Print links
      if (nprtds .gt. 0) then
         do irun = 1, numrun
            if (itdspt(irun) .ne. irun) then
               write (lunout, '(/a,i5,a,i5)')
     $            ' TAILS parameters for run', irunum(itdspt(irun)),
     $            ' will be used for run', irunum(irun)
            endif
         enddo
      endif
      if (nprsph .gt. 0) then
         do irun = 1, numrun
            if (isphpt(irun) .ne. irun) then
               write (lunout, '(/a,i5,a,i5)')
     $            ' SURFACE parameters for run', irunum(isphpt(irun)),
     $            ' will be used for run', irunum(irun)
            endif
         enddo
      endif
      return
c
 999  write (lunout, '(/a,a/a,a/)')
     $ ' **** Invalid links may arise from attempts to link runs with ',
     $   ' different TAILS or SURFACE options ****',
     $   ' **** In this case, you may need to give explicit links ',
     $   ' rather than relying on the defaults ****'
      return
      end
c
c
c
      subroutine setmtc(irun, istat)
c     =============================
c
c Set up matching symmetry operators for Anomalous Match, from information
c partly stored by s/r strmtc
c
c On entry:
c  irun         run number
c  istat        =  0 OK
c               = +1 null matrix
c               = +2 determinant of match matrix .ne. -1.0
c               = +3 duplicate operator match
c               = +4 spindle operator not spacegroup symmetry
c               = +5 match operator not spacegroup symmetry
c
      integer istat, irun
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'anomtc.fh'
      include 'rundef.fh'
c
      integer i, j, k, l
      real    Rinv(3,3), det, SRinv(3,3), a(3,3)
      logical lfound
      character anmstr*80
c
      external lenstr, matmul
      integer  lenstr
      external isunmt
      logical  isunmt
c
      istat = 0
      if (kanmtc(irun) .eq. 0) then
c SPINDLE option: all other options already set up
         l = kspnax(irun)
         if (l .gt. 0) then
c l is 1,2,3 for spindle along h,k,l
c Set matrix
            call unitmt(anmmtc(1,1,irun))
            anmmtc(l,l,irun) = -1
            kanmtc(irun) = +3
         else
            write (lunout, '(/a,i4,a/)')
     $         ' **** No spindle defined, run ',irunum(irun),' ****'
            istat = +1
         endif
c
      endif
c
c
c Check that 
c Now set up match table for each symmetry operator, for this run
      if (kanmtc(irun) .ge. +1) then
c Check that it inverts hand (det = -1)
         call minv3(Rinv, anmmtc(1,1,irun), det)
         if (abs(det+1.0) .gt. 0.001) then
            write (lunout, '(/a,i5,a/a,g12.6/a/3(3x,3f10.4/))')
     $         ' **** MATCH matrix for run ', irunum(irun),
     $         ' does not have determinant = -1.0  ****',
     $         ' **** Det = ',det,' Matrix:',
     $         ((anmmtc(i,j,irun), j=1,3),i=1,3)
            istat = +2
            return
         endif
         call ccpzi(mtcsym(1, irun), nisym)
c Test all pairs of symmetry operators j & k of opposite parity
c  Loop I+ operators
         do 100, j = 1, nisym, 2
c     Rj**-1
            call minv3(Rinv, rassym(1,1,j), det)
c     S(match) * Rj**-1
            call matmul(SRinv, anmmtc(1,1,irun), Rinv)
c   Loop I- operators to find matching operator k
c    this is the one for which Rk  S(match) Rj**-1 = I
            lfound = .false.
            do 110, k = 2, nisym, 2
               call matmul(a, rassym(1,1,k), SRinv)
               if (isunmt(a)) then
                  if (lfound) then
c        shouldn't happen
                     write (lunout, '(/a,2i4/)')
     $                  ' **** SETMTC: More than one operator matches:',
     $                  j,k
                     istat = +3
                  endif
c  Found, this matrix is unit, store pointers both ways
                  mtcsym(j,irun) = k
                  mtcsym(k,irun) = j
                  lfound = .true.
               endif
 110        continue
 100     continue
c Check all found
         lfound = .true. 
         do 120, j = 1, nisym
            if (mtcsym(j,irun) .eq. 0) then
               lfound = .false.
            endif
 120     continue
         if (.not. lfound) then
            call strrsm(anmmtc(1,1,irun), anmstr)
            if (kanmtc(irun) .eq. +3) then
c  Spindle 
               write (lunout, '(/a,a,a/)')
     $            ' **** SPINDLE axis operator ',
     $            anmstr(1:lenstr(anmstr)),
     $            ' is not a spacegroup symmetry operator ****'
               istat = +4
            else
               write (lunout, '(/a,a,a/)')
     $            ' **** Anomalous Match operator ',
     $            anmstr(1:lenstr(anmstr)),
     $            ' is not a spacegroup symmetry operator ****'
               istat = +5
            endif              
         endif
      endif
c
      return
      end
 
      subroutine setodf
c     =================
c
c  Set output defaults for SEPARATE & POSTREF options
c
      include 'parameter.fh'
      include 'outcon.fh'
c
c Omit reference run
      lrefou = .false.
c Keep reflections outside resolution limits, but set scale = 0.0
      krsexc = +1
c Keep outliers
      lolexc = .false.
c Keep partials
      nprexc = 0
c Don't write ROGUES file
      lwrrog = .false.
c
      return
      end
      subroutine setpos(numpnt, spcing, slimit, cntrct, npoint, positn)
c     =========================================================
c
c  Set number of points & spacing on coordinate from input specifications
c
c  On entry:
c     numpnt     if > 0 number of points in this coordinate
c                if <= 0, spcing is spacing
c     spcing     if numpnt .le. 0, spacing, may be negative
c     slimit(2)  minimum & maximum values of coordinate
c     cntrct     contraction of range around centre for positioning
c                scale points (0->1), = 1.0 for no contraction
c
c  On exit:
c     spcing          interval on coordinate
c     npoint          number of points
c     positn(npoint)  positions, in normalized coordinate frame
c                     (these are the numbers 0.0 -> npoint-1 modified
c                      by the contraction)
c
      implicit none
c
      real spcing, slimit(2), positn(*), cntrct
      integer numpnt, npoint
c
      integer i
      real centre
c
c
      if (numpnt .gt. 0) then
c  Number given
         npoint = max(1,numpnt)
         if (npoint .gt. 1) then
            spcing = (slimit(2) - slimit(1)) / float(npoint-1)
            if (abs(spcing) .lt. 0.000001) then
               spcing = 1.0
               npoint = 1
            endif
         else
            spcing = 1.0
         endif
      else
c  Interval given
         npoint = max(1, nint((slimit(2) - slimit(1))/spcing) + 1)
         if (npoint .eq. 1) spcing = 1.0
      endif
c
c Positions
      if (npoint .eq. 1) then
         positn(1) = 0.0
      else
         centre = float(npoint - 1) / 2.0
         do 10, i = 1, npoint
            positn(i) = (float(i-1) - centre) * cntrct + centre
 10      continue
      endif
c
      return
      end
      subroutine setprob(prob)
c     ========================
c
       real prob(512)
      
C   Build probability array
C
C     Build probability integral for X = 0 to 4.0 in
C     increments of 0.007827789.
C
      double precision  den(27), p, psum, sum
      integer i, j, k, id
      real del
c
      p = 1.0d0
      do 6 i=1,27
      p = p * i
 6     den(i) = dlog10(p * (2 * i + 1) * 2.0d0**i)
      id = 0
      prob(1) = 0.0
      do 7 i=2,512
      id = id + 1
      del = id
      p = dble(del) * 0.7827789d-02
      sum = 1.0d0
      do 8 j=1,27
      k = mod(j+1,2) - mod(j,2)
      psum = (2 * j) * dlog10(p) - den(j)
      if (psum + 7.0d0 .lt. 0.0) go to 7
 9     psum = 1.0d+01**psum
 8     sum = sum + k * psum
c
 7     prob(i) = 0.7978845608d0 * p * sum
c
       return
       end
      subroutine setrsc(
     $     irun, mode, nbfac, nbfps, time, primry, secnd1, secnd2)
c     ============================================================
c     
c     Store spacings/numbers to define scale layout in common /scales/
c Sets:-
c  spcscl(,irun)
c  ncscls(1,irun)  = +1 if only one scale for this run, else = 0
c     
c     
      implicit none
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'sdfacc.fh'
c     
      integer irun, mode, nbfac, nbfps
      real time, primry, secnd1, secnd2
c
c-save-      save /scales/, /sdfacc/
c     
      if (mode   .gt. 0) lsmode(irun) = mode
      if (nbfac  .ge. 0) then
         if (nbfac .gt. 0) then
            lbfrun(irun) = .true.
         else if (nbfac .eq. 0) then
            lbfrun(irun) = .false.
         endif
      endif
c Iso or aniso
      nbfpos(irun) = nbfps
c     "Time" for B-factors
      spcbfc(irun) = time
c     Rotation, primary beam (=z)
      spcscl(3,irun) = primry
c     Secondary, detector position etc
      spcscl(1,irun) = secnd1
      spcscl(2,irun) = secnd2
c
c Pick up special case of only one scale (primry = -1.0 = secnd1, secnd2)
c Set ncscls(1,) as flag, for s/r setscl
      if (abs(primry)-1.0 .lt. 0.01 .and.
     $   abs(secnd1)-1.0 .lt. 0.01 .and.
     $   abs(secnd2)-1.0 .lt. 0.01) then
         ncscls(1,irun) = +1
      else
         ncscls(1,irun) =  0
      endif
c     
      return
      end
c
c
c
      subroutine setrun(
     $   ngroup, idtsgroup, ibgroup, ibgapgroup, gapgroup)
c    =============================================
c
c  Set up runs if not specified explicitly 
c  Put each group into a separate run
c  A dataset consisting of a single batch is assumed to be a reference set,
c    but if there are more than one, then irefrn is set = -1 to indicate
c    there is no unique reference run. If there is only one group, then
c    it cannot be a reference run
c
c On entry:
c  ngroup     number of "groups" found: a group is a candidate
c              for a run, and are separated by a gap either in
c              batch number or in phi (or both)
c  idtsgroup  input dataset number for group
c  ibgroup(2,) batch serial number for first and last
c                batches in group
c  ibgapgroup gap in batch number from previous group
c              (= -1000 for 1st group)
c  gapgroup   gap in phi from previous group
c              (= 0.0 for 1st group)
c 

c
      include 'parameter.fh'
      include 'inout.fh'
      include 'rundef.fh'
      include 'nbtchc.fh'
      include 'datasets.fh'
c
      integer ngroup, idtsgroup(maxgrp), ibgroup(2,maxgrp),
     $   ibgapgroup(maxgrp)
      real gapgroup(maxgrp)
c
      integer i, j, n, irun
c
      numrun = 0
      irun = 1
c
      do j = 1, ngroup
         n = 0
         do  i = ibgroup(1,j), ibgroup(2,j)
            if (irunbt(i) .ge. 0) then
               irunbt(i) = irun
               n = n+1
            endif
         enddo
         if (n .gt. 0) then
c  Only accept run if is has some batches in it
            irunum(irun) = irun
            nptrun(irun) = 0
c
            if (ngroup .gt. 1 .and. nbtchdts(idtsgroup(j)) .eq. 1) then
               if (irefrn .eq. 0) then
                  irefrn = irun
               else
                  irefrn = -1
               endif
            endif
            numrun = irun
            if (numrun .gt. maxrun) then
               write (lunout, '(a,i4,a)')
     $            ' **** Too many runs defined, maximum = ', maxrun,
     $            ' ****'
               call dna_error('too many runs')
               call ccperr(1,'*** Too many runs ***')
            endif
            irun = irun+1
         endif
      enddo
c
      return
      end


      subroutine setscl(istat)
c     ========================
c
c  For all runs, find range of time, x,y,z, & set up number of scale factors,
c and pointers to different classes
c  Returns Istat .gt. 0 if problems, = 0 if OK
c
c
      integer istat
c
c
      include 'parameter.fh'
      include 'column.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'refcon.fh'
      include 'params.fh'
      include 'sdfacc.fh'
      include 'anomtc.fh'
      include 'symmty.fh'
      include 'sphhrm.fh'
      include 'outcon.fh'
      include 'rfile.fh'
c
c Locals
      real time1, time2, phi1, phi2, delphi, xydlim(2,2), ubmat(3,3),
     $   umat(3,3), bmat(3,3), s0(3), phirangelast, d,
     $   e1e2mat(3,3), e1e2inv(3,3)
      real ub0inv(3,3), amat(3,3), dircos(3), rotatn, det, rotlim,
     $   zmin, zmax, timchk, timwrn, tlim(2), gap, positn(maxxys,2),
     $   cntrct, frctol, tlast, gaptmx, gaprmx, rtlast,
     $   phichk, phiwrn, gprtmx, gprrmx, ubmax, hkl(3), x(3), z(3),
     $   bmpinv(3,3), dtmmat(3,3), qmat(3,3), tmat(3,3), bmatall(3,3),
     $   DU(3,3), wvlnth, detlimx, detlimy, range(2)
      integer nmat, i, j, k, l, m, irun, nrun, npoint(2), jpoint,
     $   nwarn, mxwarn, istat1, jptyp, jp, np, kp, jgptmx, jgprmx,
     $   kchklm, kchkor, iaxes(3), ib, ir, phi3scan, numpnt
      logical ltestr, lub0, lubmat, ldelphi, ldetdiff, lzlim
      logical lphi, ldet, lmat
      character label*30
c
c Functions
      integer ccp4_hash_lookup, lenstr
      external ccp4_hash_lookup, lenstr
      real dot
      external dot, matmul
c
c-save-      save /scales/, /sdfacc/, /params/, /refcon/
c
c rotlim    maximum allowed angular difference (degrees) between
c           setting matrices for different batches in the same run, in
c           detector frame
      data rotlim/3.0/
c timchk    maximum allowed "time" gap between succesive batches in a run,
c           as fraction of time range: timwrn is warning limit 
c phichk    similar for rotation
      data timchk,timwrn /2.0, 1.01/, phichk,phiwrn /2.0, 1.01/
c mxwarn    number of warning messages
      data mxwarn /10/
c
      jgptmx = 0
      jgprmx = 0
      istat = 0
      nmat = 0
      jpoint = 0
      nparar = 0
      nwarn = 0
      gaptmx = -1000.0
      gaprmx = -1000.0
      phirngmin = +1000.
      wvlnmn = 100.
      wvlnmx = 0.0
      detlimx = -1000000.0
      detlimy = -1000000.0
      ldetdiff = .false.
c  Clear iplloc link pointer array
      call ccpzi(iplloc,maxlnp*maxrun)
c
c  Make file-global B matrix
      iaxes(1) = 1
      iaxes(2) = 2
      iaxes(3) = 3
      call mkbmat(cell, bmatall, 1.0, iaxes)
c
c --- Loop runs ---
      do 10, irun = 1, numrun
c  ltestr is flag whether to test for presence of ROT column
c  ltsdet is flag whether to test for presence of XDET, YDET columns
         ltestr = .false.
         ltsdet = .false.
         nrun = irunum(irun)
c  note that zlim may go backwards, lzlim true at first
         lzlim = .true.
         zlimrn(1,irun) = +1.0e20
         zlimrn(2,irun) = -zlimrn(1,irun)
         tlimrn(1,irun) = +1.0e20
         tlimrn(2,irun) = -tlimrn(1,irun)
         dlimrn(1,1,irun) = +1.0e20
         dlimrn(2,1,irun) = -dlimrn(1,1,irun)
         dlimrn(1,2,irun) = +1.0e20
         dlimrn(2,2,irun) = -dlimrn(1,2,irun)
c
c
c
c--- Set flags for orientation check
c (kchklm for phi|detector limits, kchkor for orientation data): 
c (a)  kchklm = 0, kchkor = 0 :  No check at all if this run is 
c    (1) BATCH mode (2) reference set (3) only one scale factor
c
c (b)  kchklm = +1: just Phi limits required
c    (1) smooth rotation scaling
c        but not SMOOTH mode
c
c (c)  kchklm = +2: Phi & detector limits required
c    (1) detector scaling 
c
c (d)  kchkor = +1: full orientation data is only required for
c    (1) TAILS correction (2) anisotropic B (3) anomalous match spindle
c    (4) SECONDARY beam correction (lsmode = 2 or 3)
c         (in this case, Phi limits are not needed)
c    (5) OUTPUT UNMERGED BEAMS
c
         kchklm = 0
         kchkor = 0
         if (irun .eq. irefrn) then
c    Reference
c      No orientation data needed
            kchklm = 0
            kchkor = 0
         else
c
            if ((mbatsc(irun) .gt. 0) .or.
     $         (ncscls(1,irun) .eq. +1) .or.
     $         lbtsmooth(irun)) then
c   (a) pure batch mode mode || 1 scale || reference
c      No orientation data needed
               kchklm = 0
            elseif (mbatsc(irun) .le. 0) then
c   (b) smooth rotation, need phi limits
               kchklm = +1
               if (abs(spcscl(1,irun)) .gt. 1.01 .or.
     $            abs(spcscl(1,irun)) .gt. 1.01) then
c   (c) variation across detector, need detector limits
                  kchklm = +2
               endif
            endif
            if (ltdsrn(irun) .or. lsmode(irun) .eq. 2 .or.
     $         lsmode(irun) .eq. 3) then
c   (d) Tails, anisotropic B, secondary
c         Need orientation data
               kchkor = +1
            endif
c
            if (kanmtc(irun) .eq. 0 .or. mbeams .ne. 0) then
c         Need orientation data for spindle option or BEAMS
               kchkor = +1
            endif
         endif
         if (anmphd .ne. 0.0) then
c  Must have ROT column to test PHIDIF
            ltestr = .true.
         endif
c
c --- --- Loop batches ---
         lub0 = .false.
         lubmat = .false. 
         gprtmx = 0.0
         gprrmx = 0.0
c
         do 20, k = 1, nbatrn(irun)
            ib = ccp4_hash_lookup(jbatrn(k,irun))
c  Get information for this batch
            call batinf(
     $         jbatrn(k,irun), time1, time2, phi1, phi2, delphi,
     $         xydlim, dtmmat, e1e2mat, e1e2inv, phi3scan, ubmat,
     $         umat, bmat, wvlnth, s0,
     $         lphi, ldet, lmat, istat1)
c
            if (lbtsmooth(irun)) then
c  Smooth mode, phi is batch number
               phi1 = jbatrn(k,irun)
               phi2 = phi1+1
               delphi = 1.0
               phibat(1, ib) = phi1
               phibat(2, ib) = phi2
               phibat(3, ib) = delphi
               phioffset(ib) = 0.0
               time1 = phi1
               time2 = phi2
               philim(1, k, irun) = jbatrn(k,irun)
               philim(2, k, irun) = jbatrn(k,irun)
            else
c  Use Phi limits previously stored by s/r findgap: these have wrap-arounds
c 360 degrees fixed up
               phi1 = phibat(1, ib)
               phi2 = phibat(2, ib)
               delphi = phibat(3, ib)
c
c Check if invalid orientation is allowed
               if (kchklm .eq. 2) then
                  if (.not.ldet) then
                     write (lunout, '(//a,i7)')
     $                  ' **** Missing detector limits for batch ',
     $                  jbatrn(k,irun)
                     istat = istat+1
                  endif
               endif
               if (kchklm .ge. 1) then
                  if (.not.lphi) then
                     write (lunout, '(//a,i7)')
     $                  ' **** Missing phi limits for batch ',
     $                  jbatrn(k,irun)
                     istat = istat+1
                  endif
               endif
c
               if (kchkor .gt. 0) then
                  if (istat1 .gt. 0) then
c  Negative Umat, fail (for now) but print warning
                     write (lunout,'(//a/a,i7,a/a)')
     $                  'WARNING WARNING WARNING WARNING ',
     $                  '**** Orientation matrix for batch',
     $                  jbatrn(k,irun),
     $                  '  inverts hand',
     $                  '**** This is not allowed ****'
                     lmat = .false.
                  endif 
                  if (.not.lmat) then
                     write (lunout, '(///a,i7,a,a)')
     $                  ' **** No valid orientation data for batch',
     $                  jbatrn(k,irun),
     $                  ': this is not allowed with TAILS, SECONDARY,',
     $                  ' ABSORPTION or BEAMS options'
c     $                  ': this is ONLY allowed for reference batch or',
c     $                  ' for SCALES BATCH mode with isotropic B ****'
                     istat = istat+1
                     return
                  endif
               endif
            endif
            if (istat .gt. 0) return
c
c Store phi range for this batch
            philim(1,k,irun) = phi1
            philim(2,k,irun) = phi2
            phirng(k,irun) = delphi
c    note that delophi may be negative
            if (abs(delphi) .gt. 0.01)
     $         phirngmin = min(phirngmin, abs(delphi))
c
c  Update rotation limits (note phi1 may be > phi2)
            if (delphi .lt. 0.0) then
               if (lzlim) then
c    for declining phi, invert starting limits
                  zlimrn(1,irun) = -1.0e20
                  zlimrn(2,irun) = -zlimrn(1,irun)
                  lzlim = .false.
               endif
c      and zlimrn(1,) is maximum
               zlimrn(1,irun) = max(zlimrn(1,irun), max(phi1,phi2))
               zlimrn(2,irun) = min(zlimrn(2,irun), min(phi1,phi2))
               negphi(irun) = .true.
            else
c   normal ascending phi, zlimrn(1,) is minimum
               zlimrn(1,irun) = min(zlimrn(1,irun), min(phi1,phi2))
               zlimrn(2,irun) = max(zlimrn(2,irun), max(phi1,phi2))
               negphi(irun) = .false.
            endif
            if (mbatsc(irun) .ne. 0) then
               posbfc(k,irun) = 0.5 * (phi1 + phi2)
               poscsr(k,irun) = 0.5 * (phi1 + phi2)
            endif
c  Time limits
            if (nct .le. 0) then
c   no TIME column from file
c   time = +-phi
               if (delphi .lt. 0.0) then
c     declining phi, time = -phi
                  time1 = -phi1
                  time2 = -phi2
               else
                  time1 = phi1
                  time2 = phi2
               endif
            endif
            tlim(1) = min(tlimrn(1,irun), time1)
            tlim(2) = max(tlimrn(2,irun), time2)
c Store time range for this batch
            timlim(1,k,irun) = time1
            timlim(2,k,irun) = time2
c    Check for discontinuity in "time", only if smooth B-factor
            if (mbatsc(irun) .le. 0 .and. lbfrun(irun)) then
               if (k .gt. 1) then
                  gap = time1 - tlast
                  if (abs(gap) .gt. gprtmx) then
                     gprtmx = abs(gap)
                     jgptmx = k
                  endif
               endif
            endif
            tlimrn(1,irun) = min(tlimrn(1,irun), tlim(1))
            tlimrn(2,irun) = max(tlimrn(2,irun), tlim(2))
            tlast = time2
c    Check for discontinuity in Rotation, only if smooth scales
            if (mbatsc(irun) .eq. 0) then
               if (k .gt. 1) then
                  gap = phi1 - rtlast
                  if (abs(gap) .gt. gprrmx) then
                     gprrmx = abs(gap)
                     jgprmx = k
                  endif 
               endif
            endif
            rtlast = phi2
c
c  Detector limits
            dlimrn(1,1,irun) = min(dlimrn(1,1,irun), xydlim(1,1))
            dlimrn(2,1,irun) = max(dlimrn(2,1,irun), xydlim(2,1))
            dlimrn(1,2,irun) = min(dlimrn(1,2,irun), xydlim(1,2))
            dlimrn(2,2,irun) = max(dlimrn(2,2,irun), xydlim(2,2))
            if (detlimx .lt. -999990.) then
               detlimx = xydlim(2,1)
               detlimy = xydlim(2,2)
            else
c  check that all batches have the same detector size
               if (abs(detlimx-xydlim(2,1)) .gt. 0.0001)
     $            ldetdiff = .true.
               if (abs(detlimy-xydlim(2,2)) .gt. 0.0001)
     $            ldetdiff = .true.
            endif
c
            if (istat1 .ge. 0) then
c  Check if all batches have the same orientation
               if (.not.lub0) then
c  1st batch in run, store inverse setting matrix UB**-1
                  call minv3(ub0inv, ubmat, det)
                  lub0 = .true.
c    Store all new setting matrices
                  nmat = nmat + 1
                  if (nmat .gt. maxmat) then
                     write (lunout, '(/a,a,i6)')
     $                  ' ** Too many setting matrices, ',
     $                  'increase MAXMAT = ', maxmat
                     call ccperr(1,'** Increase Maxmat **')
                  endif
c       store UB, DU, E1E2, [E1E2]^-1 & B
                  call matmul(DU, dtmmat, umat)
                  call matmul(dubmtxs(1,1,nmat), DU, bmat)
                  call moveit(DU,  dumatxs(1,1,nmat), 9)
                  call moveit(e1e2mat, e1e2matxs(1,1,nmat), 9)
                  call moveit(e1e2inv, e1e2invmatxs(1,1,nmat), 9)
                  phi3scans(nmat) = phi3scan
                  call moveit(bmat,  bmatxs(1,1,nmat), 9)
                  wvlnxs(nmat) = wvlnth
                  wvlnmn = min(wvlnth, wvlnmn)
                  wvlnmx = max(wvlnth, wvlnmx)
c       store s0
                  call moveit(s0, s0xs(1,nmat), 3)
               else
c   Not 1st batch, has setting changed?
c    Amat = UBmat * UB0inv: Amat = Identity if no change
                  call matmul(amat, ubmat, ub0inv)
c    Decompose matrix into rotation rotatn (degrees)
                  call dcosfd(amat, dircos, rotatn)
                  if (abs(rotatn) .gt. rotlim) then
c       Yes it has changed
                     if (lsmode(irun) .eq. 1) then
c          Detector frame, complain only if check is required
                        if (kchklm .gt. 2) then
                           write (lunout,'(/a,i4,a,i7,a,i7)')
     $                          ' *** WARNING: problem in run ', nrun,
     $                          ', batch ',jbatrn(k,irun),
     $          ' has a different orientation that of initial batch',
     $                          jbatrn(1,irun)
                           write (lunout, '(a,f10.3,a)')
     $                        '  Orientation difference = ',rotatn,
     $                        ' degrees'
                        endif
                     endif
c          Store new matrix
                     nmat = nmat + 1
                     if (nmat .gt. maxmat) then
                        write (lunout, '(/a,a,i6)')
     $                     ' ** Too many setting matrices, ',
     $                     'increase MAXMAT = ', maxmat
                        call ccperr(1,'** Increase Maxmat **')
                     endif
                     call matmul(DU, dtmmat, umat)
                     call matmul(dubmtxs(1,1,nmat), DU, bmat)
                     call moveit(DU,  dumatxs(1,1,nmat), 9)
                     call moveit(bmat,  bmatxs(1,1,nmat), 9)
                     wvlnxs(nmat) = wvlnth
                     wvlnmn = min(wvlnth, wvlnmn)
                     wvlnmx = max(wvlnth, wvlnmx)
                     call moveit(s0, s0xs(1,nmat), 3)
                  endif
               endif
c
c      Store pointer to matrix
               jmtxbt(k,irun) = nmat
               lubmat = .true.
            else
               jmtxbt(k,irun) = 0
            endif
c
 20      continue
c
c  Store total number of matrices as flag to indicate if there is
c any orientation data
         nmatxs = nmat
c
c All batches for this run processed.
c
c Set spindle axis for this run, if matrix is given (else = 0)
c Matrix DUB = [D][U][B], for first matrix in run
         k = jmtxbt(1,irun)
c
         l = 0
         if (lubmat) then
c          Find largest z component of UB, from 1st matrix for run
            ubmax = -1.0
            l = 0
            do 21, j = 1,3
               if (abs(dubmtxs(3,j,k)) .gt. ubmax) then
                  l = j
                  ubmax = abs(dubmtxs(3,j,k))
               endif
 21         continue
            if (ubmax .lt. 0.00001) then
c    Null UB matrix, no spindle
               l = 0
            endif
         else
            l = 0
         endif
         kspnax(irun) = l
c spnang is angle between spindle axis and spindle 
         if (l .eq. 0) then
            spnang(irun) = 0.0
         else
            call ccpzr(hkl, 3)
            hkl(l) = 1.0
            call matvec(x, dubmtxs(1,1,k), hkl)
            call unit(x)
            z(1) = 0.0
            z(2) = 0.0
            z(3) = 1.0
            spnang(irun) = acos(abs(dot(x, z))) * 45./atan(1.)
         endif
c Reset rotation limits if required, from Run include/exclude options
c  doesn't work for phi decreasing
         if (nptrun(irun) .gt. 0) then
            if (negphi(irun)) then
               write (lunout, '(/a)')
     $            'You can''t have rotation include/exclude with '//
     $            'backwards phi'
               call ccperr(1,'** Negative phi with include/exclude **')
            endif
c  Includes first
            zmin = zlimrn(2,irun)
            zmax = zlimrn(1,irun)
            do 30, k = 1, nptrun(irun)
               if (.not.lptrun(k,irun)) then
                  zmin = min(zmin, zmnrun(k,irun))
                  zmax = max(zmax, zmxrun(k,irun))
               endif
 30         continue
            zlimrn(1,irun) = max(zlimrn(1,irun), zmin)
            zlimrn(2,irun) = min(zlimrn(2,irun), zmax)
c   then Excludes
            do 40, k = 1, nptrun(irun)
               if (lptrun(k,irun)) then
c         Minimum
                  if (zlimrn(1,irun) .gt. zmnrun(k,irun) .and.
     $                zlimrn(1,irun) .lt. zmxrun(k,irun))
     $                   zlimrn(1,irun) = zmxrun(k,irun)
c         Maximum
                  if (zlimrn(2,irun) .gt. zmnrun(k,irun) .and.
     $                zlimrn(2,irun) .lt. zmxrun(k,irun))
     $                   zlimrn(2,irun) = zmnrun(k,irun)
               endif
 40         continue
c
            if (zlimrn(1,irun) .gt. zlimrn(2,irun)) then
               write (lunout, '(A,I4,A)') ' **** Run ',nrun,
     $            ', ALL rotation ranges excluded! ****'
               istat = istat+1
            endif
         endif
c
c B-factors, lay out & clear = 0.0
         if (lbfrun(irun)) then
            if (mbatsc(irun) .gt. 0) then
c     Batch scale option
               nbfact(irun) = nbatrn(irun)
               spcbfc(irun) = 1.0
            else
               cntrct = 1.0
c     Reset spacing to actual range if smaller (ie force two points)
               if (spcbfc(irun) .gt. 0.0) then
                  numpnt = -1
c       spacing given (rather than number)
                  if (abs(tlimrn(2,irun) - tlimrn(1,irun))
     $               .lt. spcbfc(irun) .and.
     $               nbatrn(irun) .gt. 1) then
                     spcbfc(irun) = abs(tlimrn(2,irun) - tlimrn(1,irun))
                  endif
               else
c    Number given
                  numpnt = nint(abs(spcbfc(irun)))
               endif
               call setpos(numpnt, spcbfc(irun), tlimrn(1,irun), cntrct, 
     $           nbfact(irun), posbfc(1,irun))
            endif
            if (nbfact(irun) .gt. maxrot) then
               write (lunout, '(a,i4,a,i5,a,i5)')
     $           ' **** Too many B-factors for run number ',
     $           nrun,', there are ', nbfact(irun),
     $           ', maximum = ',maxrot
               call ccperr(1,'*** Too many B-factors ***')
            endif
c
            do 50, i = 1,nbfact(irun)
               do 51, j = 1, nbfpos(irun)
                  bfactr(j,i,irun) = 0.0
 51            continue
 50         continue
         else
            nbfact(irun) = 0
            spcbfc(irun) = 1.0
         endif
         if (nbfact(irun) .eq. 1 .and. numrun .eq. 1) then
            write (lunout, '(/a,a/)')
     $         ' *** WARNING: only one B-factor for sole run, ',
     $         'B-factor refinement switched off ***'
            nbfact(irun) = 0
            spcbfc(irun) = 1.0
         endif
c  nbparm(irun) is the total number of Bfactors for this run
         nbparm(irun) = nbfact(irun) * nbfpos(irun)
c
c Scales: rotation (z), positions
         if (mbatsc(irun) .ne. 0) then
c     Batch scale option
            if (lbtslp(irun)) then
c       Slope option
               ncscls(1,irun) = 2 * nbatrn(irun)
               spcscl(3,irun) = 1.0
c Don't test reference run, but otherwise need ROT for slope option
               if (irun .ne. irefrn) ltestr = .true.
            else
               ncscls(1,irun) = nbatrn(irun)
               spcscl(3,irun) = 1.0
            endif
         else
            if (kchklm .gt. 0) ltestr = .true.
            cntrct = 1.0
c     Reset spacing to actual range if smaller (ie force two points)
            if (spcscl(3,irun) .gt. 0.0) then
               numpnt = -1
c       spacing given (rather than number)
               if (abs(zlimrn(2,irun) - zlimrn(1,irun))
     $            .lt. spcscl(3,irun) .and.
     $            nbatrn(irun) .gt. 1) then
                  spcscl(3,irun) = (zlimrn(2,irun) - zlimrn(1,irun))
               endif
               if (zlimrn(2,irun) .lt. zlimrn(1,irun)) then
c  negative phi range, negate spacing
                  spcscl(3,irun) = -abs(spcscl(3,irun))
               endif
            else
               numpnt = nint(abs(spcscl(3,irun)))
            endif
            call setpos(numpnt, spcscl(3,irun), zlimrn(1,irun), cntrct,
     $         ncscls(1,irun), poscsr(1,irun))
         endif
         if (ncscls(1,irun) .gt. maxrot) then
            write (lunout, '(a,i4,a,i5,a,i5)')
     $           ' **** Too many rotation ranges for run number ',
     $           nrun,', there are ', ncscls(1,irun),
     $           ', maximum = ',maxrot
            call ccperr(1,'*** Too many rotation ranges ***')
         endif
c
c Scales: detector position (x,y)
         do 60, m=1,2
c%c            cntrct = 0.75
            cntrct = 1.0
            if (spcscl(m,irun) .gt. 0.0) then
               numpnt = -1
            else
               numpnt = nint(abs(spcscl(m,irun)))
            endif
            call setpos(numpnt, spcscl(m,irun), dlimrn(1,m,irun), 
     $           cntrct, npoint(m), positn(1,m))
 60      continue
c
c Set tolerances on detector position (for partial check in sumprt)
c  frctol is fraction of detector size allowed
         frctol = 0.01
         dtolrn(1,irun) =
     $        (dlimrn(2,1,irun) - dlimrn(1,1,irun)) * frctol
         dtolrn(2,irun) =
     $        (dlimrn(2,2,irun) - dlimrn(1,2,irun)) * frctol
c
         ncscls(3,irun) = npoint(1)
         ncscls(4,irun) = npoint(2)
         ncscls(2,irun) = npoint(1) * npoint(2)
         if (ncscls(2,irun) .gt. 1) ltsdet = .true.
         if (ncscls(2,irun) .gt. maxxys) then
            write (lunout, '(a,i4,a,i5,a,i5)')
     $           ' **** Too many detector positions for run number ',
     $           nrun,', there are ', ncscls(2,irun),
     $           ', maximum = ',maxxys
            call ccperr(1,'*** Too many detector ranges ***')
         endif
c
         k = 0
         do 70, j = 1, npoint(2)
            do 71, i = 1,npoint(1)
               k = k+1
               poscsx(1,k,irun) = positn(i,1)
               poscsx(2,k,irun) = positn(j,2)
 71         continue
 70      continue
c
c Set all scales = 1.0
         if (iniscl .le. 1) then
            do 80, j = 1,ncscls(1,irun)
               do 81, i = 1,ncscls(2,irun)
                  cscals(i,j,irun) = 1.0
 81            continue
 80         continue
         endif
c
c Spherical harmonic SURFACE option, set pole if not already set
c  and calculate permuted orthogonalization matrix
         ir = isphpt(irun)
         if (mhpole(irun) .lt. 1) then
c Always set pole even if not required (might as well)
c For linked runs use info from linked set            
            if (kspnax(ir) .gt. 0) then
c  Spindle axis known
               mhpole(irun) = kspnax(ir)
            else
c  Not known so set to k (monoclinic) or l
               mhpole(irun) = 3
               if (nlaue .eq. 4 .or. nlaue .eq. 5)
     $            mhpole(irun) = 2
            endif
         endif
         if (ir .ne. irun) then
c If already set, enforce consistency within linked set
            if (mhpole(irun) .ne. mhpole(ir)) then
               write (lunout, '(a/a,i6/)')
     $            ' **** WARNING inconsistent POLE between linked runs',
     $            ' **** pole reset for run ',nrun
               mhpole(irun) = mhpole(ir)
            endif
         endif
c
         if (lsmode(irun) .eq. 3 .or. lsmode(irun) .eq. 4) then
c
c Calculate permuted orthogonalization matrix
c  Use file-global cell and dummy 1A wavelength (ie matrix will be in A**-1)
            iaxes(3) = mhpole(irun)
            iaxes(1) = iaxes(3) + 1
            if (iaxes(1) .gt. 3) iaxes(1) = 1
            iaxes(2) = iaxes(1) + 1
            if (iaxes(2) .gt. 3) iaxes(2) = 1
            call mkbmat(cell, bmatrn(1,1,irun), 1.0, iaxes)
c "permutation" matrix Q = [B] [B']**1
c (not strictly permutation in non-orthogonal case)
            call minv3(bmpinv, bmatrn(1,1,irun), det)
            call matmul(qmat, bmatall, bmpinv)
         else
c Store unpermuted orthogonalization matrix for other runs,
c for completeness
            call ccpmvr(bmatrn(1,1,irun), bmatall, 9)
            call unitmt(qmat)
         endif
c  DUQinv = ([D][U][Q])**-1 (for 1st DU in run, if present)
         k = jmtxbt(1,irun)
         if (k .gt. 0) then
c   [DU][Q]
            call matmul(tmat, dumatxs(1,1,k), qmat)
            call minv3(DUQinv(1,1,irun), tmat, det)
         else 
            call unitmt(DUQinv(1,1,irun))
         endif
c=== Sanity checks
         if (nbfpos(irun) .gt. 1 .and. nrnsph(irun) .gt. 1) then
            write (lunout, '(/a/a,i5,a/)')
     $         ' **** You cannot use BFACTOR ANISOTROPIC and'//
     $         ' SECONDARY or CRYSTAL corrections in the same  '//
     $            'run ****',
     $            ' **** Run number ',nrun,' ****'
               call ccperr(1,
     $            '*** Illegal combination of SCALES arguments ***')
         endif
         if (ncscls(2,irun) .gt. 1) then
            if (nbfpos(irun) .gt. 1) then
               write (lunout, '(/a/a,i5,a/)')
     $            ' **** You cannot use BFACTOR ANISOTROPIC and'//
     $            ' vary scales across the detector in the same '//
     $            'run ****',
     $            ' **** Run number ',nrun,' ****'
               call ccperr(1,
     $            '*** Illegal combination of SCALES arguments ***')
            endif
            if (nrnsph(irun) .gt. 1) then
               if (lsmode(irun) .eq. 2) then
                  label = 'SECONDARY'
               elseif (lsmode(irun) .eq. 3) then
                  label = 'ABSORPTION'
               elseif (lsmode(irun) .eq. 4) then
                  label = 'SURFACE'
               else
                  label = 'WHAT?'
               endif
               write (lunout, '(/a,a,a/a,i5,a/)')
     $            ' **** You cannot use SCALES ',label(1:lenstr(label)),
     $            ' and'//
     $            ' vary scales across the detector in the same '//
     $            'run ****',
     $            ' **** Run number ',nrun,' ****'
               call ccperr(1,
     $            '*** Illegal combination of SCALES arguments ***')
            endif
c
            if (nbfact(irun) .ne. 0) then
               write (lunout, '(/a/a,i4,a,a/a/a/)')
     $            ' WARNING WARNING WARNING WARNING WARNING WARNING ',
     $            ' In run',nrun,
     $            ' you have selected BFACTOR ON and variation of ',
     $            'scale across the detector',
     $            ' This combination is likely to be unstable',
     $            ' WARNING WARNING WARNING WARNING WARNING WARNING '
            endif
         endif
c
c Checks on ""time" & rotation gaps for this run
         if (mbatsc(irun) .le. 0 .and. lbfrun(irun)) then
c   Scale gap & maximum range by spacing
            gap = gprtmx/spcbfc(irun)
            gaptmx = max(gaptmx, abs(gap))
            k = jgptmx
            if (abs(gap) .gt. timwrn) then
               if (nwarn.lt.mxwarn .and. k.gt.1) then
                  write (lunout,'(a,i3,a,i6,a,i6,a,f10.2,a/a,a)')
     $               ' #### WARNING: in run number',nrun,
     $               ', between batches ',jbatrn(k-1, irun),' and ',
     $               jbatrn(k, irun),' there is a gap of ',
     $               gap,' "time" (rotation) units.',
     $        ' #### This may invalidate the time(rotation)-dependent',
     $               ' B-factor'
                  nwarn = nwarn+1
               elseif (nwarn .eq.mxwarn) then
                  write (lunout, '(a)')
     $               ' #### . . Further warnings suppressed . . .'
                  nwarn = nwarn+1
               endif
            endif
         endif
c
         if (mbatsc(irun) .eq. 0 .and. ncscls(1,irun) .gt. 1) then
c  Gap & maximum range by spacing
            gap = gprrmx/spcscl(3,irun)
            gaprmx = max(gaprmx, abs(gap))
            k = jgprmx
            if (abs(gap) .gt. phiwrn) then
               if (nwarn .lt.mxwarn .and. k.gt.1) then
                  write (lunout,'(a,i3,a,i6,a,i6,a,f10.2,a/a)')
     $               ' #### WARNING: in run number',nrun,
     $               ', between batches ',jbatrn(k-1, irun),' and ',
     $               jbatrn(k, irun),' there is a gap of ',
     $               gap,' "Rotation" units.',
     $               ' #### This may invalidate the smooth scales'
                  nwarn = nwarn+1
               elseif (nwarn .eq.mxwarn) then
                  write (lunout, '(a)')
     $               ' #### . . Further warnings suppressed . . .'
                  nwarn = nwarn+1
               endif
            endif
         endif
c--- Check necessary columns are present
         if (ltestr .and. ncz .eq. 0) then
            if (anmphd .eq. 0.0) then
               write (lunout, '(/a/a/)')
     $            ' **** You must have a ROT column in the '//
     $            'input file to use smoothed scaling ****',
     $            ' **** BATCH mode scaling would be legal ****'
            else
               write (lunout, '(/a/)')
     $            ' **** You must have a ROT column in the '//
     $            'input file to use ANOMALOUS MATCH PHIDIF****'
            endif
            call ccperr(1,'*** No ROT column in input file ***')
         endif
         if (ltsdet .and. .not. lcddet) then
            write (lunout, '(/a/)')
     $           ' **** You must have XDET & YDET columns in the '//
     $           'input file, with valid data,  to vary scales '//
     $           'across the detector ****'
            call ccperr(1,
     $         '*** XDET or YDET columns not both in input file ***')
         endif
c
c Set parameter counts
c  nprrun  is total (for this run) of number of scale parameters,
c Bfactor parameters, & diffuse correction parameters
c Note that if TDS parameters are LINKED, then total number of global
c parameters will be less than the sum of the nprrun's
         nprrun(irun) = ncscls(1,irun) * ncscls(2,irun)
         if (lbfrun(irun)) nprrun(irun) = nprrun(irun) + nbparm(irun)
c  Offset to beginning of scales
         ipnscl(irun) = nbparm(irun) + 1
c  Offset to beginning of TDS parameters
         ipntds(irun) = nprrun(irun) + 1
         if (.not. ltdsrn(irun)) then
            nrntds(irun) = 0
         else
            nprrun(irun) = nprrun(irun) + nrntds(irun)
         endif
c  Offset to beginning of spherical harmonic parameters
         ipnsph(irun) = nprrun(irun) + 1
         if (nrnsph(irun) .gt. 0) then
            nprrun(irun) = nprrun(irun) + nrnsph(irun)
         endif
c
c  Offset to beginning of SD parameters
         ipnsdf(irun) = nprrun(irun) + 1
c
c         if (jsdfac(irun) .ne. 0) then
c            npqrun(irun) = maxsdf
c%c            npqrun(irun) = 1
c         else
         npqrun(irun) = 0
c         endif
         npmrun(irun) = nprrun(irun) + npqrun(irun)
         if (npmrun(irun) .gt. maxpmr) then
            write (lunout, '(a,i4,a,i5,a,i5)')
     $           ' **** Too many parameters for run number ',
     $           nrun,', there are ', npmrun(irun),
     $           ', maximum = ',maxpmr
            call ccperr(1,'*** Too many parameters ***')
         endif
c
c Set sd factors
         if (jsdfac(irun) .lt. 0) then
c   jsdfac .gt. 0 if factors already set explicitly, = -1 to be set here
            sdfact(1,irun,1) = sdfacdef(1)
            sdfact(2,irun,1) = sdfacdef(2)
            sdfact(3,irun,1) = sdfacdef(3)
            sdfact(1,irun,2) = sdfacdef(1)
            sdfact(2,irun,2) = sdfacdef(2)
            sdfact(3,irun,2) = sdfacdef(3)
            jsdfac(irun) = +1
         endif
c
c Check for legitimate sdfactors
         do 85, j = 1,2
            if (sdfact(1, irun, j) .lt. 0.0001) then
               write (lunout, '(a,i6,a,f12.6)')
     $            ' **** Illegal SD correction factor 1 for run',
     $            nrun,' = ',sdfact(1,irun,j)
               istat = istat+1
            endif
 85      continue
c
c
c Set pointers to global parameters
c   B-factors, then Scale parameters, then diffuse parameters first,
c since all scale parameters precede all sd parameters
         nlnkrn(irun) = 0
         if (ltdsrn(irun)) then
            if (itdspt(irun) .eq. irun) then
c  Count links & total TDS parameters
c  Not linked
               nprtds = nprtds + nrntds(irun)
            else
c  Linked
               nlnkrn(irun) = nlnkrn(irun) + nrntds(irun)
            endif
         endif
         if (nrnsph(irun) .gt. 0) then
            if (isphpt(irun) .eq. irun) then
c  Count links & total SPH parameters
c  Not linked
               nprsph = nprsph + nrnsph(irun)
            else
c  Linked
               nlnkrn(irun) = nlnkrn(irun) + nrnsph(irun)
            endif
c  initialize
            do i = 1, nrnsph(irun)
               sphclm(i, irun) = 0.0
            enddo
         endif
c
         np = 0
         do 90, i = 1, nprrun(irun)
            if (i .le. nbparm(irun)) then
c    Type: Bfactor
               jptyp = 1
            elseif (i .lt. ipntds(irun)) then
c          Scale
               jptyp = 2
            elseif (i .lt. ipnsph(irun)) then
c          Diffuse correction
               jptyp = 3
            else
c          Spherical harmonic
               jptyp = 4
            endif
c Process counts for Links
            if (jptyp .eq. 3 .and. itdspt(irun) .ne. irun) then
c Special for linked TAILS parameters (note link is always backwards
c to earlier run (see s/r setlnk)
c Global parameter is shared with another run: set pointers
c from global to local (ipnloc, ipnrun), local to global pointer
c ipoint
               k = i - ipntds(irun) + ipntds(itdspt(irun))
               jp = ipoint(k, itdspt(irun))
c  Link status, forward & back flags
               ilnkrn(i,irun) = - iabs(itdspt(irun))
               ilnkrn(i,itdspt(irun)) = + irun
c Which TDS parameter is this?
               kp = i - ipntds(irun) + 1
               if (kp .le. 0)
     $            call ccperr(1,'*** SETSCL: illegal kp ***')
               if (ipnloc(jp) .gt. 0) then
c   global pointer jp still normal, so store current pointer in iplloc
c & reset it, set global run pointer = 0 as marker
                  iplloc(kp, ipnrun(jp)) = ipnloc(jp)
                  ipnloc(jp) = -kp
               else
c   global pointer points to link pointer list iplloc already, check
                  if (iabs(ipnloc(jp)) .ne. kp) then
                     write (lunout, '(/a,2i6,a,i6/)')
     $                  ' **** Global pointer ',jp, ipnloc(jp),
     $                  ' should be ', -kp
                     call ccperr(1,'*** SETSCL: wrong pointer ***')
                  endif
               endif
               iplloc(kp, irun) = i
            elseif (jptyp .eq. 4 .and. isphpt(irun) .ne. irun) then
c Special for linked SURFACE parameters (note link is always backwards
c to earlier run (see s/r setlnk)
c Global parameter is shared with another run: set pointers
c from global to local (ipnloc, ipnrun), local to global pointer
c ipoint
               k = i - ipnsph(irun) + ipnsph(isphpt(irun))
               jp = ipoint(k, isphpt(irun))
c  Link status, forward & back flags
               ilnkrn(i,irun) = - iabs(isphpt(irun))
               ilnkrn(i,isphpt(irun)) = + irun
c Which SPH parameter is this?
               kp = i - ipnsph(irun) + 1
               if (kp .le. 0)
     $            call ccperr(1,'*** SETSCL: illegal kp ***')
c Offset kp by TDS parameters for this run
               kp = kp + nrntds(irun)
               if (ipnloc(jp) .gt. 0) then
c   global pointer jp still normal, so store current pointer in iplloc
c & reset it, set global run pointer = 0 as marker
                  iplloc(kp, ipnrun(jp)) = ipnloc(jp)
                  ipnloc(jp) = -kp
               else
c   global pointer points to link pointer list iplloc already, check
                  if (iabs(ipnloc(jp)) .ne. kp) then
                     write (lunout, '(/a,2i6,a,i6/)')
     $                  ' **** Global pointer ',jp, ipnloc(jp),
     $                  ' should be ', -kp
                     call ccperr(1,'*** SETSCL: wrong pointer ***')
                  endif
               endif
               iplloc(kp, irun) = i
            else
               jpoint = jpoint+1
               jp = jpoint
               ipnloc(jp) = i
               ipnrun(jp) = irun
               np = np+1
               ilnkrn(i,irun) = 0
            endif
            iptyp(jp) = jptyp
            ipoint(i,irun) = jp
 90      continue
         nparar = nparar + np
c--- end run loop
 10   continue
c
      if (istat .ne. 0) return
c
c Set up anomalous match stuff, if required, for each run,
c and print what it is doing
      call stumtc(istat1)
      if (istat1 .ne. 0) call ccperr
     $   (1,'*** Error in Anomalous Match options ***')
c Checks - now just warnings
      if (gaptmx .gt. timchk) then
         write (lunout,'(/a,f10.1,a/a/a/)')
     $      ' **** Large gap in "time" (rotation) coordinate: ',
     $      gaptmx,' ****',
     $      ' **** See WARNING above ****',
     $      ' **** Smoothed B-factor impossible ****'
         call ccperr(1,'*** Gap in "time" (rotation) ***')
      endif
      if (gaprmx .gt. phichk) then
         write (lunout,'(/a,f10.1,a/a/a/)')
     $      ' **** Large gap in rotation coordinate: ',gaprmx,' ****',
     $      ' **** See WARNING above ****',
     $      ' **** Smoothed scaling impossible ****'
         call ccperr(1,'*** Gap in rotation ***')
      endif
c
      if (lcorner .and. ldetdiff) then
         write (lunout, '(/a)')
     $      'ERROR: not all batches have the same detector size.'
         write (lunout, '(/a)')
     $      'CORNERCORRECTION cannot be used'
         call ccperr(1,
     $    '**** Cannot do CORNERCORRECTION if detector size varies ***')
      endif
c
c  Now pointers to sd parameters
      nparam = nparar
c
c      do 200, irun = 1, numrun
c         if (jsdfac(irun) .ne. 0) then
c            nparam = nparam + npqrun(irun)
c            k = nprrun(irun)
c            do 210, i = 1, npqrun(irun)
c               jpoint = jpoint+1
c               k = k+1
c               ipoint(k,irun) = jpoint
c               ipnloc(jpoint) = k
c               ipnrun(jpoint) = irun
c               iptyp(jpoint) = 5
c 210        continue
c         endif
c 200  continue
c
      if (nparam .gt. maxpar) then
         write (lunout, '(//a,i8,a,i8,a//)')
     $        ' **** Too many parameters in total:- ',nparam,
     $        ', maximum is ',maxpar,' ****'
         call ccperr(1,'** Too many parameters **')
      endif
c
      return
      end
c
c
c
      subroutine setsxy(istat)
c     =======================
c
c  Set up bins for xy analysis
c  Assume all coordinates are in same units, though size of detector
c may vary, so bin relative to centre
c
c On exit:
c  istat =  0  OK
c        = -1  both bin sizes = 0, ie no detector data
c
      integer istat
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'rundef.fh'
      include 'outcon.fh'
c
      integer j, irun
c
c Force number of bins even
      do 1, j = 1,2
         nbinxy(j) = 2*(nbinxy(j)/2)
 1    continue
c
      rbinxy(1) = +1.0e10
      rbinxy(2) = +1.0e10
      sbinxy(1) = -1.0e10
      sbinxy(2) = -1.0e10
c
c Get detector limits
      do 10, irun = 1, numrun
         do 11, j = 1,2
            rbinxy(j) = min(rbinxy(j), dlimrn(1,j,irun))
            sbinxy(j) =
     $         max(sbinxy(j), dlimrn(2,j,irun)-dlimrn(1,j,irun))
            cbinxy(j,irun) = 0.5*(dlimrn(2,j,irun)+dlimrn(1,j,irun))
 11      continue
 10   continue
c
      istat = -1
      do 20, j = 1,2
         sbinxy(j) = sbinxy(j)/float(nbinxy(j))
         if (sbinxy(j) .gt. 0.0001) istat = 0 
 20   continue
      return
      end
      subroutine settie
c     =================
c
c     Set up ties between scale factors
c
      implicit none
c
      include 'parameter.fh'
      include 'params.fh'
      include 'scales.fh'
      include 'tie.fh'
      include 'rundef.fh'
      include 'inout.fh'
      include 'refcon.fh'
      include 'glocon.fh'
      include 'sphhrm.fh'
c
c
      integer i,j,k,l,lj,lk,lr, irun, nstep
      logical lsltie
c
c-save-      save /tie/
c
c
      nties = 0
      lsltie = .false.
c
      if (ntiexy .ne. 0) then
c Ties in detector plane
c
         do 10, irun = 1,numrun
c   Global parameter pointer for first scale parameter for this run
            lr = ipoint(1,irun) + nbparm(irun) - 1
            if (ncscls(2,irun) .gt. 1) then
               do 20, k = 1,ncscls(1,irun)
                  lk = (k-1) * ncscls(2,irun) + lr
c     ties along x
                  if (ncscls(3,irun) .gt. 1) then
                     do 30, j = 1,ncscls(4,irun)
                        lj = (j-1)*ncscls(3,irun) + lk
                        do 40, i = 1, ncscls(3,irun) - 1
                           nties = nties+1
                           if (nties .gt. maxtie) go to 990
                           jties(1,nties) = lj + i
                           jties(2,nties) = lj + i + 1
                           jties(3,nties) = irun
                           jties(4,nties) = 1
 40                     continue
 30                  continue
                  endif
c
c     ties along y
                  if (ncscls(4,irun) .gt. 1) then
                     do 50, i = 1,ncscls(3,irun)
                        do 60, j = 1,ncscls(4,irun) - 1
                           nties = nties+1
                           if (nties .gt. maxtie) go to 990
                           jties(1,nties) =
     $                          (j-1)*ncscls(3,irun) + i + lk 
                           jties(2,nties) = j*ncscls(3,irun) + i + lk 
                           jties(3,nties) = irun
                           jties(4,nties) = 1
 60                     continue
 50                  continue
                  endif
 20            continue
            endif
 10      continue
      endif
      ntiexy = nties
c
      if (ntiez .ne. 0) then
c Ties along rotation
c
         do 110, irun = 1,numrun
            if (lbtslp(irun)) then
c  In Slope mode, only tie together scales which belong to same batch
               nstep = 2
               lsltie = .true.
            else
               nstep = 1
            endif
            lr = ipoint(1,irun) + nbparm(irun) - 1
            if (ncscls(1,irun) .gt. 1) then
               do 120, k = 1, ncscls(1,irun) - 1, nstep
                  lk = (k-1)*ncscls(2,irun) + lr
                  do 130, l = 1,ncscls(2,irun)
                     nties = nties+1
                     if (nties .gt. maxtie) go to 990
                     jties(1,nties) = lk + l
                     jties(2,nties) = lk + l + ncscls(2,irun)
                     jties(3,nties) = irun
                     jties(4,nties) = 2
 130              continue
 120           continue
            endif
 110     continue
      endif
      ntiez = nties - ntiexy
c
c Ties to surface parameters (only if not linked)
      if (ntiesrf .ne. 0) then
         ntiesrf = 0
         do irun = 1, numrun
            if (nrnsph(irun) .gt. 0 .and. isphpt(irun) .eq. irun) then
c   Global parameter number for first coefficient
               lk = ipoint(ipnsph(irun), irun) - 1
               do l = 1, nrnsph(irun)
                  nties = nties+1
                  ntiesrf = ntiesrf + 1
                  if (nties .gt. maxtie) go to 990
                  jties(1,nties) = lk + l
                  jties(2,nties) = 0
                  jties(3,nties) = irun
                  jties(4,nties) = 3
               enddo
            endif
         enddo
      endif
c
      if (ntiebf .ne. 0) then
c Ties on B-factor
c
         ntiebf = 0
         do irun = 1,numrun
            if (nbfact(irun) .gt. 1) then
               if (nbfpos(irun) .ne. 1) then
                  write (lunout, '(/a/)')
     $               '**** You cannot tie anisotropic Bfactors ****'
                  call ccperr(1,'*** Illegal TIE of anisotropic B ***')
               endif
               lk = ipoint(1,irun) - 1
               do k = 1, nbfact(irun)-1
                  nties = nties+1
                  if (nties .gt. maxtie) go to 990
                  jties(1,nties) = lk + k
                  jties(2,nties) = lk + k + 1
                  jties(3,nties) = irun
                  jties(4,nties) = 4
                  ntiebf = ntiebf + 1
               enddo
            endif 
         enddo 
      endif
c Ties to A1 tails parameters (only if not linked)
      if (ntieA1 .ne. 0) then
         ntieA1 = 0
         do irun = 1, numrun
            if (ltdsrn(irun) .and. itdspt(irun) .eq. irun) then
c   Global parameter number for A1 coefficient (3rd TDS parameter)
               lk = ipoint(ipntds(irun), irun) + 2
               nties = nties+1
               ntieA1 = ntieA1 + 1
               if (nties .gt. maxtie) go to 990
               jties(1,nties) = lk
               jties(2,nties) = 0
               jties(3,nties) = irun
               jties(4,nties) = 5
            endif
         enddo
      endif
c
c Print things
c
      if (ntiexy .gt. 0) then
         write (lunout, 6001) sqrt(Vtiexy), ntiexy
 6001    format(//
     $        ' Scales will be TIED together in the detector plane',
     $        ' with a standard deviation of ',f10.4,
     $        ', number of ties = ',i6/)
      endif
      if (ntiez .gt. 0) then
         write (lunout, 6002) sqrt(Vtiez), ntiez
 6002    format(//
     $        ' Scales will be TIED together along the rotation axis',
     $        ' with a standard deviation of ',f10.4,
     $        ', number of ties = ',i6/)
         if (lsltie) write (lunout, 6003)
 6003    format(' Scales for runs in SLOPE mode will be tied in pairs'/)
      endif
      if (ntiesrf .gt. 0) then
         write (lunout, 6005) sqrt(Vtiesrf), ntiesrf
 6005    format(//
     $        ' SURFACE parameters will be TIED to zero ',
     $      '(ie restrained to a spherical correction)',/,
     $        '    with a standard deviation of ',f10.4,
     $        ', number of ties = ',i6/)
      endif
      if (ntiebf .gt. 0) then
         write (lunout, 6006) sqrt(Vtiebf), ntiebf
 6006    format(//
     $      ' B-factors will be TIED together along the rotation axis',
     $      ' with a standard deviation of ',f10.4,
     $      ', number of ties = ',i6/)
      endif
      if (ntieA1 .gt. 0) then
         write (lunout, 6007) sqrt(VtieA1), ntieA1
 6007    format(//
     $        ' Tails parameters A1 will be TIED to initial values',
     $        ' with a standard deviation of ',f10.4,
     $        ', number of ties = ',i6/)
      endif
c
      if (kprint(1) .ge. 3) then
c Print all ties
         if (ntiexy .gt. 0) then
            write (lunout, '(/1x,a,/,1000(12(2x,2i4)/))')
     $           ' Ties on detector:', ((jties(i,j), i=1,2), j=1,ntiexy)
         endif
         if (ntiez .gt. 0) then
            write (lunout, '(/1x,a,/,1000(12(2x,2i4)/))')
     $           ' Ties on rotation:',
     $           ((jties(i,j+ntiexy), i=1,2), j=1,ntiez)
         endif
         if (ntiebf .gt. 0) then
            write (lunout, '(/1x,a,/,1000(12(2x,2i4)/))')
     $           ' Ties on Bfactor:',
     $           ((jties(i,j+ntiexy+ntiez), i=1,2), j=1,ntiebf)
         endif
      endif
c
      return
c
c
 990  write (lunout, '(a,i8,a)')
     $     ' **** Too many ties, maximum number (maxtie) =',maxtie,
     $   ' ****'
      call ccperr(1,'** Too many Ties, increase dimensions **')
      end
      subroutine setupf(nrffil)
C    =========================
C
C---- Open main  i/o files,
C     get column assignments from header labels
c
c On exit:
c  nrffil    number of observations in file
C
      integer nrffil
C
C Parameters
C  NCOLI    maximum number of assigned columns in file
      integer ncoli
      parameter (ncoli=26)
C
      include 'parameter.fh'
      include 'datasets.fh'
      include 'inout.fh'
C
C     ..
C     .. Local Scalars ..
      integer i,iprint,ifail,nrefl, istpsr, knumcol
      character versn*10
C     ..
C     .. Local arrays ..
      real ranges(2,maxcol)
      integer kcol(ncoli),jsort(5)
      character look(ncoli)*30, cltype(ncoli)*1
C
      external ccp4_hash_setup
C     ..
C     .. Common blocks ..
      include 'column.fh'
      include 'rfile.fh'
      include 'nbtchc.fh'
      include 'scales.fh'
C  Symmetry, cell
      include 'symmty.fh'
C     ..
C
C---- Column labels required from file
C
      data look/'H','K','L','M/ISYM','BATCH','I','SIGI',
     $   'IPR','SIGIPR','SCALE','SIGSCALE','TIME','XDET','YDET','ROT',
     $   'SIGIC','SIGIPRC','MPART','FRACTIONCALC','LP',
     $   'FLAG','BGPKRATIOS',
     $   'IMEAN','SIGIMEAN','ISUM','SIGISUM'/
C
C
C---- Compulsory/optional flags for each column
      data kcol/7*-1,19*1/
c First column number for Postref
      data  istpsr/23/
C
C---- Column types
      data cltype/
     $   'H','H','H','Y','B','J','Q','J','Q','R','Q','R',
     $     'R','R','R','Q','Q','I','R','R','I','R',
     $   'J','Q','J','Q'/
C     ..
C
C
C---- Open input file
C
      iprint = 1
      ifail = 0
C          ******************************
      call lropen(1,'HKLIN',iprint,ifail)
C          ******************************
      if (ifail .ne. 0) then
         call ccperr(1,'*** Failed to open HKLIN file ***')
      endif
C
C---- Read header stuff
C
C          *************
      call lrcell(1,cell)
      call lrinfo(1,versn,ncol,nrefl,ranges)
      call lrassn(1,look,ncoli,kcol,cltype)
      call lrrsol(1,sminf,smaxf)
      call lrtitl(1,rtitle,i)
c
c---- Read dataset definitions from hklin file
c
      ndatasets = maxinsets
c*5*
c*4*      call lridc
c*4*     $   (1,pname_in,dname_in,idtsets,dcell_in,dwavel_in,ndatasets)
c*4*c  xname_in are place-holders for future crystal names
c*4*      do i = 1,ndatasets
c*4*         xname_in(i) = ' '
c*4*      enddo
      call lridx
     $   (1,pname_in,xname_in,dname_in,idtsets,
     $   dcell_in,dwavel_in,ndatasets)
      if (ndatasets .lt. 0) then
         call ccperr(1,
     $      '*** Too many input datasets, increase MAXINSETS ***')
      endif
c*5*
c
c File with no datasets (old format) - set up sensible defaults
      if (ndatasets .eq. 0) then
         write (lunout, '(/a,a/)')
     $      ' WARNING: input HKLIN file has no datasets defined, ',
     $      'defaults have been set: WARNING'
         ndatasets = 1
         pname_in(1) = 'Unspecified'
         xname_in(1) = 'Unspecified'
         dname_in(1) = 'Unspecified'
         dwavel_in(1) = 0.0
         do i = 1,6
            dcell_in(i,1) = 0.0
         enddo
      endif
c
c
      nrffil = nrefl
C
C---- Read all batch numbers into array
C          **********************
      call lrbats(1,nbtall,ibtall)
C          **********************
      if (nbtall .gt. maxbat) then
         call ccperr
     $      (1,'*** Too many batches, increase MAXBAT & recompile ***')
      endif
C  Set up lookup table for all batchs
      do 10, i = 1, nbtall
         call ccp4_hash_setup(ibtall(i), i)
 10   continue
C
C---- Set up list of datasets (& crystal numbers) for each batch
C
      call setcry
C
C---- Read & set up symmetry
C          ******************************************
      call lrsymi(1,nsymp,lattyp,nspgrp,spgnam,pgname)
      call lrsymm(1,nsym,rsym)
      call asuset(spgnam,nspgrp,pgname,nsym,rsym,nsymp,nlaue,.true.)
C set up centric tests
      call centric(nsym, rsym, .false.)
c set up multiplicity (epsilon) calculation
      call epsln(nsym,nsym,rsym,0)
C          *********************************************************
C
C---- Store column positions
C
      ncmsym = kcol(4)
      ncbatc = kcol(5)
      nci    = kcol(6)
      ncsdi  = kcol(7)
      ncip   = kcol(8)
      ncsdip = kcol(9)
      ncsc   = kcol(10)
      ncsdsc = kcol(11)
      nct    = kcol(12)
      ncx    = kcol(13)
      ncy    = kcol(14)
      ncz    = kcol(15)
      ncsdic = kcol(16)
      ncsdpc = kcol(17)
      ncmprt = kcol(18)
      ncfrcl = kcol(19)
      nclp   = kcol(20)
      ncflag = kcol(21)
      ncbgpk = kcol(22)
C
C---- Check sort order: should be H K L M/ISYM BATCH
C          *************
      call lrsort(1,jsort)
C          *************
      do 20, i = 1, 5
         if (jsort(i) .ne. kcol(i)) then
            call ccperr(1,
     $           '*** File must be sorted on H K L M/ISYM BATCH ***')
         endif
 20   continue
C
c Store fractioncalc range
      if (ncfrcl .gt. 0) then
         frclrg(1) = ranges(1,ncfrcl)
         frclrg(2) = ranges(2,ncfrcl)
      endif
c
c Maximum intensity
      flmxin = ranges(2,nci)
c
c Postref columns must be all present or all absent, & contiguous
      do 30, i = istpsr+1, istpsr+3
         if (kcol(istpsr) .gt. 0) then 
            lpstrf = .true.
            ncopsf = kcol(istpsr)
            if (kcol(i) .ne. kcol(i-1) + 1) then
               call ccperr(1,
     $            '*** Existing columns for IMEAN SIGIMEAN '//
     +         'ISUM SIGISUM are not contiguous ***')
            endif
         else
            if (kcol(i) .ne. 0) then
               call ccperr(1,'*** Columns IMEAN SIGIMEAN ISUM SIG'//
     +         'ISUM must be all present or all absent ***')
            endif
         endif
 30   continue
c
c Partials present if maximum value of M/ISYM .gt. 256
      if (ranges(2,4) .gt. 256) then
         lprtfl = .true.
      else
         lprtfl = .false.
      endif
c
c Set flag for presence of XDET, YDET
      lcddet = .false.
      if (ncx .gt. 0 .and. ncy .gt. 0) then
         if (
     $      (abs(ranges(2,ncx)-ranges(1,ncx)) .gt. 0.001) .and.
     $      (abs(ranges(2,ncy)-ranges(1,ncy)) .gt. 0.001)) then
            lcddet = .true.
         endif
      endif
c Disable ROT column if it contains no useful information
      if (ncz .gt. 0) then
         if (abs(ranges(2,ncz)-ranges(1,ncz)) .lt. 0.001) then
            ncz = 0
         endif
      endif
c
C
      return
      end
      subroutine setwgt(kwgtyp, nrs, ffrf, sdffrf, ghl, wtrf)
c     ======================================================
c
c  Set weights for observations
c
c
c  On entry:
c     kwgtyp    weight type
c               = 1 unit weights
c               = 2 variance weights
c               = 3 sqrt scale weights
c     nrs       number of observations
c     ffrf      intensities
c     sdffrf    sd(Ihl)
c     ghl       dividing scales
c
c  On exit:
c     wtrf      weights = sqrt(1/ghl)
c
      implicit none
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
c
      integer kwgtyp, nrs, nused
      real ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs)
c
      integer l
c
c
      if (nrs .le. 0) return
c
c
      do 10, l = 1, nrs
c
         if (kwgtyp .eq. 1) then
c  Unit weights
            wtrf(l) = 1.0
         elseif (kwgtyp .eq. 2) then
c  Variance weights
            if (sdffrf(l) .le. 0.0) then
               wtrf(l) = 0.0
            else
               wtrf(l) = 1./sdffrf(l)**2
            endif
         elseif (kwgtyp .eq. 3) then
c  Sqrt scale weights
            if (ghl(l) .le. 0.0) then
               wtrf(l) = 0.0
            else
               wtrf(l) = 1./sqrt(ghl(l))
            endif
         else
            call ccperr(1,'**** SETWGT: unrecognized weight type ****')
         endif
 10   continue
c
      return
      end
      subroutine sgetref(lsanom, maxnrs, ninref, lrefst, kpflag, lselip,
     $     eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $     ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $     isymrf, sclirf, sdscrf, lflgrf, frclrf, clprf, isttrf,
     $     timerf, xdrf, ydrf, zdrf, xsirf, phirf, xdetrf, ydetrf,
     $     nmprrf, datarf, istat)
c     ===========================
c     
c     Read & store all (possibly selected) observations for the next unique hkl
c     Return either all observations, or alternatively just I+ or I- ones,
c     depending on the setting of lsanom
c     
c     Observations are not stored if:-
c     - they come from excluded batches (irun .lt. 0)
c     
c     On entry:
c     lsanom        if .false., return all observations
c     if .true., return first I+ observations, then I- ones
c     on next entry
c     maxnrs        dimension of reflection stores
c     ninref        current running total of raw input reflections 
c                     (to be updated)
c     lrefst        if .true., store observations from reference run
c     kpflag        = -1   accept fulls only, return m = -1 if rejected
c     =  0   accept fulls & previously summed partials (mpart=10)
c     = +1   accept fulls & partials
c     lselip        if .true., select IPR or I into ffrf, sdffrf according
c     to setting of IMAD (see s/r chkref)
c     eof           .true. if starting to read file, ie a new record should
c     be read rather than using stored record
c     
c     On exit:     
c     ninref        updated  running total of raw input reflections
c     eof           .true. if end-of-file found (istat = 0 in this case)
c     jhkl(3)       reflection indices
c     ss            4(sin theta/lambda)**2
c     stol          (sin theta/lambda)**2 (= ss/4)
c     nrs           number of observations stored
c     nuse          number of "useful" ones
c     nfull         number of fully recorded observations
c     lpart         .true. if any partials found
c     iserrf(nrs)   serial number in file for this observation
c     ffrf(nrs)     intensity (from column I, summation intensity)
c     sdffrf(nrs)   sd(intensity) (from column SIGI, summation intensity)
c     ffprrf(nrs)   intensity (from column IPR, profile-fitted I, if present)
c     sdfprf(nrs)   sd(intensity) (from column SIGIPR, if present)
c     ibatrf(nrs)   batch number
c     irunrf(nrs)   run number, = -1 if rejected
c     mrf(nrs)      partial/reject flag, = 0 full, = 1 partial, = 2,3,4 reject
c     = -1 for ignored partial depending on kpflag
c     = 0 for previously summed partial (mpart = 10)
c     mprf(nrs)     partial flag: number of images etc (from MPART if present
c     = mrf if no MPART column)
c     = 10 summed partial
c     .gt. 200, number of images * 100 + image number
c     .gt. 20, number of images * 10 + image number
c     isymrf(nrs)   symmetry number
c     sclirf(nrs)   scale factor read from SCALE column in input file,
c     = 1.0 if none
c     sdscrf(nrs)   sd(scale factor) read from SIGSCALE column in input file,
c     = 0.0 if none
c     lflgrf(nrs)   flag column from file (if present, else = 0)
c     frclrf(nrs)   calculated fraction (if present in file, else = 0.0)
c     clprf(nrs)    LP factor if present in file, else = 0.0
c     isttrf(nrs)   status flag  (see flags.fh)
c     = 0 unconditionally OK
c     .gt.0 conditionally OK
c     .lt.0 rejected
c     timerf(nrs)   normalized "time" coordinate
c     xdrf(nrs), ydrf(nrs), zdrf(nrs)
c     normalized 3D coordinates, depending on mode
c     xsirf(nrs)    xsi coordinate, radius from rotation axis 
c                       (=sqrt(x**2 + y**2)
c     (only calculated for mode 2 or TAILS correction)
c     phirf(nrs)    Phi coordinate from ROT column in file (=0.0 if absent)
c     xdetrf(nrs), ydetrf(nrs)
c     XDET, YDET values from input file
c     nmprrf(nrs)   number of parts contributing to this observation
c     initialized here to 1 except for previously summed partial
c     (MPART = 10), in which case = 2
c     datarf(maxcol,nrs) copy of all input records, for output
c     
c     istat         return status
c     =  0   OK
c     = +1   reflection outside resolution limits (only if all
c     observations are outside limits)
c
c  bkpkrf not returned as not needed (yet!)
c
c     Arguments
      integer maxnrs, ninref, kpflag
      logical lsanom, eof, lpart, lrefst, lselip
      real    ss, stol
      integer jhkl(3), nrs, nuse, nfull, istat
c     Reflection stores
      include 'parameter.fh'
      integer iserrf(maxnrs),
     $   ibatrf(maxnrs), irunrf(maxnrs), mrf(maxnrs), mprf(maxnrs),
     $   isymrf(maxnrs), lflgrf(maxnrs), isttrf(maxnrs), nmprrf(maxnrs)  
      real ffrf(maxnrs), sdffrf(maxnrs), ffprrf(maxnrs), sdfprf(maxnrs),
     $   sclirf(maxnrs), sdscrf(maxnrs), frclrf(maxnrs), clprf(maxnrs),
     $   timerf(maxnrs), xdrf(maxnrs), ydrf(maxnrs), zdrf(maxnrs),
     $   xsirf(maxnrs), phirf(maxnrs), xdetrf(maxnrs), ydetrf(maxnrs),
     $   datarf(maxcol,maxnrs)
c     
      include 'column.fh'
      include 'flags.fh'
c     
c     Local reflection stores, to save all observations for I+- selection
      logical eofq, lpartq
      integer nrsq, nuseq, nfullq, istatq
      integer iserrfq(maxrfl),
     $     ibatrfq(maxrfl), irunrfq(maxrfl), mrfq(maxrfl),
     $     mprfq(maxrfl), isymrfq(maxrfl), lflgrfq(maxrfl),
     $     isttrfq(maxrfl), nmprrfq(maxrfl)
      real ffrfq(maxrfl), sdffrfq(maxrfl), ffprrfq(maxrfl),
     $   sdfprfq(maxrfl), sclirfq(maxrfl), sdscrfq(maxrfl),
     $   frclrfq(maxrfl), bgpkrfq(maxrfl), clprfq(maxrfl), 
     $   timerfq(maxrfl), xdrfq(maxrfl), ydrfq(maxrfl), zdrfq(maxrfl),
     $   xsirfq(maxrfl), phirfq(maxrfl), xdetrfq(maxrfl),
     $   ydetrfq(maxrfl), datarfq(maxcol,maxrfl), bgpkrf(maxrfl)
c     
c     Locals
      integer i, l, n, mplmn
      logical havref
c
      save eofq, lpartq,  nrsq, nuseq, nfullq, istatq, iserrfq,
     $     ibatrfq, irunrfq, mrfq, mprfq, isymrfq, lflgrfq,
     $     isttrfq, nmprrfq, ffrfq, sdffrfq, ffprrfq,
     $     sdfprfq, sclirfq, sdscrfq, frclrfq, bgpkrfq, clprfq,
     $     timerfq, xdrfq, ydrfq, zdrfq, xsirfq, phirfq, xdetrfq,
     $     ydetrfq, datarfq
c     
      if (eof) then
c     First entry for this pass, reinitialize things
c     No reflection stored
         havref = .false.
         eofq = .true.
      endif
c     
c     If no I+/- selection required, just call gethkl & return
      if (.not.lsanom) then
         call gethkl(maxnrs, ninref, lrefst, kpflag, lselip,
     $      eof, jhkl, ss, stol, nrs, nuse, nfull, lpart, iserrf,
     $      ffrf, sdffrf, ffprrf, sdfprf, ibatrf, irunrf, mrf, mprf,
     $      isymrf, sclirf, sdscrf, lflgrf,  bgpkrf, frclrf, clprf, 
     $      isttrf, timerf, xdrf, ydrf, zdrf, xsirf, phirf,
     $      xdetrf, ydetrf, nmprrf, datarf, istat)
         return
      endif
c     
      eof = .false.
c     I+/- selection required, read reflection to local store 
      if (.not. havref) then
c     First entry for each record
         call gethkl(maxrfl, ninref, lrefst, kpflag, lselip,
     $        eofq, jhkl, ss, stol, nrsq, nuseq, nfullq, lpartq,
     $        iserrfq, ffrfq, sdffrfq, ffprrfq, sdfprfq, ibatrfq,
     $        irunrfq, mrfq, mprfq, isymrfq, sclirfq, sdscrfq,
     $        lflgrfq, bgpkrfq, frclrfq, clprfq, isttrfq,
     $        timerfq, xdrfq, ydrfq, zdrfq, xsirfq, phirfq,
     $        xdetrfq, ydetrfq, nmprrfq, datarfq, istat)
         if (istat .ne. 0.or. nrsq .le. 0) then
c Failure or no observations, check eof flag
            if (eofq) then
               eof = .true.
            endif
            return
         endif 
         havref = .true.
c     
c     Set flag for I+ (odd ISYM)
         mplmn = +1
      else
c     Second entry for each record, set flag for I-
         mplmn = 0
         havref = .false.
         if (eofq) then
            eof = .true.
         endif
      endif
c     Copy I+ or I- observations for output
      n = 0
      nfull = 0
      lpart = .false.
      nuse = 0
      do 10, l = 1, nrsq
c     Is this reflection I+ or I-, as selected by mplmn?
         if (mod(isymrfq(l), 2) .eq. mplmn) then
c     Yes, we want this one
            n = n+1
            iserrf(n) = iserrfq(l)
            ffrf(n)   = ffrfq(l)
            sdffrf(n) = sdffrfq(l)
            ffprrf(n) = ffprrfq(l)
            sdfprf(n) = sdfprfq(l)
            ibatrf(n) = ibatrfq(l)
            irunrf(n) = irunrfq(l)
            mrf(n)    = mrfq(l)
            mprf(n)   = mprfq(l)
            isymrf(n) = isymrfq(l)
            sclirf(n) = sclirfq(l)
            sdscrf(n) = sdscrfq(l)
            lflgrf(n) = lflgrfq(l)
            frclrf(n) = frclrfq(l)
            clprf(n)  = clprfq(l)
            isttrf(n) = isttrfq(l)
            timerf(n) = timerfq(l)
            xdrf(n)= xdrfq(l)
            ydrf(n)= ydrfq(l)
            zdrf(n)= zdrfq(l)
            xsirf(n)  = xsirfq(l)
            phirf(n)  = phirfq(l)
            xdetrf(n)= xdetrfq(l)
            ydetrf(n)= ydetrfq(l)
            nmprrf(n) = nmprrfq(l)
            do 20, i = 1, ncol
               datarf(i,n) = datarfq(i,l)
 20         continue
            if (mrf(n) .eq. 0) then
               nfull = nfull+1
            elseif (mrf(n) .eq. 1) then
               lpart = .true.
            endif
            if (isttrf(n) .ge. lqprt .and. irunrf(n) .gt. 0)
     $           nuse = nuse + 1
         endif
 10   continue
      nrs = n
      return
      end

      subroutine sharvest_init
c     ========================
c
c Initialise harvest things
c
      include 'parameter.fh'
      include 'rfile.fh'
      include 'version.fh'
      include 'symmty.fh'
      include 'sharvest.fh'
c
c
c--- initialise
c
      PRIVATE       = .false.
      USECWD        = .false.
      NOHARVEST     = .false.
      NROWLIMIT     = 132
      NSHELL = -999
      HcompltALL= -999.99
      HmulityALL = -999.99
      HpcvALL = -999.99
      HpcvoALL = -999.99
      HrmeasALL = -999.99
      HrmeasoALL = -999.99
      HanomfrcALL = -999.99
      HLaniso = .false.
      HLdiffuse = .false.
      HLNPART = .false.
      NHovercent = -999
      NHoverano  = -999
      HRIAover   = -999.99
      HSDover    = -999.99
      HoverBias  = -999.99
      NHoverntb  = -999
      HanomfrcALL = -999.99
      HcompltALL = -999.99
      NHfull = -999
      NHnpb(1) = -999.99
      NHanom(1) = -999
      NHoverano = -999
      RETURN
      end
      
      subroutine sharvest(pname, xname, dname)
c     ==================================
c
c Write harvest stuff for dataset pname/xname,dname
c
      character*(*) pname, xname, dname
c
      include 'parameter.fh'
      include 'rfile.fh'
      include 'version.fh'
      include 'symmty.fh'
      include 'sharvest.fh'
c
      INTEGER Nflag,KKFlag,Kharvest
      INTEGER CHMODRET,IFAIL,JDO,LL,MKDIRPT
      LOGICAL ISITVAX,VAXVMS
      CHARACTER CIFTIME*50,BUFFER*256,
     +          FILENAME*256,mkdirMode*3,chmodMode*3
      INTEGER lenstr
      EXTERNAL CCPDPN,HCIFTIME,VAXVMS,lenstr
C
      if (noharvest) then
         return
      endif
C
C---- is there a projectname
C
      IF (PNAME(1:1).EQ.' ') THEN
        WRITE (6,FMT=6004)
 6004   FORMAT (
     +' Harvest: NO PNAME given - no deposit file created')
        RETURN
      END IF
C
C---- is there a DNAME
C
      IF (DNAME(1:1).EQ.' ') THEN
        WRITE (6,FMT=6006)
 6006   FORMAT (
     +' Harvest: NO DNAME given - no deposit file created')
        RETURN
      END IF
C
      ISITVAX = VAXVMS()
C mode for mkdir call
      MKDIRMODE = '700'
C mode for chmod call
      CHMODMODE = '755'
C
C---- get correct time format
C
      CALL HCIFTIME(CIFTIME)
C
      IF (UseCWD) THEN
        IF ( .not. ISITVAX) THEN
          FileName = 
     +           './'//DNAME(1:lenstr(DNAME))//'.scala'
        ELSE
          FileName = DNAME(1:lenstr(DNAME))//'.scala'
        END IF
      ELSE
        Buffer = ' '
C
        CALL ugtenv('HARVESTHOME',Buffer)
C
C    If HARVESTHOME not set, default to HOME
        IF (Buffer .eq. ' ') THEN
          IF (ISITVAX) THEN
            CALL ugtenv('SYS$LOGIN',Buffer)
          ELSE
            CALL ugtenv('HOME',Buffer)
          END IF
C
          IF (Buffer .eq. ' ') THEN
            WRITE (6,FMT=6005)
 6005       FORMAT (
     +' Harvest: No HARVESTHOME or HOME environment variable found - ',
     +               'no deposit file created')
            noharvest = .true. 
            RETURN
          END IF
        END IF
C
C---- see if $HARVESTHOME/DepositFiles directory exists
C
        IF (ISITVAX) THEN
          FileName = Buffer(1:Lenstr(Buffer))//':[DepositFiles]'
        ELSE
          FileName = Buffer(1:Lenstr(Buffer))//'/DepositFiles'
        END IF
C
        CALL cmkdir(FileName(1:Lenstr(FileName)),mkdirMode,mkdirPT)
        IF (mkdirPT .lt. 0) THEN
            WRITE (6,FMT=6007)
 6007       FORMAT (
     +' Harvest: Cant mkdir HARVESTHOME/DepositFiles - ',
     +           'no deposit file created')
            noharvest = .true. 
            RETURN
        END IF
C
C--- world read for directory?
C
        IF (.NOT. Private) THEN
            CALL cchmod(FileName(1:Lenstr(FileName)),chmodMode,chmodRet)
            IF (chmodRet .ne. 0) THEN
              WRITE (6,FMT=6008)
 6008         FORMAT (
     +' Harvest: Cant chmod HARVESTHOME/DepositFiles to world',
     +               ' read - no deposit file created')
              noharvest = .true. 
              RETURN
            END IF
        END IF
C
C---- now look for ProjectName sub-directory
C
        IF (ISITVAX) THEN
          FileName = FileName(1:Lenstr(FileName)-1)//'.'//
     +               PNAME(1:lenstr(PNAME))//']'
        ELSE
          FileName = FileName(1:Lenstr(FileName))//'/'//
     +               PNAME(1:lenstr(PNAME))
        END IF
C
        CALL cmkdir(FileName(1:Lenstr(FileName)),mkdirMode,mkdirPT)
        IF (mkdirPT .lt. 0) THEN
            WRITE (6,FMT=6010) PNAME(1:lenstr(PNAME))
 6010       FORMAT (
     +' Harvest: Cant mkdir HARVESTHOME/DepositFiles/',a,
     +           ' - no deposit file created')
            noharvest = .true. 
            RETURN
        END IF
C
C--- world read for directory?
C
        IF (.NOT. Private) THEN
            CALL cchmod(FileName(1:Lenstr(FileName)),chmodMode,chmodRet)
            IF (chmodRet .ne. 0) THEN
              WRITE (6,FMT=6012) FileName(1:Lenstr(FileName))
 6012         FORMAT (
     +' Harvest: Cant chmod HARVESTHOME/DepositFiles/',a,' to',
     +               ' world read - no deposit file created')
              noharvest = .true. 
              RETURN
            END IF
        END IF
C
C---- now create FileName for deposit information
C
        IF (ISITVAX) THEN
          FileName = FileName(1:Lenstr(FileName))//
     +               DNAME(1:lenstr(DNAME))//'.scala'
        ELSE
          FileName = FileName(1:Lenstr(FileName))//'/'//
     +               DNAME(1:lenstr(DNAME))//'.scala'
        END IF
      END IF
C
      noharvest = .false.
C
C---- now open file
C
      IFAIL = 1
      KHARVEST = 88
      CALL CCPDPN(-KHARVEST,FILENAME,'UNKNOWN','F',80,IFAIL)
C
C

      Hversion = 'CCP4_' // versn(1:lenstr(versn)) // '  ' //
     +                      vrdate(1:lenstr(vrdate))
      WRITE(Kharvest,8001) PNAME(1:lenstr(PNAME)),
     +                     DNAME(1:lenstr(DNAME)),
     +                     PNAME(1:lenstr(PNAME)),
     +                     DNAME(1:lenstr(DNAME)),
     +                     CIFTIME(1:lenstr(CIFTIME)),
     +                     Hversion(1:lenstr(Hversion))
 8001 FORMAT(
     +'data_',a,'[',a,']',/,
     +'_entry.id                 ',a,/,
     +'_diffrn.id                ',a,/,
     +'_audit.creation_date      ',a,/,
     +'_software.classification ','''','data reduction','''',/,
     +'_software.contact_author ','''','P.R. Evans','''',/,
     +'_software.contact_author_email pre@mrc-lmb.cam.ac.uk',/,
     +'_software.description ','''',
     +  'scale together multiple observations of reflections','''',/,
     +'_software.name Scala',/,
     +'_software.version ','''',a,'''',/)
C
       WRITE (Kharvest,8002) cell
 8002  FORMAT(
     +'_cell.length_a      ',f9.3,/,
     +'_cell.length_b      ',f9.3,/,
     +'_cell.length_c      ',f9.3,/,
     +'_cell.angle_alpha   ',f9.3,/,
     +'_cell.angle_beta    ',f9.3,/,
     +'_cell.angle_gamma   ',f9.3,/)
C
       WRITE (Kharvest,8003) NSPGRP,SPGNAM
 8003  FORMAT(
     +'_Symmetry.Int_Tables_number     ',i4,/,
     +'_Symmetry.space_group_name_H-M  ','''',a,'''',/)
C
cloop_
c_Symmetry_equiv.id
c_Symmetry_equiv.pos_as_xyz
c1 X,Y,Z
c2 -X,-Y,Z
c3 -X,Y,-Z
c4 X,-Y,-Z
C
        WRITE (Kharvest,8004) 
 8004   FORMAT('_reflns.data_reduction_method',/,
     +'; data reduction merge and scaling used SCALA (CCP4)')
        IF (HLaniso .or. HLdiffuse) WRITE (Kharvest,8600)
 8600   FORMAT('  with options:')
       IF (HLaniso) WRITE (Kharvest,8005)
 8005   FORMAT(
     +' Anisotropic B-factor used to correct for anisotropic fall-off',
     + /,' of scattering.')
       IF (HLdiffuse) WRITE (Kharvest,8006)
 8006   FORMAT(
     +' Correct for the different truncation of tails, by using a ',/,
     +' simple model of thermal diffuse scattering, based on ',/,
     +' suggestions by R.H.Blessing, Cryst. Reviews, 1, 3-58 (1987)')
       WRITE (Kharvest,8009)
 8009  FORMAT(';',/)
C
C
       WRITE (Kharvest,8010) Hsdrej,Hsdrej2
 8010  FORMAT(
     +'_reflns.merge_reject_criterion',/,
     +';  sd multiplier for max deviation from weighted mean I :',/,
     +'     sdref = ',f8.3,/,
     +'   special value for reflections measured twice :',/,
     +'     sdrej2 = ',f8.3,/,
     +' The test for outliers is as follows:',/,
     +'(1) if there are 2 observations (left), then',/,
     +' (a) for each observation Ihl, test deviation',/,
     +'      Delta(hl) = |Chi|',/,
     +' |Ihl - ghl Iother| / sqrt[sigIhl**2 + (ghl*sdIother)**2]',/,
     +'   against sdrej2, where Iother = the other observation',/,
     +' (b) if either Delta(hl) > sdrej2, then',/,
     +'     1. in scaling, reject reflection. Or:',/,
     +'     2. in merging, keep both.',/,
     +'(2) if there 3 or more observations left, then',/,
     +' (a) for each observation Ihl,',/,
     +'  1. calculate weighted mean of all other observations ',/,
     +'     <I>n-1 & its sd(<I>n-1)',/,
     +'  2. deviation   Delta(hl) =',/,
     +'  |Ihl - ghl <I>n-1>| / sqrt[sigIhl**2 + ',
     +              '(ghl*sd(<I>n-1))**2]',/,
     +'  3. find largest deviation max|Delta(hl)|',/,
     +'  4. count number of observations for which ',/,
     +'        |Delta(hl)|.ge.0  (ngt), & ',/,
     +'    for which |Delta(hl)| .lt. 0 (nlt)',/,
     +' (b)  if max|Delta(hl)| > sdrej, ',/,
     +'       then reject one observation, but which one?',/,
     +'  1. if ngt == 1 .or. nlt == 1, then one ',/,
     +'     observation is a long way from the others and this one ',
     +      'is rejected',/,
     +'  2. else reject the one with the worst deviation ',
     +       'max|Delta(hl)|',/,
     +'(3)   iterate from beginning',/,';',/)
C
       WRITE (Kharvest,8100) HRh1,HRh2,NHallobs
 8100  FORMAT(
     +'_reflns.overall_d_res_low              ',f9.2,/,
     +'_reflns.overall_d_res_high             ',f9.2,/,
     +'_reflns.overall_number_observations    ',i10,/)
C
C
        WRITE (Kharvest,8101)
     +   HRes1over, HRes2over, NHovermeas, NHoveruniq
 8101   FORMAT(
     +'_diffrn_reflns.d_res_low            ',f9.2,/,
     +'_diffrn_reflns.d_res_high           ',f9.2,/,
     +'_diffrn_reflns.number_measured_all  ',i10,/,
     +'_diffrn_reflns.number_unique_all    ',i10)
        IF (NHovercent .gt. -999) WRITE (Kharvest,8102) NHovercent
 8102   FORMAT(
     +'_diffrn_reflns.number_centric_all   ',i10)
C
C
        IF (NHoverano .gt. 0) WRITE (Kharvest,8103) NHoverano,
     +                                              HRIAover,
     +                                              HanomfrcALL
 8103   FORMAT(
     +'_diffrn_reflns.number_anomalous_all  ',i10,/,
     +'_diffrn_reflns.Rmerge_I_anomalous_all ',f8.3,/,
     +'_diffrn_reflns.anom_diff_percent_meas  ',f6.2)
C
C
        WRITE (Kharvest,8104) HRIover
 8104   FORMAT(
     +'_diffrn_reflns.Rmerge_I_all          ',f8.3)
        IF (Hoverfsigi .gt. -999.00) WRITE(Kharvest,8106) Hoverfsigi
 8106   FORMAT(
     +'_diffrn_reflns.meanI_over_sigI_all    ',f10.4)
        IF (HSDover .gt. -999.00) WRITE (Kharvest,8107) HSDover
 8107   FORMAT(
     +'_diffrn_reflns.meanI_over_sd_all      ',f10.4)
        IF (NHoverntb .gt. -999) WRITE (Kharvest,8108) HOverBias,
     +   NHoverntb
 8108   FORMAT(
     +'_diffrn_reflns.mean_fract_bias         ',f10.4,/,
     +'_diffrn_reflns.num_fract_bias_in_mean  ',i10)
C
C
      IF (HcompltALL .gt. -999.0) 
     + WRITE (Kharvest,6002) HcompltALL, HmulityALL, HpcvALL ,
     +                       HpcvoALL,   HrmeasALL,  HrmeasoALL
6002  FORMAT(/,
     +'_diffrn_reflns.percent_possible_all    ',f6.2,/,
     +'_diffrn_reflns.multiplicity            ',f6.2,/,
     +'_diffrn_reflns.PCV                     ',f6.2,/,
     +'_diffrn_reflns.PCV_mean                ',f6.2,/,
     +'_diffrn_reflns.Rmeas                   ',f6.2,/,
     +'_diffrn_reflns.Rmeas_mean              ',f6.2)
C
C
      WRITE (Kharvest,8300) HminI,HmaxI
 8300 FORMAT(
     +'_diffrn_reflns.min_intensity      ', f10.2,/,
     +'_diffrn_reflns.max_intensity      ', f10.2)
      IF (NHfull .gt. 0) WRITE (Kharvest,8301) NHfull,
     +                               Hfull,HFFscatter,HSFscatter
 8301 FORMAT(
     +'_diffrn_reflns.num_fully_measured             ',i10,/,
     +'_diffrn_reflns.Intensity_rms_fully_recorded   ',f10.2,/,
     +'_diffrn_reflns.mean_scatter_over_sd_full      ',f10.2,/,
     +'_diffrn_reflns.sigma_scatter_over_sd_full     ',f10.2)
        IF ( .not. HLNPART) WRITE (Kharvest,8302) NHpart,
     +                 Hpart, HFPscatter, HSPscatter
 8302   FORMAT(
     +'_diffrn_reflns.num_partials_measured              ',i10,/,
     +'_diffrn_reflns.Intensity_rms_partially_recorded   ',f10.2,/,
     +'_diffrn_reflns.mean_scatter_over_sd_part          ',f10.2,/,
     +'_diffrn_reflns.sigma_scatter_over_sd_part         ',f10.2)
      WRITE (Kharvest,8304)
 8304 FORMAT(//)
C
c--- shells
C
          IF (NSHELL .gt. 0) THEN
       WRITE(Kharvest,6001)
6001   FORMAT(
     +'loop_',/,
     +'_reflns_shell.d_res_low',/,
     +'_reflns_shell.d_res_high',/,
     +'_reflns_shell.number_measured_all',/,
     +'_reflns_shell.number_unique_all',/,
     +'_reflns_shell.number_centric_all',/,
     +'_reflns_shell.Rmerge_I_all',/,
     +'_reflns_shell.Rmerge_I_all_cumulative',/,
     +'_reflns_shell.meanI_over_sigI_all',/,
     +'_reflns_shell.meanI_over_sd_all',/,
     +'_reflns_shell.percent_possible_all',/,
     +'_reflns_shell.cum_percent_possible_all',/,
     +'_reflns_shell.multiplicity',/,
     +'_reflns_shell.PCV',/,
     +'_reflns_shell.PCV_mean',/,
     +'_reflns_shell.Rmeas',/,
     +'_reflns_shell.Rmeas_mean')
c
c
       KKFlag = 1
       IF (NHnpb(1) .gt. -999.00) THEN
       WRITE (Kharvest,5001)
5001   FORMAT(
     +'_reflns_shell.num_fract_bias_in_mean',/,
     +'_reflns_shell.mean_fract_bias')
       KKFlag = 2
       END IF
C
C
       IF (NHoverano .gt. 0) THEN
         WRITE (Kharvest,5002)
5002    FORMAT(
     +'_reflns_shell.number_anomalous_all',/,
     +'_reflns_shell.Rmerge_I_anomalous_all',/,
     +'_reflns_shell.anom_diff_percent_meas')
        IF (KKFlag .eq. 1) THEN
             KKFlag = 3
        ELSE
             KKFlag = 4
        END IF
        END IF
c
c

          DO 7000 Jdo= 1,Nshell
         IF (KKFlag .eq. 1) THEN
         WRITE (Kharvest,7010) HRes1Shell(Jdo),HRes2Shell(Jdo),
     +         NHall(Jdo),NHuniq(Jdo), NHcent(Jdo), HRmerge(Jdo),
     +         HRcum(Jdo),HsigI(Jdo),Hsdall(Jdo),Hcomplt(Jdo),
     +         Hccmplt(Jdo),Hamltpl(Jdo),Hpcvo(Jdo), Hpcv(Jdo),
     +         Hrmeas(Jdo),Hrmeaso(Jdo)
 7010 FORMAT(2(1x,f7.2),1x,i9,1x,i8,1x,i7,2(1x,f5.2),5(1x,f6.2),
     +       4(1x,f5.2))
         ELSE IF (KKFlag .eq. 2) THEN
          WRITE (Kharvest,7011) HRes1Shell(Jdo),HRes2Shell(Jdo),
     +         NHall(Jdo),NHuniq(Jdo), NHcent(Jdo), HRmerge(Jdo),
     +         HRcum(Jdo),HsigI(Jdo),Hsdall(Jdo),Hcomplt(Jdo),
     +         Hccmplt(Jdo),Hamltpl(Jdo),Hpcvo(Jdo), Hpcv(Jdo),
     +         Hrmeas(Jdo),Hrmeaso(Jdo), NHnpb(Jdo),Hfbias(Jdo)
 7011    FORMAT(2(1x,f7.2),1x,i9,1x,i8,1x,i7,2(1x,f5.2),5(1x,f6.2),
     +          4(1x,f5.2),1x,i7,1x,f6.2)
         ELSE IF (KKFlag .eq. 3) THEN
          WRITE (Kharvest,7012) HRes1Shell(Jdo),HRes2Shell(Jdo),
     +         NHall(Jdo),NHuniq(Jdo), NHcent(Jdo), HRmerge(Jdo),
     +         HRcum(Jdo),HsigI(Jdo),Hsdall(Jdo),Hcomplt(Jdo),
     +         Hccmplt(Jdo),Hamltpl(Jdo),Hpcvo(Jdo), Hpcv(Jdo),
     +         Hrmeas(Jdo),Hrmeaso(Jdo),NHanom(Jdo),
     +         HRanom(Jdo),Hanomfrc(Jdo)
 7012    FORMAT(2(1x,f7.2),1x,i9,1x,i8,1x,i7,2(1x,f5.2),
     +          5(1x,f6.2),4(1x,f5.2),1x,i8,
     +          2(1x,f6.2))
         ELSE IF (KKFlag .eq. 4) THEN
          WRITE (Kharvest,7013) HRes1Shell(Jdo),HRes2Shell(Jdo),
     +         NHall(Jdo),NHuniq(Jdo), NHcent(Jdo), HRmerge(Jdo),
     +         HRcum(Jdo),HsigI(Jdo),Hsdall(Jdo),Hcomplt(Jdo),
     +         Hccmplt(Jdo),Hamltpl(Jdo),Hpcvo(Jdo), Hpcv(Jdo),
     +         Hrmeas(Jdo),Hrmeaso(Jdo),  NHnpb(Jdo), Hfbias(Jdo),
     +         NHanom(Jdo), HRanom(Jdo), Hanomfrc(Jdo)
 7013    FORMAT(2(1x,f7.2),1x,i9,i8,1x,i7,2(1x,f5.2),
     +          5(1x,f6.2),4(1x,f5.2),1x,i7,1x,f6.2,1x,i8,2(1x,f6.2))
          END IF
 7000     continue
         END IF
C
C
      CLOSE (UNIT=KHarvest)
      RETURN
      END
      subroutine simplex(nparam, func,
     $   lcntarr, rcntarr, intgarr, realarr, fail,
     $   tolerance, ncycles, gradient, points, value)
c
c Simplex optimiser
c Translated from C++ code by Kevin Cowtan (simplex-lib.h/.cpp
c
c
c On entry:
c     nparam     number of parameters
c  D  func       function evaluator: this function is minimised
c                called as function_value = func(parameters, intgarr, realarr)
c  D  fail       failure value of func: a value less than "fail"
c                indicates invalid arguments
c     lcntarr    control (integer) array to pass to func
c     rcntarr    control (real) array to pass to func
c     intgarr    an integer array to pass to func
c     realarr    a real array to pass to func
c     tolerance  tolerance on function value for convergence
c     ncycles    maximum number of cycles
c     gradient   if .true. use Kevin's "gradient" slant on
c                traditional method
c  R  points(nparam, nparam+1)
c                list of nparam+1 start points
c     
c On exit:
c     ncycles     number of cycles used
c  R  points(nparam, 1)  best point
c  D  value      function value at that point
c
      implicit none
      integer nparam, ncycles
      double precision func
      external func
      integer lcntarr(*), intgarr(*)
      real rcntarr(*), realarr(*)
      double precision fail, tolerance, value
      double precision points(nparam, nparam+1)
      logical gradient
cc
c  Maximum number of parameters+1
      integer maxpar
      parameter (maxpar=451)
      double precision fn(maxpar), centr(maxpar), shift(maxpar),
     $   t0(maxpar), t1(maxpar), t2(maxpar), sumw, w, r2, f1, f2

      integer i,j, np, iw, ib, icyc
      logical DEBUG, contract

      DEBUG = .false.

      np = nparam+1
      if (np .gt. maxpar) then
         write (6,'(/a,i8,i8/)')
     $      '**** ERROR: too many parameters in SIMPLEX: recompile ',
     $      np, maxpar
         call ccperr(1,'**** ERROR: too many parameters in SIMPLEX')
      endif

c Target function at each point
      do i=1,np
         fn(i) = func(points(1,i),lcntarr,rcntarr,intgarr,realarr)
      enddo 

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c Simplex loop
c
      do icyc=1,ncycles
         if (DEBUG) then
            write (6,'(/a,i4/a)')
     $         'Beginning cycle ', icyc, 'Points:'
            do j=1,np
               write (6,'(100f10.2)') (points(i,j), i=1,nparam)
               write (6,'(a,f15.5,i5)') 'Function value', fn(j), j
            enddo
         endif

c  Find best & worst point
         iw = 1
         ib = 1
         do i=1,np
            if (fn(i) .gt. fn(iw)) iw = i
            if (fn(i) .lt. fn(ib)) ib = i
         enddo
c Termination (convergence)
         if (DEBUG) then
            write (6,'(a,3f12.4)') 'Worst, best, tolerance',
     $         fn(iw), fn(ib), tolerance
         endif
         if (fn(iw) - fn(ib) .lt. tolerance) go to 100

c Find (weighted) centroid of points other than worst
         do i=1,np
            centr(i) = 0.0
         enddo
         sumw = 0.0
         do i=1,np
            if (i .ne. iw) then
c   not worst
               w = 1.0
               if (gradient) then
c      optionally bias centroid towards worst point
                  r2 = 0.0
                  do j=1,nparam
                     r2 = r2 + (points(j,i) - points(j,iw))**2
                  enddo
                  w = (fn(iw) - fn(i))/sqrt(r2)
               endif
               do j=1,nparam
                  centr(j) = centr(j) + w * points(j,i)
               enddo
               sumw = sumw + w
            endif
         enddo

         do j=1,nparam
c      centroid of points other than worst          
            centr(j) = centr(j)/sumw
c      shif from worst
            shift(j) = centr(j) - points(j,iw)
c  test points: t1 is 1st trial (normal step), t2 is extended step
c               t0 is contraction step
            t0(j) = centr(j) - 0.5 * shift(j)
            t1(j) = centr(j) + 1.0 * shift(j)
            t2(j) = centr(j) + 2.0 * shift(j)
         enddo
         contract = .false.
         if (DEBUG) then
            write (6,'(a,100f10.6)') 'Shift: ',(shift(j),j=1,nparam)
         endif
c Function value at 1st trial point
c   value .lt. fail indicates invalid arguments
         f1 = func(t1,lcntarr,rcntarr,intgarr,realarr)
c Simplex conditions
         if (f1 .gt. fail) then
            if (f1 .lt. fn(iw)) then
c   new value is better than worst
               if (f1 .lt. fn(ib)) then
c     new value is better than best
                  f2 = func(t2,lcntarr,rcntarr,intgarr,realarr)
                  if (f2 .gt. fail) then
                     if (f2 .lt. f1) then
c               extended step is best
                        call ccpmvd(points(1,iw), t2, nparam)
                        fn(iw) = f2
                     else
c                  normal step better than extended step
                        call ccpmvd(points(1,iw), t1, nparam)
                        fn(iw) = f1
                     endif
                  else
c            extended step is invalid
                     call ccpmvd(points(1,iw), t1, nparam)
                     fn(iw) = f1
                  endif
               else
c            normal step is neither best nor worst (default)
                  call ccpmvd(points(1,iw), t1, nparam)
                  fn(iw) = f1
               endif
            else
c         normal step is worse, contraction
               contract = .true.
               f2 = func(t0,lcntarr,rcntarr,intgarr,realarr)
            endif
         else
c      normal step is invalid, contraction
            contract = .true.
            f2 = func(t0,lcntarr,rcntarr,intgarr,realarr)
         endif
         if (contract) then
c We did a contraction, did it improve anything?
            if (f2 .lt. fn(iw)) then
c   Yes, so accept it
               call ccpmvd(points(1,iw), t0, nparam)
               fn(iw) = f2
            else
c   No, contract all points around the best
               do i=1,np
                  if (i .ne. ib) then
                     do j=1,nparam
                        points(j,i) = 0.5*(points(j,i) + points(j,ib))
                     enddo
                     fn(i) = func(points(1,i),
     $                  lcntarr,rcntarr,intgarr,realarr)
                  endif
               enddo
            endif
         endif
         if (DEBUG) then
            if (contract) then
               if (fn(iw) .eq. f2) then 
                  write (6,*) 'Contract all step'
               else
                  write (6,*) 'Contraction step'
               endif
            else if (fn(iw) .eq. f2) then 
               write (6,*) 'Extended step'
            else if (fn(iw) .eq. f1 ) then
               write (6,*) 'Normal step'
            endif
            write (6,'(a,2f10.3)') 'Worst, best', fn(iw), fn(ib)
         endif
      enddo
      icyc = ncycles
c Exit
 100  call ccpmvd(points(1,1), points(1,ib),nparam)
      value = fn(ib)
      ncycles = icyc
c DEBUG
c      write (6,*) 'Converged ;', fn(iw), fn(ib), tolerance 
      return
      end



      SUBROUTINE CCPMVD (ARR1,ARR2,NUM)
C    =================================
C
C  This routine assigns the first NUM elements of ARR2 to ARR1
C
C Arguments:
C ==========
C
C    ARR1 (O)   DOUBLE PRECISION ARRAY(*)
C    ARR2 (O)   DOUBLE PRECISION ARRAY(*)
C     NUM (I)   Number of words to copy
C_END_CCPMVR
C
C  Arguments
      INTEGER NUM
      DOUBLE PRECISION ARR1(*),ARR2(*)
C
      INTEGER J
C
      DO 10 J=1,NUM
   10 ARR1(J)=ARR2(J)
      END



c
c
c
      subroutine sphhmc(sth, cth, ph, lmax, lord, ylm)
c     ================================================
c
c==== Calculate spherical harmonics YLM up to order LMAX,
c==== given sin(theta), cos(theta), phi (radians).
c==== If LORD=1 returns all L orders (non-centrosymmetric).
c==== If LORD=2 returns only even L orders (centrosymmetric).
c==== Normalisation factor = 1/(4*pi).
c==== Spherical harmonics are Hermitian antisymmetric i.e.
c==== S(L,-M) = (-1)**M * CONJG( S(L,M) )
c
c
c On entry:
c  sth, cth        sin(theta), cos(theta)
c  ph              phi (radians)
c  lmax            maximum l-order
c  lord            = 1 all terms, = 2 even terms only
c
c On exit:
c  ylm             spherical harmonic coefficients Ylm
c                  number of terms = (lmax+1)*((lmax+lord)/lord)
c
c Stolen from Ian Tickle
c
      dimension ylm(*)
      real sth,cth,ph,ylm
      integer lmax,lord
c
      include 'parameter.fh'
c
      double precision fact(0:2*maxspl),plm1,plm2,temp
c
      integer k,l,m,m1
      real plm,phm
c
      logical first
      save fact,first
      data first /.true./
c
      if(first) then
        if(lmax.gt.maxspl) then
          write(6,fmt='(A)')  ' Array bound check in SPHHMC'
          call ccperr(1, ' Array bound check in SPHHMC')
        endif
        fact(0)=0.
        do 10 l=1,2*maxspl
10        fact(l)=fact(l-1)+log(dble(l))
        first=.false.
      endif
c
      k=0
      do 20 l=0,lmax,lord
        do 20 m=l,0,-1
          if(m.eq.l) then
            plm1=0.
            plm2=exp(.5*fact(2*l)-fact(l)-l*log(2.d0))*
     -        sqrt(dble(2*l+1))
          else
            m1=m+1
            temp=plm1*sth**2
            plm1=plm2
            plm2=(2.*m1*cth*plm2-sqrt(dble((l-m1)*(l+m1+1)))*temp)/
     -        sqrt(dble((l+m1)*(l-m)))
          endif
c
          if(m.gt.0) then
            plm=plm2*sth**m
            phm=m*ph
            k=k+1
            ylm(k)=plm*cos(phm)
            k=k+1
            ylm(k)=plm*sin(phm)
          else
            k=k+1
            ylm(k)=plm2
          endif
20    continue
      return
      end
      SUBROUTINE SPLINT(XA,YA,Y2A,N,X,Y)
C---------------------------------------
C Returns a cubic-spline interpolated value of function y(x)
C whose N values Y are tabulated for points X.
C The second derivatives of the interpolating function are input in Y2A.
C
      INTEGER N,K,KHI,KLO
      REAL XA(N),YA(N),Y2A(N),H,A,B,Y,X
C
      KLO=1
      KHI=N
C
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
C
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.0) CALL CCPERR(1,'SPLINT: BAD XA INPUT.')
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+
     *      ((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.
      RETURN
      END
c
c
c
      subroutine spltdname(xname, dname)
c     =================================
c
c Split dname into xname if it contains a "/" character
c
c On entry:
c  xname    crystal name (ignored): this may be overwritten
c  dname    dataset name
c
c On exit:
c  xname    crystal name (split out of dname if it is there)
c  dname    dataset name
c
      character xname*(*), dname*(*)
c
      integer j
c
      j = index(dname,'/')
      if (j .gt. 0) then
         xname = dname(1:j-1)
         dname = dname(j+1:)
      endif
      return
      end
      subroutine squeez(txtin, txtout)
c------squeeze out excess spaces from txtin to txtout
c
      implicit none
c
      character txtin*(*), txtout*(*)
c     
      integer i,l,k,m
c     
      integer lenstr
      external lenstr
c     
      l = lenstr(txtin)
      m = len(txtout)
      k=0
      txtout = ' '
c     
      do 10 i = 1,l
         if (txtin(i:i) .ne. ' ') then
            k = k+1
            if (k .gt. m) go to 20
            txtout(k:k) = txtin(i:i)
         endif
 10   continue
c      
 20   return
      end
      subroutine srfflg(nflags, lrfflg, krflag)
c     =========================================
c
c Set reflection error/warning flags
c
c
c  On entry:
c     nflags           number of flags to set or unset
c     lrfflg(nflags)   logical flags
c
c  On exit:
c     krflag           packed bit flags
c
c     don't force declarations so we can use ibset/ibclr as intrisic or
C     external
c      implicit none
c
      integer nflags, krflag
      logical lrfflg(nflags)
c
      integer i
c
c
      krflag = 0
      do 10, i = 1, nflags
         if (lrfflg(i)) then
            krflag = ibset(krflag, i-1)
         else
            krflag = ibclr(krflag, i-1)
         endif
 10   continue
      return
      end
c
c
c
      subroutine staxrf(jhkl, aih, sdih)
c     ==================================
c
c  Store axial reflections for later analysis in s/r praxrf,
c for systematic absences
c
c On entry:
c  jhkl(3)    reduced indices
c  aih        intensity I
c  sdih       sd(I)
c
      integer jhkl(3)
      real aih, sdih
c
      include 'parameter.fh'
      include 'axes.fh'
      include 'inout.fh'
c
      integer i, j, jax
      logical axis
c
      logical first
      save first
      data first /.true./
c
c
c Is this an axial reflection?
      axis = .false.
      if(jhkl(1) .eq. 0) then
         if(jhkl(2) .eq. 0) then
            axis = .true.
            jax = 3
         elseif(jhkl(3) .eq. 0) then
            axis = .true.
            jax = 2
         endif
      elseif(jhkl(2) .eq. 0) then
         if(jhkl(3) .eq. 0) then
            axis = .true.
            jax = 1
         endif
      endif
c
      if (first) then
c Initialize on first entry
         do 10, i=1,maxhkl
            do 11, j=1,3
               aihaxs(i,j) = 0.0
               sdiaxs(i,j) = 0.0
 11         continue
 10      continue
         first = .false.
      endif
c
      if (.not. axis) return
c
      i = jhkl(jax)
      if (i .gt. maxhkl) then
         write (lunout, '(/a,2i8,a/)')
     $      ' **** hkl index greater than axial limit (maxhkl):',
     $      maxhkl, i,' ****'
         call ccperr(1,'*** MAXHKL exceeded ***')
      endif
c
      aihaxs(i,jax) = aih
      sdiaxs(i,jax) = sdih
c
      return
      end
c
c
c
      subroutine strmtc(iflag, line, irun1, irun2, istat)
c     ==================================================
c
c Store matching symmetry operators for Anomalous Match
c This operation is completed in s/r setmtc
c
c On entry:
c  iflag        =  0   match operator in string line
c               = +1   SPINDLE option, set operator to +/- h,k,or l
c                      but don't do this here, since orientation isn't known
c               = +2   INVERT option, set operator to -h,-k,-l
c  line         match operator, if iflag = 0
c  irun1, irun2 1st & last run numbers to store matrix for
c
c On exit
c  istat        =  0   OK
c               = +1   illegal symmetry operator
c
c  Also sets (in common anomtc):-
c   kanmtc(irun) =  0  flagged as SPINDLE, waiting to be set
c                = +1  set to explicit symmetry operator
c                = +2  set to INVERT
c   anmmtc(3,3,irun)
c               symmetry matrix relating matching anomalous
c               measurements for explicit operator
c   rassym, rinsym  all symmetry operators
c
      integer iflag, istat, irun1, irun2
      character line*(*)
c
      include 'parameter.fh'
      include 'anomtc.fh'
      include 'rundef.fh'
c
      integer i, j, k, l, irun, ierr
      real tmpmat(4,4), tmpsym(4,4,maxsym), tmpinvsym(4,4,maxsym)
      external matmul
c
c
      istat = 0
c
      if (iflag .eq. 0) then
c Decode MATCH symmetry operator
         j = 1
         call symfr3(line,1,j,tmpmat,ierr)
         if (j .ne. 1 .or. ierr .ne. 0) then
            istat = +1
            return
         endif
c Copy transpose to store
         do 10, irun = irun1, irun2
            do 11, j = 1,3
               do 12, i = 1,3
                  anmmtc(i,j,irun) = tmpmat(j,i)
 12            continue
 11         continue
            kanmtc(irun) = +1
 10      continue
c
      elseif (iflag .eq. +1) then
c SPINDLE option: set matrix to invert along reciprocal space axis closest
c to spindle
c  Set flag to do this later, when orientation data has been read
         do 20, irun = irun1, irun2
            kanmtc(irun) = 0
 20      continue
c
      elseif (iflag .eq. +2) then
c INVERT
         do 30, irun = irun1, irun2
            call unitmt(anmmtc(1,1,irun))
            do 31, j = 1,3
               anmmtc(j,j,irun) = -1
 31         continue
            kanmtc(irun) = +2
 30      continue
      endif
c Pick up all symmetry operators and inverses, ordered as for ISYM flags
      call asusym(tmpsym, tmpinvsym, nisym)
c Copy 4,4 to 3,3 matrices
      do l=1,nisym
         do j=1,3
            do i=1,3
               rassym(i,j,l) = tmpsym(i,j,l)
               rinsym(i,j,l) = tmpinvsym(i,j,l)
            enddo
         enddo
      enddo
c Set flag for reference run, if required
      if (irefrn .gt. 0) then
         do 40, irun = irun1, irun2
            if (irun .eq. irefrn) kanmtc(irun) = -1
 40      continue
      endif
c         
      return
      end
      subroutine strogf(irogue)
c     =========================
c
c  Start ROGUES file on stream irogue
c
      implicit none
      integer irogue
c
      integer ldum, ifail
c
c
c---- Open file to list rejected reflections .
c
      ifail = 0
c
c          ************************************************
      call ccpdpn(irogue,'ROGUES','UNKNOWN','F',ldum,ifail)
c          ************************************************
c
      write (irogue,fmt=6001)
 6001 format (
     $' The ROGUES file contains ',
     $'all rejected reflections ',
     $   '(flag "*", "@" for I+- rejects, "#" for Emax rejects)',/,
     $'    TotFrc = total fraction, fulls (f) or partials (p)'/
     $'    Flag I+ or I- for Bijvoet classes'/
     $'    DelI/sd = (Ihl - Mn(I)others)/',
     $   'sqrt[sd(Ihl)**2 + sd(Mn(I))**2]'/
     +'   h   k   l     h   k   l  Batch      I  sigI    E  ',
     +   'TotFrc Flag Scale   LP   DelI/sd d(A)   Xdet   Ydet    Phi'/
     +'   (measured)     (unique)')
c
      return
      end
      subroutine strppr(iwrprb)
c     ========================
c
c  Start next plot in normal probability file
c
c *** For xmgr ***
c
      integer iwrprb
c
      write (iwrprb, '(a)') '@type xy'
      return 
      end
c
c
c
      subroutine strrsm(rcsym, rstrng)
c     ===============================
c
c Convert reciprocal space symmetry operation into character string
c
c On entry:
c  rcsym(3,3)   reciprocal-space symmetry matrix (forward or inverse)
c
c On exit:
c  rstrng       character version (eg "h,k,l")
c
c  arguments
      real rcsym(3,3)
      character*(*) rstrng
c
c locals
      integer i,j,m,lp
      character hkl(3)*1
      data   hkl/'h','k','l'/
c
      lp = 0
      rstrng = ' '
c
      do 130, j=1,3
         do 140, i=1,3
c     extract matrix element as +-1
            m = nint(rcsym(i,j))
c     add +-h,k,l to rstrng if m .ne. 0
            if (m .lt. 0) then
               rstrng(lp+1:lp+2) = '-'//hkl(i)
               lp = lp+2
            elseif (m .gt. 0) then
               rstrng(lp+1:lp+2) = '+'//hkl(i)
               lp = lp+2
            endif
 140     continue
c add ','
         lp = lp+1
         if (j .lt. 3) rstrng(lp:lp) = ','
 130  continue
      return
      end
c
c
c
      subroutine stumtc(istat)
c     =======================
c
c Set up anomalous match stuff for all runs
c Print what it is doing
c
c On exit:
c  istat      =  0  OK
c
      integer istat
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'anomtc.fh'
      include 'rundef.fh'
c
      integer i, j, k, l, irun, nrun, krun
      logical lsame, lsmspn
c
      external lenstr
      integer  lenstr
c
      character axis(3)*2, symstr*80
      data axis/'a*','b*','c*'/
c
      istat = 0
c If no anomalous, exit immediately
      if (kanomo .le. 0) then
         return
      endif
c Check same spindle axis for each run
      write (lunout, '(/)')
      lsmspn = .true.
      krun = 1
      if (numrun .gt. 0) then
         j = 0
         do 11,  irun = 1, numrun
            if (irun .ne. irefrn) then
               if (j .eq. 0) then
                  j = kspnax(irun)
                  krun = irun
               else
                  if (kspnax(irun) .ne. j) lsmspn = .false.
               endif
            endif
 11      continue
      endif
      do 12, irun = 1, numrun
         nrun = irunum(irun)
         if (irun .eq. irefrn) then
            write (lunout, '(a,i5,a)')
     $         ' Run ',nrun,', reference run'
         elseif (kspnax(irun) .eq. 0) then
            write (lunout, '(a,i5,a)')
     $         ' Run ',nrun,', no spindle axis defined'
         else
            write (lunout, '(a,i5,a,a,a,f8.1)')
     $         ' Run ',nrun,', spindle axis is closest to ',
     $         axis(kspnax(irun)),', angle ', spnang(irun)
         endif
 12   continue
c
c If no anomalous match, exit immediately
      if (kanomo .le. 1) then
         return
      elseif (kanomo .eq. +2) then
c anomalous match with no symmetry
         write (lunout, '(/a//a)')
     $      ' ANOMALOUS MATCH: use only matching anomalous pairs',
     $      ' No match check on symmetry'
         if (lanmtcrn) then
            write (lunout, '(a)') ' Matching pairs are in the same RUN'
         endif
      elseif (kanomo .ge. +3) then
c match symmetry
c Are all runs the same? (if they are, no need for separate print stuff)
         lsame = .true.
         if (numrun .gt. 1) then
            k = -1000
            do 10, irun = 1, numrun
               if (irun .ne. irefrn) then
                  if (k .eq. -1000) then
                     k = kanmtc(irun)
                  else
c   Check match matrix flag
                     if (kanmtc(irun) .ne. k) lsame = .false.
                  endif
               endif
 10         continue
c   If Spindle option, need to check spindle is the same in all runs
            if (lsame .and. k .eq. +5) then
               if (.not.lsmspn) lsame = .false.
            endif
         endif
c
         do 20, irun = 1, numrun
            if (irun .ne. irefrn) then
               call setmtc(irun, istat)
               if (istat .ne. 0) then
                  write (lunout, '(a,i4/)')
     $               ' ANOMALOUS MATCH failure, run',irunum(irun)
                  return
               endif
            endif
 20      continue
c
c Print things
         write (lunout, 6001)
 6001    format (/' ANOMALOUS MATCH: use only matching anomalous ',
     $      'pairs'//)
         if (lanmtcrn) then
            write (lunout, '(a)')
     $         ' Matching pairs must be in same RUN'
         else 
            write (lunout, '(a)')
     $         ' Matching pairs need not be in same RUN'
         endif
c
         if (kanomo .eq. +4) then
c     Invert
            write (lunout, '(a)')
     $         ' Matching anomalous pairs are related by inversion'
         elseif (kanomo .eq. +5) then
c     Spindle
            if (lsame) then
               write (lunout, '(a,a,a,a)')
     $            ' Matching anomalous pairs are related by ',
     $            'reflection perpendicular to axis closest to',
     $            'spindle axis ', axis(kspnax(1))
            else
               write (lunout, '(a,a,a)')
     $            ' Matching anomalous pairs are related by ',
     $            'reflection perpendicular to axis closest to',
     $            'spindle axis '
            endif
         else
            write (lunout, '(a)')
     $         ' Matching anomalous pairs are related by given symmetry'
         endif
c
         if (lsame) then
            irun = krun
            call strrsm(anmmtc(1,1,irun), symstr)
            write (lunout, '(/a,a)')
     $         ' Symmetry operators to relate matching I+ & I- : ',
     $         symstr(1:lenstr(symstr))
c
c Print all symmetry operator pairs
            call prtmtc(irun)
         else
            do 50, irun = 1, numrun
               nrun = irunum(irun)
               call strrsm(anmmtc(1,1,irun), symstr)
               write (lunout, '(/a,a,i5,a,a)')
     $            ' Symmetry operators to relate matching I+ & I- ',
     $            ' in run ',nrun,': ',
     $            symstr(1:lenstr(symstr))
c
c Print all symmetry operator pairs
               call prtmtc(irun)
 50         continue
         endif
c
      endif
c
      if (anmphd .ne. 0.0) then
         write (lunout, '(/a,a,f10.3,a/)')
     $      ' Matching anomalous pairs must have phi (ROT) values',
     $      ' differing by .lt. ',anmphd,' degrees'
      endif
      if (anmtmd .ne. 0.0) then
         write (lunout, '(/a,a,f10.3/)')
     $      ' Matching anomalous pairs must have TIME values',
     $      ' differing by .lt. ',anmtmd
      endif
c
      end            
      subroutine stwrpr(iwrprb, pltfil, legend, title1, title2)
c     =========================================================
c
c Open plotfile & write header for graph 
c
c On entry:
c  iwrprb     stream number for writing
c  pltfil     file name or logical name
c  legend     .true. to draw legend
c  title1, title2  titles
c  
c
c *** This version writes file for xmgr ***
c *** Updated for Grace May-2002
c
      integer iwrprb
      logical legend
      character pltfil*(*), title1*(*), title2*(*)
c
      integer l
c
      integer  lenstr
      external lenstr
c
c
      call ccpdpn(iwrprb,pltfil,'NEW','F',0,0)
c
      write (iwrprb, '(a)') '@version 40102'
      l = lenstr(title1)
      if (l .le. 0) then
         write (iwrprb, '(a)') '@ title "Normal probability plot"'
      else 
         write (iwrprb, '(a,a,a)')
     $      '@ title "',title1(1:l),'"'
      endif 
      l = lenstr(title2)
      if (l .gt. 0) write (iwrprb, '(a,a,a)')
     $   '@ subtitle "',title2(1:l),'"'
c
      write (iwrprb, 6001) 
 6001 format (
     $   '@with G0'/
     $   '@  world xmin -4'/
     $   '@  world xmax 4'/
     $   '@  world ymin -5'/
     $   '@  world ymax 5'/
     $   '@  view xmin 0.15'/
     $   '@  view xmax 0.71'/
     $   '@  view ymin 0.15'/
     $   '@  view ymax 0.85'/
     $   '@  xaxis  tick on'/
     $   '@  xaxis  tick major 1'/
     $   '@  xaxis  tick minor 1'/
     $   '@  xaxis  label "\\8d\\0(expected)"'/
     $   '@  xaxis  tick major grid on'/
     $   '@  yaxis  tick on'/
     $   '@  yaxis  tick major 1'/
     $   '@  yaxis  tick minor 1'/
     $   '@  yaxis  label "\\8d\\0(observed)"'/
     $   '@  yaxis  tick major grid on'/
     $   '@  legend loctype world'/
     $   '@  legend x1 0.784929'/
     $   '@  legend y1 -2.71829'/
     $   '@  legend color 1'/
     $   '@  legend vgap 2'/
     $   '@  legend hgap 1'/
     $   '@  legend length 1'/
     $   '@  legend x1 0.8'/
     $   '@  legend y1 0.0'/
     $   '@  legend font 4'/
     $   '@  legend char size 0.8')
      if (legend) then
         write (iwrprb, 6002) 
 6002    format (
     $      '@  legend on')
      else
         write (iwrprb, 6003) 
 6003    format (
     $      '@  legend off')
      endif
      write (iwrprb, 6010) 
 6010 format (
     $   '@G0 on')
c
      return
      end
c
c
c
      subroutine stwruc(jdts, xymax, title1, title2)
c    ========================================
c
c  Open output file for scatter plot of anomalous correlations
c
c  jdts           dataset number, if = -1 just one file
c  xymax          maximum limit for plot
c  title1, title2  titles
c
c *** For xmgr ***
c
      integer jdts
      real xymax
      character title1*(*), title2*(*)
c
      include 'parameter.fh'
      include 'crlncf.fh'
c
      real xmin, xmax, ymin, ymax
      integer j, k, l
      character*200 fname
      integer lenstr
      external lenstr
c
      j = jdts
      if (j .le. 0) j = 1
      k = 1
      if (jdts .ge. 0) k = 2
      call mkfilname('CORRELPLOT', j, k, .false., fname)
      iwrucf = 10
      call ccpdpn(iwrucf, fname, 'NEW', 'F', 0,0)
c
c
      write (iwrucf, '(a)') '@version 40102'
      l = lenstr(title1)
      if (l .le. 0) then
         write (iwrucf, '(a)') '@ title "DelAnom scatter plot"'
      else 
         write (iwrucf, '(a,a,a)')
     $      '@ title "',title1(1:l),'"'
      endif 
      l = lenstr(title2)
      if (l .gt. 0) write (iwrucf, '(a,a,a)')
     $   '@ subtitle "',title2(1:l),'"'
c
      xmin = -xymax
      xmax =  xymax
      ymin = -xymax
      ymax =  xymax

      write (iwrucf, 6001) xmin, xmax, ymin, ymax
 6001 format (
     $ '@with g0'/
     $ '@    world xmin ', f7.0/
     $ '@    world xmax ', f7.0/
     $ '@    world ymin ', f7.0/
     $ '@    world ymax ', f7.0/
     $ '@    view xmin 0.150000'/
     $ '@    view xmax 0.850000'/
     $ '@    view ymin 0.150000'/
     $ '@    view ymax 0.850000'/
     $ '@    s0 type xy'/
     $ '@    s0 symbol 2'/
     $ '@    s0 symbol size 0.20000'/
     $ '@    s0 symbol fill 1'/
     $ '@    s0 symbol color 1'/
     $ '@    s0 linestyle 0'/
     $ '@    xaxis  tick on'/
     $ '@    xaxis  tick major 2'/
     $ '@    xaxis  tick minor 1'/
     $ '@    xaxis  tick major grid on'/
     $ '@    yaxis  tick on'/
     $ '@    yaxis  tick major 2'/
     $ '@    yaxis  tick minor 1'/
     $ '@    yaxis  tick major grid on'/
     $  '@G0 on')
c
      return
      end
      subroutine sumprt(jhkl, nrs, ffrf, sdffrf, ffprrf, sdfprf, ghl,
     $   isttrf, mrf, mprf, isymrf, ibatrf, irunrf, 
     $   xdrf, ydrf, zdrf, phirf, frclrf,  kmptrf, nmprrf, jmprrf,
     $   jmidsl, kpropt, kprmis, prmin, prlim, prmax, maxpwd, 
     $   imid, ipower, lsclpr, nrejpt, nuse, dghldc, lavpsp, lavpsc)
c     ============================================================
c
c Sum partials
c
c  On entry:
c     jhkl(3)      unique hkl
c     nrs          number of observations
c     ffrf(nrs)    intensity
c     sdffrf(nrs)  sd(intensity)
c     ffprrf(nrs)  profile-fitted intensity
c     sdfprf(nrs)  sd(profile-fitted intensity)
c     ghl(nrs)     scales
c     isttrf(nrs)  status flags
c     mrf(nrs)     partiality flags
c     mprf(nrs)    partiality flags: mpart
c     isymrf(nrs)  symmetry numbers
c     ibatrf(nrs)  batch numbers
c     irunrf(nrs)  run numbers
c     xdrf,ydrf,zdrf(nrs) coordinates
c     frclrf(nrs)  calculated fractions
c     nmprrf(nrs)  number of parts summed into this slot, initially = 1 for all
c     jmprrf(nrs)  status of this slot
c              = -2 for otherwise rejected, initially set to this
c     jmidsl(nrs)  initially set = slot number, reset for partials
c     kpropt    controls mode of partial summation
c               .lt. 0  check if all MPART flags valid (only 
c                      allowed if MPART flags present)
c                      If valid, accept, if not valid, pass to total
c                      fraction test if = -2, reject if = -1
c               = +/-1 no test on total fraction
c               = +/-2 accept summed partial if total fraction over
c                      all parts lies between  prlim -> prmax,
c                      scale by total fraction if fraction between
c                      prmin -> prlim
c     kprmis    =  0  reject partials with gap
c               = +1  accept partials with gap
c     prmin     threshold for partial scaling                       
c     prlim     lower limit of total partial fraction for acceptance
c     prmax     upper limit of total partial fraction for acceptance
c     maxpwd    maximum number of parts to an acceptable partial
c               (a summed partial must also have at least 2 parts)
c     imid      flag for selection of profile-fitted|summation intensity
c               if imid = 0, then ffprrf is ignored
c     ipower       power for combine intensities option (imid .gt. 0)
c     lsclpr    if .false. always use integrated I for scaled partials
c                  .true. allow profile-fitted I to be used for
c                      scaled partials
cc     nrejpt(6)   current rejection numbers
c     dghldc(maxpmr,nrs) partial derivatives d(ghl)/d(param), for
c               nprrun(irun) parameters for each observation
c     lavpsp    if .true., modify dghldc if lavpsc(irun) also .true. 
c               else don't
c     lavpsc(numrun) ghl modify flag for each run
c
c  On exit:
c     nuse     number of reflections remaining,
c                number of fulls + number of accepted summed partials
c     nrejpt(6)  number rejected: 
c              (1) rejected reflections with GAP (GAP not set)
c              (2) too wide (.gt. maxpwd parts)
c              (3) total fraction too small
c              (4) total fraction too large
c              (5) inconsistent MPART flags
c              (6) some parts flagged as rejected
c     ffrf     summed (or scaled) partials where appropriate
c              profile-fitted intensity will be selected if required
c              (if imid = -1 or -2)
c     sdffrf   sds
c     zdrf     averaged normalized rotation angle for partials
c     phirf    averaged rotation angle for partials
c     isttrf   modified status flag
c     frclrf(nrs)  calculated fractions, updated to total fraction for summed
c              partial
c     kmptrf(nrs) for first part of summed partial
c              = +1  partial with consistent MPART flags
c              = -1  partial with inconsistent MPART flags
c              =  0  other: eg full, other parts of partials,
c                    no MPART flags etc
c     nmprrf(nrs) number of parts summed into this slot
c              = 1 for full (as initial value)
c              = 0 unmatched partial or 2nd & subsequent parts of partial
c                    (ie those flagged as lquprt)
c              .gt. 1 for summed partial (1st slot)
c              negated for scaled partial, = - number of parts
c     jmprrf(nrs)  status of this slot
c              =  0  principal slot (full, or summed and/or scaled partial)
c              .gt. 0 for 2nd & subsequent parts of partial (scaled or not)
c                    (ie those flagged as lquprt)
c                   set = slot number of summed partial
c              = -1 for unmatched partial
c              = -2 for otherwise rejected
c     jmidsl(nrs)   for 1st slot of partial, rest to "central" slot number
c                 else = slot number (for fulls & unmatched or other bits)
c*cc     ghl(nrs) for summed partials when lavpsc(irun) .true. (BATCH scaling),
c*cc              ghl is updated to Sum(parts) [fractioncalc * ghl]
c     dghldc(maxpmr,nrs) for summed partials when lavpsc(irun) .true.
c              sghldc is updated to Sum(parts) [fractioncalc * dghldc]
c              for all parameters

c
c  The accepted summed (scaled) partial is put into the 1st slot, &
c     the others marked as lquprt (these will always be contiguous).
c     Unmatched partials are flagged as lqnprt
c
      integer MXNERR
      parameter (MXNERR = 50)
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
      include 'scales.fh'
      include 'params.fh'
      include 'errors.fh'
c
      integer jhkl(3), nrs, nuse, kpropt, kprmis, nrejpt(6),
     $   imid, ipower, maxpwd
      real    prmin, prlim, prmax
      real ffrf(nrs), sdffrf(nrs), ffprrf(nrs), sdfprf(nrs), ghl(nrs),
     $     xdrf(nrs), ydrf(nrs), zdrf(nrs), phirf(nrs), frclrf(nrs),
     $     dghldc(maxpmr,nrs), ff, ffmax, frcmax
      integer isttrf(nrs), mrf(nrs), mprf(nrs), isymrf(nrs),
     $   ibatrf(nrs), irunrf(nrs), nmprrf(nrs), jmprrf(nrs), 
     $   jmidsl(nrs), jffmax, jfrmax, kmptrf(nrs)
      logical lavpsp, lavpsc(maxrun), lsclpr
c
c
      integer i, j, l, nerr, nfound, numprt, last, mpart, nsused,
     $   istat
      real  sumff, sumsf, sumzd, sumphi, totfrc, sumghl, sumdgd(maxpmr)
      logical lfail, lscale, lavppp, ltestf
c
      save nerr
c
      data nerr /0/
c
      do l = 1, nrs
         kmptrf(l) = 0
      enddo
      l = 1
c
c  do while l .le. nrs 
 10   if (l .le. nrs) then
         if (isttrf(l) .eq. lqprt) then
c  l'th reflection is partial
c Count & check partials which match this one
            call chkprt(nrs, l, isymrf, ibatrf, isttrf, mprf,
     $         frclrf, irunrf, nfound, numprt, totfrc, istat)
            lfail = .false.
c nfound is number of parts found: this all works if nfound = 1
            if (nfound .gt. 0) then
               lscale = .false.
               ltestf = .true.
c A group of partials has been found
               if (nfound .gt. maxpwd) then
c  Too wide
                  nrejpt(2) = nrejpt(2) + 1
                  lfail = .true.
               elseif (istat .eq. -4) then
c     Some parts flagged as reject (not lqprt, probably lqeflg)
                  nrejpt(6) = nrejpt(6) + 1
c keep them                  lfail = .true.
               elseif (istat .eq. -3 .and. kprmis .eq. 0) then
c     Partial with gap, & GAP not set
                  nrejpt(1) = nrejpt(1) + 1
                  lfail = .true.
               elseif (kpropt .lt. 0) then
c CHECK option set
                  if (istat .gt. 0) then
c     MPART flags do match properly, accept without test on total fraction
                     ltestf = .false.
                     kmptrf(l) = +1
                  else
c     MPART flags do not match properly, fraction test if kpropt = +-2,
c           else reject (if kpropt = -1)
                     if (kpropt .eq. -2) then
                        ltestf = .true.
                     else
                        lfail = .true.
                        nrejpt(5) = nrejpt(5) + 1
                     endif
                     kmptrf(l) = -1
                  endif
               elseif (kpropt .eq. +1) then
c CHECK option not set, test fraction if kpropt = +2, not if = +1
                  ltestf = .false.
                  if (istat .gt. 0) then
c     MPART flags do match properly
                     kmptrf(l) = +1
                  else
c     MPART flags do not match properly
                     kmptrf(l) = -1
                  endif
               endif
c  Test total fraction
               if (.not. lfail .and. ltestf) then
                  if (iabs(kpropt) .eq. 2) then 
c  TEST test total fraction
c       reject if total fraction outside range prmin -> prmax
c          else accept or scale by total fraction
                     if (totfrc .lt. prmin) then 
                        nrejpt(3) = nrejpt(3) + 1
                        lfail = .true.
                     elseif (totfrc .gt. prmax) then
                        nrejpt(4) = nrejpt(4) + 1
                        lfail = .true.
                     endif
                     if (totfrc .lt. prlim) then
                        lscale = .true. 
                     endif
                  elseif (iabs(kpropt) .eq. 1) then
c  No test on total fraction, don't allow single part partials
                     if (nfound .eq. 1) lfail = .true.
                  endif
               endif
               if (.not.lfail) then
c Accept, so add bits etc
                  sumff = 0.0
                  sumsf = 0.0
                  sumzd = 0.0
                  sumphi = 0.0
c Only sum partial scales & derivatives if global flag lavpsp and run flag
c lavpsc are both set .true. 
                  lavppp = lavpsp .and. lavpsc(irunrf(l))
                  if (lavppp) then
                     sumghl = 0.0
                     do 18, j = 1, nprrun(irunrf(l))
                        sumdgd(j) = 0.0
 18                  continue
                  endif
                  ffmax  = -1.0e10
                  frcmax = -0.1
                  jfrmax = -1
                  nsused = 0
c Loop parts of this observation
                  do 20, i = l, l+nfound-1
                     if (isttrf(i) .eq. lqprt) then
c  If this reflection is to be scaled, use summation integration I
c  otherwise, use profile-fitted intensity if imid .eq. -1
                        if ((.not. lscale .or. lsclpr)
     $                     .and. imid .ne. 0) then
                           call selint(
     $                        ffrf(i), sdffrf(i), ffprrf(i), sdfprf(i),
     $                        mrf(i), imid, ipower)
                        endif
                        ff = (ghl(l)/ghl(i))*ffrf(i)
                        sumff = sumff + ff
                        sumsf = sumsf + ((ghl(l)/ghl(i))*sdffrf(i))**2
                        if (ff .gt. ffmax) then
                           ffmax = ff
                           jffmax = i
                        endif
                        if (frclrf(i) .gt. frcmax) then
                           frcmax = frclrf(i)
                           jfrmax = i
                        endif
c  Check same position on detector
                        if (i .gt. l) then
                           if (abs(xdrf(i) - xdrf(i-1)) .gt.
     $                        dtolrn(1,irunrf(l))
     $                        .or.   abs(ydrf(i) - ydrf(i-1)) .gt.
     $                        dtolrn(2,irunrf(l))) then
c    Not same position
                              nerr = nerr + 1
c      Store batch numbers 
                              if (nerpdc.lt.(maxpdc-2)) then
                                 nerpdc = nerpdc + 1
                                 ibtpdc(nerpdc)=ibatrf(i-1)
                                 nerpdc = nerpdc + 1
                                 ibtpdc(nerpdc)=ibatrf(i)
                              endif
                              if (lerpdc) then
                                 if (nerr .lt. MXNERR) then
                                    write (lunout,
     +                                 '(a,a,a,i5,2f8.1,a,a,i5,2f8.1)')
     +                  ' --- WARNING ---: partials with different ',
     +                  'detector coordinates: ',' batch = ',
     +                  ibatrf(i-1),xdrf(i-1),ydrf(i-1),' | ',
     +                  ' batch = ',ibatrf(i),xdrf(i),ydrf(i)
                                 elseif (nerr .eq. MXNERR) then
                                    write (lunout, '(a,a/)')
     $ ' --- WARNING ---: partials with different detector coordinates',
     $         ': more messages suppressed . . .'
                                 endif
                              endif
                           endif
                        endif
c  Average rotation angle: don't average normalized rotation in batch mode
                        sumphi = sumphi + phirf(i)
                        if (mbatsc(irunrf(i)) .eq. 0)
     $                     sumzd = sumzd + zdrf(i)
c  Change flags, all but first becomes lquprt
                        isttrf(i) = lquprt
                        jmprrf(i) = +l
                        nmprrf(i) = 0
                        nsused = nsused+1
                        if (lavppp) then
c  Fraction-weighted sum of scales ghl & partial derivatives
c*c                        sumghl = sumghl + frclrf(i) * ghl(i)
                           do 21, j = 1, nprrun(irunrf(l))
                              sumdgd(j) = sumdgd(j) +
     $                           frclrf(i) * dghldc(j,i)
 21                        continue
                        endif
                     endif
 20               continue
c Number of parts
                  nmprrf(l) = nsused
                  jmprrf(l) = 0
c Put sums etc back into first slot
                  ffrf(l) = sumff
                  sdffrf(l) = sqrt(sumsf)
c Scale by total fraction if required
                  if (lscale) then
                     ffrf(l)   = ffrf(l)/totfrc
                     sdffrf(l) = sdffrf(l)/totfrc
c Flag for scaled partial
                     nmprrf(l) = -nmprrf(l)
                  endif
c Store total fraction in 1st slot
                  frclrf(l) = totfrc
                  if (lavppp) then
c  Fraction-weighted sums of scale ghl & partial derivatives
c*c                     ghl(l) = sumghl
                     do 25, j = 1, nprrun(irunrf(l))
                        dghldc(j,l) = sumdgd(j)
 25                  continue
                  endif
c Store slot numberfor largest fraction or largest intensity
                  if (totfrc .gt. 0.0001) then
                     jmidsl(l) = jfrmax
                  else
                     jmidsl(l) = jffmax
                  endif                     
c Mean Phi value
                  phirf(l) = sumphi/float(nfound)
                  if (mbatsc(irunrf(l)) .eq. 0)
     $               zdrf(l) = sumzd/float(nfound)
c
c Flag l'th slot as OK (summed partial)
                  isttrf(l) = lqOK
c
               endif
            else
               call ccperr(1,'*** SUMPRT: nfound .le. 0 ***')
            endif
            if (lfail) then
c For failures, mark 1st slot as unmatched & store total fraction
               isttrf(l) = lqnprt
               nmprrf(l) = 0
               jmprrf(l) = -1
               frclrf(l) = totfrc
            endif
            l = l+nfound
         else
c  Full
            l = l+1
         endif
         go to 10
      endif
c
c Check through list for any unmatched partials:
c       reset flag to non-first part of  partial
      nuse = 0
      do 50, l = 1,nrs
         if (isttrf(l) .eq. lqprt) then
            isttrf(l) = lquprt
            nmprrf(l) = 0
            jmprrf(l) = -1
         endif
c Count useful observations
         if (isttrf(l) .ge. lqOK) nuse = nuse+1
 50   continue
c
      return
      end
      subroutine tformat(t, ct, n, m)
c    ===========================
c
c  Return ct formatted string version of t, format fn.m
c  ct = '  -  '  if t == 0.0, centred in string
c
      real t
      character*(*) ct
      integer n,m

      character cfmt*80, f1*4, f2*4, f*4
      integer j, lct

      if (t .ne. 0.0) then
         write (f,'(i4)') n
         call squeez(f,f1)
         write (f,'(i4)') m
         call squeez(f,f2)
         cfmt = '(f'//f1//'.'//f2//')'
         write (ct, cfmt) t
      else
         ct = ''
         j = n-m
         ct(j:j) = '-'
      endif
      return
      end
c
c
c
      subroutine iformat(i, ct, n)
c    ========================
c
c  Return ct formatted string version of i, format In
c  ct = '  -  '  if i == 0, right-justified in string
c
      integer i
      character*(*) ct
      integer n

      character cfmt*80, f1*16, f*4
      integer j, lct

      if (i .ne. 0) then
         write (f,'(i4)') n
         f1 = '(i'//f//')'
         call squeez(f1,cfmt)
         write (ct, cfmt) i
      else
         ct = ''
         j = n
         ct(j:j) = '-'
      endif
      return
      end
c
c
c
      subroutine findlastrange(ncount, nrange, lrange)
c    ==========================================
c
c Find any trailing empty resolution ranges, search from end
c
c On entry:
c  ncount(nrange)   count of contributions to range
c
c On exit:
c  lrange   last range with ncount(lrange) > 0 (lrange .le. nrange)
c
      integer nrange, lrange
      integer ncount(nrange)
c      
      integer k

      lrange = nrange
      do k = nrange, 1, -1
         if (ncount(k) .gt. 0) then
            go to 10
         endif
c Empty resolution range, store
         lrange = k-1
 10      continue
      enddo
      return
      end
      subroutine timcdn(time, irun, timen)
c     ===================================
c
c Determine normalized "time" coordinate, using scale layout from
c     common /scales/
c
c On entry:
c     time     "time" value from file (time or rotation)
c     irun     run number
c
c On exit:
c     timen    normalized time coordinate, ie = 0.0 at beginning,
c              = 1, 2, 3 etc at positions of B-factors
c
      implicit none
c
      real time, timen
      integer irun
c
      include 'parameter.fh'
      include 'scales.fh'
c
      timen = (time - tlimrn(1,irun)) / spcbfc(irun)
c
      return
      end
c
c
c
      subroutine timcor
     $   (nrs, ffrf, sdffrf, ghl, wtrf, useflg, timerf, irunrf)
c    =======================================================
c
c
c  Extrapolate back to zero time, if more than one observation
c

c  On entry:
c     nrs       number of observations
c     ffrf      intensities
c     sdffrf    corrected sd(Ihl)
c     ghl       dividing scales
c     wtrf      weights eg = 1 /  variance
c     useflg    .true. to use this observation
c     timerf    normalised time
c     irunrf    run
c
c  On exit:
c     ffrf, sdffrf  corrected
c
      implicit none
c
      include 'parameter.fh'
      include 'timecor.fh'
      include 'scales.fh'
c
      integer nrs, nused, irunrf(nrs)
      real ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs), timerf(nrs)
      logical useflg(nrs)
c
      integer l, n, i1
c
      double precision resid, sum_w, sum_wx, sum_wy, sum_wx2, sum_wxy
      real  x, y, w, det, frac_cor, t1, y1, sy1, sl_max, sl_min,
     $   delt, s, sl0
c
      real slope, vzero, sig_slope, sig_vzero, fac, damp,
     $   c, d, corfac, ym, sig_ym, time, time_r, xr, yr,
     $   deltmin, sig_min, corfac_lim, ymin, ymax, xmin, xmax, tmax

      real pl(5)
      logical debug
c      debug = .true.
      debug = .false.
c
      n = 0
      do l = 1, nrs
         if (useflg(l)) n = n+1
      enddo
      if (n .lt. 2) return
c
c Limits
c  fractional time range 10% of whole width (5% of half range)
      deltmin = 0.05
c  maximum correction factor 1/2
      corfac_lim = 0.5
c
c Extrapolate to mid dose not to start
      time_r = 0.5*(tlimrn(2,irunrf(1))-tlimrn(1,irunrf(1)))/
     $   spcbfc(irunrf(1))
c
      resid = 0.0
      sum_w = 0.0
      sum_wx = 0.0
      sum_wy = 0.0
      sum_wx2 = 0.0
      sum_wxy = 0.0
      xmin = 1.0d10
      xmax = -xmin
      ymin = 1.0d10
      ymax = -ymin
c
      i1 = 0
      do l = 1, nrs
         if (useflg(l)) then
            x = timerf(l) - time_r
            y = ffrf(l)/ghl(l)
            w = wtrf(l)/ghl(l)**2
c
            sum_w = sum_w + w
            sum_wx = sum_wx + w*x
            sum_wy = sum_wy + w*y
            sum_wx2 = sum_wx2 + w*x*x
            sum_wxy = sum_wxy + w*x*y
            xmax = max(xmax, x)
            xmin = min(xmin, x)
            ymax = max(ymax, y)
            ymin = min(ymin, y)
c Special for n = 2
c  we need the two points in order of time
            if (n .eq. 2) then
               if (i1 .eq. 0) then
c  First point, store index, time and y values
                  i1 = l
                  t1 = x
                  y1 = y
                  sy1 = sdffrf(l)/ghl(l)
               else
c  Second point - calculate minimum and maximum slopes
                  if (t1 .lt. x) then
c    points are in time order
                     sl_max = (y + sdffrf(l)/ghl(l)) - (y1-sy1)
                     sl_min = (y - sdffrf(l)/ghl(l)) - (y1+sy1)
                  else
c    points are in reverse time order
                     sl_min = -(y + sdffrf(l)/ghl(l)) + (y1-sy1)
                     sl_max = -(y - sdffrf(l)/ghl(l)) + (y1+sy1)
                  endif
                  delt = abs(t1 - x)
                  if (delt .gt. deltmin * time_r) then
                     sl_min = sl_min/delt
                     sl_max = sl_max/delt
                  else
                     sl_min = 0.0
                     sl_max = 0.0
                  endif
               endif
            endif
         endif
      enddo
c
      slope = (sum_wxy - (sum_wx*sum_wy)/sum_w)/
     $   (sum_wx2 - sum_wx**2/sum_w)
      vzero = sum_wy/sum_w - slope * sum_wx/sum_w
      sl0 = slope
c
      ym = sum_wy/sum_w
      sig_ym = sqrt(1./sum_w)
      d = 0.0
      do l = 1, nrs
         if (useflg(l)) then
            x = timerf(l) - time_r
            y = ffrf(l)/ghl(l)
            w = wtrf(l)/ghl(l)**2
            resid = resid + w * (y - slope*x - vzero)**2
            d = d + (y-ym)**2
         endif
      enddo
c
c Ranges: time range xr is fraction of half total range
      xr = (xmax-xmin)/time_r
      tmax = max(abs(xmax), abs(xmin))
      yr = ymax-ymin
      sig_min = 0.0
c
      if (xr .lt. deltmin) then
c  Small time range, no correction
         slope = 0.0
         sig_slope = ym
         numunc = numunc + 1
      else
         if (n .eq. 2) then
c  Estimate sig(slope) for two points from max & min values
            sig_slope = sl_max - sl_min
            sig_vzero = 0.0
         else
            fac = resid/(dfloat(n-2))
            sig_slope = sqrt(fac * (sum_w/(sum_wx2*sum_w - sum_wx**2)))
            sig_vzero =
     $         sqrt(fac * (sum_wx2/(sum_wx2*sum_w - sum_wx**2)))
         endif
      endif
c
c
c Double sigma for one-sided data (all time points same side of
c reference time point)
      if (xmax*xmin .gt. 0.0) then
         sig_slope = 2.0*sig_slope
      endif
c
      if (ktimcor .eq. +2) then
         if (abs(slope)*time_r .gt. 0.01) then
c Damp shifts option
            damp = 1.0/(1.0 + sig_slope/abs(slope))
         else
            damp = 1.0
         endif
      else
         damp = 1.0
      endif
c Truncate large slopes
      if (abs(damp*slope) .gt. abs(corfac_lim * ym / tmax)) then
         s = sign(abs(corfac_lim * ym / tmax), slope)
         sig_slope = sig_slope*(slope/s)
         slope = s/damp
         nummxc = nummxc + 1
      endif
      corfac = damp * slope
      numcor = numcor+1
c
c      write (6,*) 'ym, sig_ym ', ym, sig_ym
c      write (6,*) 'slope, sig_slope ', slope, sig_slope
c      write (6,*) 'n, xr, yr, sig_min', n, xr, yr, sig_min
c
      if (debug) then
         write (21, 6002)
     $      slope, sig_slope, vzero, sig_vzero, time_r, corfac
 6002    format ('## Slope sig', f10.2, f10.2,
     $      ' Zero sig ', f10.2, f10.2,'  Time_R ',f10.1,
     $      ' Corfac: ',f10.2)
         write (21, '(a)') '@type xy'
      endif
c
      do l = 1, nrs
         if (useflg(l)) then
            if (debug) then
               pl(1) = timerf(l) - time_r
               pl(2) = ffrf(l)/ghl(l)
               pl(3) = (sl0 * pl(1) + vzero)
            endif 
            time = timerf(l) - time_r
            c =  corfac * time
            y = ffrf(l)/ghl(l)
            ffrf(l) = ffrf(l) - ghl(l) * c
            sdffrf(l) = sqrt(sdffrf(l)**2 +
     $         (ghl(l) * c)**2)
c
            sum_corfac = sum_corfac + c/ym
            corfac_max = max(corfac_max, abs(c/ym))
            frac_cor = c/vzero
            sum_tcr = sum_tcr + frac_cor
            count_tcr = count_tcr + 1
c
            timmax = max(timmax, timerf(l)-time_r)
            timmin = min(timmin, timerf(l)-time_r)
            if (debug) then
               pl(4) = ffrf(l)/ghl(l)
               write (21, '(5f12.2)') pl
            endif 
        endif
      enddo 
      if (debug) write(21, '(a)') '&'
c
      return
      end
c
c
c
      subroutine clr_tcr
c    ================
c
c  Clear time correction analyses
c
      include 'timecor.fh'
c
      logical first
      data first/.true./
c
      if (first) then
         corfac_avg = 0.0
         have_avg = .false.
      endif
      sum_tcr = 0.0
      sum_corfac = 0.0
      corfac_max = 0.0
      timmax = 0.0
      timmin = 1.0e10
      count_tcr = 0.0
      numunc = 0
      nummxc = 0
      numcor = 0
      return
      end
c
c
c
      subroutine prt_tcr
c    ================
c
c
      include 'timecor.fh'
      include 'inout.fh'
c
      if (count_tcr .gt. 0.1) then
         corfac_avg = sum_corfac/count_tcr
         corfac_guide = corfac_avg * 2.
         have_avg = .true.
         write (lunout, 6001) sum_tcr/count_tcr, corfac_avg, corfac_max,
     $      timmin, timmax, numunc, nummxc, numcor
 6001    format (//'=== Zero-dose extrapolation',//,
     $      '  Average fractional correction          ',f12.3/
     $      '  Average correction                     ',f12.3/
     $      '  Maximum correction                     ',f12.3/
     $      '  Minimum, maximum,  "time" (normalised) ',2f12.1/
     $      '  Number uncorrected (Del_t too small)   ',i10/
     $      '  Number where slope reset to maximum    ',i10/
     $      '  Total number considered                ',i10/)
      endif
      return
      end
c
c
c
      real function trnprn(value, vmin, vmax)
c    ==================================
c
c  Return vlue truncated between vmin & vmax
      real value, vmin, vmax
c
      if (value .lt. vmin) then
         trnprn = vmin
      elseif (value .gt. vmax) then
         trnprn = vmax
      else
         trnprn = value
      endif
      return
      end

c
c
      subroutine tstmfl(nrs, lflgrf, bgpkrf, isttrf, mrf,
     $   mflcnt, flmmax)
c    =============================================
c
c Extract Mosflm Flag columns, count cases,
c reset status for acceptances if needed
c
c On entry:
c  
c  nrs           number of observations
c  lflgrf(nrs)   flag column from file (if present, else = 0)
c  bgpkrf(nrs)   bg/pk ratios from file (if present, else = 0)
c  isttrf(nrs)   status flag  (see flags.fh)
c                 = 0   conditionally OK
c                 .gt.0 unconditionally OK
c                 .lt.0 rejected
c                 = lqeflg if flag .ne. 0
c  
c  mflcnt(2,maxmfl)  totals to be incremented
c  flmmax(3,2)     current maximum & maximum accepted values of 
c                bgratio, pkratio, gradient
c
c On exit:
c  isttrf(nrs)   status flag, possibly reset
c  mflcnt(2,maxmfl) incremented totals
c                mflcnt(j,k) k = 1:maxmfl flagged categories
c                            j = 1 number in file
c                            j = 2 number accepted
c  flmmax(3,2)     current maximum & maximum accepted values of
c                bgratio, pkratio, gradient


      include 'parameter.fh'
      include 'inout.fh'
      include 'outcon.fh'
      include 'column.fh'
      include 'flags.fh'
c
      integer nrs
      integer lflgrf(nrs), isttrf(nrs), mrf(nrs), mflcnt(2,maxmfl)
      real bgpkrf(nrs), flmmax(3,2)
c
c
      integer i, l, istat, nfail
      logical  lmbgratio, lmpkratio, lmnegsd, lmgrad, lmovld, lmedge
      real bgratio, pkratio, gradient
c!!!
c!      integer istatf(maxrfl)
c!      logical changed
c!      do i=1,nrs
c!         istatf(i) = isttrf(i)
c!      enddo 
c!!!-
c
      do l = 1, nrs
         if (isttrf(l) .eq. lqeflg) then
c Mosflm set, unpack it, count & test
            call unpmfl(lflgrf(l), bgpkrf(l),
     $         lmbgratio, lmpkratio, lmnegsd, lmgrad, lmovld, lmedge,
     $         bgratio, pkratio, gradient, istat)
            flmmax(1,1) = max(flmmax(1,1), bgratio)
            flmmax(2,1) = max(flmmax(2,1), pkratio)
            flmmax(3,1) = max(flmmax(3,1), gradient)
            if (istat .lt. 0) then
c   Shouldn't happen!
               write (lunout, '(a)')
     $            ' **** WARNING: observation has illegal FLAG ****'
            elseif (istat .gt. 0) then
               nfail = 0
c Counts
               if(lmbgratio) then
                  mflcnt(1,1) = mflcnt(1,1)+1
                  nfail = nfail+1
               endif
               if(lmpkratio) then
                  mflcnt(1,2) = mflcnt(1,2)+1
                  nfail = nfail+1
               endif
               if(lmnegsd) then
                  mflcnt(1,3) = mflcnt(1,3)+1
                  nfail = nfail+1
               endif
               if(lmgrad) then
                  mflcnt(1,4) = mflcnt(1,4)+1
                  nfail = nfail+1
               endif
               if(lmovld) then
                  mflcnt(1,5) = mflcnt(1,5)+1
                  nfail = nfail+1
               endif
               if(lmedge) then
                  mflcnt(1,6) = mflcnt(1,6)+1
                  nfail = nfail+1
               endif
c Acceptance tests only if needed
               if (nfail .gt. 0 .and. ltsmfl) then
c BGratio                  
                  if(lmbgratio .and. bgrlim .ge. 0.0 .and.
     $               bgratio .lt. bgrlim) then
c    accept
                     nfail = nfail-1
                     mflcnt(2,1) = mflcnt(2,1)+1
                     lmbgratio = .false.
                  endif
                  if(lmpkratio .and. pkrlim .ge. 0.0 .and.
     $               pkratio .lt. pkrlim) then
                     nfail = nfail-1
                     mflcnt(2,2) = mflcnt(2,2)+1
                     lmpkratio = .false.
                  endif
                  if(lmgrad .and.  grdlim .ge. 0.0 .and.
     $               gradient .lt. grdlim) then
                     nfail = nfail-1
                     mflcnt(2,4) = mflcnt(2,4)+1
                     lmgrad = .false.
                  endif
                  if(lmovld .and. lovld) then
                     nfail = nfail-1
                     mflcnt(2,5) = mflcnt(2,5)+1
                     lmovld = .false.
                  endif
                  if(lmedge .and. ledge) then
                     nfail = nfail-1
                     mflcnt(2,6) = mflcnt(2,6)+1
                     lmedge = .false.
                  endif
c
c  Reset status flag to ACCEPT, but only if not rejected for some other
c reason
                  if (nfail .eq. 0) then
                     if (mrf(l) .eq. 0) then
c                     Full
                        isttrf(l) = lqOK
                     else
c                     Partial
                        isttrf(l) = lqprt
                     endif
                  endif
               endif
            endif
            if (.not. lmbgratio)
     $         flmmax(1,2) = max(flmmax(1,2), bgratio)
            if (.not. lmpkratio)
     $         flmmax(2,2) = max(flmmax(2,2), pkratio)
            if (.not. lmgrad)
     $         flmmax(3,2) = max(flmmax(3,2), gradient)
         endif
      enddo
c!!!
c!      changed = .false.      
c!      do i=1,nrs
c!         if (istatf(i) .ne. isttrf(i)) changed = .true.
c!      enddo 
c!      if (changed) then
c!         write (6,*) 'Status flags changed:'
c!         write (6,'(3x,200(i3))') (istatf(i), i=1,nrs)
c!         write (6,'(3x,200(i3))') (isttrf(i), i=1,nrs)
c!      endif 
c!!!-
      return
      end


      subroutine unitmt(a)
c     ===================
c
c  Set 3x3 matrix to unit matrix
c
      real a(3,3)
c
      integer i,j
c
      do 10, j = 1,3
         do 11, i = 1,3
            a(i,j) = 0.0
 11      continue
         a(j,j) = 1.0
 10   continue
      return
      end
      subroutine unpckf(irun, m, nmpr, icntrc, ifrflg)
c     ================================================
c
c Unpack things from reflection flag
c
c On entry:
c  ifrflg     packed flag
c
c On exit:
c  irun       run number
c  m          full/partial flag, = 0 full, = +1 partial
c  nmpr       partial scale flag, -1 for scaled partial, else = 0
c  icntrc     centric flag, = 1 centric else = 0
c
c
      integer irun, m, nmpr, icntrc, ifrflg
c
      integer iflg
c
      iflg = ifrflg
c
      if (iflg .ge. 10000) then
         icntrc = 1
         iflg = iflg - 10000
      else
         icntrc = 0
      endif
      if (iflg .ge. 1000) then
         m = 1
         if (iflg .ge. 2000) then
            nmpr = -1
            iflg = iflg - 2000
         else
            nmpr =  0
            iflg = iflg - 1000
         endif
      else
         m = 0
         nmpr = 0
      endif
      irun = iflg
      return
      end
      subroutine unploc(irun, ipglob, iploc, ipb, ipxy, ipz)
c     =====================================================
c
c  Unpack local parameter pointers from global parameter number
c
c On entry:
c     irun         run number
c     ipglob       global parameter number
c
c On exit:
c     iploc        local parameter number
c     ipb          index to B-factor (= 0 if scale)
c     ipxy         index to position in xy plane
c     ipz          index to position on z
c
      implicit none
c
      integer irun, ipglob, iploc, ipb, ipxy, ipz
c
      include 'parameter.fh'
      include 'scales.fh'
      include 'params.fh'
c
c  Local index
      iploc = ipnloc(ipglob)
      if (iploc .gt. 0) then
         if (iploc .le. nbparm(irun)) then
            ipb = iploc
            ipz = 0
            ipxy = 0
         else
            ipb = 0
            ipz = (iploc - nbparm(irun) - 1) / ncscls(2,irun) + 1
            ipxy = (iploc - nbparm(irun)) - (ipz - 1) * ncscls(2,irun)
         endif
      endif
      return
      end
c
c
c
      subroutine unpmfl(lmflag, bgpk,
     $   lmbgratio, lmpkratio, lmnegsd, lmgrad, lmovld, lmedge,
     $   bgratio, pkratio, gradient, istat)
c    =================================
c
c Unpack status information from Mosflm
c
c On entry
c  lmflag       packed flag
c      1         BGRATIO too large
c      2         PKRATIO too large
c      4         Negative > 5*sigma
c      8         Gradient too high
c     16         Profile fitted overload
c     32         Profile fitted "edge" reflection
c     other values invalid
c
c  bgpk       packed BG/PK ratios
c
c On exit
c  flags = .true. if condition satisfied
c    lmbgratio   BGratio too large
c    lmpkratio   PKratio too large
c    lmnegsd     I < 5 sigma(I)
c    lmgrad      gradient too high
c    lmovld      profile fitted overload
c    lmedge      profile fitted edge reflection
c
c  bgratio       (rms deviation from background plane)/sd(background)
c  pkratio       (rms deviation from peak profile)/sd(peak)
c  gradient      (larger xy component of background slope)/(mean background)
c
c  istat        =  0 no fail flags set
c               = +1 at least one set
c               = -1 illegal flag set (lmflag .ge. 64)
c
c
      integer lmflag, istat
      real  bgpk, bgratio, pkratio, gradient
      logical  lmbgratio, lmpkratio, lmnegsd, lmgrad, lmovld, lmedge
c
      integer igood, ipkr
c
      istat = 0
c
      if (lmflag .ge. 64) then
         istat = -1
         return
      endif
c Unpack bits
      lmbgratio = iand(lmflag, 1) .ne. 0
      lmpkratio = iand(lmflag, 2) .ne. 0
      lmnegsd   = iand(lmflag, 4) .ne. 0
      lmgrad    = iand(lmflag, 8) .ne. 0
      lmovld    = iand(lmflag,16) .ne. 0
      lmedge    = iand(lmflag,32) .ne. 0
c
c Are any set?
      if (iand(lmflag, 63) .ne. 0) istat = +1
c
c Unpack bgpk
      igood = int(bgpk)
      gradient = bgpk - igood
      ipkr = igood/100
      bgratio = 0.1*(igood-ipkr*100)
      pkratio = 0.01*ipkr
c
      end
c
c
c
      subroutine wdsclrcc(delIoutlier, jdts, title1, title2)
c     ==============================================
c
c  Clear all sums for correlation coefficients within datasets
c
c On entry:
c  DelIoutlier    outlier limit on |I+ - I-| in RMS multiples
c                 for this dataset
c  jdts           dataset number, if = -1 just one file
c  title1, title2  titles
c
      real delIoutlier
      integer jdts, kdts
      character title1*(*), title2*(*)
c
      include 'parameter.fh'
      include 'crlncf.fh'
      include 'means.fh'
c
      integer i, j
      real dx
      integer maxptducf
c
      logical lsample
      external lsample
c
      cossin45 = sqrt(2.0)/2.0
      kdts = max(jdts, 1)
      jdts_ucw = kdts
c Scale factor to convert DelI to multiple of RMS
      if (abs(qsctsa(kdts)) .gt. 0.1) then
         ucfscl = 1./qsctsa(kdts)
      else
         ucfscl = 1.
      endif

c Maximum value to truncate
      ucfmax = 10.0
c
c Set up sampling to avoid plotting too many points
      xsmmaxucf = 3.0
      dxsmplucf = xsmmaxucf/float(maxsmpucf)
      maxptducf = 200
c  number of points, maximum point density,
c   number of sampling bins (parameter), width of bins
c  returns true if sampling, last argument is sample array
      lsmplucf = lsample(nrefsa(kdts), maxptducf, maxsmpucf, dxsmplucf,
     $   sampleucf)
c
c
      do i = 1, maxres

         RMSdelI(i) = qsctsr(i,kdts) * delIoutlier
         do j = 1,2
            sumccwa_x(i,j) = 0.0
            sumccwa_y(i,j) = 0.0
            sumccwa_x2(i,j) = 0.0
            sumccwa_y2(i,j) = 0.0
            sumccwa_xy(i,j) = 0.0
            nsumccwa(i,j) = 0
            sumucwa_c2(i,j) = 0.0
            sumucwa_e2(i,j) = 0.0
            nsumucwa(i,j) = 0
         enddo
c
         sumccwi_x(i) = 0.0
         sumccwi_y(i) = 0.0
         sumccwi_x2(i) = 0.0
         sumccwi_y2(i) = 0.0
         sumccwi_xy(i) = 0.0
         nsumccwi(i) = 0
      enddo


c Open output file
      call stwruc(jdts, ucfmax, title1, title2)

      return
      end
c
      subroutine wdsadcc(icen, mbn_s, danom, ffmean)
c     =======================================
c
c Add contributions into sums for correlation coefficients
c within dataset
c
c On entry:
c  icen             = 1 acentric, =2 centric
c  mbn_s            resolution bin number
c  danom(2)         pair of anomalous differences 
c  ffmean(2)        pair of Imean
c
      integer icen, mbn_s
      real danom(2), ffmean(2)
c
      include 'parameter.fh'
      include 'crlncf.fh'
      include 'means.fh'
c
      integer i, j, k, idummy
      real x, y, xy
      real rndmnm
      external rndmnm

      logical lwrite
      data idummy /0/
c
c Anomalous differences
      i = 1
      j = 2      
      if (danom(1) .ne. 0.0 .and. danom(2) .ne. 0.0) then         
c  Correlation analysis excluding outliers beyond delIlim (= "5" * RMS)
         if ((abs(danom(i)) .le. RMSdelI(mbn_s)) .and.
     $      (abs(danom(j)) .le.  RMSdelI(mbn_s))) then
            nsumccwa(mbn_s,icen) = nsumccwa(mbn_s,icen) + 1
            sumccwa_x(mbn_s,icen) =
     $         sumccwa_x(mbn_s,icen) + danom(i)
            sumccwa_y(mbn_s,icen) =
     $         sumccwa_y(mbn_s,icen) + danom(j)
            sumccwa_x2(mbn_s,icen) =
     $         sumccwa_x2(mbn_s,icen) + danom(i)**2
            sumccwa_y2(mbn_s,icen) =
     $         sumccwa_y2(mbn_s,icen) + danom(j)**2
            sumccwa_xy(mbn_s,icen) =
     $         sumccwa_xy(mbn_s,icen) + danom(i) * danom(j)
c
            sumucwa_c2(mbn_s,icen) = sumucwa_c2(mbn_s,icen) +
     $         (cossin45 * (danom(j) + danom(i)))**2
            sumucwa_e2(mbn_s,icen) = sumucwa_e2(mbn_s,icen) +
     $         (cossin45 * (danom(j) - danom(i)))**2
            nsumucwa(mbn_s,icen) = nsumucwa(mbn_s,icen) + 1
         endif
c
c Output to scatterplot file, omit gross outliers
c  ucgscl = 1/OverallRMS(DelAnom)
         x = ucfscl*danom(i)
         y = ucfscl*danom(j)
c  ucfmax = 10
         if (abs(x) .lt. ucfmax .and. abs(y) .lt.ucfmax) then
c Optional sampling
            lwrite = .true.
            if (lsmplucf) then
               xy = max(abs(x), abs(y))
               if (xy .lt. xsmmaxucf) then
                  k = int(xy/dxsmplucf)
                  if (rndmnm(idummy) .gt. sampleucf(k)) lwrite = .false.
               endif
            endif
            if (lwrite) write (iwrucf, 6001) x,y,jdts_ucw
 6001       format(2f9.3, i3)
         endif 
      endif
c
c Imean
      nsumccwi(mbn_s) = nsumccwi(mbn_s) + 1
      sumccwi_x(mbn_s) =
     $   sumccwi_x(mbn_s) + ffmean(i)
      sumccwi_y(mbn_s) =
     $   sumccwi_y(mbn_s) + ffmean(j)
      sumccwi_x2(mbn_s) =
     $   sumccwi_x2(mbn_s) + ffmean(i)**2
      sumccwi_y2(mbn_s) =
     $   sumccwi_y2(mbn_s) + ffmean(j)**2
      sumccwi_xy(mbn_s) =
     $   sumccwi_xy(mbn_s) + ffmean(i) * ffmean(j)
c
      return 
      end
c
c
c
      subroutine wdsprcc(nsrmax, reso, labdts,
     $     CCthrshld, RCRthrshld, nCC, nRCR)
c     =====================================
c
c Print correlation tables within dataset
c
c On entry:
c  nsrmax     maximum resolution bin
c  reso       resolution limits for printing
c  labdts     dataset label
c  CCthrshld   threshold for CCanom to be considered "significant"
c  RCRthrshld  threshold for RCRanom to be considered "significant"
c
c On exit:
c  nCC      number of bins with CC  > CCthrshld
c  nRCR     number of bins with RCR > RCRthrshld
c
      integer nsrmax,  nCC, nRCR
      real reso(nsrmax),  CCthrshld, RCRthrshld
      character*(*) labdts
c
      include 'parameter.fh'
      include 'inout.fh'
      include 'crlncf.fh'
      include 'sharvest.fh'
c
      integer k, l, nsum, nucf, nsmax

      real shbb
      double precision
     $   sum_x, sum_y,
     $   sum_x2, sum_y2,
     $   sum_xy, denom, sumn,
     $   sum_c2, sum_e2
      character cccwanom(maxres,2)*8,  crcrwanom(maxres,2)*8, cccwimn*8
c
      integer lenstr
      external lenstr
c
      write (lunout, '(/a,a/a,a/)')
     $   'Correlation coefficients for anomalous differences & ',
     $   'Imean between random subsets within dataset',
     $   '====================================================',
     $   '==========================================='
      write (lunout, '(/a,a/a,a/a,a/)')
     $ 'The RMS Correlation Ratio (RCR) is calculated from',
     $ ' a scatter plot of pairs of DeltaI(anom) from the two subsets',
     $ 'by comparing the RMS value (excluding extremes) projected on ',
     $ 'the line with slope = 1 ("correlation") with the ',
     $ 'RMS value perpendicular to this ("error"). ',
     $ 'This ratio will be > 1 if there is an anomalous signal'
c
c Count number of bins with CC or RCR above "threshold"
      nCC = 0
      nRCR = 0

c  Anomalous
      do k = 1,2
         sum_x  = 0.0
         sum_y  = 0.0
         sum_x2 = 0.0
         sum_y2 = 0.0
         sum_xy = 0.0
         nsum = 0
         sum_c2 = 0.0
         sum_e2 = 0.0
         nucf = 0
c
         do l = 1, nsrmax
            ccwanom(l,k) = 0.0
            rcrwanom(l,k) = 0.0
            if (nsumccwa(l,k) .gt. 2) then
               sumn = nsumccwa(l,k)
               denom =
     $            (sumccwa_x2(l,k) - sumccwa_x(l,k)**2/sumn)*
     $            (sumccwa_y2(l,k) - sumccwa_y(l,k)**2/sumn)
               if (denom .gt. 0.000001) then
                  ccwanom(l,k) = (sumccwa_xy(l,k) -
     $               sumccwa_x(l,k)*sumccwa_y(l,k)/sumn)
     $               /dsqrt(denom)
               endif
               sum_x  = sum_x  + sumccwa_x(l,k)
               sum_y  = sum_y  + sumccwa_y(l,k)
               sum_x2  = sum_x2  + sumccwa_x2(l,k)
               sum_y2  = sum_y2  + sumccwa_y2(l,k)
               sum_xy  = sum_xy  + sumccwa_xy(l,k)
               nsum = nsum + nsumccwa(l, k)
            endif
c  RMS correlation ratio
            if (nsumucwa(l,k) .gt. 2) then
               rcrwanom(l,k) = sqrt(sumucwa_c2(l,k)/sumucwa_e2(l,k))
               sum_c2 = sum_c2 + sumucwa_c2(l,k)
               sum_e2 = sum_e2 + sumucwa_e2(l,k)
               nucf = nucf + nsumucwa(l,k)
            endif
         enddo
         ccwanomov(k) = 0.0
         nswanomov(k) = nsum
         if (nsum .gt. 0) then
            sumn = nsum
            denom = (sum_x2 - sum_x**2/sumn)*
     $         (sum_y2 - sum_y**2/sumn)
            if (denom .gt. 0.000001) then
               ccwanomov(k) = (sum_xy -
     $            sum_x*sum_y/sumn)/dsqrt(denom)
            endif
         endif
c
         rcrov(k) = 0.0
         nrcrov(k) = 0
         if (nucf .gt. 0) then
            rcrov(k) = sqrt(sum_c2/sum_e2)
            nrcrov(k) = nucf
         endif
      enddo
c Imean
      sum_x  = 0.0
      sum_y  = 0.0
      sum_x2 = 0.0
      sum_y2 = 0.0
      sum_xy = 0.0
      nsum = 0
      do l = 1, nsrmax
         ccwimn(l) = 0.0
         if (nsumccwi(l) .gt. 0) then
            sumn = nsumccwi(l)
            denom =
     $         (sumccwi_x2(l) - sumccwi_x(l)**2/sumn)*
     $         (sumccwi_y2(l) - sumccwi_y(l)**2/sumn)
            if (denom .gt. 0.000001) then
               ccwimn(l) = (sumccwi_xy(l) -
     $            sumccwi_x(l)*sumccwi_y(l)/sumn)
     $            /dsqrt(denom)
            endif
            sum_x  = sum_x  + sumccwi_x(l)
            sum_y  = sum_y  + sumccwi_y(l)
            sum_x2  = sum_x2  + sumccwi_x2(l)
            sum_y2  = sum_y2  + sumccwi_y2(l)
            sum_xy  = sum_xy  + sumccwi_xy(l)
            nsum = nsum + nsumccwi(l)
         endif
      enddo
      ccwimnov = 0.0
      nswimnov = nsum
      if (nsum .gt. 0) then
         sumn = nsum
         denom = (sum_x2 - sum_x**2/sumn)*
     $      (sum_y2 - sum_y**2/sumn)
         if (denom .gt. 0.000001) then
            ccwimnov = (sum_xy -
     $         sum_x*sum_y/sumn)/dsqrt(denom)
         endif
      endif
c
c Write out as table against resolution
c
      call htm_stgraph

      call dna_table_start('correlations_within_dataset')


      write (lunout, 6001)  labdts(1:lenstr(labdts))
 6001 format (/
     $'$TABLE: Correlations within dataset',a,' :')
      write (lunout, 6002)
 6002 format (
     $  '$GRAPHS: Anom & Imean CCs v resolution - :A:2,4,6,12:',/,
     $   ':RMS correlation ratio:A:2,8,10:$$',/)
c
      write (lunout, '(a)')
     $   ' N 1/resol^2 dmax CC_anom  N_anom  CC_cen   N_cen'//
     $   ' RCR_anom  N_anom  RCR_cen   N_cen'//
     $   ' CC_Imean  N_Imean    $$' 
      write (lunout, '(a)') '$$'
c
c  Find last range with any contributions to eliminate empty ranges
      call findlastrange(nsumccwi, nsrmax, nsmax)

      do l = 1,nsmax
         shbb = 1./reso(l)**2
         do k=1,2
            call tformat(ccwanom(l,k), cccwanom(l,k), 8,3)
            call tformat(rcrwanom(l,k), crcrwanom(l,k), 8,3)
         enddo 
         call tformat(ccwimn(l), cccwimn, 8,3)

         write (lunout, '(i3,f8.4,f6.2,4(a8,i8),a8,i8)')
     $      l, shbb, reso(l),
     $      (cccwanom(l,k), nsumccwa(l,k), k=1,2),
     $      (crcrwanom(l,k), nsumucwa(l,k), k=1,2),
     $      cccwimn, nsumccwi(l)

         call dna_ilist_start('bin', l)
         call dna_integer_item('bin', l)
         call dna_real_item('inv_resol_sq', shbb)
         call dna_real_item('dmax', reso(l))
         call dna_real_item('cc_anom', ccwanom(l, 1))
         call dna_integer_item('n_anom', nsumccwa(l, 1))
         call dna_real_item('cc_cen', ccwanom(2, 1))
         call dna_integer_item('n_cen', nsumccwa(2, 1))
         call dna_real_item('cc_imean', ccwimn(l))
         call dna_integer_item('n_imean', nsumccwi(l))
         call dna_list_end

         HanoCC(l) = ccwanom(l,1)

         if (ccwanom(l,1) .gt. CCthrshld) nCC = nCC+1
         if (rcrwanom(l,1) .gt. RCRthrshld) nRCR = nRCR+1
      enddo
      
      call dna_table_end

      write (lunout, '(a)') ' $$'
      write (lunout, '(a,9x, 5(f8.3,i8))')
     $   ' Overall', (ccwanomov(k), nswanomov(k), k = 1,2),
     $   (rcrov(k), nrcrov(k), k = 1,2),
     $   ccwimnov, nswimnov
      write (lunout, '(/)')
      HanoCCALL = ccwanomov(1)
      call htm_endgraph
c
      call endppr(iwrucf)
      call linprb(iwrucf, 1, 10.0)
      close (unit=iwrucf)
c
      return
      end
c
c
c
      subroutine wdsanl(icntrc, nrs,  mbn_s, ffrf, sdffrf, isymrf,
     $      ghl, useflg, nupls, numns)
c     =============================================
c
c Randomly assign I+ & I- sets to two equal(ish) groups, then
c accumulate sums for correlation analysis between groups,
c on anomalous differences and on Imean
c
c Observations for centric reflections are randomly split 
c into two groups, to dummy I+ and I-
c
c Assignment into two groups is done by sorting into a random order,
c then taking the first & second "halves"
c (ie 1st half contains int(n/2) values)
c
c  On entry:
c     icntrc    Centric if icntrc = 1, else = 0
c     nrs       number of observations
c     mbn_s     resolution bin
c     ffrf      intensities
c     sdffrf    corrected sd(Ihl)
c     isymrf    symmetry number
c     ghl       dividing scales
c     useflg    .true. to use this observation
c                (set for both I+ & I-)
c     nupls, numns  number of useful observations for I+ & I- set
c
c
      implicit none
c
      include 'parameter.fh'
      include 'flags.fh'
      include 'inout.fh'
c
      integer icntrc, nrs, mbn_s, nupls, numns
      integer isymrf(nrs)
      real ffrf(nrs), sdffrf(nrs), wtrf(nrs), ghl(nrs)
      logical useflg(nrs)
c
      real ffs(maxrfl,2), sdffs(maxrfl,2), ffm(2,2), sdffm(2,2),
     $   danom(2), ffmean(2)
      integer i, j, l, n, np(2), nm(2), icen, nff
      logical ladd
c
c Need at least two of each I+ & I- for DelAnom analysis, just 2 for Imean
      if (nrs .le. 0) return
      if (nupls+numns .lt. 2) return

c Totals for plus & minus, groups 1 & 2
      np(1) = 0
      np(2) = 0
      nm(1) = 0
      nm(2) = 0
c
      if (icntrc .eq. 0) then
c Acentric
         icen = 1
c Fill intensity &sd  arrays, & count all into slot 1
         do l = 1,nrs
            if (useflg(l)) then
c  + or -?
               if (mod(isymrf(l), 2) .eq. 0) then
c   I- isym even
                  nm(1) = nm(1) + 1
                  ffs(nm(1), 2) = ffrf(l)/ghl(l)
                  sdffs(nm(1), 2) = sdffrf(l)/ghl(l)
               else
c   I+ isym odd
                  np(1) = np(1) + 1
                  ffs(np(1), 1) = ffrf(l)/ghl(l)
                  sdffs(np(1), 1) = sdffrf(l)/ghl(l)
               endif
            endif
         enddo
      else
c Centric
         icen = 2
c Fill intensity & sd  arrays, & count all into I+ slot 1
         n = 0
         do l = 1,nrs
            if (useflg(l)) then
c   I+ initially
               n = n + 1
               ffs(n, 1) = ffrf(l)/ghl(l)
               sdffs(n, 1) = sdffrf(l)/ghl(l)
            endif
         enddo
c Sort into random order
         call asnrnd(ffs(1,1), sdffs(1,1), n)
c Treat the first half of the reordered array as I+,
c  copy the rest for I-
         np(1) = n/2
         nm(1) = n - np(1)
         do l = 1, nm(1)
            ffs(l, 2) = ffs(l+np(1), 1)
            sdffs(l, 2) = sdffs(l+np(1), 1)
         enddo 
      endif
c
c Now assign each of I+ & I- sets to two groups
c Sort into random order
      call asnrnd(ffs(1,1), sdffs(1,1), np(1))
      call asnrnd(ffs(1,2), sdffs(1,2), nm(1))
      n = np(1)/2
      np(2) = np(1) - n
      np(1) = n
      n = nm(1)/2
      nm(2) = nm(1) - n
      nm(1) = n
c
c Get weighted measn of each of the 4 classes (I+/-, groups 1 & 2)
c  I+, group 1
      call wmean(ffs(1,1), sdffs(1,1), np(1), ffm(1,1), sdffm(1,1))
c  I+, group 2
      call wmean(ffs(np(1)+1,1), sdffs(np(1)+1,1), np(2),
     $   ffm(2,1), sdffm(2,1))
c  I-, group 1
      call wmean(ffs(1,2), sdffs(1,2), nm(1), ffm(1,2), sdffm(1,2))
c  I-, group 2
      call wmean(ffs(nm(1)+1,2), sdffs(nm(1)+1,2), nm(2),
     $   ffm(2,2), sdffm(2,2))
c
c Anomalous differences and unweighted means of I+ & I-
c for each group
      ladd = .false.
      nff = 0
      do j = 1,2
         if (np(j) .eq. 0 .and. nm(j) .eq. 0) then
            danom(j)  = 0.0
            ffmean(j) = 0.0            
         else
            if (np(j)*nm(j) .ne. 0) then
               danom(j)  = ffm(j,1) - ffm(j,2)
               ladd = .true.
            else
               danom(j)  = 0.0
            endif
            ffmean(j) = ffm(j,1) + ffm(j,2)
            if (np(j)*nm(j) .ne. 0) then
               ffmean(j) = 0.5 * ffmean(j) 
            endif
            nff = nff+1
         endif
      enddo
c Add into sums for correlation
      if (ladd .and. nff .eq. 2)
     $   call wdsadcc(icen, mbn_s, danom, ffmean)
c
      return
      end
c
c
c
      subroutine wmean(a, s, n, wm, ws)
c    ============================
c
c Weighted mean of n values in a, weights 1/s**2
c Returns wm weighted mean, ws its sd
c
      integer n
      real a(n), s(n), wm, ws
c
      integer i
      real w, sumwa, sumw
c
      if (n .le. 0) then
         wm = 0.0
         ws = 0.0
         return
      endif
      sumwa = 0.0
      sumw  = 0.0
      do i = 1, n
         if (s(i) .gt. 0.0) then
            w = 1./(s(i)**2)
            sumwa = sumwa + w * a(i)
            sumw  = sumw  + w
            if (sumw .le. 0.0) then
               wm = 0.0
               ws = 0.0
            else
               wm = sumwa/sumw
               ws = 1./sqrt(sumw)
            endif
         endif
      enddo
c
      return
      end
c
c
c
      subroutine asnrnd(a, b, n)
c    ====================
c
c Randomly reorder a(n), b(n) pairs
c
      integer n
      real a(n), b(n)
c
      include 'parameter.fh'
      real r(maxrfl)
      integer i, idummy
c
      real rndmnm
      external rndmnm
c
      if (n .le. 0) return
c
      do i = 1,n
c fill r array with random numbers between 0 & 1
         r(i) = rndmnm(idummy)
      enddo
c Sort
      call sort2(n, r, a, b)
      return
      end
c
      subroutine sort2(n,a0,a1,a2)
c     =========================
c
c****
c**** n is the length of the array to be sorted i.e. the number of
c**** items to be sorted
c**** the elements of array a0(n) are sorted into increasing order of
c**** magnitude.  arrays a1 to a8 may be carried with a0 as required
c****

c     .. scalar arguments ..
      integer n
c     ..
c     .. array arguments ..
      real a0(n),a1(n),a2(n)
c     ..
c     .. local scalars ..
      real a,c,d
      integer i,ih,ip1,j,k,no2,np1
c     ..
      no2 = n/2
      do 80 i = 1,no2
        c = a0(i)
        d = c
        ih = i
        k = i
        ip1 = i + 1
        np1 = n + 1 - i
        do 20 j = ip1,np1
          a = a0(j)
          if (a.ge.c) go to 10
          ih = j
          c = a
          go to 20

   10     if (d.ge.a) go to 20
          d = a
          k = j
   20   continue
        if (c.eq.d) go to 90
        if (ih.eq.i) go to 50
   30   c = a1(ih)
        a1(ih) = a1(i)
        a1(i) = c
        c=a2(ih)
        a2(ih)=a2(i)
        a2(i)=c
   40   c = a0(ih)
        a0(ih) = a0(i)
        a0(i) = c
   50   if (k.eq.i) k = ih
        if (k.eq.np1) go to 80
   60   d = a1(k)
        a1(k) = a1(np1)
        a1(np1) = d
        d=a2(k)
        a2(k)=a2(np1)
        a2(np1)=d
   70   d = a0(k)
        a0(k) = a0(np1)
        a0(np1) = d
   80 continue
   90 return

      end
      subroutine wrovlm(amatrx, smatrx, nmatrx, matdim)
c     ================================================
c
c  Write out overlap array as map
c
c  On entry:
c     amatrx(matdim, nmatrx)      overlap array (double precision)
c     smatrx(nmatrx, nmatrx)      single precision workspace
c
      implicit none
c
c
      integer nmatrx, matdim
cDP      real amatrx(matdim, nmatrx), smatrx(nmatrx, nmatrx)
      double precision amatrx(matdim, nmatrx)
      real smatrx(nmatrx, nmatrx)
c
      character*80 title
      integer nsec, iuvw(3), mxyz(3), nw1,nu1,nu2,nv1,nv2,lspgrp,
     $     lmode,i,j
      real cell(6)
c
      title = ' Overlap map from Scala'
      nsec = 1
      iuvw(1) = 1
      iuvw(2) = 2
      iuvw(3) = 3
      mxyz(1) = nmatrx
      mxyz(2) = nmatrx
      mxyz(3) = 1
      nw1 = 0
      nu1 = 1
      nu2 = nmatrx
      nv1 = 1
      nv2 = nmatrx
      cell(1) = nmatrx
      cell(2) = nmatrx
      cell(3) = 1.
      cell(4) = 90.
      cell(5) = 90.
      cell(6) = 90.
      lspgrp = 1
      lmode = 2
c
      call mwrhdr(1,title,nsec,iuvw,mxyz,nw1,nu1,nu2,
     $     nv1,nv2,cell,lspgrp,lmode)
c
      do 10, j = 1, nmatrx
         do 11, i = 1, nmatrx
            smatrx(i,j) = amatrx(i,j)
 11      continue
 10   continue
      call mspew(1, smatrx)
      call mwclose(1)
      return
      end
      subroutine xrfflg(nflags, lrfflg, krflag)
c     =========================================
c
c Extract reflection error/warning flags
c
c
c  On entry:
c     nflags           number of flags to set or unset
c     krflag           packed bit flags
c
c  On exit:
c     lrfflg(nflags)   logical flags
c
      implicit none
c
      integer nflags, krflag
      logical lrfflg(nflags)
c
      logical btest
      integer i
c
c
      do 10, i = 1, nflags
         lrfflg(i) = btest(krflag, i-1)
 10   continue
      return
      end
